#ifndef GUI_CORE___APP_EXPLORER_CMD_HANDLER__HPP
#define GUI_CORE___APP_EXPLORER_CMD_HANDLER__HPP

/*  $Id: app_explorer_cmd_handler.hpp 20873 2010-02-19 18:19:29Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *  CAppExplorerCmdHandler - the command handler for CAppExplorerService.
 *
 */

#include <corelib/ncbistl.hpp>
#include <corelib/ncbiobj.hpp>

#include <gui/gui_export.h>

#include <gui/core/app_explorer_item.hpp>

#include <gui/utils/extension.hpp>

#include <wx/event.h>


BEGIN_NCBI_SCOPE

class CAppTaskService;
class IAppTask;
class IServiceLocator;

///////////////////////////////////////////////////////////////////////////////
/// CAppExplorerItemCmdContributor - an extension contributing App Explorer
/// Service commands.
class CAppExplorerItemCmdContributor :
    public CObject,
    public IExtension,
    public IExplorerItemCmdContributor
{
public:
    /// @name IExplorerItemCmdContributor implementation
    /// @{
    virtual TContribution   GetMenu(TItemRefVector& items,
                                    CAppExplorerService& app_service);
    /// @}

    /// @name IExtension implementation
    /// @{
    virtual string  GetExtensionIdentifier() const;
    virtual string  GetExtensionLabel() const;
    /// @}
};


///////////////////////////////////////////////////////////////////////////////
/// IExplorerItemSelectionSource - the interface that provides selection to
/// CAppExplorerCmdHandler.
/// Views that need to support App Explorer Service commands should implement
/// this interface.
class IExplorerItemSelectionSource
{
public:
    typedef CRef<CExplorerItem> TItemRef;
    typedef vector<TItemRef>    TItemRefVector;

public:
    virtual void    GetSelectedItems(TItemRefVector& items) = 0;

    virtual ~IExplorerItemSelectionSource()    {};
};


///////////////////////////////////////////////////////////////////////////////
/// CAppExplorerCmdHandler - handles wxWidget commands and translates them
/// into calls to CAppExplorerService
/// IExplorerItemSelectionSource provides selection for the commands.

class  NCBI_GUICORE_EXPORT  CAppExplorerCmdHandler :
    public wxEvtHandler
{
    DECLARE_EVENT_TABLE();
public:
    typedef CRef<CExplorerItem> TItemRef;
    typedef vector<TItemRef>    TItemRefVector;

public:
    CAppExplorerCmdHandler(IServiceLocator* srv_locator);
    virtual ~CAppExplorerCmdHandler();

    /// @name Command handlers
    /// @{
    void OnCut(wxCommandEvent& event);
    void OnUpdateCut(wxUpdateUIEvent& event);
    void OnCopy(wxCommandEvent& event);
    void OnUpdateCopy(wxUpdateUIEvent& event);
    void OnPaste(wxCommandEvent& event);
    void OnUpdatePaste(wxUpdateUIEvent& event);

    void OnRemove(wxCommandEvent& event);
    void OnUpdateRemove(wxUpdateUIEvent& event);

    void OnEnableDisable(wxCommandEvent& event);
    void OnUpdateEnableDisable(wxUpdateUIEvent& event);
    void OnNewFolder(wxCommandEvent& event);
    void OnUpdateNewFolder(wxUpdateUIEvent& event);
    void OnCloseView(wxCommandEvent& event);
    void OnUpdateCloseView(wxUpdateUIEvent& event);
    void OnActivateClients(wxCommandEvent& event);
    void OnUpdateActivateClients(wxUpdateUIEvent& event);

    void OnProperties(wxCommandEvent& event);
    void OnUpdateProperties(wxUpdateUIEvent& event);

    void OnUpdateSettings(wxUpdateUIEvent& event);
    /// @}

protected:
    void    x_CutOrCopy(bool cut);
    bool    x_CanUpdateCutOrCopy();

    CAppExplorerService*    x_GetAppExplorerService();
    CAppTaskService*        x_GetAppTaskService();

    void    x_LaunchTask(IAppTask* task);
    void    x_GetSelectedItems(TItemRefVector& items);

protected:
    IServiceLocator*  m_ServiceLocator;
};


END_NCBI_SCOPE


#endif  // GUI_CORE___APP_EXPLORER_CMD_HANDLER__HPP

