#ifndef GUI_CORE___APP_TASKS__HPP
#define GUI_CORE___APP_TASKS__HPP

/*  $Id: app_tasks.hpp 26466 2012-09-19 21:18:47Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

#include <gui/gui_export.h>

#include <corelib/ncbiobj.hpp>

#include <gui/framework/app_task_impl.hpp>
#include <gui/objutils/objects.hpp>

#include <wx/string.h>

BEGIN_NCBI_SCOPE

class IWorkbench;

///////////////////////////////////////////////////////////////////////////////
/// COpenDlgTask - shows "Open" dialog
class NCBI_GUICORE_EXPORT COpenDlgTask : public CAppTask
{
public:
    /// shows Open dialog
    COpenDlgTask(IWorkbench* workbench);

    /// shows Open dialog and activates the specified loading option
    COpenDlgTask(IWorkbench* workbench, const string& loader_label);

    /// shows Open dialog, intializes it with the given files and activates File option
    COpenDlgTask(IWorkbench* workbench, const vector<wxString>& filenames);

protected:
    virtual ETaskState    x_Run();

protected:
    IWorkbench* m_Workbench;
    string      m_LoaderLabel;  /// loading option to select
    vector<wxString>  m_Filenames;
};


///////////////////////////////////////////////////////////////////////////////
/// CRunToolDlgTask - shows "Run Tool" dialog
class NCBI_GUICORE_EXPORT CRunToolDlgTask : public CAppTask
{
public:
    CRunToolDlgTask(IWorkbench* workbench, const string& toolLabel = NcbiEmptyString);

protected:
    virtual ETaskState    x_Run();

protected:
    IWorkbench* m_Workbench;
    string m_ToolLabel;
};


///////////////////////////////////////////////////////////////////////////////
/// COpenViewlDlgTask - shows "Open View" dialog
BEGIN_objects_SCOPE
class CSeq_id;
END_objects_SCOPE

class NCBI_GUICORE_EXPORT COpenViewlDlgTask : public CAppTask
{
public:
    COpenViewlDlgTask(IWorkbench* workbench, bool openDefault = false);

protected:
    virtual ETaskState    x_Run();

private:
    void x_CheckDataStatus(SConstScopedObject& object);

protected:
    IWorkbench* m_Workbench;
    bool        m_OpenDefault;
};

///////////////////////////////////////////////////////////////////////////////
/// CExportDlgTask - shows "Export" dialog
class NCBI_GUICORE_EXPORT CExportDlgTask : public CAppTask
{
public:
    /// shows Open dialog
    CExportDlgTask(IWorkbench* workbench);

protected:
    virtual ETaskState    x_Run();

protected:
    IWorkbench* m_Workbench;
};

END_NCBI_SCOPE

#endif  // GUI_CORE___APP_TASKS__HPP
