#ifndef GUI_CORE___ASYNC_OBJ_CONVERT__HPP
#define GUI_CORE___ASYNC_OBJ_CONVERT__HPP

/*  $Id: async_obj_convert.hpp 25493 2012-03-27 18:28:58Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 */

#include <corelib/ncbistl.hpp>

#include <gui/gui_export.h>

#include <gui/objutils/objects.hpp>
#include <gui/widgets/wx/async_call.hpp>

BEGIN_NCBI_SCOPE

class CTypeInfo;

class NCBI_GUICORE_EXPORT CAsyncObjConvert: public CAsyncCall::Call
{
public:
    CAsyncObjConvert(const CTypeInfo* typeInfo,
                     TConstScopedObjects& inputObjects,
                     map<string, TConstScopedObjects>& results) :
      Call(wxT("Doing object conversions...")),
          m_TypeInfo(typeInfo), m_InputObjects(inputObjects), m_Results(results) {}

    virtual bool CanCancel() const { return true; }
    virtual void Execute();

private:
    const CTypeInfo*                  m_TypeInfo;
    TConstScopedObjects&              m_InputObjects;
    map<string, TConstScopedObjects>& m_Results;
};

class NCBI_GUICORE_EXPORT CAsyncObjConvert2 : public CAsyncCall::Call
{
public:
    CAsyncObjConvert2(const CTypeInfo* typeInfo,
                      TConstScopedObjects& inputObjects,
                      TConstScopedObjects& results) :
      Call(wxT("Doing object conversions...")),
          m_TypeInfo(typeInfo), m_InputObjects(inputObjects), m_Results(results) {}

    virtual bool CanCancel() const { return true; }
    virtual void Execute();

private:
    const CTypeInfo*     m_TypeInfo;
    TConstScopedObjects& m_InputObjects;
    TConstScopedObjects& m_Results;
};

END_NCBI_SCOPE

#endif  // GUI_CORE___ASYNC_OBJ_CONVERT__HPP
