#ifndef GUI_CORE___GCASSEMBLY_SEARCH_WIZARD__HPP
#define GUI_CORE___GCASSEMBLY_SEARCH_WIZARD__HPP

/*  $Id: gcassembly_search_wizard.hpp 28155 2013-05-17 18:17:47Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Liangshou Wu
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>
#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>
#include <gui/core/ui_tool_manager.hpp>
#include <gui/objutils/reg_settings.hpp>
#include <gui/objects/DL_AssmInfoResponse.hpp>
#include <gui/objects/DL_AssmInfo.hpp>
#include <gui/objects/DL_Assembly.hpp>
#include <gui/objects/DL_TaxId.hpp>


BEGIN_NCBI_SCOPE

class CAssemblyListPanel;

///////////////////////////////////////////////////////////////////////////////
/// CGCAssemblySearchWizard - A wizard that helps searching gc assembly
class  NCBI_GUICORE_EXPORT  CGCAssemblySearchWizard :
    public CObject,
    public IRegSettings
{
public:
    enum EUsage {
        eLoadGenome,
        eIdMapping
    };

    CGCAssemblySearchWizard(EUsage usage = eLoadGenome);

    void    SetParentWindow(wxWindow* parent);
    void    InitUI();
    void    CleanUI();
    wxPanel*    GetCurrentPanel();
    bool    CanDo(IUIToolManager::EAction action);
    bool    DoTransition(IUIToolManager::EAction action);
    /// @}

    /// @name IRegSettings interface implementation
    /// @{
    virtual void    SetRegistryPath(const string& path);
    virtual void    LoadSettings();
    virtual void    SaveSettings() const;
    /// @}

    vector<string> GetAssmAccs() const;
    bool           IsInitialState() const;

protected:
    enum ESearchMode {
        eSearch_Term,
        eSearch_TaxId,
        eSearch_Gi
    };

private:
    CRef<objects::CDL_AssmInfoResponse> x_GetAssmInfo(
        const string& query_str, ESearchMode mode);

    void x_CreateAssmPanel();

private:
    EUsage           m_Usage;
    wxWindow*        m_ParentWindow;

    string           m_RegPath;
    mutable string   m_SearchTerm;

    objects::CDL_AssmInfo::TAssemblies    m_Assemblies;

    CAssemblyListPanel*  m_AssmPanel;
};



END_NCBI_SCOPE


#endif  // GUI_CORE___GCASSEMBLY_SEARCH_WIZARD__HPP

