#ifndef GUI_CORE___NAMED_PIPE__HPP
#define GUI_CORE___NAMED_PIPE__HPP

/*  $Id: named_pipe.hpp 22957 2011-01-14 19:50:58Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Josh Cherry
 *
 * File Description:
 *    Named pipe functionality for Genome Workbench
 */

#include <gui/gui_export.h>
#include <connect/ncbi_namedpipe.hpp>

BEGIN_NCBI_SCOPE

///////////////////////////////////////////////////////////////////////////////
/// CGBenchPipe
class NCBI_GUICORE_EXPORT CGBenchPipe
{
public:
    /// Attempt to instruct another application instance
    /// to open a file via the named pipe.
    static void OpenRemote(const string& fname, const string& ftype = "auto");

    /// Check if remote opened fine
    static bool IsOpen();

    /// Create a pipe for reading
    static void Start();

    /// Check to see if there are any incoming messages to be processed
    static string Read();

private:
    static string x_GetPipeName(void);

private:
    static CNamedPipeServer sm_NamedPipeServer;
    static bool             sm_IsOpen;
};

END_NCBI_SCOPE


#endif  // GUI_CORE___NAMED_PIPE__HPP
