#ifndef GUI_CORE___PROJECT_CMD_HANDLER__HPP
#define GUI_CORE___PROJECT_CMD_HANDLER__HPP

/*  $Id: project_cmd_handler.hpp 27086 2012-12-17 19:43:41Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>
#include <corelib/ncbiobj.hpp>

#include <gui/gui_export.h>

#include <wx/event.h>

#include <gui/core/project.hpp>


BEGIN_NCBI_SCOPE

class CProjectService;
class CAppTaskService;
class IAppTask;
class IServiceLocator;


///////////////////////////////////////////////////////////////////////////////
/// IProjectSelectionSource
class IProjectSelectionSource
{
public:
    typedef CProjectPrx::TId    TProjectId;
    typedef vector<TProjectId>  TProjectIdVector;

public:
    virtual bool    IsWorkspaceSelected() = 0;
    virtual void    GetSelectedProjectIds(TProjectIdVector& ids) = 0;

    virtual ~IProjectSelectionSource()    {};
};


///////////////////////////////////////////////////////////////////////////////
/// CProjectCmdHandler - handles wxWidget commands and translates them
/// to CProjectTasks.
/// IProjectSelectionSource provides selection for the commands.
/// TODO

class  NCBI_GUICORE_EXPORT  CProjectCmdHandler :
    public wxEvtHandler
{
    DECLARE_EVENT_TABLE();
public:
    typedef IProjectSelectionSource::TProjectId TProjectId;
    typedef IProjectSelectionSource::TProjectIdVector TProjectIdVector;

public:
    CProjectCmdHandler(IServiceLocator* srv_locator);
    virtual ~CProjectCmdHandler();

    /// @name Command handlers
    /// @{
    void OnSave(wxCommandEvent& event);
    void OnSaveUpdate(wxUpdateUIEvent& event);
    void OnSaveAs(wxCommandEvent& event);
    void OnSaveAsUpdate(wxUpdateUIEvent& event);

    void OnLoadProject(wxCommandEvent& event);
    void OnUpdateLoadProject(wxUpdateUIEvent& event);
    void OnUnLoadProject(wxCommandEvent& event);
    void OnUpdateUnLoadProject(wxUpdateUIEvent& event);
    void OnProjectTableView(wxCommandEvent& event);
    void OnUpdateProjectTableView(wxUpdateUIEvent& event);

    void OnRemove(wxCommandEvent& event);
    void OnUpdateRemove(wxUpdateUIEvent& event);
    void OnRemoveProject(wxCommandEvent& event);
    void OnUpdateRemoveProject(wxUpdateUIEvent& event);

    void OnExport(wxCommandEvent& event);
    void OnUpdateExport(wxUpdateUIEvent& event);
    /// @}
protected:
    void    x_LoadUnloadProject(bool load);

    CProjectService*    x_GetProjectService();
    CAppTaskService*    x_GetAppTaskService();

    void    x_LaunchTask(IAppTask* task);
    void    x_GetSelected(TProjectIdVector& projects, bool* workspace = NULL);

protected:
    IServiceLocator*  m_ServiceLocator;
};


END_NCBI_SCOPE


#endif  // GUI_CORE___PROJECT_CMD_HANDLER__HPP

