#ifndef GUI_CORE___PROJECT_FOLDER__HPP
#define GUI_CORE___PROJECT_FOLDER__HPP

/*  $Id: project_folder.hpp 20748 2010-01-27 19:03:26Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>

#include <gui/gui_export.h>

#include <gui/core/project_item.hpp>

#include <gui/objects/ProjectFolder.hpp>

#include <objmgr/scope.hpp>


BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// CProjectFolderConstPrx
class NCBI_GUICORE_EXPORT CProjectFolderConstPrx : public CObject
{
    friend class CProjectPrx;
    friend class CProjectConstPrx;
    friend class CProjectItemPrx;
    friend class CProjectItemConstPrx;
public:
    typedef int TId;
    typedef vector< CRef<CProjectFolderConstPrx> >  TFolderConstVector;
    typedef vector< CRef<CProjectItemConstPrx> >    TProjectItemConstVector;

public:
    TId GetId() const;

    string  GetName() const;
    string  GetDescr() const;
    bool    GetOpen() const;

    CRef<CProjectItemConstPrx>
            FindProjectItemById_C(CProjectItemConstPrx::TId id, bool recursive);
    CRef<CProjectItemConstPrx>
            FindProjectItemByData_C(const CSerialObject& data, bool recursive);

    CRef<CProjectFolderConstPrx> FindChildFolderById_C(TId id, bool recursive);

    void    GetChildFolders_C(TFolderConstVector& folders);
    void    GetProjectItems_C(TProjectItemConstVector& prj_items);

protected:
    CProjectFolderConstPrx(objects::CProjectFolder& folder,
                           CProjectConstPrx& project);

    /// applies the given visitor to all project items until the visitor returns false
    bool    x_ForEachProjectItem(objects::CProjectFolder::IProjectItemVisitor& visitor,
                                 bool recursive);

protected:
    CRef<objects::CProjectFolder>  m_Folder;
    CProjectConstPrx&   m_ProjectConst;
};


////////////////////////////////////////////////////////////////////////////////
/// CProjectFolderPrx
class NCBI_GUICORE_EXPORT CProjectFolderPrx : public CProjectFolderConstPrx
{
    friend class CProjectPrx;
    friend class CProjectItemPrx;
public:
    typedef vector< CRef<CProjectFolderPrx> > TFolderVector;
    typedef vector< CRef<CProjectItemPrx> > TProjectItemVector;

    void    SetName(const string& name);
    void    SetDescr(const string& descr);
    void    SetOpen(bool open);

    void    GetChildFolders(TFolderVector& folders);
    void    GetProjectItems(TProjectItemVector& prj_items);

    CRef<CProjectFolderPrx>   GetParentFolder();

    // find a child item (immediate)
    CRef<CProjectItemPrx>
            FindProjectItemById(CProjectItemPrx::TId id, bool recursive);
    CRef<CProjectItemPrx>
            FindProjectItemByData(const CSerialObject& data, bool recursive);

    // find a child item (immediate)
    CRef<CProjectItemPrx>   FindProjectItemByName(const string& name);

    CRef<CProjectFolderPrx> FindChildFolderById(TId id, bool recursive);

    // find immediate child by name
    CRef<CProjectFolderPrx> FindChildFolderByName(const string& name);

    CRef<CProjectItemPrx>   AddProjectItem(const string& label,
                                           CSerialObject& obj,
                                           const string& descr = kEmptyStr);
    // TODO - deprecated
    CRef<CProjectItemPrx>   AddProjectItem(objects::CProjectItem& item);

    CRef<CProjectFolderPrx> AddChildFolder(const string& name,
                                           const string& descr);

    /// removes a child Project Folder and all its contents from this folder
    void    RemoveChildFolder(TId id, bool remove_children);

    void    RemoveChildItem(CProjectItemPrx::TId id);

protected:
    CProjectFolderPrx(objects::CProjectFolder& folder, CProjectPrx& project);

    void    x_AddChildItem(objects::CProjectItem& child_item);
    void    x_AddChildFolder(objects::CProjectFolder& child_folder);

    void    x_RemoveChildren();

protected:
    CProjectPrx*    m_Project;
};


END_NCBI_SCOPE

#endif  /// GUI_CORE___PROJECT_FOLDER__HPP
