#ifndef GUI_CORE___PROJECT_ITEM__HPP
#define GUI_CORE___PROJECT_ITEM__HPP

/*  $Id: project_item.hpp 26413 2012-09-13 16:53:52Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>

#include <gui/gui_export.h>

#include <gui/objects/ProjectItem.hpp>


BEGIN_NCBI_SCOPE

class CProjectFolderConstPrx;
class CProjectFolderPrx;
class CProjectConstPrx;
class CProjectPrx;
    
////////////////////////////////////////////////////////////////////////////////
/// CProjectItemConstPrx
class NCBI_GUICORE_EXPORT CProjectItemConstPrx : public CObject
{
    friend class CProjectFolderConstPrx;
    friend class CProjectFolderPrx;
    friend class CProjectConstPrx;
    friend class CProjectPrx;
public:
    typedef int TId;
public:
    TId GetId() const;

    bool    IsEnabled() const;
    string  GetName() const;
    string  GetDescr() const;

    const CSerialObject* GetObject() const;

    CRef<CProjectFolderConstPrx>   GetParentFolder_C();

protected:
    CProjectItemConstPrx(objects::CProjectItem& item, CProjectConstPrx& project);

protected:
    CRef<objects::CProjectItem>  m_Item;
    CProjectConstPrx&    m_ProjectConst;
};


////////////////////////////////////////////////////////////////////////////////
/// CProjectItemPrx
class NCBI_GUICORE_EXPORT CProjectItemPrx : public CProjectItemConstPrx
{
    friend class CProjectFolderPrx;
public:
    void    Enable(bool en);
    void    SetName(const string& name);
    void    SetDescr(const string& descr);

    void    SetObject(CSerialObject& object);

    CRef<CProjectFolderPrx>   GetParentFolder();

    // call this function to indicate that data object associated with the item
    // has changed
    void    OnItemDataChanged();

protected:
    CProjectItemPrx(objects::CProjectItem& item, CProjectPrx& project);

protected:
    CProjectPrx* m_Project;
};


END_NCBI_SCOPE

#endif  /// GUI_CORE___PROJECT_ITEM__HPP
