#ifndef GUI_CORE___TABLE_OBJECT_LOADER__HPP
#define GUI_CORE___TABLE_OBJECT_LOADER__HPP

/*  $Id: table_object_loader.hpp 27061 2012-12-13 15:49:53Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*
* File Description:
*
*/

#include <corelib/ncbistd.hpp>

#include <gui/utils/execute_unit.hpp>
#include <gui/utils/object_loader.hpp>

#include <gui/packages/pkg_sequence/gff_load_params.hpp>

#include <objects/seq/Seq_annot.hpp>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_CORE
 *
 * @{
 */

class CTableAnnotDataSource;

///////////////////////////////////////////////////////////////////////////////
/// CTableObjectLoader
class CTableObjectLoader : public CObject, public IObjectLoader, public IExecuteUnit
{
public:
    typedef CRef<objects::CSeq_annot> TAnnotRef;

    CTableObjectLoader(CTableAnnotDataSource& annotTableData, const vector<wxString>& filenames);

    /// @name IObjectLoader implementation
    /// @{
    virtual TObjects& GetObjects();
    virtual string GetDescription() const;
    /// @}

    /// @name IExecuteUnit implementation
    /// @{
    virtual bool PreExecute() { return true; }
    virtual bool Execute(ICanceled&) { return true; }
    virtual bool PostExecute()  { return true; }
    /// @}

private:
    TObjects m_Objects;
};

/* @} */

END_NCBI_SCOPE


#endif // GUI_CORE___TABLE_OBJECT_LOADER__HPP
