#ifndef GUI_CORE___UNDO_MANAGER__HPP
#define GUI_CORE___UNDO_MANAGER__HPP

/*  $Id: undo_manager.hpp 26031 2012-06-29 15:00:37Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <gui/gui_export.h>
#include <gui/objutils/undo_manager_interface.hpp>

BEGIN_NCBI_SCOPE

/////////////////////////////////////////////////////////////////////////////

class CGBDocument;

/////////////////////////////////////////////////////////////////////////////

class NCBI_GUICORE_EXPORT CUndoManager : public CObject, public IUndoManager
{
public:
    CUndoManager(CGBDocument* document = NULL) : m_Document(document) {}
    virtual ~CUndoManager() {}

    // IUndoManager
    virtual void Execute(IEditCommand* command);
    virtual void Undo();
    virtual void Redo();
    virtual bool CanUndo();
    virtual bool CanRedo();
    virtual string GetUndoLabel();
    virtual string GetRedoLabel();

    void Reset();

protected:
    typedef list< CIRef<IEditCommand> > TCommands;

    void ShowErrorMsg (IEditCommand* pCommand, const string& errMsg);
    void x_SendChangingEvent();
    void x_SendChangedEvent();

    TCommands m_UndoBuffer;
    TCommands m_RedoBuffer;
    CGBDocument* m_Document;
};

END_NCBI_SCOPE

#endif  // GUI_CORE___UNDO_MANAGER__HPP
