#ifndef GUI_CORE__WINMASK_FILES__HPP
#define GUI_CORE__WINMASK_FILES__HPP

/*  $Id $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Yury Voronov
 *
 * File Description:
 *
 */

#include <gui/gui_export.h>

#include <corelib/ncbiobj.hpp>

#include <wx/string.h>

class wxWindow;

BEGIN_NCBI_SCOPE

class NCBI_GUICORE_EXPORT CWinMaskerFileStorage : public CObject
{    
public:
    CWinMaskerFileStorage();
    virtual ~CWinMaskerFileStorage();

    /// @name Singleton maintenance
    /// @{

    /// access the application-wide singleton
    static CWinMaskerFileStorage& GetInstance();

    /// @}

    void SetRegistryPath( const string& path );
    void SaveSettings() const;
    void LoadSettings();

    wxString GetPath() const;
    wxString GetLocalPath() const;
    wxString GetPredefinedPath() const;
    bool GetEnvPredefined() const { return m_EnvVarPreferred; }

    void SetLocalPath( const wxString& path );
    void ResetPredefinedPath() { m_EnvVarPreferred = true; };

    void GetWinMaskerTaxIds( vector<int>& available_win_mask_ids );

private:

    string m_RegPath;
    wxString m_LocalWinMaskDirPath;
    bool m_EnvVarPreferred;
};

END_NCBI_SCOPE

#endif // GUI_CORE__WINMASK_FILES__HPP
