/*  $Id: winmask_files_dlg.hpp 23363 2011-03-14 22:45:59Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Yury Voronov
 *
 * File Description:
 *
 */
#ifndef _WINMASK_FILES_DLG_H_
#define _WINMASK_FILES_DLG_H_

#include <corelib/ncbistd.hpp>

#include <gui/widgets/wx/dialog.hpp>
#include <wx/dialog.h>


/*!
 * Forward declarations
 */

class wxTextCtrl;
class wxButton;
class wxCheckListBox;
class wxStaticText;
class wxCheckBox;

////@begin forward declarations
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CWINMASKFILESDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CWINMASKFILESDLG_TITLE _("Window Masker files information")
#define SYMBOL_CWINMASKFILESDLG_IDNAME ID_CWIN_MASK_FILES_DLG
#define SYMBOL_CWINMASKFILESDLG_SIZE wxDefaultSize
#define SYMBOL_CWINMASKFILESDLG_POSITION wxDefaultPosition
////@end control identifiers

BEGIN_NCBI_SCOPE

struct CWinMaskFilesParams {

    wxString m_LocalPath;
    wxString m_PredefPath;
    bool m_EnvPreferred;
    vector<string> m_Masks;
    vector<int> m_ChosenIdxs;
    string m_HelpText;
};

/*!
 * CWinMaskFilesDlg class declaration
 */

class NCBI_GUICORE_EXPORT CWinMaskFilesDlg : public CDialog
{    
    DECLARE_DYNAMIC_CLASS( CWinMaskFilesDlg )
    DECLARE_EVENT_TABLE()

public:
    CWinMaskFilesDlg();
    CWinMaskFilesDlg( wxWindow* parent, wxWindowID id = SYMBOL_CWINMASKFILESDLG_IDNAME, const wxString& caption = SYMBOL_CWINMASKFILESDLG_TITLE, const wxPoint& pos = SYMBOL_CWINMASKFILESDLG_POSITION, const wxSize& size = SYMBOL_CWINMASKFILESDLG_SIZE, long style = SYMBOL_CWINMASKFILESDLG_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CWINMASKFILESDLG_IDNAME, const wxString& caption = SYMBOL_CWINMASKFILESDLG_TITLE, const wxPoint& pos = SYMBOL_CWINMASKFILESDLG_POSITION, const wxSize& size = SYMBOL_CWINMASKFILESDLG_SIZE, long style = SYMBOL_CWINMASKFILESDLG_STYLE );

    ~CWinMaskFilesDlg();

    void Init();

    void CreateControls();
    
    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

////@begin CWinMaskFilesDlg event handler declarations

    void OnWinmaskPathTextUpdated( wxCommandEvent& event );

    void OnBrowseBtnClick( wxCommandEvent& event );

    void OnEnvPreferredClick( wxCommandEvent& event );

////@end CWinMaskFilesDlg event handler declarations

    void x_OnEnvPreferred( bool preferred );

////@begin CWinMaskFilesDlg member function declarations

    CWinMaskFilesParams GetParams() const { return m_Params ; }
    void SetParams(CWinMaskFilesParams value) { m_Params = value ; }

    wxBitmap GetBitmapResource( const wxString& name );

    wxIcon GetIconResource( const wxString& name );
////@end CWinMaskFilesDlg member function declarations

    static bool ShowToolTips();

////@begin CWinMaskFilesDlg member variables
    wxTextCtrl* m_WinMaskPathCtrl;
    wxButton* m_WinMaskPathBtn;
    wxCheckBox* m_PreferredWay;
    wxCheckListBox* m_MasksList;
    wxStaticText* m_HelpLine;
private:
    CWinMaskFilesParams m_Params;
    wxString m_DirPath;
    enum {
        ID_CWIN_MASK_FILES_DLG = 10012,
        ID_WINMASK_PATH = 10015,
        ID_PREFERRED_WAY = 10016,
        ID_BROWSE_BTN = 10017,
        ID_CHECKLISTBOX = 10034
    };
////@end CWinMaskFilesDlg member variables
};

/* @} */

END_NCBI_SCOPE

#endif // _WINMASK_FILES_DLG_H_
