#ifndef GUI_CORE___WORKSPACE_FOLDER__HPP
#define GUI_CORE___WORKSPACE_FOLDER__HPP

/*  $Id: workspace_folder.hpp 22695 2010-12-14 20:19:11Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */
#include <corelib/ncbistl.hpp>

#include <gui/gui_export.h>

#include <gui/core/project.hpp>

#include <gui/objects/WorkspaceFolder.hpp>


BEGIN_NCBI_SCOPE

class CWorkspaceConstPrx;
class CWorkspacePrx;
class CProjectService;


BEGIN_SCOPE(objects)
END_SCOPE(objects)

///////////////////////////////////////////////////////////////////////////////
/// CWorkspaceFolderConstPrx
class NCBI_GUICORE_EXPORT CWorkspaceFolderConstPrx : public CObject
{
    friend class CWorkspaceConstPrx;
    friend class CWorkspacePrx;
    friend class CProjectService; //TODO
public:
    typedef objects::CWorkspaceFolder::TId  TId;
    typedef vector< CRef<CWorkspaceFolderConstPrx> > TFolderConstVector;
    typedef vector< CRef<CProjectPrx> > TProjectVector;

public:
    TId     GetId() const;
    string  GetName() const;
    string  GetDescr() const;

    void    GetChildFolders(TFolderConstVector& folders);
    void    GetProjects(TProjectVector& folders);

    CRef<CWorkspaceFolderConstPrx>
            FindChildFolderById_C(TId id, bool recursive = false);

    CRef<CProjectConstPrx>   FindProjectById_C(CProjectPrx::TId id, bool recursive);

    CRef<CProjectPrx>   FindProjectById(CProjectPrx::TId id, bool recursive);
    CRef<CProjectPrx>   FindProjectByFilename(const wxString& filename, bool recursive);

protected:
    CWorkspaceFolderConstPrx(objects::CWorkspaceFolder& folder,
                             CWorkspaceConstPrx& workspace);

    /// applies the given visitor to all projects until visitor returns false
    bool    x_ForEachProject(objects::CWorkspaceFolder::IProjectVisitor& visitor,
                             bool recursive);

protected:
    /// the folder this proxy refers to
    CRef<objects::CWorkspaceFolder> m_Folder;

    CWorkspaceConstPrx& m_WorkspaceConst;
    CProjectService*    m_Service; // TODO
};


///////////////////////////////////////////////////////////////////////////////
/// CWorkspaceFolderPrx
class NCBI_GUICORE_EXPORT CWorkspaceFolderPrx : public CWorkspaceFolderConstPrx
{
    friend class CWorkspaceConstPrx;
    friend class CWorkspacePrx;
    friend class CProjectService; //TODO
public:
    typedef vector< CRef<CWorkspaceFolderPrx> > TFolderVector;

    void    SetName(const string& name);
    void    SetDescr(const string& descr);

    void    GetChildFolders(TFolderVector& folders);

    CRef<CWorkspaceFolderPrx>   FindChildFolderById(TId id, bool recursive = false);
    CRef<CWorkspaceFolderPrx>   FindChildFolderByName(const string& name);

    CRef<CWorkspaceFolderPrx>   AddChildFolder(const string& name,
                                               const string& descr);
    void    RemoveChildFolder(TId child_id);

protected:
    CWorkspaceFolderPrx(objects::CWorkspaceFolder& folder, CWorkspacePrx& workspace);

    void    x_AddProject(objects::CGBProjectHandle& project);

protected:
    CWorkspacePrx* m_Workspace;
};


END_NCBI_SCOPE

#endif  /// GUI_CORE___WORKSPACE_FOLDER__HPP
