#ifndef GUI_FRAMEWORK___MEMENTO__HPP
#define GUI_FRAMEWORK___MEMENTO__HPP

/*  $Id: memento.hpp 18397 2008-11-28 19:45:17Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

/** @addtogroup GUI_FRAMEWORK
*
* @{
*/

#include <corelib/ncbiobj.hpp>

#include <gui/gui_export.h>


BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects);
    class CUser_object;
END_SCOPE(objects);


///////////////////////////////////////////////////////////////////////////////
/// IMemento - abstract base for Memento objects.
class NCBI_GUIFRAMEWORK_EXPORT IMemento
{
public:
    virtual ~IMemento() {};
};


///////////////////////////////////////////////////////////////////////////////
/// IMementoOriginator
class NCBI_GUIFRAMEWORK_EXPORT IMementoOriginator
{
public:
    /// create an object describing internal state of the Data Source
    virtual IMemento*   GetMemento() const = 0;

    /// initialize the object with the memento
    virtual void    SetMemento(const IMemento& memento) = 0;

    virtual ~IMementoOriginator() {};
};


///////////////////////////////////////////////////////////////////////////////
/// IMemento - abstract base for Memento objects
class NCBI_GUIFRAMEWORK_EXPORT IUserObjectConvertible
{
public:
    virtual objects::CUser_object*  ToUserObject() const = 0;
    virtual void    FromUserObject(const objects::CUser_object& user_obj) = 0;

    virtual ~IUserObjectConvertible() {};
};


END_NCBI_SCOPE

/* @} */

#endif  // GUI_FRAMEWORK___MEMENTO__HPP

