#ifndef GUI_FRAMEWORK___UPDATE_LOCK__HPP
#define GUI_FRAMEWORK___UPDATE_LOCK__HPP

/*  $Id: update_lock.hpp 18397 2008-11-28 19:45:17Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software / database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software / database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

/** @addtogroup GUI_FRAMEWORK
*
* @{
*/

#include <corelib/ncbistd.hpp>


BEGIN_NCBI_SCOPE

//////////////////////////////////////////////////////////////////////////////
/// CUpdateGuard - works like CGuard, but relies on functions
/// LockUpdates() and UnlockUpdates()

template <class Resource>   class CUpdateGuard
{
public:
    typedef Resource TResourceType;

    /// Manually force the guard to protect some other resource.
    explicit CUpdateGuard(TResourceType& resource)
        :   m_Resource(NULL)
    {
        Release();
        resource.LockUpdates();
        m_Resource = &resource;
    }

    /// Destructor releases the resource.
    ~CUpdateGuard()
    {
        try {
            Release();
        } catch(std::exception& ) {
            // catch and ignore std exceptions in destructor
        }
    }

    /// Manually force the resource to be released.
    void Release()
    {
        if (m_Resource) {
            m_Resource->UnlockUpdates();
            m_Resource = NULL;
        }
    }

private:
    TResourceType*  m_Resource;

private:
    // prevent copying
    CUpdateGuard(const CUpdateGuard<TResourceType>&);
    void operator=(const CUpdateGuard<TResourceType>&);
};


END_NCBI_SCOPE

/* @} */

#endif  /// GUI_FRAMEWORK___UPDATE_LOCK__HPP
