/* $Id: ProjectItem.hpp 27086 2012-12-17 19:43:41Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

/// @ProjectItem.hpp
/// User-defined methods of the data storage class.
///
/// This file was originally generated by application DATATOOL
/// using the following specifications:
/// 'gui_project.asn'.
///
/// New methods or data members can be added to it if needed.
/// See also: ProjectItem_.hpp


#ifndef GUI_CORE_PROJECTITEM_HPP
#define GUI_CORE_PROJECTITEM_HPP


// generated includes
#include <corelib/ncbitime.hpp>
#include <gui/objects/ProjectItem_.hpp>

#include <objmgr/scope.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <objmgr/seq_annot_handle.hpp>

// generated classes

BEGIN_NCBI_SCOPE

class CProjectService;
class IProjectItemAssistant;


BEGIN_objects_SCOPE // namespace ncbi::objects::

class CDate;
class CProjectFolder;

/////////////////////////////////////////////////////////////////////////////
///
class NCBI_GUIOBJECTS_EXPORT CProjectItem : public CProjectItem_Base
{
    typedef CProjectItem_Base Tparent;
public:
    CProjectItem(void);
    ~CProjectItem(void);

    /// SetCreateDate() will add a descriptor for creation date, and insure
    /// that only one such descriptor exists
    void SetCreateDate(const CTime& time);
    void SetCreateDate(const CDate& date);

    /// SetModifiedDate() will add a descriptor for the update date, and insure
    /// that only one such descriptor exists
    void SetModifiedDate(const CTime& time);
    void SetModifiedDate(const CDate& date);

    /// wrapper for setting the object pointed to by this item
    void SetObject(CSerialObject& object);

    /// retrieve the object pointed to as a CObject*
    const CSerialObject* GetObject() const;

    /// automatically set label based on the data
    string  GetLabelByData(CScope* scope = NULL) const;

    /// enabled flag
    bool IsEnabled(void) const;

    void    SetParentFolder(CProjectFolder* parent);
    CProjectFolder* GetParentFolder();
    const CProjectFolder* GetParentFolder() const;

    /// refresh any indexed data.
    /// Since a project item holds a set of handles, this will insure that
    /// the data are correctly editable first
    void Refresh();

    /// @name Object Manager Interface
    /// @{
    void Attach(CScope& scope);
    void Detach(CScope& scope);
    /// @}

private:
    // Prohibit copy constructor and assignment operator
    CProjectItem(const CProjectItem& value);
    CProjectItem& operator=(const CProjectItem& value);

protected:
    CProjectFolder* m_ParentFolder;

    list<CSeq_entry_Handle> m_Seq_entry_Handles;
    list<CSeq_annot_Handle> m_Seq_annot_Handles;
};

/////////////////// CProjectItem inline methods

/////////////////// end of CProjectItem inline methods

END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE

#endif // GUI_CORE_PROJECTITEM_HPP
/* Original file checksum: lines: 94, chars: 2592, CRC32: 25ddc98d */
