#ifndef GUI_OBJUTILS___ENTREZ_CACHE__HPP
#define GUI_OBJUTILS___ENTREZ_CACHE__HPP

/*  $Id: entrez_cache.hpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <corelib/ncbimtx.hpp>
#include <corelib/hash_map.hpp>

#include <gui/gui_export.h>


BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects)
    class CEntrez2_docsum_list;
    class CEntrez2_docsum;
END_SCOPE(objects)

///
/// class CEntrezCache provides a simple, thread-safe interface for storing
/// the temporary results of queries to Entrez using CEntrez2Client.  This is useful
/// for avoiding calling a network search service multiple times per object
///
class NCBI_GUIOBJUTILS_EXPORT CEntrezCache : public CObject
{
public:
    /// the key for Entrez objects is the namespace (=database) and the UID
    struct SKey {
        string db;
        int    uid;

        SKey()
            : uid(0)
        {
        }

        bool operator==(const SKey& other) const
        {
            return db  == other.db  &&
                   uid == other.uid;
        }
    };
    struct SKeyHash : public unary_function<SKey, size_t>
    {
        size_t operator()(const SKey& key) const
        {
            return hash<string>()(key.db) + hash<int>()(key.uid);
        }
    };
    typedef hash_map<SKey, CConstRef<objects::CEntrez2_docsum>, SKeyHash> TCache;

    /// the default ctor establishes an appropriate hash table size
    CEntrezCache();
    ~CEntrezCache();

    /// Store a list of docsums in the internal hash
    void Put(const string& db, objects::CEntrez2_docsum_list& docsums);

    /// Store a single docsum in the internal hash
    void Put(const string& db, objects::CEntrez2_docsum&      docsum);

    /// retrieve a single docsum
    CConstRef<objects::CEntrez2_docsum> Get(const string& db, int uid) const;

    /// retrieve a singk
    static CEntrezCache& GetInstance();

private:
    CFastMutex m_Mutex;
    TCache m_Cache;

    static CRef<CEntrezCache> m_CacheInstance;

    /// forbidden
    CEntrezCache(const CEntrezCache&);
    CEntrezCache& operator=(const CEntrezCache&);
};


END_NCBI_SCOPE


#endif  // GUI_OBJUTILS___ENTREZ_CACHE__HPP
