#ifndef GUI_OBJUTILS___ENTREZ_CONVERSION__HPP
#define GUI_OBJUTILS___ENTREZ_CONVERSION__HPP

/*  $Id: entrez_conversion.hpp 14562 2007-05-18 11:48:21Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>

#include <objects/seqloc/Seq_id.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <objects/entrez2/entrez2_client.hpp>
#include <objects/entrez2/Entrez2_docsum.hpp>
#include <objects/entrez2/Entrez2_docsum_list.hpp>
#include <objects/entrezgene/Entrezgene_Set.hpp>


BEGIN_NCBI_SCOPE

///////////////////////////////////////////////////////////////////////////////
/// CEntrezDB
class NCBI_GUIOBJUTILS_EXPORT   CEntrezDB {
public:
    typedef objects::CEntrez2_docsum TDocsum;
    typedef list< CConstRef<TDocsum> > TDocsumList;
    typedef pair<string, string>    TNamePair;

    static void     GetDbNames(vector<string>& names);
    static void     GetDbNames(vector<TNamePair>& name_pairs);
    static string   GetVisibleName(const string& db_name);
    static string   GetDbName(const string& visible_name);
    static const string&    GetDbFieldName();

    /// if "add_db_field" == true a special filed holding the "db_name" value
    /// will be added to return CEntrez2_docsum objects
    static CRef<objects::CEntrez2_docsum_list>
            Query(objects::CEntrez2Client& client,
                  const string& db_name,
                  const string& terms,
                  size_t& total_uids,
                  size_t start,
                  size_t count,
                  bool add_db_field);


    static CRef<objects::CSeq_annot>
            CreateAnnot_Nuc_Prot(const string& query_str,
                                 const TDocsumList& ds_list);

    static CRef<objects::CSeq_id>
            CreateId_Nuc_Prot(const string& query_str, const TDocsum& ds);

    static CRef<objects::CEntrezgene> CreateGene_Gene(const TDocsum& ds);

    static CRef<objects::CEntrezgene_Set>
            CreateGeneSet_Gene(const TDocsumList& ds_list);

    static string
            CreateEntrezLinkCountLabel(const objects::CEntrez2_link_count& lc);


    static CRef<objects::CSeq_annot>
            CreateAnnot_Genome(const string& query_str,
                               const TDocsumList& ds_list,
                               const string& vis_db_name);

    static string
            CreateAnnotTitle_GenomeProject(const objects::CEntrez2_docsum& ds,
                                           const objects::CEntrez2_link_count& lc);

    static CRef<objects::CSeq_annot>
            CreateSeqAnnot_GenomeProject(objects::CEntrez2Client& client,
                                         const objects::CEntrez2_docsum& ds,
                                         const objects::CEntrez2_link_count& lc,
                                         const string& db_name);
};


END_NCBI_SCOPE

#endif  /// GUI_OBJUTILS___ENTREZ_CONVERSION__HPP
