#ifndef GUI_OBJUTILS___CLIPBOARD__HPP
#define GUI_OBJUTILS___CLIPBOARD__HPP

/*  $Id: obj_clipboard.hpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */


#include <corelib/ncbiobj.hpp>
#include <objmgr/scope.hpp>
#include <gui/gui.hpp>
#include <gui/objutils/objects.hpp>


/** @addtogroup GUI_OBJUTILS
 *
 * @{
 */


BEGIN_NCBI_SCOPE


class NCBI_GUIOBJUTILS_EXPORT CObjClipboard
{
public:

    // modes for text copy
    enum ECopyMode {
        eCopy_Label,
        eCopy_ASN,
        eCopy_XML
    };

    // clear the contents of the current clipboard
    NCBI_DEPRECATED static void Clear(void);

    // Copy an object to the clipboard.  This version will use a best-fit
    // to try and find the text to be copied.
    NCBI_DEPRECATED static void Add(objects::CScope& scope, const CObject& object);

    // Copy an object to the clipboard.  This version will use a supplied
    // text parameter for its copied text; the text parameter overrides
    // the current text copy mode.
    NCBI_DEPRECATED static void Add(objects::CScope& scope, const CObject& object,
                    const string& text);

    // Copy just a piece of text to the clipboard.
    NCBI_DEPRECATED static void Add(const string& text);
private:
    // the mode of text copy
    static ECopyMode sm_Mode;


    // append an appropriate piece of text onto the passed string
    static void x_GetText(objects::CScope& scope,
                          const CObject& object,
                          string* text);

    // forbidden
    CObjClipboard();
    CObjClipboard(const CObjClipboard&);
    CObjClipboard& operator=(const CObjClipboard&);
};


END_NCBI_SCOPE

/* @} */

#endif  // GUI_OBJUTILS___CLIPBOARD__HPP
