#ifndef GUI_UTILS___OBJECTS__HPP
#define GUI_UTILS___OBJECTS__HPP

/*  $Id: objects.hpp 24177 2011-08-09 14:27:04Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <objmgr/scope.hpp>

/** @addtogroup GUI_UTILS
 *
 * @{
 */


BEGIN_NCBI_SCOPE


struct SScopedObject
{
    SScopedObject() { }
    SScopedObject(CObject& o, objects::CScope& s)
        : object(&o)
        , scope(&s) { }

    CRef<CObject>         object;
    CRef<objects::CScope> scope;
};

inline
bool operator==(const SScopedObject& s1, const SScopedObject& s2)
{
    return (s1.object == s2.object  && s1.scope == s2.scope);
}

struct SScopedObjectLess
{
    bool operator()(const SScopedObject& s1, const SScopedObject& s2) const
    {
        return (s1.object < s2.object);
    }
};


struct SConstScopedObject
{
    SConstScopedObject() { }
    SConstScopedObject(const CObject& o, objects::CScope& s)
        : object(&o)
        , scope(&s) { }

    CConstRef<CObject>    object;
    CRef<objects::CScope> scope;
};

inline
bool operator==(const SConstScopedObject& s1, const SConstScopedObject& s2)
{
    return (s1.object == s2.object  && s1.scope == s2.scope);
}

struct SConstScopedObjectLess
{
    bool operator()(const SConstScopedObject& s1,
                    const SConstScopedObject& s2) const
    {
        return (s1.object < s2.object);
    }
};


typedef vector< CRef<CObject> >      TObjects;
typedef vector< CConstRef<CObject> > TConstObjects;
typedef vector<SScopedObject>        TScopedObjects;
typedef vector<SConstScopedObject>   TConstScopedObjects;
typedef map<string, TConstScopedObjects> TRelatedObjectsMap;

typedef pair<SConstScopedObject, CConstRef<CObject> > TChangeObject;
typedef vector<TChangeObject> TChangeObjects;

END_NCBI_SCOPE

/* @} */

#endif  // GUI_UTILS___OBJECTS__HPP
