#ifndef GUI_OBJUTILS___TAXON_CACHE__HPP
#define GUI_OBJUTILS___TAXON_CACHE__HPP

/*  $Id: taxon_cache.hpp 27909 2013-04-23 21:12:43Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin, Anatoliy Kuznetsov
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>
#include <objects/seqfeat/Org_ref.hpp>
#include <objects/taxon1/taxon1.hpp>

BEGIN_NCBI_SCOPE


class NCBI_GUIOBJUTILS_EXPORT CTaxonCache : public CObject
{
public:
    static CTaxonCache& GetInstance(const string& cache_path);

    void Initialize(const vector<int>& tax_ids);
    string GetLabel(int tax_id);

    /// Drop connection to NCBI taxon service
    void ResetConnection();

private:
    /// forbidden
    CTaxonCache(const string& cache_path);
    ~CTaxonCache();
    CTaxonCache(const CTaxonCache&);
    CTaxonCache& operator=(const CTaxonCache&);

    // internal GetLabel (not thread-protected)
    void x_GetLabel(int tax_id, string& label);

    typedef map<int, string> TMap;

private:
    string                      m_CachePath;
    TMap                        m_Map;
    auto_ptr<objects::CTaxon1>  m_TaxonClient;
};

END_NCBI_SCOPE

#endif  // GUI_OBJUTILS___TAXON_CACHE__HPP
