#ifndef GUI_OPENGL___GLVIEWPORT__HPP
#define GUI_OPENGL___GLVIEWPORT__HPP

/*  $Id: glviewport.hpp 14562 2007-05-18 11:48:21Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>
#include <gui/opengl.h>
#include <gui/gui.hpp>


/** @addtogroup GUI_OPENGL
 *
 * @{
 */

BEGIN_NCBI_SCOPE


//
// class CGlViewport provides a simple abstraction for viewports in OpenGL
//
// Remember that, in OpenGL, the position (0, 0) is the lower left corner, not
// the upper left corner (inverted with respect to most windowing systems)
//

class NCBI_GUIOPENGL_EXPORT CGlViewport
{
public:
    CGlViewport();
    CGlViewport(GLint x, GLint y, GLint w, GLint h);

    // make the viewport current.  The boolean parameter controls whether the
    // previous viewport settings are saved via glPushAttrib(GL_VIEWPORT_BIT).
    // The default is to overwrite the current settings; if 'true' is passed
    // in, be sure to call ReleaseCurrent() when completed.
    void MakeCurrent(bool save_viewport = false);

    // release the current viewport.  This is a NULL op unless 'true' is passed
    // in, at which point this call is equivalent to glPopAttrib()
    void ReleaseCurrent(bool restore_viewport = false);

    //
    // access the dimensions of the viewport
    //
    GLint GetX     (void) const;
    GLint GetY     (void) const;
    GLint GetWidth (void) const;
    GLint GetHeight(void) const;

    void SetX     (GLint i);
    void SetY     (GLint i);
    void SetWidth (GLint i);
    void SetHeight(GLint i);

private:
    // the actual viewport params.
    GLint m_X;
    GLint m_Y;
    GLint m_Width;
    GLint m_Height;
};


//
// inlined accessors
//

inline
GLint CGlViewport::GetX(void) const
{
    return m_X;
}


inline
GLint CGlViewport::GetY(void) const
{
    return m_Y;
}


inline
GLint CGlViewport::GetWidth(void) const
{
    return m_Width;
}


inline
GLint CGlViewport::GetHeight(void) const
{
    return m_Height;
}


inline
void CGlViewport::SetX(GLint i)
{
    m_X = i;
}


inline
void CGlViewport::SetY(GLint i)
{
    m_Y = i;
}


inline
void CGlViewport::SetWidth(GLint i)
{
    m_Width = i;
}


inline
void CGlViewport::SetHeight(GLint i)
{
    m_Height = i;
}

END_NCBI_SCOPE

/* @} */

#endif  // GUI_OPENGL___GLVIEWPORT__HPP
