#ifndef GUI_OPENGL___GL_ZOOM_LENS__HPP
#define GUI_OPENGL___GL_ZOOM_LENS__HPP

/*  $Id: glzoomlens.hpp 14562 2007-05-18 11:48:21Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Peter Meric
 *
 * File Description:
 *     CGlZoomLens - OpenGL zoom lens
 */

#include <corelib/ncbiobj.hpp>
#include <gui/gui.hpp>
#include <gui/opengl/glwidget.hpp>
#include <gui/opengl/glevent.hpp>
#include <gui/opengl/glcamera.hpp>
#include <gui/utils/function.hpp>


/** @addtogroup GUI_OPENGL
 *
 * @{
 */

BEGIN_NCBI_SCOPE


struct GlZoomSpec
{
    enum EZoomLensType
    {
        eQuad = 1,
        eFullHeight = 2,
        eFullWidth = 4
    };

    unsigned int type;
    float xratio;
    float yratio;

    struct PixelDim
    {
        int width, height, xpos, ypos;
    } pixel_dim;

    struct GlDim
    {
        float x1, x2, y1, y2;
    } gl_dim;

    CRef<CFuncPtr> DrawFunc;
};


class NCBI_GUIOPENGL_EXPORT CGlZoomLens : public CGlWidget
{
public:
    CGlZoomLens(bool visible = false);
    virtual ~CGlZoomLens();

    virtual void SetProperties(struct GlZoomSpec& spec, CGlCamera& zoomcam);

    virtual void Draw(void);
    virtual void ProcessEvent(CGlEvent& event);

private:
    struct GlZoomSpec m_Spec;
    CGlCamera m_ZoomCamera;
};


END_NCBI_SCOPE

/* @} */

#endif  // GUI_OPENGL___GL_ZOOM_LENS__HPP
