#ifndef PKG_ALIGNMENT___CLUSTALW_TOOL_JOB__HPP
#define PKG_ALIGNMENT___CLUSTALW_TOOL_JOB__HPP

/*  $Id: clustalw_tool_job.hpp 23162 2011-02-15 16:56:51Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*
* File Description:
*
*/

#include <corelib/ncbistd.hpp>

#include <gui/core/loading_app_job.hpp>
#include <gui/packages/pkg_alignment/clustalw_tool_params.hpp>

BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects)
    class CScope;
    class CSeq_loc;
END_SCOPE(objects)

/** @addtogroup GUI_PKG_ALIGNMENT
*
* @{
*/

///////////////////////////////////////////////////////////////////////////////
/// CClustalwToolJob
class  CClustalwToolJob : public CDataLoadingAppJob
{
public:
    CClustalwToolJob(CProjectService* service,
        const CClustalwToolParams& params);

    typedef pair< CConstRef<objects::CSeq_loc>, CRef<objects::CScope> > TLocPair;
    typedef map<string, TLocPair> TIdMap;

protected:
    virtual bool    BeforeRun();
    virtual void    x_CreateProjectItems();

private:
    void x_CleanUp();

    CClustalwToolParams m_Params;

    wxString m_TmpIn;
    wxString m_TmpOut;
    wxString m_TmpTreeOut;
    long m_PId;
    bool m_IsProtein;

    /// to make the alignment as robust as possible, we assign every
    /// sequence a unique identifer independeont of its existing
    /// identifiers
    /// this map allows us to convert back at the end
    TIdMap   m_IdMap;
};

/* @} */

END_NCBI_SCOPE

#endif // PKG_ALIGNMENT___CLUSTALW_TOOL_JOB__HPP
