#ifndef PKG_ALIGNMENT___MUSCLE_TOOL_JOB__HPP
#define PKG_ALIGNMENT___MUSCLE_TOOL_JOB__HPP

/*  $Id: muscle_tool_job.hpp 26428 2012-09-14 21:36:51Z voronov $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*
* File Description:
*
*/

#include <corelib/ncbistd.hpp>

#include <gui/core/loading_app_job.hpp>
#include <gui/packages/pkg_alignment/muscle_tool_params.hpp>

#include <objmgr/seq_loc_mapper.hpp>

BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects)
    class CScope;
    class CSeq_loc;
    class CSeq_id_Handle;
END_SCOPE(objects)

/** @addtogroup GUI_PKG_ALIGNMENT
*
* @{
*/

///////////////////////////////////////////////////////////////////////////////
/// CMuscleToolJob
class  CMuscleToolJob : public CDataLoadingAppJob
{
public:
    CMuscleToolJob(CProjectService* service,
        const CMuscleToolParams& params);

    typedef pair< CConstRef<objects::CSeq_loc>, CRef<objects::CScope> > TLocPair;
    typedef map<string, TLocPair> TIdMap;
    typedef map<objects::CSeq_id_Handle, bool> TSeqTypeMap;

protected:
    virtual bool    BeforeRun();
    virtual void    x_CreateProjectItems();

private:
    void x_CleanUp();

    CMuscleToolParams m_Params;

    wxString m_TmpIn;
    wxString m_TmpOut;
    wxString m_TmpTreeOut;
    long m_PId;

    /// to make the alignment as robust as possible, we assign every
    /// sequence a unique identifer independeont of its existing
    /// identifiers
    /// this map allows us to convert back at the end
    TIdMap   m_IdMap;
    TSeqTypeMap m_SeqTypes;
    objects::CMappingRanges m_Ranges;

    wxString m_CmdLine;
};

/* @} */

END_NCBI_SCOPE

#endif // PKG_ALIGNMENT___MUSCLE_TOOL_JOB__HPP
