#ifndef PKG_ALIGNMENT___PHY_LOAD_PARAMS__HPP
#define PKG_ALIGNMENT___PHY_LOAD_PARAMS__HPP

/*  $Id: phy_load_params.hpp 18626 2009-01-13 17:01:20Z tereshko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 */

#include <corelib/ncbistd.hpp>

#include <gui/objutils/reg_settings.hpp>
#include <algo/phy_tree/dist_methods.hpp>


#include <wx/string.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(ncbi::objects);

/*!
 * CPhyLoadParams class declaration
 */

class CPhyLoadParams: public IRegSettings
{
    friend class CPhyLoadPage;
public:
    /// Default constructor for CPhyLoadParams
    CPhyLoadParams();

    /// Copy constructor for CPhyLoadParams
    CPhyLoadParams(const CPhyLoadParams& data);

    /// Destructor for CPhyLoadParams
    ~CPhyLoadParams();

    /// Assignment operator for CPhyLoadParams
    void operator=(const CPhyLoadParams& data);

    /// Equality operator for CPhyLoadParams
    bool operator==(const CPhyLoadParams& data) const;

    /// Copy function for CPhyLoadParams
    void Copy(const CPhyLoadParams& data);

    /// Initialises member variables
    void Init();

    /// IRegSettings
    virtual void SetRegistryPath(const string& path) { m_RegPath = path; }
    virtual void SaveSettings() const;
    virtual void LoadSettings();

////@begin CPhyLoadParams member function declarations    
    wxString GetFileName()  { return m_FileName ; }
    wxString * SetFileName()  { return &m_FileName ; }
    void SetFileName(wxString value) { m_FileName = value ; }

    wxString GetFileFormat()  { return m_FileFormat ; }
    void SetFileFormat(wxString value) { m_FileFormat = value ; }

    CBioTreeContainer * GetBioTree()  { return m_pBioTree ; }
    void SetBioTree(CBioTreeContainer * value) { m_pBioTree = value ; }

////@end CPhyLoadParams member function declarations

////@begin CPhyLoadParams member variables
private:
    wxString m_FileName;
    wxString m_FileFormat;
    CBioTreeContainer * m_pBioTree;

////@end CPhyLoadParams member variables

private:
    string m_RegPath;
};

END_NCBI_SCOPE

#endif // PKG_ALIGNMENT___PHY_LOAD_PARAMS__HPP
