#ifndef PKG_SEQUENCE___GFF_LOAD_PARAMS__HPP
#define PKG_SEQUENCE___GFF_LOAD_PARAMS__HPP

/*  $Id: gff_load_params.hpp 28165 2013-05-20 20:15:13Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */

#include <corelib/ncbistd.hpp>

#include <gui/objutils/reg_settings.hpp>
#include <util/format_guess.hpp>

#include <wx/string.h>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

/*!
 * CGffLoadParams class declaration
 */

class CGffLoadParams: public IRegSettings
{
    friend class CGffParamsPanel;
public:
    /// Default constructor for CGffLoadParams
    CGffLoadParams();

    /// Copy constructor for CGffLoadParams
    CGffLoadParams(const CGffLoadParams& data);

    /// Destructor for CGffLoadParams
    ~CGffLoadParams();

    /// Assignment operator for CGffLoadParams
    void operator=(const CGffLoadParams& data);

    /// Equality operator for CGffLoadParams
    bool operator==(const CGffLoadParams& data) const;

    /// Copy function for CGffLoadParams
    void Copy(const CGffLoadParams& data);

    /// Initialises member variables
    void Init();

    /// IRegSettings
    virtual void SetRegistryPath(const string& path) { m_RegPath = path; }
    virtual void SaveSettings() const;
    virtual void LoadSettings();

////@begin CGffLoadParams member function declarations
    bool GetNoGTF() const { return m_NoGTF ; }
    void SetNoGTF(bool value) { m_NoGTF = value ; }

    bool GetMergeExons() const { return m_MergeExons ; }
    void SetMergeExons(bool value) { m_MergeExons = value ; }

    bool GetMergeOnyCdsMrna() const { return m_MergeOnyCdsMrna ; }
    void SetMergeOnyCdsMrna(bool value) { m_MergeOnyCdsMrna = value ; }

    bool GetGBQuals() const { return m_GBQuals ; }
    void SetGBQuals(bool value) { m_GBQuals = value ; }

    bool GetCreateGeneFeats() const { return m_CreateGeneFeats ; }
    void SetCreateGeneFeats(bool value) { m_CreateGeneFeats = value ; }

    bool GetSetProducts() const { return m_SetProducts ; }
    void SetSetProducts(bool value) { m_SetProducts = value ; }

    bool GetSetVersion3() const { return m_SetVersion3 ; }
    void SetSetVersion3(bool value) { m_SetVersion3 = value ; }

    int GetParseSeqIds() const { return m_ParseSeqIds ; }
    void SetParseSeqIds(int value) { m_ParseSeqIds = value ; }

    wxString GetNameFeatSet() const { return m_NameFeatSet ; }
    void SetNameFeatSet(wxString value) { m_NameFeatSet = value ; }

    string GetMapAssemblyAcc() const { return m_MapAssemblyAcc ; }
    void SetMapAssemblyAcc(string value) { m_MapAssemblyAcc = value ; }

    bool GetUseAssemblyMapping() const { return m_UseAssemblyMapping ; }
    void SetUseAssemblyMapping(bool value) { m_UseAssemblyMapping = value ; }

////@end CGffLoadParams member function declarations

////@begin CGffLoadParams member variables
private:
    bool m_NoGTF;
    bool m_MergeExons;
    bool m_MergeOnyCdsMrna;
    bool m_GBQuals;
    bool m_CreateGeneFeats;
    bool m_SetProducts;
    bool m_SetVersion3;
    int m_ParseSeqIds;
    wxString m_NameFeatSet;
    string m_MapAssemblyAcc;
    bool m_UseAssemblyMapping;
////@end CGffLoadParams member variables

private:
    string m_RegPath;
};

/* @} */

END_NCBI_SCOPE

#endif // PKG_SEQUENCE___GFF_LOAD_PARAMS__HPP
