#ifndef PKG_SEQUENCE___BTNS_TOOL__HPP
#define PKG_SEQUENCE___BTNS_TOOL__HPP

/*  $Id: btns_tool.hpp 26938 2012-11-28 15:46:40Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Colleen Bollin, based on a file by Roman Katargin
*
* File Description:
*      Implementation of "Find Overlap" tool for GenomeWorkbench.
*/

#include <corelib/ncbistd.hpp>

#include <gui/core/ui_tool_manager.hpp>
#include <gui/utils/extension.hpp>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

///////////////////////////////////////////////////////////////////////////////
/// CEditingBtnsTool
class  CEditingBtnsTool :
    public CObject,
    public IUIAlgoToolManager,
    public IExtension
{
public:
    CEditingBtnsTool();

    /// @name overrding IUIToolManager functions implemented in CAlgoToolManagerBase
    /// @{
    virtual const IUIObject&  GetDescriptor() const;
    virtual void        SetServiceLocator(IServiceLocator* srv_locator);
    virtual void        SetParentWindow(wxWindow* parent);
    virtual void        InitUI();
    virtual void        CleanUI();
    virtual wxPanel*    GetCurrentPanel();
    virtual bool        IsFinalState();
    virtual bool        IsCompletedState();
    virtual bool        CanDo(EAction action);
    virtual bool        DoTransition(EAction action);
    virtual IAppTask*   GetTask();
    /// @}

    /// @name IUIAlgoToolManager interface implementation
    /// @{
    virtual string    GetCategory();
    virtual void      SetInputObjects(const TConstScopedObjects& input);
    virtual bool      CanQuickLaunch() const { return false; }
    virtual IAppTask* QuickLaunch() { return 0; }
    /// @}

    /// @name IExtension interface implementation
    /// @{
    virtual string  GetExtensionIdentifier() const;
    virtual string  GetExtensionLabel() const;
    /// @}

protected:
    /// describes the Manager's UI properties
    CUIObject       m_Descriptor;

    /// defines tool category
    const string    m_Category;

    IServiceLocator* m_SrvLocator;

    /// a window that will serve as a parent for our panels
    wxWindow*   m_ParentWindow;

    TConstScopedObjects     m_Objects;
};

/* @} */

END_NCBI_SCOPE

#endif // PKG_SEQUENCE___BTNS_TOOL__HPP

