#ifndef GUI_WIDGETS_SEQ___BTNS_VIEW__HPP
#define GUI_WIDGETS_SEQ___BTNS_VIEW__HPP

/*  $Id: btns_view.hpp 27556 2013-03-05 16:04:52Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>

#include <gui/core/project_view_impl.hpp>
#include <gui/core/open_view_manager_impl.hpp>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE_EDIT
 *
 * @{
 */

///////////////////////////////////////////////////////////////////////////////
/// CEditingBtnsView
class CEditingBtnsView : public CProjectView
{
    friend class CEditingBtnsViewFactory;
public:
    CEditingBtnsView();
    ~CEditingBtnsView();

    /// @name IWMClient interface implementation
    /// @{
    virtual wxWindow*      GetWindow();
    virtual wxEvtHandler* GetCommandHandler();
    /// @}

    /// @name IView implementation
    /// @(
    virtual const CViewTypeDescriptor&  GetTypeDescriptor() const;
    virtual void    CreateViewWindow(wxWindow* parent);
    virtual void    DestroyViewWindow();
    /// @}

    /// @name IView implementation
    /// @(
    virtual bool IsAForm() const { return true; }
    /// @}

    /// @name IProjectView implementation
    /// @{
    virtual bool InitView(TConstScopedObjects& objects, const objects::CUser_object* params);
    virtual void OnProjectChanged(CProjectViewEvent& evt);
    /// @}

    /// @name ISelectionClient Implementation
    /// @{
    virtual void GetSelection(CSelectionEvent& evt) const;
    virtual void GetSelection(TConstScopedObjects& objs) const;
    virtual void GetMainObject(TConstScopedObjects& objs) const;
    /// @}

    /// @addtogroup CProjectViewBase overridables
    /// @{
    virtual void GetCompatibleToolBars(vector<string>& names);
    virtual const CObject* x_GetOrigObject() const;
    /// @}

protected:
    virtual void x_OnSetSelection(CSelectionEvent& evt);

    void x_OnWrapSeq_submit(CEvent* evt);

private:
    wxWindow* m_Window;
    CConstRef<CObject> m_OrigObj;

    DECLARE_EVENT_MAP();
};


///////////////////////////////////////////////////////////////////////////////
/// CEditingBtnsViewFactory
class CEditingBtnsViewFactory :
    public CObject,
    public IExtension,
    public IProjectViewFactory
{
public:
    /// @name IExtension interface implementation
    /// @{
    virtual string  GetExtensionIdentifier() const;
    virtual string  GetExtensionLabel() const;
    /// @}

    /// @name IProjectViewFactory interface implementation
    /// @{
    virtual IView* CreateInstance() const;
    virtual IView* CreateInstanceByFingerprint(const TFingerprint& fingerprint) const;
    virtual void RegisterIconAliases(wxFileArtProvider& provider);
    virtual void RegisterCommands(CUICommandRegistry& cmd_reg, wxFileArtProvider& provider);

    virtual const CProjectViewTypeDescriptor& GetProjectViewTypeDescriptor() const;
    virtual bool IsCompatibleWith(const CObject& object, objects::CScope& scope);
    virtual int TestInputObjects( TConstScopedObjects& objects );

    virtual bool ShowInOpenViewDlg() const { return false; }
    /// @}

protected:
    /// @name COpenViewManager implementation
    /// @{
    virtual string GetTargetType() const
    { return "SerialObject"; }
    virtual bool AllowMerge() const { return false; }
    /// @}
};


/* @} */

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ___BTNS_VIEW__HPP
