/*  $Id: bulkfeatureadddlg.hpp 28171 2013-05-21 12:54:22Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */
#ifndef _BULKFEATUREADDDLG_H_
#define _BULKFEATUREADDDLG_H_

#include <corelib/ncbistd.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <objects/seqfeat/SeqFeatData.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <gui/packages/pkg_sequence_edit/srcedit_util.hpp>
#include <gui/packages/pkg_sequence_edit/virusnoncodingfeaturespanel.hpp>
#include <gui/packages/pkg_sequence_edit/bulk_cmd_panel.hpp>
#include <gui/packages/pkg_sequence_edit/bulk_cmd_dlg.hpp>
#include <gui/objutils/cmd_composite.hpp>
#include <gui/framework/workbench.hpp>

/*!
 * Includes
 */

////@begin includes
////@end includes

#include <wx/radiobox.h>
#include <wx/dialog.h>
#include <wx/radiobut.h>
#include <wx/sizer.h>
#include <wx/statbox.h>
#include <wx/button.h>

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

BEGIN_NCBI_SCOPE

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CBULKFEATUREADDDLG 10029
#define SYMBOL_CBULKFEATUREADDDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CBULKFEATUREADDDLG_TITLE _("Add Features")
#define SYMBOL_CBULKFEATUREADDDLG_IDNAME ID_CBULKFEATUREADDDLG
#define SYMBOL_CBULKFEATUREADDDLG_SIZE wxSize(400, 300)
#define SYMBOL_CBULKFEATUREADDDLG_POSITION wxDefaultPosition
////@end control identifiers
#define ID_FEATURE_TABLE_TEXT 10030

/*!
 * CBulkFeatureAddDlg class declaration
 */

class CBulkFeatureAddDlg: public CBulkCmdDlg
{    
    DECLARE_DYNAMIC_CLASS( CBulkFeatureAddDlg )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CBulkFeatureAddDlg();
    CBulkFeatureAddDlg( wxWindow* parent, objects::CSeq_entry_Handle seh, 
                   CSourceRequirements::EWizardType wizard_type = CSourceRequirements::eWizardType_standard,
                   CSourceRequirements::EWizardSrcType src_type = CSourceRequirements::eWizardSrcType_any,
                   IWorkbench* workbench = NULL,
                   wxWindowID id = SYMBOL_CBULKFEATUREADDDLG_IDNAME, const wxString& caption = SYMBOL_CBULKFEATUREADDDLG_TITLE, const wxPoint& pos = SYMBOL_CBULKFEATUREADDDLG_POSITION, const wxSize& size = SYMBOL_CBULKFEATUREADDDLG_SIZE, long style = SYMBOL_CBULKFEATUREADDDLG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CBULKFEATUREADDDLG_IDNAME, const wxString& caption = SYMBOL_CBULKFEATUREADDDLG_TITLE, const wxPoint& pos = SYMBOL_CBULKFEATUREADDDLG_POSITION, const wxSize& size = SYMBOL_CBULKFEATUREADDDLG_SIZE, long style = SYMBOL_CBULKFEATUREADDDLG_STYLE );

    /// Destructor
    ~CBulkFeatureAddDlg();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CBulkFeatureAddDlg event handler declarations

////@end CBulkFeatureAddDlg event handler declarations
    void OnFeatureChoiceSelected( wxCommandEvent& event );
    void OnTextClickUrl(  wxTextUrlEvent& event );

////@begin CBulkFeatureAddDlg member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CBulkFeatureAddDlg member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CBulkFeatureAddDlg member variables
    wxStaticBoxSizer* m_FeatureChoice;
    wxBoxSizer* m_FeatureOptions;
////@end CBulkFeatureAddDlg member variables

    virtual CRef<CCmdComposite> GetCommand();
    virtual string GetErrorMessage();
    void ChooseFeatureType(objects::CSeqFeatData::ESubtype subtype);

private:
    objects::CSeq_entry_Handle m_TopSeqEntry;
    CSourceRequirements::EWizardType m_WizardType;
    CSourceRequirements::EWizardSrcType m_SrcType;
    bool m_LoadTable;
    IWorkbench* m_Workbench;

    CBulkCmdPanel* m_FeatureOptionsPanel;

    wxRadioButton* m_CDSBtn;

    CRef<CCmdComposite> x_CreateImportFeatures(string key, string comment, bool skip_existing = false);
    CRef<CCmdComposite> x_CreateDloops();
    CRef<CCmdComposite> x_CreateControlRegions();
    int x_GetFeatureChoice ();
    void x_ShowFeatureChoice ();
    void x_ClearFeatureOptions ();
    void x_AddLoadTableText();
};

void AddSeqAnnotToSeqEntry(const objects::CSeq_annot* annot, objects::CSeq_entry_Handle seh, CRef<CCmdComposite> cmd);
CRef<CCmdComposite> ImportFeaturesFromFile (IWorkbench* workbench, objects::CSeq_entry_Handle seh);

END_NCBI_SCOPE

#endif
    // _BULKFEATUREADDDLG_H_
