/*  $Id: capitalization_abbreviation.hpp 27132 2012-12-26 20:23:14Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */
#ifndef _CAPITALIZATION_ABBREVIATION_H_
#define _CAPITALIZATION_ABBREVIATION_H_

#include <corelib/ncbistd.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <util/xregexp/regexp.hpp>

BEGIN_NCBI_SCOPE
using namespace objects;

typedef CStaticPairArrayMap<const char*, const char*, PCase_CStr> TCStringPairsMap;
class CCapitalizationAbbreviations
{
public:
    void InsertMissingSpacesAfterCommas(string& result);
    void InsertMissingSpacesAfterNo(string& result);
    void FixCapitalizationInElement(string& result);
    void FixShortWordsInElement(string& result);
    void FixAbbreviationsInElement(string& result);
    void FindReplaceString_CountryFixes(string& result);
    void CapitalizeAfterApostrophe(string& result);
    void CapitalizeSAfterNumber(string& result);
    void FixStateAbbreviationsInAffil(string& state);
    void FixAffiliationShortWordsInElement(string& result);
    void FixOrdinalNumbers(string& result);
    void FixKnownAbbreviationsInElement(string& result);
    void FixOrgNames(CSeq_entry_Handle seh, string& result);
    void FindOrgNames(CSeq_entry_Handle seh, vector<string>& taxnames);
};

END_NCBI_SCOPE

#endif
    // _CAPITALIZATION_ABBREVIATION_H_
