/*  $Id: editingbtnspanel.hpp 27853 2013-04-12 20:28:17Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */
#ifndef _EDITINGBTNSPANEL_H_
#define _EDITINGBTNSPANEL_H_

#include <corelib/ncbistd.hpp>
#include <objects/submit/Seq_submit.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_params.hpp>

#include <gui/utils/event_handler.hpp>

#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>

#include <gui/objutils/undo_manager_interface.hpp>
#include <objmgr/seq_entry_handle.hpp>

#include <gui/packages/pkg_sequence_edit/bulk_cmd_dlg.hpp>

#include <gui/packages/pkg_sequence_edit/tbl_edit_dlg.hpp>

#include <gui/packages/pkg_sequence_edit/remove_sequences.hpp>


/*!
 * Includes
 */

////@begin includes
////@end includes

#include <wx/panel.h>
#include <wx/button.h>
#include <wx/textctrl.h>

/*!
 * Forward declarations
 */

////@begin forward declarations
////@end forward declarations

BEGIN_NCBI_SCOPE
using namespace objects;

class ICommandProccessor;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CEDITINGBTNSPANEL 10020
#define ID_BUTTON1 10035
#define ID_BUTTON2 10044
#define ID_BUTTON3 10045
#define ID_BUTTON4 10046
#define ID_BUTTON5 10047
#define ID_BUTTON6 10048
#define ID_BUTTON7 10049
#define ID_BUTTON8 10050
#define ID_BUTTON9 10051
#define ID_BUTTON10 10052
#define ID_BUTTON11 10053
#define ID_BUTTON12 10054
#define ID_BUTTON13 10055
#define ID_BUTTON14 10056
#define ID_BUTTON15 10057
#define ID_BUTTON16 10058
#define ID_BUTTON17 10059
#define ID_BUTTON18 10060
#define ID_BUTTON19 10061
#define ID_BUTTON20 10062
#define ID_BUTTON21 10063
#define ID_BUTTON22 10064
#define ID_BUTTON23 10065
#define ID_BUTTON24 10066
#define ID_BUTTON25 10067
#define ID_BUTTON26 10068
#define ID_BUTTON27 10069
#define ID_BUTTON28 10070
#define ID_BUTTON29 10071
#define ID_BUTTON30 10072
#define ID_BUTTON31 10073
#define ID_BUTTON32 10074
#define ID_BUTTON33 10075
#define ID_BUTTON34 10076
#define ID_BUTTON35 10077
#define ID_BUTTON36 10078
#define ID_BUTTON37 10079
#define ID_BUTTON38 10080
#define ID_BUTTON39 10081
#define ID_BUTTON40 10082
#define ID_BUTTON41 10083
#define ID_BUTTON42 10084
#define ID_BUTTON43 10085
#define ID_BUTTON44 10086
#define ID_BUTTON45 10087
#define ID_BUTTON46 10088
#define ID_BUTTON47 10089
#define ID_BUTTON48 10090
#define ID_BUTTON49 10091
#define ID_BUTTON50 10092
#define ID_BUTTON51 10093
#define ID_BUTTON52 10094
#define ID_BUTTON53 10095
#define SYMBOL_CEDITINGBTNSPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CEDITINGBTNSPANEL_TITLE _("EditingBtns")
#define SYMBOL_CEDITINGBTNSPANEL_IDNAME ID_CEDITINGBTNSPANEL
#define SYMBOL_CEDITINGBTNSPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CEDITINGBTNSPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CEditingBtnsPanel class declaration
 */

class CEditingBtnsPanel: public wxPanel, public CEventHandler
{    
    DECLARE_DYNAMIC_CLASS( CEditingBtnsPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CEditingBtnsPanel();
    CEditingBtnsPanel( wxWindow* parent, wxWindowID id = SYMBOL_CEDITINGBTNSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CEDITINGBTNSPANEL_POSITION, const wxSize& size = SYMBOL_CEDITINGBTNSPANEL_SIZE, long style = SYMBOL_CEDITINGBTNSPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CEDITINGBTNSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CEDITINGBTNSPANEL_POSITION, const wxSize& size = SYMBOL_CEDITINGBTNSPANEL_SIZE, long style = SYMBOL_CEDITINGBTNSPANEL_STYLE );

    /// Destructor
    ~CEditingBtnsPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    /// Transfer data to the window
    virtual bool TransferDataToWindow();

////@begin CEditingBtnsPanel event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
    void OnReverseAuthorNamesClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON2
    void OnTruncateAuthorMiddleInitials( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON3
    void OnFixUsaAndStates( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON4
    void OnFixCapitalizationAll( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON5
    void OnFixCapitalizationAuthors( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON6
    void OnFixCapitalizationTitles( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON7
    void OnFixCapitalizationAffiliation( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON8
    void OnFixCapitalizationCountry( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON9
    void OnRemoveUnpublishedPublications( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON10
    void OnRemovePublishedPublications( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON11
    void OnRemoveCollidingPublications( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON12
    void OnRemoveAllPublications( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON13
    void OnStripAuthorSuffixes( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON14
    void OnRemoveAuthorConsortiums( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON15
    void OnConvertAuthorToConsortiumWithStringConstraint( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON16
    void OnConvertAuthorToConstortiumWhereLastName( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON17
    void OnConvertAuthorToConsortiumAll( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON18
    void OnReloadPublications( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON19
    void OnAddCitSubForUpdate( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON20
    void OnRemoveSequences( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON21
    void OnRemoveGenomeProjectsDB( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON22
    void OnRemoveSeqAnnotAlign( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON23
    void OnRemoveSeqAnnotGraph( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON24
    void OnRemoveDuplicateStructuredComments( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON25
    void OnRemoveDescriptors( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON26
    void OnRemoveSeqAnnotIds( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON27
    void OnRemoveAllStructuredComments( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON28
    void OnRemoveEmptyStructuredComments( wxCommandEvent& event );

    void OnRemoveSequencesFromAlignments( wxCommandEvent& event );

    void OnRemoveFeatures( wxCommandEvent& event );

    void OnModifyStructuredComment( wxCommandEvent& event );

    void OnRemoveBankitComment( wxCommandEvent& event );

    void OnRemoveStructuredCommentKeyword( wxCommandEvent& event );

    void OnAddStructuredCommentKeyword( wxCommandEvent& event );

    void OnReorderStructuredComment( wxCommandEvent& event );

    void OnReorderSequences( wxCommandEvent& event );

    void OnConvertComment( wxCommandEvent& event );

    void OnParseComment( wxCommandEvent& event );

    void OnAssignFeatureIds( wxCommandEvent& event );

    void OnClearFeatureIds( wxCommandEvent& event );

    void OnReassignFeatureIds( wxCommandEvent& event );

    void OnUniqifyFeatureIds( wxCommandEvent& event );

    void OnTrimJunkInPrimerSeqs( wxCommandEvent& event );

    void OnFixiInPrimerSeqs( wxCommandEvent& event );

    void OnFixOrgModInstitution( wxCommandEvent& event );

    void OnSwapPrimerSeqName( wxCommandEvent& event );

    void OnMergePrimerSets( wxCommandEvent& event );

    void OnSplitPrimerSets( wxCommandEvent& event );

    void OnAppendModToOrg( wxCommandEvent& event );

    void OnParseCollectionDateMonthFirst( wxCommandEvent& event );

    void OnParseCollectionDateDayFirst( wxCommandEvent& event );

    void OnCountryFixupCap ( wxCommandEvent& event );

    void OnCountryFixupNoCap ( wxCommandEvent& event );

////@end CEditingBtnsPanel event handler declarations

////@begin CEditingBtnsPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CEditingBtnsPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CEditingBtnsPanel member variables
////@end CEditingBtnsPanel member variables

    void SetObjects(TConstScopedObjects* objects) { m_InputObjects = objects; }
    void SetUndoManager(ICommandProccessor* cmdProccessor) { m_CmdProccessor = cmdProccessor; }
    void OnDataChange ();

private:
    TConstScopedObjects*    m_InputObjects;
    string m_RegPath;
    ICommandProccessor* m_CmdProccessor;
    objects::CSeq_entry_Handle m_TopSeqEntry;
    CRef<CSeq_submit> m_SeqSubmit;
};

END_NCBI_SCOPE

#endif
    // _EDITINGBTNSPANEL_H_
