/*  $Id: modify_feature_id.hpp 27406 2013-02-08 18:35:12Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 *
 * Authors:  Igor Filippov
 */
#ifndef _MODIFY_FEATURE_ID_H_
#define _MODIFY_FEATURE_ID_H_
#include <corelib/ncbistd.hpp>
#include <corelib/ncbiobj.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <gui/objutils/cmd_composite.hpp>
#include <objects/misc/sequence_macros.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/util/feature.hpp>

BEGIN_NCBI_SCOPE
using namespace objects;

class CAssignFeatureId
{
public: 
    bool apply(CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
private:
    bool AssignHighestFeatureId(const CSeq_entry_EditHandle& entry, int &feat_id, CCmdComposite* composite);
};

class CClearFeatureId
{
public: 
    bool apply(CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
private:
    void ClearFeatureIds(const CSeq_entry_EditHandle& entry, CCmdComposite* composite);
};

class  CReassignFeatureId
{
public:
    bool apply(objects::CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
private:
    void MakeIDPairs(const CSeq_entry_EditHandle& entry, map<int,int> &id_pairs);
    void ReassignFeatureIds(const CSeq_entry_EditHandle& entry, CCmdComposite* composite, map<int,int> &id_pairs);
};

class CUniqifyFeatureId
{
public:
    bool apply(objects::CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
private:
    void ApplyToSeqInSet(objects::CSeq_entry_Handle tse, const CBioseq_set_Handle& bioset, CCmdComposite* composite);
    void UpdateFeatureIds(const CSeq_entry_EditHandle& entry, CCmdComposite* composite, int offset);
};

END_NCBI_SCOPE

#endif
    // _MODIFY_FEATURE_ID_H_
