/*  $Id: modify_structured_comment.hpp 27394 2013-02-07 15:22:09Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 *
 * Authors:  Igor Filippov
 */
#ifndef _MODIFY_STRUCTURED_COMMENT_H_
#define _MODIFY_STRUCTURED_COMMENT_H_
#include <corelib/ncbistd.hpp>
#include <corelib/ncbiobj.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <objects/general/User_object.hpp>
#include <objects/general/User_field.hpp>
#include <objects/valid/Comment_set.hpp>
#include <objects/valid/Comment_rule.hpp>
#include <gui/objutils/cmd_composite.hpp>
#include <gui/objutils/descriptor_change.hpp>
#include <objects/misc/sequence_macros.hpp>
#include <objmgr/feat_ci.hpp>

BEGIN_NCBI_SCOPE
using namespace objects;

class CModifyStructuredComment
{
public: 
    typedef  CChangeUnindexedObjectCommand<objects::CUser_object> CChangeUserCommand;
    bool ApplyToUserObject(CUser_object& user);
    void ApplyToCSeq_entry (CSeq_entry_Handle tse, const CSeq_entry& se, CCmdComposite* composite);
    void apply_to_seq_and_feat(CSeq_entry_Handle tse, CCmdComposite* composite);
    bool apply(CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
private:
    string m_Field;
    wxString m_FindStr;
};

class CReorderStructuredComment
{
public:
    typedef  CChangeUnindexedObjectCommand<objects::CUser_object> CChangeUserCommand;
    bool apply(objects::CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
    void ApplyToCSeq_entry_user (objects::CSeq_entry_Handle tse, const CSeq_entry& se, CCmdComposite* composite);
    void reorder_comment_by_rule(const CUser_object& user, const CComment_rule* rule, CCmdComposite* composite);
private:
    CRef<CComment_set> m_Rules;
};


END_NCBI_SCOPE

#endif
    // _MODIFY_STRUCTURED_COMMENT_H_
