/*  $Id: pcr-primers-edit.hpp 27536 2013-02-28 20:40:27Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */
#ifndef _PCR_PRIMERS_EDIT_H_
#define _PCR_PRIMERS_EDIT_H_

#include <corelib/ncbistd.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <gui/objutils/cmd_composite.hpp>
#include <objects/misc/sequence_macros.hpp>
#include <objmgr/feat_ci.hpp>

BEGIN_NCBI_SCOPE
using namespace objects;

class CPCRPrimerEdit
{
typedef CChangeUnindexedObjectCommand<objects::CBioSource> CChangeBioSourceCommand ;
public:
    bool apply(CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title);
    void apply_to_seq_and_feat(CSeq_entry_Handle tse, CCmdComposite* composite);
    void ApplyToCSeq_entry (const CSeq_entry& se, CCmdComposite* composite);
    void ApplyToBioSource (const CBioSource& biosource, CCmdComposite* composite, const CSeq_entry* se, const CSeqdesc* seqDesq, const CSeq_feat* seqFeat);
    virtual ~CPCRPrimerEdit() {}
    virtual void ApplyToPCRPrimer(CBioSource& biosource, bool& modified);
    virtual void EditPrimerSet(string &seq, bool& modified) = 0;
};

class CPCRPrimerTrimJunk : public CPCRPrimerEdit
{
    virtual void EditPrimerSet(string &seq, bool& modified);
};

class CPCRPrimerFixI : public CPCRPrimerEdit
{
    virtual void EditPrimerSet(string &seq, bool& modified);
};

class CSwapPrimerNameSeq : public  CPCRPrimerEdit
{
public:
    virtual void ApplyToPCRPrimer(CBioSource& biosource, bool& modified);
    virtual void EditPrimerSet(string &seq, bool& modified) {}
};

class CMergePrimerSets : public  CPCRPrimerEdit
{
public:
    virtual void ApplyToPCRPrimer(CBioSource& biosource, bool& modified);
    virtual void EditPrimerSet(string &seq, bool& modified) {}
};

class CSplitPrimerSets : public  CPCRPrimerEdit
{
public:
    virtual void ApplyToPCRPrimer(CBioSource& biosource, bool& modified);
    virtual void EditPrimerSet(string &seq, bool& modified) {}
};


END_NCBI_SCOPE

#endif
    // _PCR_PRIMERS_EDIT_H_
