/*  $Id: remove_desc_dlg.hpp 27294 2013-01-23 16:06:42Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */
#ifndef _REMOVE_DESC_DLG_H_
#define _REMOVE_DESC_DLG_H_

#include <corelib/ncbistd.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <gui/objutils/cmd_composite.hpp>


#include <wx/dialog.h>

/*!
 * Includes
 */

////@begin includes
////@end includes
#include <wx/sizer.h>
#include <wx/listctrl.h>
#include <wx/radiobut.h>
#include <wx/checkbox.h>
#include <wx/textctrl.h>
#include <gui/widgets/wx/wx_utils.hpp>

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

BEGIN_NCBI_SCOPE
using namespace objects;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CREMOVE_DESC_DLG 11000
#define SYMBOL_CREMOVE_DESC_DLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CREMOVE_DESC_DLG_TITLE _("Remove Descriptors Dialog")
#define SYMBOL_CREMOVE_DESC_DLG_IDNAME ID_CREMOVE_DESC_DLG
#define SYMBOL_CREMOVE_DESC_DLG_SIZE wxDefaultSize
#define SYMBOL_CREMOVE_DESC_DLG_POSITION wxDefaultPosition
////@end control identifiers
#define ID_LISTCTRL_DESC_DLG 11001
#define ID_RADIOBUTTON 11002
#define ID_RADIOBUTTON1 11003
#define ID_TEXTCTRL 11004
#define ID_CHECKBOX 11005

/*!
 * CRemoveDescDlg class declaration
 */

class CRemoveDescDlg: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CRemoveDescDlg )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CRemoveDescDlg();
    CRemoveDescDlg( wxWindow* parent, objects::CSeq_entry_Handle seh, wxWindowID id = SYMBOL_CREMOVE_DESC_DLG_IDNAME, const wxString& caption = SYMBOL_CREMOVE_DESC_DLG_TITLE, const wxPoint& pos = SYMBOL_CREMOVE_DESC_DLG_POSITION, const wxSize& size = SYMBOL_CREMOVE_DESC_DLG_SIZE, long style = SYMBOL_CREMOVE_DESC_DLG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CREMOVE_DESC_DLG_IDNAME, const wxString& caption = SYMBOL_CREMOVE_DESC_DLG_TITLE, const wxPoint& pos = SYMBOL_CREMOVE_DESC_DLG_POSITION, const wxSize& size = SYMBOL_CREMOVE_DESC_DLG_SIZE, long style = SYMBOL_CREMOVE_DESC_DLG_STYLE );

    /// Destructor
    ~CRemoveDescDlg();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );

    /// Should we show tooltips?
    static bool ShowToolTips();

    CRef<CCmdComposite> GetCommand();

private:
    objects::CSeq_entry_Handle m_TopSeqEntry;
    map<string,CSeqdesc::E_Choice> m_Types;
    wxListCtrl* m_ListCtrl;
    wxTextCtrl* m_TextCtrl;
    wxRadioButton* m_RadioButton;
    wxCheckBox* m_CheckBox;
    void ApplyToCSeq_entry (objects::CSeq_entry_Handle tse, const CSeq_entry& se, CCmdComposite* composite, CSeqdesc::E_Choice choice, string name);
};

END_NCBI_SCOPE

#endif
    // _REMOVE_DESC_DLG_H_
