/*  $Id: seq_table_grid.hpp 27955 2013-04-30 17:54:28Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */
#ifndef _SEQ_TABLE_GRID_H_
#define _SEQ_TABLE_GRID_H_

#include <corelib/ncbistd.hpp>
#include <gui/packages/pkg_sequence_edit/bulk_cmd_panel.hpp>
#include <gui/packages/pkg_sequence_edit/srcedit_util.hpp>


/*!
 * Includes
 */

////@begin includes
#include "wx/grid.h"
////@end includes


/*!
 * Forward declarations
 */

////@begin forward declarations
////@end forward declarations

BEGIN_NCBI_SCOPE

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CSEQTABLEGRID 10078
#define ID_GRID 10012
#define SYMBOL_CSEQTABLEGRIDPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CSEQTABLEGRIDPANEL_TITLE _("SeqTableGrid")
#define SYMBOL_CSEQTABLEGRIDPANEL_IDNAME ID_CSEQTABLEGRID
#define SYMBOL_CSEQTABLEGRIDPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CSEQTABLEGRIDPANEL_POSITION wxDefaultPosition
////@end control identifiers

enum
{
    eCmdCopyTableValues = 30000,
    eCmdPasteTableValues,
    eCmdAppendTableValues,
    eCmdCopyTableValuesFromId,
    eCmdSearchTable,
    eCmdDeleteTableCol,
    eCmdRenameTableCol
};

class SrcEditDialog;
/*!
 * CSeqTableGridPanel class declaration
 */

class CSeqTableGridPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CSeqTableGridPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CSeqTableGridPanel();
    CSeqTableGridPanel( wxWindow* parent, CRef<objects::CSeq_table> values, CRef<objects::CSeq_table> choices,
                   wxWindowID id = SYMBOL_CSEQTABLEGRIDPANEL_IDNAME, const wxPoint& pos = SYMBOL_CSEQTABLEGRIDPANEL_POSITION, const wxSize& size = SYMBOL_CSEQTABLEGRIDPANEL_SIZE, long style = SYMBOL_CSEQTABLEGRIDPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CSEQTABLEGRIDPANEL_IDNAME, const wxPoint& pos = SYMBOL_CSEQTABLEGRIDPANEL_POSITION, const wxSize& size = SYMBOL_CSEQTABLEGRIDPANEL_SIZE, long style = SYMBOL_CSEQTABLEGRIDPANEL_STYLE );

    /// Destructor
    ~CSeqTableGridPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CSeqTableGridPanel event handler declarations

    /// wxEVT_GRID_CELL_RIGHT_CLICK event handler for ID_GRID
    void OnCellRightClick( wxGridEvent& event );

    /// wxEVT_GRID_LABEL_RIGHT_CLICK event handler for ID_GRID
    void OnLabelRightClick( wxGridEvent& event );

////@end CSeqTableGridPanel event handler declarations

    void OnCopyCells( wxCommandEvent& event );
    void OnPasteCells( wxCommandEvent& event );
    void OnPasteAppendCells( wxCommandEvent& event );
    void OnCopyCellsFromId( wxCommandEvent& event );
    void OnSearchTable( wxCommandEvent& event );
    void OnSortTableByColumn( wxGridEvent& event );
    void OnLabelLeftClick( wxGridEvent& event );
    void OnLabelLeftDClick( wxGridEvent& event );
    void OnCellLeftClick( wxGridEvent& event );
    void OnDeleteColumn( wxCommandEvent& event );
    void OnRenameColumn( wxCommandEvent& event );
////@begin CSeqTableGridPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CSeqTableGridPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CSeqTableGridPanel member variables
////@end CSeqTableGridPanel member variables
    wxGrid* m_Grid;
    CRef<objects::CSeq_table> GetValuesTable();
    void SetValuesTable(CRef<objects::CSeq_table> table);
    void CopyPasteDataByKeyboard(wxKeyEvent& event);

    void UpdateColumnLabelHeight ();
    int GetBestTableWidth();
    void SetBestTableWidth();
    void MakeColumnReadOnly(int pos, bool val = true);
    void MakeColumnReadOnly(string name, bool val = true);
    void MakeAllColumnsReadOnly();
    void SetColumnSizesAndChoices();
    void UpdateColumnChoices(int pos, vector<string> choices);

private:
    CRef<objects::CSeq_table> m_Table;
    CRef<objects::CSeq_table> m_Choices;
    CRef<objects::CSeq_table> m_Copied;
    bool m_SortByRowLabel;
    bool m_SortByRowLabelAscend;
    map<string,int> m_MapRowLabelToIndex;
    wxString m_FindStr;
    void InitMapRowLabelToIndex();
    void x_SortByRowLabel();
    void x_SetUpMenu();
    void x_CreateMenu(wxGridEvent& evt);
    void x_PasteCells(CRef<objects::CSeq_table> copied, bool append = false, string delim = ";");

    bool x_ValuesOkForNewColumn (CConstRef<objects::CSeqTable_column> values, int col_pos);
    void x_OnCopyCells();
    void x_OnPasteCells();
    void x_MakeProblemsColumnReadOnly();
    SrcEditDialog* x_GetParent();
};



END_NCBI_SCOPE

#endif
    // _SEQ_TABLE_GRID_H_
