#ifndef GUI_UTILS___SYSTEM_PATH__HPP
#define GUI_UTILS___SYSTEM_PATH__HPP

/*  $Id: system_path.hpp 22795 2010-12-22 19:48:17Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software / database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software / database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith, Michael DiCuccio
 *
 * File Description:
 *    CSystemPath -- System dependent functions needed by the GUI framework
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui.hpp>

/** @addtogroup GUI_UTILS
 *
 * @{
 */


BEGIN_NCBI_SCOPE


class NCBI_GUIUTILS_EXPORT CSystemPath
{
public:
    /// Bind an alias to an existing path
    /// The application expects certain aliases to be present; if they are
    /// not provided, reasonable defaults will be used.  Note that this
    /// function is not thread-safe by design; application aliases shall
    /// be created in a single thread at application start-up.
    /// Possible aliases include:
    ///  '<std>'  = location of the application installation
    ///  '<home>' = application- and user-specific data directory
    ///  '<res>'  = location of the standard resource directory
    ///  '<pkgdir>' = location of package installation
    NCBI_DEPRECATED static void BindAlias(const string& alias, const string& path);

    /// Utility function to hide the platform specifics of locating our
    /// standard directories and files.  This function is responsible for
    /// resolving '<std>', '<home>', and '<res>' correctly.
    NCBI_DEPRECATED static string  ResolvePath(const string& path, const string& rel_name);

    /// Call the two paramter ResolvePath after splitting the path.
    NCBI_DEPRECATED static string  ResolvePath(const string& path);

    /// Utility function to hide the platform specifics of locating our
    /// standard directories.  This function is responsible for resolving
    /// '<std>', '<home>', and '<res>' correctly, and further ensures that
    /// the relevant final name already exists as a file or directory.
    /// The 'path' parameter here contains a delimited set of directories
    /// to search in order.
    NCBI_DEPRECATED static string  ResolvePathExisting(const string& path,
                                       const string& delim = ",");

    /// These versions of ResolvePathExisting will resolve <std>, <home>, and
    /// <res> as parts of paths, and will return the first existing file
    /// from the supplied vector of paths.  Internally, the implementation is
    /// templatized on an iterator for reuse; this is not exposed to maintain
    /// a solid ABI.
    NCBI_DEPRECATED static string  ResolvePathExisting(const vector<string>& paths);
    NCBI_DEPRECATED static string  ResolvePathExisting(const list<string>& paths);

    /// the applicaton installation directory.  Usually the parent of the
    /// App Path.
    /// corresponds to the <std> alias above.
    NCBI_DEPRECATED static string  GetStdPath(void);

    /// the user-specific home directory
    /// <home> is a private uswer-writable directory under this
    /// this function is useful for applications that wish to customize
    /// the meaning of <home>
    NCBI_DEPRECATED static string  GetUserPath(void);

    /// the user-specific application-specific home directory
    /// corresponds to the <home> alias above.
    NCBI_DEPRECATED static string  GetHomePath(void);

    /// the shared resource path
    /// corresponds to the <res> alias above.
    NCBI_DEPRECATED static string  GetResourcePath(void);

    /// Add an element to the environment's PATH variable.  This will make
    /// sure that the element is inserted in the approrpiate place, and will
    /// erase any occurrences of the path in the existing value
    enum EPathMode {
        ePath_Prepend,
        ePath_Append
    };
    NCBI_DEPRECATED static void AddToPath(const string& path, EPathMode mode = ePath_Prepend);

private:
    typedef map<string, string> TAliasMap;
    static TAliasMap sm_AliasMap;
};

END_NCBI_SCOPE

/* @} */

#endif /// GUI_UTILS___SYSTEM_PATH__HPP
