#ifndef GUI_UTILS___VIEW_EVENT__HPP
#define GUI_UTILS___VIEW_EVENT__HPP

/*  $Id: view_event.hpp 24586 2011-10-21 16:46:34Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed CAnyType restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for CAnyType particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *    Event object for view communications
 */

#include <gui/utils/event.hpp>
#include <gui/objutils/objects.hpp>

BEGIN_NCBI_SCOPE

class CScopedObjectsAttachment : public IEventAttachment
{
public:
    CScopedObjectsAttachment(TConstScopedObjects& objs) : m_Objects(objs) {}
    TConstScopedObjects m_Objects;
};

class CViewEvent : public CEvent
{
public:
    enum EEventType {
        eUnknownEvent,

        /// general notification that selections have changed
        /// This is to be packaged in a CSelectionEvent.
        eSelectionChanged,
        eProjectChanged, /// general data update notification
        eViewCreated,   /// a view has been created
        eViewAttached,   /// a view has been initialized and ready
        eViewReleased,    /// a view has been destroyed
        eViewerCreated,
        eViewerReleased,

        /// notification from child to parent that a selection has changed
        eWidgetSelectionChanged,

        /// notification from child to parent that the visible range has changed
        eWidgetRangeChanged,

        /// notification from child to parent that the underlying data has changed
        eWidgetDataChanged,

        /// general notification to subscribers that the visible range
        /// has changed.  This is to be packaged in a CVisibleRangeEvent.
        eVisibleRangeChanged,

        eEditSelectedObjects,
        eDeleteSelectedObjects,
        eWidgetItemActivated
    };

    enum EEventBroadcast {
        eEventProject,  // all views attached to current document
        eEventViews,    // all identical views attached to current document
        eEventReflect   // reflect event to current view
    };

    /// default ctor
    CViewEvent()
        : CEvent()
    {
    }

    /// create an event for a given event ID, with event class Unknown
    CViewEvent(TEventID eid)
        : CEvent(eid)
    {
    }

    /// create an event for a specific event class and ID
    CViewEvent(TEventClass ecl, TEventID eid)
        : CEvent(ecl, eid)
    {
    }

    /// create an event with all the bells and whistles
    CViewEvent(TEventClass ecl, TEventID eid,
               IEventAttachment* att, EOwnershipPolicy policy, TEventSender* tr)
        : CEvent(ecl, eid, att, policy, tr)
    {
    }

    EEventType GetType() const { return (EEventType)GetID(); }

};

END_NCBI_SCOPE

#endif  // GUI_UTILS___VIEW_EVENT__HPP
