#ifndef __GUI_WIDGETS_ALNMULTI___ALIGN_ROW__HPP
#define __GUI_WIDGETS_ALNMULTI___ALIGN_ROW__HPP

/*  $Id: align_row.hpp 26774 2012-11-06 16:13:53Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk, Anatoliy Kuznetsov
 *
 * File Description:
 *
 */

#include <gui/opengl/gldlist.hpp>

#include <objtools/alnmgr/aln_explorer.hpp>

#include <gui/utils/attr_range_coll.hpp>
#include <gui/widgets/aln_score/aln_scoring.hpp>

#include <gui/widgets/aln_multiple/ialign_row.hpp>
#include <gui/widgets/aln_multiple/alnvec_row_handle.hpp>
#include <gui/widgets/aln_multiple/align_row_graph_cont.hpp>

BEGIN_NCBI_SCOPE


class CScoreCache;
class IAlnVecRowGraph;
class IAlnRowGraphProperties;

////////////////////////////////////////////////////////////////////////////////
/// CAlnRowState - implements state management for Align Row
/// 

class NCBI_GUIWIDGETS_ALNMULTIPLE_EXPORT CAlnRowState : public IAlignRow
{
public:
    CAlnRowState();

    virtual int GetRowState() const;
    virtual void SetRowState(int mask, bool b_set);

private:
    int m_RowState;
};

////////////////////////////////////////////////////////////////////////////////
/// class CAlnVecRow - implementation of IAlignRow for CAlnVec-based alignments.
/// CAlnVecRow uses CAlnVecRowHandle as proxy for accesing data in CAlnVec.

class NCBI_GUIWIDGETS_ALNMULTIPLE_EXPORT CAlnVecRow :
    public CObjectEx,
    public CEventHandler,
    public CAlnRowState
{
public:
    typedef IAlnExplorer::TSignedRange    TSignedRange;

public:
    CAlnVecRow(const IAlignRowHandle& handle);
    CAlnVecRow(const IAlignRowHandle& handle, vector<IAlnVecRowGraph*>& graphs);
    ~CAlnVecRow();

    /// @name IAlignRow implementation
    /// @{
    virtual void    SetHost(IAlignRowHost* host);

    virtual void    SetDisplayStyle(const CRowDisplayStyle* style);
    virtual const CRowDisplayStyle*    GetDisplayStyle();

    virtual int     GetRowNum() const;
    virtual int     GetHeightPixels() const;

    virtual void    RenderColumn(TColumnType  col_type, CGlPane& pane, int state);

    virtual void    OnMouseEvent(wxMouseEvent& event, TColumnType  col_type, CGlPane& pane);

    /// returns tooltip text by point in the viewport coordinates
    virtual string  GetTooltip(const TVPPoint& vp_pt, TColumnType col_type, CGlPane& pane);

    virtual void    GraphicsCacheCmd(EGraphCacheCmd cmd);

    virtual void    GetHTMLActiveAreas(TColumnType col_type, CGlPane& pane, TAreaVector& areas);

    virtual void    UpdateOnStyleChanged();
    virtual void    UpdateOnAnchorChanged();
    /// @}

    virtual void    SetScoreCache(CScoreCache* cache);

    virtual bool    IsExpandable() const;
    virtual bool    IsExpanded() const;
    virtual void    Expand(bool b_exp);

    virtual const IAlnRowGraphProperties*     GetProperties() const;
    virtual void    SetProperties(IAlnRowGraphProperties* props);

    virtual TSeqPos GetColumnAsSeqPos(TColumnType type, const TModelRect& rc_vis);
    virtual string  GetColumnAsString(TColumnType type);

	virtual string& GetAlnSeqString( string &buffer, const IAlnExplorer::TSignedRange &aln_rng ) const;

    virtual string& GetRowStatistics( string &buffer ) const;


protected:
    DECLARE_EVENT_MAP();

    enum EButtons   {
        eInvalid = -1,
        eStrandMarker,
        eExpand,
        eSetupGraphs
    };
    typedef CScoreCache::TScoreColl::const_iterator TScoreConstIterator;

    virtual int x_GetBaseHeight() const;

    virtual void    x_RenderDescr(CGlPane& pane, int state);
    virtual void    x_RenderTaxLabel(CGlPane& pane, int state);
    virtual void    x_RenderAlign(CGlPane& pane);
    virtual void    x_RenderIcons(CGlPane& pane, int state);
    virtual void    x_RenderIconStrand(CGlPane& pane, int state);
    virtual void    x_RenderIconExpand(CGlPane& pane, int state);
    virtual void    x_RenderEmptyBox(CGlPane& pane);

    virtual void    x_RenderInserts(CGlPane& pane, int from, int to,
                                    TModelUnit top_y, TModelUnit bottom_y);
    virtual void    x_RenderUnaligned(CGlPane& pane, int from, int to,
                                    TModelUnit top_y, TModelUnit bottom_y);
    virtual void    x_RenderAlignSegments(CGlPane& pane, IAlnSegmentIterator& it_seg,
                                          TModelUnit top_y, TModelUnit bottom_y,
                                          const CRgbaColor& color);
    virtual void    x_RenderAlignScores(CGlPane& pane, IAlnSegmentIterator& it_seg,
                                        int top_y, int bottom_y, bool averageable);
    virtual void    x_RenderAveragableScores(CGlPane& pane,
                                                IAlnSegmentIterator& it_seg,
                                                TScoreConstIterator it,
                                                const TScoreConstIterator it_end,
                                                int vp_top_y, int vp_bottom_y);

    virtual void    x_RenderNonAveragableScores(CGlPane& pane,
                                                IAlnSegmentIterator& it_seg,
                                                TScoreConstIterator it,
                                                const TScoreConstIterator it_end,
                                                int vp_top_y, int vp_bottom_y);
    virtual bool    x_CanRenderSeq(CGlPane& pane);
    virtual void    x_RenderSequence(CGlPane& pane, TModelUnit top_y, TModelUnit bottom_y,
                                     int start, int stop);
    virtual void    x_RenderGraph(CGlPane& pane, const TSignedRange& range);
    virtual void    x_RenderStartPos(CGlPane& pane, int state);
    virtual void    x_RenderEndPos(CGlPane& pane, int state);
    virtual void    x_RenderSeqStart(CGlPane& pane, int state);
    virtual void    x_RenderSeqEnd(CGlPane& pane, int state);
    virtual void    x_RenderSeqLength(CGlPane& pane, int state);

    virtual void    x_RenderAlnPos(CGlPane& pane, TSeqPos aln_pos,
                                   IAlnExplorer::ESearchDirection dir, int state);
    virtual void    x_RenderTextCell(CGlPane& pane, const string& text,
                                     int state, IGlFont::TAlign align);
    virtual TVPRect x_GetButtonRect(CGlPane& pane, EButtons btn, int shift=1);
    virtual EButtons    x_HitTestButton(CGlPane& pane, const TVPPoint& pt);
    virtual void    x_OnClickExpandBtn();

    virtual string  x_GetDescrTooltip    (const TVPPoint& pt, CGlPane& pane);
    virtual string  x_GetIconsTooltip    (const TVPPoint& pt, CGlPane& pane);
    virtual string  x_GetAlignmentTooltip(const TVPPoint& pt, CGlPane& pane);

    string  x_GetAlignmentTooltip_General(const TSeqPos pos);
    string  x_GetAlignmentTooltip_Insert(const TSignedSeqRange& aln_r,
                                         const TSignedSeqRange& seq_r);
    string  x_GetAlignmentTooltip_Gap(const TSignedSeqRange& aln_r,
                                      const TSignedSeqRange& seq_r);
    string  x_GetAlignmentTooltip_Unaligned(TSignedSeqPos aln_prev_to,
                                            TSignedSeqPos aln_from,
                                            TSignedSeqPos prev_to,
                                            TSignedSeqPos from);
    string  x_GetTooltipSequence(TSeqPos from, TSeqPos to);

    IAlignRowHost*  x_GetHost()     {   return m_Host; }

    void    x_OnGraphContentChanged(CEvent* evt);
    void    x_OnGraphSizeChanged(CEvent* evt);

    virtual void    x_UpdateRowHeight();

protected:
    const IAlignRowHandle&  m_Handle;

    IAlignRowHost*  m_Host;

    int     m_Height;
    int     m_BaseHeight;
    string  m_Text;
    int     m_TaxId;
    string  m_TaxLabel;

    bool m_bExpanded;

    const CRowDisplayStyle*   m_Style; /// row display style

    CScoreCache*    m_Cache;

    CRef<CAlignRowGraphCont>   m_GraphCont; /// graph hosted in the expandable area

    /// Graphics caching
    CGlDisplayList  m_DList;
    TVPUnit         m_PixLeft, m_PixRight;
    TModelUnit      m_ModelLeft, m_ModelRight;
};


END_NCBI_SCOPE

#endif  // __GUI_WIDGETS_ALNMULTI___ALIGN_ROW__HPP
