#error don't include me
#ifndef GUI_DIALOGS_CONFIG___CONFIG_MEDIATOR__HPP
#define GUI_DIALOGS_CONFIG___CONFIG_MEDIATOR__HPP

/*  $Id: config_mediator.hpp 16214 2008-02-29 02:44:06Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *  Configuration Panel Interpreter default implementation.
 *  The is the default implementation of those methods of the IConfigMediator
 *  interface that any such mediator should have in common.
 *
 *  Derived classes with this interface & implementation mediate between IConfigPanel's
 *  and classes with the CSettingsSet base class, in the context of a
 *  CConfigDlg.
 */

#include <gui/widgets/config/iconfig_mediator.hpp> // base class

BEGIN_NCBI_SCOPE

class CSettingsSet;
class IConfigPanel;


class NCBI_GUIWIDGETS_CONFIG_EXPORT CConfigMediator : public IConfigMediator
{
public:
    CConfigMediator(CSettingsSet& data, IConfigPanel& widget_group);
    virtual ~CConfigMediator();

    /// return the object that can give us our Fl_Widget.
    IConfigPanel&       GetPanel(void);

    /// return all the states this dialog knows about
    /// These constitute the valid arguments to the other methods taking
    /// 'state' parameters.
    list<string>        GetStates(void) const;

    /// return the data's current state.
    /// This is the state that Load and Save will operate on.
    /// Used by the dialog only at startup.
    string              GetCurrentState(void) const;

/*  // The following must be defined in the sub class.
    /// What should we title the dialog window?
    virtual string              GetWindowTitle(void) const = 0;
    /// Load data into that panel
    void                Load(const string& state) = 0;
    /// Load 'factory default' data into that panel.
    void                LoadDefault(void) = 0;
    /// Save the panel's data.
    void                Save(const string& state) = 0;
    /// Has the data represented by the panel's widgets been modified?
    bool                Modified(void) const;
*/

    /// Have the data prepare itself to be modified.
    /// must be followed (eventually) by a call to CancelTempData() or MergeTempData().
    virtual void                MakeTempData();
    /// Throw away any modifications made to the data.
    virtual void                CancelTempData();
    /// Save modifications to the data.
    virtual void                MergeTempData();

    /// If the following methods return true the list of states has changed.
    /// and the dialog should do GetStates to repopulate and redisplay the list of states.
    /// if they return false the method failed and no changes to the states were done.
    /// If succesful new_state is the name of the state just added.
    bool                AddState(string& new_state);
    bool                DuplicateState(const string& state, string& new_state);
    bool                RenameState(const string& state,
                                            const string& req_state,
                                            string& new_state);
    bool                DeleteState(const string& state);
    bool                CanRenameState(const string& state);

protected:
    CRef<CSettingsSet>      m_DataSet;
    CRef<IConfigPanel>      m_ConfigPanel;
};


END_NCBI_SCOPE

#endif  /* GUI_DIALOGS_CONFIG___CONFIG_MEDIATOR__HPP */
