#error don't include me
#ifndef GUI_DIALOGS_CONFIG___THEME_CONFIG_PANEL_BASE__HPP
#define GUI_DIALOGS_CONFIG___THEME_CONFIG_PANEL_BASE__HPP

/*  $Id: theme_config_panel_base.hpp 16214 2008-02-29 02:44:06Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 */

/// @file theme_config_panel_base.hpp
///


#include <corelib/ncbistd.hpp>
#include <gui/gui.hpp>
#include <gui/widgets/config/config_panel.hpp>

class wxPanel;

/** @addtogroup Miscellaneous
 *
 * @{
 */
BEGIN_NCBI_SCOPE



class IClickAction : public CObject
{
public:
    virtual void DoClick() = 0;
};


/// IThemeConfigPanelBase.
/// The interace CThemeMediator expects of any widget panel it can interact with,
/// to display and modify data from a CThemeSet.

class IThemeConfigPanelBase : public IConfigPanel
{
public:
    typedef list<string>    TStyleList;

    virtual         ~IThemeConfigPanelBase();

    virtual void   Clear() = 0;
    virtual string GetCurrentStyle(const string& type) const = 0;
    virtual void   SetCurrentStyle(const string& type, const string& style) = 0;
    virtual void   AddItem(const string& type,
                           const string& cur_style,
                           const TStyleList& styles,
                           const string& desc) = 0;
    virtual void   SetStyles(const string& type, const TStyleList& styles) = 0;

    virtual void   UpdateWidgets() = 0;
    virtual void   MakeWidgets() = 0;

    /// Information about links to detail config dialogs.
    virtual void   AddDetailAction(const string& type, IClickAction& act) = 0;

};



/// CThemeConfigPanelBase
/// The default implementation for Theme widget panels.
class NCBI_GUIWIDGETS_CONFIG_EXPORT CThemeConfigPanelBase
    : public IThemeConfigPanelBase
{
public:
    CThemeConfigPanelBase();
    virtual ~CThemeConfigPanelBase();

    virtual void    Clear();
    virtual string  GetCurrentStyle(const string& type) const;
    virtual void    SetCurrentStyle(const string& type, const string& style);
    virtual void    AddItem(const string& type,
                            const string& cur_style,
                            const TStyleList& styles,
                            const string& desc);
    virtual void   SetStyles(const string& type, const TStyleList& styles);

    struct SItemData
    {
        SItemData(const string& t,      const string& cs,
                  const TStyleList& ss, const string& d)
            : type(t), current_style(cs), styles(ss), desc(d) {}

        string  type;
        string  current_style;
        TStyleList  styles;
        string  desc;
    };

    /// Information about links to detail config dialogs.
    virtual void    AddDetailAction(const string& type, IClickAction& act);


protected:

    string          GetCurrentStyleByDesc(const string& desc) const;
    void            SetCurrentStyleByDesc(const string& desc, const string& style);
    TStyleList      GetStylesByDesc(const string& desc) const;

    // mess to help with lookups by description or type.

    enum EKeyType {
        eFindByType,
        eFindByDesc
    };

    const SItemData&    x_GetItemData(const string& key, EKeyType e ) const;
    SItemData&          x_SetItemData(const string& key, EKeyType e );

    // predicates for find_if.
    struct PFindType
    {
        PFindType(const string& s) : m_Val(s) {};
        bool operator()(const SItemData& d) const
        {
            return d.type == m_Val;
        }

    private:
        string      m_Val;
    };

    struct PFindDesc
    {
        PFindDesc(const string& s) : m_Val(s) {};
        bool operator()(const SItemData& d) const
        {
            return d.desc == m_Val;
        }

    private:
        string      m_Val;
    };

    CRef<IClickAction>   GetClickAction(const string&) const;
    bool            IsClickActions() const;

    wxPanel*              m_ConfigPanel;
    vector <SItemData>      m_ItemData;
    /// what to do when someone clicks on a detail button.
    typedef map <string, CRef<IClickAction> >     TClickActionMap;
    TClickActionMap   m_DetailClickActions;
};



END_NCBI_SCOPE


/* @} */

#endif  /* GUI_DIALOGS_CONFIG___THEME_CONFIG_PANEL_BASE__HPP */
