#ifndef GUI_WIDGETS_EDIT___AUTHORNAMES_PANEL__HPP
#define GUI_WIDGETS_EDIT___AUTHORNAMES_PANEL__HPP

/*  $Id: authornames_panel.hpp 27497 2013-02-26 14:57:50Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */

#include <corelib/ncbistd.hpp>

#include <objmgr/scope.hpp>

#include <wx/panel.h>

/*!
 * Includes
 */

////@begin includes
#include "wx/hyperlink.h"
#include "wx/statline.h"
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

class wxScrolledWindow;

BEGIN_NCBI_SCOPE

class CSerialObject;

/*!
 * Control identifiers
 */


////@begin control identifiers
#define ID_AUTHORNAMES 10022
#define ID_ADDAUTH 10023
#define ID_ADDCONSORTIUM 10013
#define ID_PANEL 10012
#define SYMBOL_CAUTHORNAMESPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CAUTHORNAMESPANEL_TITLE _("AuthorNames")
#define SYMBOL_CAUTHORNAMESPANEL_IDNAME ID_AUTHORNAMES
#define SYMBOL_CAUTHORNAMESPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CAUTHORNAMESPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CAuthorNamesPanel class declaration
 */

class NCBI_GUIWIDGETS_EDIT_EXPORT CAuthorNamesPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CAuthorNamesPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CAuthorNamesPanel();
    CAuthorNamesPanel( wxWindow* parent, CSerialObject& object, wxWindowID id = SYMBOL_CAUTHORNAMESPANEL_IDNAME, const wxPoint& pos = SYMBOL_CAUTHORNAMESPANEL_POSITION, const wxSize& size = SYMBOL_CAUTHORNAMESPANEL_SIZE, long style = SYMBOL_CAUTHORNAMESPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CAUTHORNAMESPANEL_IDNAME, const wxPoint& pos = SYMBOL_CAUTHORNAMESPANEL_POSITION, const wxSize& size = SYMBOL_CAUTHORNAMESPANEL_SIZE, long style = SYMBOL_CAUTHORNAMESPANEL_STYLE );

    /// Destructor
    ~CAuthorNamesPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
    bool x_GetAuthors(bool keep_blank = false);
    void PopulateAuthors (objects::CAuth_list& auth_list);
    void SetAuthors(const objects::CAuth_list& auth_list);

    void DeleteAuthor (wxWindow* link);
    void AddLastAuthor (wxWindow* link);

////@begin CAuthorNamesPanel event handler declarations

    /// wxEVT_COMMAND_HYPERLINK event handler for ID_ADDAUTH
    void OnAddauthHyperlinkClicked( wxHyperlinkEvent& event );

    /// wxEVT_COMMAND_HYPERLINK event handler for ID_ADDCONSORTIUM
    void OnAddconsortiumHyperlinkClicked( wxHyperlinkEvent& event );

////@end CAuthorNamesPanel event handler declarations

////@begin CAuthorNamesPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CAuthorNamesPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CAuthorNamesPanel member variables
    wxScrolledWindow* m_ScrolledWindow;
    wxBoxSizer* m_Sizer;
////@end CAuthorNamesPanel member variables
    wxWindow* m_Consortium;
private:
    CSerialObject* m_Object;
    CRef<CSerialObject> m_EditedAuthList;
    int m_RowHeight;

    void s_AddRowToWindow(wxWindow* row);

};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___AUTHORNAMES_PANEL__HPP
