#ifndef GUI_WIDGETS_EDIT___BIOSEQ_EDITOR__HPP
#define GUI_WIDGETS_EDIT___BIOSEQ_EDITOR__HPP

/*  $Id: bioseq_editor.hpp 27922 2013-04-25 16:51:42Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

#include <objects/seqfeat/Seq_feat.hpp>
#include <gui/utils/extension.hpp>

#include <gui/widgets/wx/ibioseq_editor.hpp>
#include <gui/widgets/wx/ieditor_factory.hpp>

BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects)
    class CSeqdesc;
    class CSeq_feat;
END_SCOPE(objects)

class IEditObject;

enum EBioseqEditCommands   {
        eCmdEditObjects = 29000,
        eCmdDeleteObjects,
        eCmdEditBiosourceDesc,
        eCmdDeleteBiosourceDesc,
        eCmdCreateBiosourceDesc,
        eCmdEditBiosourceFeat,
        eCmdDeleteBiosourceFeat,
        eCmdCreateBiosourceFeat,
        eCmdCreateCodingRegion,
        eCmdCreateGene,
        eCmdCreateProt,
        eCmdCreateLTR,
        eCmdCreateRepeatRegion,
        eCmdCreateStemLoop,
        eCmdCreateSTS,
        eCmdCreateRegion,
        eCmdCreatePreRNA,
        eCmdCreateMRNA,
        eCmdCreateExon,
        eCmdCreateIntron,
        eCmdCreate3UTR,
        eCmdCreate5UTR,
        eCmdCreatetRNA,
        eCmdCreaterRNA,
        eCmdCreateRNA,
        eCmdCreatencRNA,
        eCmdCreatetmRNA,
        eCmdCreatePolyASignal,
        eCmdCreatePolyASite,
        eCmdCreatePrimerBind,
        eCmdCreateProteinBind,
        eCmdCreateRBS,
        eCmdCreateRepOrigin,
        eCmdCreateTATASignal,
        eCmdCreateTerminator,
        eCmdCreate10Signal,
        eCmdCreate35Signal,
        eCmdCreateBond,
        eCmdCreateSite,
        eCmdCreateOriT,
        eCmdCreateAttenuator,
        eCmdCreateCRegion, 
        eCmdCreateCAATSignal,
        eCmdCreateDLoop, 
        eCmdCreateDSegment, 
        eCmdCreateEnhancer, 
        eCmdCreateGCSignal, 
        eCmdCreateJSegment, 
        eCmdCreateMiscBinding,
        eCmdCreateMiscDifference,  
        eCmdCreateMiscFeature, 
        eCmdCreateMiscRecomb,
        eCmdCreateMiscSignal,   
        eCmdCreateMiscStructure, 
        eCmdCreateModifiedBase,  
        eCmdCreateNRegion, 
        eCmdCreatePrimTranscript,
        eCmdCreatePromoter,
        eCmdCreateSRegion, 
        eCmdCreateSatellite,  
        eCmdCreateUnsure, 
        eCmdCreateVRegion, 
        eCmdCreateVSegment,   
        eCmdCreateVariation,
        eCmdCreateAssemblyGap,
        eCmdCreateStructuredComment,
        eCmdCreateDBLink,
        eCmdCreateRegionDescriptor,
        eCmdCreateCommentDescriptor,
        eCmdCreateTitle,
        eCmdCreateMolInfo,
        eCmdCreatePubDescriptor,
        eCmdPropagateAllFeatures,
        eCmdPropagateSelectedFeatures,
};

class CBioseqEditor :
    public CObject,
    public IBioseqEditor
{
    DECLARE_EVENT_TABLE()
public:

    CBioseqEditor(ICommandProccessor& cmdProccessor) : m_CmdProccessor(cmdProccessor) {}

    virtual void SetCallBack(IBioseqEditorCB* cb) { m_CB = cb; }
    virtual wxMenu* CreateContextMenu() const;
    virtual wxMenu* CreateBioseqMenu() const;
    virtual void EditSelection();

    void OnEditBiosourceDesc(wxCommandEvent& event);
    void OnUpdateEditBiosourceDesc(wxUpdateUIEvent& event);
    void OnCreateBiosourceDesc(wxCommandEvent& event);
    void OnUpdateCreateBiosourceDesc(wxUpdateUIEvent& event);
    void OnDeleteBiosourceDesc(wxCommandEvent& event);
    void OnUpdateDeleteBiosourceDesc(wxUpdateUIEvent& event);

    void OnEditBiosourceFeat(wxCommandEvent& event);
    void OnUpdateEditBiosourceFeat(wxUpdateUIEvent& event);
    void OnCreateBiosourceFeat(wxCommandEvent& event);
    void OnUpdateCreateBiosourceFeat(wxUpdateUIEvent& event);
    void OnDeleteBiosourceFeat(wxCommandEvent& event);
    void OnUpdateDeleteBiosourceFeat(wxUpdateUIEvent& event);

    void OnPropagateAllFeatures(wxCommandEvent& event);
    void OnUpdatePropagateAllFeatures(wxUpdateUIEvent& event);
    void OnPropagateSelectedFeatures(wxCommandEvent& event);
    void OnUpdatePropagateSelectedFeatures(wxUpdateUIEvent& event);

    // feature menu items
    void OnCreateFeature(CRef<objects::CSeq_feat> feat);
    void OnUpdateNucleotideFeat(wxUpdateUIEvent& event);
    void OnUpdateProteinFeat(wxUpdateUIEvent& event);
    void OnCreateCodingRegion(wxCommandEvent& event);
    void OnCreateGene(wxCommandEvent& event);
    void OnCreateProt(wxCommandEvent& event);
    void OnCreateImport(wxCommandEvent& evt);
    void OnCreateRegion(wxCommandEvent& evt);
    void OnCreateRNA(wxCommandEvent& evt);
    void OnCreateSite(wxCommandEvent& event);
    void OnCreateBond(wxCommandEvent& event);
    void OnCreateDescriptor(wxCommandEvent& evt);

    void OnEditSelection(wxCommandEvent& event);
    void OnUpdateEditSelection(wxUpdateUIEvent& event);

    void OnDeleteSelection(wxCommandEvent& event);
    void OnUpdateDeleteSelection(wxUpdateUIEvent& event);

    virtual bool ProcessEvent(wxEvent& event);

private:
    void x_DeleteObject(objects::CSeq_entry_Handle& seh, const objects::CSeqdesc& seqDesq);
    void x_DeleteObject(objects::CSeq_entry_Handle& seh, const objects::CSeq_feat& seqFeat);
    void x_EditObject(IEditObject& edit);

    void x_ReportError(const string& cmd, const std::exception& e);

    bool x_HaveBiosourceDesc();
    bool x_HaveBiosourceFeat();

    ICommandProccessor& m_CmdProccessor;
    IBioseqEditorCB* m_CB;
};

class CBioseqEditorFactory :
    public CObject,
    public IExtension,
    public IEditorFactory
{
public:
    /// @name IExtension interface implementation
    /// @{
    virtual string  GetExtensionIdentifier() const;
    virtual string  GetExtensionLabel() const;
    /// @}

    /// @name IEditorFactory interface implementation
    /// @{
    virtual void RegisterIconAliases(wxFileArtProvider& provider) {}
    virtual void RegisterCommands(CUICommandRegistry&, wxFileArtProvider&);

    virtual CObject* CreateEditor(const string& interface_name, ICommandProccessor& cmdProccessor);
    /// @}
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___BIOSEQ_EDITOR__HPP
