#ifndef GUI_WIDGETS_EDIT___EDIT_OBJECT_DLG__HPP
#define GUI_WIDGETS_EDIT___EDIT_OBJECT_DLG__HPP

/*  $Id: edit_object_dlg.hpp 24168 2011-08-08 16:52:28Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */

/*!
 * Includes
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>

#include <wx/dialog.h>

////@begin includes
#include "wx/statline.h"
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

BEGIN_NCBI_SCOPE

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CEDITOBJECTDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CEDITOBJECTDLG_TITLE _("Edit Object")
#define SYMBOL_CEDITOBJECTDLG_IDNAME ID_CEDITFEATUREDLG
#define SYMBOL_CEDITOBJECTDLG_SIZE wxSize(400, 300)
#define SYMBOL_CEDITOBJECTDLG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CEditObjectDlg class declaration
 */

class NCBI_GUIWIDGETS_EDIT_EXPORT CEditObjectDlg: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CEditObjectDlg )
    DECLARE_EVENT_TABLE()

public:

    /// Constructors
    CEditObjectDlg();
    CEditObjectDlg( wxWindow* parent, wxWindowID id = SYMBOL_CEDITOBJECTDLG_IDNAME, const wxString& caption = SYMBOL_CEDITOBJECTDLG_TITLE, const wxPoint& pos = SYMBOL_CEDITOBJECTDLG_POSITION, const wxSize& size = SYMBOL_CEDITOBJECTDLG_SIZE, long style = SYMBOL_CEDITOBJECTDLG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CEDITOBJECTDLG_IDNAME, const wxString& caption = SYMBOL_CEDITOBJECTDLG_TITLE, const wxPoint& pos = SYMBOL_CEDITOBJECTDLG_POSITION, const wxSize& size = SYMBOL_CEDITOBJECTDLG_SIZE, long style = SYMBOL_CEDITOBJECTDLG_STYLE );

    /// Destructor
    ~CEditObjectDlg();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    // Creates panels corresponding to object
    void SetEditorWindow(wxWindow* editorWindow);

////@begin CEditObjectDlg event handler declarations

////@end CEditObjectDlg event handler declarations

    /// wxEVT_CLOSE_WINDOW event handler for ID_CEDITFEATUREDLG
    void OnCloseWindow( wxCloseEvent& event );

////@begin CEditObjectDlg member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CEditObjectDlg member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CEditObjectDlg member variables
    wxBoxSizer* m_EditWindowSizer;
    /// Control identifiers
    enum {
        ID_CEDITFEATUREDLG = 10000
    };
////@end CEditObjectDlg member variables

private:
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___EDIT_OBJECT_DLG__HPP
