/* $Id: vectorscreen.hpp 26890 2012-11-20 17:04:00Z choi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Yoon Choi
 */
#ifndef _VECTORSCREEN_H_
#define _VECTORSCREEN_H_

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <objmgr/seq_entry_handle.hpp>

#include <wx/gauge.h>

/*!
 * Includes
 */

////@begin includes
////@end includes


/*!
 * Forward declarations
 */

////@begin forward declarations
////@end forward declarations


BEGIN_NCBI_SCOPE

/*!
 * Control identifiers
 */

////@begin control identifiers
////@end control identifiers


struct SRangeMatch
{
    enum EHitLocation {
        eHitLocation_Terminal,
	eHitLocation_Internal,
	eHitLocation_Mixed,
	eHitLocation_Unknown
    };

    CRange<TSeqPos>   m_range;
    string            m_match_type;
};


struct SVecscreenResult
{
    // Fasta format seqid string
    string              m_FastaSeqid;

    // Sequence length
    TSeqPos             m_SeqLen;

    // Suspect;Strong, Suspect;Moderate, Suspect;Weak, etc.
    vector<SRangeMatch> m_arrRangeMatch;

    // Keep track of selection in the gui
    bool                m_Selected;

    // Alignment graphic in html 
    string              m_AlignmentGraphic;

    SVecscreenResult(const string& sFastaSeqid,
                     TSeqPos seqlen, 
		     const vector<SRangeMatch>& arrRangeMatch,
                     const string& sAlignGraphic)
      : m_FastaSeqid(sFastaSeqid),
        m_SeqLen(seqlen),
        m_arrRangeMatch(arrRangeMatch),
        m_Selected(true),
	m_AlignmentGraphic(sAlignGraphic)
    {}
};


// Re-organize CVecscreenRun::SVecscreenSummary around seqid
typedef map<const CSeq_id*, SVecscreenResult> TVecscreenSummaryBySeqid; 


/*!
 * CVectorScreen class declaration
 */

class NCBI_GUIWIDGETS_EDIT_EXPORT CVectorScreen
{    
public:
    /// Constructors
    CVectorScreen();

    /// Destructor
    ~CVectorScreen();

    void Run(objects::CSeq_entry_Handle seh, wxGauge& progress);

    const vector<CVecscreenRun::SVecscreenSummary>& 
        GetVecscreenSummary() const;

    const TVecscreenSummaryBySeqid&
        GetVecscreenSummaryBySeqid() const;

protected:
    CRef<CSeq_id> x_SelectSeqId(CBioseq_Handle bioseq_handle);
    CRef<CSeq_id> x_FindAccession(CBioseq_Handle bioseq_handle);
    CRef<CSeq_id> x_FindTypeGeneral(CBioseq_Handle bioseq_handle);
    CRef<CSeq_id> x_FindLocalId(CBioseq_Handle bioseq_handle);

private:
    objects::CSeq_entry_Handle m_TopSeqEntry;
    typedef vector< CRef<CSeq_loc> > TSearchLocVec;
    TSearchLocVec m_vecSearchLocs;

    // Standard vecscreen summary
    vector<CVecscreenRun::SVecscreenSummary> m_vecscreen_summary;

    // Vecscreen summary reorganized around seqid
    TVecscreenSummaryBySeqid m_seqidSummary;
};

END_NCBI_SCOPE

#endif
    // _VECTORSCREEN_H_
