/* $Id: vectortrimpanel.hpp 27499 2013-02-26 15:26:33Z choi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Yoon Choi
 */
#ifndef _VECTORTRIMPANEL_H_
#define _VECTORTRIMPANEL_H_

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <objmgr/seq_entry_handle.hpp>
#include <objmgr/graph_ci.hpp>

/*!
 * Includes
 */

////@begin includes
////@end includes

#include <wx/panel.h>
#include <wx/gauge.h>
#include <wx/listctrl.h>
#include <wx/html/htmlwin.h>
#include <wx/checkbox.h>
#include <wx/dialog.h>

#include <algo/blast/format/vecscreen_run.hpp>
#include <gui/objutils/cmd_composite.hpp>
#include <gui/widgets/edit/vectorscreen.hpp>

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
class wxTextCtrl;
class wxStaticText;
////@end forward declarations

class wxButton;

BEGIN_NCBI_SCOPE

class CMatchesListCtrl;
class CLocationsListCtrl;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CVECTORTRIMPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CVECTORTRIMPANEL_TITLE _("Vector Trim Panel")
#define SYMBOL_CVECTORTRIMPANEL_IDNAME ID_CVECTORTRIMPANEL
#define SYMBOL_CVECTORTRIMPANEL_SIZE wxDefaultSize
#define SYMBOL_CVECTORTRIMPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CVectorTrimPanel class declaration
 */

class NCBI_GUIWIDGETS_EDIT_EXPORT CVectorTrimPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CVectorTrimPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CVectorTrimPanel();
    CVectorTrimPanel( wxWindow* parent, 
		      objects::CSeq_entry_Handle seh,
                      wxWindowID id = SYMBOL_CVECTORTRIMPANEL_IDNAME, 
                      const wxPoint& pos = SYMBOL_CVECTORTRIMPANEL_POSITION, 
                      const wxSize& size = SYMBOL_CVECTORTRIMPANEL_SIZE, 
		      long style = SYMBOL_CVECTORTRIMPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, 
                 wxWindowID id = SYMBOL_CVECTORTRIMPANEL_IDNAME, 
		 const wxPoint& pos = SYMBOL_CVECTORTRIMPANEL_POSITION, 
		 const wxSize& size = SYMBOL_CVECTORTRIMPANEL_SIZE, 
		 long style = SYMBOL_CVECTORTRIMPANEL_STYLE );

    /// Destructor
    ~CVectorTrimPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

////@begin CVectorTrimPanel event handler declarations

////@end CVectorTrimPanel event handler declarations
    void OnRunVecscreen( wxCommandEvent& event );
    void OnSelectAll( wxCommandEvent& event );
    void OnSelectStrongModerate( wxCommandEvent& event );
    void OnUnselectAll( wxCommandEvent& event );
    void OnViewAlignmentGraphic( wxCommandEvent& event );
    void OnMakeReport( wxCommandEvent& event );
    void OnTrimSelected( wxCommandEvent& event );

////@begin CVectorTrimPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVectorTrimPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

    void DisplaySummary(const vector<CVecscreenRun::SVecscreenSummary>& 
			vecscreen_summary);
    void DisplaySummaryBySeqid(const TVecscreenSummaryBySeqid& 
			       vecscreen_summary);

    CRef<CCmdComposite> GetCommand();

////@begin CVectorTrimPanel member variables
    /// Control identifiers
    enum {
        ID_CVECTORTRIMPANEL = wxID_ANY
    };
////@end CVectorTrimPanel member variables

private:
    typedef vector<SRangeMatch> TCuts;

    void x_TrimSelected();
    void x_TrimSelected(CRef<CCmdComposite> command,
			const CSeq_id* seqid,
			const SVecscreenResult& vecres);
    void x_SetSeqData(const CBioseq_Handle& bsh,
		      CRef<objects::CSeq_inst> inst,
		      const TCuts& sorted_cuts);
    void x_TrimFeatureLocations(CRef<CSeq_feat> feat,
				const TCuts& sorted_cuts,
				const CSeq_id* seqid,
				bool& bFeatureDeleted,
				bool& bFeatureTrimmed,
				bool& bProdDeleted,
				bool& bProdTrimmed);
    void x_AdjustCdregionFrame(CScope& scope, 
			       CRef<CSeq_feat> feat,
			       const TCuts& sorted_cuts,
			       const CSeq_id* seqid,
			       bool bFeatureTrimmed,
			       bool bProdTrimmed);
    void x_SeqLocDelete(CRef<CSeq_loc> loc, 
			TSeqPos from, TSeqPos to,
			const CSeq_id* seqid,
			bool& bCompleteCut,
			bool& bTrimmed);
    void x_SeqIntervalDelete(CRef<CSeq_interval> interval, 
			     TSeqPos cut_from, TSeqPos cut_to,
			     const CSeq_id* seqid,
			     bool& bCompleteCut,
			     bool& bTrimmed);
    void x_AdjustInternalCutLocations(TCuts& cuts, TSeqPos seq_length);
    void x_FindCompleteDeletions(const TVecscreenSummaryBySeqid& seqidSummary,
				 TVecscreenSummaryBySeqid& completeDeletions,
				 TVecscreenSummaryBySeqid& trimOnly);
    bool x_IsCompleteCut(const TCuts& sorted_cuts, TSeqPos seq_length);
    void x_MergeCuts(TCuts& sorted_cuts);
    void x_DeleteSelected(CRef<CCmdComposite> command,
			  const CSeq_id* seqid);
    void x_RetranslateCDS(CScope& scope,
			  CRef<CCmdComposite> command,
			  CRef<CSeq_feat> cds);
    void x_AdjustDensegAlignment(CRef<CSeq_align> align,
				 CDense_seg::TDim row,
				 const TCuts& sorted_cuts);
    void x_CutDensegSegment(CRef<CSeq_align> align,
			    CDense_seg::TDim row,
			    TSeqPos pos);
    bool x_FindSegment(const CDense_seg& denseg,
		       CDense_seg::TDim row,
		       TSeqPos pos,
		       CDense_seg::TNumseg& seg,
		       TSeqPos& seg_start) const;
    void x_TrimAlignments(CRef<CCmdComposite> command,
			  const TVecscreenSummaryBySeqid& trimOnly);
    void x_GetSortedCuts(TCuts& sorted_cuts,
			 const CSeq_id* seqid);
    void x_TrimSeqGraphs(CRef<CCmdComposite> command,
			 const TVecscreenSummaryBySeqid& trimOnly);
    void x_TrimSeqGraphData(CRef<CSeq_graph> new_graph, 
			    const CMappedGraph& orig_graph,
			    const TCuts& sorted_cuts,
			    const CSeq_id* seqid);
    void x_UpdateSeqGraphLoc(CRef<CSeq_graph> new_graph, 
			     const TCuts& sorted_cuts,
			     const CSeq_id* seqid);
    void x_GetTrimCoordinates(const TCuts& sorted_cuts, 
			      const CSeq_id* seqid,
			      TSeqPos& trim_start,
			      TSeqPos& trim_stop);

private:
    objects::CSeq_entry_Handle m_TopSeqEntry;
    CVectorScreen m_vectorscreen;
    CRef<CCmdComposite> m_Command;

    //
    // Layout of elements in this panel
    //

    // Row 1 - display results
    wxBoxSizer*         m_sizerVectorResults;
    CMatchesListCtrl*   m_listMatches;
    // The following 2 elements are hidden/shown in the same location
    // depending on a toggle.
    wxHtmlWindow*       m_viewAlignGraphic;
    CLocationsListCtrl* m_listLocations;

    // Row 2 - run blast vecscreen against UniVec db
    wxButton* m_buttonRunVecscreen;
    wxGauge*  m_gaugeProgressBar;

    // Row 3 - select results
    wxButton* m_buttonSelectAll;
    wxButton* m_buttonSelectStrongModerate;
    wxButton* m_buttonUnselectAll;
    wxCheckBox* m_checkboxViewGraphic;

    // Row 4 - Trim selected sequences
    wxButton* m_buttonMakeReport;
    wxButton* m_buttonTrimSelected;
};


class CMatchesListCtrl: public wxListCtrl
{
public:
    CMatchesListCtrl();
    CMatchesListCtrl(wxWindow *parent,
		     const wxWindowID id,
		     const wxPoint& pos=wxDefaultPosition,
		     const wxSize& size=wxDefaultSize,
		     long style=wxLC_ICON);
    ~CMatchesListCtrl();
 
    bool Create(wxWindow *parent, 
		wxWindowID id,
		const wxPoint &pos=wxDefaultPosition, 
		const wxSize &size=wxDefaultSize, 
		long style=wxLC_ICON);

    void SetAlignGraphicView( wxHtmlWindow* view );
    void SetRangeView( CLocationsListCtrl* view );

    bool UpdateData(const vector<CVecscreenRun::SVecscreenSummary>& 
		    vecscreen_summary);
    bool UpdateData(const TVecscreenSummaryBySeqid& 
		    vecscreen_summary);

    const TVecscreenSummaryBySeqid& GetVecscreenSummaryBySeqid() const
    {
        return m_seqidSummary;
    }

    void SelectAll();
    void SelectStrongModerate();
    void UnselectAll();

    void OnColClick(wxListEvent& event);
    void OnSelected(wxListEvent& event);
    void OnDoubleClick(wxListEvent& event);

protected:
    const wxString& x_GetSelected(const SVecscreenResult& vecres);
    const wxString& x_GetHitLocation(const SVecscreenResult& vecres);
    const wxString& x_GetSeqid(const SVecscreenResult& vecres);
    const wxString& x_GetMatchType(const SVecscreenResult& vecres);

    void x_UpdateData();
    void x_MergeHitLoc(SRangeMatch::EHitLocation& final_hitloc,
		       const SRangeMatch::EHitLocation& current_hitloc);

private:
    wxListItemAttr m_listItemAttr;

    // Standard vecscreen summary
    vector<CVecscreenRun::SVecscreenSummary> m_vecscreen_summary;

    // Vecscreen summary reorganized around seqid
    TVecscreenSummaryBySeqid m_seqidSummary;

    // Show alignment graphic
    wxHtmlWindow* m_AlignView;
    CLocationsListCtrl* m_RangeView;

    // Do not allow copy/assignment
    wxDECLARE_NO_COPY_CLASS(CMatchesListCtrl);

    DECLARE_DYNAMIC_CLASS( CMatchesListCtrl )
    DECLARE_EVENT_TABLE()
};


class CLocationsListCtrl: public wxListCtrl
{
public:
    CLocationsListCtrl();
    CLocationsListCtrl(wxWindow *parent,
		       const wxWindowID id,
		       const wxPoint& pos=wxDefaultPosition,
		       const wxSize& size=wxDefaultSize,
		       long style=wxLC_ICON);
    ~CLocationsListCtrl();

    bool Create(wxWindow *parent, 
		wxWindowID id,
		const wxPoint &pos=wxDefaultPosition, 
		const wxSize &size=wxDefaultSize, 
		long style=wxLC_ICON);

    void OnSelected(wxListEvent& event);

    bool UpdateData(const SVecscreenResult& vecres);

private:
    wxListItemAttr m_listItemAttr;

    // Do not allow copy/assignment
    wxDECLARE_NO_COPY_CLASS(CLocationsListCtrl);

    DECLARE_DYNAMIC_CLASS( CLocationsListCtrl )
    DECLARE_EVENT_TABLE()
};


#define SYMBOL_CLISTREPORTDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CLISTREPORTDLG_TITLE _("Report Dialog")
#define SYMBOL_CLISTREPORTDLG_SIZE wxDefaultSize

class CListReportDlg: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CListReportDlg )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CListReportDlg();
    CListReportDlg( wxWindow* parent, 
		    wxWindowID id = wxID_ANY, 
		    const wxString& caption = SYMBOL_CLISTREPORTDLG_TITLE, 
		    const wxPoint& pos = wxDefaultPosition, 
		    const wxSize& size = SYMBOL_CLISTREPORTDLG_SIZE, 
		    long style = SYMBOL_CLISTREPORTDLG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, 
		 wxWindowID id = wxID_ANY, 
		 const wxString& caption = SYMBOL_CLISTREPORTDLG_TITLE,
		 const wxPoint& pos = wxDefaultPosition,
		 const wxSize& size = SYMBOL_CLISTREPORTDLG_SIZE,
		 long style = SYMBOL_CLISTREPORTDLG_STYLE );

    /// Destructor
    ~CListReportDlg();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    void SetHeader(const wxString& header);
    void SetList(const wxString& list);
    void SetFooter(const wxString& footer);

private:
    wxStaticText* m_textHeader;
    wxTextCtrl*   m_textList;
    wxStaticText* m_textFooter;
};


END_NCBI_SCOPE

#endif
    // _VECTORTRIMPANEL_H_
