#ifndef __GUI_WIDGETS_GL___BOOKMARKS_HANDLER__HPP
#define __GUI_WIDGETS_GL___BOOKMARKS_HANDLER__HPP

/*  $Id: bookmarks_handler.hpp 17877 2008-09-24 13:03:09Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vlad Lebedev
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>

#include <gui/opengl/glpane.hpp>

#include <gui/widgets/gl/ievent_handler.hpp>
//#include <gui/widgets/fl/tooltip.hpp>

#include <gui/objutils/bookmark.hpp>

BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// IBookmarksHandlerHost
class IBookmarksHandlerHost
{
public:
    typedef list< CRef<CBookmark> > TBookmarks;

    virtual ~IBookmarksHandlerHost() { }

    virtual void BHH_ZoomToRange(TSeqRange range) = 0;
    virtual TModelUnit  BHH_GetModelByWindow(int z, EOrientation orient) = 0;
    virtual TBookmarks  BHH_GetBookmarks(TSeqPos from, TSeqPos to) const = 0;
};


////////////////////////////////////////////////////////////////////////////////
/// Class CBookmarksHandler represents in GUI a bookmarks on a sequence bar.
///
/// CBookmarksHandler implements IGlEventHandler what makes it compatible with
/// IGlEventHandler-supproting widgets.
class NCBI_GUIWIDGETS_GL_EXPORT CBookmarksHandler :
        public wxEvtHandler,
        //public ITooltipClient,
        public IGlEventHandler
{
    DECLARE_EVENT_TABLE()
public:
    typedef list< CRef<CBookmark> > TBookmarks;

    CBookmarksHandler(EOrientation orient = eHorz);
    virtual ~CBookmarksHandler();

    /// @name IGlEventHandler implementation
    /// @{
    virtual void    SetPane(CGlPane* pane);
    virtual wxEvtHandler*   GetEvtHandler();
    /// @}

    void    SetHost(IBookmarksHandlerHost* pHost);
    virtual IGenericHandlerHost*    GetGenericHost();
    void    SetOrientation(EOrientation orient);
    void    Render(CGlPane& Pane);

    /// @name ITooltipClient implementation
    /// @{
    /*virtual bool    TC_NeedTooltip(int x, int y);
    virtual string  TC_GetTooltip(int& x, int& y, int& w, int& h);
    virtual const Fl_Widget*  TC_GetWidget() const;*/
    /// @}

protected:
    void    OnLeftDown(wxMouseEvent& event);
    void    OnLeftDoubleClick(wxMouseEvent& event);
    void    OnMotion(wxMouseEvent& event);

    TModelRect x_GetMarkRect(CRef<CBookmark> mark) const;
    void x_DrawMarker(int x_c, int y_c, int half) const;

private:
    EOrientation m_Orientation;
    IBookmarksHandlerHost* m_Host;

    CGlPane* m_Pane;

    // For Tooltips
    //CTooltip  m_Tooltip;
    string    m_TooltipText;
};


END_NCBI_SCOPE

#endif  // __GUI_WIDGETS_GL___BOOKMARKS_HANDLER__HPP
