#ifndef __GUI_WIDGETS_GL___LEGEND_HANDLER__HPP
#define __GUI_WIDGETS_GL___LEGEND_HANDLER__HPP

/*  $Id: legend_handler.hpp 23688 2011-05-18 14:15:42Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vlad Lebedev, Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>

#include <gui/opengl/glpane.hpp>
#include <gui/opengl/glbitmapfont.hpp>

#include <gui/widgets/gl/ievent_handler.hpp>


BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// class ILegendHandlerHost

class ILegendHandlerHost
{
public:
    enum EValueCode {
        eColor,
        eKey,
        eSeparator
    };

    typedef pair<EValueCode, string> TLegendValue;
    typedef pair<string, TLegendValue> TLegendPair;
    typedef vector<TLegendPair> TColorLegend;

    virtual ~ILegendHandlerHost() { }

    virtual TColorLegend LHH_GetLegend() const = 0;
    virtual TVPUnit     LHH_GetVPPosByY(int y) const = 0;
};


////////////////////////////////////////////////////////////////////////////////
/// Class CLegendHandler represents in GUI a informational popup with color legend
///
/// CLegendHandler implements IGlEventHandler what makes it compatible with
/// IGlEventHandler-supproting widgets.
class NCBI_GUIWIDGETS_GL_EXPORT CLegendHandler :
        public wxEvtHandler,
        public IGlEventHandler

{
    DECLARE_EVENT_TABLE()
public:
    CLegendHandler();
    virtual ~CLegendHandler();

    virtual void    SetHost(ILegendHandlerHost* pHost);
    virtual IGenericHandlerHost*    GetGenericHost();
    virtual void    Render(CGlPane& Pane);
    void SetNeedLegend(bool f);
    void SetShowButton(bool f);
    void ToggleLegend();

    bool NeedLegend() const;

    /// @name IGlEventHandler implementaion
    /// @{
    virtual void    SetPane(CGlPane* pane);
    virtual wxEvtHandler*   GetEvtHandler();
    /// @}

    void    OnLeftDown(wxMouseEvent& event);
    void    OnLeftUp(wxMouseEvent& event);
    void    OnMotion(wxMouseEvent& event);
    void    OnKeyDown(wxKeyEvent& event);
    void    OnKeyUp(wxKeyEvent& event);

protected:
    void x_DrawColorLegendBackGround(const TVPRect& rc) const;
    void x_DrawCloseButton(const TVPRect& rc) const;
    void x_GetCloseButtonRect(const TVPRect& rc, TModelRect& rect) const;
    bool x_PntOnCloseButton(int x, int y) const;

private:
    CGlPane* m_Pane;
    ILegendHandlerHost* m_Host;

    CGlBitmapFont   m_Font_Helv14;
    CGlBitmapFont   m_Font_Helv20;
    bool            m_NeedLegend;         ///< show legend
    bool            m_ShowCloseButton;    ///< show close button on the panel
    bool            m_Highlighted;        ///< highlight close button
    bool            m_ReadyToHideLegend;  ///< ready to hide legend, but may not
};


END_NCBI_SCOPE

#endif  // __GUI_WIDGETS_GL___LEGEND_HANDLER__HPP
