#ifndef GUI_WIDGETS_OBJECT_LIST_WIDGET___REL_OBJ_LIST_CTRL__HPP
#define GUI_WIDGETS_OBJECT_LIST_WIDGET___REL_OBJ_LIST_CTRL__HPP

/*  $Id: rel_obj_list_ctrl.hpp 20611 2010-01-04 21:48:24Z voronov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Yury Voronov
 */

#include <corelib/ncbistd.hpp>

/*!
 * Includes
 */

////@begin includes
#include "wx/listctrl.h"
////@end includes

#include <wx/panel.h>

#include <objmgr/scope.hpp>

#include <gui/objutils/objects.hpp>
#include <gui/objutils/reg_settings.hpp>


/*!
 * Forward declarations
 */

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_RELOBJLISTCTRL_IDNAME ID_RELOBJLISTCTRL
#define SYMBOL_RELOBJLISTCTRL_TITLE _("List of related objects with selector")
#define SYMBOL_RELOBJLISTCTRL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_RELOBJLISTCTRL_SIZE wxSize(350, 80)
#define SYMBOL_RELOBJLISTCTRL_POSITION wxDefaultPosition
////@end control identifiers

class wxChoice;
class wxButton;

/*!
 * CRelatedObjectListCtrl class declaration
 */

BEGIN_NCBI_SCOPE

class CObjectListWidget;
    
class IObjectListAccValidator;
/*
{
public:
    virtual ~IObjectListAccValidator() {}
    virtual CConstRef<CObject> Validate(const objects::CSeq_id& id, objects::CScope& scope, wxString& errMessage) = 0;
};
*/

class NCBI_GUIWIDGETS_OBJECT_LIST_EXPORT CRelatedObjectListCtrl 
    : public wxPanel
    , public IRegSettings
{
    DECLARE_DYNAMIC_CLASS( CRelatedObjectListCtrl )
    DECLARE_EVENT_TABLE()

public:
    typedef map<string, TConstScopedObjects> TRelationMap;

    /// Constructors
    CRelatedObjectListCtrl();
    CRelatedObjectListCtrl( 
        wxWindow* parent, 
        wxWindowID id = SYMBOL_RELOBJLISTCTRL_IDNAME, 
        const wxPoint& pos = SYMBOL_RELOBJLISTCTRL_POSITION, 
        const wxSize& size = SYMBOL_RELOBJLISTCTRL_SIZE, 
        long style = SYMBOL_RELOBJLISTCTRL_STYLE 
    );

    /// Creation
    bool Create( 
        wxWindow* parent, 
        wxWindowID id = SYMBOL_RELOBJLISTCTRL_IDNAME, 
        const wxPoint& pos = SYMBOL_RELOBJLISTCTRL_POSITION, 
        const wxSize& size = SYMBOL_RELOBJLISTCTRL_SIZE, 
        long style = SYMBOL_RELOBJLISTCTRL_STYLE 
    );

    /// Destructor
    ~CRelatedObjectListCtrl();


    /// IRegSettings
    virtual void SetRegistryPath( const string& path ) { m_RegPath = path; }
    virtual void SaveSettings() const;
    virtual void LoadSettings();

    void SetTitle( const wxString& value );

    /// Creates the controls and sizers
    void CreateControls();

    /// Transfer data to the window
    virtual bool TransferDataToWindow();

    /// Transfer data from the window
    virtual bool TransferDataFromWindow();

    void SetAccessionValidator( IObjectListAccValidator* validator );

////@begin CRelatedObjectListCtrl event handler declarations

    /// wxEVT_COMMAND_TEXT_UPDATED event handler for ID_TEXTCTRL1
    void OnAccessionUpdated( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
    void OnAddAccessionClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE1
    void OnChoice1Selected( wxCommandEvent& event );

////@end CObjectListWidgetSel event handler declarations

////@begin CObjectListWidgetSel member function declarations

    void GetObjects( TConstScopedObjects& objects ) const;
    void SetObjects( TConstScopedObjects& objects, const string& type );

    //map<string, TConstScopedObjects>* GetObjects() const { return m_Objects ; }
    //void SetObjects( map<string, TConstScopedObjects>* value) { m_Objects = value ; }

    void GetSelection( TConstScopedObjects& objects ) const { objects = m_Selection; }
    void SetSelection( TConstScopedObjects& objects ){ m_Selection = objects ; }

    bool GetDoSelectAll() const { return m_DoSelectAll ; }
    void SetDoSelectAll( bool value ){ m_DoSelectAll = value ; }

////@end CObjectListWidgetSel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

private:
    /// Initialises member variables
    void Init();
    void x_FillRelationMap( const string& type );
    void x_FillObjects( const wxString& conv );
    void x_UpdateUI();
    void x_UpdateAddBtn();

    ////@begin CObjectListWidgetSel member variables
    /// Control identifiers
    enum {
        ID_RELOBJLISTCTRL = 10000,
        ID_TITLE_BOX = 10003,
        ID_TEXTCTRL1 = 10005,
        ID_BUTTON1 = 10004,
        ID_CHOICE1 = 10001,
        ID_LISTCTRL1 = 10002
    };

    wxTextCtrl* m_Accession;
    wxButton* m_AddBtn;
    wxChoice* m_Choice1;
    CObjectListWidget* m_List1;

    TRelationMap m_RelationMap;
    TConstScopedObjects m_Objects;
    TConstScopedObjects m_Selection;
    bool m_DoSelectAll;

////@end CObjectListWidgetSel member variables

    string  m_RegPath;

    IObjectListAccValidator* m_AccValidator;

    CRef<objects::CScope> m_Scope;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_OBJECT_LIST_WIDGET___REL_OBJ_LIST_CTRL__HPP
