#ifndef GUI_WIDGETS_PHY_TREE___PHYLO_TREE_LABEL__HPP
#define GUI_WIDGETS_PHY_TREE___PHYLO_TREE_LABEL__HPP

/*  $Id: phylo_tree_label.hpp 26700 2012-10-26 16:01:53Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *
 */

#include <gui/opengl.h>
#include <gui/opengl/glrect.hpp>
#include <gui/opengl/glbitmapfont.hpp>
#include <gui/opengl/glpane.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_scheme.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_node.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_lod.hpp>


BEGIN_NCBI_SCOPE

class CPhyloTreeLabel
{
public:
    CPhyloTreeLabel();
    virtual ~CPhyloTreeLabel();

    virtual bool Render(CGlPane& pane, 
                        CPhyloTreeNode * node, 
                        bool bLeft=false, 
                        IPhyloTreeLOD * lod = NULL);
    virtual CGlRect<double>   GetRect(CGlPane& pane, 
                                      CPhyloTreeNode * node, 
                                      bool bLeft=false, 
                                      IPhyloTreeLOD * lod = NULL);
    virtual CVect2<float>  GetSize(CPhyloTreeNode * node);
    virtual void GetNodeLabelOffset(CPhyloTreeNode * node,
                                    IPhyloTreeLOD * lod,                                
                                    CVect2<float>& scalable_offset,
                                    CVect2<float>& pixel_offset);

    TVPUnit                   GetMinVerticalSeparation() const;
    bool                      IsVisible(IPhyloTreeLOD * lod);  
#ifndef TEXTURED_FONTS
    const CGlBitmapFont *     GetCurrFont(void);
#else
    const CGlTextureFont* GetCurrFont();
#endif

    const string GetLabelForNode(CPhyloTreeNode * node, const string & format);
    void SetLabelFormat(const string fmt);
    void SetScheme(CPhyloTreeScheme & sl);

    const string & x_GetLabel(CPhyloTreeNode * node);

private:
    // reference to style
    CRef <CPhyloTreeScheme> m_SL;

    // current font
    //CGlBitmapFont *  m_pFont;

    // label formatting string
    string m_Format;

    // current label
    string m_CurrLabel;

    // label cache
    typedef map<int, string> TLabelCache;
    TLabelCache m_Cache;

    // labels generation
    string x_GenerateLabel(CPhyloTreeNode * node);
    string x_GenerateAutoLabel(CPhyloTreeNode * node,
                               CPhyloTreeScheme::TAutoLabels lbl);
};

END_NCBI_SCOPE

#endif //  GUI_WIDGETS_PHY_TREE___PHYLO_TREE_LABEL__HPP
