#ifndef __GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_PANE__HPP
#define __GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_PANE__HPP

/*  $Id: phylo_tree_pane.hpp 27036 2012-12-11 19:29:57Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *
 */

#include <gui/graph/axis.hpp>
#include <gui/widgets/gl/gl_widget_base.hpp>
#include <gui/widgets/gl/mouse_zoom_handler.hpp>
#include <gui/widgets/gl/linear_sel_handler.hpp>
#include <gui/widgets/gl/minimap_handler.hpp>
#include <gui/opengl/gltexturefont.hpp>
#include <gui/widgets/gl/ruler.hpp>
#include <gui/widgets/gl/tooltip_handler.hpp>
#include <gui/widgets/wx/sticky_tooltip_handler.hpp>
#include <gui/print/pdf.hpp>

#include <gui/widgets/phylo_tree/phylo_tree_render.hpp>

BEGIN_NCBI_SCOPE

class CPhyloTreeWidget;

////////////////////////////////////////////////////////////////////////////////
/// class CPhyloTreePane
class NCBI_GUIWIDGETS_PHYLO_TREE_EXPORT  CPhyloTreePane
                        : public CGlWidgetPane,
                          public CEventHandler,
                          public IMouseZoomHandlerHost,
                          public IMinimapHandlerHost,
                          public ISelHandlerHost,
                          public IPhyloTreeRendererHost
                        
{
    DECLARE_EVENT_TABLE()
public:
    typedef CLinearSelHandler::TRangeColl   TRangeColl;
    typedef vector<IPhyloTreeRenderer*>     TRenderers;

    //CPhyloTreePane(int PosX, int PosY, int Width, int Height, const char* Label = NULL);
    CPhyloTreePane(CPhyloTreeWidget* parent);
	~CPhyloTreePane();

    virtual TVPPoint GetPortSize(void);

    TRenderers & GetRenderers(void)
    {
        return m_Renderers;
    }

    Int4 GetCurrRendererIdx(void)
    {
        return m_CurrRenderer;
    }

    IPhyloTreeRenderer* GetCurrRenderer(void)
    {
        return m_Renderers[m_CurrRenderer];
    }

    CGlPane& GetPane() { return m_MatrixPane; }
    void UpdatePane(const TModelRect& r);

    void    EditCurrentNode();
    void    SetCurrRendererIdx(Int4 idx);
    void    AddRenderer(IPhyloTreeRenderer * renderer);

    void    SetupHardcopyRender();
    void    RenderHardcopy();
    void    RenderPdf(CRef<CPdf> pdf);
    void    BufferedRender();

    //selections
    const vector<int> & GetSelectedIDs(void)
    {
        return GetCurrRenderer()->GetSelectedIDs();
    }

    void SetSelectedIDs(const vector<int> & vect)
    {
        GetCurrRenderer()->SetSelectedIDs(vect);
    }

    void    RemoveCurrentDataSource();
    void    UpdateDataSource();
    void    OnSize(wxSizeEvent& event);
    void    OnLeftDown(wxMouseEvent& evt);
    void    OnPaint(wxPaintEvent& event);

    // interface for a Widget
    virtual void    SetContext(void) { x_SetupGLContext(); }
    virtual void    Update(void);
    virtual void    SoftUpdate(void);

    const   TRangeColl&     GetSubjectSelection() const;
    const   TRangeColl&     GetQuerySelection() const;

    /// Record current zoom/pan so that user can undo/redo navigation
    void SaveCurrentView();
    /// Return true if there are previous view positions you can nav back to
    bool CanGoBack() const;
    /// Return true if there are saved view positions you can nav forward to
    bool CanGoForward() const;
    /// move to view position prior to most recent zoom/pan
    void GoBack();
    /// return to view position that you just left through 'GoBack()'
    void GoForward();

    /// @name IMouseZoomHandlerHost implementation
    /// @{
    virtual TModelUnit  MZHH_GetScale(IMouseZoomHandlerHost::EScaleType type);
    virtual void        MZHH_SetScale(TModelUnit scale, const TModelPoint& point);
    virtual void        MZHH_ZoomRect(const TModelRect& rc);
    virtual void        MZHH_ZoomPoint(const TModelPoint& point, TModelUnit factor);
    virtual void        MZHH_Scroll(TModelUnit d_x, TModelUnit d_y);
    virtual void        MZHH_EndOp();
    virtual TVPUnit     MZHH_GetVPPosByY(int y) const;
    /// @}

    /// @name IMinimapHandlerHost implementation
    /// @{
    virtual TModelUnit  MMHH_GetScale(IMinimapHandlerHost::EMMScaleType type);
    virtual void        MMHH_Scroll(TModelUnit d_x, TModelUnit d_y);
    virtual void        MMHH_EndOp();
    virtual TVPUnit     MMHH_GetVPPosByY(int y) const;
    virtual CGlTexture* MMHH_GetTexture(float& xcoord_limit,
                                        float& ycoord_limit);
    /// @}

    virtual void     HMGH_OnChanged(void);
    virtual void     HMGH_OnRefresh(void);
    virtual void     HMGH_UpdateLimits(bool force_redraw = false);
    virtual TVPUnit  HMGH_GetVPPosByY(int y) const;
    
    virtual void     FireCBEvent(void);
    virtual void     FireEditEvent(void);

    /// @name ISelHandlerHost implementation
    /// @{
    virtual void        SHH_OnChanged();
    virtual TModelUnit  SHH_GetModelByWindow(int z, EOrientation orient);
    virtual TVPUnit     SHH_GetWindowByModel(TModelUnit z, EOrientation orient);
    virtual void        SHH_SetCursor(const wxCursor& cursor);
    /// @}


    /// @name ITooltipClient implementation
    /// @{
    virtual string  TTHH_NeedTooltip(const wxPoint & pt);
    virtual CTooltipInfo  TTHH_GetTooltip(const wxRect & rect);
    /// @}

protected:
    Int4        m_CurrRenderer;
    TRenderers  m_Renderers;

    void    x_SetupAxes();

    /// CGlWidgetPane overridables
    virtual void    x_Render(void);
    virtual void    x_OnShowPopup();
    //virtual int     x_HandleMouseRelease();
    void            x_SendCommand(TCmdID cmd);

    virtual bool    x_GrabFocus();

public:
    CPhyloTreeWidget*       x_GetParent()    {   return m_pParent;   }
    const CPhyloTreeWidget* x_GetParent()  const    {   return m_pParent;   }
protected:

    void    x_Layout(void);

    // Rendering functions
    void    x_RenderContent(void);
    void    x_RenderMouseZoomHandler(CGlPane& pane);
    void    x_RenderSelHandler(bool b_horz, CGlPane& pane,
                               CLinearSelHandler::ERenderingOption option);

    /// adjusts limits and visible rect of the given pane to the master pane
    void    x_AdjsutToMasterPane(CGlPane& pane, bool b_model_x, bool b_model_y);

    virtual int     x_GetAreaByVPPos(TVPUnit vp_x, TVPUnit vp_y);

protected:
    enum EHandlerArea   {
        fMatrixArea =   0x1,
        fSubjectArea =  0x2,
        fQueryArea =    0x4,
        fOther = 0x80000000,
        fAllAreas = fMatrixArea | fSubjectArea | fQueryArea | fOther
    };

    CPhyloTreeWidget    *m_pParent;
    CGlPane m_MatrixPane;
  
    /// @name navigation history
    /// @{
    struct TreeNavHistory {
        TreeNavHistory() : m_BackForwardPos(0) {}
        TreeNavHistory(const CGlPane& p) : m_BackForwardPos(0) 
            { m_History.push_back(p); }

        CGlPane Current() const;
        bool Empty() const { return m_History.size() == 0; }
        void Add(const CGlPane& p);
        bool EqualPanes(const CGlPane& lhs, const CGlPane& rhs) const;
        bool CanGoBack() const { return m_BackForwardPos>0; }
        bool CanGoForward() const { return m_BackForwardPos < m_History.size()-1; }
        CGlPane Back(){ return m_History[--m_BackForwardPos]; }
        CGlPane Forward(){ return m_History[++m_BackForwardPos]; }

        void Reset(const CGlPane& p);

        std::vector<CGlPane>  m_History;
        size_t m_BackForwardPos;        ///< position in history
    };
    TreeNavHistory m_NavHistory;
    /// @}

    /// rectangle in OpenGL coordinates

    TVPRect m_rcMatrix;
    TVPRect m_rcLeftSeq;
    TVPRect m_rcBottomSeq;
    TVPRect m_rcAxes;

    CRgbaColor m_BackColor;
#ifndef TEXTURED_FONTS
    CGlBitmapFont   *m_pTextFont;
    CGlBitmapFont   *m_pLblFont;
#else
    CGlTextureFont   *m_pTextFont;
    CGlTextureFont   *m_pLblFont;
#endif

    // event handlers
    CMouseZoomHandler       m_MouseZoomHandler;
    CMinimapHandler         m_MinimapHandler;
    CLinearSelHandler       m_HorzSelHandler;
    CLinearSelHandler       m_VertSelHandler;
    CTooltipHandler         m_TooltipHandler;
    CStickyTooltipHandler   m_TooltipManager;

    // graphical objects
    CRegularGridGen         m_Gen;
    CRegularGridRenderer    m_Grid;

    CRuler      m_BottomRuler;
    CRuler      m_TopRuler;
    CRuler      m_LeftRuler;
    CRuler      m_RightRuler;

    CMutex  m_Mutex;

    // To force a draw after mouse down (resolves a mac-specific bug)
    bool m_MouseLeftDownEvent;
};

END_NCBI_SCOPE

#endif  // __GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_PANE__HPP
