#ifndef GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_SCHEME__HPP
#define GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_SCHEME__HPP
/*  $Id: phylo_tree_scheme.hpp 27010 2012-12-07 16:37:16Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *
 */

#include <gui/utils/rgba_color.hpp>
#include <gui/opengl/glbitmapfont.hpp>
#include <gui/opengl/gltexturefont.hpp>
#include <gui/objutils/registry.hpp>


#define BUFFERED_RENDER
#define TEXTURED_FONTS

BEGIN_NCBI_SCOPE


//"GBPlugins.PhyloTreeView"
static const string kPhyloBaseKey("GBPlugins.PhyloTreeView"); 

class NCBI_GUIWIDGETS_PHYLO_TREE_EXPORT CPhyloTreeScheme :  public CObject
{
public:
	typedef enum {
		eColored,
		eClusters,
		eMonochrome
	} TColoration;

    typedef enum {
        eTree,
        eNode,
        eLine,
        eLabel
    } TPhyloTreeObject;

    typedef enum {
        eBgColor,
        eColor,
        eSelColor,
        eTraceColor,
        eSharedColor,
        eTipSelColor
    } TPhyloTreeColors;

    typedef enum {
        eLeftMargin,
        eTopMargin,
        eRightMargin,
        eBottomMargin,
        eNodeSize,
        eLineWidth,
        eMaxNodeSize,
        eMinNodeSize,
        eMaxChildsNmb,
        eMinChildsNmb
    } TPhyloTreeSizes;

    typedef enum {
        eLabelsForLeavesOnly,
        eLabelsVisible,      
        eLabelsHidden
    } TLabelsVisibility;

    // Determines if labels going past the edge are truncated or not
    typedef enum {
        eLabelsTruncated,
        eLabelsFull
    } TLabelTruncation;

    typedef enum {
        eSimpleLabels,
        eFancyLabels
    } TLabelStyle;

    typedef enum {
        eAlNone,
        eAlPhylip,
        eAlPaml
    } TAutoLabels;

    typedef enum {
        eNormalSelection,
        eHighlightSelection
    } TSelectionVisibility;

    typedef enum {eZoomXY, eZoomX, eZoomY}  TZoomBehavior;

public:
    CPhyloTreeScheme(CGuiRegistry* registry = NULL);
    virtual ~CPhyloTreeScheme(void) {}

    // interface functions
    void SetMargins(const GLdouble & left, const GLdouble & top,
                    const GLdouble & right, const GLdouble & bottom)
    {
        m_TopMargin     = top;
        m_BottomMargin  = bottom;
        m_LeftMargin    = left;
        m_RightMargin   = right;
    }

    void GetMargins(GLdouble & left, GLdouble & top,
                    GLdouble & right, GLdouble & bottom) const
    {
        top = m_TopMargin;
        bottom = m_BottomMargin;
        left = m_LeftMargin;
        right = m_RightMargin;
    }

    string & SetLabelFormat(void) { return m_LabelFormat; }
    string & SetTooltipFormat(void) { return m_TooltipFormat; }

    const TLabelsVisibility & GetLabelVisibility(void) const {
       return m_LabelVisibility;
    }
    void SetLabelVisibility(const TLabelsVisibility & lv) {
        m_LabelVisibility = lv;
    }

    TSelectionVisibility GetSelectionVisibility() const
        { return m_SelectionVisibility; }    
    void SetSelectionVisibility(const TSelectionVisibility sv) 
        { m_SelectionVisibility = sv; }

    float GetNonSelectedAlpha() const { return m_NonSelectedAlpha; }    
    void SetNonSelectedAlpha(float a) { m_NonSelectedAlpha = a; }

    TLabelTruncation& GetLabelTruncation() { return m_LabelTruncation; }
    void SetLabelTruncation(TLabelTruncation lt) { m_LabelTruncation = lt; }

    const TLabelStyle & GetLabelStyle(void) const { return m_LabelStyle; }
    void  SetLabelStyle(const TLabelStyle & ls) { m_LabelStyle = ls; }

	const TColoration & GetColoration(void) const { return m_Coloration; }
	void  SetColoration(const TColoration & col) { m_Coloration = col; }

    const TAutoLabels & GetAutoLabels(void) const { return m_AutoLabels; }
    void  SetAutoLabels(const TAutoLabels & al) { m_AutoLabels = al; }

#ifndef TEXTURED_FONTS
	const CGlBitmapFont & GetFont(void) const { return m_Font; }
	void SetFont(const CGlBitmapFont & font)  { m_Font = font; }
#else
	const CGlTextureFont & GetFont(void) const { return m_Font; }
	void SetFont(const CGlTextureFont & font)  { m_Font = font; }
#endif

    bool GetBoaNodes() { return m_bBoaNodes; }
    void SetBoaNodes(bool x_bnodes) { m_bBoaNodes = x_bnodes; }


    // zoom behavior
    void            SetZoomBehavior(TZoomBehavior zb) {m_ZoomBehavior = zb;     }
    TZoomBehavior   GetZoomBehavior(void)             {return m_ZoomBehavior;   }

    CRgbaColor & SetColor(TPhyloTreeObject obj, TPhyloTreeColors col);    
    GLdouble & SetSize(TPhyloTreeSizes obj);    

    virtual bool    LoadCurrentSettings();
    virtual bool    SaveCurrentSettings();

protected:
    CRef<CGuiRegistry>  m_Registry; ///> the registry these views come from.

    // themes
    string  m_CurrTheme;
    string  m_ThemeKey;
    string  m_DefaultTheme;
    string  m_CurrColorTheme;
    string  m_CurrSizeTheme;
    string  m_CurrGlobalTheme;

    // margins
    GLdouble m_TopMargin;
    GLdouble m_BottomMargin;
    GLdouble m_LeftMargin;
    GLdouble m_RightMargin;

    // sizes
    GLdouble m_LineWidth;
    GLdouble m_NodeSize;
    GLdouble m_MaxNodeSize;
    GLdouble m_MinNodeSize;
    GLdouble m_MinChildsNmb;
    GLdouble m_MaxChildsNmb;

    // colors
    CRgbaColor m_BgColor;
    CRgbaColor m_LineColor;
    CRgbaColor m_LineSelColor;
    CRgbaColor m_LineTraceColor;
    CRgbaColor m_LineSharedColor;
    CRgbaColor m_NodeColor;
    CRgbaColor m_NodeSelColor;
    CRgbaColor m_NodeTraceColor;
    CRgbaColor m_NodeSharedColor;
    CRgbaColor m_NodeTipSelColor;
    CRgbaColor m_LabelColor;
    CRgbaColor m_LabelSelColor;
    CRgbaColor m_LabelTraceColor;
    CRgbaColor m_LabelSharedColor;

    // monochrome colors
    CRgbaColor m_BgColorMono;
    CRgbaColor m_LineColorMono;
    CRgbaColor m_LineSelColorMono;
    CRgbaColor m_LineTraceColorMono;
    CRgbaColor m_LineSharedColorMono;
    CRgbaColor m_NodeColorMono;
    CRgbaColor m_NodeSelColorMono;
    CRgbaColor m_NodeTraceColorMono;
    CRgbaColor m_NodeSharedColorMono;
    CRgbaColor m_NodeTipSelColorMono;
    CRgbaColor m_LabelColorMono;
    CRgbaColor m_LabelSelColorMono;
    CRgbaColor m_LabelTraceColorMono;
    CRgbaColor m_LabelSharedColorMono;

    // label format
    string   m_LabelFormat;

    // tooltip format
    string   m_TooltipFormat;

    // label visibility
    TLabelsVisibility m_LabelVisibility;

    // visibility of selected nodes vs. non selected nodes
    TSelectionVisibility m_SelectionVisibility;
    
    // alpha value of non-selected glyphs when visibility is eHighlightSelection
    float m_NonSelectedAlpha;

    // Label truncation (how are labels clipped at edges)
    TLabelTruncation m_LabelTruncation;

    // labels style
    TLabelStyle      m_LabelStyle;

    // auto-labeling
    TAutoLabels m_AutoLabels;

    // zoom behavior
    TZoomBehavior m_ZoomBehavior;


	TColoration m_Coloration;

#ifndef TEXTURED_FONTS
	CGlBitmapFont m_Font;
#else
    CGlTextureFont m_Font;
#endif

    // boa - node size behavior
    bool m_bBoaNodes;

    void x_DefaultScheme(void);
    CRgbaColor & x_SetColorMono(TPhyloTreeObject obj, TPhyloTreeColors col);

    /// Return a read view of the registry given a base and 2 subkeys
    CGuiRegistry::CReadView x_GetReadView(const string& base_key,
                                          const string& curr_key,
                                          const string& def_key1="");
    /// Return a read-write view of the registry given a base and 2 subkeys
    CGuiRegistry::CReadWriteView x_GetReadWriteView(const string& base_key,
                                                    const string& curr_key,
                                                    const string& def_key1="");
    /// Load a color from the registry
    bool x_UserFieldToColor(const CGuiRegistry::TReadView& view,
                            const string& key, 
                            CRgbaColor& c);
    void x_ColorToUserField(CGuiRegistry::TReadWriteView& view,
                            const string& key,
                            const CRgbaColor& c);
};


END_NCBI_SCOPE

#endif  // GUI_WIDGETS_PHYLO_TREE___PHYLO_TREE_SCHEME__HPP
