#ifndef __GUI_WIDGETS_PHYLO_TREE___SPATIAL_HASH__HPP
#define __GUI_WIDGETS_PHYLO_TREE___SPATIAL_HASH__HPP

/*  $Id: spatial_hash.hpp 23839 2011-06-10 15:04:51Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Bob Falk
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <corelib/hash_impl/stlp_defs.hpp>
#include <corelib/hash_impl/_hash_map.h>
#include <gui/gui_export.h>

#include <gui/utils/vect2.hpp>
#include <gui/utils/vect3.hpp>

#include <float.h>

BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// class CCollisionHash2D
///
/*
template<class ElemType>
class NCBI_GUIWIDGETS_PHYLO_TREE_EXPORT  CCollisionHash2D
{
public:
    typedef CVect2<float> TVecf;
    typedef CVect2<int> TVeci;

    class HashCode {
    public:
        bool operator()(const TVeci& lhs, const TVeci& rhs) const 
            { return (lhs<rhs); }

        //size_t operator()(const TVeci& pos) cnost
        //    { return (pos[0]*73856093)^(pos[1]*19349663)^(pos[2]*83492791)); }

       size_t operator()(const TVeci& pos) const
            { return (pos[0]*73856093)^(pos[1]*19349663); }
    };

    typedef hash_map<TVeci, ElemType, HashCode> TTableType;

    ElemType* Find(const TVeci& item) { 
        TTableType::iterator it; 
        return ((it=m_HashTable.find(item)) == m_HashTable.end() ? NULL : &(*it).second);
    }
    const ElemType* Find(const TVeci& item) const { 
        TTableType::const_iterator it; 
        return ((it=m_HashTable.find(item)) == m_HashTable.end() ? NULL : &(*it).second);
    }

    ElemType& Get(const TVeci& item) { return m_HashTable[item]; }
    const ElemType& Get(const TVeci& item) const { return m_HashTable[item]; }

    TTableType& GetTable() { return m_HashTable; }
    const TTableType& GetTable() const { return m_HashTable; }

    void SetMin(TVeci& idx) {m_MinIdx = idx;}
    void SetMax(TVeci& idx) {m_MaxIdx = idx;}

    TVeci& GetMin() {return m_MinIdx;}
    TVeci& GetMax() {return m_MaxIdx;}

protected:
    /// Hashtable that stores the data elements
    TTableType  m_HashTable;

    /// Min/Max indicies in the hash table - handy if you want to iterate over
    /// the elements spatially
    TVeci m_MinIdx;
    TVeci m_MaxIdx;
};
*/

////////////////////////////////////////////////////////////////////////////////
/// class CCollisionGrid2D
template<class ElemType>
class NCBI_GUIWIDGETS_PHYLO_TREE_EXPORT  CCollisionGrid2D
{
public:
    typedef CVect2<float> TVecf;
    typedef CVect2<int> TVeci;
    typedef std::vector<ElemType> TTableType;

    int GetTableIdx(const TVeci& item) { return (item.Y()*m_Width + item.X()); }

    void Resize() { if (m_Width*m_Height != m_GridTable.size()) m_GridTable.resize( m_Width*m_Height); }
    void Clear(const ElemType& e) { for (size_t i=0; i<m_GridTable.size(); ++i)  m_GridTable[i]=e; }

    void ResizeFast() { if (m_Width*m_Height > (int)m_GridTable.size()) m_GridTable.resize( m_Width*m_Height); }
    void Clear() { for (size_t i=0; i<m_GridTable.size(); ++i)  m_GridTable[i].clear(); }

    ElemType& Get(const TVeci& item) { return m_GridTable[GetTableIdx(item)]; }
    const ElemType& Get(const TVeci& item) const { return m_GridTable[GetTableIdx(item)]; }

    TTableType& GetTable() { return m_GridTable; }
    const TTableType& GetTable() const { return m_GridTable; }

    void SetMin(TVeci& idx) {
        m_MinIdx = idx; 
        m_Width = m_MaxIdx.X()-m_MinIdx.X() + 1;
        m_Height = m_MaxIdx.Y()-m_MinIdx.Y() + 1;
    }
    void SetMax(TVeci& idx) {
        m_MaxIdx = idx;
        m_Width = m_MaxIdx.X()-m_MinIdx.X() + 1;
        m_Height = m_MaxIdx.Y()-m_MinIdx.Y() + 1;
    }

    TVeci& GetMin() {return m_MinIdx;}
    TVeci& GetMax() {return m_MaxIdx;}

    int GetWidth() const { return m_Width; }
    int GetHeight() const { return m_Height; }

protected:
    /// Hashtable that stores the data elements
    TTableType  m_GridTable;

    /// Min/Max indicies in the table
    TVeci m_MinIdx;
    TVeci m_MaxIdx;

    int m_Width;
    int m_Height;
};


END_NCBI_SCOPE

#endif //__GUI_WIDGETS_PHYLO_TREE___SPATIAL_HASH__HPP
