#ifndef GUI_WIDGETS_SEQ___ASN_VIEW_CONTEXT__HPP
#define GUI_WIDGETS_SEQ___ASN_VIEW_CONTEXT__HPP

/*  $Id: asn_view_context.hpp 25865 2012-05-24 19:08:37Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>

#include <objtools/format/item_formatter.hpp>
#include <objmgr/seq_entry_handle.hpp>

#include <gui/widgets/wx/ibioseq_editor.hpp>

#include <gui/widgets/text_widget/text_item_panel.hpp>

BEGIN_NCBI_SCOPE

class ITextItem;

class CAsnBioContext
{
public:
    CAsnBioContext(const objects::CSeq_entry_Handle& seh) : m_SEH(seh) {}
    objects::CSeq_entry_Handle& GetSeq_entry_Handle() { return m_SEH; }

private:
    objects::CSeq_entry_Handle m_SEH;
};

class NCBI_GUIWIDGETS_SEQ_EXPORT CAsnViewContext
    : public CTextPanelContext, public IBioseqEditorCB
{
public:
    CAsnViewContext(objects::CScope& scope, IBioseqEditor* bioseqEditor = 0, bool tooltpContext = false);
    ~CAsnViewContext();

    virtual void MouseMoveInContainer(ITextItem* container);
    virtual wxWindow* CreateTooltipWindow();

    virtual wxMenu* CreateMenu() const;

    bool GetShowAsnTypes() const { return m_ShowAsnTypes; }
    void SetShowAsnTypes(bool show);

    bool GetShowPathToRoot() const { return m_ShowPathToRoot; }
    void SetShowPathToRoot(bool show);

    objects::CScope& GetScope() { return *m_Scope; }
    void AddBioContext(CAsnBioContext* bioContext) { m_Contexts.push_back(bioContext); }

    virtual void SetPanel(CTextItemPanel* panel);

    virtual void GetCurrentSelection(TEditObjectList& objects);
    virtual objects::CBioseq_Handle GetCurrentBioseq();

protected:
    CRef<objects::CScope> m_Scope;

    CIRef<IBioseqEditor> m_BioseqEditor;

    vector<CAsnBioContext*> m_Contexts;

    bool m_TooltipContext;
    bool m_ShowAsnTypes;
    bool m_ShowPathToRoot;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ___ASN_VIEW_CONTEXT__HPP
