#ifndef GUI_WIDGETS_SEQ___FLAT_FILE_TEXT_ITEM__HPP
#define GUI_WIDGETS_SEQ___FLAT_FILE_TEXT_ITEM__HPP

/*  $Id: flat_file_text_item.hpp 27269 2013-01-18 17:40:13Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>

#include <gui/widgets/text_widget/text_select_item.hpp>

BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects) // namespace ncbi::objects::
class IFlatItem;
END_SCOPE(objects)

class CStyledTextOStream;
class CFlatFileViewContext;

class CFlatFileTextItem : public CTextSelectItem
{
public:
    friend class CFlatFileViewContext;

    CFlatFileTextItem(CConstRef<objects::IFlatItem> item)
        : m_Item(item), m_Selectable(true) {}

    void SetSelectable(bool selectable = true)
        { m_Selectable = selectable; }

    virtual void    CalcSize(wxDC& dc, CTextPanelContext* context);

    virtual string  GetToolTipText(CTextPanelContext* context) const;

    virtual const   CConstRef<CObject> GetAssosiatedObject() const;

    virtual bool    IsSelectable() const
        { return GetAssosiatedObject() && m_Selectable; }

    virtual int     GetTextOffset(CTextPanelContext* context) const;

protected:
    virtual void    x_RenderText(CStyledTextOStream& ostream, CTextPanelContext* context) const;

    CConstRef<objects::IFlatItem> m_Item;

    bool   m_Selectable;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ___FLAT_FILE_TEXT_ITEM__HPP
