#ifndef GUI_WIDGETS_SEQ___FASTA_DS__HPP
#define GUI_WIDGETS_SEQ___FASTA_DS__HPP

/*  $Id: text_ds.hpp 26031 2012-06-29 15:00:37Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>

#include <objmgr/bioseq_handle.hpp>

#include <gui/utils/event_handler.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/utils/app_job_impl.hpp>

#include <gui/widgets/text_widget/composite_text_item.hpp>

class wxFileArtProvider;

BEGIN_NCBI_SCOPE

class CTextItemPanel;
class CTextPanelState;
class ICommandProccessor;
class CUICommandRegistry;

BEGIN_SCOPE(objects)
    class CFlatFileConfig;
    class CSeq_feat;
    class CSeq_loc;
END_SCOPE(objects)

/// Class to deliver partial results as part of progress report of text viewer
///
class CTextViewRetrieveResult : public CObject
{
public:
    CTextViewRetrieveResult();
    virtual ~CTextViewRetrieveResult();

    void SetItems(const CCompositeTextItem::TInsertBlockVec& items);
    /// Accessor to text search results
    /// Whoever gets access and gets items ownership HAS TO call ForgetItems()
    /// to commit the ptrs transfer and avoid double deletion
    const CCompositeTextItem::TInsertBlockVec& GetItems() const { return m_NewItems; }

    /// Add single result item to the container
    void AddItem(ITextItem* item, CCompositeTextItem* container = 0);

    /// Clear the items vector, forget the items ownership
    ///
    /// Warning: this function is not truly const. 
    /// It's declared const to guarantee CAppJobProgress is deliveres as a const object
    /// (the convention we decided to violate in this case)
    ///
    void ForgetItems() const;
private:
    /// free the memory
    void x_FreeItems();
private:
    CTextViewRetrieveResult(const CTextViewRetrieveResult&);
    CTextViewRetrieveResult& operator=(const CTextViewRetrieveResult&);
private:
    mutable CCompositeTextItem::TInsertBlockVec m_NewItems; ///< Text search result items
};

/// Progress report class for text viewer
///
class CTextViewRetrieveProgress : public CAppJobProgress
{
public:
    CTextViewRetrieveProgress(CTextViewRetrieveResult* result) : m_Result(result) {}
public:
    CRef<CTextViewRetrieveResult>   m_Result;
};

class NCBI_GUIWIDGETS_SEQ_EXPORT CTextDataSource :
        public CObjectEx, 
        public CEventHandler
{
    DECLARE_EVENT_MAP();
public:
    CTextDataSource();
    virtual ~CTextDataSource();

    static void RegisterCommands(CUICommandRegistry& cmd_reg, wxFileArtProvider& provider);

    void SetUndoManager(ICommandProccessor* cmdProccessor) { m_CmdProccessor = cmdProccessor; }

    void RemoveJob(CTextItemPanel* panel);
    void LoadFasta(CTextItemPanel* panel, objects::CSeq_entry_Handle& h);
    void LoadFasta(CTextItemPanel* panel, const objects::CSeq_loc& loc, objects::CScope& scope);
    void LoadFlatFile(CTextItemPanel* panel, objects::CSeq_entry_Handle& h,
                      const objects::CSeq_loc* seq_loc,
                      const objects::CFlatFileConfig& config,
                      CTextPanelState* state);
    void LoadAsn(CTextItemPanel* panel, const CSerialObject& so, objects::CScope& scope);
    void LoadXml(CTextItemPanel* panel, const CSerialObject& so);

    void LoadFlatFile(CTextItemPanel* panel,
                      objects::CScope& scope,
                      const objects::CSeq_feat& seq_feat,
                      bool getGeneModel,
                      const objects::CFlatFileConfig& config,
                      CTextPanelState* state);

    /// CAppJobDispatcher notification handler
    void OnJobNotification(CEvent* evt);
    void OnProgress(CEvent* evt);   

private:
    void x_ProcessResult(CTextItemPanel*panel, const CTextViewRetrieveResult* result);

    CTextItemPanel* x_GetPanel(CAppJobDispatcher::TJobID jobId) const;
    void x_RemoveJob(CAppJobDispatcher::TJobID jobId);

    typedef pair<CTextItemPanel*, CAppJobDispatcher::TJobID> TJob;
    typedef list<TJob> TJobs;
    TJobs m_Jobs;
    typedef map<CTextItemPanel*, CRef<CTextPanelState> > TPanelStates;
    TPanelStates m_PanelStates;
    ICommandProccessor* m_CmdProccessor;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ___FASTA_DS__HPP
