#ifndef GUI_WIDGETS_SEQ___TEXT_PANEL__HPP
#define GUI_WIDGETS_SEQ___TEXT_PANEL__HPP

/*  $Id: text_panel.hpp 27567 2013-03-05 20:44:07Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */

#include <corelib/ncbistd.hpp>

#include <gui/gui_export.h>
#include <objmgr/scope.hpp>

#include <wx/panel.h>


////@begin includes
#include "wx/valgen.h"
////@end includes

class wxTextCtrl;
class wxCheckBox;

BEGIN_NCBI_SCOPE

////@begin control identifiers
#define SYMBOL_CTEXTPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CTEXTPANEL_TITLE _("Text Panel")
#define SYMBOL_CTEXTPANEL_IDNAME ID_CTEXTPANEL
#define SYMBOL_CTEXTPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CTEXTPANEL_POSITION wxDefaultPosition
////@end control identifiers

struct SConstScopedObject;
class CTextItemPanel;
class CTextPanelState;
class CTextDataSource;
class ICommandProccessor;

class NCBI_GUIWIDGETS_SEQ_EXPORT CTextPanel: public wxPanel
{
    friend class CAsnViewContext;

    DECLARE_DYNAMIC_CLASS( CTextPanel )
    DECLARE_EVENT_TABLE()

public:
    enum TViewType { kFlatFile, kFastA, kASN, kXML, kUninitialized };

    /// Constructors
    CTextPanel();
    CTextPanel( wxWindow* parent, wxWindowID id = SYMBOL_CTEXTPANEL_IDNAME, const wxPoint& pos = SYMBOL_CTEXTPANEL_POSITION, const wxSize& size = SYMBOL_CTEXTPANEL_SIZE, long style = SYMBOL_CTEXTPANEL_STYLE );

    /// Destructor
    ~CTextPanel();

    /// Initialises member variables
    void Init();
    void Init(SConstScopedObject& object);
    void Reload();

    void SetUndoManager(ICommandProccessor* cmdProccessor);
    void SetInitialViewType(const string& type);
    void SetInitialExpandAll() { m_ExpandAll = true; }

    /// Creates the controls and sizers
    void CreateControls();

////@begin CTextPanel event handler declarations

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE1
    void OnViewTypeSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_MODE
    void OnFlatFileModeSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_STYLE
    void OnFlatFileStyleSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_FONT
    void OnFontFamilySelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_SIZE
    void OnFontSizeSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_FEAT_MODE
    void OnChoiceFeatModeSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_SHOW_ASN_TYPES
    void OnShowAsnTypesClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_SHOW_ASN_TYPES
    void OnShowAsnTypesUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_SHOW_PATH_TO_ROOT
    void OnShowPathToRootClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_SHOW_PATH_TO_ROOT
    void OnShowPathToRootUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_TEXT_ENTER event handler for ID_TEXTCTRL
    void OnTextctrlEnter( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
    void OnFindNextClick( wxCommandEvent& event );

////@end CTextPanel event handler declarations


////@begin CTextPanel member function declarations

    TViewType GetViewType() const { return m_ViewType ; }
    void SetViewType(TViewType value) { m_ViewType = value ; }

    int GetFontFamily() const { return m_FontFamily ; }
    void SetFontFamily(int value) { m_FontFamily = value ; }

    int GetFontSize() const { return m_FontSize ; }
    void SetFontSize(int value) { m_FontSize = value ; }

    int GetFlatFileMode() const { return m_FlatFileMode ; }
    void SetFlatFileMode(int value) { m_FlatFileMode = value ; }

    int GetFlatFileStyle() const { return m_FlatFileStyle ; }
    void SetFlatFileStyle(int value) { m_FlatFileStyle = value ; }

    int GetFeatMode() const { return m_FeatMode ; }
    void SetFeatMode(int value) { m_FeatMode = value ; }

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CTextPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

    const CSerialObject* GetDataObject() const { return m_SO.GetPointer(); }

    CTextItemPanel* GetWidget() const { return m_TextWindow; }

////@begin CTextPanel member variables
    wxTextCtrl* m_FindTextCtrl;
    wxCheckBox* m_SearchUpCtrl;
    CTextItemPanel* m_TextWindow;
private:
    TViewType m_ViewType;
    int  m_FontFamily;
    int  m_FontSize;
    int  m_FlatFileMode;
    int  m_FlatFileStyle;
    int  m_FeatMode;
    bool m_ExpandAll;
    /// Control identifiers
    enum {
        ID_CTEXTPANEL = 10004,
        ID_CHOICE1 = 10005,
        ID_STATIC_MODE = 10020,
        ID_CHOICE_MODE = 10006,
        ID_STATIC_STYLE = 10021,
        ID_CHOICE_STYLE = 10007,
        ID_STATIC_FONT = 10022,
        ID_CHOICE_FONT = 10008,
        ID_STATIC_SIZE = 10023,
        ID_CHOICE_SIZE = 10009,
        ID_STATIC_FEAT_MODE = 10014,
        ID_CHOICE_FEAT_MODE = 10024,
        ID_SHOW_ASN_TYPES = 10013,
        ID_SHOW_PATH_TO_ROOT = 10025,
        ID_TEXTCTRL = 10026,
        ID_BUTTON = 10027,
        ID_CHECKBOX = 10028,
        ID_WIDGET = 10010
    };
////@end CTextPanel member variables

    void x_Load(CTextPanelState* state = NULL);
    void x_LoadFlatFile(CTextPanelState* state = NULL);
    void x_LoadASN();
    void x_LoadXML();
    void x_UpdateToolbar();
    void x_UpdateViewType(const wxString& type);
    void x_UpdatePanelFont();
    void x_LoadFastA();
    void x_SaveViewType();
    void x_FindText();

    CConstRef<CSerialObject> m_SO;
    CRef<objects::CScope> m_Scope;
    CRef<CTextDataSource> m_DataSource;
};


END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ___TEXT_PANEL__HPP
