/*  $Id: annot_conf.hpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 */

/// @file annot_conf.hpp
///

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <objects/general/User_object.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <gui/utils/rgba_color.hpp>
#include <gui/config/settings_set.hpp>

/** @addtogroup GUI_CONFIG
 *
 * @{
 */

BEGIN_NCBI_SCOPE

/******
*  @class CAnnotReadConfig
*  for use by the renderer.
******/
class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CAnnotReadConfig : public CSettingsSet
{
public:
    /// @param sa   The Seq-annot to be configured.
    /// @param registry The registry object to use in memory. by default will
    ///                 use the one loaded by CSettings.
    CAnnotReadConfig(const objects::CSeq_annot& sa,
                     CGuiRegistry* registry = 0);
    ~CAnnotReadConfig();

    // settings retrieval.
    bool        IsSmear() const;
    CRgbaColor    BGColor() const;
    bool        ShowLabel() const;

private:

};


/******
*  @class CAnnotWriteConfig
*  for use by the configuration dialog.
******/

class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CAnnotWriteConfig : public CSettingsSet
{
public:
    CAnnotWriteConfig(CGuiRegistry* registry = 0);
    ~CAnnotWriteConfig();

    bool IsAnnotConfigChanged() const;
    bool IsAnnotSet() const;
    /// return true if the configuration of the old annot was saved.
    bool SetAnnot(objects::CSeq_annot& sa);
    void LoadConfig();
    void SaveConfig();

    // settings retrieval.
    // Modify this struct and two methods together.
    struct CAnnotConfigSettings
    {
        bool    m_Smear;
        bool    m_ShowLabel;
        CRgbaColor    m_BGColor;

        bool operator==(const CAnnotConfigSettings& s) const {
            return m_Smear == s.m_Smear &&
                    m_ShowLabel == s.m_ShowLabel &&
                    m_BGColor == s.m_BGColor ;
        }
    };
    void GetAnnotConfigSettings(CAnnotConfigSettings& settings) const;
    void SetAnnotConfigSettings(const CAnnotConfigSettings& settings);

private:
    CRef<objects::CSeq_annot>   m_SeqAnnot;
    CAnnotConfigSettings        m_LastValues;

};


class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CAnnotConfigUtils
{
public:
    /// Retrieve a configuration object from a Seq-annot which is usable with
    /// a SettingsSet class or any other CGuiRegistry.
    static CConstRef<objects::CUser_object> GetConfigObject(const objects::CSeq_annot& sa);

    /// store a configuration object from a CGuiRegistry into a Seq-annot.
    static void SetConfigObject( objects::CSeq_annot& sa, const objects::CUser_object& uo);
};


END_NCBI_SCOPE

/* @} */
