/*  $Id: color_elements.hpp 19013 2009-03-24 19:03:05Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *  mapping between enums and strings for looking up the colors of elements
 *  of the graphical viewer.
 *
 *  All of this rigamorale is to put the enum name and the string together
 *  in one place. The idea came from newsgroup comp.lang.c++.moderated,
 *  thread subject: "Defining an enum and some strings in parallel"
 *  in a post by msalters on February 2, 2005.
 */

#if !defined(GUI_WIDGETS_SEQ_GRAPHIC___SEQGRAPHIC_COLOR_CONF__HPP)  &&  !defined(DEFINE_COLOR_ELEMENTS)
#error "Do not include color_elements.hpp in your file."
#endif

#ifdef COLOR_ELEMENT_DEF
    // second and third pass
// Add new color elements to this list anywhere.
    // Label colors
COLOR_ELEMENT_DEF( eLabel_Alignment ,   "Label.Alignment" ),
COLOR_ELEMENT_DEF( eLabel_PWAlign ,     "Label.PWAlign" ),
COLOR_ELEMENT_DEF( eLabel_AlignSmear ,  "Label.AlignSmear" ),
COLOR_ELEMENT_DEF( eLabel_Comment ,     "Label.Comment" ),
COLOR_ELEMENT_DEF( eLabel_Feature ,     "Label.Feature" ),
COLOR_ELEMENT_DEF( eLabel_SeqGraph ,    "Label.SeqGraph" ),
COLOR_ELEMENT_DEF( eLabel_Ruler ,       "Label.Ruler" ),
COLOR_ELEMENT_DEF( eLabel_MatePair ,    "Label.MatePair" ),

    // Selected label
COLOR_ELEMENT_DEF( eSelLabel_Alignment ,   "SelLabel.Alignment" ),
COLOR_ELEMENT_DEF( eSelLabel_PWAlign ,     "SelLabel.PWAlign" ),
COLOR_ELEMENT_DEF( eSelLabel_AlignSmear ,  "SelLabel.AlignSmear" ),
COLOR_ELEMENT_DEF( eSelLabel_Feature ,     "SelLabel.Feature" ),
COLOR_ELEMENT_DEF( eSelLabel_SeqGraph ,    "SelLabel.SeqGraph" ),
COLOR_ELEMENT_DEF( eSelLabel_SeqSegment ,  "SelLabel.SeqSegment" ),
COLOR_ELEMENT_DEF( eSelLabel_MatePair ,    "SelLabel.MatePair" ),

    // Selection color
COLOR_ELEMENT_DEF( eSelection_Alignment ,  "Selection.Alignment" ),
COLOR_ELEMENT_DEF( eSelection_PWAlign ,    "Selection.PWAlign" ),
COLOR_ELEMENT_DEF( eSelection_AlignSmear , "Selection.AlignSmear" ),
COLOR_ELEMENT_DEF( eSelection_Feature ,    "Selection.Feature" ),
COLOR_ELEMENT_DEF( eSelection_SeqGraph ,   "Selection.SeqGraph" ),
COLOR_ELEMENT_DEF( eSelection_Histogram ,  "Selection.Histogram" ),
COLOR_ELEMENT_DEF( eSelection_SeqSegment , "Selection.SeqSegment" ),
COLOR_ELEMENT_DEF( eSelection_Ruler ,      "Selection.Ruler" ),
COLOR_ELEMENT_DEF( eSelection_Panel ,      "Selection.Panel" ),
COLOR_ELEMENT_DEF( eSelection_MatePair ,   "Selection.MatePair" ),
COLOR_ELEMENT_DEF( eSelection_HairlineOne ,"Selection.HairlineOne" ),
COLOR_ELEMENT_DEF( eSelection_HairlineMany,"Selection.HairlineMany" ),

    // Sequence letters
COLOR_ELEMENT_DEF( eSeq_SequenceBar ,      "Seq.SequenceBar" ),
COLOR_ELEMENT_DEF( eSeq_ProtOriginal ,     "Seq.ProtOriginal" ),
COLOR_ELEMENT_DEF( eSeq_ProtTranslated ,   "Seq.ProtTranslated" ),
COLOR_ELEMENT_DEF( eSeq_ProtMismatch ,     "Seq.ProtMismatch" ),
COLOR_ELEMENT_DEF( eSeq_MatePairNo ,       "Seq.MatePairNo" ),
COLOR_ELEMENT_DEF( eSeq_MatePairMismatchNo , "Seq.MatePairMismatchNo" ),
COLOR_ELEMENT_DEF( eSeq_MatePairOrientation , "Seq.MatePairOrientation" ),
COLOR_ELEMENT_DEF( eSeq_MatePairMismatchOrientation , "Seq.MatePairMismatchOrientation" ),
COLOR_ELEMENT_DEF( eSeq_MatePairNonUnique , "Seq.MatePairNonUnique" ),
COLOR_ELEMENT_DEF( eSeq_MatePairMismatchNonUnique , "Seq.MatePairMismatchNonUnique" ),
COLOR_ELEMENT_DEF( eSeq_MatePairDistance , "Seq.MatePairDistance" ),
COLOR_ELEMENT_DEF( eSeq_MatePairMismatchDistance , "Seq.MatePairMismatchDistance" ),
COLOR_ELEMENT_DEF( eSeq_PWAlign ,          "Seq.PWAlign" ),
COLOR_ELEMENT_DEF( eSeq_PWAlignMismatch ,  "Seq.PWAlignMismatch" ),

    // Foreground
COLOR_ELEMENT_DEF( eFG_Alignment ,         "FG.Alignment" ),
COLOR_ELEMENT_DEF( eFG_PWAlign ,           "FG.PWAlign" ),
COLOR_ELEMENT_DEF( eFG_AlignSmear ,        "FG.AlignSmear" ),
COLOR_ELEMENT_DEF( eFG_SeqGraph ,          "FG.SeqGraph" ),
COLOR_ELEMENT_DEF( eFG_Comment ,           "FG.Comment" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentInc_F ,     "FG.SeqSegmentInc_F" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentInc_D ,     "FG.SeqSegmentInc_D" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentInc_W ,     "FG.SeqSegmentInc_W" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentInc_U ,     "FG.SeqSegmentInc_U" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentGap ,     "FG.SeqSegmentGap" ),
COLOR_ELEMENT_DEF( eFG_SeqSegmentExc ,     "FG.SeqSegmentExc" ),
COLOR_ELEMENT_DEF( eFG_MatePairNo ,        "FG.MatePairNo" ),
COLOR_ELEMENT_DEF( eFG_MatePairOrientation , "FG.MatePairOrientation" ),
COLOR_ELEMENT_DEF( eFG_MatePairNonUnique , "FG.MatePairNonUnique" ),
COLOR_ELEMENT_DEF( eFG_MatePairDistance ,  "FG.MatePairDistance" ),
COLOR_ELEMENT_DEF( eFG_MatePairLink ,      "FG.MatePairLink" ),
COLOR_ELEMENT_DEF( eFG_ClonePlacementConcordant ,      "FG.ClonePlacementConcordant" ),
COLOR_ELEMENT_DEF( eFG_ClonePlacementDiscordant ,      "FG.ClonePlacementDiscordant" ),
COLOR_ELEMENT_DEF( eFG_ClonePlacementFakeEnd ,      "FG.ClonePlacementFakeEnd" ),

    // Background Color
COLOR_ELEMENT_DEF( eBG_Alignment ,     "BG.Alignment" ),
COLOR_ELEMENT_DEF( eBG_PWAlign ,       "BG.PWAlign" ),
COLOR_ELEMENT_DEF( eBG_AlignSmear ,    "BG.AlignSmear" ),
COLOR_ELEMENT_DEF( eBG_SeqGraph ,      "BG.SeqGraph" ),
COLOR_ELEMENT_DEF( eBG_Comment ,       "BG.Comment" ),
COLOR_ELEMENT_DEF( eBG_SeqSegment ,    "BG.SeqSegment" ),
COLOR_ELEMENT_DEF( eBG_MatePair ,      "BG.MatePair" ),

    // General
COLOR_ELEMENT_DEF( eBackground ,       "Background" ),
COLOR_ELEMENT_DEF( eRuler_Grid ,       "Ruler_Grid" ),
COLOR_ELEMENT_DEF( eGridVertical ,     "GridVertical" ),
COLOR_ELEMENT_DEF( eGridHorizontal ,   "GridHorizontal" ),


#else
     // first pass
    #ifndef DEFINE_COLOR_ELEMENTS
    //

         #define COLOR_ELEMENT_DEF(name, str) name
         enum EDisplayElement {
         // recurse once
        #include <gui/widgets/seq_graphic/color_elements.hpp>
         eEndOfList
        };

        #undef COLOR_ELEMENT_DEF

    #else // DEFINE_COLOR_ELEMENTS
    // inside .cpp file
        #undef DEFINE_COLOR_ELEMENTS

    namespace {
        #define COLOR_ELEMENT_DEF(name, str) str
         const char* sm_Colors[] = {
         // recurse twice
        #include <gui/widgets/seq_graphic/color_elements.hpp>
        ""
        };

        std::string colorelement2str(CSeqGraphicColorConfig::EDisplayElement e)
        {
            return sm_Colors[e];
        }
    }
    #endif // DEFINE_COLOR_ELEMENTS
    #undef COLOR_ELEMENT_DEF

 #endif // COLOR_ELEMENT_DEF
