#ifndef GUI_WIDGETS_SEQ_GRAPHIC___FONT_CONF__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___FONT_CONF__HPP

/* $Id: font_conf.hpp 14562 2007-05-18 11:48:21Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author:  Robert G. Smith
 *
 */

 /**
 * File Description:
 */


#include <gui/config/settings_set.hpp>
#include <gui/config/single_value_config.hpp>

BEGIN_NCBI_SCOPE

/// configuration class for types of layouts.
/// the EFont enums should correspond to the descedants of CFontEngine.


class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CFontConfig : public CSettingsSet
{
public:
    enum EDisplayElement {
        // Labels
        eLabel_ProtProduct,
        eLabel_Alignment,
        eLabel_PWAlign,
        eLabel_AlignSmear,
        eLabel_Comment,
        eLabel_Feature,
        eLabel_SeqGraph,
        eLabel_Ruler,
        eLabel_MatePair
    };

    enum EFont {
        eFont_Default,
        eFont_8,
        eFont_10,
        eFont_12,
        eFont_14,
        eFont_18
    };

    CFontConfig(objects::CPluginConfigCache* config_cache,
                  const string& desc = "Fonts");
    virtual ~CFontConfig();

    virtual bool    LoadCurrentSettings(ELoadValueSource src);
    virtual bool    SaveCurrentSettings(void);

    EFont  GetFont() const { return m_FontValue.GetValue(); }
    void     SetFont(EFont v) { m_FontValue.SetValue(v); }

    typedef CSingleValueConfig<EFont>   TFontValue;
private:
    TFontValue      m_FontValue;
};


END_NCBI_SCOPE

#endif // GUI_WIDGETS_SEQ_GRAPHIC___FONT_CONF__HPP
