#ifndef GUI_WIDGETS_SEQ_GRAPHIC___GRAPH_TRACK__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___GRAPH_TRACK__HPP

/*  $Id: graph_track.hpp 25658 2012-04-18 14:54:03Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Liangshou Wu
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/utils/extension.hpp>
#include <gui/widgets/seq_graphic/data_track.hpp>
#include <objmgr/annot_selector.hpp>


BEGIN_NCBI_SCOPE

class CSGGraphDS;

///////////////////////////////////////////////////////////////////////////////
///   CGraphTrack -- 
///

class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CGraphTrack :
    public CDataTrack
{
    friend class CGraphTrackFactory;

public:
    CGraphTrack(CSGGraphDS* ds, CRenderingContext* r_cntx);
    virtual ~CGraphTrack();

    /// @name CLayoutTrack public methods.
    /// @{
    virtual const CTrackTypeInfo&  GetTypeInfo() const;
    /// @}

    virtual CRef<CSGGenBankDS>  GetDataSource();
    virtual CConstRef<CSGGenBankDS>  GetDataSource() const;

    virtual void GetHTMLActiveAreas(TAreaVector* p_areas) const;
    virtual CHTMLActiveArea* InitHTMLActiveArea(TAreaVector* p_areas) const;

    void SetAnnot(const string& annot);

protected:
    /// @name CLayoutTrack virtual methods overloading.
    /// @{
    virtual void x_LoadSettings(const string& preset_style,
                                const TKeyValuePairs& settings);

    /// save part of settings to a profile string.
    /// Only the potential settings changed by users will be saved.
    virtual void x_SaveSettings(const string& preset_style);

    virtual void x_OnIconClicked(TIconID id);
    /// @}

    /// @name CDataTrack pure virtual interfaces
    /// @{
    virtual void x_UpdateData();
    virtual void x_OnJobCompleted(CAppJobNotification& notify);
    /// @}

    objects::SAnnotSelector& x_GetAnnotSelector();
    CSGGraphDS& x_GetGraphDS();

private:
    /// @name prohibited copy constructor and assignment operator.
    /// @{
    CGraphTrack(const CGraphTrack&);
    CGraphTrack& operator=(const CGraphTrack&);
    /// @}

    void x_OnSettingsIconClicked();

private:
    string                  m_AnnotName;
    CRef<CSGGraphDS>        m_DS;
    objects::SAnnotSelector m_Sel;
    /// Dynamic or fixed graph scale.
    /// By default, the graph scale is a dynamic value which
    /// is updated within the visible range.
    bool                    m_FixedScale;

    static CTrackTypeInfo   m_TypeInfo;
};

///////////////////////////////////////////////////////////////////////////////
///   CGraphTrackFactory
///
class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CGraphTrackFactory :
    public CObject,
    public ILayoutTrackFactory,
    public ITrackConfigurable,
    public IExtension
{
public:
    CGraphTrackFactory(){}

    /// create a layout track based on the input objects and extra parameters.
    virtual TTrackMap CreateTracks(
        SConstScopedObject& object,
        ISGDataSourceContext* ds_context,
        CRenderingContext* r_cntx,
        const SExtraParams& params = SExtraParams(),
        const TAnnotMetaDataList& src_annots = TAnnotMetaDataList()) const;

    virtual bool UnderstandLevel() const
    { return true; }

    virtual bool NeedBackgroundInit() const
    { return true; }

    virtual const CTrackTypeInfo&  GetThisTypeInfo() const
    { return GetTypeInfo(); }

    static const CTrackTypeInfo& GetTypeInfo()
    { return CGraphTrack::m_TypeInfo; }

    /// @name ITrackConfigurable interface implementation
    /// @{
    virtual CRef<objects::CTrackConfigSet>
        GetSettings(CSeqGraphicConfig* g_conf,
        const string& profile,
        const TKeyValuePairs& settings,
        const CTempTrackProxy* track_proxy) const;
    /// @}

    /// @name IExtension interface implementation
    /// @{
    virtual string GetExtensionIdentifier() const;
    virtual string GetExtensionLabel() const;
    /// @}
};


inline
objects::SAnnotSelector& CGraphTrack::x_GetAnnotSelector()
{
    return m_Sel;
}


inline CSGGraphDS& CGraphTrack::x_GetGraphDS()
{
    return *m_DS;
}

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ_GRAPHIC___GRAPH_TRACK__HPP
