#ifndef GUI_WIDGETS_SEQ_GRAPHIC___PANECONTENT_CONF__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___PANECONTENT_CONF__HPP

/*  $Id: pane_content_conf.hpp 17711 2008-09-04 20:33:08Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *   Configuration file for Sequence Graphical Widget
 *
 */


#include <gui/gui_export.h>
#include <gui/config/feat_show.hpp>
#include <gui/config/settings_set.hpp>
//#include <gui/widgets/config/config_mediator.hpp>

BEGIN_NCBI_SCOPE


///
/// class: CPaneContentConf
/// configuration class for the graphics view, specifying what is displayed.

class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CPaneContentConf : public CFeatConfigShow
{
public:
    /// we use int instead of CSeqFeatData::ESubtype for two reasons:
    ///  - avoids unnecessary includes
    ///  - gives transparent warning-free compiles with generic serialized
    ///    code (i.e., when we serialize a subtype, it's as an int not as an
    ///    enum)
    typedef CSettingsSet::TFeatSubtype TFeatSubtype;

    CPaneContentConf(CSettingsSet& settings);

    // individual histogram settings
    bool GetHistogram(TFeatSubtype subtype) const;
    void SetHistogram(TFeatSubtype subtype, bool hist);

    bool GetShowAlignments(void) const;
    void SetShowAlignments(bool b);

    bool GetShowPWAlignLabels() const;
    void SetShowPWAlignLabels(bool);

    bool GetShowSwitchPoints() const;
    void SetShowSwitchPoints(bool);

    bool GetShowCDSProductFeats() const;
    void SetShowCDSProductFeats(bool);

    bool GetShowAlignmentFeats() const;
    void SetShowAlignmentFeats(bool);

    bool GetShowSegmentMap() const;
    void SetShowSegmentMap(bool);

    bool GetShowGraphs(void) const;
    void SetShowGraphs(bool b);

    bool GetShowProtProd() const;
    void SetShowProtProd(bool);

    enum EShowAnnotType {
        eShowAlignments,
        eShowPWAlignLabels,
        eShowSwitchPoints,
        eShowCDSProductFeats,
        eShowAlignmentFeats,
        eShowSegmentMap,
        eShowGraphs,
        eShowProtProd
    };

    /// generic accessors.
    bool GetShowAnnot(EShowAnnotType e);
    void SetShowAnnot(bool b, EShowAnnotType e);

    bool GetSmearAlignments() const;
    void SetSmearAlignments(bool);


};


inline
bool CPaneContentConf::GetShowAlignments(void) const
{
    return m_Settings.GetBool("ShowAlignments");
}

inline
void CPaneContentConf::SetShowAlignments(bool b)
{
    m_Settings.Set("ShowAlignments", b);
}


inline
bool CPaneContentConf::GetShowGraphs(void) const
{
    return m_Settings.GetBool("ShowGraphs");
}

inline
void CPaneContentConf::SetShowGraphs(bool b)
{
     m_Settings.Set("ShowGraphs", b);
}


inline
bool CPaneContentConf::GetShowPWAlignLabels(void) const
{
    return m_Settings.GetBool("ShowPWAlignLabels");
}

inline
void CPaneContentConf::SetShowPWAlignLabels(bool b)
{
     m_Settings.Set("ShowPWAlignLabels", b);
}

inline
bool CPaneContentConf::GetShowSwitchPoints(void) const
{
    return m_Settings.GetBool("ShowSwitchPoints");
}

inline
void CPaneContentConf::SetShowSwitchPoints(bool b)
{
     m_Settings.Set("ShowSwitchPoints", b);
}

inline
bool CPaneContentConf::GetShowCDSProductFeats(void) const
{
    return m_Settings.GetBool("ShowCDSProductFeats");
}

inline
void CPaneContentConf::SetShowCDSProductFeats(bool b)
{
     m_Settings.Set("ShowCDSProductFeats", b);
}

inline
bool CPaneContentConf::GetShowAlignmentFeats(void) const
{
    return m_Settings.GetBool("ShowAlignmentFeats");
}

inline
void CPaneContentConf::SetShowAlignmentFeats(bool b)
{
     m_Settings.Set("ShowAlignmentFeats", b);
}

inline
bool CPaneContentConf::GetShowSegmentMap(void) const
{
    return m_Settings.GetBool("ShowSegmentMap");
}

inline
void CPaneContentConf::SetShowSegmentMap(bool b)
{
     m_Settings.Set("ShowSegmentMap", b);
}


inline
bool CPaneContentConf::GetShowProtProd(void) const
{
    return m_Settings.GetBool("ShowProtProd");
}

inline
void CPaneContentConf::SetShowProtProd(bool b)
{
     m_Settings.Set("ShowProtProd", b);
}


inline
bool CPaneContentConf::GetSmearAlignments(void) const
{
    return m_Settings.GetBool("SmearAlignments");
}

inline
void CPaneContentConf::SetSmearAlignments(bool b)
{
     m_Settings.Set("SmearAlignments", b);
}



///
/// class CPaneContentMediator
/// mediate between CPaneContentConf configuration data and
/// and CFeatShowPanel widget.

/*class CFeatShowPanel;

class CPaneContentMediator : public CConfigMediator
{
public:
    CPaneContentMediator(CSettingsSet& data, CFeatShowPanel& panel);
    virtual ~CPaneContentMediator();

    /// What should we title the dialog window?
    virtual string      GetWindowTitle(void) const;
    /// Load data into that panel
    void                Load(const string& state);
    /// Load 'factory default' data into that panel.
    void                LoadDefault(void);
    /// Save the panel's data.
    void                Save(const string& state);
    /// Has the data represented by the panel's widgets been modified?
    bool                Modified(void) const;

protected:
    void x_Load(CSettingsSet& data, CFeatShowPanel& widgets);

// the following accessors are just to hide the ugly down casts.
    CFeatShowPanel&         x_GetMyPanel(void);

};*/


END_NCBI_SCOPE

#endif  /* GUI_WIDGETS_SEQ_GRAPHIC___PANECONTENT_CONF__HPP */
