#ifndef GUI_WIDGETS_SEQ_GRAPHIC___SEGMENT_MAP_DS__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___SEGMENT_MAP_DS__HPP

/*  $Id: segment_map_ds.hpp 25703 2012-04-24 19:57:50Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Liangshou Wu
 *
 * File Description:
 *
 */

#include <gui/widgets/seq_graphic/seqgraphic_genbank_ds.hpp>
#include <gui/widgets/seq_graphic/seq_glyph.hpp>

BEGIN_NCBI_SCOPE

///////////////////////////////////////////////////////////////////////////////
/// CSGSegmentMapDS
///
class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CSGSegmentMapDS : public CSGGenBankDS
{
public:
    CSGSegmentMapDS(objects::CScope& scope, const objects::CSeq_id& id);

    void LoadSegmentMap(const TSeqRange& range, int cutoff, TJobToken token);

    void LoadSegmentSmear(const TSeqRange& range, TModelUnit scale);

    void LoadSegmentMapSeqIDs(CSeqGlyph::TObjects& objs, TJobToken token);

    void LoadSwitchPoints();

    /// Get total number of segment map levels.
    int GetSegmentMapLevels(const TSeqRange& range) const;

    /// Query if there is segment maps for given level.
    /// @param level if it is -1, it checks all the way down until
    /// either finding one or reaching the bottom.
    bool HasSegmentMap(int level, const TSeqRange& range) const;

    void GetAnnotNames(objects::SAnnotSelector& sel,
        const TSeqRange& range, TAnnotNameTitleMap& names) const;

    void SetAnnot(const string& annot);
    const string& GetAnnot() const;


private:
    /// the named annotation storing segment map data.
    string m_Annot;
};

///////////////////////////////////////////////////////////////////////////////
/// CSGFeatureDSType
///
class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CSGSegmentMapDSType :
    public CObject,
    public ISGDataSourceType,
    public IExtension
{
public:
    /// create an instance of the layout track type using default settings.
    virtual ISGDataSource* CreateDS(SConstScopedObject& object) const;

    /// @name IExtension interface implementation
    /// @{
    virtual string GetExtensionIdentifier() const;
    virtual string GetExtensionLabel() const;
    /// @}

    /// check if the data source can be shared.
    virtual bool IsSharable() const;
};


///////////////////////////////////////////////////////////////////////////////
/// CSGFeatureDS inline methods
///
inline
void CSGSegmentMapDS::SetAnnot(const string& annot)
{ m_Annot = annot; }


inline
const string& CSGSegmentMapDS::GetAnnot() const
{ return m_Annot; }


END_NCBI_SCOPE

#endif  /* GUI_WIDGETS_SEQ_GRAPHIC___SEGMENT_MAP_DS__HPP */
