#ifndef GUI_WIDGETS_SEQ_GRAPHIC___SEQGRAPHIC_UTILS__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___SEQGRAPHIC_UTILS__HPP

/* $Id: seqgraphic_utils.hpp 28192 2013-05-23 14:52:01Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author:  Liangshou Wu
 *
 */

 /**
 * File Description:
 */

#include <corelib/ncbistr.hpp>
#include <corelib/ncbitime.hpp>
#include <gui/gui_export.h>


BEGIN_NCBI_SCOPE

BEGIN_SCOPE(objects)
class CSeq_annot_Handle;
class CScope;
class CBioseq_Handle;
END_SCOPE(objects)

///////////////////////////////////////////////////////////////////////////////
/// Utility classes for sequence graphical view rendering.
///

class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CSGUtils
{
public:
    static bool IsChromosome(const objects::CBioseq_Handle& handle,
        objects::CScope& scope);

    static bool IsSegSet(const objects::CBioseq_Handle& handle,
                         objects::CScope& scope);

    static bool IsmRNA(const objects::CBioseq_Handle& handle,
        objects::CScope& scope);

    static bool IsRefSeq(const objects::CBioseq_Handle& handle);

    static bool IsMitochondrion(const objects::CBioseq_Handle& handle);
    static bool IsPlastid(const objects::CBioseq_Handle& handle);

    static bool IsMainFeature(int type, int subtype);

    static bool GetFeatFetchDepth(const objects::CBioseq_Handle& handle,
        objects::CScope& scope, int& depth);

};


///////////////////////////////////////////////////////////////////////////////
/// CTimedReporter -- helper class for reporting time.
///
class CTimedReporter
{
public:
    CTimedReporter(const string& str);
    ~CTimedReporter();

private:
    CStopWatch m_sw;
    string m_Tag;
    static const int m_Step = 5;
    static int m_Indent;
};

#ifdef _DEBUG
#define TIMED_REPORTER_ENABLED
#endif

#ifdef TIMED_REPORTER_ENABLED
#define TIME_ME(msg) CTimedReporter SG_TIME_REPORTER(msg)
#else
#define TIME_ME(msg)
#endif


END_NCBI_SCOPE

#endif // GUI_WIDGETS_SEQ_GRAPHIC___SEQGRAPHIC_UTILS__HPP
