#ifndef GUI_WX_DEMO___GL_CANVAS__HPP
#define GUI_WX_DEMO___GL_CANVAS__HPP

/*  $Id: glcanvas.hpp 24998 2012-01-06 18:12:40Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 */

#include <gui/opengl.h>

// Disable deprecated warning for AGLDrawable in wx code (MAC specific)
#ifdef __WXMAC__
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#endif
#include <wx/glcanvas.h>
#ifdef __WXMAC__
#pragma GCC diagnostic warning "-Wdeprecated-declarations"
#endif
 
#include <gui/gui.hpp>
#include <gui/opengl/glviewport.hpp>
#include <gui/opengl/glpane.hpp>


BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// CGLCanvas
class NCBI_GUIWIDGETS_WX_EXPORT CGLCanvas :
    public wxGLCanvas
{
    DECLARE_EVENT_TABLE();
public:
    CGLCanvas(wxWindow* parent, wxWindowID id,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize,
                long style = 0);

    virtual ~CGLCanvas();

    /// Only used by some demo programs.  Note that Create
    /// is not a virtual function, so this would have to be
    /// called directly
    bool Create(wxWindow* parent, wxWindowID id,
                const wxPoint& pos, const wxSize& size,
                long style, const wxString& name);                

    void OnPaint(wxPaintEvent& event);
    void OnEraseBackground(wxEraseEvent& event);

    void DoNotUpdate() { m_DoNotUpdate = true; }

#ifdef __WXMAC_CARBON__
    static GLEWContext* GetGlewContext();
#endif

protected:
    void x_SetupGLContext();
    void x_CheckGlVersion();
    virtual void x_Render();
    bool m_DoNotUpdate;

#ifdef __WXMAC_CARBON__
    GLEWContext  m_GLEWContext;

    /// Only one context can be active at a time - the one for which makeCurrent
    /// was most recently called.
    static GLEWContext*  m_CurrentGLEWContext;

    // Assign a glew context to each unique wxGLContext
    std::vector<pair<wxGLContext*, GLEWContext*> > m_Contexts;
#endif
};


END_NCBI_SCOPE

#endif // GUI_WX_DEMO___GL_CANVAS__HPP
