#ifndef GUI_WIDGETS_WX__IBIOSEQ_EDITOR__HPP
#define GUI_WIDGETS_WX__IBIOSEQ_EDITOR__HPP

/*  $Id: ibioseq_editor.hpp 27922 2013-04-25 16:51:42Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

#include <objmgr/seq_entry_handle.hpp>
#include <objmgr/scope.hpp>

#include <wx/clntdata.h>
#include <wx/event.h>

class wxMenu;

BEGIN_NCBI_SCOPE

class ICommandProccessor;

class CTitleEdit : public CObject
{
public:
    void   SetTitle(const string& title) { m_Title = title; }
    string GetTitle() const { return m_Title; }

private:
    string m_Title;
};

class CEditObject
{
public:
    CEditObject(const CObject& obj, const objects::CSeq_entry_Handle& seh)
        : m_Obj(&obj), m_SEH(seh) {}
    CEditObject(const CObject& obj, objects::CScope& scope)
        : m_Obj(&obj), m_Scope(&scope) {}

    CConstRef<CObject> m_Obj;
    objects::CSeq_entry_Handle m_SEH;
    CRef<objects::CScope> m_Scope;
};

typedef vector<CEditObject> TEditObjectList;

class IBioseqEditorCB
{
public:
    virtual ~IBioseqEditorCB() {}

    virtual void GetCurrentSelection(TEditObjectList&) = 0;
    virtual objects::CBioseq_Handle GetCurrentBioseq() = 0;
};

/////////////////////////////////////////////////////////////////////////////
/// IEditor
class IBioseqEditor : public wxEvtHandler
{
public:
    virtual void SetCallBack(IBioseqEditorCB* cb) = 0;

    virtual wxMenu* CreateContextMenu() const = 0;
    virtual wxMenu* CreateBioseqMenu() const = 0;
    virtual void EditSelection() = 0;
};


END_NCBI_SCOPE

#endif //GUI_WIDGETS_WX__IBIOSEQ_EDITOR__HPP
