#ifndef GUI_WIDGETS_WORKSPACE___WM_CONTAINER_HPP
#define GUI_WIDGETS_WORKSPACE___WM_CONTAINER_HPP

/*  $Id: iwindow_manager.hpp 26234 2012-08-08 20:22:24Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

#include <gui/utils/command.hpp>

#include <gui/widgets/wx/wm_client.hpp>

#include <wx/defs.h>


BEGIN_NCBI_SCOPE

///////////////////////////////////////////////////////////////////////////////
/// IWindowManager - interface for Window Manager Containers
/// Callback interface allowing IWMContainers to access Window Manager

/// CWindowManger commands
enum    EWindowManagerCmds  {
    eCmdSetClientColor = 5100,
    eCmdShowWindowsDlg,         // shows "Windows" dialog

    /// a range of automatically generated commands that will be used for
    /// activating windows
    eCmdWindowXXXX = wxID_HIGHEST + 500,
    eCmdWindowLast = wxID_HIGHEST + 599
};


///////////////////////////////////////////////////////////////////////////////
/// IDockPanel
class IDockPanel
{
public:
    virtual IWMClient*  GetClient() = 0;
    virtual const   IWMClient*  GetClient() const = 0;

    virtual ~IDockPanel()  {}
};

///////////////////////////////////////////////////////////////////////////////
/// IDockContWindow - abstract window container for use with Dock Manager.
/// This interfaces allows Dock Manager to work transparently with different
/// container implementations.
class IDockContWindow
{
public:
    virtual void    Cont_Add(wxWindow* child, bool resizable) = 0;
    virtual void    Cont_Add(vector<wxWindow*> children, int resizable_index) = 0;
    virtual void    Cont_Remove(wxWindow* child) = 0;
    virtual void    Cont_Replace(wxWindow* old_child, wxWindow* new_child) = 0;

    virtual ~IDockContWindow()  {};
};


///////////////////////////////////////////////////////////////////////////////
/// IWindowManager - the callback interface to Window Manager used by
/// its components.
class IWindowManager
{
public:
    virtual bool    OnCaptionPanelCommand(IDockPanel& panel, TCmdID cmd) = 0;
    virtual bool    OnCloseClientsRequest(const vector<IWMClient*>& clients) = 0;
    virtual IWMClient*  GetClientByWindow(wxWindow* widget) = 0;
    virtual void    OnFocusChanged(wxWindow* new_focus) = 0;

    /// returns a title for a new floating frame
    virtual string  GetNewFloatingFrameTitle() = 0;

    virtual ~IWindowManager()   {};
};


///////////////////////////////////////////////////////////////////////////////
///
class IWindowManagerMenuListener
{
public:
    virtual ~IWindowManagerMenuListener() {}
    virtual void    UpdateMenuBar() = 0;
    /// no destructor - this interface should not be deleted
};


END_NCBI_SCOPE

#endif  // GUI_WIDGETS_WORKSPACE___WM_CONTAINER_HPP
