/*  $Id: commands.cpp 25479 2012-03-27 14:55:33Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include "commands.hpp"

#include <gui/widgets/wx/ui_command.hpp>
#include <gui/widgets/wx/fileartprov.hpp>

#include <wx/wx.h>


BEGIN_NCBI_SCOPE


void Gbench_RegisterCommands(CUICommandRegistry& cmd_reg, wxFileArtProvider& provider)
{
    /// Standard wxWidgets commands that have non-standard attributes
    cmd_reg.RegisterCommand(wxID_OPEN, "&Open...\tCtrl+O", "Open...",
                            "menu::open", "Opens Workspace or Project, or loads data");
    cmd_reg.RegisterCommand(wxID_NEW, "&New...", "New...",
                             "", "Creates a new Workspace or Project");
    cmd_reg.RegisterCommand(wxID_SAVE, "&Save...\tCtrl+S", "Save...",
                             "menu::save", "Saves Workspace and Projects");
    cmd_reg.RegisterCommand(wxID_SAVEAS, "Save &As...", "Save As...",
                             "", "");

    cmd_reg.RegisterCommand(eCmdShowProjectTreeView, "&Project Tree", "Project Tree",
                            "", "Show Project Tree");
    cmd_reg.RegisterCommand(eCmdShowSearchView, "&Search View", "Search View",
                            "menu::search_view", "Shows Search View");
    cmd_reg.RegisterCommand(eCmdShowSelectionInspector, "Selection &Inspector", "Selection Inspector",
                            "", "Shows Selection Inspector");
    cmd_reg.RegisterCommand(eCmdShowTaskView, "&Task View", "Task View",
                            "menu::task_view", "Shows Task View");
    cmd_reg.RegisterCommand(eCmdShowEventView, "&Event View", "Event View",
                            "", "Shows Event View");
    cmd_reg.RegisterCommand(eCmdShowDiagConsole, "&Diagnostic Console", "Diagnostic Console",
                            "", "Shows Diagnostic Console");

    cmd_reg.RegisterCommand(eCmdSaveLayout, "S&ave Layout", "Save Layout",
                            "", "Saves window layout");
    cmd_reg.RegisterCommand(eCmdLoadLayout, "&Load Layout", "Load Layout",
                            "", "Loads window layout");
    cmd_reg.RegisterCommand(eCmdResetLayout, "Reset &Layout", "Reset Layout",
                            "", "Resets window layout");

    cmd_reg.RegisterCommand(eCmdRunTool, "Run &Tool...\tCtrl+T", "Run Tool...",
                            "menu::tool", "Shows a dialog with a list of available tools");
    cmd_reg.RegisterCommand(eCmdSearch, "&Search...", "Search...", "menu::search", "");

    cmd_reg.RegisterCommand(eCmdToolsOptions, "&Options", "Options",
                            "", "Options");

    cmd_reg.RegisterCommand(eCmdToolsPackages, "&Packages...", "Packages",
                            "", "Packages");

    // menu Help TODO - move to Help Service
    #if defined( __WXMAC__ )
    cmd_reg.RegisterCommand(eCmdAbout, "&About Genome Workbench", "About Genome Workbench",
                            "", "About Genome Workbench");
    #else
    cmd_reg.RegisterCommand(wxID_ABOUT, "&About Genome Workbench", "About Genome Workbench",
                            "", "About Genome Workbench");
    #endif

    // Help commands
    cmd_reg.RegisterCommand(eCmdSendFeedback, "&Send feedback...", "Send feedback...",
        "", "Opens wizard to send feedback");

    cmd_reg.RegisterCommand(eCmdTestPrjSrv, "Test Project Service", "Test Project Service",
                            "", "");
    cmd_reg.RegisterCommand(
        eCmdTestLoadBlastMask, "Test Load Blast Mask", "Test Load Blast Mask", "", ""
    );
    cmd_reg.RegisterCommand(eCmdMacroEditDlg, "Open Macro Editor", "Open Macro Editor",
                            "", "");

    /*cmd_reg.RegisterCommand(, "", "",
                            "", "");
    */

    /// register menu images
    provider.RegisterFileAlias(wxT("menu::open"), wxT("file_open.png"));
    provider.RegisterFileAlias(wxT("menu::save"), wxT("file_save.png"));
    provider.RegisterFileAlias(wxT("menu::search"), wxT("search.png"));
    provider.RegisterFileAlias(wxT("menu::tool"), wxT("tool.png"));
    provider.RegisterFileAlias(wxT("menu::broadcast_options"), wxT("broadcast_options.png"));
    //provider.RegisterFileAlias(wxT("menu::winmask_files_dnld"), wxT("winmask_files_dnld.png"));

    provider.RegisterFileAlias(wxT("menu::search_view"), wxT("search.png"));
    provider.RegisterFileAlias(wxT("menu::task_view"), wxT("task_view.png"));

}


END_NCBI_SCOPE
