/*  $Id: named_pipe_def_client.cpp 25479 2012-03-27 14:55:33Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include "gbench_app.hpp"
#include "named_pipe_def_client.hpp"

#include <gui/utils/extension_impl.hpp>
#include <gui/core/project_task.hpp>
#include <gui/core/ui_file_load_manager.hpp>
#include <gui/core/loading_app_job.hpp>
#include <gui/framework/app_task_service.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

BEGIN_NCBI_SCOPE

string CNamedPipeDefClient::GetExtensionIdentifier() const
{
    static string sid("gbench_named_pipe_default_client");
    return sid;
}

string CNamedPipeDefClient::GetExtensionLabel() const
{
    static string slabel("Genome Workbench Named Pipe Default Client");
    return slabel;
}

bool CNamedPipeDefClient::Process(const string& data, IWorkbench* workbench)
{
    vector<string> tokens;
    string delim;
    delim = '\0';  // not an empty string
    NStr::Tokenize (data, delim, tokens);
    if (tokens.size() > 1 && tokens[1] == "app_activate")
        return true;

    wxString fname = wxString::FromUTF8((tokens[0].c_str()));

    wxString l_ftype = wxT("auto");
    {{
        wxString l_fname = fname;
        l_fname.MakeLower();
        l_ftype.MakeLower();

        if (l_fname.EndsWith(wxT(".gbw")))
            l_ftype = wxT("workspace");
        else if (l_fname.EndsWith(wxT(".gbp")))
            l_ftype = wxT("project");
    }}

    if (l_ftype == wxT("project")) {
        CProjectTask* task = new CProjectTask(workbench->GetServiceLocator());
        vector<wxString> filenames;
        filenames.push_back(fname);
        task->Init_OpenProjects(filenames);
        workbench->GetAppTaskService()->AddTask(*task);
    }
    else if (l_ftype == wxT("workspace")) {
        CProjectTask* task = new CProjectTask(workbench->GetServiceLocator());
        task->Init_OpenWorkspace(fname);
        workbench->GetAppTaskService()->AddTask(*task);
    }
    else {
        CIRef<IFileFormatLoaderManager> formatManager;
        CFormatGuess::EFormat fmt = CFormatGuess::eUnknown;

        try {
            CNcbiIfstream istr(fname.fn_str(), ios::binary);
            fmt = CFormatGuess(istr).GuessFormat();
        }
        catch (CException&) {
        }

        if (fmt != CFormatGuess::eUnknown) {
            vector< CIRef<IFileFormatLoaderManager> > managers;
            GetExtensionAsInterface("file_format_loader_manager", managers);
            for (size_t i = 0;  i < managers.size();  ++i) {
                IFileFormatLoaderManager* mgr = managers[i].GetPointer();
                if (mgr->RecognizeFormat(fmt)) {
                    formatManager.Reset(mgr);
                    break;
                }
            }
        }

        if (formatManager) {
            formatManager->SetServiceLocator(workbench);

            vector<wxString> filenames;
            filenames.push_back(fname);
            formatManager->SetFilenames(filenames);

            IAppTask* task = formatManager->GetTask();
            if (task) {
                CDataLoadingAppTask* dlTask = dynamic_cast<CDataLoadingAppTask*>(task);
                if (dlTask) {
                    CIRef<IAppJob> job = dlTask->GetJob();
                    CDataLoadingAppJob* dlJob =
                        dynamic_cast<CDataLoadingAppJob*>(job.GetPointer());

                    if (dlJob) {
                        CDataLoadingOptions options;
                        options.Set_CreateNewProject();
                        dlJob->SetOptions(options);
                    }
                }
                workbench->GetAppTaskService()->AddTask(*task);
            }
        }
    }

    return true;
}

END_NCBI_SCOPE
