/*  $Id: named_pipe_process_task.cpp 25164 2012-01-27 21:06:51Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include "named_pipe_process_task.hpp"
#include "gbench_app.hpp"

#include <gui/utils/extension_impl.hpp>
#include <gui/core/app_named_pipe_client.hpp>

BEGIN_NCBI_SCOPE

DECLARE_APP(ncbi::CGBenchApp)

CNamedPipeProcessTask::CNamedPipeProcessTask(IWorkbench* workbench, const string& data, bool bringToFront)
:   CAppTask("Process data from named pipe", false),
    m_Workbench(workbench), m_Data(data), m_BringToFront(bringToFront)
{
    _ASSERT(m_Workbench);
}

IAppTask::ETaskState CNamedPipeProcessTask::x_Run()
{   
    bool processed = false;

    vector<IAppNamedPipeClient*> clients;
    IAppNamedPipeClient* default_client = 0;
    GetExtensionAsInterface(EXT_POINT__GBENCH_NAMED_PIPE_CLIENTS, clients);

    ITERATE(vector<IAppNamedPipeClient*>, it, clients) {
        if ((*it)->IsDefault()) {
            default_client = *it;
            continue;
        }

        if ((*it)->Process(m_Data, m_Workbench)) {
            processed = true;
            break;
        }
    }

    if (!processed && default_client)
        processed = default_client->Process(m_Data, m_Workbench);

    if (processed && m_BringToFront) {
        CGBenchApp& app = wxGetApp();
        wxTopLevelWindow* win = (wxTopLevelWindow*)app.GetTopWindow();
        if(win) {
            if (win->IsIconized())
                win->Maximize(false);
            win->RequestUserAttention();
        }
    }

    return eCompleted;
}

END_NCBI_SCOPE
