/*  $Id: read_pipe_thread.cpp 26425 2012-09-14 19:44:45Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/core/named_pipe.hpp>
#include <gui/framework/workbench.hpp>
#include <gui/framework/app_task_service.hpp>

#include "named_pipe_process_task.hpp"
#include "read_pipe_thread.hpp"

BEGIN_NCBI_SCOPE

CReadPipeThread::CReadPipeThread(IWorkbench* workbench)
    : m_Workbench(workbench), m_StopSignal(0, 10000000)
{
}

void CReadPipeThread::RequestStop()
{
    m_StopSignal.Post();
}

void* CReadPipeThread::Main(void)
{
    string err_msg;

    try {
        for (;;) {
            string data = CGBenchPipe::Read();
            if (m_StopSignal.TryWait())
                break;

            if (!data.empty()) {
                CAppTaskService* app_task = m_Workbench->GetAppTaskService();
                if (app_task) {
                    CNamedPipeProcessTask* task = new CNamedPipeProcessTask(m_Workbench, data, true);
                    app_task->AddTask(*task);
                } else {
                    LOG_POST(Error << "Application Task Service not found: cannot start CNamedPipeProcessTask");
                }
            }

            if (m_StopSignal.TryWait())
                break;
        }
    }
    catch (std::exception& e) {
        err_msg = "Failed to read via named pipe: ";
        err_msg += e.what();
    }
    catch (...) {
        err_msg = "Failed to read via named pipe: Unknown error.";
    }

    if (!err_msg.empty())
        LOG_POST(Error << err_msg);

    return 0;
}

END_NCBI_SCOPE
