#!/bin/sh
#Shell script to create a double-clickable Mac application package from parts.

if [ $# -ne 3 ] ; then
    echo Usage: create_mac_app.sh top_source_directory build_root_directory package_name
    exit 1
fi

abs_top_srcdir=$1
build_root=$2
package_name=$3
mac_res_dir=${abs_top_srcdir}/src/gui/res/share/gbench
install_dir=${build_root}/bin/gbench
mac_app_dir=${install_dir}/${package_name}.app
bindir=${build_root}/bin
windowmasker_ini='window-masker.ini' 

sysver=`sw_vers -productVersion | cut -c 1-4`
echo $sysver
if [ $sysver = 10.6 ]; then
echo "This is a Snow Leopard System"
elif [ $sysver = 10.5 ]; then
echo "This is a Leopard System"
else
echo "This system is too old, proceed on your own risk"
fi


#First, create the app package directory structure, copy the binary, plugins, packages, icons, etc.
echo "Installing Mac App Package: Genome Workbench"
install -d "${mac_app_dir}/Contents/MacOS"
install -d "${mac_app_dir}/Contents/Resources"
install -d "${mac_app_dir}/Contents/Frameworks"

echo Installing ${bindir}/gbench-bin as "${mac_app_dir}/Contents/MacOS/Genome Workbench"
install ${bindir}/gbench-bin "${mac_app_dir}/Contents/MacOS/Genome Workbench"

utils="gbench_monitor gbench_feedback_agent windowmasker blastp blastn blastx tblastn tblastx"
(cd ${bindir}; install ${utils} "${mac_app_dir}/Contents/MacOS/")

util_dirs="etc share packages"
(cd ${bindir}/gbench; cp -R ${util_dirs} "${mac_app_dir}/Contents/MacOS/")

ln -f ${mac_res_dir}/gbench.icns  "${mac_app_dir}/Contents/Resources/Genome Workbench.icns"
ln -f ${mac_res_dir}/gbench_project.icns  "${mac_app_dir}/Contents/Resources/gbench_project.icns"
ln -f ${mac_res_dir}/gbench_workspace.icns  "${mac_app_dir}/Contents/Resources/gbench_workspace.icns"
sed 's/XX.X/'${sysver}'/g' ${mac_res_dir}/Info.plist > "${mac_app_dir}/Contents/Info.plist"

#create widowmasker ini if absent
if [ -e $windowmasker_ini ]; then
	echo "Windowmasker file exists and will not be generated."
else 
	echo "Windo does not exists - generating..."
	echo "[WindowMasker]" >> "${mac_app_dir}/Contents/MacOS/etc/${windowmasker_ini}"	
	echo "DataPath=/Volumes/gbench/data/window-masker" >> "${mac_app_dir}/Contents/MacOS/etc/${windowmasker_ini}"	
fi 


#We explicitly copy the dylibs that are not explicitly linked.
#ncbi_xloader_*, ncbi_xreader_*, ncbi_xcache_*, and ncbi_xdbapi_*
install "${build_root}"/lib/libncbi_xloader_*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libncbi_xreader_*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libncbi_xcache_*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libncbi_xcache_sqlite3.so "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libncbi_xdbapi_*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libblast*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libxblastformat*.dylib "${mac_app_dir}/Contents/Frameworks"
install "${build_root}"/lib/libblastdb_format*.dylib "${mac_app_dir}/Contents/Frameworks"


#Now copy all the dylibs that we built and are needed by the binary.
echo Copying Mac App Package Libraries
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/Genome Workbench" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    echo Installing Mac App Package: `basename $ff`
    if [ -h "$ff" ]; then
        export LINK_NAME=`readlink $ff`
        export FULL_LINK_NAME=`readlink -f $ff`

        if [ -h `readlink $FULL_LINK_NAME` ]; then
            export FULL_SUB_LINK_NAME=`readlink -f $FULL_LINK_NAME`
            export SUB_LINK_NAME=`basename $FULL_SUB_LINK_NAME`

            echo $ff is a double symbolic link to $SUB_LINK_NAME or $FULL_SUB_LINK_NAME
        
            echo install  "$FULL_SUB_LINK_NAME"  "${mac_app_dir}/Contents/Frameworks/$SUB_LINK_NAME"
            install  "$FULL_SUB_LINK_NAME"  "${mac_app_dir}/Contents/Frameworks/$SUB_LINK_NAME"
           
            echo ln -s  "$SUB_LINK_NAME" "${mac_app_dir}/Contents/Frameworks/$LINK_NAME"
            ln -s  "$SUB_LINK_NAME" "${mac_app_dir}/Contents/Frameworks/$LINK_NAME"

            echo ln -s  "$LINK_NAME" "${mac_app_dir}/Contents/Frameworks/`basename $ff`"
            ln -s  "$LINK_NAME" "${mac_app_dir}/Contents/Frameworks/`basename $ff`"
        else 
            echo $ff is a symbolic link to $LINK_NAME
            echo install  "`readlink -f $ff`"  "${mac_app_dir}/Contents/Frameworks/$LINK_NAME"
            install "`readlink -f $ff`"  "${mac_app_dir}/Contents/Frameworks/$LINK_NAME"

            echo ln -s  "$LINK_NAME" "${mac_app_dir}/Contents/Frameworks/`basename $ff`"
            ln -s  "$LINK_NAME" "${mac_app_dir}/Contents/Frameworks/`basename $ff`"
        fi
    else
        install "$ff" "${mac_app_dir}/Contents/Frameworks/`basename $ff`"
    fi
done

#We copy the dylibs needed by the .so plugins.
for ff in `find "${mac_app_dir}/Contents/MacOS/packages" -name '*.so' -exec otool -L {} \\; \
  | awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    if [ ! -f "${mac_app_dir}"/Contents/Frameworks/`basename $ff` ] ; then
        install $ff "${mac_app_dir}/Contents/Frameworks/"
        echo Installed package dylib: `basename $ff`
    fi
done

#We copy the dylibs needed by the .dylib plugins.
for ff in `find "${mac_app_dir}/Contents/MacOS/packages" -name '*.dylib' -exec otool -L {} \\; \
  | awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    if [ ! -f "${mac_app_dir}"/Contents/Frameworks/`basename $ff` ] ; then
        install $ff "${mac_app_dir}/Contents/Frameworks/"
        echo Installed package dylib: `basename $ff`
    fi
done

#Next we copy all the dylibs that the dylibs we copied in the previous step may require.  We
#repeat this step in case the new dylibs require other dylibs.
echo Finding missing Mac App Package Libraries
for i in 1 2 3; do
    for ff in "${mac_app_dir}"/Contents/Frameworks/*.dylib ; do
        for ll in `otool -L "$ff" | awk '{print $1}' | \
          egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
            if [ ! -f "${mac_app_dir}"/Contents/Frameworks/`basename $ll` ] ; then
                install "$ll" "${mac_app_dir}/Contents/Frameworks/"
                echo Pass ${i}: Installed missing dylib: `basename $ll`
            fi
        done
    done
done

#Once all the dylibs are copied into the package, we change the path for the binary dylibs
#to the package itself where we just copied everything.
echo Adjusting Mac App Package Binary Paths
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/Genome Workbench" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/Genome Workbench"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/gbench_monitor" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/gbench_monitor"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/gbench_feedback_agent" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/gbench_feedback_agent"
done
#for ff in `otool -L "${mac_app_dir}/Contents/MacOS/libncbi_xcache_sqlite3.so" | \
#  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
#    install_name_tool -change $ff \
#      @executable_path/../Frameworks/`basename $ff` \
#      "${mac_app_dir}/Contents/MacOS/libncbi_xcache_sqlite3.so"
#done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/windowmasker" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/windowmasker"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/blastn" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/blastn"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/blastp" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/blastp"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/blastx" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/blastx"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/tblastn" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/tblastn"
done
for ff in `otool -L "${mac_app_dir}/Contents/MacOS/tblastx" | \
  awk '{print $1}' | egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
    install_name_tool -change $ff \
      @executable_path/../Frameworks/`basename $ff` \
      "${mac_app_dir}/Contents/MacOS/tblastx"
done


#We update the dylib path for the .so packages.
echo Adjusting .so package library paths
find "${mac_app_dir}/Contents/MacOS/packages" -name '*.so' | while read ff ; do
    install_name_tool -id \@executable_path/../Frameworks/`basename "$ff"` "$ff"
    for ll in `otool -L "$ff" | awk '{print $1}' | \
      egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
        install_name_tool -change $ll \
          @executable_path/../Frameworks/`basename $ll` \
          "$ff" ; \
    done
done

#We update the dylib path for the .dlylib packages.
echo Adjusting .dylib package library paths
find "${mac_app_dir}/Contents/MacOS/packages" -name '*.dylib' | while read ff ; do
    install_name_tool -id \@executable_path/../Frameworks/`basename "$ff"` "$ff"
    for ll in `otool -L "$ff" | awk '{print $1}' | \
      egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
        install_name_tool -change $ll \
          @executable_path/../Frameworks/`basename $ll` \
          "$ff" ; \
    done
done

#Finally we update the path of the dylibs themselves so they match their physical location in the
#package.
echo Adjusting Mac App Package Library Paths ; \
for ff in "${mac_app_dir}"/Contents/Frameworks/*.dylib ; do
    install_name_tool -id \@executable_path/../Frameworks/`basename "$ff"` "$ff"
    for ll in `otool -L "$ff" | awk '{print $1}' | \
      egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
        install_name_tool -change $ll \
          @executable_path/../Frameworks/`basename $ll` \
          "$ff" ; \
    done
done
for ff in "${mac_app_dir}"/Contents/Frameworks/*.so ; do
    if [ -e "$ff" ]; then
    	install_name_tool -id \@executable_path/../Frameworks/`basename "$ff"` "$ff"
    	for ll in `otool -L "$ff" | awk '{print $1}' | \
      	egrep "(^/(netopt).*\.dylib$)|(${build_root}/lib)"` ; do
        	install_name_tool -change $ll \
          	@executable_path/../Frameworks/`basename $ll` \
          	"$ff" ; \
    	done
    fi
done

#Now that we have the app package built, we put it into a dmg
echo Installing disk image: "${package_name}.dmg"
hdiutil create -srcfolder "${mac_app_dir}" -ov -fs HFS+ -volname 'NCBI Genome Workbench' \
	"${install_dir}/${package_name}.dmg"

