/*  $Id: app_explorer_data_object.cpp 26121 2012-07-23 19:35:02Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/core/app_explorer_data_object.hpp>


BEGIN_NCBI_SCOPE


///////////////////////////////////////////////////////////////////////////////
///  CAppExplorerData

CAppExplorerData::CAppExplorerData()
    : m_Valid(false)
{
}

CAppExplorerData::CAppExplorerData( TItemRefVector& items, bool cut )
    : m_Valid(true)
    , m_Items(items)
    , m_Cut(cut)
{
}

CAppExplorerData::~CAppExplorerData()
{
}

void CAppExplorerData::SetItems( TItemRefVector& items, bool cut )
{
    m_Items = items;
    m_Valid = true;
    m_Cut = cut;
}


void CAppExplorerData::GetItems( TItemRefVector& items )
{
    _ASSERT( m_Valid );

    if( m_Valid ){
        items = m_Items;
    }
}

bool CAppExplorerData::IsCut() const
{
    return m_Cut;
}


bool CAppExplorerData::IsValid() const
{
    return m_Valid;
}


void CAppExplorerData::Invalidate()
{
    m_Items.clear();
    m_Valid = false;
}


///////////////////////////////////////////////////////////////////////////////
///  CAppExplorerDataObject

const wxChar* CAppExplorerDataObject::sm_ItemsFormatName = wxT("gbench/app_explorer_items");

CAppExplorerDataObject::CAppExplorerDataObject()
    : m_ItemsFormat( sm_ItemsFormatName )
{
    SetFormat(m_ItemsFormat);
}


CAppExplorerDataObject::CAppExplorerDataObject( 
    CAppExplorerData& data
)
    : m_ItemsFormat( sm_ItemsFormatName )
    , m_Data(&data)
{
    SetFormat(m_ItemsFormat);
    _ASSERT(data.IsValid());
}

//TODO In all functions check whether text object is full (see if it can do it itself)

size_t CAppExplorerDataObject::GetDataSize() const
{
    return sizeof(TAppExplExchange);
}

bool CAppExplorerDataObject::GetDataHere(void * buf) const
{
    if( m_Data && m_Data->IsValid() ){
        TAppExplExchange* exch = reinterpret_cast<TAppExplExchange*>(buf);
        CAppExplorerData* data =
            const_cast<CAppExplorerData*>( m_Data.GetPointer() )
        ;
        exch->Init( data );

        return true;
    }
    return false;
}

bool CAppExplorerDataObject::SetData(size_t len, const void *buf)
{
    bool ok = buf && (len == sizeof(TAppExplExchange));
    _ASSERT(ok);

    if( ok ){
        TAppExplExchange* exch = (TAppExplExchange*) buf;
        if( exch->IsLocal() ){
            m_Data.Reset( exch->GetData() );

            return true;
        }
    }
    return false;
}

END_NCBI_SCOPE
