/*  $Id: app_explorer_item.cpp 22701 2010-12-14 21:35:07Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/core/app_explorer_item.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

BEGIN_NCBI_SCOPE

set<CExplorerItem*>  CExplorerItem::sm_AliveItems;

CExplorerItem::CExplorerItem(const wxString& label, bool is_expanded)
:   m_Label(label),
    m_NormalIcon(-1),
    m_ExpandedIcon(-1),
    m_Enabled(true),
    m_Expanded(is_expanded)
{
    sm_AliveItems.insert(this);
}


CExplorerItem::CExplorerItem(const wxString& label, 
                             int normal, int expanded,
                             bool is_expanded)
:   m_Label(label),
    m_Enabled(true),
    m_Expanded(is_expanded)
{
    SetIconIndex(normal, expanded);
    sm_AliveItems.insert(this);
}


CExplorerItem::~CExplorerItem()
{
    EraseAllChildItems(true);
    sm_AliveItems.erase(this);
}


const wxString& CExplorerItem::GetLabel() const
{
    return m_Label;
}


void CExplorerItem::SetLabel(const wxString& s)
{
    m_Label = s;
}

void CExplorerItem::SetLabel(const string& s)
{
    m_Label = ToWxString(s);
}

int CExplorerItem::GetIconIndex(wxTreeItemIcon which) const
{
    switch(which)   {
    case wxTreeItemIcon_Normal:
    case wxTreeItemIcon_Selected:
        return m_NormalIcon;

    case wxTreeItemIcon_Expanded:
    case wxTreeItemIcon_SelectedExpanded:
        return m_ExpandedIcon;

    default:
        _ASSERT(false); // unexpected
    }
    return -1;
}


void CExplorerItem::SetIconIndex(int normal, int expanded)
{
    m_NormalIcon = normal;
    m_ExpandedIcon = (expanded >= 0) ? expanded : normal;
}


bool CExplorerItem::IsExpanded() const
{
    return m_Expanded;
}


void CExplorerItem::Expand(bool exp)
{
    m_Expanded = exp;
}


bool CExplorerItem::IsEnabled() const
{
    return m_Enabled;
}


bool CExplorerItem::IsDisabled() const
{
    return ! m_Enabled;
}


void CExplorerItem::Enable(bool en)
{
    m_Enabled = en;
}


CExplorerItem* CExplorerItem::GetParent()
{
    return m_Parent;
}


const CExplorerItem* CExplorerItem::GetParent() const
{
    return m_Parent;
}


const CExplorerItem::TItemRefVector& CExplorerItem::GetChildItems() const
{
    return m_ChildItems;
}


CExplorerItem::TItemRefVector& CExplorerItem::x_GetChildItems()
{
    return m_ChildItems;
}


void CExplorerItem::AddChildItem(CExplorerItem& ch_item)
{
    m_ChildItems.push_back(TItemRef(&ch_item));
    ch_item.m_Parent.Reset(this);
}


void CExplorerItem::EraseChildItem(CExplorerItem& ch_item)
{
    TItemRefVector::iterator it =
        std::find(m_ChildItems.begin(), m_ChildItems.end(), TItemRef(&ch_item));

    _ASSERT(it != m_ChildItems.end());

    ch_item.m_Parent.Reset();
    m_ChildItems.erase(it);
}


void CExplorerItem::EraseChildItem(TItemRefVector::iterator it)
{
    (*it)->m_Parent.Reset();
    m_ChildItems.erase(it);
}


void CExplorerItem::EraseChildItem(size_t index)
{
    EraseChildItem(m_ChildItems.begin() + index);
}


void CExplorerItem::EraseAllChildItems(bool recursively)
{
    for(  size_t i = 0;  i < m_ChildItems.size();  i++ )    {
        CExplorerItem& item = *m_ChildItems[i];
        if(recursively) {
            item.EraseAllChildItems(true);
        }
        item.m_Parent.Reset();
    }
    m_ChildItems.clear();
}


END_NCBI_SCOPE
