/*  $Id: asn_export_page.cpp 25479 2012-03-27 14:55:33Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>////@begin includes
////@end includes

#include "asn_export_page.hpp"

#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/radiobox.h>
#include <wx/valgen.h>
#include <wx/valtext.h>
#include <wx/icon.h>
#include <wx/bitmap.h>
#include <wx/textctrl.h>
#include <wx/filedlg.h>
#include <wx/msgdlg.h>

#include <gui/widgets/wx/table_utils.hpp>

#include <gui/widgets/object_list/object_list_widget.hpp>

#include <gui/core/file_extensions.hpp>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CAsnExportPage, wxPanel )

BEGIN_EVENT_TABLE( CAsnExportPage, wxPanel )

////@begin CAsnExportPage event table entries
    EVT_BUTTON( ID_BUTTON3, CAsnExportPage::OnButton3Click )

////@end CAsnExportPage event table entries

END_EVENT_TABLE()

CAsnExportPage::CAsnExportPage()
{
    Init();
}

CAsnExportPage::CAsnExportPage( wxWindow* parent, wxWindowID id, const wxPoint& pos, long style )
{
    Init();
    Create(parent, id, pos, style);
}

bool CAsnExportPage::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, long style )
{
    wxPanel::Create( parent, id, pos, wxSize(0,0), style );
    Hide();

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();

    return true;
}

CAsnExportPage::~CAsnExportPage()
{
////@begin CAsnExportPage destruction
////@end CAsnExportPage destruction
}

void CAsnExportPage::Init()
{
////@begin CAsnExportPage member initialisation
    m_ObjectSel = NULL;
////@end CAsnExportPage member initialisation
}

void CAsnExportPage::CreateControls()
{
////@begin CAsnExportPage content construction
    CAsnExportPage* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    m_ObjectSel = new CObjectListWidget( itemPanel1, ID_PANEL2, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(m_ObjectSel, 1, wxGROW|wxALL, 0);

    wxArrayString itemRadioBox4Strings;
    itemRadioBox4Strings.Add(_("&Text"));
    itemRadioBox4Strings.Add(_("&Binary"));
    wxRadioBox* itemRadioBox4 = new wxRadioBox( itemPanel1, ID_RADIOBOX3, _("ASN Type"), wxDefaultPosition, wxDefaultSize, itemRadioBox4Strings, 1, wxRA_SPECIFY_ROWS );
    itemRadioBox4->SetSelection(0);
    itemBoxSizer2->Add(itemRadioBox4, 0, wxALIGN_RIGHT|wxALL, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer5, 0, wxGROW|wxALL, 0);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("File name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl7 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL6, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemTextCtrl7, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemPanel1, ID_BUTTON3, _("..."), wxDefaultPosition, wxSize(itemPanel1->ConvertDialogToPixels(wxSize(24, -1)).x, -1), 0 );
    itemBoxSizer5->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Set validators
    itemRadioBox4->SetValidator( wxGenericValidator(& GetData().m_AsnType) );
    itemTextCtrl7->SetValidator( wxTextValidator(wxFILTER_NONE, & GetData().m_FileName) );
////@end CAsnExportPage content construction
}

bool CAsnExportPage::ShowToolTips()
{
    return true;
}
wxBitmap CAsnExportPage::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CAsnExportPage bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CAsnExportPage bitmap retrieval
}
wxIcon CAsnExportPage::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CAsnExportPage icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CAsnExportPage icon retrieval
}

bool CAsnExportPage::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;

    m_ObjectSel->SetObjects(m_data.m_Objects);
    m_ObjectSel->SelectAll();

    return true;
}
bool CAsnExportPage::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    wxString path = GetData().GetFileName();
    path.Trim();

    if (path.empty()) {
        wxMessageBox(wxT("Please, select file name"), wxT("Error"),
                     wxOK | wxICON_ERROR, this);
        wxTextCtrl* textCtrl = (wxTextCtrl*)FindWindow(ID_TEXTCTRL6);
        textCtrl->SetFocus();
        return false;
    }

    TConstScopedObjects objects;
    m_ObjectSel->GetSelection(objects);

    if (objects.empty()) {
        wxMessageBox(wxT("Please, select at least one object to export"), wxT("Error"),
                     wxOK | wxICON_ERROR, this);
        FindWindow(ID_PANEL2)->SetFocus();
        return false;
    }

    GetData().m_Objects = objects;

    return true;
}

static const char* kObjectList = "ObjectList";

void CAsnExportPage::SetRegistryPath(const string& path)
{
    m_RegPath = path;
}

void CAsnExportPage::SaveSettings() const
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadWriteView table_view;

        reg_path = CGuiRegistryUtil::MakeKey(m_RegPath, kObjectList);
        table_view = gui_reg.GetReadWriteView(reg_path);
        ncbi::SaveTableSettings(*m_ObjectSel, table_view);
    }
}

void CAsnExportPage::LoadSettings()
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadView table_view;

        reg_path = CGuiRegistryUtil::MakeKey(m_RegPath, kObjectList);
        table_view = gui_reg.GetReadView(reg_path);
        ncbi::LoadTableSettings(*m_ObjectSel, table_view);
    }
}

void CAsnExportPage::OnButton3Click( wxCommandEvent& WXUNUSED(event) )
{
    wxTextCtrl* textCtrl = (wxTextCtrl*)FindWindow(ID_TEXTCTRL6);

    wxString path = textCtrl->GetValue();

    wxFileDialog dlg(this, wxT("Select a file"), wxT(""), wxT(""),
        CFileExtensions::GetDialogFilter(CFileExtensions::kASN) + wxT("|") +
        CFileExtensions::GetDialogFilter(CFileExtensions::kAllFiles),
        wxFD_SAVE|wxFD_OVERWRITE_PROMPT);

    dlg.SetPath(path);

    if (dlg.ShowModal() != wxID_OK)
        return;

    path = dlg.GetPath();
    textCtrl->SetValue(path);
}

END_NCBI_SCOPE
