/*  $Id: async_obj_convert.cpp 26637 2012-10-17 18:02:26Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 */

#include <ncbi_pch.hpp>

#include <serial/typeinfo.hpp>
#include <gui/core/async_obj_convert.hpp>
#include <gui/objutils/obj_convert.hpp>

BEGIN_NCBI_SCOPE

void CAsyncObjConvert::Execute()
{
    m_Results.clear();

    NON_CONST_ITERATE(TConstScopedObjects, it, m_InputObjects) {
        CObjectConverter::TRelationVector relations;
        CObjectConverter::FindRelations(*it->scope, *it->object,
                m_TypeInfo->GetName(), relations);

        ITERATE(CObjectConverter::TRelationVector, itr, relations) {
            if (relations.size() > 1 && itr == relations.begin()) {
                continue;
            }

            if (IsCanceled())
                return;

            const CRelation& rel = **itr;
            string relName = rel.GetProperty(m_TypeInfo->GetName());
            TConstScopedObjects& vec = m_Results[relName];

            CRelation::TObjects related;
            rel.GetRelated(*it->scope, *it->object, related, CRelation::fConvert_All, this);

            ITERATE(CRelation::TObjects, it2, related) {
                const CSerialObject* serial = dynamic_cast<const CSerialObject*>(it2->GetObjectPtr());
                if (serial && serial->GetThisTypeInfo() == m_TypeInfo)
                    vec.push_back(SConstScopedObject(it2->GetObject(), *it->scope));
                else
                    _ASSERT(false);
            }
        }
    }
}

void CAsyncObjConvert2::Execute()
{
    m_Results.clear();

    NON_CONST_ITERATE(TConstScopedObjects, it, m_InputObjects) {
        CObjectConverter::TRelationVector relations;
        CObjectConverter::FindRelations(*it->scope, *it->object,
                m_TypeInfo->GetName(), relations);

        if (IsCanceled())
            return;

        if (relations.empty())
            continue;

        CRelation::TObjects related;
        relations.front()->GetRelated(*it->scope, *it->object, related, CRelation::fConvert_NoExpensive, this);

        ITERATE(CRelation::TObjects, it2, related) {
            const CSerialObject* serial = dynamic_cast<const CSerialObject*>(it2->GetObjectPtr());
            if (serial && serial->GetThisTypeInfo() == m_TypeInfo)
                m_Results.push_back(SConstScopedObject(it2->GetObject(), *it->scope));
            else
                _ASSERT(false);
        }
    }
}

END_NCBI_SCOPE
