/*  $Id: broadcast_settings_dlg.cpp 25493 2012-03-27 18:28:58Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/core/broadcast_settings_dlg.hpp>
#include <gui/objutils/obj_event.hpp>

#include <wx/sizer.h>
#include <wx/statbox.h>
#include <wx/choice.h>
#include <wx/button.h>
#include <wx/radiobox.h>
#include <wx/checkbox.h>
#include <wx/icon.h>

////@begin includes
////@end includes

BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CBroadcastDialog, wxDialog )

BEGIN_EVENT_TABLE( CBroadcastDialog, wxDialog )

////@begin CBroadcastDialog event table entries
    EVT_BUTTON( wxID_APPLY, CBroadcastDialog::OnApplyClick )

////@end CBroadcastDialog event table entries

END_EVENT_TABLE()

CBroadcastDialog::CBroadcastDialog()
{
    Init();
}

CBroadcastDialog::CBroadcastDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

bool CBroadcastDialog::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBroadcastDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CBroadcastDialog creation
    return true;
}

CBroadcastDialog::~CBroadcastDialog()
{
////@begin CBroadcastDialog destruction
////@end CBroadcastDialog destruction
}

void CBroadcastDialog::Init()
{
////@begin CBroadcastDialog member initialisation
////@end CBroadcastDialog member initialisation
}

void CBroadcastDialog::CreateControls()
{
////@begin CBroadcastDialog content construction
    // Generated by DialogBlocks, 16/04/2009 01:56:16 (unregistered)

    CBroadcastDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 1, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 7);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer5Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Policy"));
    wxStaticBoxSizer* itemStaticBoxSizer5 = new wxStaticBoxSizer(itemStaticBoxSizer5Static, wxHORIZONTAL);
    itemBoxSizer4->Add(itemStaticBoxSizer5, 0, wxGROW|wxALL, 0);

    wxArrayString itemChoice6Strings;
    itemChoice6Strings.Add(_("Default"));
    itemChoice6Strings.Add(_("Strict"));
    itemChoice6Strings.Add(_("Relaxed"));
    wxChoice* itemChoice6 = new wxChoice( itemDialog1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, itemChoice6Strings, 0 );
    itemChoice6->SetStringSelection(_("Default"));
    itemStaticBoxSizer5->Add(itemChoice6, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemDialog1, wxID_APPLY, _("&Apply"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer5->Add(itemButton7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemRadioBox8Strings;
    itemRadioBox8Strings.Add(_("Exact Match by all IDs"));
    itemRadioBox8Strings.Add(_("Match at least one ID"));
    itemRadioBox8Strings.Add(_("Match at least one ID, ignore version"));
    wxRadioBox* itemRadioBox8 = new wxRadioBox( itemDialog1, ID_RADIOBOX1, _("Sequence Matching Selection Options"), wxDefaultPosition, wxDefaultSize, itemRadioBox8Strings, 1, wxRA_SPECIFY_COLS );
    itemRadioBox8->SetSelection(0);
    itemBoxSizer4->Add(itemRadioBox8, 0, wxGROW|wxALL, 0);

    wxStaticBox* itemStaticBoxSizer9Static = new wxStaticBox(itemDialog1, wxID_ANY, wxT(""));
    wxStaticBoxSizer* itemStaticBoxSizer9 = new wxStaticBoxSizer(itemStaticBoxSizer9Static, wxVERTICAL);
    itemBoxSizer4->Add(itemStaticBoxSizer9, 0, wxGROW|wxALL, 0);

    wxCheckBox* itemCheckBox10 = new wxCheckBox( itemDialog1, ID_CHECKBOX2, _("Use locations to identify alignment rows"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox10->SetValue(false);
    itemStaticBoxSizer9->Add(itemCheckBox10, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox11 = new wxCheckBox( itemDialog1, ID_CHECKBOX1, _("Match Features by Products"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox11->SetValue(false);
    itemStaticBoxSizer9->Add(itemCheckBox11, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox12 = new wxCheckBox( itemDialog1, ID_CHECKBOX4, _("Broadcast between Projects"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox12->SetValue(false);
    itemStaticBoxSizer9->Add(itemCheckBox12, 0, wxALIGN_LEFT|wxALL, 5);

    wxArrayString itemRadioBox13Strings;
    itemRadioBox13Strings.Add(_("&Individual ranges"));
    itemRadioBox13Strings.Add(_("&Total range"));
    wxRadioBox* itemRadioBox13 = new wxRadioBox( itemDialog1, ID_RADIOBOX3, _("Range options"), wxDefaultPosition, wxDefaultSize, itemRadioBox13Strings, 1, wxRA_SPECIFY_COLS );
    itemRadioBox13->SetSelection(0);
    itemBoxSizer4->Add(itemRadioBox13, 0, wxGROW|wxALL, 5);

    wxCheckBox* itemCheckBox14 = new wxCheckBox( itemDialog1, ID_CHECKBOX3, _("Auto broadcast selection changes"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox14->SetValue(false);
    itemBoxSizer4->Add(itemCheckBox14, 0, wxALIGN_LEFT|wxALL, 5);

    itemBoxSizer4->Add(0, 10, 0, wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer3->Add(itemBoxSizer16, 0, wxGROW|wxALL, 5);

    wxArrayString itemRadioBox17Strings;
    itemRadioBox17Strings.Add(_("Track range changes"));
    itemRadioBox17Strings.Add(_("Track and Center range changes"));
    itemRadioBox17Strings.Add(_("Track and Expand range changes"));
    itemRadioBox17Strings.Add(_("Track and Contract range changes"));
    itemRadioBox17Strings.Add(_("Lock and Follow ranges"));
    wxRadioBox* itemRadioBox17 = new wxRadioBox( itemDialog1, ID_RADIOBOX2, _("Visible Range Synchronization Options"), wxDefaultPosition, wxDefaultSize, itemRadioBox17Strings, 1, wxRA_SPECIFY_COLS );
    itemRadioBox17->SetSelection(0);
    itemBoxSizer16->Add(itemRadioBox17, 0, wxALIGN_CENTER_HORIZONTAL, 5);

    wxCheckBox* itemCheckBox18 = new wxCheckBox( itemDialog1, ID_CHECKBOX5, _("Auto broadcast range changes"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox18->SetValue(false);
    itemBoxSizer16->Add(itemCheckBox18, 0, wxALIGN_LEFT|wxALL, 5);

    itemBoxSizer16->Add(0, 0, 1, wxALIGN_CENTER_HORIZONTAL, 0);

    wxStaticLine* itemStaticLine20 = new wxStaticLine( itemDialog1, ID_STATICLINE2, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemBoxSizer2->Add(itemStaticLine20, 0, wxGROW|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer21 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer21, 0, wxALIGN_RIGHT|wxALL, 5);
    wxButton* itemButton22 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemButton22->SetDefault();
    itemStdDialogButtonSizer21->AddButton(itemButton22);

    wxButton* itemButton23 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer21->AddButton(itemButton23);

    itemStdDialogButtonSizer21->Realize();

    // Set validators
    itemRadioBox8->SetValidator( wxGenericValidator(& m_MatchRadios) );
    itemCheckBox10->SetValidator( wxGenericValidator(& m_MatchAlnLocs) );
    itemCheckBox11->SetValidator( wxGenericValidator(& m_MatchByProduct) );
    itemCheckBox12->SetValidator( wxGenericValidator(& m_BroadcastBetweenProjects) );
    itemCheckBox14->SetValidator( wxGenericValidator(& m_AutoSelBroadcast) );
    itemRadioBox17->SetValidator( wxGenericValidator(& m_VRBPolicy) );
    itemCheckBox18->SetValidator( wxGenericValidator(& m_AutoVRBroadcast) );
////@end CBroadcastDialog content construction
}

void CBroadcastDialog::SetData( const SBroadcastParams& params )
{
    m_MatchRadios = 1;
    if (params.m_BioseqMatching == CSelectionEvent::eAllIds)
        m_MatchRadios = 0;
    else if(params.m_SeqIDMatching == CSelectionEvent::eAccOnly)
        m_MatchRadios = 2;

    m_MatchByProduct = params.m_MatchByProduct;
    m_MatchAlnLocs = params.m_MatchAlnLocs;
    m_BroadcastBetweenProjects = params.m_BroadcastBetweenProjects;
    m_AutoSelBroadcast = params.m_AutoSelBroadcast;
    m_AutoVRBroadcast = params.m_AutoVRBroadcast;
    m_VRBPolicy = params.m_VRBPolicy;
}

SBroadcastParams CBroadcastDialog::GetData() const
{
    SBroadcastParams params;

    params.m_BioseqMatching = (m_MatchRadios == 0) ?
            CSelectionEvent::eAllIds : CSelectionEvent::eAtLeastOne;
    params.m_SeqIDMatching = (m_MatchRadios == 2) ?
            CSelectionEvent::eAccOnly : CSelectionEvent::eExact;

    params.m_MatchByProduct = m_MatchByProduct;
    params.m_MatchAlnLocs = m_MatchAlnLocs;
    params.m_BroadcastBetweenProjects = m_BroadcastBetweenProjects;
    params.m_AutoSelBroadcast = m_AutoSelBroadcast;
    params.m_AutoVRBroadcast = m_AutoVRBroadcast;
    params.m_VRBPolicy = m_VRBPolicy;

    return params;
}

void CBroadcastDialog::OnApplyClick( wxCommandEvent& WXUNUSED(event) )
{
    wxChoice* policyCtrl = (wxChoice*)FindWindow(ID_CHOICE1);
    wxRadioBox* selMatchCtrl = (wxRadioBox*)FindWindow(ID_RADIOBOX1);
    wxCheckBox* matchByProduct = (wxCheckBox*)FindWindow(ID_CHECKBOX1);
    wxCheckBox* matchAlnLocs = (wxCheckBox*)FindWindow(ID_CHECKBOX2);

    switch(policyCtrl->GetSelection())
    {
    case 1:
        selMatchCtrl->SetSelection(0);
        matchAlnLocs->SetValue(false);
        matchByProduct->SetValue(true);
        break;
    case 2:
        selMatchCtrl->SetSelection(2);
        matchAlnLocs->SetValue(true);
        matchByProduct->SetValue(false);
        break;
    default:
        selMatchCtrl->SetSelection(1);
        matchAlnLocs->SetValue(true);
        matchByProduct->SetValue(true);
        break;
    }
}

bool CBroadcastDialog::ShowToolTips()
{
    return true;
}
wxBitmap CBroadcastDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBroadcastDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBroadcastDialog bitmap retrieval
}
wxIcon CBroadcastDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBroadcastDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBroadcastDialog icon retrieval
}

END_NCBI_SCOPE
