/*  $Id: data_store.cpp 14565 2007-05-18 12:32:01Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *    CDataStore -- general clearing-house storage for abstract data
 */

#include <ncbi_pch.hpp>
#include <corelib/ncbiobj.hpp>
#include <corelib/ncbimtx.hpp>

#include <gui/core/data_store.hpp>


BEGIN_NCBI_SCOPE


DEFINE_STATIC_MUTEX(s_DataStoreMutex);

CDataStore::TObjMap CDataStore::sm_Map;


void CDataStore::Clear(void)
{
    sm_Map.clear();
}

CObject& CDataStore::GetObject(const string& name)
{
    CMutexGuard GUARD(s_DataStoreMutex);
    TObjMap::iterator iter = sm_Map.find(name);
    if (iter == sm_Map.end()) {
        // throw
    }
    return *(iter->second);
}


bool CDataStore::PutObject(const string& name, CObject& obj)
{
    CMutexGuard GUARD(s_DataStoreMutex);

    TObjMap::value_type val(name, CRef<CObject>(&obj));
    return sm_Map.insert(val).second;
}


void CDataStore::ReleaseObject(const string& name)
{
    CMutexGuard GUARD(s_DataStoreMutex);
    sm_Map.erase(name);
}


bool CDataStore::HasObject(const string& name)
{
    CMutexGuard GUARD(s_DataStoreMutex);
    TObjMap::const_iterator iter = sm_Map.find(name);
    return (iter != sm_Map.end());
}


bool CDataStore::HasObject(const CObject& obj)
{
    CMutexGuard GUARD(s_DataStoreMutex);
    ITERATE(TObjMap, iter, sm_Map) {
        if (iter->second == &obj) {
            return true;
        }
    }

    return false;
}


END_NCBI_SCOPE
