/*  $Id: dh_manager.cpp 17733 2008-09-10 20:26:39Z yazhuk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/core/data_store.hpp>
#include <gui/core/dh_manager.hpp>

#include <gui/objects/GBWorkspace.hpp>
#include <gui/objects/ProjectItem.hpp>
#include <gui/objects/FolderInfo.hpp>
#include <gui/objects/GBProject.hpp>
#include <gui/objects/ProjectFolder.hpp>
#include <gui/objects/WorkspaceFolder.hpp>
#include <gui/objects/DataHandle.hpp>

#include <objects/general/Date.hpp>
#include <objects/submit/Seq_submit.hpp>

#include <objmgr/object_manager.hpp>

#include <algorithm>


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

DEFINE_STATIC_MUTEX(s_DHMgrMutex);


CDHManager::TDHMap   CDHManager::sm_DataHandles;


void CDHManager::Init()
{
}

void CDHManager::ShutDown()
{
    CMutexGuard LOCK(s_DHMgrMutex);
    sm_DataHandles.clear();
}


CDHManager::TDHMap& CDHManager::GetDataHandles()
{
    return sm_DataHandles;
}


CDataHandle * CDHManager::GetDataHandle(const CObject& obj)
{
    CMutexGuard LOCK(s_DHMgrMutex);
    TDHMap::iterator iter = sm_DataHandles.find(const_cast<CObject*>(&obj));
    if (iter!=sm_DataHandles.end()){
        return iter->second;
    }
    return NULL;
}

CDataHandle* CDHManager::CreateDataHandle(objects::CProjectItem& obj)
{
    CMutexGuard LOCK(s_DHMgrMutex);
    CRef<CDataHandle> dh(new CDataHandle(obj));
    sm_DataHandles[ const_cast<CSerialObject*>(obj.GetObject()) ] = dh;

    //_TRACE("CDHManager::CreateDataHandle(" << obj.GetObject() << ") succeeded");

    return dh.GetPointer();
}

bool CDHManager::ReleaseDataHandle(objects::CProjectItem& obj)
{
    CMutexGuard LOCK(s_DHMgrMutex);

    TDHMap::iterator iter = sm_DataHandles.find(const_cast<CSerialObject*>(obj.GetObject()));
    if (iter!=sm_DataHandles.end()){
        sm_DataHandles.erase(iter);
        //_TRACE("CDHManager::ReleaseDataHandle(" << obj.GetObject() << ") succeeded");
        return true;
    }
    //_TRACE("CDHManager::ReleaseDataHandle(" << obj.GetObject() << ") failed");
    return false;
}

void CDHManager::ReleaseAllDataHandles()
{
    CMutexGuard LOCK(s_DHMgrMutex);
    sm_DataHandles.clear();
}

END_NCBI_SCOPE
