/*  $Id: seq_utils.cpp 27815 2013-04-08 15:10:18Z rudnev $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
 * Author:  Dmitry Rudnev
 *
 * File Description:
 *   An utility class providing some useful sequence-related functionality
 //!! the only reason it is placed in gui/core is that it depends on assm_info.hpp here in gui/core
 //!! if assm_info.hpp will ever be able to move anywhere else, than this file should follow
 //!! (and probably merge into some existing file in that place)
*
*/

#include <ncbi_pch.hpp>
#include <gui/core/assm_info.hpp>
#include <gui/core/seq_utils.hpp>

#include <objects/genomecoll/genomic_collections_cli.hpp>
#include <objects/genomecoll/GC_Sequence.hpp>
#include <objects/genomecoll/GC_Replicon.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

string NSeqUtils::GetChrGI(int gi)
{
    CAssemblyInfo::TIds gc_ids;
    CAssemblyInfo::GetAssmIds_GIChr(gc_ids, gi);
    CGenomicCollectionsService gcs;
    CSeq_id_Handle idh(CSeq_id_Handle::GetHandle(gi));

    ITERATE(CAssemblyInfo::TIds, iGCId, gc_ids) {
        CRef<CGC_Assembly> assm(gcs.GetAssembly(NStr::StringToNumeric(*iGCId), CGCClient_GetAssemblyRequest::eLevel_replicon));

        CGC_Assembly::TSequenceList sequences;
        assm->Find(idh, sequences);

        ITERATE(CGC_Assembly::TSequenceList, iSequences, sequences) {
            CConstRef<CGC_Replicon> replicon((*iSequences)->GetReplicon());
            if(replicon->IsSetName()) {
                return replicon->GetName();
            }
        }
    }
    return "";
}






END_NCBI_SCOPE
