/* $Id: GBWorkspace.cpp 22688 2010-12-13 22:02:43Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author:  .......
 *
 * File Description:
 *   .......
 *
 * Remark:
 *   This code was originally generated by application DATATOOL
 *   using the following specifications:
 *   'gui_project.asn'.
 */

// standard includes
#include <ncbi_pch.hpp>

// generated includes
#include <gui/objects/GBWorkspace.hpp>
#include <gui/objects/WorkspaceFolder.hpp>
#include <gui/objects/GBProject.hpp>
#include <gui/objects/GBProjectHandle.hpp>
#include <serial/iterator.hpp>
#include <gui/objutils/label.hpp>

#include <algorithm>

// generated classes

BEGIN_NCBI_SCOPE

BEGIN_objects_SCOPE // namespace ncbi::objects::


class CGBWorkspaceLabelHandler : public ILabelHandler
{
public:
    void GetLabel(const CObject& obj, string* str,
                  CLabel::ELabelType type,
                  objects::CScope* scope) const
    {
        const CGBWorkspace* wks = dynamic_cast<const CGBWorkspace*>(&obj);
        if (wks  &&  wks->IsSetWorkspace()) {
            CTypeConstIterator<CGBProject> iter(wks->GetWorkspace());
            string s;
            for ( ;  iter;  ++iter) {
                if ( !s.empty() ) {
                    s += "; ";
                }
                CLabel::GetLabel(*iter, &s, type, scope);
            }
            *str += s;
        }
    }
};


///////////////////////////////////////////////////////////////////////////////
/// CGBWorkspace

CAtomicCounter CGBWorkspace::m_IdCounter;

CGBWorkspace::CGBWorkspace(void)
{
    m_Id = m_IdCounter.Add(1) - 1;

    if ( !CLabel::HasHandler(*CGBWorkspace::GetTypeInfo()) ) {
        CLabel::RegisterLabelHandler(*CGBWorkspace::GetTypeInfo(),
                                     *new CGBWorkspaceLabelHandler());
    }
}


// destructor
CGBWorkspace::~CGBWorkspace(void)
{
}


bool CGBWorkspace::IsDirty() const
{
    return m_Dirty;
}


void CGBWorkspace::SetDirty(bool dirty)
{
    m_Dirty = dirty;

    if(dirty) {
        CTime time;
        time.SetCurrent();
        SetDescr().SetModified_date().SetToTime(time);
    }
}


void CGBWorkspace::LinkFolders()
{
    x_LinkChildFolders(SetWorkspace());
}


CGBProjectHandle* CGBWorkspace::GetProjectFromScope(CScope& scope)
{
    TScopeIndex::iterator it = m_ScopeIdx.find(CRef<CScope>(&scope));
    return (it == m_ScopeIdx.end()) ? NULL : it->second.GetPointer();
}


const CGBProjectHandle* CGBWorkspace::GetProjectFromScope(const CScope& scope)
{
    CRef<CScope> ref(const_cast<CScope*>(&scope));
    TScopeIndex::const_iterator it = m_ScopeIdx.find(ref);
    return (it == m_ScopeIdx.end()) ? NULL : it->second.GetPointer();
}


string CGBWorkspace::MakeUniqueProjectTitle(const string& label) const
{
    CUniqueLabelGenerator generator(label);
    const CWorkspaceFolder& root = GetWorkspace();
    for ( CTypeConstIterator<CGBProjectHandle> it(root);  it;  ++it) {
        const string& s = it->GetTitle();
        generator.AddExistingLabel(s);
    }
    string s = generator.MakeUniqueLabel();
    return s;
}


void CGBWorkspace::AddScopeToIndex(CGBProjectHandle& project)
{
    CScope* scope = project.GetScope();

    _ASSERT(scope);

    m_ScopeIdx[CRef<CScope>(scope)] = CRef<CGBProjectHandle>(&project);
}


void CGBWorkspace::RemoveScopeFromIndex(CGBProjectHandle& project)
{
    CRef<CScope> scope(project.GetScope());

    _ASSERT(scope);

    TScopeIndex::iterator it = m_ScopeIdx.find(scope);
    if (it != m_ScopeIdx.end()) {
        m_ScopeIdx.erase(it);
    }
}


void CGBWorkspace::x_LinkChildFolders(CWorkspaceFolder& folder)
{
    if(folder.CanGetFolders())  {
        CWorkspaceFolder::TFolders& sub_folders = folder.SetFolders();
        NON_CONST_ITERATE(CWorkspaceFolder::TFolders, it, sub_folders)  {
            CWorkspaceFolder& child = **it;
            child.SetParentFolder(&folder);
            x_LinkChildFolders(child);
        }
    }
}


END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE

/* Original file checksum: lines: 65, chars: 1894, CRC32: 164c029b */
