/* $Id: ProjectItem.cpp 27086 2012-12-17 19:43:41Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author:  .......
 *
 * File Description:
 *   .......
 *
 * Remark:
 *   This code was originally generated by application DATATOOL
 *   using the following specifications:
 *   'gui_project.asn'.
 */

// standard includes
#include <ncbi_pch.hpp>

#include <gui/objutils/user_type.hpp>

// generated includes
#include <gui/objects/ProjectItem.hpp>
#include <gui/objutils/obj_convert.hpp>

#include <objects/entrezgene/Entrezgene.hpp>
#include <objects/entrezgene/Entrezgene_Set.hpp>
#include <objects/seqloc/Seq_id.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <objects/seq/Annot_descr.hpp>
#include <objects/seqset/Seq_entry.hpp>
#include <objects/seqalign/Seq_align.hpp>
#include <objects/seqalign/Seq_align_set.hpp>
#include <objects/submit/Seq_submit.hpp>
#include <objects/genomecoll/GC_Assembly.hpp>
#include <objects/genomecoll/GC_AssemblyUnit.hpp>
#include <objects/biotree/BioTreeContainer.hpp>

#include <objmgr/util/sequence.hpp>

#include <serial/objostr.hpp>
#include <serial/objistr.hpp>
#include <gui/objutils/label.hpp>

// generated classes

BEGIN_NCBI_SCOPE

BEGIN_objects_SCOPE // namespace ncbi::objects::

////////////////////////////////////////////////////////////
// label handlers for genome workbench project types
// these will be regsitered at application initialization
class CProjectItemLabelHandler : public ILabelHandler
{
public:
    void GetLabel(const CObject& obj, string* str,
                  CLabel::ELabelType type,
                  objects::CScope* scope) const
    {
        if(str == NULL) {
            return;
        }
        const CProjectItem* item = dynamic_cast<const CProjectItem*>(&obj);
        if (item) {
            switch(type)    {
            case CLabel::eContent:
                *str = item->IsSetLabel() ? item->GetLabel() : "";
                break;
            case CLabel::eUserType:
                *str += CGUIUserType::sm_Tp_ProjectItem;
                break;

            case CLabel::eUserSubtype:
                break;

            case CLabel::eType:
                *str = "Project Item";
                break;

            case CLabel::eUserTypeAndContent:
            case CLabel::eDescriptionBrief:
            case CLabel::eDescription:
                *str += CGUIUserType::sm_Tp_ProjectItem;
                *str = " : " + item->GetLabel();
                break;

            default:
                break;
            }
        }
        /* the old way of getting the Label, now it shall be set explicitly
        const CObject* obj = item->GetObject();
        if (obj) {
            CLabel::GetLabel(*obj, str, type, scope);
        }*/
    }
};


// constructor
CProjectItem::CProjectItem(void)
:   m_ParentFolder(NULL)
{
    if ( !CLabel::HasHandler(*CProjectItem::GetTypeInfo()) ) {
        CLabel::RegisterLabelHandler(*CProjectItem::GetTypeInfo(),
                                     *new CProjectItemLabelHandler());
    }
}


// destructor
CProjectItem::~CProjectItem(void)
{
}


void CProjectItem::SetCreateDate(const CTime& time)
{
    CDate date(time);
    SetCreateDate(date);
}


void CProjectItem::SetCreateDate(const CDate& date)
{
    CRef<CAnnotdesc> desc;
    TDescr::iterator iter = SetDescr().begin();
    for ( ;  iter != SetDescr().end();  ) {
        if ( (*iter)->IsCreate_date() ) {
            if (desc) {
                iter = SetDescr().erase(iter);
            } else {
                desc = *iter++;
            }
        } else {
            ++iter;
        }
    }

    if ( !desc ) {
        desc.Reset(new CAnnotdesc());
        SetDescr().push_back(desc);
    }
    desc->SetCreate_date().Assign(date);
}


void CProjectItem::SetModifiedDate(const CTime& time)
{
    CDate date(time);
    SetModifiedDate(date);
}


void CProjectItem::SetModifiedDate(const CDate& date)
{
    CRef<CAnnotdesc> desc;
    TDescr::iterator iter = SetDescr().begin();
    for ( ;  iter != SetDescr().end();  ) {
        if ( (*iter)->IsUpdate_date() ) {
            if (desc) {
                iter = SetDescr().erase(iter);
            } else {
                desc = *iter++;
            }
        } else {
            ++iter;
        }
    }

    if ( !desc ) {
        desc.Reset(new CAnnotdesc());
        SetDescr().push_back(desc);
    }
    desc->SetUpdate_date().Assign(date);
}


void CProjectItem::SetObject(CSerialObject& object)
{
    if (object.GetThisTypeInfo() == CSeq_id::GetTypeInfo()) {
        SetItem().SetId(dynamic_cast<CSeq_id&>(object));
    } else if (object.GetThisTypeInfo() == CSeq_entry::GetTypeInfo()) {
        SetItem().SetEntry(dynamic_cast<CSeq_entry&>(object));
    } else if (object.GetThisTypeInfo() == CSeq_annot::GetTypeInfo()) {
        SetItem().SetAnnot(dynamic_cast<CSeq_annot&>(object));
    } else if (object.GetThisTypeInfo() == CSeq_submit::GetTypeInfo()) {
        SetItem().SetSubmit(dynamic_cast<CSeq_submit&>(object));
    } else if (object.GetThisTypeInfo() == CSeq_align::GetTypeInfo()) {
        SetItem().SetSeq_align(dynamic_cast<CSeq_align&>(object));
    } else if (object.GetThisTypeInfo() == CSeq_align_set::GetTypeInfo()) {
        SetItem().SetSeq_align_set(dynamic_cast<CSeq_align_set&>(object));
    } else {
        SetItem().SetOther().Set(object);
    }
}


string CProjectItem::GetLabelByData(CScope* scope) const
{
    if (IsSetItem()) {
        const CSerialObject* obj = GetObject();
        if(obj) {
            string str;
            CLabel::GetLabel(*obj, &str, CLabel::eDefault, scope);
             return str;
        }
    }
    return "No data";
}


const CSerialObject* CProjectItem::GetObject() const
{
    if (IsSetItem()) {
        switch (GetItem().Which()) {
        case TItem::e_Id:
            return &GetItem().GetId();

        case TItem::e_Entry:
            return &GetItem().GetEntry();

        case TItem::e_Annot:
            return &GetItem().GetAnnot();

        case TItem::e_Submit:
            return &GetItem().GetSubmit();

        case TItem::e_Seq_align:
            return &GetItem().GetSeq_align();

        case TItem::e_Seq_align_set:
            return &GetItem().GetSeq_align_set();

        //TODO ePlugin, ePmid and eTaxid are not supported
        case TItem::e_Other:
            return GetItem().GetOther().GetObject();

        default:
            break;
        }
    } else {
        ERR_POST(Warning << "CProjectItem::GetObject(): attempt to access unset project-item");
    }

    return NULL;
}


bool CProjectItem::IsEnabled() const
{
    if (CanGetDisabled()){
        return ! GetDisabled();
    }
    return true;
}


void CProjectItem::SetParentFolder(CProjectFolder* parent)
{
    m_ParentFolder = parent;
}


CProjectFolder* CProjectItem::GetParentFolder()
{
    return m_ParentFolder;
}

const CProjectFolder* CProjectItem::GetParentFolder() const
{
    return m_ParentFolder;
}

static CSeq_entry_Handle s_AttachEntryToScope( CScope& scope, CSeq_entry& entry )
{
    CSeq_entry_Handle seh;
    try {
        seh = scope.AddTopLevelSeqEntry( entry, CScope::kPriority_Default, CScope::eExist_Get );

    } catch( CException& ex ){
//         try {
//             seh = scope.GetSeq_entryHandle(entry);
//             LOG_POST(Warning << "attempted to attach already existing seq-entry");
//         }
//        catch( CException& ){
            LOG_POST( Error << "Failed to attach seq-entry: " << ex.GetMsg() );
//        }
    }

    return seh;
}

static void s_DetachEntryFromScope(CScope& scope, CSeq_entry_Handle& handle)
{
    if( !handle ){
        return;
    }

    try {
        scope.RemoveTopLevelSeqEntry( const_cast<CTSE_Handle&>(handle.GetTSE_Handle()) );

    } catch( CException& ex ){
        LOG_POST(
            Error << "s_DetachEntryFromScope(): error removing seq-entry: " << ex.GetMsg()
        );
    }
}


static CSeq_annot_Handle s_AttachAnnotToScope( CScope& scope, CSeq_annot& annot )
{
    switch( annot.GetData().Which() ){
    case CSeq_annot::TData::e_Seq_table:
    case CSeq_annot::TData::e_Ftable:
    case CSeq_annot::TData::e_Align:
    case CSeq_annot::TData::e_Graph:
        try{ 
            return scope.AddSeq_annot( annot, CScope::kPriority_Default, CScope::eExist_Get );

        } catch( CException& ex ){
            LOG_POST( Error << "Failed to attach seq-annot: " << ex.GetMsg() );
        }

    default:
        /// bags of IDs or locs ignored
        break;
    }

    return CSeq_annot_Handle();
}

static void s_DetachAnnotFromScope(CScope& scope, CSeq_annot_Handle& annot)
{
    if( annot ){
        try {
            annot.GetEditHandle().Remove();

        } catch( CException& ex ){
            LOG_POST(Error 
                << "s_DetachAnnotFromScope(): error removing seq-annot: "
                << ex.GetMsg()
            );
        }
    }
}


void CProjectItem::Attach(CScope& scope)
{
    switch (GetItem().Which()) {
    case TItem::e_Entry:
        {{
            /// CSeq_entry already - just add to scope
            CSeq_entry_Handle seh =
                s_AttachEntryToScope(scope, SetItem().SetEntry());
            if (seh) {
                m_Seq_entry_Handles.push_back(seh);
            }
        }}
        break;

    case TItem::e_Annot:
        {{
            /// Annotation - call CScope::AddSeq_annot()
            CSeq_annot_Handle h =
                s_AttachAnnotToScope(scope, SetItem().SetAnnot());
            if (h) {
                m_Seq_annot_Handles.push_back(h);
            }
        }}
        break;

    case TItem::e_Submit:
        /// Seq-submit may be either entries or annotations
        /// these are handled as above
        switch (GetItem().GetSubmit().GetData().Which()) {
        case CSeq_submit::TData::e_Entrys:
            NON_CONST_ITERATE (CSeq_submit::TData::TEntrys, iter, SetItem().SetSubmit().SetData().SetEntrys()) {
                CSeq_entry_Handle h =
                    s_AttachEntryToScope(scope, **iter);
                if (h) {
                    m_Seq_entry_Handles.push_back(h);
                }
            }
            break;

        case CSeq_submit::TData::e_Annots:
            NON_CONST_ITERATE (CSeq_submit::TData::TAnnots, iter, SetItem().SetSubmit().SetData().SetAnnots()) {
                CSeq_annot_Handle h =
                    s_AttachAnnotToScope(scope, **iter);
                if (h) {
                    m_Seq_annot_Handles.push_back(h);
                }
            }
            break;

        default:
            break;
        }
        break;

    case TItem::e_Seq_align:
    case TItem::e_Seq_align_set:
        /// alignments have a simple annotation representation
        /// and can be converted inexpensively
        {{
            CObjectConverter::TObjList objs;
            CObjectConverter::Convert(scope, *GetObject(),
                                      CSeq_annot::GetTypeInfo(), objs);
            ITERATE (CObjectConverter::TObjList, obj_iter, objs) {
                CSeq_annot& annot =
                    const_cast<CSeq_annot&>
                    (dynamic_cast<const CSeq_annot&>(obj_iter->GetObject()));
                CSeq_annot_Handle h =
                    s_AttachAnnotToScope(scope, annot);
                if (h) {
                    m_Seq_annot_Handles.push_back(h);
                }
            }
        }}
        break;

    case TItem::e_Other:
        /// other items should be converted to top-level
        /// seq-entries and added to the scope
        {{
            const CObject* obj = GetObject();
            {{
                 /// exclude certain types of object
                 const CSerialObject* cso = dynamic_cast<const CSerialObject*>(obj);
                 if (cso) {
                     const CTypeInfo* ti = cso->GetThisTypeInfo();
                     if (ti == CSeq_loc::GetTypeInfo()) {
                         /// nothing to do here - seq-locs are independent
                         /// entities
                         break;
                     }
                     if (ti == CEntrezgene::GetTypeInfo()) {
                         /// nothing to do here - Entrezgene records
                         /// are independent entities
                         break;
                     }
                     if (ti == CEntrezgene_Set::GetTypeInfo()) {
                         /// nothing to do here - Entrezgene records
                         /// are independent entities
                         break;
                     }
                     if (ti == CSeq_feat::GetTypeInfo()) {
                         CSeq_feat& feat = const_cast<CSeq_feat&>
                             (dynamic_cast<const CSeq_feat&>(*obj));

                         CRef<CSeq_annot> annot(new CSeq_annot());
                         annot->SetData().SetFtable().push_back(CRef<CSeq_feat>(&feat));;
                         CSeq_annot_Handle h = s_AttachAnnotToScope(scope, annot.GetObject());
                         if (h) {
                             m_Seq_annot_Handles.push_back(h);
                         }
                         break;
                     }
                     if (ti == CGC_Assembly::GetTypeInfo()) {
                         // Attach all alignment annotations to scope
                         CGC_Assembly* assm = const_cast<CGC_Assembly*>(
                             dynamic_cast<const CGC_Assembly*>(cso));
                         CTypeIterator<CSeq_annot> annot_iter(*assm);
                         for (; annot_iter; ++annot_iter) {
                             if (annot_iter->IsAlign()) {
                                 // We only collect seq-aligns with type not set to 'global', and
                                 // alignments will be sorted by assembly unit, JIRA GB-2344
                                 typedef map<string, CSeq_annot::TData::TAlign> TSortedAligns;
                                 TSortedAligns sorted_aligns;
                                 NON_CONST_ITERATE(CSeq_annot::TData::TAlign, aln_iter, annot_iter->SetData().SetAlign()) {
                                     if ((*aln_iter)->GetType() != CSeq_align::eType_global) {
                                         CSeq_id_Handle alt_locus_id =
                                             sequence::GetId((*aln_iter)->GetSeq_id(0), scope, sequence::eGetId_Canonical);
                                         CConstRef< CGC_Sequence > gc_seq = assm->Find (alt_locus_id) ;
                                         if (gc_seq) {
                                             string assm_unit = gc_seq->GetAssemblyUnit()->GetName();
                                             sorted_aligns[assm_unit].push_back(*aln_iter);
                                         }
                                     }
                                 }

                                 if ( !sorted_aligns.empty() ) {
                                     string annot_name = "";
                                     if (annot_iter->IsSetDesc()) {
                                         ITERATE (objects::CAnnot_descr::Tdata, descrIter, annot_iter->GetDesc().Get()) {
                                             if ((*descrIter)->IsTitle()) {
                                                 annot_name = (*descrIter)->GetTitle();
                                                 break;
                                             } else if ((*descrIter)->IsName()) {
                                                 annot_name = (*descrIter)->GetName();
                                             }
                                         }
                                     }
                                     if ( !annot_name.empty() ) {
                                         annot_name += " - ";
                                     }

                                     NON_CONST_ITERATE(TSortedAligns, sorted_annot_iter, sorted_aligns) {
                                         CRef<CSeq_annot> annot(new CSeq_annot());
                                         annot->SetNameDesc(annot_name + sorted_annot_iter->first);
                                         annot->SetData().SetAlign() = sorted_annot_iter->second;
                                         CSeq_annot_Handle h =
                                             s_AttachAnnotToScope(scope, *annot);
                                         if (h) {
                                             m_Seq_annot_Handles.push_back(h);
                                         }
                                     }
                                 }
                             }
                         }
                         break;
                     }

                     if (ti == CBioTreeContainer::GetTypeInfo()) {
                         // nothing to do here
                         break;
                     }

                     if (ti == CProjectItem::GetTypeInfo()) {
                        CProjectItem& prj_item = const_cast<CProjectItem&>
                            (dynamic_cast<const CProjectItem&>(*obj));
                        prj_item.Attach(scope);
                        break;
                     }
                 }
             }}

            if (obj) {
                CObjectConverter::TObjList objs;
                CObjectConverter::Convert(scope, *obj,
                                          CSeq_entry::GetTypeInfo(), objs);
                ITERATE (CObjectConverter::TObjList, obj_iter, objs) {
                    CSeq_entry& entry =
                        const_cast<CSeq_entry&>
                        (dynamic_cast<const CSeq_entry&>(obj_iter->GetObject()));
                    CSeq_entry_Handle h =
                        s_AttachEntryToScope(scope, entry);
                    if (h) {
                        m_Seq_entry_Handles.push_back(h);
                    }
                }
            }
        }}
        break;

    case TItem::e_Id:
        /// ID -> do nothing; IDs should be referrable directly
    case TItem::e_Plugin:
        /// don't do anything special to a plugin message
    default:
        /// default trap: pubmed ID, taxonomy ID
        break;
    }
}


void CProjectItem::Detach(CScope& scope)
{
    NON_CONST_ITERATE (list<CSeq_entry_Handle>, iter, m_Seq_entry_Handles) {
        s_DetachEntryFromScope(scope, *iter);
    }
    m_Seq_entry_Handles.clear();

    NON_CONST_ITERATE (list<CSeq_annot_Handle>, iter, m_Seq_annot_Handles) {
        s_DetachAnnotFromScope(scope, *iter);
    }
    m_Seq_annot_Handles.clear();
}


void CProjectItem::Refresh()
{
    NON_CONST_ITERATE (list<CSeq_entry_Handle>, iter, m_Seq_entry_Handles) {
        try {
            /// currently, we remove and re-add the handle
            CSeq_entry_EditHandle seh = iter->GetEditHandle();
            CConstRef<CSeq_entry> entry = seh.GetCompleteSeq_entry();
            CRef<CScope> scope(&seh.GetScope());
            seh.Remove();
            scope->AddTopLevelSeqEntry(const_cast<CSeq_entry&>(*entry));
        }
        catch (CException& e) {
            LOG_POST(Error << "CProjectItem::Refresh(): error refreshing entry: "
                << e.GetMsg());
        }
    }

    NON_CONST_ITERATE (list<CSeq_annot_Handle>, iter, m_Seq_annot_Handles) {
        try {
            /// currently, we remove and re-add the handle
            CSeq_annot_EditHandle sah = iter->GetEditHandle();
            CConstRef<CSeq_annot> annot = sah.GetCompleteSeq_annot();
            CRef<CScope> scope(&sah.GetScope());
            sah.Remove();
            scope->AddSeq_annot(const_cast<CSeq_annot&>(*annot));
        }
        catch (CException& e) {
            LOG_POST(Error << "CProjectItem::Refresh(): error refreshing annot: "
                << e.GetMsg());
        }
    }
}


END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE

/* Original file checksum: lines: 65, chars: 1894, CRC32: b2655f3b */
