/*  $Id: cmd_change_seq_feat.cpp 27471 2013-02-21 20:45:48Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>
#include <gui/objutils/cmd_change_seq_feat.hpp>
#include <objects/seq/Bioseq.hpp>
#include <objmgr/scope.hpp>
#include <objmgr/seq_annot_ci.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


void CCmdChangeSeq_feat::Execute()
{
    // This is necessary, to make sure that we are in "editing mode"
    const CSeq_annot_Handle& annot_handle = m_Orig_feat.GetAnnot();
    CSeq_entry_EditHandle eh = annot_handle.GetParentEntry().GetEditHandle();

    // Now actually edit the feature
    CConstRef<CSeq_feat> saveFeat = m_Orig_feat.GetOriginalSeq_feat();
    CSeq_feat_EditHandle feh(m_Orig_feat);
    CSeq_entry_Handle parent_entry = feh.GetAnnot().GetParentEntry();

    feh.Replace(*m_New_feat);
    if (feh.IsSetData() && feh.GetData().IsCdregion() && feh.IsSetProduct()) {
        bool on_nuc_prot_set = false;
        if (parent_entry.IsSet()
             && parent_entry.GetSet().IsSetClass()
             && parent_entry.GetSet().GetClass() == CBioseq_set::eClass_nuc_prot) {
            // already on nuc-prot set, leave it alone
        } else {
            CBioseq_set_Handle nuc_parent = parent_entry.GetParentBioseq_set();
            if (nuc_parent && nuc_parent.IsSetClass() && nuc_parent.GetClass() == CBioseq_set::eClass_nuc_prot) {
                CSeq_annot_Handle ftable;
                CSeq_entry_Handle parent_seh = nuc_parent.GetParentEntry();
                CSeq_annot_CI annot_ci(parent_seh, CSeq_annot_CI::eSearch_entry);
                for (; annot_ci; ++annot_ci) {
                    if ((*annot_ci).IsFtable()) {
                        ftable = *annot_ci;
                        break;
                    }
                }

                if (!ftable) {
                    CRef<CSeq_annot> new_annot(new CSeq_annot());
                    new_annot->SetData().SetFtable();
                    CSeq_entry_EditHandle eh = parent_seh.GetEditHandle();
                    ftable = eh.AttachAnnot(*new_annot);
                }

                CSeq_annot_EditHandle new_annot = ftable.GetEditHandle();
                m_Orig_feat = new_annot.TakeFeat(feh);
            }
        }
    }

    m_New_feat = saveFeat;
}

void CCmdChangeSeq_feat::Unexecute()
{
    Execute();
}

string CCmdChangeSeq_feat::GetLabel()
{
    return "Modify feature";
}

END_NCBI_SCOPE
