/*  $Id: entrez_cache.cpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/objutils/entrez_cache.hpp>
#include <objects/entrez2/Entrez2_docsum_list.hpp>
#include <objects/entrez2/Entrez2_docsum.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


CRef<CEntrezCache> CEntrezCache::m_CacheInstance;


CEntrezCache::CEntrezCache()
    : m_Cache(50000)
{
}

CEntrezCache::~CEntrezCache()
{
}



void CEntrezCache::Put(const string& db, CEntrez2_docsum_list& docsums)
{
    CFastMutexGuard LOCK(m_Mutex);
    SKey key;
    key.db = db;
    if (docsums.IsSetList()) {
        NON_CONST_ITERATE (CEntrez2_docsum_list::TList, iter, docsums.SetList()) {
            key.uid = (*iter)->GetUid();
            m_Cache.insert_noresize(TCache::value_type(key, *iter));
        }
    }
}


void CEntrezCache::Put(const string& db, CEntrez2_docsum& docsum)
{
    CFastMutexGuard LOCK(m_Mutex);
    SKey key;
    key.db = db;
    key.uid = docsum.GetUid();
    m_Cache.insert_noresize(TCache::value_type(key, CConstRef<CEntrez2_docsum>(&docsum)));
}


CConstRef<CEntrez2_docsum> CEntrezCache::Get(const string& db, int uid) const
{
    SKey key;
    key.db = db;
    key.uid = uid;
    TCache::const_iterator iter = m_Cache.find(key);
    if (iter != m_Cache.end()) {
        return iter->second;
    }

    return CConstRef<CEntrez2_docsum>();
}


CEntrezCache& CEntrezCache::GetInstance()
{
    if ( !m_CacheInstance ) {
        DEFINE_STATIC_FAST_MUTEX(s_Mutex);
        CFastMutexGuard LOCK(s_Mutex);
        if ( !m_CacheInstance ) {
            m_CacheInstance.Reset(new CEntrezCache);
        }
    }
    return *m_CacheInstance;
}


END_NCBI_SCOPE


/*
 * ===========================================================================
 * $Log$
 * ===========================================================================
 */
