/*  $Id: obj_clipboard.cpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/utils/clipboard.hpp>
#include <gui/objutils/obj_clipboard.hpp>
#include <gui/objutils/label.hpp>
#include <connect/ncbi_conn_stream.hpp>
#include <serial/objostr.hpp>
#include <corelib/ncbimtx.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


// the mode of text copy
CObjClipboard::ECopyMode CObjClipboard::sm_Mode = CObjClipboard::eCopy_Label;


void CObjClipboard::Add(CScope& scope, const CObject& obj)
{
    SConstScopedObject clip_object(obj, scope);
    //sm_Objects.push_back(clip_object);
    CClipboard::Instance().Set(clip_object);

    string str;
    x_GetText(scope, obj, &str);
    CClipboard::Instance().AddFlavor(str);
}


void CObjClipboard::Add(CScope& scope, const CObject& obj, const string& text)
{
    SConstScopedObject clip_object(obj, scope);
    CClipboard::Instance().Set(clip_object);
    CClipboard::Instance().AddFlavor(text);
}


void CObjClipboard::Add(const string& text)
{
    CClipboard::Instance().Set(text);
}


void CObjClipboard::Clear()
{
    CClipboard::Instance().Clear();
}


void CObjClipboard::x_GetText(CScope& scope, const CObject& object,
                              string* text)
{
    if ( !text ) {
        return;
    }

    switch (sm_Mode) {
    case eCopy_Label:
        CLabel::GetLabel(object, text, CLabel::eDefault, &scope);
        break;

    case eCopy_ASN:
    case eCopy_XML:
        {{
            const CSerialObject* obj =
                dynamic_cast<const CSerialObject*>(&object);
            if (obj) {
                CConn_MemoryStream ostr;
                {{
                    auto_ptr<CObjectOStream> os
                        (CObjectOStream::Open(eSerial_AsnText, ostr));
                    os->Write(obj, obj->GetThisTypeInfo());
                }}

                string str;
                str.resize(ostr.tellp() - CT_POS_TYPE(0));
                ostr.read(const_cast<char*>(str.data()), str.size());
                *text += str;
            }
        }}
        break;
    }
}


END_NCBI_SCOPE
