/*  $Id: table_data_sorter.cpp 23918 2011-06-21 19:38:01Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/objutils/table_data_sorter.hpp>

BEGIN_NCBI_SCOPE

namespace // anonymous
{
    class CStrAsc
    {
    public:
        CStrAsc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetStringValue(r1, m_Col) < m_TableData.GetStringValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };

    class CStrDesc
    {
    public:
        CStrDesc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetStringValue(r1, m_Col) > m_TableData.GetStringValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };

    class CIntAsc
    {
    public:
        CIntAsc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetIntValue(r1, m_Col) < m_TableData.GetIntValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };

    class CIntDesc
    {
    public:
        CIntDesc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetIntValue(r1, m_Col) > m_TableData.GetIntValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };

    class CRealAsc
    {
    public:
        CRealAsc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetRealValue(r1, m_Col) < m_TableData.GetRealValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };

    class CRealDesc
    {
    public:
        CRealDesc(ITableData& tableData, size_t col) : m_TableData(tableData), m_Col(col) {}
        bool operator()(size_t r1, size_t r2)
        {
            return m_TableData.GetRealValue(r1, m_Col) > m_TableData.GetRealValue(r2, m_Col);
        }

    private:
        ITableData& m_TableData;
        size_t m_Col;
    };
} // anonymous namespace

void CTableDataSorter::Sort(ITableData& tableData, size_t column,
                            size_t* begin, size_t* end, bool descending)
{
    if (column >= tableData.GetColsCount())
        return;

    ITableData::ColumnType type = tableData.GetColumnType(column);
    if (type == ITableData::kInt) {
        if (descending)
            sort(begin, end, CIntDesc(tableData, column));
        else
            sort(begin, end, CIntAsc(tableData, column));
    }
    else if (type == ITableData::kReal) {
        if (descending)
            sort(begin, end, CRealDesc(tableData, column));
        else
            sort(begin, end, CRealAsc(tableData, column));
    }
    else {
        if (descending)
            sort(begin, end, CStrDesc(tableData, column));
        else
            sort(begin, end, CStrAsc(tableData, column));
    }
}

END_NCBI_SCOPE
