/*  $Id: test_bookmarks.cpp 14565 2007-05-18 12:32:01Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Mike DiCuccio
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <corelib/ncbiapp.hpp>
#include <corelib/ncbienv.hpp>
#include <corelib/ncbiargs.hpp>
#include <gui/objutils/registry.hpp>
#include <gui/objutils/bookmark.hpp>
#include <objects/seqloc/Seq_id.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


class CTestBookmakrsApp : public CNcbiApplication
{
private:
    void Init();
    int Run();
};


void CTestBookmakrsApp::Init()
{
    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions());

    SetupArgDescriptions(arg_desc.release());
}


int CTestBookmakrsApp::Run()
{
    CArgs args = GetArgs();

    CBookmarkManager& mgr = CBookmarkManager::GetInstance();

    CSeq_id id("nt_011515");
    mgr.AddBookmark(id, "Bookmark 1", 123);
    mgr.AddBookmark(id, "Bookmark 1", 123, 1000);
    mgr.AddBookmark(id, "Bookmark 1", 800, 1000);

    {{
        CBookmarkManager::TBookmarks mks = mgr.GetBookmarks(id, 100, 20000);
        cout << "bookmarks in range 100-20000: " << mks.size() << endl;
    }}

    {{
        CBookmarkManager::TBookmarks mks = mgr.GetBookmarks(id, 100, 2000);
        cout << "bookmarks in range 100-2000: " << mks.size() << endl;
    }}

    {{
        CBookmarkManager::TBookmarks mks = mgr.GetBookmarks(id, 100, 200);
        cout << "bookmarks in range 100-200: " << mks.size() << endl;
    }}

    {{
        CBookmarkManager::TBookmarks mks = mgr.GetBookmarks(id, 100, 110);
        cout << "bookmarks in range 100-110: " << mks.size() << endl;
    }}

    CGuiRegistry& reg = CGuiRegistry::GetInstance();
    CGuiRegistry::TReadWriteView view =
        reg.GetReadWriteView("GBENCH.Bookmarks");
    mgr.SaveSettings(view);
    reg.Write(cout);

    return 0;
}


END_NCBI_SCOPE
USING_SCOPE(ncbi);


int main(int argc, const char* argv[])
{
    return CTestBookmakrsApp().AppMain(argc, argv, 0, eDS_Default, 0);
}
