/*  $Id: glrendernode.cpp 27574 2013-03-07 14:54:31Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Bob Falk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/opengl/glrendernode.hpp>
#include <gui/opengl/glutils.hpp>

BEGIN_NCBI_SCOPE

CGlRenderNode::CGlRenderNode() 
: m_Visible(true)
, m_Scale(1.0f, 1.0f, 1.0f) 
{
    m_State.Reset(new CGlState());
    SetDefaultPosition();
}

void CGlRenderNode::Render() 
{
    m_State->MakeCurrent();

    x_Render();
}

void CGlRenderNode::SetState(CGlState* state) 
{ 
    m_State.Reset(state);
}

CGlState& CGlRenderNode::GetState() 
{ 
    return m_State.GetNCObject();
}

void CGlRenderNode::SkipTarget(ERenderTarget target, bool skip)
{
    vector<ERenderTarget>::iterator iter;
    
    iter = std::find(m_SkippedTargets.begin(), m_SkippedTargets.end(), target);

    // Not currently skipped.  Add if skip==true
    if (iter == m_SkippedTargets.end()) {
        if (skip) {
            m_SkippedTargets.push_back(target);
        }
    }
    // Currently skipped. Remove if skip==false
    else {
        if (!skip) {
            m_SkippedTargets.erase(iter);
        }
    }
    
    return;
}

bool CGlRenderNode::IsSkipped(ERenderTarget target) const
{
    vector<ERenderTarget>::const_iterator iter;
    
    iter = std::find(m_SkippedTargets.begin(), m_SkippedTargets.end(), target);

    // Not currently skipped.  Add if skip==true
    if (iter != m_SkippedTargets.end())
        return true;
    
    return false;
}

void CGlRenderNode::SetDefaultPosition() 
{ 
    ClearPositions(); 

    CMatrix4<float> mat;
    mat.Identity();
    m_Positions.push_back(mat);
}

void CGlRenderNode::SetPosition(const CMatrix4<float>& mat)
{ 
    if (m_Positions.size() == 1) {
        m_Positions[0] = mat;
    }
    else {
        ClearPositions(); 
        m_Positions.push_back(mat);
    }
}

END_NCBI_SCOPE