/*  $Id: glvbogeom.cpp 27503 2013-02-26 18:16:55Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Bob Falk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/opengl/glvbogeom.hpp>
#include <gui/opengl/glmodel2d.hpp>
#include <gui/opengl/glutils.hpp>
#include <gui/opengl/glresmgr.hpp>

BEGIN_NCBI_SCOPE


CGlVboGeom20::CGlVboGeom20()
{
}

CGlVboGeom20::~CGlVboGeom20()
{
    // Buffers will automatically release resources through crefs...
    
    size_t i;
    
    // The resetting here should be unnecessary...
    for (i=0; i<m_FloatBuffers.size(); ++i)
        m_FloatBuffers[i].GetBuffer().Reset();
    m_FloatBuffers.clear();

    for (i=0; i<m_Vec2Buffers.size(); ++i)
        m_Vec2Buffers[i].GetBuffer().Reset();
     m_Vec2Buffers.clear();

    for (i=0; i<m_Vec3Buffers.size(); ++i)
        m_Vec3Buffers[i].GetBuffer().Reset();
     m_Vec3Buffers.clear();

    for (i=0; i<m_Vec4Buffers.size(); ++i)
        m_Vec4Buffers[i].GetBuffer().Reset();
     m_Vec4Buffers.clear();

    for (i=0; i<m_Mat33Buffers.size(); ++i)
        m_Mat33Buffers[i].GetBuffer().Reset();
     m_Mat33Buffers.clear();

    for (i=0; i<m_Mat44Buffers.size(); ++i)
        m_Mat44Buffers[i].GetBuffer().Reset();
     m_Mat44Buffers.clear();
}

void CGlVboGeom20::Destroy()
{
    if (!m_ColorBuffer.GetBuffer().IsNull())
        m_ColorBuffer.Destroy();

    if (!m_SecondaryColorBuffer.GetBuffer().IsNull())
        m_SecondaryColorBuffer.Destroy();

    if (!m_NormalBuffer.GetBuffer().IsNull())
        m_NormalBuffer.Destroy();

    if (!m_TexCoordBuffer.GetBuffer().IsNull())
        m_TexCoordBuffer.Destroy();

    if (!m_VertexBuffer2D.GetBuffer().IsNull())
        m_VertexBuffer2D.Destroy();

    if (!m_VertexBuffer3D.GetBuffer().IsNull())
        m_VertexBuffer3D.Destroy();

    size_t i;

    for (i=0; i<m_FloatBuffers.size(); ++i)
        m_FloatBuffers[i].Destroy();

    for (i=0; i<m_Vec2Buffers.size(); ++i)
        m_Vec2Buffers[i].Destroy();

    for (i=0; i<m_Vec3Buffers.size(); ++i)
        m_Vec3Buffers[i].Destroy();

    for (i=0; i<m_Vec4Buffers.size(); ++i)
        m_Vec4Buffers[i].Destroy();

    for (i=0; i<m_Mat33Buffers.size(); ++i)
        m_Mat33Buffers[i].Destroy();

    for (i=0; i<m_Mat44Buffers.size(); ++i)
        m_Mat44Buffers[i].Destroy();
}

GLsizei CGlVboGeom20::GetSize()
{
    if (!m_VertexBuffer2D.GetBuffer().IsNull() && 
        m_VertexBuffer2D.GetDrawEnabled()) {
            return m_VertexBuffer2D.GetActiveBufferSize();
    }
    else if (!m_VertexBuffer3D.GetBuffer().IsNull() &&
             m_VertexBuffer3D.GetDrawEnabled()) {
            return m_VertexBuffer3D.GetActiveBufferSize();       
    }

    return 0;
}


CGlRenderBuffer<CRgbaColor>&
CGlVboGeom20::GetColorBuffer()
{
    if (m_ColorBuffer.GetBuffer().IsNull()) {       
        m_ColorBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CRgbaColor>("ColorBuf"));
    }

    return m_ColorBuffer;
}

CGlRenderBuffer<CRgbaColor>&
CGlVboGeom20::GetSecondaryColorBuffer()
{
    if (m_SecondaryColorBuffer.GetBuffer().IsNull()) {
        m_SecondaryColorBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CRgbaColor>("SecondaryColorBuf"));
    }

    return m_SecondaryColorBuffer;
}

CGlRenderBuffer<CVect3<float> >&
CGlVboGeom20::GetNormalBuffer()
{
    if (m_NormalBuffer.GetBuffer().IsNull()) {
        m_NormalBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect3<float> >("NormalBuffer"));
    }

    return m_NormalBuffer;
}

CGlRenderBuffer<CVect2<float> >&
CGlVboGeom20::GetTexCoordBuffer()
{
    if (m_TexCoordBuffer.GetBuffer().IsNull()) {
        m_TexCoordBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect2<float> >("TexCoordBuffer"));
    }

    return m_TexCoordBuffer;
}

CGlRenderBuffer<CVect2<float> >&
CGlVboGeom20::GetVertexBuffer2D()
{
    if (m_VertexBuffer2D.GetBuffer().IsNull()) {
        m_VertexBuffer2D.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect2<float> >("VertexBuffer2D"));
    }

    return m_VertexBuffer2D;
}

CGlRenderBuffer<CVect3<float> >&
CGlVboGeom20::GetVertexBuffer3D()
{
    if (m_VertexBuffer3D.GetBuffer().IsNull()) {
        m_VertexBuffer3D.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect3<float> >("VertexBuffer3D"));
    }

    return m_VertexBuffer3D;
}

   
CGlRenderBuffer<float>&
CGlVboGeom20::GetFloatBuffer(const string& attrib_name)
{
    for (size_t i=0; i<m_FloatBuffers.size(); ++i) {
        if (m_FloatBuffers[i].GetAttributeName() == attrib_name)
            return m_FloatBuffers[i];
    }

    CGlRenderBuffer<float> buf(CGlResMgr::Instance().GetBuffer<float>("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_FloatBuffers.push_back(buf);

    return m_FloatBuffers.back();
}

CGlRenderBuffer<CVect2<float> >&
CGlVboGeom20::GetVec2Buffer(const string& attrib_name)
{
    for (size_t i=0; i<m_Vec2Buffers.size(); ++i) {
        if (m_Vec2Buffers[i].GetAttributeName() == attrib_name)
            return m_Vec2Buffers[i];
    }

    CGlRenderBuffer<CVect2<float> > buf(CGlResMgr::Instance().GetBuffer<CVect2<float> >("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_Vec2Buffers.push_back(buf);

    return m_Vec2Buffers.back();
}
    
CGlRenderBuffer<CVect3<float> >&
CGlVboGeom20::GetVec3Buffer(const string& attrib_name)
{
    for (size_t i=0; i<m_Vec3Buffers.size(); ++i) {
        if (m_Vec3Buffers[i].GetAttributeName() == attrib_name)
            return m_Vec3Buffers[i];
    }

    CGlRenderBuffer<CVect3<float> > buf(CGlResMgr::Instance().GetBuffer<CVect3<float> >("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_Vec3Buffers.push_back(buf);

    return m_Vec3Buffers.back();
}

CGlRenderBuffer<CVect4<float> >&
CGlVboGeom20::GetVec4Buffer(const string& attrib_name)
{
    for (size_t i=0; i<m_Vec4Buffers.size(); ++i) {
        if (m_Vec4Buffers[i].GetAttributeName() == attrib_name)
            return m_Vec4Buffers[i];
    }

    CGlRenderBuffer<CVect4<float> > buf(CGlResMgr::Instance().GetBuffer<CVect4<float> >("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_Vec4Buffers.push_back(buf);

    return m_Vec4Buffers.back();
}

CGlRenderBuffer<CMatrix3<float> >&
CGlVboGeom20::GetMat33Buffer(const string& attrib_name)
{
    for (size_t i=0; i<m_Mat33Buffers.size(); ++i) {
        if (m_Mat33Buffers[i].GetAttributeName() == attrib_name)
            return m_Mat33Buffers[i];
    }

    CGlRenderBuffer<CMatrix3<float> > buf(CGlResMgr::Instance().GetBuffer<CMatrix3<float> >("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_Mat33Buffers.push_back(buf);

    return m_Mat33Buffers.back();
}

CGlRenderBuffer<CMatrix4<float> >&
CGlVboGeom20::GetMat44Buffer(const string& attrib_name)
{
    for (size_t i=0; i<m_Mat44Buffers.size(); ++i) {
        if (m_Mat44Buffers[i].GetAttributeName() == attrib_name)
            return m_Mat44Buffers[i];
    }

    CGlRenderBuffer<CMatrix4<float> > buf(CGlResMgr::Instance().GetBuffer<CMatrix4<float> >("attrib_name"));
    buf.SetAttributeName(attrib_name);
    m_Mat44Buffers.push_back(buf);

    return m_Mat44Buffers.back();
}



void CGlVboGeom20::x_EnableBuffers(vector<GLenum>& attrib_states, int num_verts)
{        
    if (!m_ColorBuffer.GetBuffer().IsNull() &&
        m_ColorBuffer.GetDrawEnabled() &&
        m_ColorBuffer.GetActiveBufferSize() == num_verts) {
            m_ColorBuffer.BindBuffer();
            glColorPointer(4, GL_FLOAT, 
                           m_ColorBuffer.GetStride(),
                           (void*)m_ColorBuffer.GetOffset());
            attrib_states.push_back(GL_COLOR_ARRAY);
    }

    if (!m_SecondaryColorBuffer.GetBuffer().IsNull() && 
        m_SecondaryColorBuffer.GetDrawEnabled() &&
        m_SecondaryColorBuffer.GetActiveBufferSize() == num_verts) {
            m_SecondaryColorBuffer.BindBuffer();
            glSecondaryColorPointer(4, GL_FLOAT, 
                                    m_SecondaryColorBuffer.GetStride(), 
                                    (void*)m_SecondaryColorBuffer.GetOffset());
            attrib_states.push_back(GL_SECONDARY_COLOR_ARRAY);
    }

    if (!m_NormalBuffer.GetBuffer().IsNull() && 
        m_NormalBuffer.GetDrawEnabled() &&
        m_NormalBuffer.GetActiveBufferSize() == num_verts) {
            m_NormalBuffer.BindBuffer();
            glNormalPointer(GL_FLOAT,
                            m_NormalBuffer.GetStride(), 
                            (void*)m_NormalBuffer.GetOffset());
            attrib_states.push_back(GL_NORMAL_ARRAY);
    }

    if (!m_TexCoordBuffer.GetBuffer().IsNull() && 
        m_TexCoordBuffer.GetDrawEnabled() &&
        m_TexCoordBuffer.GetActiveBufferSize() == num_verts) {
            m_TexCoordBuffer.BindBuffer();
            glTexCoordPointer(2, GL_FLOAT,
                              m_TexCoordBuffer.GetStride(), 
                              (void*)m_TexCoordBuffer.GetOffset());
            attrib_states.push_back(GL_TEXTURE_COORD_ARRAY);
    }

    if (!m_VertexBuffer2D.GetBuffer().IsNull() && 
        m_VertexBuffer2D.GetDrawEnabled() &&
        m_VertexBuffer2D.GetActiveBufferSize() == num_verts) {
            m_VertexBuffer2D.BindBuffer();
            glVertexPointer(2, GL_FLOAT, 
                m_VertexBuffer2D.GetStride(), 
                (GLvoid*)m_VertexBuffer2D.GetOffset());
            attrib_states.push_back(GL_VERTEX_ARRAY);
    }
    else if (!m_VertexBuffer3D.GetBuffer().IsNull() && 
        m_VertexBuffer3D.GetDrawEnabled() &&
        m_VertexBuffer3D.GetActiveBufferSize() == num_verts) {
            m_VertexBuffer3D.BindBuffer();
            glVertexPointer(3, GL_FLOAT,
                m_VertexBuffer3D.GetStride(), 
                (GLvoid*)m_VertexBuffer3D.GetOffset());
            attrib_states.push_back(GL_VERTEX_ARRAY);
    }
}

void CGlVboGeom20::Render()
{
    /// Immediate-mode rendering - support for all standard buffers
    /// available via glEnableClientState except for:
    /// GL_EDGE_FLAG_ARRAY, and GL_FOG_COORD_ARRAY which are not 
    /// frequently used.  
    bool draw_2d = (!m_VertexBuffer2D.GetBuffer().IsNull() && 
                    m_VertexBuffer2D.GetSize() > 0);
    bool draw_3d = (!m_VertexBuffer3D.GetBuffer().IsNull() && 
                    m_VertexBuffer3D.GetSize() > 0);

    if (!draw_2d && !draw_3d)
        return;

    vector<GLenum> attrib_states;
    int vertex_count;

    if (draw_2d)  {       
        x_EnableBuffers(attrib_states, m_VertexBuffer2D.GetActiveBufferSize());

        size_t i;
        for (i=0; i<attrib_states.size(); ++i)
            glEnableClientState(attrib_states[i]);
                  
        vertex_count = m_VertexBuffer2D.GetActiveBufferSize();
    }
    else {
        x_EnableBuffers(attrib_states, m_VertexBuffer3D.GetActiveBufferSize());

        size_t i;
        for (i=0; i<attrib_states.size(); ++i)
            glEnableClientState(attrib_states[i]);
                  
        vertex_count = m_VertexBuffer3D.GetActiveBufferSize();    
    }

    if (m_VertexCount != -1)
        vertex_count = m_VertexCount;

    glDrawArrays(m_DrawMode, 0, vertex_count);

    for (size_t i=0; i<attrib_states.size(); ++i)
        glDisableClientState(attrib_states[i]);   
}















CGlVboGeom11::CGlVboGeom11()
{
}

CGlVboGeom11::~CGlVboGeom11()
{
    // Buffers will automatically release resources through crefs...
}

void CGlVboGeom11::Destroy()
{
    if (!m_ColorBuffer.GetBuffer().IsNull())
        m_ColorBuffer.Destroy();

    if (!m_SecondaryColorBuffer.GetBuffer().IsNull())
        m_SecondaryColorBuffer.Destroy();

    if (!m_NormalBuffer.GetBuffer().IsNull())
        m_NormalBuffer.Destroy();

    if (!m_TexCoordBuffer.GetBuffer().IsNull())
        m_TexCoordBuffer.Destroy();

    if (!m_VertexBuffer2D.GetBuffer().IsNull())
        m_VertexBuffer2D.Destroy();

    if (!m_VertexBuffer3D.GetBuffer().IsNull())
        m_VertexBuffer3D.Destroy();
}

GLsizei CGlVboGeom11::GetSize()
{
    if (!m_VertexBuffer2D.GetBuffer().IsNull() && 
        m_VertexBuffer2D.GetDrawEnabled()) {
            return m_VertexBuffer2D.GetActiveBufferSize();
    }
    else if (!m_VertexBuffer3D.GetBuffer().IsNull() &&
             m_VertexBuffer3D.GetDrawEnabled()) {
            return m_VertexBuffer3D.GetActiveBufferSize();       
    }

    return 0;
}

CGlRenderBuffer<CRgbaColor>&
CGlVboGeom11::GetColorBuffer()
{
    if (m_ColorBuffer.GetBuffer().IsNull()) {       
        m_ColorBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CRgbaColor>("ColorBuf"));
    }

    return m_ColorBuffer;
}

CGlRenderBuffer<CRgbaColor>&
CGlVboGeom11::GetSecondaryColorBuffer()
{
    if (m_SecondaryColorBuffer.GetBuffer().IsNull()) {
        m_SecondaryColorBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CRgbaColor>("SecondaryColorBuf"));
    }

    return m_SecondaryColorBuffer;
}

CGlRenderBuffer<CVect3<float> >&
CGlVboGeom11::GetNormalBuffer()
{
    if (m_NormalBuffer.GetBuffer().IsNull()) {
        m_NormalBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect3<float> >("NormalBuffer"));
    }

    return m_NormalBuffer;
}

CGlRenderBuffer<CVect2<float> >&
CGlVboGeom11::GetTexCoordBuffer()
{
    if (m_TexCoordBuffer.GetBuffer().IsNull()) {
        m_TexCoordBuffer.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect2<float> >("TexCoordBuffer"));
    }

    return m_TexCoordBuffer;
}

CGlRenderBuffer<CVect2<float> >&
CGlVboGeom11::GetVertexBuffer2D()
{
    if (m_VertexBuffer2D.GetBuffer().IsNull()) {
        m_VertexBuffer2D.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect2<float> >("VertexBuffer2D"));
    }

    return m_VertexBuffer2D;
}

CGlRenderBuffer<CVect3<float> >&
CGlVboGeom11::GetVertexBuffer3D()
{
    if (m_VertexBuffer3D.GetBuffer().IsNull()) {
        m_VertexBuffer3D.SetBuffer(CGlResMgr::Instance().GetBuffer<CVect3<float> >("VertexBuffer3D"));
    }

    return m_VertexBuffer3D;
}

void CGlVboGeom11::x_EnableBuffers(vector<GLenum>& attrib_states, int num_verts)
{        
    if (!m_ColorBuffer.GetBuffer().IsNull() &&
        m_ColorBuffer.GetDrawEnabled() &&
        m_ColorBuffer.GetActiveBufferSize() == num_verts) {
            m_ColorBuffer.BindBuffer();
            glColorPointer(4, GL_FLOAT, 
                           m_ColorBuffer.GetStride(),
                           (GLvoid*)( ((size_t)m_ColorBuffer.MapBuffer(GL_READ_ONLY)) + 
                                        (size_t)m_ColorBuffer.GetOffset()));
            attrib_states.push_back(GL_COLOR_ARRAY);
    }

    if (!m_SecondaryColorBuffer.GetBuffer().IsNull() && 
        m_SecondaryColorBuffer.GetDrawEnabled() &&
        m_SecondaryColorBuffer.GetActiveBufferSize() == num_verts) {
            m_SecondaryColorBuffer.BindBuffer();
            glSecondaryColorPointer(4, GL_FLOAT, 
                m_SecondaryColorBuffer.GetStride(), 
                (GLvoid*)(m_SecondaryColorBuffer.MapBuffer(GL_READ_ONLY) +
                          m_SecondaryColorBuffer.GetOffset()));
            attrib_states.push_back(GL_SECONDARY_COLOR_ARRAY);
    }

    if (!m_NormalBuffer.GetBuffer().IsNull() && 
        m_NormalBuffer.GetDrawEnabled() &&
        m_NormalBuffer.GetActiveBufferSize() == num_verts) {
            m_NormalBuffer.BindBuffer();
            glNormalPointer(GL_FLOAT, 
                 m_NormalBuffer.GetStride(), 
                 (GLvoid*)(m_NormalBuffer.MapBuffer(GL_READ_ONLY) +
                           m_NormalBuffer.GetOffset()));
            attrib_states.push_back(GL_NORMAL_ARRAY);
    }

    if (!m_TexCoordBuffer.GetBuffer().IsNull() && 
        m_TexCoordBuffer.GetDrawEnabled() &&
        m_TexCoordBuffer.GetActiveBufferSize() == num_verts) {
            m_TexCoordBuffer.BindBuffer();
            glTexCoordPointer(2, GL_FLOAT, 
                m_TexCoordBuffer.GetStride(), 
                (GLvoid*)(m_TexCoordBuffer.MapBuffer(GL_READ_ONLY) +
                          m_TexCoordBuffer.GetOffset()));
            attrib_states.push_back(GL_TEXTURE_COORD_ARRAY);
    }

    if (!m_VertexBuffer2D.GetBuffer().IsNull() && 
        m_VertexBuffer2D.GetDrawEnabled() &&
        m_VertexBuffer2D.GetActiveBufferSize() == num_verts) {
            m_VertexBuffer2D.BindBuffer();
            glVertexPointer(2, GL_FLOAT, 
                m_VertexBuffer2D.GetStride(), 
                (GLvoid*)( ((size_t)m_VertexBuffer2D.MapBuffer(GL_READ_ONLY)) +
                            (size_t)m_VertexBuffer2D.GetOffset()));
            attrib_states.push_back(GL_VERTEX_ARRAY);
    }
    else if (!m_VertexBuffer3D.GetBuffer().IsNull() && 
        m_VertexBuffer3D.GetDrawEnabled() &&
        m_VertexBuffer3D.GetActiveBufferSize() == num_verts) {
            m_VertexBuffer3D.BindBuffer();
            glVertexPointer(3, GL_FLOAT,
                m_VertexBuffer3D.GetStride(), 
                (GLvoid*)( ((size_t)m_VertexBuffer3D.MapBuffer(GL_READ_ONLY)) +
                            (size_t)m_VertexBuffer3D.GetOffset()));
            attrib_states.push_back(GL_VERTEX_ARRAY);
    }
}

void CGlVboGeom11::x_DisableBuffers()
{        
    if (!m_ColorBuffer.GetBuffer().IsNull()) {
        m_ColorBuffer.UnmapBuffer();
    }

    if (!m_SecondaryColorBuffer.GetBuffer().IsNull()) {
        m_SecondaryColorBuffer.UnmapBuffer();
    }

    if (!m_NormalBuffer.GetBuffer().IsNull()) {
        m_NormalBuffer.UnmapBuffer();
    }

    if (!m_TexCoordBuffer.GetBuffer().IsNull()) {
        m_TexCoordBuffer.UnmapBuffer();
    }

    if (!m_VertexBuffer2D.GetBuffer().IsNull()) {
        m_VertexBuffer2D.UnmapBuffer();
    }
    else if (!m_VertexBuffer3D.GetBuffer().IsNull()) {
        m_VertexBuffer3D.UnmapBuffer();
    }
}

void CGlVboGeom11::Render()
{
    /// Immediate-mode rendering - support for all standard buffers
    /// available via glEnableClientState except for:
    /// GL_EDGE_FLAG_ARRAY, and GL_FOG_COORD_ARRAY which are not 
    /// frequently used.  
    bool draw_2d = (!m_VertexBuffer2D.GetBuffer().IsNull() && 
                    m_VertexBuffer2D.GetSize() > 0);
    bool draw_3d = (!m_VertexBuffer3D.GetBuffer().IsNull() && 
                    m_VertexBuffer3D.GetSize() > 0);

    if (!draw_2d && !draw_3d)
        return;

    vector<GLenum> attrib_states;
    int vertex_count;

    if (draw_2d) {  
        x_EnableBuffers(attrib_states, m_VertexBuffer2D.GetActiveBufferSize());

        size_t i;
        for (i=0; i<attrib_states.size(); ++i)
            glEnableClientState(attrib_states[i]);
                  
        vertex_count = m_VertexBuffer2D.GetActiveBufferSize();
    }
    else {
        x_EnableBuffers(attrib_states, m_VertexBuffer3D.GetActiveBufferSize());

        size_t i;
        for (i=0; i<attrib_states.size(); ++i)
            glEnableClientState(attrib_states[i]);
                  
        vertex_count = m_VertexBuffer3D.GetActiveBufferSize();    
    }

    if (m_VertexCount != -1)
        vertex_count = m_VertexCount;

    glDrawArrays(m_DrawMode, 0, vertex_count);

    for (size_t i=0; i<attrib_states.size(); ++i)
        glDisableClientState(attrib_states[i]);

    x_DisableBuffers();

    CGlUtils::CheckGlError();   
}

END_NCBI_SCOPE