/*  $Id: glvbonode.cpp 27141 2012-12-27 21:45:56Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Bob Falk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/opengl/glvbonode.hpp>
#include <gui/opengl/glutils.hpp>
#include <gui/opengl/glexception.hpp>
#include <gui/opengl/glresmgr.hpp>

BEGIN_NCBI_SCOPE

CGlVboNode::CGlVboNode()
: m_VBOGeom(NULL)
{
}

// Draw mode is one of: GL_{POINTS, LINES, LINE_STRIP, LINE_LOOP, 
// TRIANGLES, TRIANGLE_STRIP TRIANGLE_FAN, QUADS, QUAD_STRIP}
CGlVboNode::CGlVboNode(GLenum draw_mode)
: m_VBOGeom(NULL)
{
    //if (CGlVboGeom2D::IsValidDrawMode(draw_mode)) {
        m_VBOGeom = CGlResMgr::Instance().GetVboGeom("geom node vbo");
        m_VBOGeom->SetDrawMode(draw_mode);
    //}   
    //else {
    //    _TRACE("Error: OpenGL draw mode not recognized: " << draw_mode);
    //}
}

CGlVboNode::~CGlVboNode()
{    
    m_VBOGeom.Reset();
}

void CGlVboNode::Destroy()
{
    if (m_VBOGeom != NULL)
        m_VBOGeom->Destroy();
}

 // set appropriate draw mode for primitive type (GL_LINES, GL_TRIS, etc)
void  CGlVboNode::SetDrawMode(GLenum m)
{
    if (m_VBOGeom != NULL)
        m_VBOGeom->SetDrawMode(m);
}

CGlRenderBuffer<CVect2<float> >& CGlVboNode::GetVertexBuffer2D()
{
    CGlVboGeom* vbo = dynamic_cast<CGlVboGeom*>(m_VBOGeom.GetNCPointerOrNull());
    if (vbo != NULL)
        return vbo->GetVertexBuffer2D();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Geometry Node");
}


CGlRenderBuffer<CVect3<float> >& CGlVboNode::GetVertexBuffer3D()
{
    CGlVboGeom* vbo = dynamic_cast<CGlVboGeom*>(m_VBOGeom.GetNCPointerOrNull());
    if (vbo != NULL)
        return vbo->GetVertexBuffer3D();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Geometry Node");
}

CGlRenderBuffer<CRgbaColor>&
CGlVboNode::GetColorBuffer()
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetColorBuffer();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Color Buffer");
}

CGlRenderBuffer<CRgbaColor>&
CGlVboNode::GetSecondaryColorBuffer()
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetSecondaryColorBuffer();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Color Buffer");
}

CGlRenderBuffer<CVect3<float> >&
CGlVboNode::GetNormalBuffer()
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetNormalBuffer();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Normal Buffer");
}

CGlRenderBuffer<CVect2<float> >&
CGlVboNode::GetTexCoordBuffer()
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetTexCoordBuffer();
    else NCBI_THROW(COpenGLException, eGenericError, "NULL Texture Coord Buffer");
}

/*
   
CGlRenderBuffer<float>&
CGlVboNode::GetFloatBuffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetFloatBuffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}

CGlRenderBuffer<CVect2<float> >&
CGlVboNode::GetVec2Buffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetVec2Buffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}
    
CGlRenderBuffer<CVect3<float> >&
CGlVboNode::GetVec3Buffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetVec3Buffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}

CGlRenderBuffer<CVect4<float> >&
CGlVboNode::GetVec4Buffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetVec4Buffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}

CGlRenderBuffer<CMatrix3<float> >&
CGlVboNode::GetMat33Buffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetMat33Buffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}

CGlRenderBuffer<CMatrix4<float> >&
CGlVboNode::GetMat44Buffer(const string& attrib_name)
{
    if (m_VBOGeom != NULL)
        return m_VBOGeom->GetMat44Buffer(attrib_name);
    else NCBI_THROW(COpenGLException, eGenericError, "NULL OpenGL Buffer");
}
*/

void CGlVboNode::x_Render()
{      
    if (m_VBOGeom != NULL) {
        for (size_t i=0; i<m_Positions.size(); ++i) {
            glMatrixMode(GL_MODELVIEW);

            glPushMatrix();
            glLoadIdentity();
            CMatrix4<float> mat(m_Positions[i]);
            mat(0,0) *= m_Scale[0];
            mat(1,1) *= m_Scale[1];
            mat(2,2) *= m_Scale[2];      

            // This is GL version 1.3+ (we want 1.1 here)
            //glMultTransposeMatrixf(mat.GetData());
            mat.Transpose();
            glMultMatrixf(mat.GetData());

            m_VBOGeom->Render();

            glPopMatrix();
        }
    }
}


END_NCBI_SCOPE