/*  $Id: phy_load_job.cpp 22651 2010-12-08 19:53:01Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data,  the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties,  express or implied,  including
*  warranties of performance,  merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Josh Cherry, Roman Katargin, Vladimir Tereshkov
*/


#include <ncbi_pch.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/packages/pkg_alignment/phy_load_job.hpp>

#include <objmgr/object_manager.hpp>

#include <algo/phy_tree/phy_node.hpp>
#include <algo/phy_tree/dist_methods.hpp>


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CPhyLoadJob::CPhyLoadJob(CProjectService* service,
                         const CPhyLoadParams& params,
                         const vector<wxString>& filenames)
: CDataLoadingAppJob(service), m_Params(params), m_FileNames(filenames)
{
    m_Descr = "Loading Newick Tree Files";
}

void CPhyLoadJob::x_CreateProjectItems()
{
    ITERATE(vector<wxString>, it, m_FileNames) {
        CNcbiIfstream istr(it->fn_str());

        // load the tree
        auto_ptr<TPhyTreeNode>  tree(ReadNewickTree(istr));
        CRef<CBioTreeContainer> btc = MakeBioTreeContainer(tree.get());

        // not implemented now
        /*
        if (m_Params.GetSort()) {
            CRef<CFeatureDescr> fd(new CFeatureDescr());
            fd->SetId(btc->SetFdict().Get().size());
            fd->SetName("do_sort");
            btc->SetFdict().Set().push_back(fd);
        }
        */
        CRef<CProjectItem> item(new CProjectItem());
        item->SetObject(*btc);
        string label = item->GetLabelByData();
        item->SetLabel(label);

        
        m_Items.push_back(item);
    }
}

END_NCBI_SCOPE

