/*  $Id: rm_load_job.cpp 25153 2012-01-26 20:34:18Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data,  the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties,  express or implied,  including
*  warranties of performance,  merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*/


#include <ncbi_pch.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/packages/pkg_alignment/rm_load_job.hpp>

#include <objmgr/object_manager.hpp>

#include <objtools/readers/rm_reader.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CRMLoadJob::CRMLoadJob(CProjectService* service,
                       const vector<wxString>& filenames)
: CDataLoadingAppJob(service), m_FileNames(filenames)
{
    m_Descr = "Loading RepeatMasker Output Files";
}

void CRMLoadJob::x_CreateProjectItems()
{
    /// a scope for ID validation
    CRef<CObjectManager> obj_mgr = CObjectManager::GetInstance();
    m_Scope.Reset(new CScope(*obj_mgr));
    m_Scope->AddDefaults();

    ITERATE(vector<wxString>, it, m_FileNames) {
        CNcbiIfstream istr(it->fn_str());

        CRepeatMaskerReader reader(CRepeatMaskerReader::fDefaults);
        CRef<CSeq_annot> annot(reader.ReadSeqAnnot(istr));

        CRef<CProjectItem> item(new CProjectItem());
        item->SetItem().SetAnnot( *annot );

        string label = item->GetLabelByData(m_Scope.GetPointer());
        item->SetLabel(label);

        m_Items.push_back( item );
    }
}

END_NCBI_SCOPE

