/*  $Id: agp_load_job.cpp 27904 2013-04-22 19:44:03Z voronov $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data,  the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties,  express or implied,  including
*  warranties of performance,  merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Josh Cherry, Roman Katargin
*/


#include <ncbi_pch.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/packages/pkg_sequence/agp_load_job.hpp>

#include <objmgr/object_manager.hpp>
//---#include <objtools/readers/agp_read.hpp>
#include <objtools/readers/agp_seq_entry.hpp>
#include <objtools/readers/fasta.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CAgpLoadJob::CAgpLoadJob(CProjectService* service,
                         const CAgpLoadParams& params,
                         const vector<wxString>& filenames)
: CDataLoadingAppJob(service), m_Params(params), m_FileNames(filenames)
{
    m_Descr = "Loading AGP Files";
}

void CAgpLoadJob::x_CreateProjectItems()
{
    /*---
    EAgpRead_IdRule comp_id_rule =
        (m_Params.GetParseIDs() == 1) ? eAgpRead_ForceLocalId : eAgpRead_ParseId;
    bool set_gap_info = m_Params.GetSetGapInfo();
    */

    CAgpToSeqEntry::TFlags parse_flags = (m_Params.GetParseIDs() == 1) ? CAgpToSeqEntry::fForceLocalId : 0;
    parse_flags |= m_Params.GetSetGapInfo() ? CAgpToSeqEntry::fSetSeqGap : 0;

    wxString fasta_file = m_Params.GetFastaFile();

    CRef<CObjectManager> obj_mgr = CObjectManager::GetInstance();
    CRef<CScope> scope(new CScope(*obj_mgr));
    scope->AddDefaults();

    CRef<CSeq_entry> fasta_seqs;
    if ( !fasta_file.empty() ) {
        CNcbiIfstream istr(fasta_file.fn_str());
        CStreamLineReader line_reader(istr);
        CFastaReader reader(line_reader, CFastaReader::fParseGaps);
        fasta_seqs = reader.ReadSet();
    }

    ITERATE(vector<wxString>, it, m_FileNames) {
        const wxString& fn = *it;
        CDataLoadingJob::CCompressStreamHolder  istream_hold(fn);
        CNcbiIstream* istr_p = istream_hold.GetIStreamP();

        vector<CRef<CSeq_entry> > entries;
        //--- AgpRead(*istr_p, entries, comp_id_rule, set_gap_info);

        CAgpToSeqEntry agpToSeqEntry( parse_flags );
        agpToSeqEntry.ReadStream( *istr_p );
        // swap is faster than "="
        entries.swap( agpToSeqEntry.GetResult() );

        NON_CONST_ITERATE(vector<CRef<CSeq_entry> >, it2, entries) {
            CRef<CSeq_entry> &entry = *it2;
            CRef<CProjectItem> item(new CProjectItem());
            item->SetItem().SetEntry(*entry);

            string label = item->GetLabelByData(scope.GetPointer());
            item->SetLabel(label);

            m_Items.push_back(item);
        }
    }

    if (fasta_seqs) {
        CRef<CProjectItem> item(new CProjectItem());
        item->SetItem().SetEntry(*fasta_seqs);

        string label = item->GetLabelByData(scope.GetPointer());
        item->SetLabel(label);

        m_Items.push_back(item);
    }
}

END_NCBI_SCOPE

