#ifndef PKG_SEQUENCE___GB_LOAD_OPTION_PANEL__HPP
#define PKG_SEQUENCE___GB_LOAD_OPTION_PANEL__HPP

/*  $Id: gb_load_option_panel.hpp 23046 2011-01-25 21:46:58Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistl.hpp>
#include <corelib/ncbiobj.hpp>

#include <wx/panel.h>
#include <gui/widgets/wx/richtextctrl.hpp>

////@begin control identifiers
#define SYMBOL_CGENBANKLOADOPTIONPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CGENBANKLOADOPTIONPANEL_TITLE _("GenBank Load Option Panel")
#define SYMBOL_CGENBANKLOADOPTIONPANEL_IDNAME ID_CGENBANKLOADOPTIONPANEL
#define SYMBOL_CGENBANKLOADOPTIONPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CGENBANKLOADOPTIONPANEL_POSITION wxDefaultPosition
////@end control identifiers


BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

BEGIN_SCOPE(objects);
    class CSeq_id;
END_SCOPE(objects);


///////////////////////////////////////////////////////////////////////////////
/// CGenBankLoadOptionPanel

class CGenBankLoadOptionPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS( CGenBankLoadOptionPanel )
    DECLARE_EVENT_TABLE()

public:
    CGenBankLoadOptionPanel();
    CGenBankLoadOptionPanel( wxWindow* parent, wxWindowID id = SYMBOL_CGENBANKLOADOPTIONPANEL_IDNAME, const wxPoint& pos = SYMBOL_CGENBANKLOADOPTIONPANEL_POSITION, const wxSize& size = SYMBOL_CGENBANKLOADOPTIONPANEL_SIZE, long style = SYMBOL_CGENBANKLOADOPTIONPANEL_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CGENBANKLOADOPTIONPANEL_IDNAME, const wxPoint& pos = SYMBOL_CGENBANKLOADOPTIONPANEL_POSITION, const wxSize& size = SYMBOL_CGENBANKLOADOPTIONPANEL_SIZE, long style = SYMBOL_CGENBANKLOADOPTIONPANEL_STYLE );

    ~CGenBankLoadOptionPanel();

    void Init();

    void CreateControls();

////@begin CGenBankLoadOptionPanel event handler declarations

    /// wxEVT_COMMAND_TEXT_UPDATED event handler for ID_ACC_INPUT
    void OnAccInputUpdated( wxCommandEvent& event );

////@end CGenBankLoadOptionPanel event handler declarations

////@begin CGenBankLoadOptionPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CGenBankLoadOptionPanel member function declarations

    static bool ShowToolTips();

////@begin CGenBankLoadOptionPanel member variables
    CRichTextCtrl* m_AccInput;
    /// Control identifiers
    enum {
        ID_CGENBANKLOADOPTIONPANEL = 10013,
        ID_ACC_INPUT = 10022
    };
////@end CGenBankLoadOptionPanel member variables

public:
    typedef vector< CRef<CObject> > TIdsVec;


    string GetInput() const;
    void SetInput(const string& input);

    bool IsInputValid();
    TIdsVec& GetSeqIds();

protected:
    void    x_ValidateInput();

protected:
    wxTextAttr  m_DefStyle;
    wxTextAttr  m_ErrStyle;
    int     m_TokensNum; // total number of tokens
    int     m_ErrNum; // number of token with errors

    TIdsVec m_Ids;
};


/* @} */

END_NCBI_SCOPE

#endif
    // PKG_SEQUENCE___GB_LOAD_OPTION_PANEL__HPP
