/*  $Id: gff_load_params.cpp 28165 2013-05-20 20:15:13Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <gui/objutils/registry.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/packages/pkg_sequence/gff_load_params.hpp>

BEGIN_NCBI_SCOPE

/*!
 * CGffLoadParams type definition
 */

/*!
 * Default constructor for CGffLoadParams
 */

CGffLoadParams::CGffLoadParams()
{
    Init();
}

/*!
 * Copy constructor for CGffLoadParams
 */

CGffLoadParams::CGffLoadParams(const CGffLoadParams& data)
{
    Init();
    Copy(data);
}

/*!
 * Destructor for CGffLoadParams
 */

CGffLoadParams::~CGffLoadParams()
{
}

/*!
 * Assignment operator for CGffLoadParams
 */

void CGffLoadParams::operator=(const CGffLoadParams& data)
{
    Copy(data);
}

/*!
 * Equality operator for CGffLoadParams
 */

bool CGffLoadParams::operator==(const CGffLoadParams& data) const
{
////@begin CGffLoadParams equality operator
     if (!(m_NoGTF == data.m_NoGTF)) return false;
     if (!(m_MergeExons == data.m_MergeExons)) return false;
     if (!(m_MergeOnyCdsMrna == data.m_MergeOnyCdsMrna)) return false;
     if (!(m_GBQuals == data.m_GBQuals)) return false;
     if (!(m_CreateGeneFeats == data.m_CreateGeneFeats)) return false;
     if (!(m_SetProducts == data.m_SetProducts)) return false;
     if (!(m_SetVersion3 == data.m_SetVersion3)) return false;
     if (!(m_ParseSeqIds == data.m_ParseSeqIds)) return false;
     if (!(m_NameFeatSet == data.m_NameFeatSet)) return false;
     if (!(m_MapAssemblyAcc == data.m_MapAssemblyAcc)) return false;
     if (!(m_UseAssemblyMapping == data.m_UseAssemblyMapping)) return false;
////@end CGffLoadParams equality operator
    return true;
}

/*!
 * Copy function for CGffLoadParams
 */

void CGffLoadParams::Copy(const CGffLoadParams& data)
{
////@begin CGffLoadParams copy function
    m_NoGTF = data.m_NoGTF;
    m_MergeExons = data.m_MergeExons;
    m_MergeOnyCdsMrna = data.m_MergeOnyCdsMrna;
    m_GBQuals = data.m_GBQuals;
    m_CreateGeneFeats = data.m_CreateGeneFeats;
    m_SetProducts = data.m_SetProducts;
    m_SetVersion3 = data.m_SetVersion3;
    m_ParseSeqIds = data.m_ParseSeqIds;
    m_NameFeatSet = data.m_NameFeatSet;
    m_MapAssemblyAcc = data.m_MapAssemblyAcc;
    m_UseAssemblyMapping = data.m_UseAssemblyMapping;
////@end CGffLoadParams copy function
}

/*!
 * Member initialisation for CGffLoadParams
 */

void CGffLoadParams::Init()
{
////@begin CGffLoadParams member initialisation
    m_NoGTF = false;
    m_MergeExons = true;
    m_MergeOnyCdsMrna = false;
    m_GBQuals = true;
    m_CreateGeneFeats = true;
    m_SetProducts = true;
    m_SetVersion3 = false;
    m_ParseSeqIds = 0;
    m_UseAssemblyMapping = true;
////@end CGffLoadParams member initialisation
}

static const char* kNoGTF = "NoGTF";
static const char* kMergeExons = "MergeExons";
static const char* kMergeOnyCdsMrna = "MergeOnyCdsMrna";
static const char* kGBQuals = "GBQuals";
static const char* kCreateGeneFeats = "CreateGeneFeats";
static const char* kSetProducts = "SetProducts";
static const char* kSetVersion3 = "SetVersion3";
static const char* kParseSeqIds = "ParseSeqIds";
static const char* kNameFeatSet = "NameFeatSet";
static const char* kMappingAssembly = "MappingAssembly";
static const char* kUseAssemblyMapping = "UseAssemblyMapping";

/// IRegSettings
void CGffLoadParams::SaveSettings() const
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        CGuiRegistry::TReadWriteView view = gui_reg.GetReadWriteView(m_RegPath);

        view.Set(kNoGTF, m_NoGTF);
        view.Set(kMergeExons, m_MergeExons);
        view.Set(kMergeOnyCdsMrna, m_MergeOnyCdsMrna);
        view.Set(kGBQuals, m_GBQuals);
        view.Set(kCreateGeneFeats, m_CreateGeneFeats);
        view.Set(kSetProducts, m_SetProducts);
        view.Set(kSetVersion3, m_SetVersion3);
        view.Set(kParseSeqIds, m_ParseSeqIds);
        view.Set(kNameFeatSet, ToStdString( m_NameFeatSet ));
        view.Set(kMappingAssembly, m_MapAssemblyAcc);
        view.Set(kUseAssemblyMapping, m_UseAssemblyMapping);
    }
}

void CGffLoadParams::LoadSettings()
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        CGuiRegistry::TReadView view = gui_reg.GetReadView(m_RegPath);

        m_NoGTF = view.GetBool(kNoGTF, m_NoGTF);
        m_MergeExons = view.GetBool(kMergeExons, m_MergeExons);
        m_MergeOnyCdsMrna = view.GetBool(kMergeOnyCdsMrna, m_MergeOnyCdsMrna);
        m_GBQuals = view.GetBool(kGBQuals, m_GBQuals);
        m_CreateGeneFeats = view.GetBool(kCreateGeneFeats, m_CreateGeneFeats);
        m_SetProducts = view.GetBool(kSetProducts, m_SetProducts);
        m_SetVersion3 = view.GetBool(kSetVersion3, m_SetVersion3);
        m_ParseSeqIds = view.GetInt(kParseSeqIds, m_ParseSeqIds);
        m_NameFeatSet = ToWxString(view.GetString(kNameFeatSet,
                                                  ToStdString(m_NameFeatSet)));
        m_MapAssemblyAcc = view.GetString(kMappingAssembly, m_MapAssemblyAcc);
        m_UseAssemblyMapping = view.GetBool(kUseAssemblyMapping, m_UseAssemblyMapping);
    }
}

END_NCBI_SCOPE
