/*  $Id: gff_params_panel.cpp 28181 2013-05-21 18:50:52Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <wx/sizer.h>
#include <wx/checkbox.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/valtext.h>
#include <wx/valgen.h>
#include <wx/stattext.h>
#include <wx/statbox.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/msgdlg.h>

////@begin includes
////@end includes

#include "gff_params_panel.hpp"
#include <gui/core/assembly_sel_panel.hpp>

BEGIN_NCBI_SCOPE

////@begin XPM images
////@end XPM images


/*!
 * CGffParamsPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CGffParamsPanel, wxPanel )


/*!
 * CGffParamsPanel event table definition
 */

BEGIN_EVENT_TABLE( CGffParamsPanel, wxPanel )

////@begin CGffParamsPanel event table entries
////@end CGffParamsPanel event table entries

END_EVENT_TABLE()


/*!
 * CGffParamsPanel constructors
 */

CGffParamsPanel::CGffParamsPanel()
{
    Init();
}

CGffParamsPanel::CGffParamsPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CGFFParamsPanel creator
 */

bool CGffParamsPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CGffParamsPanel creation
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CGffParamsPanel creation
    return true;
}


/*!
 * CGffParamsPanel destructor
 */

CGffParamsPanel::~CGffParamsPanel()
{
////@begin CGffParamsPanel destruction
////@end CGffParamsPanel destruction
}


/*!
 * Member initialisation
 */

void CGffParamsPanel::Init()
{
////@begin CGffParamsPanel member initialisation
    m_AssemblyPanel = NULL;
////@end CGffParamsPanel member initialisation
}


/*!
 * Control creation for CGFFParamsPanel
 */

void CGffParamsPanel::CreateControls()
{
////@begin CGffParamsPanel content construction
    CGffParamsPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemPanel1, wxID_ANY, _("GFF Load Parameters"));
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 0, wxALIGN_TOP|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer4 = new wxFlexGridSizer(3, 2, 0, 0);
    itemStaticBoxSizer3->Add(itemFlexGridSizer4, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("Sequence Identifiers"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText5, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemChoice6Strings;
    itemChoice6Strings.Add(_("Parse Normally"));
    itemChoice6Strings.Add(_("Numeric Identifiers as Local Identifiers"));
    itemChoice6Strings.Add(_("All Identifiers as Local Identifiers"));
    wxChoice* itemChoice6 = new wxChoice( itemPanel1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, itemChoice6Strings, 0 );
    itemChoice6->SetStringSelection(_("Parse Normally"));
    itemFlexGridSizer4->Add(itemChoice6, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Name for feature set"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText7, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl8 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemTextCtrl8, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxCheckBox* itemCheckBox9 = new wxCheckBox( itemPanel1, ID_CHECKBOX6, _("Do not restrict to GTF conventions"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox9->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox9, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox10 = new wxCheckBox( itemPanel1, ID_CHECKBOX8, _("GFF/GTF Attributes as GenBank qualifiers"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox10->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox10, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox11 = new wxCheckBox( itemPanel1, ID_CHECKBOX10, _("transcript_id, protein_id as products"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox11->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox11, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox12 = new wxCheckBox( itemPanel1, ID_CHECKBOX12, _("Create missing genes for CDS, mRNA features"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox12->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox12, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox13 = new wxCheckBox( itemPanel1, ID_CHECKBOX7, _("Merge intervals (by transcript_id/gene_id)"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox13->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox13, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox14 = new wxCheckBox( itemPanel1, ID_CHECKBOX9, _("Merge intervals only in mRNA/CDS features"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox14->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox14, 0, wxALIGN_LEFT|wxALL, 5);

    wxCheckBox* itemCheckBox15 = new wxCheckBox( itemPanel1, ID_CHECKBOX11, _("Use GFF version 3 parser by default"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox15->SetValue(false);
    itemStaticBoxSizer3->Add(itemCheckBox15, 0, wxALIGN_LEFT|wxALL, 5);

    m_AssemblyPanel = new CAssemblySelPanel( itemPanel1, ID_PANEL7, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(m_AssemblyPanel, 1, wxALIGN_TOP|wxALL, 5);

    // Set validators
    itemChoice6->SetValidator( wxGenericValidator(& GetData().m_ParseSeqIds) );
    itemTextCtrl8->SetValidator( wxTextValidator(wxFILTER_NONE, & GetData().m_NameFeatSet) );
    itemCheckBox9->SetValidator( wxGenericValidator(& GetData().m_NoGTF) );
    itemCheckBox10->SetValidator( wxGenericValidator(& GetData().m_GBQuals) );
    itemCheckBox11->SetValidator( wxGenericValidator(& GetData().m_SetProducts) );
    itemCheckBox12->SetValidator( wxGenericValidator(& GetData().m_CreateGeneFeats) );
    itemCheckBox13->SetValidator( wxGenericValidator(& GetData().m_MergeExons) );
    itemCheckBox14->SetValidator( wxGenericValidator(& GetData().m_MergeOnyCdsMrna) );
    itemCheckBox15->SetValidator( wxGenericValidator(& GetData().m_SetVersion3) );
////@end CGffParamsPanel content construction

    m_AssemblyPanel->SetMinSize(wxSize(300, 250));
}


/*!
 * Should we show tooltips?
 */

bool CGffParamsPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CGffParamsPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CGffParamsPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CGffParamsPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CGffParamsPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CGffParamsPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CGffParamsPanel icon retrieval
}

/*!
 * Transfer data to the window
 */

bool CGffParamsPanel::TransferDataToWindow()
{
    m_AssemblyPanel->SetUseMapping(GetData().GetUseAssemblyMapping());
    m_AssemblyPanel->SetAccession(GetData().GetMapAssemblyAcc());

    if (!wxPanel::TransferDataToWindow())
        return false;

    return true;
}

/*!
 * Transfer data from the window
 */

bool CGffParamsPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    if (m_AssemblyPanel->GetUseMapping() && m_AssemblyPanel->GetAccession().empty()) {
        wxMessageBox(wxT("Please, select assembly for Mapping."), wxT("Error"),
                     wxOK | wxICON_ERROR, this);
        return false;
    }

    GetData().SetUseAssemblyMapping(m_AssemblyPanel->GetUseMapping());
    GetData().SetMapAssemblyAcc(m_AssemblyPanel->GetAccession());

    return true;
}

END_NCBI_SCOPE
