#ifndef PKG_SEQUENCE___GFF_PARAMS_PANEL__HPP
#define PKG_SEQUENCE___GFF_PARAMS_PANEL__HPP

/*  $Id: gff_params_panel.hpp 28159 2013-05-17 20:51:07Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */

#include <corelib/ncbistd.hpp>

/*!
 * Includes
 */

#include <wx/panel.h>

#include <gui/packages/pkg_sequence/gff_load_params.hpp>

/*!
 * Forward declarations
 */

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CGFFPARAMSPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CGFFPARAMSPANEL_TITLE _("GFF Format Params Panel")
#define SYMBOL_CGFFPARAMSPANEL_IDNAME ID_CGFFPARAMSPANEL
#define SYMBOL_CGFFPARAMSPANEL_SIZE wxSize(400, 500)
#define SYMBOL_CGFFPARAMSPANEL_POSITION wxDefaultPosition
////@end control identifiers


BEGIN_NCBI_SCOPE

class CAssemblySelPanel;

/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

/*!
 * CGffParamsPanel class declaration
 */

class CGffParamsPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS( CGffParamsPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CGffParamsPanel();
    CGffParamsPanel( wxWindow* parent, wxWindowID id = SYMBOL_CGFFPARAMSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CGFFPARAMSPANEL_POSITION, const wxSize& size = SYMBOL_CGFFPARAMSPANEL_SIZE, long style = SYMBOL_CGFFPARAMSPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CGFFPARAMSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CGFFPARAMSPANEL_POSITION, const wxSize& size = SYMBOL_CGFFPARAMSPANEL_SIZE, long style = SYMBOL_CGFFPARAMSPANEL_STYLE );

    /// Destructor
    ~CGffParamsPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

////@begin CGffParamsPanel event handler declarations

////@end CGffParamsPanel event handler declarations

////@begin CGffParamsPanel member function declarations

    /// Data access
    CGffLoadParams& GetData() { return m_data; }
    const CGffLoadParams& GetData() const { return m_data; }
    void SetData(const CGffLoadParams& data) { m_data = data; }

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CGffParamsPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CGffParamsPanel member variables
    CAssemblySelPanel* m_AssemblyPanel;
    /// The data edited by this window
    CGffLoadParams m_data;
    /// Control identifiers
    enum {
        ID_CGFFPARAMSPANEL = 10003,
        ID_CHOICE1 = 10004,
        ID_TEXTCTRL1 = 10012,
        ID_CHECKBOX6 = 10005,
        ID_CHECKBOX8 = 10007,
        ID_CHECKBOX10 = 10009,
        ID_CHECKBOX12 = 10011,
        ID_CHECKBOX7 = 10006,
        ID_CHECKBOX9 = 10008,
        ID_CHECKBOX11 = 10010,
        ID_PANEL7 = 10091
    };
////@end CGffParamsPanel member variables
};

/* @} */

END_NCBI_SCOPE

#endif // PKG_SEQUENCE___GFF_PARAMS_PANEL__HPP
