/*  $Id: wig_params_panel.cpp 25798 2012-05-10 16:34:05Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <wx/sizer.h>
#include "wx/valgen.h"
#include "wx/valtext.h"
#include "wx/stattext.h"
#include "wx/choice.h"
#include "wx/spinctrl.h"
#include "wx/bitmap.h"
#include "wx/icon.h"

#include "wig_params_panel.hpp"

BEGIN_NCBI_SCOPE

/*!
 * CWigParamsPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CWigParamsPanel, wxPanel )


/*!
 * CWigParamsPanel event table definition
 */

BEGIN_EVENT_TABLE( CWigParamsPanel, wxPanel )

////@begin CWigParamsPanel event table entries
////@end CWigParamsPanel event table entries

END_EVENT_TABLE()


/*!
 * CWigParamsPanel constructors
 */

CWigParamsPanel::CWigParamsPanel()
{
    Init();
}

CWigParamsPanel::CWigParamsPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CWigParamsPanel creator
 */

bool CWigParamsPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CWigParamsPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CWigParamsPanel creation
    return true;
}


/*!
 * CWigParamsPanel destructor
 */

CWigParamsPanel::~CWigParamsPanel()
{
////@begin CWigParamsPanel destruction
////@end CWigParamsPanel destruction
}


/*!
 * Member initialisation
 */

void CWigParamsPanel::Init()
{
////@begin CWigParamsPanel member initialisation
////@end CWigParamsPanel member initialisation
}


/*!
 * Control creation for CWigParamsPanel
 */

void CWigParamsPanel::CreateControls()
{    
////@begin CWigParamsPanel content construction
    CWigParamsPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText( itemPanel1, wxID_STATIC, _("WIG Load Parameters"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticText3->SetFont(wxFont(8, wxSWISS, wxNORMAL, wxBOLD, false, wxT("Tahoma")));
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer4 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer4, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("Map ids using context"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemChoice6Strings;
    itemChoice6Strings.Add(_("none"));
    itemChoice6Strings.Add(_("rn4"));
    itemChoice6Strings.Add(_("mm7"));
    itemChoice6Strings.Add(_("mm8"));
    itemChoice6Strings.Add(_("mm9"));
    itemChoice6Strings.Add(_("hg17"));
    itemChoice6Strings.Add(_("hg18"));
    itemChoice6Strings.Add(_("hg19"));
    wxChoice* itemChoice6 = new wxChoice( itemPanel1, ID_CHOICE5, wxDefaultPosition, wxDefaultSize, itemChoice6Strings, 0 );
    itemChoice6->SetStringSelection(_("none"));
    itemFlexGridSizer4->Add(itemChoice6, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Stop when number of errors reaches"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText7, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxSpinCtrl* itemSpinCtrl8 = new wxSpinCtrl( itemPanel1, ID_SPINCTRL1, wxT("10"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 1000, 10 );
    itemFlexGridSizer4->Add(itemSpinCtrl8, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Set validators
    itemChoice6->SetValidator( wxGenericValidator(& GetData().m_MappingContext) );
    itemSpinCtrl8->SetValidator( wxGenericValidator(& GetData().m_NumErrors) );
////@end CWigParamsPanel content construction
}


/*!
 * Should we show tooltips?
 */

bool CWigParamsPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CWigParamsPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CWigParamsPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CWigParamsPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CWigParamsPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CWigParamsPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CWigParamsPanel icon retrieval
}

/*!
 * Transfer data to the window
 */

bool CWigParamsPanel::TransferDataToWindow()
{
    return wxPanel::TransferDataToWindow();
}

/*!
 * Transfer data from the window
 */

bool CWigParamsPanel::TransferDataFromWindow()
{
    return wxPanel::TransferDataFromWindow();
}

END_NCBI_SCOPE
