/*  $Id: windowmasker_panel.cpp 25807 2012-05-11 16:21:46Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin, Yury Voronov
 */

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_sequence/windowmasker_panel.hpp>
#include <gui/widgets/object_list/object_list_widget.hpp>
#include <gui/widgets/wx/table_utils.hpp>
#include <gui/widgets/wx/sys_path.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/tax_id_helper.hpp>

#include <gui/core/winmask_files.hpp>
#include <gui/core/commands.hpp>

#include <objmgr/util/sequence.hpp>
#include <corelib/ncbifile.hpp>
#include <corelib/ncbireg.hpp>

#include <wx/sizer.h>
#include <wx/statbox.h>
#include <wx/radiobox.h>
#include <wx/msgdlg.h>
#include <wx/stattext.h>
#include <wx/choice.h>
#include <wx/combobox.h>
#include <wx/checkbox.h>
#include <wx/imaglist.h>
#include <wx/filename.h>
#include <wx/dir.h>

////@begin XPM images
////@end XPM images


/*!
 * CWindowMaskerPanel type definition
 */

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

IMPLEMENT_DYNAMIC_CLASS( CWindowMaskerPanel, CAlgoToolManagerParamsPanel )


/*!
 * CWindowMaskerPanel event table definition
 */

BEGIN_EVENT_TABLE( CWindowMaskerPanel, CAlgoToolManagerParamsPanel )

////@begin CWindowMaskerPanel event table entries
    EVT_BUTTON( ID_BUTTON5, CWindowMaskerPanel::OnWMDownload )

////@end CWindowMaskerPanel event table entries

END_EVENT_TABLE()


CWindowMaskerPanel::TMaskerPaths CWindowMaskerPanel::sm_MaskerPaths;

/*!
 * CWindowMaskerPanel constructors
 */

CWindowMaskerPanel::CWindowMaskerPanel()
{
    Init();
}

CWindowMaskerPanel::CWindowMaskerPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CWindowMaskerPanel creator
 */

bool CWindowMaskerPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CWindowMaskerPanel creation
    CAlgoToolManagerParamsPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CWindowMaskerPanel creation
    return true;
}


/*!
 * CWindowMaskerPanel destructor
 */

CWindowMaskerPanel::~CWindowMaskerPanel()
{
////@begin CWindowMaskerPanel destruction
////@end CWindowMaskerPanel destruction
}


/*!
 * Member initialisation
 */

void CWindowMaskerPanel::Init()
{
////@begin CWindowMaskerPanel member initialisation
    m_ObjectList = NULL;
    m_cbParam = NULL;
    m_cbOut = NULL;
    m_chbStandalone = NULL;
////@end CWindowMaskerPanel member initialisation
}


/*!
 * Control creation for CWindowMaskerPanel
 */

void CWindowMaskerPanel::CreateControls()
{
////@begin CWindowMaskerPanel content construction
    CWindowMaskerPanel* itemCAlgoToolManagerParamsPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCAlgoToolManagerParamsPanel1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemCAlgoToolManagerParamsPanel1, wxID_ANY, _("Locations to Mask"));
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 0, wxGROW|wxALL, 5);

    m_ObjectList = new CObjectListWidget( itemCAlgoToolManagerParamsPanel1, ID_COBJECTLISTWIDGET, wxDefaultPosition, wxSize(300, 150), wxLC_REPORT );
    itemStaticBoxSizer3->Add(m_ObjectList, 1, wxGROW|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer5 = new wxFlexGridSizer(0, 3, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer5, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemCAlgoToolManagerParamsPanel1, wxID_STATIC, _("Mask using parameters for"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemStaticText6, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_cbParamStrings;
    m_cbParam = new wxChoice( itemCAlgoToolManagerParamsPanel1, ID_CHOICE7, wxDefaultPosition, wxDefaultSize, m_cbParamStrings, 0 );
    itemFlexGridSizer5->Add(m_cbParam, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemCAlgoToolManagerParamsPanel1, ID_BUTTON5, _("Download..."), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemButton8, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemCAlgoToolManagerParamsPanel1, wxID_STATIC, _("Objects to Create"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemStaticText9, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_cbOutStrings;
    m_cbOut = new wxComboBox( itemCAlgoToolManagerParamsPanel1, ID_COMBOBOX2, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_cbOutStrings, wxCB_READONLY );
    itemFlexGridSizer5->Add(m_cbOut, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer5->AddGrowableCol(1);

    itemBoxSizer2->Add(5, 20, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_chbStandalone = new wxCheckBox( itemCAlgoToolManagerParamsPanel1, ID_LOCALRUN, _("Run tool as standalone"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chbStandalone->SetValue(true);
    itemBoxSizer2->Add(m_chbStandalone, 0, wxALIGN_LEFT|wxALL, 5);

////@end CWindowMaskerPanel content construction    
}


/*!
 * Should we show tooltips?
 */

bool CWindowMaskerPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CWindowMaskerPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CWindowMaskerPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CWindowMaskerPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CWindowMaskerPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CWindowMaskerPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CWindowMaskerPanel icon retrieval
}

void CWindowMaskerPanel::OnWMDownload( wxCommandEvent& event )
{
    event.SetEventType( wxEVT_COMMAND_MENU_SELECTED );
    event.SetId( eCmdWindowMaskerFiles );

    wxTheApp->GetTopWindow()->GetEventHandler()->ProcessEvent( event );

    wxString result = event.GetString();
    if( !result.empty() ){

        string msg;
        if( result == wxT("Download") ){
            msg = "WindowMasker database download task started.";
            msg += "\nPlease re-run Tools dialog later, when the task finishes.";

        } else if( result == wxT("Reconfigured") ){
            msg = "WindowMasker database path configuration changed.";
            msg += "\nPlease re-start Tools dialog.";
        } else {
            msg = "WindowMasker configuration changed.";
            msg += "\nPlease re-start Tools dialog.";
        }

        wxMessageBox( ToWxString(msg) ,wxT("Genome Workbench"), wxOK | wxICON_EXCLAMATION );

        wxCloseEvent clev( wxEVT_CLOSE_WINDOW, wxID_ANY );
        clev.SetEventObject( this );
        clev.ResumePropagation( wxEVENT_PROPAGATE_MAX );

        ProcessEvent( clev );
    }
}


static const char *kTableTag = "Table";

void CWindowMaskerPanel::SaveSettings() const
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadWriteView table_view;

        reg_path = CGuiRegistryUtil::MakeKey(m_RegPath, kTableTag);
        table_view = gui_reg.GetReadWriteView(reg_path);
        ncbi::SaveTableSettings(*m_ObjectList, table_view);
    }
}

void CWindowMaskerPanel::LoadSettings()
{
    if (!m_RegPath.empty()) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadView table_view;

        reg_path = CGuiRegistryUtil::MakeKey(m_RegPath, kTableTag);
        table_view = gui_reg.GetReadView(reg_path);
        ncbi::LoadTableSettings(*m_ObjectList, table_view);
    }
}


bool CWindowMaskerPanel::TransferDataToWindow()
{
    if (m_InputObjects) {
        m_ObjectList->SetObjects(*m_InputObjects);
        m_ObjectList->SelectAll();
    }
    if (m_cbOut) {
        m_cbOut->Append(wxT("Features representing masked regions"));    
        m_cbOut->Append(wxT("A sequence location representing masked regions"));
        m_cbOut->Append(wxT("Both features and sequence location"));
        m_cbOut->SetSelection(0);
    }
    
    int selectTaxId = 0;//prg_params.m_WM_TaxId;
    vector<int> masks;
    CWinMaskerFileStorage::GetInstance().GetWinMaskerTaxIds( masks );
    CTaxIdHelper::FillTaxIds(*m_cbParam, masks, selectTaxId);

    /*
    // loading params
    /// identify the location of the WindowMasker data installation
    wxString path = CSysPath::ResolvePathExisting
        (wxT("<std>/etc/window-masker.ini"));
    if ( !path.empty() ) {
        CMemoryRegistry reg;
        CNcbiIfstream istr(path.fn_str());
        reg.Read(istr);
        path = ToWxString(reg.Get("WindowMasker", "DataPath"));
    }

    if (path.empty()) {
        /// scan the GUI registry for the data path first
        /// this allows site-specific overrides
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        path = ToWxString(gui_reg.GetString
            ("GBENCH.Plugins.Algo.WindowMasker.DataPath"));
    }

    // hack
    if (path.empty()) {
        path = wxT("\\\\SNOWMAN\\gbench\\data\\window-masker");
    }

    if ( !path.empty() ) {
        /// now, read the configuration for the data
        CNcbiIfstream istr((path + wxT("/window-masker.ini")).fn_str());
        CMemoryRegistry reg;
        reg.Read(istr);

        list<string> sections;
        reg.EnumerateSections(&sections);
        ITERATE (list<string>, iter, sections) {
            string org_name = reg.Get(*iter, "Organism");
            string build_str = reg.Get(*iter, "Builds");
            list<string> builds;
            NStr::Split(build_str, ", ;", builds);
            ITERATE (list<string>, it, builds) {
                string name = org_name;
                name += " (build ";
                name += *it;
                name += ")";

                string this_path;
                this_path += "/";
                this_path += *iter;
                this_path += "/";
                this_path += *it;

                SMaskerInfo& info = sm_MaskerPaths[name];
                try {
                    info.taxid = NStr::StringToInt(*iter);
                }
                catch (CException&) {
                }
                info.org_name = org_name;
                info.build = *it;
                info.path = ToStdString(path) + this_path; // TODO this won't work with intl fnames
            }
        }



        ITERATE (TMaskerPaths, iter, sm_MaskerPaths) {
            m_cbParam->Append(ToWxString(iter->first));
        }
    } else {
        LOG_POST(Error << "CAlgoPlugin_WindowMasker::FinalizeArgs(): "
                 "no window masker data files available");
    }
    */


    ///
    /// now, try to determine the best default stat file
    /// we do this based on the stats file
    ///

    /* TODO
    typedef map<int, int> TTaxIdCounts;
    TTaxIdCounts taxid_counts;

    ITERATE (TConstScopedObjects, iter, *m_InputObjects) {
        const CSeq_loc * loc = dynamic_cast<const CSeq_loc*>(iter->object.GetPointer());      
        if (!loc) continue;

        CBioseq_Handle handle = (const_cast<CScope*>(iter->scope.GetPointer()))->GetBioseqHandle(*loc);
        int taxid = sequence::GetTaxId(handle);
        TTaxIdCounts::iterator i = taxid_counts.insert
            (TTaxIdCounts::value_type(taxid, 0)).first;
        ++i->second;
    }

    typedef map<string, int> TCounts;
    TCounts counts;
    ITERATE (TTaxIdCounts, iter, taxid_counts) {
        ITERATE (TMaskerPaths, it, sm_MaskerPaths) {
            if (it->second.taxid == iter->first) {
                TCounts::iterator i =
                    counts.insert(TCounts::value_type(it->first, 0)).first;
                ++i->second;
            }
        }
    }

    string best_path;
    int best_count = 0;
    ITERATE (TCounts, iter, counts) {
        if (iter->second >= best_count) {
            best_count = iter->second;
            best_path = iter->first;
        }
    }
    m_cbParam->SetStringSelection(ToWxString(best_path));
    */

    m_chbStandalone->SetValue( m_Params.mf_Standalone );

    return wxPanel::TransferDataToWindow();
}

/*!
 * Transfer data from the window
 */

bool CWindowMaskerPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    m_Params.m_Objects.clear();
    m_ObjectList->GetSelection( m_Params.m_Objects );

    if( m_Params.m_Objects.empty() ){
        wxMessageBox( 
            wxT("Please, select sequences to mask."), wxT("Error"),
            wxOK | wxICON_ERROR, this
        );
        m_ObjectList->SetFocus();
        return false;
    }
    
    /*
    TMaskerPaths::const_iterator iter =
        sm_MaskerPaths.find(ToStdString(m_cbParam->GetValue()));
    if( iter != sm_MaskerPaths.end() ){
        m_Params.m_Path = iter->second.path;
    }
    */

    int taxId = 0, index = m_cbParam->GetSelection();
    if (wxNOT_FOUND != index){
        wxItemContainer* container = dynamic_cast<wxItemContainer*>(m_cbParam);
        if (container) {
            taxId = (int)(size_t)container->GetClientData(index);
        }
    }

    if( taxId != 0 ){
        CWinMaskerFileStorage& storage = CWinMaskerFileStorage::GetInstance();
        wxFileName fname(storage.GetPath(), wxEmptyString);
        fname.AppendDir(wxString::Format(wxT("%u"), taxId));

        wxDir dir(fname.GetPath());
        if (dir.IsOpened()) {
            wxString dirName;
            for (
                bool cont = dir.GetFirst(&dirName, wxEmptyString, wxDIR_DIRS);
                cont;
                cont = dir.GetNext(&dirName) 
            ) {
                fname.AppendDir(dirName);
                break;
            }
        }

        m_Params.m_Path = fname.GetPath();
    }

    switch( m_cbOut->GetSelection() ){
        case 0: m_Params.m_Output = CWindowMaskerParams::eFeatures; break;
        case 1: m_Params.m_Output = CWindowMaskerParams::eLocation; break;
        case 2: m_Params.m_Output = CWindowMaskerParams::eBoth;     break;
        default: m_Params.m_Output = CWindowMaskerParams::eNone;     
    }

    m_Params.mf_Standalone = m_chbStandalone->GetValue();

    return true;
}

END_NCBI_SCOPE
