/*  $Id: add_citsubupdate.cpp 27132 2012-12-26 20:23:14Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */

#include <ncbi_pch.hpp>
#include <ctime>
#include <gui/objutils/cmd_create_desc.hpp>
#include <objects/misc/sequence_macros.hpp>
#include <gui/packages/pkg_sequence_edit/add_citsubupdate.hpp>


BEGIN_NCBI_SCOPE
using namespace objects;

bool CAddCitSubForUpdate::apply(objects::CSeq_entry_Handle tse, ICommandProccessor* cmdProcessor, string title)
{
    if (tse) {
        CRef<CCmdComposite> composite(new CCmdComposite(title));    // See CEditObjectPubdesc::GetEditCommand from edit_object_seq_desc.cpp
        apply_to_seq_and_feat(tse,composite);
        cmdProcessor->Execute(composite.GetPointer());
        return true;
    }
    else
        return false;
}


void CAddCitSubForUpdate::apply_to_seq_and_feat(objects::CSeq_entry_Handle tse, CCmdComposite* composite)
{
    ApplyToCSeq_entry (tse, *(tse.GetCompleteSeq_entry()), composite);
    //CFeat_CI feat (tse, SAnnotSelector(CSeqFeatData::e_Pub));   TODO
    //while (feat) {
    //   ApplyToPub (feat->GetData().GetPub(), composite, NULL, NULL,&(feat->GetOriginalFeature())); 
    //       ++feat;
    //}
}


void CAddCitSubForUpdate::ApplyToCSeq_entry (CSeq_entry_Handle tse, const CSeq_entry& se, CCmdComposite* composite)  // s_CollectPubDescriptorLabels in validatorp.cpp
{
    if (se.IsSet()) 
    {
        FOR_EACH_SEQENTRY_ON_SEQSET (it, se.GetSet()) 
        {
            ApplyToCSeq_entry (tse, **it, composite);
        }
    }
    const CDate* MaxDate = NULL;
    const objects::CPubdesc* LatestPub = NULL;
    FOR_EACH_SEQDESC_ON_SEQENTRY (it, se) 
        if ((*it)->IsPub()) 
        {
            GetLatestPub ((*it)->GetPub(),MaxDate,LatestPub);
        }
    CRef<CDate> today(new CDate);
    Today(today);
    if (LatestPub && today->Compare(*MaxDate) != CDate::eCompare_same)  
    {
        CRef<CSerialObject> edited_object;
        edited_object.Reset((CSerialObject*)CPubdesc::GetTypeInfo()->Create());
        edited_object->Assign(*LatestPub);
        CPubdesc& edited_pubdesc = dynamic_cast<CPubdesc&>(*edited_object);

        EDIT_EACH_PUB_ON_PUBDESC (pub_iter, edited_pubdesc) {
            CPub& pub = **pub_iter;
            switch( pub.Which() ) {
            case CPub::e_Sub: {
                pub.SetSub().SetDescr("Sequence update by submitter"); 
                //pub.SetSub().SetImp().SetDate(today);
                pub.SetSub().SetDate(*today); 
            }
                break;
            default:
                break;
            }
        }   

        CRef<CSeqdesc> descr(new CSeqdesc());
        descr->SetPub(edited_pubdesc);
        CSeq_entry_Handle seh = tse.GetScope().GetSeq_entryHandle(se);
        CIRef<IEditCommand> cmdAddDesc(new CCmdCreateDesc(seh, *descr));
        composite->AddCommand(*cmdAddDesc);
    }
   
}

void CAddCitSubForUpdate::Today(CDate* today)
{
    time_t t = time(0);   // get time now
    struct tm * now = localtime( & t );
    today->SetStd().SetYear(now->tm_year + 1900);
    today->SetStd().SetMonth(now->tm_mon + 1);
    today->SetStd().SetDay(now->tm_mday);
}

void CAddCitSubForUpdate::GetLatestPub (const objects::CPubdesc& pubdesc,  const CDate* &MaxDate, const objects::CPubdesc* &LatestPub)
{
    FOR_EACH_PUB_ON_PUBDESC (pub_iter, pubdesc) {
        const CPub& pub = **pub_iter;
        switch( pub.Which() ) {
        case CPub::e_Sub: {
            if ( pub.GetSub().IsSetDate() ) 
            {
                MaxDate = GetLaterDate(MaxDate,&pub.GetSub().GetDate());
                if (MaxDate == &pub.GetSub().GetDate())
                    LatestPub = &pubdesc;
            }
            if ( pub.GetSub().IsSetImp() && pub.GetSub().GetImp().IsSetDate()) 
            {
                MaxDate = GetLaterDate(MaxDate,&pub.GetSub().GetImp().GetDate());
                if (MaxDate == &pub.GetSub().GetImp().GetDate())
                    LatestPub = &pubdesc;
            }
        }
            break;
        default:
            break;
        }
    }  
}

const CDate* CAddCitSubForUpdate::GetLaterDate(const CDate* d1, const CDate* d2) const  // see CLocusItem::x_GetLaterDate in locus_item.cpp  
{
    if ( d1 == 0 || d1->Which() == CDate::e_Str ) {
        return d2;
    }

    if ( d2 == 0 || d2->Which() == CDate::e_Str ) {
        return d1;
    }

    return (d1->Compare(*d2) == CDate::eCompare_after) ? d1 : d2;  
}

END_NCBI_SCOPE
