/*  $Id: btns_tool.cpp 27556 2013-03-05 16:04:52Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Colleen Bollin, based on a file by Roman Katargin
*
*/

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_sequence_edit/btns_tool.hpp>
#include <gui/framework/app_task_impl.hpp>
#include <gui/core/project_service.hpp>
#include <gui/framework/window_manager_service.hpp>

#include <objects/seqloc/Seq_id.hpp>
#include <objects/submit/Seq_submit.hpp>
#include <objects/submit/Submit_block.hpp>

#include <wx/frame.h>
#include <wx/msgdlg.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(ncbi::objects);

///////////////////////////////////////////////////////////////////////////////
/// CEditingBtnsTool
CEditingBtnsTool::CEditingBtnsTool()
:   m_Descriptor("Sequin Buttons",
                 "",
                 "Sequin Buttons",
                 "Sequence Editing Buttons",
                 "BTNS"),
    m_Category("Submission Preparation"),
    m_SrvLocator(),
    m_ParentWindow()
{
}

string CEditingBtnsTool::GetExtensionIdentifier() const
{
    return "btns_tool";
}


string CEditingBtnsTool::GetExtensionLabel() const
{
    return "Sequin Buttons";
}

const IUIObject& CEditingBtnsTool::GetDescriptor() const
{
    return m_Descriptor;
}

void CEditingBtnsTool::SetServiceLocator(IServiceLocator* srv_locator)
{
    m_SrvLocator = srv_locator;
}


void CEditingBtnsTool::SetParentWindow(wxWindow* parent)
{
    m_ParentWindow = parent;
}

void CEditingBtnsTool::InitUI()
{
}

void CEditingBtnsTool::CleanUI()
{
}

wxPanel* CEditingBtnsTool::GetCurrentPanel()
{
    return 0;
}

bool CEditingBtnsTool::IsFinalState()
{
    return true;
}

bool CEditingBtnsTool::IsCompletedState()
{
    return true;
}

bool CEditingBtnsTool::CanDo(EAction action)
{
    return true;
}

bool CEditingBtnsTool::DoTransition(EAction action)
{
    if (m_Objects.empty()) {
        wxMessageBox(wxT("Selected object(s) can't be used for Submission!"), wxT("Error"),
                     wxOK | wxICON_ERROR, m_ParentWindow);
        return false;
    }
    return true;
}

string CEditingBtnsTool::GetCategory()
{
    return m_Category;
}

void CEditingBtnsTool::SetInputObjects(const TConstScopedObjects& input)
{
    m_Objects.clear();

    ITERATE(TConstScopedObjects, it, input) {
        const CObject* ptr = it->object.GetPointer();
        if (dynamic_cast<const CSeq_entry*>(ptr) ||
            dynamic_cast<const CSeq_annot*>(ptr) ||
            dynamic_cast<const CSeq_submit*>(ptr) ||
            dynamic_cast<const CSeq_id*>(ptr))
            m_Objects.push_back(*it);
    }
}

class COpenEditingBtnsViewTask : public CAppTask
{
public:
    COpenEditingBtnsViewTask(IServiceLocator* srv_locator, const TConstScopedObjects& obj_group);

    virtual ETaskState    x_Run();

private:
    IServiceLocator*    m_SrvLocator;
    TConstScopedObjects m_Objects;
};

COpenEditingBtnsViewTask::COpenEditingBtnsViewTask(IServiceLocator* srv_locator, const TConstScopedObjects& objects)
    : CAppTask(NcbiEmptyString, false), m_SrvLocator(srv_locator), m_Objects(objects)
{
}

IAppTask::ETaskState COpenEditingBtnsViewTask::x_Run()
{
    CIRef<CProjectService> prj_srv = m_SrvLocator->GetServiceByType<CProjectService>();
    CIRef<IProjectView> view = prj_srv->AddProjectView("Sequin Buttons", m_Objects, 0);

    if (view) {
      CIRef<IWindowManagerService> wm_srv = m_SrvLocator->GetServiceByType<IWindowManagerService>();
      wm_srv->MoveToFloatingFrame(*view);
    }

    return eCompleted;
}

IAppTask* CEditingBtnsTool::GetTask()
{
    return new COpenEditingBtnsViewTask(m_SrvLocator, m_Objects);
}

END_NCBI_SCOPE
