/*  $Id: bulk_other_add_panel.cpp 27620 2013-03-13 19:03:35Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>
#include <objects/seqfeat/Seq_feat.hpp>
#include <objects/seqfeat/SeqFeatData.hpp>
#include <objects/seqfeat/Imp_feat.hpp>
#include <objmgr/bioseq_ci.hpp>

////@begin includes
////@end includes

#include <gui/packages/pkg_sequence_edit/bulk_other_add_panel.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_util.hpp>
#include <gui/objutils/cmd_create_feat.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <wx/stattext.h>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

USING_SCOPE(ncbi::objects);

/*!
 * CBulkOtherAddPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBulkOtherAddPanel, CBulkCmdPanel )


/*!
 * CBulkOtherAddPanel event table definition
 */

BEGIN_EVENT_TABLE( CBulkOtherAddPanel, CBulkCmdPanel )

////@begin CBulkOtherAddPanel event table entries
////@end CBulkOtherAddPanel event table entries

END_EVENT_TABLE()


/*!
 * CBulkOtherAddPanel constructors
 */

CBulkOtherAddPanel::CBulkOtherAddPanel()
{
    Init();
}

CBulkOtherAddPanel::CBulkOtherAddPanel( wxWindow* parent, CSeq_entry_Handle seh, string key, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
: m_TopSeqEntry(seh), m_DefaultKey(key)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CBulkOtherAddPanel creator
 */

bool CBulkOtherAddPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBulkOtherAddPanel creation
    CBulkCmdPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CBulkOtherAddPanel creation
    return true;
}


/*!
 * CBulkOtherAddPanel destructor
 */

CBulkOtherAddPanel::~CBulkOtherAddPanel()
{
////@begin CBulkOtherAddPanel destruction
////@end CBulkOtherAddPanel destruction
}


/*!
 * Member initialisation
 */

void CBulkOtherAddPanel::Init()
{
////@begin CBulkOtherAddPanel member initialisation
    m_FeatureType = NULL;
    m_Locus = NULL;
    m_GeneDesc = NULL;
    m_Comment = NULL;
    m_Location = NULL;
////@end CBulkOtherAddPanel member initialisation
}


static bool s_IsRarelyUsedOrDiscouragedFeatureType(int subtype)
{
    if (subtype == CSeqFeatData::eSubtype_conflict
        || subtype == CSeqFeatData::eSubtype_old_sequence
        || subtype == CSeqFeatData::eSubtype_prim_transcript
        || subtype == CSeqFeatData::eSubtype_STS
        || subtype == CSeqFeatData::eSubtype_unsure) {
        return true;
    } else {
        return false;
    }
}


/*!
 * Control creation for CBulkOtherAddPanel
 */

void CBulkOtherAddPanel::CreateControls()
{    
////@begin CBulkOtherAddPanel content construction
    CBulkOtherAddPanel* itemCBulkCmdPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCBulkCmdPanel1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText( itemCBulkCmdPanel1, wxID_STATIC, _("Feature Type"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxArrayString m_FeatureTypeStrings;
    m_FeatureType = new wxListBox( itemCBulkCmdPanel1, ID_LISTBOX, wxDefaultPosition, wxDefaultSize, m_FeatureTypeStrings, wxLB_SINGLE );
    itemBoxSizer2->Add(m_FeatureType, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer5 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemCBulkCmdPanel1, wxID_STATIC, _("Gene Symbol"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Locus = new wxTextCtrl( itemCBulkCmdPanel1, ID_TEXTCTRL11, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(m_Locus, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemCBulkCmdPanel1, wxID_STATIC, _("Gene Description"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemStaticText8, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_GeneDesc = new wxTextCtrl( itemCBulkCmdPanel1, ID_TEXTCTRL12, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(m_GeneDesc, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( itemCBulkCmdPanel1, wxID_STATIC, _("Comment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(itemStaticText10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Comment = new wxTextCtrl( itemCBulkCmdPanel1, ID_TEXTCTRL13, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer5->Add(m_Comment, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Location = new CBulkLocationPanel( itemCBulkCmdPanel1, ID_WINDOW1, wxDefaultPosition, wxSize(100, 100), wxSIMPLE_BORDER );
    itemBoxSizer2->Add(m_Location, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

////@end CBulkOtherAddPanel content construction

    int pos = 0;
    int sel = -1;
    m_FeatureType->Append(wxT("Gene"));
    if (NStr::EqualNocase(m_DefaultKey, "Gene")) {
        sel = pos;
    }
    pos++;
    const CFeatList* feat_list = CSeqFeatData::GetFeatList();
    CFeatList::const_iterator it = feat_list->begin();
    while (it != feat_list->end()) {
        if (it->GetType() == CSeqFeatData::e_Imp 
            && it->GetSubtype() != CSeqFeatData::eSubtype_any
            && it->GetSubtype() != CSeqFeatData::eSubtype_imp
            && it->GetSubtype() != CSeqFeatData::eSubtype_Imp_CDS
            && !s_IsRarelyUsedOrDiscouragedFeatureType(it->GetSubtype())) {
            string key = it->GetStoragekey();
            m_FeatureType->Append(ToWxString(key));
            if (NStr::EqualNocase(m_DefaultKey, key)) {
                sel = pos;
            }
            pos++;
        }
        ++it;
    }
    if (sel > -1) {
        m_FeatureType->SetSelection(sel);
    }
}


/*!
 * Should we show tooltips?
 */

bool CBulkOtherAddPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CBulkOtherAddPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBulkOtherAddPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBulkOtherAddPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CBulkOtherAddPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBulkOtherAddPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBulkOtherAddPanel icon retrieval
}

CRef<CCmdComposite> CBulkOtherAddPanel::GetCommand()
{
    string key = ToStdString(m_FeatureType->GetStringSelection());
    if (NStr::IsBlank(key)) {
        CRef<CCmdComposite> empty;
        return empty;
    }
    CRef<CCmdComposite> cmd (new CCmdComposite("Bulk Add Other Feature"));
    string comment = ToStdString(m_Comment->GetValue());
    string gene_symbol = ToStdString(m_Locus->GetValue());
    string gene_desc = ToStdString(m_GeneDesc->GetValue());

    objects::CBioseq_CI b_iter(m_TopSeqEntry, objects::CSeq_inst::eMol_na);
    for ( ; b_iter ; ++b_iter ) {
        objects::CSeq_entry_Handle seh = b_iter->GetSeq_entry_Handle();
        if (!m_Location->ShouldAddToExisting() && AlreadyHasFeature(*b_iter, key, comment)) {
            continue;
        }
        CRef<objects::CSeq_loc> loc = m_Location->GetLocForBioseq(*(b_iter->GetCompleteBioseq()));
        if (!loc) {
            CRef<CCmdComposite> empty;
            return empty;
        }
        if (!NStr::EqualNocase(key, "Gene")) {
            CRef<objects::CSeq_feat> feat(new objects::CSeq_feat());
            feat->SetData().SetImp().SetKey(key);
            if (!NStr::IsBlank(comment)) {
                feat->SetComment(comment);
            }
            feat->SetLocation().Assign(*loc);
            if (feat->GetLocation().IsPartialStart(objects::eExtreme_Biological)
                || feat->GetLocation().IsPartialStop(objects::eExtreme_Biological)) {
                feat->SetPartial(true);
            }
            cmd->AddCommand(*CRef<CCmdCreateFeat>(new CCmdCreateFeat(seh, *feat)));
        }
        if (!NStr::IsBlank(gene_symbol) || !NStr::IsBlank(gene_desc) || NStr::EqualNocase(key, "Gene")) {
            // create gene feature
            CRef<objects::CSeq_feat> new_gene(new objects::CSeq_feat());
            new_gene->SetData().SetGene().SetLocus(gene_symbol);
            new_gene->SetData().SetGene().SetDesc(gene_desc);
            if (NStr::EqualNocase(key, "Gene") && !NStr::IsBlank(comment)) {
                new_gene->SetComment(comment);
            }
            new_gene->SetLocation().Assign(*loc);
            if (new_gene->GetLocation().IsPartialStart(objects::eExtreme_Biological)
                || new_gene->GetLocation().IsPartialStop(objects::eExtreme_Biological)) {
                new_gene->SetPartial(true);
            }
            cmd->AddCommand(*CRef<CCmdCreateFeat>(new CCmdCreateFeat(seh, *new_gene)));
        }
    }

    return cmd;
}


string CBulkOtherAddPanel::GetErrorMessage()
{
    string key = ToStdString(m_FeatureType->GetStringSelection());
    if (NStr::IsBlank(key)) {
        return "You must select feature type!";
    }
    objects::CBioseq_CI b_iter(m_TopSeqEntry, objects::CSeq_inst::eMol_na);
    for ( ; b_iter ; ++b_iter ) {
        objects::CSeq_entry_Handle seh = b_iter->GetSeq_entry_Handle();
        CRef<objects::CSeq_loc> loc = m_Location->GetLocForBioseq(*(b_iter->GetCompleteBioseq()));
        if (!loc) {
            return "You must specify valid location values!";
        }
    }
    return "Unknown error";
}

END_NCBI_SCOPE
