/*  $Id: bulkfeatureadddlg.cpp 28171 2013-05-21 12:54:22Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>
#include <objects/seqfeat/Imp_feat.hpp>
#include <objects/seqfeat/Seq_feat.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objmgr/bioseq_handle.hpp>

////@begin includes
////@end includes

#include <gui/packages/pkg_sequence_edit/bulkfeatureadddlg.hpp>
#include <gui/packages/pkg_sequence_edit/rnanamepanel.hpp>
#include <gui/packages/pkg_sequence_edit/igspanel.hpp>
#include <gui/objutils/cmd_create_feat.hpp>
#include <gui/packages/pkg_sequence_edit/bulk_rna_add_panel.hpp>
#include <gui/packages/pkg_sequence_edit/bulk_other_add_panel.hpp>
#include <gui/packages/pkg_sequence_edit/bulkcdsaddpanel.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_util.hpp>

#include <gui/core/file_extensions.hpp>
#include <gui/core/open_objects_dlg.hpp>
#include <gui/core/ui_file_load_manager.hpp>
#include <gui/utils/object_loader.hpp>
#include <gui/utils/execute_unit.hpp>
#include <gui/utils/extension_impl.hpp>
#include <gui/utils/app_popup.hpp>
#include <gui/widgets/wx/async_call.hpp>

#include <wx/msgdlg.h>
#include <wx/textctrl.h>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

/*!
 * CBulkFeatureAddDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBulkFeatureAddDlg, CBulkCmdDlg )


/*!
 * CBulkFeatureAddDlg event table definition
 */

BEGIN_EVENT_TABLE( CBulkFeatureAddDlg, CBulkCmdDlg )

////@begin CBulkFeatureAddDlg event table entries
////@end CBulkFeatureAddDlg event table entries
    EVT_RADIOBUTTON( wxID_ANY, CBulkFeatureAddDlg::OnFeatureChoiceSelected )
    EVT_TEXT_URL( ID_FEATURE_TABLE_TEXT, CBulkFeatureAddDlg::OnTextClickUrl )

END_EVENT_TABLE()


/*!
 * CBulkFeatureAddDlg constructors
 */

CBulkFeatureAddDlg::CBulkFeatureAddDlg()
{
    Init();
}

CBulkFeatureAddDlg::CBulkFeatureAddDlg( wxWindow* parent, objects::CSeq_entry_Handle seh, 
                             CSourceRequirements::EWizardType wizard_type,
                             CSourceRequirements::EWizardSrcType src_type,
                             IWorkbench* workbench,
                             wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
: m_TopSeqEntry(seh),
  m_WizardType (wizard_type),
  m_SrcType (src_type),
  m_FeatureOptionsPanel (NULL),
  m_LoadTable(false),
  m_Workbench (workbench)
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CBulkFeatureAddDlg creator
 */

bool CBulkFeatureAddDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBulkFeatureAddDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    CBulkCmdDlg::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CBulkFeatureAddDlg creation
    return true;
}


/*!
 * CBulkFeatureAddDlg destructor
 */

CBulkFeatureAddDlg::~CBulkFeatureAddDlg()
{
////@begin CBulkFeatureAddDlg destruction
////@end CBulkFeatureAddDlg destruction
}


/*!
 * Member initialisation
 */

void CBulkFeatureAddDlg::Init()
{
////@begin CBulkFeatureAddDlg member initialisation
    m_FeatureChoice = NULL;
    m_FeatureOptions = NULL;
////@end CBulkFeatureAddDlg member initialisation
}


/*!
 * Control creation for CBulkFeatureAddDlg
 */

void CBulkFeatureAddDlg::CreateControls()
{    
////@begin CBulkFeatureAddDlg content construction
    CBulkFeatureAddDlg* itemCBulkCmdDlg1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCBulkCmdDlg1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemCBulkCmdDlg1, wxID_ANY, _("What do your sequences contain?"));
    m_FeatureChoice = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxHORIZONTAL);
    itemBoxSizer2->Add(m_FeatureChoice, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_FeatureOptions = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(m_FeatureOptions, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton6 = new wxButton( itemCBulkCmdDlg1, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemButton6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemCBulkCmdDlg1, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemButton7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CBulkFeatureAddDlg content construction

    wxRadioButton* r_btn;

    // add different options, depending on wizard type
    switch (m_WizardType) {
        case CSourceRequirements::eWizardType_viruses:
            m_CDSBtn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single coding region on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            m_CDSBtn->SetValue(false);
            m_FeatureChoice->Add(m_CDSBtn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);            
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single non-coding feature on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5); 
            break;
        case CSourceRequirements::eWizardType_d_loop:
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single D-loop on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);            
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single control region on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);            
            break;
        case CSourceRequirements::eWizardType_uncultured_samples:
            r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Single rRNA, ITS, or IGS on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);      
            r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Multiple rRNA, ITS, or IGS on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);      
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single coding region on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5); 
            break;
        case CSourceRequirements::eWizardType_rrna_its_igs:
            r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Single rRNA, ITS, or IGS on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);            
            r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Multiple rRNA, ITS, or IGS on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);      
            break;
        case CSourceRequirements::eWizardType_igs:
            r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Intergenic spacer"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
            break;
        case CSourceRequirements::eWizardType_standard:
        default:
            m_CDSBtn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single coding region on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            m_FeatureChoice->Add(m_CDSBtn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single RNA feature on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
            r_btn = new wxRadioButton( itemCBulkCmdDlg1, wxID_ANY, _("Single feature on every sequence"), wxDefaultPosition, wxDefaultSize, 0 );
            r_btn->SetValue(false);
            m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
            break;
    }
    r_btn = new wxRadioButton ( itemCBulkCmdDlg1, wxID_ANY, _("Import Table"), wxDefaultPosition, wxDefaultSize, 0 );
    r_btn->SetValue(false);
    m_FeatureChoice->Add(r_btn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    x_ShowFeatureChoice ();
}


/*!
 * Should we show tooltips?
 */

bool CBulkFeatureAddDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CBulkFeatureAddDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBulkFeatureAddDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBulkFeatureAddDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CBulkFeatureAddDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBulkFeatureAddDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBulkFeatureAddDlg icon retrieval
}


int  CBulkFeatureAddDlg::x_GetFeatureChoice()
{
    wxSizerItemList& itemList = m_FeatureChoice->GetChildren();
    if (itemList.size() == 0) {
        return 0;
    }
    size_t pos = 0, x = 0;
    while (x < itemList.size()) {
        wxRadioButton* r_btn = dynamic_cast<wxRadioButton*>(itemList[x]->GetWindow());
        if (r_btn) {
            if (r_btn->GetValue()) {
                return pos + 1;
            }
            pos++;
        }        
        x++;
    }
    return 0;
}


CRef<CCmdComposite> CBulkFeatureAddDlg::x_CreateImportFeatures(string key, string comment, bool skip_existing)
{
    string cmd_label = "Bulk Add " + key + " Features";
    CRef<CCmdComposite> cmd (new CCmdComposite(cmd_label));

    objects::CBioseq_CI b_iter(m_TopSeqEntry, objects::CSeq_inst::eMol_na);
    for ( ; b_iter ; ++b_iter ) {
        if (skip_existing && AlreadyHasFeature(*b_iter, key, comment)) {
            continue;
        }

        CRef<objects::CSeq_feat> new_feat(new objects::CSeq_feat());
        new_feat->SetData().SetImp().SetKey(key);
        if (!NStr::IsBlank(comment)) {
            new_feat->SetComment(comment);
        }
        CRef<objects::CSeq_id> new_feat_id(new objects::CSeq_id());
        new_feat_id->Assign(*(b_iter->GetCompleteBioseq()->GetId().front()));
        new_feat->SetLocation().SetInt().SetId(*new_feat_id);
        new_feat->SetLocation().SetInt().SetFrom(0);
        new_feat->SetLocation().SetInt().SetTo(b_iter->GetBioseqLength() - 1);
        objects::CSeq_entry_Handle seh = b_iter->GetSeq_entry_Handle();
        cmd->AddCommand(*CRef<CCmdCreateFeat>(new CCmdCreateFeat(seh, *new_feat)));
    }
    return cmd;
}


CRef<CCmdComposite> CBulkFeatureAddDlg::x_CreateDloops()
{
    return x_CreateImportFeatures ("D-loop", "", true);
}


CRef<CCmdComposite> CBulkFeatureAddDlg::x_CreateControlRegions()
{
    return x_CreateImportFeatures ("misc_feature", "control region", true);
}


CRef<CCmdComposite> CBulkFeatureAddDlg::GetCommand()
{
    CRef<CCmdComposite> cmd;

    if (m_LoadTable) {
        cmd = ImportFeaturesFromFile (m_Workbench, m_TopSeqEntry);
    } else if (m_FeatureOptionsPanel) {
        cmd = m_FeatureOptionsPanel->GetCommand();
    } else {
        int feature_type_choice = x_GetFeatureChoice();

        switch (m_WizardType) {
            case CSourceRequirements::eWizardType_d_loop:
                if (feature_type_choice == 1) {
                    cmd = x_CreateDloops();
                } else if (feature_type_choice == 2) {
                    cmd = x_CreateControlRegions();
                }
                break;
            default:
                break;
        }
    }

    return cmd;
}


string CBulkFeatureAddDlg::GetErrorMessage ()
{
    string error = "Unknown error";

    if (m_LoadTable) {
        error = "";
    } else if (m_FeatureOptionsPanel) {
        error = m_FeatureOptionsPanel->GetErrorMessage();
    }
    return error;
}


void CBulkFeatureAddDlg::x_ClearFeatureOptions ()
{
    // hide feature details
    if (m_FeatureOptions->GetItemCount() > 0) {
        size_t pos = 0;
        m_FeatureOptions->GetItem(pos)->DeleteWindows();
        m_FeatureOptions->Remove(pos);
        m_FeatureOptionsPanel = NULL;
    }
}


const wxString kFeatureTableInstructions = "\
You can import a GFF3 file, an NCBI 5-column feature table file, or a FASTA file with protein sequences for your coding regions.  Click OK to select and load the file.\r\n\
\r\n\
If you are importing a protein FASTA file, make sure that the sequence IDs for the protein sequences match the nucleotide sequences on which the coding regions should be created.\r\n\
\r\n\
Features tables for Influenza A and B sequences can be created here:\r\n\
http://www.ncbi.nlm.nih.gov/genomes/FLU/Database/annotation.cgi \r\n\
\r\n\
An NCBI 5-column feature table must be a plain text file. \r\n\
\r\n\
- The header line begins with >Feature lcl| \r\n\
\r\n\
- The text following \"lcl|\" must contain the sequence ID of the sequences in your records.\r\n\
  For example: >Feature lcl|abc-1\r\n\
  In this example, abc-1 is the sequence ID. \r\n\
\r\n\
- The table is composed of 5, tab-separated columns:\r\n\
  Column 1- nucleotide position of the start of the feature\r\n\
  Column 2- nucleotide location of the end of a feature\r\n\
  Column 3- feature type (gene, CDS, etc.)\r\n\
  Column 4- feature qualifier (note, product, etc.)\r\n\
  Column 5- qualifier value (for example: gag protein)\r\n\
\r\n\
- The columns in the table MUST be separated by tabs. \r\n\
  Use the Tab key on your keyboard to separate each column.\r\n\
  The qualifiers follow on lines starting with three tabs. \r\n\
\r\n\
- For more feature table format information: \r\n\
http://www.ncbi.nlm.nih.gov/Sequin/table.html#Table Layout \r\n\
\r\n\
- Questions about the feature table format? Write to: info@ncbi.nlm.nih.gov\r\n\
--------------------------------------------------\r\n\
Example feature table for 2 sequences:\r\n\
--------------------------------------------------\r\n\
>Feature lcl|abc-1\r\n\
24	1458	gene\r\n\
			gene	PB2\r\n\
24	1458	CDS\r\n\
			product	polymerase PB2\r\n\
>Feature lcl|abc-2\r\n\
4	985	gene\r\n\
			gene	M2\r\n\
4	29	CDS\r\n\
718	985\r\n\
			product	matrix protein 2\r\n\
4	762	gene\r\n\
			gene	M1\r\n\
4	762	CDS\r\n\
      product	matrix protein 1\r\n\
";


void CBulkFeatureAddDlg::x_AddLoadTableText ()
{
    wxTextCtrl* msg = new wxTextCtrl( this, ID_FEATURE_TABLE_TEXT, wxEmptyString, 
                                       wxDefaultPosition, wxSize(800, 200), 
                                       wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH|wxTE_AUTO_URL );
    m_FeatureOptions->Add (msg, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxRESERVE_SPACE_EVEN_IF_HIDDEN, 5);
    msg->SetLabel(kFeatureTableInstructions);
}


void CBulkFeatureAddDlg::x_ShowFeatureChoice ()
{
    int feature_type_choice = x_GetFeatureChoice();

    x_ClearFeatureOptions();

    if (feature_type_choice == m_FeatureChoice->GetChildren().size()) {
        x_AddLoadTableText();
        m_LoadTable = true;
    } else {
        m_LoadTable = false;
        switch (m_WizardType) {
            case CSourceRequirements::eWizardType_d_loop:
                // do nothing
                break;
            case CSourceRequirements::eWizardType_viruses:
                if (feature_type_choice == 1) {
                    m_FeatureOptionsPanel = new CBulkCDSAddPanel( this, m_TopSeqEntry, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                } else if (feature_type_choice == 2) {
                    m_FeatureOptionsPanel = new CVirusNonCodingFeaturesPanel (this, m_TopSeqEntry, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                }
                break;
            case CSourceRequirements::eWizardType_uncultured_samples:
                if (feature_type_choice == 1) {
                    m_FeatureOptionsPanel = new CRNANamePanel ( this, m_TopSeqEntry, m_WizardType, m_SrcType, false, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                } else if (feature_type_choice == 2) {
                    m_FeatureOptionsPanel = new CRNANamePanel ( this, m_TopSeqEntry, m_WizardType, m_SrcType, true, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                } else if (feature_type_choice == 3) {
                    m_FeatureOptionsPanel = new CBulkCDSAddPanel (this, m_TopSeqEntry, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                }
                break;
            case CSourceRequirements::eWizardType_rrna_its_igs:
                if (feature_type_choice == 1) {
                    m_FeatureOptionsPanel = new CRNANamePanel ( this, m_TopSeqEntry, m_WizardType, m_SrcType, false, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                } else if (feature_type_choice == 2) {
                    m_FeatureOptionsPanel = new CRNANamePanel ( this, m_TopSeqEntry, m_WizardType, m_SrcType, true, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                }
                break;
            case CSourceRequirements::eWizardType_igs:
                if (feature_type_choice == 1) {
                    m_FeatureOptionsPanel = new CIGSPanel ( this, m_TopSeqEntry, m_WizardType, m_SrcType, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                }
                break;
            case CSourceRequirements::eWizardType_standard:
            default:
                switch (feature_type_choice) {
                    case 1:
                        m_FeatureOptionsPanel = new CBulkCDSAddPanel( this, m_TopSeqEntry, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                        break;
                    case 2:
                        m_FeatureOptionsPanel = new CBulkRNAAddPanel( this, m_TopSeqEntry, wxID_ANY, wxDefaultPosition, wxDefaultSize);
                        break;
                    case 3:
                        m_FeatureOptionsPanel = new CBulkOtherAddPanel( this, m_TopSeqEntry, "", wxID_ANY, wxDefaultPosition, wxDefaultSize);
                        break;
                    case 4:
                        break;
                }
                break;
        }
        if (m_FeatureOptionsPanel) {
            m_FeatureOptions->Add (m_FeatureOptionsPanel, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxRESERVE_SPACE_EVEN_IF_HIDDEN, 5);
        }
    }
    Layout();
    Fit();
}


/*!
 * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON
 */


void CBulkFeatureAddDlg::OnFeatureChoiceSelected( wxCommandEvent& event )
{
    x_ShowFeatureChoice ();
}


void CBulkFeatureAddDlg::OnTextClickUrl(  wxTextUrlEvent& event )
{
    if (event.GetMouseEvent().ButtonDown()) {
        wxTextCtrl* text_ctrl = dynamic_cast<wxTextCtrl*>(event.GetEventObject());
        if (text_ctrl) {
            wxString str = text_ctrl->GetValue();
            wxString url = str.substr(event.GetURLStart(), event.GetURLEnd() - event.GetURLStart() + 1); 
            CAppPopup::PopupURL(ToStdString(url));
        }
    }
}


void AddSeqAnnotToSeqEntry(const objects::CSeq_annot* annot, objects::CSeq_entry_Handle seh, CRef<CCmdComposite> cmd)
{
    if (!cmd) {
        return;
    }
    if (annot->IsFtable()) {
        ITERATE (list<CRef<objects::CSeq_feat> >, feat_it, annot->GetData().GetFtable()) {            
            // where does this feature go?
            objects::CBioseq_Handle bsh = seh.GetScope().GetBioseqHandle (*((*feat_it)->GetLocation().GetId()));
            if (!bsh) {
                // do something about not being able to locate appropriate sequence
            } else {
                objects::CSeq_entry_Handle add_seh = bsh.GetSeq_entry_Handle();
                CRef<objects::CSeq_feat> new_feat(new objects::CSeq_feat());
                new_feat->Assign(**feat_it);
                cmd->AddCommand(*CRef<CCmdCreateFeat>(new CCmdCreateFeat(add_seh, *new_feat)));
            }
        }
    }
}


CRef<CCmdComposite> ImportFeaturesFromFile (IWorkbench* workbench, objects::CSeq_entry_Handle seh)
{
    CRef<CCmdComposite> cmd(NULL);
    CRef<CFileLoadManager> fileManager(new CFileLoadManager());

    // get all other file format from an Extension Point
    vector< CIRef<IFileFormatLoaderManager> > fileManagers;
    GetExtensionAsInterface("file_format_loader_manager", fileManagers);

    for(  size_t i = 0;  i < fileManagers.size();  ++i  )   {
        IFileFormatLoaderManager* mgr = fileManagers[i].GetPointer();
        string loader_id = mgr->GetFileLoaderId();
        if ("file_loader_gff" == loader_id
            || "file_loader_5col" == loader_id
            || "file_loader_fasta" == loader_id
            || "file_loader_asn" == loader_id) {
            fileManager->AddFileFormat(mgr);
        }
    }

    vector<CIRef<IUIToolManager> > loadManagers;
    loadManagers.push_back(CIRef<IUIToolManager>(fileManager.GetPointer()));

    COpenObjectsDlg dlg(NULL);
    dlg.SetSize(710, 480);

    dlg.SetRegistryPath("Dialogs.Edit.OpenTable");
    dlg.SetServiceLocator(workbench);
    dlg.SetManagers(loadManagers);

    if (dlg.ShowModal() == wxID_OK) {
        CIRef<IObjectLoader> object_loader(dlg.GetObjectLoader());
        if (!object_loader) {
            wxMessageBox(wxT("Failed to get object loader"), wxT("Error"),
                         wxOK | wxICON_ERROR);
        }
        else {
            IExecuteUnit* execute_unit = dynamic_cast<IExecuteUnit*>(object_loader.GetPointer());
            if (execute_unit) {
                if (!execute_unit->PreExecute())
                    return cmd;

                CExecuteUnitAsyncCall call(*execute_unit, wxT("Reading file(s)..."));
                CAsyncCall::Execute(call);
                if (!call.GetResult()) // Canceled
                    return cmd;

                if (!execute_unit->PostExecute())
                    return cmd;
            }
            bool has_nuc = false;
            bool has_prot = false;
            bool success = true;
            bool has_features = false;

            cmd.Reset(new CCmdComposite("Import Feature File"));
            const IObjectLoader::TObjects& objects = object_loader->GetObjects();
            ITERATE(IObjectLoader::TObjects, obj_it, objects) {
                const CObject& ptr = obj_it->GetObject();
                const objects::CSeq_annot* annot = dynamic_cast<const objects::CSeq_annot*>(&ptr);
                if (annot) {
                    if (has_nuc || has_prot) {
                         wxMessageBox(wxT("Please load only feature tables or only protein sequences"), wxT("Error"),
                                      wxOK | wxICON_ERROR);
                         success = false;
                         break;
                    } else {
                        AddSeqAnnotToSeqEntry(annot, seh, cmd);
                        has_features = true;
                    }
                } else {
                    const objects::CSeq_entry* entry = dynamic_cast<const objects::CSeq_entry*>(&ptr);
                    if (entry) {
                        if (has_features) {
                            wxMessageBox(wxT("Please load only feature tables or only protein sequences"), wxT("Error"),
                                          wxOK | wxICON_ERROR);
                            success = false;
                            break;
                        } else {
                            success = AddSeqEntryToSeqEntry(entry, seh, cmd, has_nuc, has_prot);
                            if (!success) {
                                break;
                            } else if (has_nuc) {
                                wxMessageBox(wxT("Please load only feature tables or only protein sequences"), wxT("Error"),
                                              wxOK | wxICON_ERROR);
                                success = false;
                                break;
                            }
                        }
                    } else {
                        wxMessageBox(wxT("Unable to recognize imported object"), wxT("Error"),
                                      wxOK | wxICON_ERROR);
                        success = false;
                        break;
                    }
                }
            }
            if (!success) {
                cmd.Reset(NULL);
            }
        }
    }
    return cmd;
}


void CBulkFeatureAddDlg::ChooseFeatureType(objects::CSeqFeatData::ESubtype subtype)
{
    wxSizerItemList& itemList = m_FeatureChoice->GetChildren();
    if (itemList.size() == 0) {
        return;
    }
    string search_str = "";
    switch (subtype) {
        case objects::CSeqFeatData::eSubtype_cdregion:
            search_str = "coding region";
            break;
        case objects::CSeqFeatData::eSubtype_rRNA:
            search_str = "RNA feature";
            break;
        default:
            search_str = "Single feature";
            break;
    }
    size_t pos = 0, x = 0;
    while (x < itemList.size()) {
        wxRadioButton* r_btn = dynamic_cast<wxRadioButton*>(itemList[x]->GetWindow());
        if (r_btn) {
            if (NStr::FindNoCase(ToStdString(r_btn->GetLabelText()), search_str) != string::npos) {
                r_btn->SetValue(true);
            } else {
                r_btn->SetValue(false);
            }
            pos++;
        }        
        x++;
    }
    
    x_ShowFeatureChoice();
}


END_NCBI_SCOPE
