/*  $Id: capitalization_abbreviation.cpp 27132 2012-12-26 20:23:14Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */

#include <ncbi_pch.hpp>
#include <objmgr/seqdesc_ci.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objects/seqfeat/BioSource.hpp>
#include <gui/packages/pkg_sequence_edit/capitalization_abbreviation.hpp>

BEGIN_NCBI_SCOPE
using namespace objects;

static const SStaticPair<const char*, const char*> map_state_to_abbrev[] = 
{
    {"alabama" , "AL"},
    {"alaska" , "AK"},
    {"american samoa" , "AS"},
    {"arizona" , "AZ"},
    {"arkansas" , "AR"},
    {"california" , "CA"},
    {"colorado" , "CO"},
    {"connecticut" , "CT"},
    {"delaware" , "DE"},
    {"district of columbia" , "DC"},
    {"florida" , "FL"},
    {"georgia" , "GA"},
    {"guam" , "GU"},
    {"hawaii" , "HI"},
    {"idaho" , "ID"},
    {"illinois" , "IL"},
    {"indiana" , "IN"},
    {"iowa" , "IA"},
    {"kansas" , "KS"},
    {"kentucky" , "KY"},
    {"louisiana" , "LA"},
    {"maine" , "ME"},
    {"maryland" , "MD"},
    {"massachusetts" , "MA"},
    {"michigan" , "MI"},
    {"minnesota" , "MN"},
    {"mississippi" , "MS"},
    {"missouri" , "MO"},
    {"montana" , "MT"},
    {"nebraska" , "NE"},
    {"nevada" , "NV"},
    {"new hampshire" , "NH"},
    {"new jersey" , "NJ"},
    {"new mexico" , "NM"},
    {"new york" , "NY"},
    {"north carolina" , "NC"},
    {"north dakota" , "ND"},
    {"northern mariana islands" , "MP"},
    {"ohio" , "OH"},
    {"oklahoma" , "OK"},
    {"oregon" , "OR"},
    {"pennsylvania" , "PA"},
    {"puerto rico" , "PR"},
    {"rhode island" , "RI"},
    {"south carolina" , "SC"},
    {"south dakota" , "SD"},
    {"tennessee" , "TN"},
    {"texas" , "TX"},
    {"utah" , "UT"},
    {"vermont" , "VT"},
    {"virgin islands" , "VI"},
    {"virginia" , "VA"},
    {"washington" , "WA"},
    {"west virginia" , "WV"},
    {"wisconsin" , "WI"},
    {"wyoming" , "WY"}
};


DEFINE_STATIC_ARRAY_MAP(TCStringPairsMap,k_state_abbrev, map_state_to_abbrev);

static const SStaticPair<const char*, const char*> set_abbreviation_list[] = 
{
    {"\\barabidopsis thaliana\\b","Arabidopsis thaliana"},
    {"\\badp\\b", "ADP" },
    {"\\batp\\b", "ATP" },
    {"\\bbac\\b", "BAC" },
    {"\\bcaenorhabditis elegans\\b", "Caenorhabditis elegans" },
    {"\\bcdna\\b", "cDNA" },
    {"\\bcdnas\\b", "cDNAs" },
    {"\\bcoa\\b", "CoA" },
    {"\\bcoi\\b", "COI" },
    {"\\bcoii\\b", "COII" },
    {"\\bdanio rerio\\b", "Danio rerio" },
    {"\\bdna\\b", "DNA" },
    {"\\bdrosophila melanogaster\\b", "Drosophila melanogaster" },
    {"\\bdsrna\\b", "dsRNA" },
    {"\\bescherichia coli\\b", "Escherichia coli" },
    {"\\bhiv\\b", "HIV" },
    {"\\bhiv\\-1\\b", "HIV-1" },
    {"\\bhiv\\-2\\b", "HIV-2" },
    {"\\bhnrna\\b", "hnRNA" },
    {"\\bhomo sapiens\\b", "Homo sapiens" },
    {"\\bmhc\\b", "MHC" },
    {"\\bmrna\\b", "mRNA" },
    {"\\bmtdna\\b", "mtDNA" },
    {"\\bmus musculus\\b", "Mus musculus" },
    {"\\bnadh\\b", "NADH" },
    {"\\bnov\\.\\b", "nov." },
    {"\\bnov\\.\\.\\b", "nov.." },
    {"\\bpcr\\b", "PCR" },
    {"\\brattus norvegicus\\b", "Rattus norvegicus" },
    {"\\brapd\\b", "RAPD" },
    {"\\brdna\\b", "rDNA" },
    {"\\brna\\b", "RNA" },
    {"\\brrna\\b", "rRNA" },
    {"\\brt\\-pcr\\b", "RT-PCR" },
    {"\\bsaccharomyces cerevisiae\\b", "Saccharomyces cerevisiae" },
    {"\\bscrna\\b", "scRNA" },
    {"\\bsiv\\-1\\b", "SIV-1" },
    {"\\bsnp\\b", "SNP"     },
    {"\\bsnps\\b", "SNPs"   },
    {"\\bsnrna\\b", "snRNA" },
    {"\\bsp\\.\\b", "sp." },
    {"\\bsp\\.\\.\\b", "sp.." },
    {"\\bssp\\.\\b", "ssp." },
    {"\\bssp\\.\\.\\b", "ssp.." },
    {"\\bssrna\\b", "ssRNA" },
    {"\\bsubsp\\.\\b", "subsp." },
    {"\\bsubsp\\.\\.\\b", "subsp.." },
    {"\\btrna\\b", "tRNA" },
    {"\\bvar\\.\\b", "var." },
    {"\\bvar\\.\\.\\b", "var.." },
    {"\\buk\\b", "UK" },
    {"\\busa\\b", "USA" },
    {"\\bU\\.S\\.A\\.\\b", "USA" },
    {"\\bU\\.S\\.A\\b", "USA" },
    {"\\bUnited States of America\\b", "USA" },
    {"\\b\\(hiv\\)\\b", "(HIV)" },
    {"\\b\\(hiv1\\)\\b", "(HIV1)" },
    {"\\b\\(hiv\\-1\\)\\b", "(HIV-1)"},

    {"\\bsp\\.$", "sp.." },
    {"\\bnov\\.$", "nov.." },
    {"\\bssp\\.$", "ssp.." },
    {"\\bvar\\.$", "var.." },
    {"\\bsubsp\\.$", "subsp.."},
    {"\0","\0"}
};




static const SStaticPair<const char*, const char*> set_short_words[] = 
{
    {"\\bA\\b", "a" },
    {"\\bAbout\\b", "about" },
    {"\\bAnd\\b", "and" },
    {"\\bAt\\b", "at" },
    {"\\bBut\\b", "but" },
    {"\\bBy\\b", "by" },
    {"\\bFor\\b", "for" },
    {"\\bIn\\b", "in" },
    {"\\bIs\\b", "is" },
    {"\\bOf\\b", "of" },
    {"\\bOn\\b", "on" },
    {"\\bOr\\b", "or" },
    {"\\bThe\\b", "the" },
    {"\\bTo\\b", "to" },
    {"\\bWith\\b", "with" },
    {"\0","\0"}
};


static const SStaticPair<const char*, const char*> set_country_fixes[] = 
{

    {"\\bchnia\\b", "China" },
    {"\\bpr china\\b", "P.R. China" },
    {"\\bprchina\\b", "P.R. China" },
    {"\\bp\\.r\\.china\\b", "P.R. China" },
    {"\\bp\\.r china\\b", "P.R. China" },
    {"\\bp\\, r\\, china\\b", "P.R. China" },
    {"\\brok\\b", "ROK" },
    {"\\brsa\\b", "RSA" },
    {"\\broc\\b", "ROC" },
    {"\\buae\\b", "UAE" },
    {"\\bK\\.S\\.A\\.\\b", "K.S.A." },
    {"\\bk\\. s\\. a\\.\\b", "K. S. A." },
    {"\\bksa\\b", "KSA" },
    {"\0","\0"}
};

static const SStaticPair<const char*, const char*> set_AffiliationShortWordList[] = 
{
    {"\\bAu\\b", "au" },
    {"\\bAux\\b", "aux" },
    {"\\bA La\\b", "a la" },
    {"\\bDe La\\b", "de la" },
    {"\\bDe\\b", "de" },
    {"\\bDel\\b", "del"},
    {"\\bDes\\b", "des" },
    {"\\bDu\\b", "du" },
    {"\\bEt\\b", "et" },
    {"\\bLa\\b", "la" },
    {"\\bLe\\b", "le" },
    {"\\bLes\\b", "les" },
    {"\\bRue\\b", "rue" },
    {"\\bPo Box\\b", "PO Box" },
    {"\\bPobox\\b", "PO Box" },
    {"\\bP\\.O box\\b", "P.O. Box" },
    {"\\bP\\.Obox\\b", "P.O. Box" },
    {"\\bY\\b", "y" },
    {"\0","\0"}
};

static const char* set_ordinal_endings[] = 
{
    "\\dth\\b",
    "\\dst\\b",
    "\\dnd\\b",
    "\\drd\\b",
    "\0"
};

static const SStaticPair<const char*, const char*> set_KnownAbbreviationList[] = 
{
    {"\\bpo box\\b", "PO Box" },
    {"\\bPobox\\b", "PO Box" },
    {"\\bP\\.O box\\b", "P.O. Box" },
    {"\\bP\\.Obox\\b", "P.O. Box" },
    {"\\bPO\\.Box\\b", "P.O. Box" },
    {"\\bPO\\. Box\\b", "P.O. Box" },
    {"\\bpr china\\b", "P.R. China"},
    {"\\bprchina\\b", "P.R. China" },
    {"\\bp\\.r\\.china\\b", "P.R. China" },
    {"\\bp\\.r china\\b", "P.R. China" },
    {"\\bp\\, r\\, china\\b", "P.R. China" },
    {"\\bp\\,r\\, china\\b", "P.R. China" },
    {"\\bp\\,r\\,china\\b", "P.R. China" },
    {"\0","\0"}  // end of array
};

void CCapitalizationAbbreviations::FixOrgNames(objects::CSeq_entry_Handle seh, string& result)
{
    vector<string> taxnames;
    FindOrgNames(seh,taxnames);
    for(vector<string>::const_iterator name=taxnames.begin(); name!=taxnames.end(); ++name)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( "\\b"+*name+"\\b", *name, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
}

void CCapitalizationAbbreviations::FindOrgNames(objects::CSeq_entry_Handle seh, vector<string>& taxnames)
{
    objects::CBioseq_CI b_iter(seh, objects::CSeq_inst::eMol_na);
    for ( ; b_iter ; ++b_iter ) 
    {
        objects::CSeqdesc_CI it (*b_iter, objects::CSeqdesc::e_Source);
        if (it) 
        {
            if (it->GetSource().IsSetTaxname()) 
                taxnames.push_back(it->GetSource().GetTaxname());
        }
    }
}

void CCapitalizationAbbreviations::InsertMissingSpacesAfterCommas(string& result)
{
    CRegexpUtil replacer( result );
    int num_replacements = replacer.Replace( "\\,(\\S)", ", $1", CRegexp::fCompile_default, CRegexp::fMatch_default, 0);
    replacer.GetResult().swap( result );
}

void CCapitalizationAbbreviations::InsertMissingSpacesAfterNo(string& result)
{
    CRegexpUtil replacer( result );
    int num_replacements = replacer.Replace( "No\\.(\\w)", "No. $1", CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
    replacer.GetResult().swap( result );
}

void CCapitalizationAbbreviations::FixCapitalizationInElement(string& result)
{
    result = NStr::ToLower(result);
    result.at(0) = toupper(result.at(0));
}

void CCapitalizationAbbreviations::FixShortWordsInElement(string& result)
{
    for (int pat=0; set_short_words[pat].first[0]!='\0'; ++pat)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( set_short_words[pat].first, set_short_words[pat].second, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
    result.at(0) = toupper(result.at(0));
}

void CCapitalizationAbbreviations::FixAbbreviationsInElement(string& result)
{
    for (int pat=0; set_abbreviation_list[pat].first[0]!='\0'; ++pat)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( set_abbreviation_list[pat].first, set_abbreviation_list[pat].second, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
}

void CCapitalizationAbbreviations::FindReplaceString_CountryFixes(string& result)
{
    for (int pat=0; set_country_fixes[pat].first[0] != '\0'; ++pat)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( set_country_fixes[pat].first, set_country_fixes[pat].second, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
}

void CCapitalizationAbbreviations::CapitalizeAfterApostrophe(string& input)
{
    string result;
    CRegexp pattern("\\'\\w");
    size_t start = 0;
    for (;;) {
        pattern.GetMatch(input, start, 0, CRegexp::fMatch_default, true);
        if (pattern.NumFound() > 0) {
            const int* rslt = pattern.GetResults(0);
            if (rslt[0] != start)
                result += input.substr(start,rslt[0]-start);
            string tmp = input.substr(rslt[0], rslt[1] - rslt[0]);
            result += NStr::ToUpper(tmp);
            start = rslt[1];
        } else {
            result += input.substr(start,input.length()-start);
            break;
        }
    }
    input = result;
}

void CCapitalizationAbbreviations::FixStateAbbreviationsInAffil(string& state)
{
    NStr::ReplaceInPlace (state, "  ", " ");
    NStr::TruncateSpacesInPlace (state);
    TCStringPairsMap::const_iterator found = k_state_abbrev.find(NStr::ToLower(state).c_str());
    if (found != k_state_abbrev.end())
        state = found->second;
}

void CCapitalizationAbbreviations::FixAffiliationShortWordsInElement(string& result)
{
    if (result.empty()) return;
    for (int pat=0; set_AffiliationShortWordList[pat].first[0]!='\0'; ++pat)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( set_AffiliationShortWordList[pat].first, set_AffiliationShortWordList[pat].second, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
    result.at(0) = toupper(result.at(0));
    // fix d'
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( "\\bD\\'", "d'", CRegexp::fCompile_default, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
        
        string temp;
        CRegexp pattern("\\bd\\'\\w");
        size_t start = 0;
        for (;;) {
            pattern.GetMatch(result, start, 0, CRegexp::fMatch_default, true);
            if (pattern.NumFound() > 0) {
                const int* rslt = pattern.GetResults(0);
                if (rslt[0] != start)
                    temp += result.substr(start,rslt[0]-start);
                string tmp = result.substr(rslt[0], rslt[1] - rslt[0]);
                tmp = NStr::ToUpper(tmp);
                tmp.at(0) = 'd';
                temp += tmp;
                start = rslt[1];
            } else {
                temp += result.substr(start,result.length()-start);
                break;
            }
        }
        result = temp;
    }
}

void CCapitalizationAbbreviations::FixOrdinalNumbers(string& result)
{
    for(int p = 0; set_ordinal_endings[p][0] != '\0'; ++p)
    {
        CRegexp pattern(set_ordinal_endings[p],CRegexp::fCompile_ignore_case);
        string temp;
        size_t start = 0;
        for (;;) {
            pattern.GetMatch(result, start, 0, CRegexp::fMatch_default, true);
            if (pattern.NumFound() > 0) {
                const int* rslt = pattern.GetResults(0);
                if (rslt[0] != start)
                    temp += result.substr(start,rslt[0]-start);
                string tmp = result.substr(rslt[0], rslt[1] - rslt[0]);
                tmp = NStr::ToLower(tmp);
                temp += tmp;
                start = rslt[1];
            } else {
                temp += result.substr(start,result.length()-start);
                break;
            }
        }
        result = temp;
    }
}

void CCapitalizationAbbreviations::FixKnownAbbreviationsInElement(string& result)
{
    if (result.empty()) return;
    for (int pat=0; set_KnownAbbreviationList[pat].first[0] != '\0' ; ++pat)
    {
        CRegexpUtil replacer( result );
        int num_replacements = replacer.Replace( set_KnownAbbreviationList[pat].first, set_KnownAbbreviationList[pat].second, CRegexp::fCompile_ignore_case, CRegexp::fMatch_default, 0);
        replacer.GetResult().swap( result );
    }
}

void CCapitalizationAbbreviations::CapitalizeSAfterNumber(string& result)
{
    CRegexpUtil replacer( result );
    int num_replacements = replacer.Replace( "(\\d)s\\b", "$1S", CRegexp::fCompile_default, CRegexp::fMatch_default, 0);
    replacer.GetResult().swap( result );
}

END_NCBI_SCOPE
