/*  $Id: editingbtnspanel.cpp 27853 2013-04-12 20:28:17Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin and Igor Filippov
 */


#include <ncbi_pch.hpp>


////@begin includes
////@end includes

#include <gui/objutils/cmd_factory.hpp>
#include <gui/objutils/cmd_del_bioseq.hpp>
#include <gui/objutils/cmd_del_bioseq_set.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objmgr/bioseq_handle.hpp>

#include <wx/sizer.h>
#include <wx/icon.h>
#include <wx/msgdlg.h>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/text_widget/text_item_panel.hpp>

#include <gui/packages/pkg_sequence_edit/editingbtnspanel.hpp>
#include <gui/packages/pkg_sequence_edit/remove_desc_dlg.hpp>
#include <gui/packages/pkg_sequence_edit/remove_sequences_from_alignments.hpp>
#include <gui/packages/pkg_sequence_edit/remove_features_dlg.hpp>
#include <gui/packages/pkg_sequence_edit/remove_structured_comment.hpp>
#include <gui/packages/pkg_sequence_edit/remove_annot.hpp>
#include <gui/packages/pkg_sequence_edit/remove_genomeprojectdb.hpp>
#include <gui/packages/pkg_sequence_edit/add_citsubupdate.hpp>
#include <gui/packages/pkg_sequence_edit/pubdesc_editor.hpp>
#include <gui/packages/pkg_sequence_edit/modify_structured_comment.hpp>
#include <gui/packages/pkg_sequence_edit/remove_bankit_comment.hpp>
#include <gui/packages/pkg_sequence_edit/remove_structuredcomment_keyword.hpp>
#include <gui/packages/pkg_sequence_edit/reorder_sequences_dlg.hpp>
#include <gui/packages/pkg_sequence_edit/convert_comment.hpp>
#include <gui/packages/pkg_sequence_edit/modify_feature_id.hpp>
#include <gui/packages/pkg_sequence_edit/pcr-primers-edit.hpp>
#include <gui/packages/pkg_sequence_edit/fix_orgmod_institution.hpp>
#include <gui/packages/pkg_sequence_edit/append_mod_to_org_dlg.hpp>
#include <gui/packages/pkg_sequence_edit/parse_collectiondate_formats.hpp>
#include <gui/packages/pkg_sequence_edit/country_fixup.hpp>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE
using namespace objects;


/*!
 * CEditingBtnsPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CEditingBtnsPanel, wxPanel )


/*!
 * CEditingBtnsPanel event table definition
 */

BEGIN_EVENT_TABLE( CEditingBtnsPanel, wxPanel )

////@begin CEditingBtnsPanel event table entries
    EVT_BUTTON( ID_BUTTON1, CEditingBtnsPanel::OnReverseAuthorNamesClick )

    EVT_BUTTON( ID_BUTTON2, CEditingBtnsPanel::OnTruncateAuthorMiddleInitials )

    EVT_BUTTON( ID_BUTTON3, CEditingBtnsPanel::OnFixUsaAndStates )

    EVT_BUTTON( ID_BUTTON4, CEditingBtnsPanel::OnFixCapitalizationAll )

    EVT_BUTTON( ID_BUTTON5, CEditingBtnsPanel::OnFixCapitalizationAuthors )

    EVT_BUTTON( ID_BUTTON6, CEditingBtnsPanel::OnFixCapitalizationTitles )

    EVT_BUTTON( ID_BUTTON7, CEditingBtnsPanel::OnFixCapitalizationAffiliation )

    EVT_BUTTON( ID_BUTTON8, CEditingBtnsPanel::OnFixCapitalizationCountry )

    EVT_BUTTON( ID_BUTTON9, CEditingBtnsPanel::OnRemoveUnpublishedPublications )

    EVT_BUTTON( ID_BUTTON10, CEditingBtnsPanel::OnRemovePublishedPublications )

    EVT_BUTTON( ID_BUTTON11, CEditingBtnsPanel::OnRemoveCollidingPublications )

    EVT_BUTTON( ID_BUTTON12, CEditingBtnsPanel::OnRemoveAllPublications )

    EVT_BUTTON( ID_BUTTON13, CEditingBtnsPanel::OnStripAuthorSuffixes )

    EVT_BUTTON( ID_BUTTON14, CEditingBtnsPanel::OnRemoveAuthorConsortiums )

    EVT_BUTTON( ID_BUTTON15, CEditingBtnsPanel::OnConvertAuthorToConsortiumWithStringConstraint )

    EVT_BUTTON( ID_BUTTON16, CEditingBtnsPanel::OnConvertAuthorToConstortiumWhereLastName )

    EVT_BUTTON( ID_BUTTON17, CEditingBtnsPanel::OnConvertAuthorToConsortiumAll )

    EVT_BUTTON( ID_BUTTON18, CEditingBtnsPanel::OnReloadPublications )

    EVT_BUTTON( ID_BUTTON19, CEditingBtnsPanel::OnAddCitSubForUpdate )

    EVT_BUTTON( ID_BUTTON20, CEditingBtnsPanel::OnRemoveSequences )

    EVT_BUTTON( ID_BUTTON21, CEditingBtnsPanel::OnRemoveGenomeProjectsDB )

    EVT_BUTTON( ID_BUTTON22, CEditingBtnsPanel::OnRemoveSeqAnnotAlign )

    EVT_BUTTON( ID_BUTTON23, CEditingBtnsPanel::OnRemoveSeqAnnotGraph )

    EVT_BUTTON( ID_BUTTON24, CEditingBtnsPanel::OnRemoveDuplicateStructuredComments )

    EVT_BUTTON( ID_BUTTON25, CEditingBtnsPanel::OnRemoveDescriptors )

    EVT_BUTTON( ID_BUTTON26, CEditingBtnsPanel::OnRemoveSeqAnnotIds )

    EVT_BUTTON( ID_BUTTON27, CEditingBtnsPanel::OnRemoveAllStructuredComments )

    EVT_BUTTON( ID_BUTTON28, CEditingBtnsPanel::OnRemoveEmptyStructuredComments )

    EVT_BUTTON( ID_BUTTON29, CEditingBtnsPanel::OnRemoveSequencesFromAlignments )

    EVT_BUTTON( ID_BUTTON30, CEditingBtnsPanel::OnRemoveFeatures )

    EVT_BUTTON( ID_BUTTON31, CEditingBtnsPanel::OnModifyStructuredComment )

    EVT_BUTTON( ID_BUTTON32, CEditingBtnsPanel::OnRemoveBankitComment )

    EVT_BUTTON( ID_BUTTON33, CEditingBtnsPanel::OnRemoveStructuredCommentKeyword )

    EVT_BUTTON( ID_BUTTON34, CEditingBtnsPanel::OnAddStructuredCommentKeyword )

    EVT_BUTTON( ID_BUTTON35, CEditingBtnsPanel::OnReorderStructuredComment )

    EVT_BUTTON( ID_BUTTON36, CEditingBtnsPanel::OnReorderSequences )

    EVT_BUTTON( ID_BUTTON37, CEditingBtnsPanel::OnConvertComment )

    EVT_BUTTON( ID_BUTTON38, CEditingBtnsPanel::OnParseComment )

    EVT_BUTTON( ID_BUTTON39, CEditingBtnsPanel::OnAssignFeatureIds )

    EVT_BUTTON( ID_BUTTON40, CEditingBtnsPanel::OnClearFeatureIds )

    EVT_BUTTON( ID_BUTTON41, CEditingBtnsPanel::OnReassignFeatureIds )

    EVT_BUTTON( ID_BUTTON42, CEditingBtnsPanel::OnUniqifyFeatureIds )

    EVT_BUTTON( ID_BUTTON43, CEditingBtnsPanel::OnTrimJunkInPrimerSeqs )

    EVT_BUTTON( ID_BUTTON44, CEditingBtnsPanel::OnFixiInPrimerSeqs )

    EVT_BUTTON( ID_BUTTON45, CEditingBtnsPanel::OnFixOrgModInstitution )

    EVT_BUTTON( ID_BUTTON46, CEditingBtnsPanel::OnSwapPrimerSeqName )

    EVT_BUTTON( ID_BUTTON47, CEditingBtnsPanel::OnMergePrimerSets )

    EVT_BUTTON( ID_BUTTON48, CEditingBtnsPanel::OnSplitPrimerSets )

    EVT_BUTTON( ID_BUTTON49, CEditingBtnsPanel::OnAppendModToOrg )

    EVT_BUTTON( ID_BUTTON50, CEditingBtnsPanel::OnParseCollectionDateMonthFirst )

    EVT_BUTTON( ID_BUTTON51, CEditingBtnsPanel::OnParseCollectionDateDayFirst )

    EVT_BUTTON( ID_BUTTON52, CEditingBtnsPanel::OnCountryFixupCap )

    EVT_BUTTON( ID_BUTTON53, CEditingBtnsPanel::OnCountryFixupNoCap )
////@end CEditingBtnsPanel event table entries

END_EVENT_TABLE()


/*!
 * CEditingBtnsPanel constructors
 */

CEditingBtnsPanel::CEditingBtnsPanel()
{
    Init();
}

CEditingBtnsPanel::CEditingBtnsPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CEditingBtnsPanel creator
 */

bool CEditingBtnsPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CEditingBtnsPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CEditingBtnsPanel creation
    return true;
}


/*!
 * CEditingBtnsPanel destructor
 */

CEditingBtnsPanel::~CEditingBtnsPanel()
{
////@begin CEditingBtnsPanel destruction
////@end CEditingBtnsPanel destruction
}


/*!
 * Member initialisation
 */

void CEditingBtnsPanel::Init()
{
////@begin CEditingBtnsPanel member initialisation
////@end CEditingBtnsPanel member initialisation
}


/*!
 * Control creation for CEditingBtnsPanel
 */

void CEditingBtnsPanel::CreateControls()
{    
////@begin CEditingBtnsPanel content construction
    // Generated by DialogBlocks, 26/10/2012 15:32:03 (unregistered)

    CEditingBtnsPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer3 = new wxFlexGridSizer(0, 4, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton1 = new wxButton( itemPanel1, ID_BUTTON1, _("Reverse last and first name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton1, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton2 = new wxButton( itemPanel1, ID_BUTTON2, _("Truncate Author Middle Initials"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton2, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton3 = new wxButton( itemPanel1, ID_BUTTON3, _("Fix USA and States"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton3, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton4 = new wxButton( itemPanel1, ID_BUTTON4, _("Fix Capitalization"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton4, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton5 = new wxButton( itemPanel1, ID_BUTTON5, _("Fix Capitalization in Authors"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton5, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton6 = new wxButton( itemPanel1, ID_BUTTON6, _("Fix Capitalization in Titles"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton6, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemPanel1, ID_BUTTON7, _("Fix Capitalization in Affiliations"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton7, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemPanel1, ID_BUTTON8, _("Fix Capitalization in Country"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton8, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton( itemPanel1, ID_BUTTON9, _("Remove Unpublished Publications"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton9, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton10 = new wxButton( itemPanel1, ID_BUTTON10, _("Remove Published Publications"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton10, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton11 = new wxButton( itemPanel1, ID_BUTTON11, _("Remove Colliding Publications"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton11, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton12 = new wxButton( itemPanel1, ID_BUTTON12, _("Remove All Publications"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton12, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton13 = new wxButton( itemPanel1, ID_BUTTON13, _("Strip Author Suffixes"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton13, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton14 = new wxButton( itemPanel1, ID_BUTTON14, _("Remove Author Consortiums"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton14, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton15 = new wxButton( itemPanel1, ID_BUTTON15, _("Convert Author to Consortium with String Constraint"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton15, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton16 = new wxButton( itemPanel1, ID_BUTTON16, _("Convert Author to Consortium where last name contains Consortium"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton16, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton17 = new wxButton( itemPanel1, ID_BUTTON17, _("Convert Author to Consortium All"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton17, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton18 = new wxButton( itemPanel1, ID_BUTTON18, _("Reload Publications"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton18, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton19 = new wxButton( itemPanel1, ID_BUTTON19, _("Add Cit-sub for update"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton19, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton20 = new wxButton( itemPanel1, ID_BUTTON20, _("Remove Sequences"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton20, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton21 = new wxButton( itemPanel1, ID_BUTTON21, _("Remove GenomeProjectsDB"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton21, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton22 = new wxButton( itemPanel1, ID_BUTTON22, _("Remove Alignments"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton22, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton23 = new wxButton( itemPanel1, ID_BUTTON23, _("Remove Graphs"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton23, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton24 = new wxButton( itemPanel1, ID_BUTTON24, _("Remove Duplicate Structured Comments"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton24, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton25 = new wxButton( itemPanel1, ID_BUTTON25, _("Remove Descriptors"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton25, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton26 = new wxButton( itemPanel1, ID_BUTTON26, _("Remove Seq_annot Ids"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton26, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton27 = new wxButton( itemPanel1, ID_BUTTON27, _("Remove All Structured Comments"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton27, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton28 = new wxButton( itemPanel1, ID_BUTTON28, _("Remove Empty Structured Comments"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton28, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton29 = new wxButton( itemPanel1, ID_BUTTON29, _("Remove Sequences From Alignments"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton29, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton30 = new wxButton( itemPanel1, ID_BUTTON30, _("Remove Features"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton30, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton31 = new wxButton( itemPanel1, ID_BUTTON31, _("Remove Structured Comment Field"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton31, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton32 = new wxButton( itemPanel1, ID_BUTTON32, _("Remove Bankit Comment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton32, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton33 = new wxButton( itemPanel1, ID_BUTTON33, _("Remove Structured Comment Keyword"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton33, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton34 = new wxButton( itemPanel1, ID_BUTTON34, _("Add Structured Comment Keyword"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton34, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton35 = new wxButton( itemPanel1, ID_BUTTON35, _("Reorder Structured Comment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton35, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton36 = new wxButton( itemPanel1, ID_BUTTON36, _("Reorder Sequences"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton36, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton37 = new wxButton( itemPanel1, ID_BUTTON37, _("Convert Comment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton37, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton38 = new wxButton( itemPanel1, ID_BUTTON38, _("Parse Comment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton38, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton39 = new wxButton( itemPanel1, ID_BUTTON39, _("Assign Feature Ids"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton39, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton40 = new wxButton( itemPanel1, ID_BUTTON40, _("Clear Feature Ids"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton40, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton41 = new wxButton( itemPanel1, ID_BUTTON41, _("Reassign Feature Ids"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton41, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton42 = new wxButton( itemPanel1, ID_BUTTON42, _("Uniqify Feature Ids"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton42, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton43 = new wxButton( itemPanel1, ID_BUTTON43, _("Trim Junk In Primer Seqs"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton43, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton44 = new wxButton( itemPanel1, ID_BUTTON44, _("Fix i In Primer Seqs"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton44, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton45 = new wxButton( itemPanel1, ID_BUTTON45, _("Fix OrgMod Institution"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton45, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton46 = new wxButton( itemPanel1, ID_BUTTON46, _("Swap Primer Seq and Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton46, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton47 = new wxButton( itemPanel1, ID_BUTTON47, _("Merge Primer Sets"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton47, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton48 = new wxButton( itemPanel1, ID_BUTTON48, _("Split Primer Sets By Position"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton48, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton49 = new wxButton( itemPanel1, ID_BUTTON49, _("Append Mod To Org"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton49, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton50 = new wxButton( itemPanel1, ID_BUTTON50, _("Parse Collection Date Month First"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton50, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton51 = new wxButton( itemPanel1, ID_BUTTON51, _("Parse Collection Date Day First"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton51, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton52 = new wxButton( itemPanel1, ID_BUTTON52, _("Country Fixup Fix Capitalization After Colon"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton52, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton53 = new wxButton( itemPanel1, ID_BUTTON53, _("Country Fixup Do Not Fix Capitalization After Colon"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemButton53, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
////@end CEditingBtnsPanel content construction
}



/*!
 * Transfer data to the window
 */

bool CEditingBtnsPanel::TransferDataToWindow()
{
    m_TopSeqEntry.Reset();
    m_SeqSubmit.Reset();
    if (m_InputObjects) {
        NON_CONST_ITERATE (TConstScopedObjects, it, *m_InputObjects) {
            const CObject* ptr = it->object.GetPointer();

            /// CSeq_entry
            const objects::CSeq_entry* seqEntry = dynamic_cast<const objects::CSeq_entry*>(ptr);
            objects::CSeq_entry_Handle seh;
            if (seqEntry) {
                seh = it->scope->GetSeq_entryHandle (*seqEntry);
            } else {
                const objects::CSeq_submit* seqsubmit = dynamic_cast<const objects::CSeq_submit*>(ptr);
                if (seqsubmit) {
                    m_SeqSubmit = const_cast<objects::CSeq_submit*>(seqsubmit);
                    if (seqsubmit->IsEntrys()) {
                        seh = it->scope->GetSeq_entryHandle(*(seqsubmit->GetData().GetEntrys().front()));
                    }
                }
            } 
            if (seh) {
                m_TopSeqEntry = seh;
            }
        }
    }
    return wxPanel::TransferDataToWindow();
}


/*!
 * Should we show tooltips?
 */

bool CEditingBtnsPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CEditingBtnsPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CEditingBtnsPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CEditingBtnsPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CEditingBtnsPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CEditingBtnsPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CEditingBtnsPanel icon retrieval
}


void CEditingBtnsPanel::OnDataChange()
{
    
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
 */

void CEditingBtnsPanel::OnReverseAuthorNamesClick( wxCommandEvent& event )
{
    CReverseAuthorNames worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Reverse Author Last and First Name"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON2
 */

void CEditingBtnsPanel::OnTruncateAuthorMiddleInitials( wxCommandEvent& event )
{
    CFixAuthorInitials worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Truncate Author Middle Initials"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON3
 */

void CEditingBtnsPanel::OnFixUsaAndStates( wxCommandEvent& event )
{
    CFixUSA worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix USA and States"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON4
 */

void CEditingBtnsPanel::OnFixCapitalizationAll( wxCommandEvent& event )
{
    CFixAllCap worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix All Capitalization"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON5
 */

void CEditingBtnsPanel::OnFixCapitalizationAuthors( wxCommandEvent& event )
{
    CFixAuthorCap worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix Author Capitalization"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON6
 */

void CEditingBtnsPanel::OnFixCapitalizationTitles( wxCommandEvent& event )
{
    CFixTitleCap worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix Title Capitalization"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON7
 */

void CEditingBtnsPanel::OnFixCapitalizationAffiliation( wxCommandEvent& event )
{
    CFixAffilCap worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix Affiliation Capitalization"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON8
 */

void CEditingBtnsPanel::OnFixCapitalizationCountry( wxCommandEvent& event )
{
    CFixCountryCap worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix Country Capitalization"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON9
 */

void CEditingBtnsPanel::OnRemoveUnpublishedPublications( wxCommandEvent& event )
{
    CRemoveUnpublishedPubs worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Delete Unpublished Publications"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON10
 */

void CEditingBtnsPanel::OnRemovePublishedPublications( wxCommandEvent& event )
{
    CRemovePublishedPubs worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Delete Published Publications"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON11
 */

void CEditingBtnsPanel::OnRemoveCollidingPublications( wxCommandEvent& event )
{
    CRemoveCollidingPubs worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Delete Colliding Publications"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON12
 */

void CEditingBtnsPanel::OnRemoveAllPublications( wxCommandEvent& event )
{
    CRemoveAllPubs worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Delete Colliding Publications"))
        event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON13
 */

void CEditingBtnsPanel::OnStripAuthorSuffixes( wxCommandEvent& event )
{
  CStripAuthorSuffix worker;
  if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Strip Author Suffixes"))
    event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON14
 */

void CEditingBtnsPanel::OnConvertAuthorToConsortiumWithStringConstraint( wxCommandEvent& event ) // TODO
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON14 in CEditingBtnsPanel.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON14 in CEditingBtnsPanel. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON15
 */

void CEditingBtnsPanel::OnConvertAuthorToConstortiumWhereLastName( wxCommandEvent& event )
{
  CConvertAuthorToConsortiumWhereConstraint worker;
  worker.SetConstraint("consortium");
  if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Convert Author to Consortiums Where Last Name Contains Consortium"))
    event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON16
 */

void CEditingBtnsPanel::OnConvertAuthorToConsortiumAll( wxCommandEvent& event )
{
  CConvertAllAuthorToConsortium worker;
  if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Convert All Authors To Consortiums"))
    event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON17
 */

void CEditingBtnsPanel::OnRemoveAuthorConsortiums( wxCommandEvent& event )
{
  CRemoveAuthorConsortium worker;
  if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Author Consortiums"))
    event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON18
 */

void CEditingBtnsPanel::OnReloadPublications( wxCommandEvent& event )
{
    CReloadPublications worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Reload Publications"))
        event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON19
 */

void CEditingBtnsPanel::OnAddCitSubForUpdate( wxCommandEvent& event )
{
    CAddCitSubForUpdate worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Add Cit-sub for update"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON20
 */

void CEditingBtnsPanel::OnRemoveSequences( wxCommandEvent& event )
{
    CRemoveSequencesCommandConverter* sequence_remover = new CRemoveSequencesCommandConverter();
    CTblEditDlg dlg(this, m_TopSeqEntry, sequence_remover, NULL, SYMBOL_CTBLEDITDLG_IDNAME, SYMBOL_CTBLEDITDLG_TITLE, SYMBOL_CTBLEDITDLG_POSITION, SYMBOL_CTBLEDITDLG_SIZE, SYMBOL_CTBLEDITDLG_STYLE,
                    wxT("Please select the sequences you would like to remove"));

    bool done = false;
    while (!done) {
        if (dlg.ShowModal() == wxID_OK) {
            sequence_remover->SetGrid(dlg.GetGrid());
            CRef<CCmdComposite> cmd = dlg.GetCommand();
            if (cmd) {
                m_CmdProccessor->Execute(cmd);                       
                dlg.Close();        
                done = true;
            } else {
                wxString error = ToWxString(dlg.GetErrorMessage());
                wxMessageBox(error, wxT("Error"),
                         wxOK | wxICON_ERROR, this);
            }
        } else {
            done = true;
        }
    }
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON21
 */

void CEditingBtnsPanel::OnRemoveGenomeProjectsDB( wxCommandEvent& event )
{
    CRemoveGenomeProjectsDB worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove GenomeProjectsDB"))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON22
 */

void CEditingBtnsPanel::OnRemoveSeqAnnotAlign( wxCommandEvent& event )
{
    CRemoveSeqAnnot worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Alignments",CSeq_annot::C_Data::e_Align))
        event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON23
 */

void CEditingBtnsPanel::OnRemoveSeqAnnotGraph( wxCommandEvent& event )
{
    CRemoveSeqAnnot worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Graphs",CSeq_annot::C_Data::e_Graph))
        event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON26
 */

void CEditingBtnsPanel::OnRemoveSeqAnnotIds( wxCommandEvent& event )
{
    CRemoveSeqAnnot worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Seq-Annot Ids",CSeq_annot::C_Data::e_Ids))
        event.Skip();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON24
 */

void CEditingBtnsPanel::OnRemoveDuplicateStructuredComments( wxCommandEvent& event )
{
    CRemoveDuplicateStructuredComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Duplicate Structured Comment"))
        event.Skip();
}

void CEditingBtnsPanel::OnRemoveDescriptors( wxCommandEvent& event )
{  
    CRemoveDescDlg dlg(NULL, m_TopSeqEntry);
    if (dlg.ShowModal() == wxID_OK) 
    {
        CRef<CCmdComposite> cmd = dlg.GetCommand();
        if (cmd)  m_CmdProccessor->Execute(cmd);                       
        dlg.Close();        
    }
}  


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON27
 */

void CEditingBtnsPanel::OnRemoveAllStructuredComments( wxCommandEvent& event )
{
    CRemoveAllStructuredComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove All Structured Comments"))
        event.Skip();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON28
 */

void CEditingBtnsPanel::OnRemoveEmptyStructuredComments( wxCommandEvent& event )
{
    CRemoveEmptyStructuredComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Empty Structured Comments"))
        event.Skip();
}

void CEditingBtnsPanel::OnRemoveSequencesFromAlignments( wxCommandEvent& event )
{  
    CRemoveSeqFromAlignDlg dlg(NULL, m_TopSeqEntry);
    if (dlg.ShowModal() == wxID_OK) 
    {
        CRef<CCmdComposite> cmd = dlg.GetCommand();
        if (cmd)  m_CmdProccessor->Execute(cmd);                       
        dlg.Close();        
    }
}  

void CEditingBtnsPanel::OnRemoveFeatures( wxCommandEvent& event )
{  
    CRemoveFeaturesDlg dlg(NULL, m_TopSeqEntry);
    if (dlg.ShowModal() == wxID_OK) 
    {
        CRef<CCmdComposite> cmd = dlg.GetCommand();
        if (cmd)
        {
            m_CmdProccessor->Execute(cmd);                       
        }
        dlg.Close();        
    }
}  

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON31
 */

void CEditingBtnsPanel::OnModifyStructuredComment( wxCommandEvent& event )
{
    CModifyStructuredComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Structured Comment Field"))
        event.Skip();
}

void CEditingBtnsPanel::OnRemoveBankitComment( wxCommandEvent& event )
{
    CRemoveBankitComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Bankit Comment"))
        event.Skip();
}

void CEditingBtnsPanel::OnRemoveStructuredCommentKeyword( wxCommandEvent& event )
{
    CRemoveStructuredCommentKeyword worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Remove Structured Comment Keyword"))
        event.Skip();
}

void CEditingBtnsPanel::OnAddStructuredCommentKeyword( wxCommandEvent& event )
{
    CAddStructuredCommentKeyword worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Add Structured Comment Keyword"))
        event.Skip();
}

void CEditingBtnsPanel::OnReorderStructuredComment( wxCommandEvent& event )
{
    CReorderStructuredComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Reorder Structured Comment"))
        event.Skip();
}

void CEditingBtnsPanel::OnReorderSequences( wxCommandEvent& event )
{  
    CReorderSequencesDlg dlg(NULL, m_TopSeqEntry);
    if (dlg.ShowModal() == wxID_OK) 
    {
        CRef<CCmdComposite> cmd = dlg.GetCommand();
        if (cmd)  m_CmdProccessor->Execute(cmd);                       
        dlg.Close();        
    }
}  

void CEditingBtnsPanel::OnConvertComment( wxCommandEvent& event )
{
    CConvertComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Convert Comment to Structured Comment", "::","  "))
        event.Skip();
}

void CEditingBtnsPanel::OnParseComment( wxCommandEvent& event )
{
    CConvertComment worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Parse Comment to Structured Comment", "=",";"))
        event.Skip();
}

void CEditingBtnsPanel::OnAssignFeatureIds( wxCommandEvent& event )
{
    CAssignFeatureId worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Assign Feature Ids"))
        event.Skip();
}

void CEditingBtnsPanel::OnClearFeatureIds( wxCommandEvent& event )
{
    CClearFeatureId worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Clear Feature Ids"))
        event.Skip();
}

void CEditingBtnsPanel::OnReassignFeatureIds( wxCommandEvent& event )
{
    CReassignFeatureId worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Reassign Feature Ids"))
        event.Skip();
}

void CEditingBtnsPanel::OnUniqifyFeatureIds( wxCommandEvent& event )
{
    CUniqifyFeatureId worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Uniqify Feature Ids"))
        event.Skip();
}

void CEditingBtnsPanel::OnTrimJunkInPrimerSeqs( wxCommandEvent& event )
{
    CPCRPrimerTrimJunk worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Trim Junk In Primer Seqs"))
        event.Skip();
}

void CEditingBtnsPanel::OnFixiInPrimerSeqs( wxCommandEvent& event )
{
    CPCRPrimerFixI worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix i In Primer Seqs"))
        event.Skip();
}

void CEditingBtnsPanel::OnFixOrgModInstitution( wxCommandEvent& event )
{
    CFixOrgModInst worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Fix OrgMod Institution"))
        event.Skip();
}

void CEditingBtnsPanel::OnSwapPrimerSeqName( wxCommandEvent& event )
{
    CSwapPrimerNameSeq worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Swap Primer Seq and Name"))
        event.Skip();
}

void CEditingBtnsPanel::OnMergePrimerSets( wxCommandEvent& event )
{
    CMergePrimerSets worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Merge Primer Sets"))
        event.Skip();
}

void CEditingBtnsPanel::OnSplitPrimerSets( wxCommandEvent& event )
{
    CSplitPrimerSets worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Split Primer Sets By Position"))
        event.Skip();
}

void CEditingBtnsPanel::OnAppendModToOrg( wxCommandEvent& event )
{  
    CAppendModToOrgDlg dlg(NULL, m_TopSeqEntry);
    if (dlg.ShowModal() == wxID_OK) 
    {
        CRef<CCmdComposite> cmd = dlg.GetCommand();
        if (cmd)  m_CmdProccessor->Execute(cmd);                       
        dlg.Close();        
    }
}  

void CEditingBtnsPanel::OnParseCollectionDateMonthFirst( wxCommandEvent& event )
{
    CParseCollectionDateFormats worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Parse Collection Date Month First",true))
        event.Skip();
}

void CEditingBtnsPanel::OnParseCollectionDateDayFirst( wxCommandEvent& event )
{
    CParseCollectionDateFormats worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Parse Collection Date Day First",false))
        event.Skip();
}

void CEditingBtnsPanel::OnCountryFixupCap( wxCommandEvent& event )
{
    CCountryFixup worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Country Fixup Fix Capitalization After Colon",true))
        event.Skip();
}

void CEditingBtnsPanel::OnCountryFixupNoCap( wxCommandEvent& event )
{
    CCountryFixup worker;
    if (!worker.apply(m_TopSeqEntry,m_CmdProccessor,"Country Fixup Do Not Fix Capitalization After Colon",false))
        event.Skip();
}


END_NCBI_SCOPE


