/*  $Id: remove_features_dlg.cpp 27321 2013-01-25 21:23:09Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <objmgr/scope.hpp>
#include <objects/misc/sequence_macros.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objmgr/seq_feat_handle.hpp>
#include <objmgr/util/feature.hpp>
#include <gui/objutils/util_cmds.hpp>
#include <gui/packages/pkg_sequence_edit/remove_features_dlg.hpp>

#include <util/xregexp/regexp.hpp>

#include <wx/button.h>
#include <wx/statbox.h>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE



IMPLEMENT_DYNAMIC_CLASS( CRemoveFeaturesDlg, wxDialog )


BEGIN_EVENT_TABLE( CRemoveFeaturesDlg, wxDialog )


END_EVENT_TABLE()

CRemoveFeaturesDlg::CRemoveFeaturesDlg()
{
    Init();
}

CRemoveFeaturesDlg::CRemoveFeaturesDlg( wxWindow* parent, objects::CSeq_entry_Handle seh, 
                                        wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
: m_TopSeqEntry(seh)
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


bool CRemoveFeaturesDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();

    SetSize(wxSize(280, 750));
    return true;
}


CRemoveFeaturesDlg::~CRemoveFeaturesDlg()
{
}


/*!
 * Member initialisation
 */

void CRemoveFeaturesDlg::Init()
{
    const CFeatList* feat_list = CSeqFeatData::GetFeatList();
    ITERATE(CFeatList, ft_it, *feat_list) 
    {
        const CFeatListItem& item = *ft_it;
        string desc = item.GetDescription();
        int feat_type = item.GetType();
        int feat_subtype = item.GetSubtype();
        m_Types[desc] = pair<int,int>(feat_type,feat_subtype);
    }
        
    m_ListCtrl = NULL;
}




void CRemoveFeaturesDlg::CreateControls()
{    
    CRemoveFeaturesDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_ListCtrl = new wxListCtrl( itemDialog1, ID_LISTCTRL_FTR, wxDefaultPosition, wxSize(250, 550), wxLC_REPORT );
    itemBoxSizer3->Add(m_ListCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ListCtrl->InsertColumn(0,"    Feature",wxLIST_FORMAT_LEFT,237);
    int i = 0;
    for (map<string,pair<int,int> >::iterator e = m_Types.begin(); e != m_Types.end(); e++)
    {
        m_ListCtrl->InsertItem(i, ToWxString(e->first));
        i++;
    }

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer5->Add(itemBoxSizer6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer7Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Optional String Constraint"));
    wxStaticBoxSizer* itemStaticBoxSizer7 = new wxStaticBoxSizer(itemStaticBoxSizer7Static, wxVERTICAL);
    itemBoxSizer6->Add(itemStaticBoxSizer7, 0, wxALIGN_LEFT|wxALL, 5);

    m_RadioButton = new wxRadioButton( itemDialog1, ID_RADIOBUTTON, _("Remove When Text Is Present"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
    m_RadioButton->SetValue(true);
    itemStaticBoxSizer7->Add(m_RadioButton, 0, wxALIGN_LEFT|wxALL, 5);

    wxRadioButton* itemRadioButton9 = new wxRadioButton( itemDialog1, ID_RADIOBUTTON1, _("Remove When Text Is Not Present"));
    itemRadioButton9->SetValue(false);
    itemStaticBoxSizer7->Add(itemRadioButton9, 0, wxALIGN_LEFT|wxALL, 5);

    m_TextCtrl = new wxTextCtrl( itemDialog1, ID_TEXTCTRL, wxEmptyString, wxDefaultPosition, wxSize(237, -1), 0 );
    itemStaticBoxSizer7->Add(m_TextCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    m_CheckBox = new wxCheckBox( itemDialog1, ID_CHECKBOX, _("Case Insensitive"), wxDefaultPosition, wxDefaultSize, 0 );
    m_CheckBox->SetValue(false);
    itemStaticBoxSizer7->Add(m_CheckBox, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer15Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Optional Seq-id Constraint"));
    wxStaticBoxSizer* itemStaticBoxSizer15 = new wxStaticBoxSizer(itemStaticBoxSizer15Static, wxVERTICAL);
    itemBoxSizer6->Add(itemStaticBoxSizer15, 0, wxALIGN_LEFT|wxALL, 5);

    m_RadioButton_id = new wxRadioButton( itemDialog1, ID_RADIOBUTTON2, _("Remove When Text Is Present"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
    m_RadioButton_id->SetValue(true);
    itemStaticBoxSizer15->Add(m_RadioButton_id, 0, wxALIGN_LEFT|wxALL, 5);

    wxRadioButton* itemRadioButton16 = new wxRadioButton( itemDialog1, ID_RADIOBUTTON3, _("Remove When Text Is Not Present"));
    itemRadioButton16->SetValue(false);
    itemStaticBoxSizer15->Add(itemRadioButton16, 0, wxALIGN_LEFT|wxALL, 5);

    m_TextCtrl_id = new wxTextCtrl( itemDialog1, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(237, -1), 0 );
    itemStaticBoxSizer15->Add(m_TextCtrl_id, 0, wxALIGN_LEFT|wxALL, 5);

    m_CheckBox_id = new wxCheckBox( itemDialog1, ID_CHECKBOX1, _("Case Insensitive"), wxDefaultPosition, wxDefaultSize, 0 );
    m_CheckBox->SetValue(false);
    itemStaticBoxSizer15->Add(m_CheckBox_id, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer17Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Optional Location Constraint"));
    wxStaticBoxSizer* itemStaticBoxSizer17 = new wxStaticBoxSizer(itemStaticBoxSizer17Static, wxVERTICAL);
    itemBoxSizer6->Add(itemStaticBoxSizer17, 0, wxALIGN_LEFT|wxALL, 5);

    wxArrayString strands,types;
    strands.Add(wxT("Any Strand"));
    strands.Add(wxT("Plus Strand"));
    strands.Add(wxT("Minus Strand"));
    types.Add(wxT("Nucleotide and protein sequences"));
    types.Add(wxT("Nucleotide sequences only"));
    types.Add(wxT("Protein sequences only"));

    m_ChoiceStrand = new wxChoice(itemDialog1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, strands,0,wxDefaultValidator,"Strand");
    m_ChoiceStrand->SetSelection(0);
    itemStaticBoxSizer17->Add(m_ChoiceStrand, 0, wxALIGN_LEFT|wxALL, 5);
    m_ChoiceType = new wxChoice(itemDialog1, ID_CHOICE2, wxDefaultPosition, wxDefaultSize, types,0,wxDefaultValidator,"Type");
    m_ChoiceType->SetSelection(0);
    itemStaticBoxSizer17->Add(m_ChoiceType, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer12 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer12, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton13 = new wxButton( itemDialog1, wxID_OK, _("Accept"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer12->Add(itemButton13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton14 = new wxButton( itemDialog1, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer12->Add(itemButton14, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
}


/*!
 * Should we show tooltips?
 */

bool CRemoveFeaturesDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CRemoveFeaturesDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
}

/*!
 * Get icon resources
 */

wxIcon CRemoveFeaturesDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
}

CRef<CCmdComposite> CRemoveFeaturesDlg::GetCommand()
{
    long item = -1;
    CRef<CCmdComposite> cmd( new CCmdComposite("Remove Features") );
    if (m_ListCtrl && m_TopSeqEntry)
        for ( ;; )
        {
            item = m_ListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
            if ( item == -1 )  break;
            string name = ToStdString(m_ListCtrl->GetItemText(item));
            ApplyToCSeq_entry (m_TopSeqEntry, cmd, name);
        }
    return cmd;
}


/*

  OperateOnSeqEntryConstrainedObjects
  ConstrainedBioseqCallback
  DoesIDListMeetStringConstraint
  DoesStringMatchConstraintX      --- this seems to compare seq-id for feature with a text constraint

    VisitBioseqsInSep() -> calls
    BioseqConstrainedFeaturesCallback() <void BioseqConstrainedFeaturesCallback (BioseqPtr bsp,Pointer userdata) at sequin5.c:11438>:
        SeqEntryConstrainedFeaturesCallback() <void SeqEntryConstrainedFeaturesCallback (SeqFeatPtr sfp,Pointer userdata) at sequin5.c:11390>: - filters by type/subtype
            SeqMgrGetDesiredFeature() - not sure what it does, searches through a sorted array of feature pointers?
            DoesObjectMatchStringConstraint() <Boolean DoesObjectMatchStringConstraint (ObjMgrTypePtr omtp,AsnIoPtr aip,Pointer ptr,ObjectHasStringPtr ohsp) at sequin5.c:11024>: - matches string with feat->label?
                SeqMgrFeaturesAreIndexed()
                SeqMgrIndexFeatures()
                SeqMgrGetDesiredFeature()
                DoesStringMatchConstraintX() <Boolean DoesStringMatchConstraintX (CharPtr pchSource,StringConstraintXPtr scp) at sequin5.c:10844>: - string matching: whole/starts with...etc.
                   
           
            DoesLocationMatchConstraint() <Boolean DoesLocationMatchConstraint (SeqLocPtr slp,LocationConstraintXPtr lcp) at sequin5.c:10386>:
                DoesStrandMatchConstraint() <Boolean DoesStrandMatchConstraint (SeqLocPtr slp,LocationConstraintXPtr lcp) at sequin5.c:10313>:
                    SeqLocStrand()
                DoesSequenceTypeMatchContraint() <Boolean DoesSequenceTypeMatchContraint (SeqLocPtr slp,LocationConstraintXPtr lcp) at sequin5.c:10355>:
                    BioseqFindFromSeqLoc()
                    ISA_na() 
                    ISA_aa() 
                SeqLocStop() 
                SeqLocStart()

FeatureRemoveOrConvertAction -  TODO
 RemoveProteinProducts (rfd.bsplist, mrfp->input_entityID, sep);  -- bsplist populated by RemoveFeatureCallback
        RemoveCDNANucProtProducts (rfd.bssplist, mrfp->input_entityID);
        rfd.bsplist = ValNodeFree (rfd.bsplist);
        rfd.bssplist = ValNodeFree (rfd.bssplist);
  
*/


void CRemoveFeaturesDlg::ApplyToCSeq_entry (objects::CSeq_entry_Handle tse, CCmdComposite* composite, string name)  // see SeqEntryConstrainedFeaturesCallback
{
    int type = m_Types[name].first;
    int subtype = m_Types[name].second;
    
    CFeat_CI feat (tse, SAnnotSelector(CSeqFeatData::E_Choice(type)));
    while (feat) 
    { 
        if (feat->GetOriginalFeature().CanGetData() && feat->GetOriginalFeature().GetData().GetSubtype() == subtype)
        {
            bool go = true;
            if (m_TextCtrl->GetLineLength(0) > 0)
            {
                go = false;
                string str;
                feature::GetLabel(feat->GetOriginalFeature(),&str,feature::fFGL_Both,&tse.GetScope());
                CRegexp pattern(ToStdString(m_TextCtrl->GetLineText(0))); 
                if (m_CheckBox->GetValue())
                    pattern.Set(ToStdString(m_TextCtrl->GetLineText(0)),CRegexp::fCompile_ignore_case);
                bool match = pattern.IsMatch(str);
                if (match && m_RadioButton->GetValue()) go = true;
                else if (!match && !m_RadioButton->GetValue()) go = true;
            }
            if (go && m_TextCtrl_id->GetLineLength(0) > 0)
            {
                go = false;
                if (feat->GetSeq_feat_Handle().GetAnnot().GetParentEntry().IsSeq())
                {
                    string str;
                    try
                    {
                        CConstRef<CSeq_id>  id = feat->GetSeq_feat_Handle().GetAnnot().GetParentEntry().GetSeq().GetSeqId();
                        id->GetLabel(&str);
                    } catch(...) {}
                    if (!str.empty())
                    {
                        CRegexp pattern(ToStdString(m_TextCtrl_id->GetLineText(0))); 
                        if (m_CheckBox_id->GetValue())
                            pattern.Set(ToStdString(m_TextCtrl_id->GetLineText(0)),CRegexp::fCompile_ignore_case);
                        bool match = pattern.IsMatch(str);
                        if (match && m_RadioButton_id->GetValue()) go = true;
                        else if (!match && !m_RadioButton_id->GetValue()) go = true;
                    }
                }
            }

            if (go && m_ChoiceStrand->GetSelection()>0)
            {
                go = false;
                int n = m_ChoiceStrand->GetSelection();
                wxString strand = m_ChoiceStrand->GetString(n);
                if (strand == wxT("Plus Strand") && feat->GetOriginalFeature().CanGetLocation() && feat->GetOriginalFeature().GetLocation().GetStrand() == eNa_strand_plus) go = true;
                if (strand == wxT("Minus Strand") && feat->GetOriginalFeature().CanGetLocation() && feat->GetOriginalFeature().GetLocation().GetStrand() == eNa_strand_minus) go = true;
            }

            if (go && m_ChoiceType->GetSelection()>0)
            {
                go = false;
                int n = m_ChoiceType->GetSelection();
                wxString type = m_ChoiceType->GetString(n);
                if (type == wxT("Nucleotide sequences only") && feat->GetSeq_feat_Handle().GetAnnot().GetParentEntry().GetSeq().IsNa()) go = true;
                if (type == wxT("Protein sequences only") &&  feat->GetSeq_feat_Handle().GetAnnot().GetParentEntry().GetSeq().IsAa()) go = true;
            }
            if (go)
            {
                CSeq_feat_Handle fh = feat->GetSeq_feat_Handle();
                if (fh)
                {
                    CRef<CCmdComposite> cmd = GetDeleteFeatureCommand(fh);
                    composite->AddCommand(*cmd);
                }
            }
        }
        ++feat;
    }
}


END_NCBI_SCOPE
