/*  $Id: remove_sequences.cpp 27319 2013-01-25 20:37:22Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */


#include <ncbi_pch.hpp>
#include <objects/misc/sequence_macros.hpp>

#include <objmgr/seq_entry_ci.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <gui/objutils/util_cmds.hpp>

#include <gui/packages/pkg_sequence_edit/tbl_edit_dlg.hpp>
#include <gui/packages/pkg_sequence_edit/remove_sequences.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_util.hpp>

#include <objects/seq/MolInfo.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(ncbi::objects);



CRef<objects::CSeq_table> CRemoveSequencesCommandConverter::GetValuesTableFromSeqEntry(CSeq_entry_Handle seh)
{
    CRef<objects::CSeq_table> table(new objects::CSeq_table());
    CRef<objects::CSeqTable_column> id_col(new objects::CSeqTable_column());
    id_col->SetHeader().SetField_id(objects::CSeqTable_column_info::eField_id_location_id);
    id_col->SetHeader().SetTitle("Sequence ID");
    table->SetColumns().push_back(id_col);

    CRef<objects::CSeqTable_column> length_col;
    length_col = new objects::CSeqTable_column();
    length_col->SetHeader().SetTitle("Length");
    length_col->SetHeader().SetField_name("length");
    length_col->SetData().SetString();
    table->SetColumns().push_back(length_col);

    size_t row = 0;
    if (seh)
    {
        objects::CBioseq_CI b_iter(seh, objects::CSeq_inst::eMol_na);                
        for ( ; b_iter ; ++b_iter ) {
            CRef<objects::CSeq_id> id(new objects::CSeq_id());
            id->Assign (*(b_iter->GetSeqId()));
            id_col->SetData().SetId().push_back(id);
            if (length_col) {
                TSeqPos length = b_iter->GetBioseqLength();
                length_col->SetData().SetString().push_back(NStr::IntToString(length));
            }
            row++;
        }            
    }
    table->SetNum_rows(row);
    return table;
}


CRef<CCmdComposite> CRemoveSequencesCommandConverter::GetCommandFromValuesTable(CRef<CSeq_table> values_table, CSeq_entry_Handle seh)
{
    CRef<CCmdComposite> cmd( new CCmdComposite("Remove Sequences") );
    
    CRef<CCmdComposite> cmd_null;
    if (!m_Grid || !m_Grid->IsSelection()) return cmd_null;
    
    const wxArrayInt& rows_control = m_Grid->GetSelectedRows();  // Only GetSelectedRows() is not enough - see http://trac.wxwidgets.org/changeset/54665    
    const wxGridCellCoordsArray& rows_shift_top = m_Grid->GetSelectionBlockTopLeft();
    const wxGridCellCoordsArray& rows_shift_bottom = m_Grid->GetSelectionBlockBottomRight();
    set<int> rows(rows_control.begin(),rows_control.end());

    for (int i = 0; i<rows_shift_top.size(); i++)
        for (int j = rows_shift_top[i].GetRow(); j <= rows_shift_bottom[i].GetRow(); j++)
            rows.insert(j);

    size_t num_rows = rows.size();

    CRef<objects::CSeqTable_column> id_col = FindSeqTableColumnByName (values_table, "Sequence ID");
    if (!id_col) return cmd_null;

    bool modified = false;
    for (set<int>::iterator row = rows.begin(); row != rows.end(); row++)
        if (*row < values_table->GetNum_rows() && *row < id_col->GetData().GetSize())
        { 
            CRef<CSeq_id> id(new objects::CSeq_id());
            id->Assign (*(id_col->GetData().GetId()[*row]));
            CBioseq_Handle bsh = seh.GetBioseqHandle(*id);
            cmd->AddCommand(*GetDeleteSequenceCommand(bsh));
            modified = true;
        }  
    if (!modified) return cmd_null;
    
    // send composite command
    return cmd;
}



string CRemoveSequencesCommandConverter::GetErrorMessage(CRef<CSeq_table>)
{
    return "Nothing selected or the only Bioseq on record";
}

END_NCBI_SCOPE
