/*  $Id: seq_table_grid.cpp 27955 2013-04-30 17:54:28Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>
#include <wx/sizer.h>
#include <wx/msgdlg.h>
#include <wx/dc.h>
#include <wx/settings.h>
#include <wx/dcclient.h>

#include <wx/textdlg.h>
#include <wx/tokenzr.h>
#include <wx/clipbrd.h>

////@begin includes
////@end includes

#include <gui/objutils/interface_registry.hpp>
#include <gui/packages/pkg_sequence_edit/seq_table_grid.hpp>
#include <gui/packages/pkg_sequence_edit/srceditdialog.hpp>
#include <gui/packages/pkg_sequence_edit/molinfoedit_util.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_util.hpp>
#include <gui/packages/pkg_sequence_edit/rename_column_dlg.hpp>
#include <gui/widgets/wx/ui_command.hpp>
#include <gui/widgets/wx/message_box.hpp>



////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

USING_SCOPE(ncbi::objects);

/*!
 * CSeqTableGridPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSeqTableGridPanel, wxPanel )


/*!
 * CSeqTableGridPanel event table definition
 */

BEGIN_EVENT_TABLE( CSeqTableGridPanel, wxPanel )

////@begin CSeqTableGridPanel event table entries
    EVT_GRID_CELL_RIGHT_CLICK( CSeqTableGridPanel::OnCellRightClick )
    EVT_GRID_LABEL_RIGHT_CLICK( CSeqTableGridPanel::OnLabelRightClick )

////@end CSeqTableGridPanel event table entries
    EVT_GRID_CELL_LEFT_CLICK( CSeqTableGridPanel::OnCellLeftClick )
    EVT_GRID_LABEL_LEFT_CLICK( CSeqTableGridPanel::OnLabelLeftClick )
    EVT_GRID_LABEL_LEFT_DCLICK( CSeqTableGridPanel::OnLabelLeftDClick )
    EVT_GRID_COL_SORT( CSeqTableGridPanel::OnSortTableByColumn )
    EVT_MENU( eCmdCopyTableValues, CSeqTableGridPanel::OnCopyCells )
    EVT_MENU( eCmdPasteTableValues, CSeqTableGridPanel::OnPasteCells )
    EVT_MENU( eCmdAppendTableValues, CSeqTableGridPanel::OnPasteAppendCells )
    EVT_MENU( eCmdCopyTableValuesFromId, CSeqTableGridPanel::OnCopyCellsFromId )
    EVT_MENU( eCmdSearchTable, CSeqTableGridPanel::OnSearchTable )
    EVT_MENU( eCmdDeleteTableCol, CSeqTableGridPanel::OnDeleteColumn )
    EVT_MENU( eCmdRenameTableCol, CSeqTableGridPanel::OnRenameColumn )
END_EVENT_TABLE()


/*!
 * CSeqTableGridPanel constructors
 */

CSeqTableGridPanel::CSeqTableGridPanel()
{
    Init();
}

CSeqTableGridPanel::CSeqTableGridPanel( wxWindow* parent, CRef<CSeq_table> values_table, CRef<CSeq_table> choices,
                                        wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
    : m_Table(values_table), m_Choices(choices), m_Copied (NULL)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CSeqTableGrid creator
 */

bool CSeqTableGridPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSeqTableGridPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSeqTableGridPanel creation
    return true;
}


/*!
 * CSeqTableGridPanel destructor
 */

CSeqTableGridPanel::~CSeqTableGridPanel()
{
////@begin CSeqTableGridPanel destruction
////@end CSeqTableGridPanel destruction
}


/*!
 * Member initialisation
 */

void CSeqTableGridPanel::Init()
{
////@begin CSeqTableGridPanel member initialisation
////@end CSeqTableGridPanel member initialisation
    x_SetUpMenu();
    m_SortByRowLabel = false;
    m_SortByRowLabelAscend = false;
}


/*!
 * Control creation for CSeqTableGrid
 */

void CSeqTableGridPanel::CreateControls()
{    
////@begin CSeqTableGridPanel content construction
    CSeqTableGridPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxGrid* itemGrid3 = new wxGrid( itemPanel1, ID_GRID, wxDefaultPosition, wxSize(600, 300), wxSUNKEN_BORDER|wxHSCROLL|wxVSCROLL );
    itemGrid3->SetDefaultColSize(50);
    itemGrid3->SetDefaultRowSize(25);
    itemGrid3->SetColLabelSize(25);
    itemGrid3->SetRowLabelSize(100);
    itemGrid3->CreateGrid(5, 5, wxGrid::wxGridSelectCells);
    itemBoxSizer2->Add(itemGrid3, 0, wxGROW|wxALL, 5);

////@end CSeqTableGridPanel content construction
    m_Grid = itemGrid3;
    CSeqTableGrid *gridAdapter = new CSeqTableGrid(m_Table);
    m_Grid->SetTable(gridAdapter, true);
    x_MakeProblemsColumnReadOnly();
    SetColumnSizesAndChoices();

    m_Grid->Connect(wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(CSeqTableGridPanel::CopyPasteDataByKeyboard), (wxObject*) NULL, this);  // connect m_Grid to ctrl-v/ctrl-c event handlers
}


void CSeqTableGridPanel::MakeColumnReadOnly(int pos, bool val)
{
    if (pos < m_Grid->GetNumberCols()) {
        for (int i = 0; i < m_Grid->GetNumberRows(); i++) {
            m_Grid->GetOrCreateCellAttr(i, pos)->SetReadOnly(val);
        }
    }
}


void CSeqTableGridPanel::MakeColumnReadOnly(string name, bool val)
{
    int pos = m_Grid->GetNumberCols();
    for (int i = 0; i < m_Grid->GetNumberCols(); i++) {
        if (NStr::EqualNocase(ToStdString(m_Grid->GetColLabelValue(i)), name)) {
            pos = i;
            break;
        }
    }

    MakeColumnReadOnly(pos, val);
}


void CSeqTableGridPanel::MakeAllColumnsReadOnly()
{
    for (int c = 0; c < m_Grid->GetNumberCols(); c++) {
        for (int i = 0; i < m_Grid->GetNumberRows(); i++) {
            m_Grid->GetOrCreateCellAttr(i, c)->SetReadOnly(true);
        }
    }
}


void CSeqTableGridPanel::x_MakeProblemsColumnReadOnly()
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if (!gridAdapter) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
    if (!values_table) {
        return;
    }

    int problem_pos = m_Grid->GetNumberCols();
    for (int i = 0; i < m_Grid->GetNumberCols(); i++) {
        if (NStr::EqualNocase(ToStdString(m_Grid->GetColLabelValue(i)), kProblems)) {
            problem_pos = i;
            break;
        }
    }
    MakeColumnReadOnly(problem_pos);
}


void CSeqTableGridPanel::UpdateColumnLabelHeight()
{
    int lines = 1;
    for (int i = 0; i < m_Grid->GetNumberCols(); i++) {
        string label = ToStdString(m_Grid->GetColLabelValue(i));
        size_t pos = NStr::Find(label, "\n");
        int lines_this_column = 1;
        while (pos != string::npos) {
            lines_this_column++;
            pos = NStr::Find(label, "\n", pos + 1);
        }
        if (lines_this_column > lines) {
            lines = lines_this_column;
        }            
    }
    int l_height = m_Grid->GetColLabelSize();
    m_Grid->SetColLabelSize( lines * l_height );
}


void CSeqTableGridPanel::UpdateColumnChoices(int pos, vector<string> choices)
{
    if (!m_Choices) {
        if (choices.size() == 0) {
            // no choices before, no new choices, nothing to do
            return;
        }
        m_Choices = new objects::CSeq_table();
        CRef<objects::CSeqTable_column> col(new objects::CSeqTable_column());
        m_Choices->SetColumns().push_back(col);
    }

    while (pos >= m_Choices->GetColumns().size()) {
        CRef<objects::CSeqTable_column> col(new objects::CSeqTable_column());
        m_Choices->SetColumns().push_back(col);
    }
    m_Choices->SetColumns()[pos]->ResetData();
    if (choices.size() > 0) {
        CRef<objects::CSeqTable_column> col = m_Choices->SetColumns()[pos];
        ITERATE (vector<string>, it, choices) {
            col->SetData().SetString().push_back(*it);
        }
        wxArrayString val_list;
        size_t max_len = 10;
        wxClientDC dc(this);
        for (size_t j = 0; j < col->GetData().GetString().size(); j++) {
            string val = col->GetData().GetString()[j];
            val_list.push_back(ToWxString(val));  
            wxSize text_size = dc.GetTextExtent(val);
            size_t this_len = text_size.GetWidth();
            if (this_len > max_len) {
                max_len = this_len;
            }
        }
        if (col->GetData().GetString().size() == 2
            && IsSynonymForTrue(col->GetData().GetString()[0])
            && IsSynonymForFalse(col->GetData().GetString()[1])) {
            wxGridCellBoolEditor::UseStringValues(wxT("true"), wxEmptyString);
            for (int row = 0; row < m_Grid->GetNumberRows(); row++) {
                m_Grid->SetCellEditor(row, pos, new wxGridCellBoolEditor ());
            }
            m_Grid->AutoSizeColLabelSize(pos);
        } else {
            for (int row = 0; row < m_Grid->GetNumberRows(); row++) {
                m_Grid->SetCellEditor(row, pos, new wxGridCellChoiceEditor (val_list, false));
            }
            m_Grid->SetColMinimalWidth(pos, max_len + wxSYS_VSCROLL_X);
            m_Grid->SetColumnWidth(pos, max_len + wxSYS_VSCROLL_X);
        }
    } else {
        for (int row = 0; row < m_Grid->GetNumberRows(); row++) {
            m_Grid->SetCellEditor(row, pos, new wxGridCellTextEditor ());
        }
        m_Grid->AutoSizeColLabelSize(pos);
    }

}


void CSeqTableGridPanel::SetColumnSizesAndChoices()
{
    if (m_Choices) {
        size_t i = 0;
        for (; i < m_Choices->GetColumns().size(); i++) {
            CConstRef<CSeqTable_column> col = m_Choices->GetColumns()[i];
            if (col->IsSetData() && col->GetData().IsString() && col->GetData().GetString().size() > 0) {
                wxArrayString val_list;
                size_t max_len = 10;
                wxClientDC dc(this);
                for (size_t j = 0; j < col->GetData().GetString().size(); j++) {
                    string val = col->GetData().GetString()[j];
                    val_list.push_back(ToWxString(val));  
                    wxSize text_size = dc.GetTextExtent(val);
                    size_t this_len = text_size.GetWidth();
                    if (this_len > max_len) {
                        max_len = this_len;
                    }
                }
                if (col->GetData().GetString().size() == 2
                    && IsSynonymForTrue(col->GetData().GetString()[0])
                    && IsSynonymForFalse(col->GetData().GetString()[1])) {
                    wxGridCellBoolEditor::UseStringValues(wxT("true"), wxEmptyString);
                    for (int row = 0; row < m_Grid->GetNumberRows(); row++) {
                        m_Grid->SetCellEditor(row, i, new wxGridCellBoolEditor ());
                    }
                    m_Grid->AutoSizeColLabelSize(i);
                } else {
                    for (int row = 0; row < m_Grid->GetNumberRows(); row++) {
                        m_Grid->SetCellEditor(row, i, new wxGridCellChoiceEditor (val_list, false));
                    }
                    m_Grid->SetColMinimalWidth(i, max_len + wxSYS_VSCROLL_X);
                    m_Grid->SetColumnWidth(i, max_len + wxSYS_VSCROLL_X);
                }
            } else {
                m_Grid->AutoSizeColLabelSize(i);
            }
        }
        while (i < m_Grid->GetNumberCols()) {
            string label = ToStdString(m_Grid->GetColLabelValue(i));
            if (NStr::EqualNocase(label, kProblems)) {
                m_Grid->AutoSizeColumn(i);
            } else {
                m_Grid->AutoSizeColLabelSize(i);
            }
            i++;
        }
    } else {
        m_Grid->AutoSizeColumns();
    }
    InitMapRowLabelToIndex();
    SetBestTableWidth();
}


int CSeqTableGridPanel::GetBestTableWidth()
{
    // calculate "best" table width
    int sum = m_Grid->GetRowLabelSize();
    for (size_t i = 0; i < m_Grid->GetNumberCols(); i++) {
        sum += m_Grid->GetColumnWidth(i);
    }
    return sum;
}


void CSeqTableGridPanel::SetBestTableWidth()
{
    // calculate "best" table width
    int sum = GetBestTableWidth();

    wxSize current_size = m_Grid->GetSize();
    if (sum > current_size.x) {
        current_size.x = sum;
        m_Grid->SetMinClientSize(current_size);
    }
}


/*!
 * Should we show tooltips?
 */

bool CSeqTableGridPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSeqTableGridPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSeqTableGridPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSeqTableGridPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSeqTableGridPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSeqTableGridPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSeqTableGridPanel icon retrieval
}


CRef<CSeq_table> CSeqTableGridPanel::GetValuesTable()
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        CRef<objects::CSeq_table> empty;
        return empty;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
    return values_table;
}


void CSeqTableGridPanel::SetValuesTable(CRef<objects::CSeq_table> table)
{
    CSeqTableGrid *gridAdapter = new CSeqTableGrid(table);
    m_Grid->SetTable(gridAdapter, true);
    x_MakeProblemsColumnReadOnly();
    SetColumnSizesAndChoices();
    m_Grid->ForceRefresh();        
}


void CSeqTableGridPanel::x_SetUpMenu()
{
    static bool registered = false;
    if (!registered) {
        registered = true;

        CUICommandRegistry& cmd_reg = CUICommandRegistry::GetInstance();
        cmd_reg.RegisterCommand(eCmdCopyTableValues,
            "Copy",
            "Copy",
            NcbiEmptyString,
            NcbiEmptyString,
            "Copy cell or cells");
        cmd_reg.RegisterCommand(eCmdPasteTableValues,
            "Paste",
            "Paste",
            NcbiEmptyString,
            NcbiEmptyString,
            "Paste cell or cells");
        cmd_reg.RegisterCommand(eCmdAppendTableValues,
            "Append",
            "Append",
            NcbiEmptyString,
            NcbiEmptyString,
            "Append to cell or cells");
        cmd_reg.RegisterCommand(eCmdCopyTableValuesFromId,
            "Paste from SeqID",
            "Paste from SeqID",
            NcbiEmptyString,
            NcbiEmptyString,
            "Paste to cell or cells from SeqId"); 
        cmd_reg.RegisterCommand(eCmdSearchTable,
            "Search",
            "Search",
            NcbiEmptyString,
            NcbiEmptyString,
            "Search Table");
        cmd_reg.RegisterCommand(eCmdDeleteTableCol,
            "Delete",
            "Delete",
            NcbiEmptyString,
            NcbiEmptyString,
            "Delete Column");
        cmd_reg.RegisterCommand(eCmdRenameTableCol,
            "Rename",
            "Rename",
            NcbiEmptyString,
            NcbiEmptyString,
            "Rename Column");
    }
}



static
WX_DEFINE_MENU(kEditSeqTableMenu)
    WX_MENU_ITEM(eCmdCopyTableValues)
    WX_MENU_ITEM(eCmdPasteTableValues)
    WX_MENU_ITEM(eCmdAppendTableValues)
    WX_MENU_ITEM(eCmdCopyTableValuesFromId) 
    WX_MENU_ITEM(eCmdSearchTable)
    WX_MENU_ITEM(eCmdDeleteTableCol) 
    WX_MENU_ITEM(eCmdRenameTableCol) 
WX_END_MENU()

void CSeqTableGridPanel::x_CreateMenu( wxGridEvent& evt )
{
    wxMenu* menu = CUICommandRegistry::GetInstance().CreateMenu(kEditSeqTableMenu);
    if (menu) {
        m_Grid->PopupMenu(menu, evt.GetPosition());
    }
}

void CSeqTableGridPanel::InitMapRowLabelToIndex()
{
    m_MapRowLabelToIndex.clear();
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    int num_rows = gridAdapter->GetNumberRows();
    for (int i=0; i<num_rows; i++)
    {
        string s = ToStdString(gridAdapter->GetRowLabelValue(i));
        m_MapRowLabelToIndex[s]= i;
    }
}


bool  cmpTableValues(const pair< pair<string,int>,int> &a, const pair< pair<string,int>,int> &b) // we don't seem to have tuple yet (?!) so pairs it is
{
    double l, r;
    if (a.first.first == b.first.first) return a.first.second < b.first.second;
    if (NStr::StringToNumeric(a.first.first,&l,NStr::fConvErr_NoThrow | NStr::fAllowLeadingSpaces | NStr::fAllowTrailingSpaces) && 
        NStr::StringToNumeric(b.first.first,&r,NStr::fConvErr_NoThrow | NStr::fAllowLeadingSpaces | NStr::fAllowTrailingSpaces))
        return l<r;
    return a.first.first < b.first.first;
}

/*!
 * wxEVT_GRID_COL_SORT event handler for ID_GRID
 */

void CSeqTableGridPanel::OnSortTableByColumn( wxGridEvent& event )  // see "Sorting support" in http://docs.wxwidgets.org/trunk/classwx_grid.html
{
    if (m_Grid->GetSortingColumn() == -1) 
    { 
        m_Grid->Refresh();
        return;
    }
    
    int col = m_Grid->GetSortingColumn();
    bool ascend = m_Grid->IsSortOrderAscending();
    
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
    CRef<objects::CSeq_table> old_table(new CSeq_table);
    old_table->Assign(*values_table);
    

    CConstRef<objects::CSeqTable_column> column = values_table->GetColumns()[col+1];
    if (!column->IsSetData() || column->GetData().IsString())
    { 
        m_Grid->Refresh();
        return;
    }
    int num_rows = column->GetData().GetString().size();                                 
    int num_cols = values_table->GetColumns().size();

    vector < pair< pair<string,int>,int> > sorted;
    for (int i=0; i<num_rows; i++)
    {
        string s1 = column->GetData().GetString()[i];
        string s2 = ToStdString(gridAdapter->GetRowLabelValue(i));
        pair<string,int> p(s1,m_MapRowLabelToIndex[s2]);
        sorted.push_back(pair< pair<string,int>,int>(p,i));
    }
    std::sort(sorted.begin(),sorted.end(),cmpTableValues); 
   
    if (!ascend)
        std::reverse(sorted.begin(),sorted.end());

    for (int i=0; i<num_rows; i++)
    {
        int old_i = sorted[i].second;
        for (int j=0; j<num_cols; j++)
        {
            switch (values_table->GetColumns()[j]->GetData().Which())
            {
                case CSeqTable_multi_data::e_Id:
                    values_table->SetColumns()[j]->SetData().SetId()[i] = old_table->SetColumns()[j]->GetData().GetId()[old_i];
                    break;
                case CSeqTable_multi_data::e_String:
                    values_table->SetColumns()[j]->SetData().SetString()[i] = old_table->SetColumns()[j]->GetData().GetString()[old_i];
                    break;
            default:
                break;
            }
        }
    }
    m_SortByRowLabel = true;
    m_SortByRowLabelAscend = true;
    m_Grid->Refresh();
}

SrcEditDialog* CSeqTableGridPanel::x_GetParent()
{
    wxWindow* w = this->GetParent();
    while (w != NULL) {
        SrcEditDialog* parent = dynamic_cast<SrcEditDialog*>(w);
        if (parent) {
            return parent;
        } else {
            w = w->GetParent();
        }
    }
    return NULL;
}  

void CSeqTableGridPanel::OnRenameColumn( wxCommandEvent& event )  
{ 
    SrcEditDialog* dlg = x_GetParent();
    if (!dlg) 
        return;

    if (!m_Grid->IsSelection()) 
    {
        wxMessageBox( wxT("Please select a single column to rename"), wxT("Error"), wxOK | wxICON_ERROR, this); 
        return;
    }

    const wxGridCellCoordsArray& selected_blocks = m_Grid->GetSelectionBlockTopLeft();
    size_t num_blocks = selected_blocks.size();

    const wxArrayInt& cols = m_Grid->GetSelectedCols();
    size_t num_cols = cols.size();

    const wxGridCellCoordsArray& selected_block_right = m_Grid->GetSelectionBlockBottomRight();
    if (num_blocks != selected_block_right.size()) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    
    int col = -1;
    if (num_blocks == 1) 
    {
        int left_col = selected_blocks[0].GetCol();
        int right_col = selected_block_right[0].GetCol();
        num_cols = right_col - left_col + 1;
        if (num_cols == 1)
            col = left_col;
    }
    else if (num_cols == 1)
        col = cols[0];
    
    if ( col >= 0 )
    {
        wxArrayString srcModNameStrings;
        dlg->GetQualChoices(srcModNameStrings);
        CRenameColumnDlg choice(NULL, srcModNameStrings);
        if (choice.ShowModal() == wxID_OK) 
        {
            wxString qual = choice.GetSelection();
            choice.Close();      
            if (!qual.IsEmpty())
            {
                m_Grid->GetTable()->SetColLabelValue(col, qual);
                dlg->ChangeColumnName(col, ToStdString(qual));
                m_Grid->Refresh();
            }
        }
    }
    else
        wxMessageBox( wxT("Please select a single column to rename"), wxT("Error"), wxOK | wxICON_ERROR, this); 
}

void CSeqTableGridPanel::OnDeleteColumn( wxCommandEvent& event )  
{ 

    if (!m_Grid->IsSelection()) {
        return;
    }
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }

    const wxGridCellCoordsArray& selected_blocks = m_Grid->GetSelectionBlockTopLeft();
    size_t num_blocks = selected_blocks.size();

    const wxArrayInt& cols = m_Grid->GetSelectedCols();
    size_t num_cols = cols.size();

    if (num_blocks == 0 && num_cols == 0) {
        wxMessageBox( wxT("Nothing selected!"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    } else if (num_blocks > 1) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    const wxGridCellCoordsArray& selected_block_right = m_Grid->GetSelectionBlockBottomRight();
    if (num_blocks != selected_block_right.size()) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    if (num_blocks == 1) 
    {
        int left_col = selected_blocks[0].GetCol();
        int right_col = selected_block_right[0].GetCol();
        num_cols = right_col - left_col + 1;
        gridAdapter->DeleteCols(left_col+1,num_cols);
    }
    else if (num_cols > 0)
    {
        for (int i = 0; i < num_cols; i++) 
        {
            gridAdapter->DeleteCols(cols[i] + 1,1);
        }
    }
        
    m_Grid->Refresh();
}



/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for eCmdCopyTableValues
 */
void CSeqTableGridPanel::OnCopyCells( wxCommandEvent& event )
{
    x_OnCopyCells();
}

void CSeqTableGridPanel::x_OnCopyCells()
{
    if (!m_Grid->IsSelection()) {
        return;
    }

    const wxGridCellCoordsArray& selected_blocks = m_Grid->GetSelectionBlockTopLeft();
    size_t num_blocks = selected_blocks.size();

    const wxArrayInt& rows = m_Grid->GetSelectedRows();
    size_t num_rows = rows.size();

    const wxArrayInt& cols = m_Grid->GetSelectedCols();
    size_t num_cols = cols.size();

    if (num_blocks == 0 && num_rows == 0 && num_cols == 0) {
        wxMessageBox( wxT("Nothing selected!"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    } else if (num_blocks > 1 
               || (num_blocks > 0 && num_rows > 0) 
               || (num_blocks > 0 && num_cols > 0)
               || (num_rows > 0 && num_cols > 0)) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    const wxGridCellCoordsArray& selected_block_right = m_Grid->GetSelectionBlockBottomRight();
    if (num_blocks != selected_block_right.size()) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();

    m_Copied.Reset(new objects::CSeq_table());
    int top_row = 0;
    int left_col = 0;
    int bot_row = values_table->GetNum_rows() - 1;
    int right_col = values_table->GetColumns().size() - 1;
    if (num_blocks == 1) {
        top_row = selected_blocks[0].GetRow();
        left_col = selected_blocks[0].GetCol();
        bot_row = selected_block_right[0].GetRow();
        right_col = selected_block_right[0].GetCol();
        num_rows = bot_row - top_row + 1;
        num_cols = right_col - left_col + 1;

        for (int i = 0; i < num_cols; i++) {
            CConstRef<objects::CSeqTable_column> old_column = values_table->GetColumns()[left_col + i + 1];
            CRef<objects::CSeqTable_column> new_column(new objects::CSeqTable_column());
            new_column->SetHeader().Assign(old_column->GetHeader());
            int j;
            if (old_column->IsSetData() && old_column->GetData().IsString())
                for (j = top_row; j <= bot_row && j < old_column->GetData().GetString().size(); j++) {
                    if (j < old_column->GetData().GetString().size()) {
                        new_column->SetData().SetString().push_back(old_column->GetData().GetString()[j]);
                    } else {
                        new_column->SetData().SetString().push_back("");
                    }
                }
            m_Copied->SetColumns().push_back(new_column);
        }
        m_Copied->SetNum_rows(num_rows);
    } else if (num_cols > 0) {
        for (int i = 0; i < num_cols; i++) {
            CConstRef<objects::CSeqTable_column> old_column = values_table->GetColumns()[cols[i] + 1];
            CRef<objects::CSeqTable_column> new_column(new objects::CSeqTable_column());
            new_column->Assign(*old_column);
            if (old_column->IsSetData() && old_column->GetData().IsString())
                while (new_column->GetData().GetString().size() < values_table->GetNum_rows()) {
                    new_column->SetData().SetString().push_back("");
                }
            m_Copied->SetColumns().push_back(new_column);
        }
        m_Copied->SetNum_rows(values_table->GetNum_rows());
    } else if (num_rows > 0) {        
        for (int i = 1; i < values_table->GetColumns().size(); i++) {
            CConstRef<objects::CSeqTable_column> old_column = values_table->GetColumns()[i];
            CRef<objects::CSeqTable_column> new_column(new objects::CSeqTable_column());
            new_column->SetHeader().Assign(old_column->GetHeader());
            for (int j = 0; j < num_rows; j++) {
                if (j<rows.size() && old_column->IsSetData() && old_column->GetData().IsString() && rows[j] < old_column->GetData().GetString().size()) {
                    string str = old_column->GetData().GetString()[rows[j]];
                    new_column->SetData().SetString().push_back(str);
                } else {
                    new_column->SetData().SetString().push_back("");
                }
            }
            m_Copied->SetColumns().push_back(new_column);
        }
        m_Copied->SetNum_rows(num_rows);
    }
}


static void s_CopyToIntColumn
(CConstRef<objects::CSeqTable_column> src, 
 CRef<objects::CSeqTable_column> dst, 
 int top, int bot)
{
    int j, k = 0;
    for (j = top; j <= bot; j++) {
        while (dst->SetData().SetInt().size() <= j) {
            dst->SetData().SetInt().push_back(0);
        }
        if (src->GetData().IsInt()) {
            dst->SetData().SetInt()[j] = src->GetData().GetInt()[k];
        } else if (src->GetData().IsString()) {
            dst->SetData().SetInt()[j] = NStr::StringToInt(src->GetData().GetString()[k]);
        } else {
            // skip it
        }
        k++;
        if (k >= src->GetData().GetSize()) {
            k = 0;
        }
    }

}


static void s_CopyToStringColumn 
(CConstRef<objects::CSeqTable_column> src, 
 CRef<objects::CSeqTable_column> dst, 
 int top, int bot, 
 bool append, string delim)
{
    int j, k = 0;
    for (j = top; j <= bot; j++) {
        while (dst->SetData().SetString().size() <= j) {
            dst->SetData().SetString().push_back("");
        }

        string val = "";
        if (src->GetData().IsString()) {
            val = src->GetData().GetString()[k];
        } else if (src->GetData().IsInt()) {
            val = NStr::NumericToString(src->GetData().GetInt()[k]);
        }
        if (!append || NStr::IsBlank(val)) {
            dst->SetData().SetString()[j] = val;
        } else {
            dst->SetData().SetString()[j] += delim + val;
        }

        k++;
        if (k >= src->GetData().GetSize()) {
            k = 0;
        }
    } 
}


static void s_CopyColumn 
(CConstRef<objects::CSeqTable_column> src, 
 CRef<objects::CSeqTable_column> dst, 
 int top, int bot, 
 bool append, string delim)
{
    if (dst->GetData().IsInt()) {
        s_CopyToIntColumn(src, dst, top, bot);
    } else if (dst->GetData().IsString()) {
        s_CopyToStringColumn(src, dst, top, bot, append, delim);
    } else {
        wxMessageBox( wxT("Bad column type!"), wxT("Error"), wxOK | wxICON_ERROR, NULL);
    }

}


static void s_CopyRow 
(CConstRef<objects::CSeq_table> src,
 CRef<objects::CSeq_table> dst,
 int left, int right, int src_row, int dst_row,
 bool append, string delim)
{
    int j, k = 0;

    for (j = left; j <= right; j++) {
        CConstRef<objects::CSeqTable_column> src_col = src->GetColumns()[k];
        CRef<objects::CSeqTable_column> dst_col = dst->SetColumns()[j];
        while (dst_col->SetData().SetString().size() <= dst_row) {
            dst_col->SetData().SetString().push_back("");
        }
        if (src_col->IsSetData() && src_col->GetData().IsString())
        {
            if (!append || NStr::IsBlank(dst_col->SetData().SetString()[dst_row])) {
                dst_col->SetData().SetString()[dst_row] = src_col->GetData().GetString()[src_row];
            } else {
                dst_col->SetData().SetString()[dst_row] += delim + src_col->GetData().GetString()[src_row];
            }
        }

        k++;
        if (k >= src->GetColumns().size()) {
            k = 0;
        }
    }
}


bool CSeqTableGridPanel::x_ValuesOkForNewColumn (CConstRef<CSeqTable_column> values, int col_pos)
{
    if (!m_Choices || m_Choices->GetColumns().size() <= col_pos) {
        // ok because there is no set choice list for this column
        return true;
    }
    CConstRef<CSeqTable_column> choices = m_Choices->GetColumns()[col_pos];
    if (!choices->IsSetData() || !choices->GetData().IsString()) {
        // ok because there is no set choice list for this column
        return true;
    }
    if (values->IsSetData() && values->GetData().IsString())
        for (size_t i = 0; i < values->GetData().GetString().size(); i++) {
            string new_val = values->GetData().GetString()[i];
            bool found = false;
            for (size_t j = 0; j < choices->GetData().GetString().size() && !found; j++) {
                string option = choices->GetData().GetString()[j];
                if (NStr::Equal(new_val, option)) {
                    found = true;
                }
            }
            if (!found) {
                return false;
            }
        }
    return true;
}


void CSeqTableGridPanel::x_PasteCells(CRef<objects::CSeq_table> copied, bool append, string delim)
{
    if (!m_Grid->IsSelection()) {
        return;
    }

    const wxGridCellCoordsArray& selected_blocks = m_Grid->GetSelectionBlockTopLeft();
    size_t num_blocks = selected_blocks.size();

    const wxArrayInt& rows = m_Grid->GetSelectedRows();
    size_t num_rows = rows.size();

    const wxArrayInt& cols = m_Grid->GetSelectedCols();
    size_t num_cols = cols.size();

    if (num_blocks == 0 && num_rows == 0 && num_cols == 0) {
        wxMessageBox( wxT("Nothing selected!"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    } else if (num_blocks > 1 
               || (num_blocks > 0 && num_rows > 0) 
               || (num_blocks > 0 && num_cols > 0)
               || (num_rows > 0 && num_cols > 0)) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    const wxGridCellCoordsArray& selected_block_right = m_Grid->GetSelectionBlockBottomRight();
    if (num_blocks != selected_block_right.size()) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();

    int top_row = 0;
    int left_col = 0;
    int bot_row = values_table->GetNum_rows() - 1;
    int right_col = values_table->GetColumns().size() - 1;
    if (num_blocks == 1) {
        top_row = selected_blocks[0].GetRow();
        left_col = selected_blocks[0].GetCol();
        bot_row = selected_block_right[0].GetRow();
        right_col = selected_block_right[0].GetCol();
        num_rows = bot_row - top_row + 1;
        num_cols = right_col - left_col + 1;

        if (num_rows % copied->GetNum_rows() != 0 || num_cols % copied->GetColumns().size() != 0) {
            wxMessageBox( wxT("Source size does not match destination size!"), wxT("Error"), wxOK | wxICON_ERROR, this);
            return;
        }
        // check to make sure that column types match
        int j = 0;
        for (int i = left_col; i <= right_col; i++) {
            CRef<objects::CSeqTable_column> new_column = values_table->SetColumns()[i + 1];
            CConstRef<objects::CSeqTable_column> cpy_column = copied->GetColumns()[j];
            if (!x_ValuesOkForNewColumn (cpy_column, i)) {
                wxMessageBox( wxT("Source values are inappropriate for destination values!"), wxT("Error"), wxOK | wxICON_ERROR, this);
                return;
            }
            j++;
            if (j >= copied->GetColumns().size()) {
                j = 0;
            }
        }
        // then paste
        j = 0;
        for (int i = left_col; i <= right_col; i++) {
            CRef<objects::CSeqTable_column> new_column = values_table->SetColumns()[i + 1];
            CConstRef<objects::CSeqTable_column> cpy_column = copied->GetColumns()[j];
            s_CopyColumn (cpy_column, new_column, top_row, bot_row, append, delim);
            j++;
            if (j >= copied->GetColumns().size()) {
                j = 0;
            }
        }
    } else if (num_cols > 0) {
        if (num_cols % copied->GetColumns().size() != 0) {
            wxMessageBox( wxT("Source size does not match destination size!"), wxT("Error"), wxOK | wxICON_ERROR, this);
            return;
        }
        // check to make sure that column types match
        int j = 0;
        for (int i = 0; i < num_cols; i++) {
            CRef<objects::CSeqTable_column> new_column = values_table->SetColumns()[cols[i] + 1];
            CConstRef<objects::CSeqTable_column> cpy_column = copied->GetColumns()[j];
            if (!x_ValuesOkForNewColumn (cpy_column, cols[i])) {
                wxMessageBox( wxT("Source values are inappropriate for destination values!"), wxT("Error"), wxOK | wxICON_ERROR, this);
                return;
            }
            j++;
            if (j >= copied->GetColumns().size()) {
                j = 0;
            }
        }

        // then paste
        j = 0;
        for (int i = 0; i < num_cols; i++) {            
            CRef<objects::CSeqTable_column> new_column = values_table->SetColumns()[cols[i] + 1];
            CConstRef<objects::CSeqTable_column> cpy_column = copied->GetColumns()[j];
            s_CopyColumn (cpy_column, new_column, 0, values_table->GetNum_rows() - 1, append, delim);
            j++;
            if (j >= copied->GetColumns().size()) {
                j = 0;
            }
        }
    } else if (num_rows > 0) {
        // TODO
        // check to make sure that column types match
        int j = 0;
        for (int i = 1; i < values_table->GetColumns().size(); i++) {
            if (!x_ValuesOkForNewColumn(copied->GetColumns()[j], i - 1)) {
                wxMessageBox( wxT("Source values are inappropriate for destination values!"), wxT("Error"), wxOK | wxICON_ERROR, this);
                return;
            }
            j++;
            if (j >= copied->GetColumns().size()) {
                j = 0;
            }
        }
      
        // then paste
        j = 0;
        for (int i = 0; i < num_rows; i++) {
            s_CopyRow (copied, values_table, 1, values_table->GetColumns().size() - 1, j, rows[i], append, delim);
            j++;
            if (j >= copied->GetNum_rows()) {
                j = 0;
            }
        }
    }
    m_Grid->Refresh();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for eCmdPasteTableValues
 */

void CSeqTableGridPanel::OnPasteCells( wxCommandEvent& event )
{
    x_OnPasteCells();
}

void CSeqTableGridPanel::x_OnPasteCells()
{
    if (!m_Copied || !m_Copied->IsSetColumns() || !m_Copied->IsSetNum_rows()
        || m_Copied->GetColumns().size() == 0 || m_Copied->GetNum_rows() == 0) {
        wxTheClipboard->Open();
        wxTextDataObject data;
        wxTheClipboard->GetData( data );
        string val = ToStdString(data.GetText());
        if (!m_Copied) {
            m_Copied.Reset(new objects::CSeq_table());
        }
        if (!m_Copied->IsSetColumns() || m_Copied->GetColumns().size() == 0) {
            CRef<objects::CSeqTable_column> col(new objects::CSeqTable_column());
            m_Copied->SetColumns().push_back(col);
        } 
        m_Copied->SetColumns().front()->SetData().SetString().push_back(val);    
        m_Copied->SetNum_rows(1);
        wxTheClipboard->Close();
    }
    x_PasteCells (m_Copied);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for eCmdAppendTableValues
 */

void CSeqTableGridPanel::OnPasteAppendCells( wxCommandEvent& event )
{
    x_PasteCells (m_Copied, true);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for eCmdCopyTableValuesFromId
 */

void CSeqTableGridPanel::OnCopyCellsFromId( wxCommandEvent& event )
{
    if (!m_Grid->IsSelection()) {
        wxMessageBox( wxT("Nothing selected!"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    const wxArrayInt& rows = m_Grid->GetSelectedRows();
    size_t num_rows = rows.size();
    if (num_rows > 0) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    const wxGridCellCoordsArray& selected_blocks = m_Grid->GetSelectionBlockTopLeft();
    size_t num_blocks = selected_blocks.size();

    const wxArrayInt& cols = m_Grid->GetSelectedCols();
    size_t num_cols = cols.size();

    if (num_blocks == 0 && num_cols == 0) {
        wxMessageBox( wxT("Nothing selected!"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    } else if (num_blocks > 1 
               || (num_blocks > 0 && num_cols > 0)) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }
    const wxGridCellCoordsArray& selected_block_right = m_Grid->GetSelectionBlockBottomRight();
    if (num_blocks != selected_block_right.size()) {
        wxMessageBox( wxT("Selection is too complex"), wxT("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();

    // populate copy_data with ID values
    CRef<objects::CSeq_table> copy_data (new objects::CSeq_table());

    int top_row = 0;
    int bot_row = values_table->GetNum_rows() - 1;
    num_rows = values_table->GetNum_rows();
    if (num_blocks == 1) {
        top_row = selected_blocks[0].GetRow();
        bot_row = selected_block_right[0].GetRow();
        num_rows = bot_row - top_row + 1;
    }

    CConstRef<objects::CSeqTable_column> id_column = values_table->GetColumns()[0];
    CRef<objects::CSeqTable_column> new_column(new objects::CSeqTable_column());
    new_column->SetHeader().SetTitle("Copied ID");

    for (int j = top_row; j <= bot_row; j++) {
        string label = "";
        id_column->GetData().GetId()[j]->GetLabel (&label, objects::CSeq_id::eContent);
        new_column->SetData().SetString().push_back(label);
    }
    copy_data->SetColumns().push_back(new_column);
    copy_data->SetNum_rows(num_rows);

    x_PasteCells (copy_data);
}


/*!
 * wxEVT_GRID_CELL_RIGHT_CLICK event handler for ID_GRID
 */

void CSeqTableGridPanel::OnCellRightClick( wxGridEvent& event )
{
    x_CreateMenu(event);
}

void CSeqTableGridPanel::OnCellLeftClick( wxGridEvent& event )
{
    if (event.GetRow() >=0 && event.GetCol() >=0 )
    {
        m_Grid->SetGridCursor( event.GetRow(), event.GetCol() );
        if  (m_Grid->CanEnableCellControl())
            m_Grid->EnableCellEditControl();
    }
    event.Skip();
}


/*!
 * wxEVT_GRID_LABEL_RIGHT_CLICK event handler for ID_GRID
 */

void CSeqTableGridPanel::OnLabelRightClick( wxGridEvent& event )
{
    x_CreateMenu(event);
}

void CSeqTableGridPanel::x_SortByRowLabel()
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
    CRef<objects::CSeq_table> old_table(new CSeq_table);
    old_table->Assign(*values_table);
    

    int num_rows = gridAdapter->GetNumberRows();
    int num_cols = values_table->GetColumns().size();

    vector < pair< pair<string,int>,int> > sorted;
    for (int i=0; i<num_rows; i++)
    {
        string s = ToStdString(gridAdapter->GetRowLabelValue(i));
        pair<string,int> p("",m_MapRowLabelToIndex[s]);
        sorted.push_back(pair< pair<string,int>,int>(p,i));
    }
    std::sort(sorted.begin(),sorted.end(),cmpTableValues); 
   
    if (!m_SortByRowLabelAscend)
        std::reverse(sorted.begin(),sorted.end());

    for (int i=0; i<num_rows; i++)
    {
        int old_i = sorted[i].second;
        for (int j=0; j<num_cols; j++)
        {
            switch (values_table->GetColumns()[j]->GetData().Which())
            {
                case CSeqTable_multi_data::e_Id:
                    values_table->SetColumns()[j]->SetData().SetId()[i] = old_table->SetColumns()[j]->GetData().GetId()[old_i];
                    break;
                case CSeqTable_multi_data::e_String:
                    values_table->SetColumns()[j]->SetData().SetString()[i] = old_table->SetColumns()[j]->GetData().GetString()[old_i];
                    break;
            default:
                break;
            }
        }
    }
    m_SortByRowLabelAscend = !m_SortByRowLabelAscend;
    m_Grid->Refresh();
}

void CSeqTableGridPanel::OnLabelLeftClick( wxGridEvent& event )
{
    if (event.GetCol() == -1 && event.GetRow() == -1 && m_SortByRowLabel)
    {
        x_SortByRowLabel();
    }
    else if (event.GetCol() == -1 && event.GetRow() == -1)
        m_SortByRowLabel = true;
    if (event.GetCol() == -1 || event.GetRow() == -1)
        m_Grid->DisableCellEditControl();
    else
        m_Grid->EnableCellEditControl();
    event.Skip();
}

void CSeqTableGridPanel::OnLabelLeftDClick( wxGridEvent& event )
{
    if (event.GetCol() == -1 && event.GetRow() == -1)  
    {
        m_SortByRowLabel = true;
        x_SortByRowLabel();
    }
    if (event.GetCol() == -1 || event.GetRow() == -1)
        m_Grid->DisableCellEditControl();
    else
        m_Grid->EnableCellEditControl();
    event.Skip();
}


void CSeqTableGridPanel::OnSearchTable( wxCommandEvent& event )
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( !gridAdapter ) return;
    
    int num_rows = gridAdapter->GetNumberRows();

    wxTextEntryDialog dlg(this,wxT("Enter the sequences you would like to select separated by space"), wxT("Enter Sequence IDs"), m_FindStr, wxOK|wxCANCEL);
    if (dlg.ShowModal() == wxID_OK)
    {
        m_FindStr = dlg.GetValue();
        if (m_FindStr.empty()) return;
        wxStringTokenizer tkz(m_FindStr);
        m_Grid->ClearSelection();
        while ( tkz.HasMoreTokens() )
        {
            wxString token = tkz.GetNextToken();
            if (!token.empty())
                for (int i=0; i<num_rows; i++)
                    if (token == gridAdapter->GetRowLabelValue(i))
                        m_Grid->SelectRow(i,true);
        }
    }
}



void  CSeqTableGridPanel::CopyPasteDataByKeyboard(wxKeyEvent& event) // See http://forums.wxwidgets.org/viewtopic.php?f=20&t=2200
{
    if ((event.GetKeyCode() == 'C') && (event.ControlDown() == true)) 
    {
        x_OnCopyCells();
    }
    else if ((event.GetKeyCode() == 'V') && (event.ControlDown() == true)) 
    {
        x_OnPasteCells();
    }
    event.Skip();
}


END_NCBI_SCOPE


