/*  $Id: srceditdialog.cpp 27955 2013-04-30 17:54:28Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <objects/general/Object_id.hpp>
#include <gui/packages/pkg_sequence_edit/srceditdialog.hpp>
#include <gui/packages/pkg_sequence_edit/srcedit_util.hpp>
#include <gui/packages/pkg_sequence_edit/subprep_util.hpp>
#include <gui/widgets/wx/table_utils.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/grid_widget/grid_widget.hpp>
#include <gui/objutils/interface_registry.hpp>

#include <gui/core/file_extensions.hpp>
#include <gui/core/open_objects_dlg.hpp>
#include <gui/core/ui_file_load_manager.hpp>
#include <gui/utils/object_loader.hpp>
#include <gui/utils/execute_unit.hpp>
#include <gui/utils/extension_impl.hpp>
#include <gui/widgets/wx/async_call.hpp>

#include <wx/grid.h>

#include <gui/widgets/wx/ui_command.hpp>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE


/*!
 * SrcEditDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( SrcEditDialog, wxDialog )


/*!
 * SrcEditDialog event table definition
 */

BEGIN_EVENT_TABLE( SrcEditDialog, wxDialog )

////@begin SrcEditDialog event table entries
    EVT_CHOICE( ID_QUALCHOICE, SrcEditDialog::OnQualchoiceSelected )

    EVT_BUTTON( ID_ADD_QUAL_BUTTON, SrcEditDialog::OnAddQual )

    EVT_BUTTON( ID_LOADQUALS, SrcEditDialog::OnLoadqualsClick )

    EVT_BUTTON( ID_EXPORT_QUALS, SrcEditDialog::OnExportQualsClick )

    EVT_BUTTON( ID_RECHECK_PROBLEMS_BTN, SrcEditDialog::OnRecheckProblemsBtnClick )

    EVT_BUTTON( ID_CLEAR_QUALS_BTN, SrcEditDialog::OnClearQuals )

    EVT_BUTTON( wxID_OK, SrcEditDialog::OnClickOk )

    EVT_BUTTON( wxID_CANCEL, SrcEditDialog::OnClickCancel )

////@end SrcEditDialog event table entries

END_EVENT_TABLE()


/*!
 * SrcEditDialog constructors
 */

SrcEditDialog::SrcEditDialog()
{
    Init();
}

SrcEditDialog::SrcEditDialog( wxWindow* parent, objects::CSeq_entry_Handle seh, 
                             IWorkbench* workbench,
                             CSourceRequirements::EWizardType wizard_type,
                             CSourceRequirements::EWizardSrcType src_type,
                             wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
: m_TopSeqEntry(seh),
  m_WizardType (wizard_type),
  m_SrcType (src_type),
  m_Copied (NULL),
  m_Workbench (workbench)
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * SrcEditDialog creator
 */

bool SrcEditDialog::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin SrcEditDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end SrcEditDialog creation
    return true;
}


/*!
 * SrcEditDialog destructor
 */

SrcEditDialog::~SrcEditDialog()
{
////@begin SrcEditDialog destruction
////@end SrcEditDialog destruction
}


/*!
 * Member initialisation
 */

void SrcEditDialog::Init()
{
////@begin SrcEditDialog member initialisation
    m_QualList = NULL;
    m_AddQualBtn = NULL;
    m_ExampleText = NULL;
////@end SrcEditDialog member initialisation
    m_SaveFileDir = wxEmptyString;
    m_SaveFileName = wxEmptyString;

}


/*!
 * Control creation for SrcEditDialog
 */

void SrcEditDialog::CreateControls()
{    
////@begin SrcEditDialog content construction
    SrcEditDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxArrayString m_QualListStrings;
    m_QualList = new wxChoice( itemDialog1, ID_QUALCHOICE, wxDefaultPosition, wxDefaultSize, m_QualListStrings, 0 );
    itemBoxSizer4->Add(m_QualList, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_AddQualBtn = new wxButton( itemDialog1, ID_ADD_QUAL_BUTTON, _("Add Qualifier"), wxDefaultPosition, wxDefaultSize, 0 );
    m_AddQualBtn->Enable(false);
    itemBoxSizer4->Add(m_AddQualBtn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer7, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemDialog1, ID_LOADQUALS, _("Import Qualifier Table"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton( itemDialog1, ID_EXPORT_QUALS, _("Export Qualifier Table"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(itemButton9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton10 = new wxButton( itemDialog1, ID_RECHECK_PROBLEMS_BTN, _("Recheck Problems"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(itemButton10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton11 = new wxButton( itemDialog1, ID_CLEAR_QUALS_BTN, _("Clear Qualifiers"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(itemButton11, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ExampleText = new wxStaticText( itemDialog1, wxID_STATIC, _("[Example values are listed in brackets]"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(m_ExampleText, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer13 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer13, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton14 = new wxButton( itemDialog1, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer13->Add(itemButton14, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton15 = new wxButton( itemDialog1, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer13->Add(itemButton15, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end SrcEditDialog content construction

    bool have_problems_column = false;
    if (m_TopSeqEntry) {
        /* this is where we need to construct a CSeqTable object from the BioSources in the record */
        CRef<objects::CSeq_table> seqTable = GetSeqTableFromSeqEntry(m_TopSeqEntry);
        CSourceRequirements *requirements = GetSrcRequirements (m_WizardType, m_SrcType, seqTable);
        if (requirements) {
            requirements->AddColumnsToSeqTable (seqTable);
            CRef<objects::CSeqTable_column> problems = requirements->CheckSourceQuals(seqTable);
            seqTable->SetColumns().push_back(problems);
            have_problems_column = true;
            delete requirements;
        }
        CRef<objects::CSeq_table> choices = x_GetSourceTableChoices(seqTable);
        m_GridPanel = new CSeqTableGridPanel(this, seqTable, choices);
        itemBoxSizer3->Add(m_GridPanel, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
        m_Grid = m_GridPanel->m_Grid;

        CSeqTableGrid *gridAdapter = new CSeqTableGrid(seqTable);
        m_Grid->SetTable(gridAdapter, true);

        m_Grid->AutoSizeColumns();
        // set problems column to read-only
        if (have_problems_column) {
            size_t problem_column = m_Grid->GetNumberCols() - 1;
            for (int i = 0; i < m_Grid->GetNumberRows(); i++) {
                m_Grid->GetOrCreateCellAttr(i, problem_column)->SetReadOnly(true);
            }
        } else {
            itemButton8->Show(false);
        }
        int l_height = m_Grid->GetColLabelSize();
        m_Grid->SetColLabelSize( 2 * l_height );

        x_RepopulateAddQualList();
        m_GridPanel->SetColumnSizesAndChoices();
    }

}


CRef<objects::CSeq_table> SrcEditDialog::x_GetSourceTableChoices (CRef<objects::CSeq_table> values)
{
    vector<bool> tf_positions;
    bool any_tf = false;

    ITERATE( objects::CSeq_table::TColumns, col_iter, values->GetColumns() ) {
        bool is_tf = false;
        if ((*col_iter)->IsSetHeader() && (*col_iter)->GetHeader().IsSetTitle()) {
            string label = (*col_iter)->GetHeader().GetTitle();
            size_t pos = NStr::Find(label, "\n");
            if (pos != string::npos) {
                label = label.substr(0, pos);
            }

            // see if it's a SubSource subtype:
            try {
                objects::CSubSource::TSubtype st = objects::CSubSource::GetSubtypeValue (label, objects::CSubSource::eVocabulary_insdc);
                if (objects::CSubSource::NeedsNoText(st)) {
                    is_tf = true;
                    any_tf = true;
                }
            } catch (...) {
                // no true/false orgmods
            }
        }
        tf_positions.push_back(is_tf);
    }
    if (!any_tf) {
        CRef<objects::CSeq_table> no_choices(NULL);
        return no_choices;
    }

    CRef<objects::CSeq_table> choices (new objects::CSeq_table());
    vector<string> tf_strings = GetTrueFalseList();

    for (size_t i = 1; i < tf_positions.size(); i++) {
        CRef<objects::CSeqTable_column> col(new objects::CSeqTable_column());
        if (tf_positions[i]) {
            for (size_t j = 0; j < tf_strings.size(); j++) {
                col->SetData().SetString().push_back(tf_strings[j]);
            }
        }
        choices->SetColumns().push_back(col);
    }
    return choices;
}


int SrcEditDialog::x_FindColumn( const wxString& name )
{
    int i = 0;

    for (i = 0; i < m_Grid->GetTable()->GetColsCount(); i++) {
        wxString col_name = m_Grid->GetTable()->GetColLabelValue(i);
        if (NStr::EqualNocase(ToStdString (name), ToStdString (col_name))) {
            return i;
        }
    }
    return -1;
}


const wxString s_SubSourceNoteName = _("SubSource Note");
const wxString s_OrgModNoteName = _("OrgMod Note");

void SrcEditDialog::x_RepopulateAddQualList()
{
    wxArrayString srcModNameStrings;
    GetQualChoices(srcModNameStrings);

    m_QualList->Clear();
    ITERATE (wxArrayString, it, srcModNameStrings) {
        m_QualList->Append (*it);
    }
}


void SrcEditDialog::ChangeColumnName(int col, string qual_name)
{
    // update choices for new column
    bool is_tf = false;
    // see if it's a SubSource subtype:
    try {
        objects::CSubSource::TSubtype st = objects::CSubSource::GetSubtypeValue (qual_name, objects::CSubSource::eVocabulary_insdc);
        if (objects::CSubSource::NeedsNoText(st)) {
            is_tf = true;
        }
    } catch (...) {
        // no true/false orgmods
    }
    vector<string> new_choices;
    if (is_tf) {
        new_choices = GetTrueFalseList();
    }
    m_GridPanel->UpdateColumnChoices(col, new_choices);

    x_RepopulateAddQualList();    
}


/*!
 * Should we show tooltips?
 */

bool SrcEditDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap SrcEditDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin SrcEditDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end SrcEditDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon SrcEditDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin SrcEditDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end SrcEditDialog icon retrieval
}


CRef<CCmdComposite> SrcEditDialog::GetCommand()
{
    CRef<CCmdComposite> cmd;

    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if( gridAdapter ) {
        CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
        cmd = ApplySrcTableToSeqEntry (values_table, m_TopSeqEntry);
    }
    return cmd;
}



/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
 */

void SrcEditDialog::OnClickOk( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON in SrcEditDialog.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON in SrcEditDialog. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
 */

void SrcEditDialog::OnClickCancel( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1 in SrcEditDialog.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1 in SrcEditDialog. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADD_QUAL_BUTTON
 */

void SrcEditDialog::OnAddQual( wxCommandEvent& event )
{
    wxString qual_name = m_QualList->GetStringSelection();
    if (NStr::IsBlank(ToStdString(qual_name))) {
        return;
    }
    int num_cols = m_Grid->GetTable()->GetNumberCols();
    m_Grid->GetTable()->InsertCols(num_cols - 1);
    m_Grid->GetTable()->SetColLabelValue(num_cols - 1, qual_name);
    m_Grid->AutoSizeColumns();

    ChangeColumnName(num_cols - 1, ToStdString(qual_name));
    m_AddQualBtn->Enable(false);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_CLEAR_QUALS_BTN
 */

void SrcEditDialog::OnClearQuals( wxCommandEvent& event )
{
    int num_cols = m_Grid->GetTable()->GetNumberCols();
    int num_rows = m_Grid->GetTable()->GetNumberRows();
    for (int col = 0; col < num_cols; col++) {
        for (int row = 0; row < num_rows; row++) {
            m_Grid->GetTable()->SetValue(row, col, "");
        }
    }
    m_Grid->ForceRefresh();
}


void SrcEditDialog::x_UpdateProblems()
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if (!gridAdapter) {
        return;
    }
    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
    if (!values_table) {
        return;
    }

    int problem_pos = m_Grid->GetNumberCols();
    for (int i = 0; i < m_Grid->GetNumberCols(); i++) {
        if (NStr::EqualNocase(ToStdString(m_Grid->GetColLabelValue(i)), kProblems)) {
            problem_pos = i;
            break;
        }
    }
    CRef<objects::CSeqTable_column> new_problems;

    CSourceRequirements *requirements = GetSrcRequirements (m_WizardType, m_SrcType, values_table);
    if (requirements) {
        new_problems = requirements->CheckSourceQuals(values_table);
        delete requirements;
    }
    if (problem_pos == m_Grid->GetNumberCols()) {
        if (new_problems) {
            // add problems column to table
            m_Grid->AppendCols();
            m_Grid->GetTable()->SetColLabelValue(problem_pos, kProblems);
            for (int i = 0; i < m_Grid->GetNumberRows(); i++) {
                m_Grid->GetOrCreateCellAttr(i, problem_pos)->SetReadOnly(true);
            }
        } else {
            // no problems column, no problems to report, done here
            return;
        }
    }
    int problem_rows = 0;
    if (new_problems) {
        problem_rows = new_problems->GetData().GetString().size();
    }
    int row;
    for (row = 0; row < m_Grid->GetNumberRows(); row++) {
        if (row < problem_rows) {
            m_Grid->SetCellValue(row, problem_pos, ToWxString(new_problems->GetData().GetString()[row]));
        } else {
            m_Grid->SetCellValue(row, problem_pos, wxEmptyString);
        }
    }
    m_Grid->AutoSizeColumns();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RECHECK_PROBLEMS_BTN
 */

void SrcEditDialog::OnRecheckProblemsBtnClick( wxCommandEvent& event )
{
    x_UpdateProblems();
}



static string FindBadColumns (CRef<objects::CSeq_table> table)
{
    int col = 1;
    string error = "";
    vector<string> already_seen;
    NON_CONST_ITERATE (objects::CSeq_table::TColumns, cit, table->SetColumns()) {
        if (!(*cit)->GetData().IsId()) {
            string label = "";
            if ((*cit)->IsSetHeader() 
                && (*cit)->GetHeader().IsSetTitle()) {
                label = (*cit)->GetHeader().GetTitle();
            }
            if (NStr::IsBlank(label)) {
                error += "Column " + NStr::NumericToString(col) + " has no header; ";
            } else {
                CRef<CSrcTableColumnBase> adapter = CSrcTableColumnBaseFactory::Create(**cit);
                if (!adapter) {
                    error += "Column " + NStr::NumericToString(col) + " header '" + label + "' is not recognized; ";
                } else {
                    string new_name = adapter->GetLabel();
                    (*cit)->SetHeader().SetTitle(new_name);
                    int as_col = 1;
                    ITERATE (vector<string>, sit, already_seen) {
                        if (NStr::Equal(new_name, *sit)) {
                            error += "Column " + NStr::NumericToString(col) + " (" 
                                  + label + ") and column " + NStr::NumericToString(as_col) 
                                  + " refer to the same field (" + new_name + "); ";
                        }
                        as_col++;
                    }
                    already_seen.push_back(new_name);
                }
            }
        }
        col++;
    }
    if (!NStr::IsBlank(error)) {
        error = error.substr(0, error.length() - 2);
    }
    return error;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_LOADQUALS
 */

void SrcEditDialog::OnLoadqualsClick( wxCommandEvent& event )
{
    CRef<CFileLoadManager> fileManager(new CFileLoadManager());

    // get all other file format from an Extension Point
    vector< CIRef<IFileFormatLoaderManager> > fileManagers;
    GetExtensionAsInterface("file_format_loader_manager", fileManagers);

    for(  size_t i = 0;  i < fileManagers.size();  ++i  )   {
        IFileFormatLoaderManager* mgr = fileManagers[i].GetPointer();
        string loader_id = mgr->GetFileLoaderId();
        if (loader_id == "file_loader_table") {
            fileManager->AddFileFormat(mgr);
        }
    }

    vector<CIRef<IUIToolManager> > loadManagers;
    loadManagers.push_back(CIRef<IUIToolManager>(fileManager.GetPointer()));

    COpenObjectsDlg dlg(NULL);
    dlg.SetSize(710, 480);

    dlg.SetRegistryPath("Dialogs.Edit.OpenTable");
    dlg.SetServiceLocator(m_Workbench);
    dlg.SetManagers(loadManagers);

    if (dlg.ShowModal() == wxID_OK) {
        CIRef<IObjectLoader> object_loader(dlg.GetObjectLoader());
        if (!object_loader) {
            wxMessageBox(wxT("Failed to get object loader"), wxT("Error"),
                         wxOK | wxICON_ERROR);
        }
        else {
            IExecuteUnit* execute_unit = dynamic_cast<IExecuteUnit*>(object_loader.GetPointer());
            if (execute_unit) {
                if (!execute_unit->PreExecute())
                    return;

                CExecuteUnitAsyncCall call(*execute_unit, wxT("Reading file(s)..."));
                CAsyncCall::Execute(call);
                if (!call.GetResult()) // Canceled
                    return;

                if (!execute_unit->PostExecute())
                    return;
            }
            const IObjectLoader::TObjects& objects = object_loader->GetObjects();
            ITERATE(IObjectLoader::TObjects, obj_it, objects) {
                const CObject& ptr = obj_it->GetObject();
                const objects::CSeq_annot* annot = dynamic_cast<const objects::CSeq_annot*>(&ptr);
                if (annot && annot->IsSeq_table()) {
                    CRef<objects::CSeq_table> input_table (new objects::CSeq_table());
                    input_table->Assign(annot->GetData().GetSeq_table());
                    // adjust for import
                    FixTableAfterImport (input_table);

                    // make sure qual names are all legit
                    string error = FindBadColumns(input_table);
                    if (!NStr::IsBlank(error)) {
                        wxMessageBox(ToWxString(error), wxT("Error"),
                                         wxOK | wxICON_ERROR, this);
                        return;
                    }

                    // get existing table
                    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
                    if (!gridAdapter) {
                        return;
                    }
                    CRef<objects::CSeq_table> values_table = gridAdapter->GetValuesTable();
                    if (!values_table) {
                        return;
                    }

                    error = FindBadRows (input_table, values_table, m_TopSeqEntry);
                    if (!NStr::IsBlank(error)) {
                        wxMessageBox(ToWxString(error), wxT("Error"),
                                         wxOK | wxICON_ERROR, this);
                        return;
                    }

                    if (CountTableColumnConflicts(values_table, *input_table) > 0) {
                        wxMessageBox(wxT("New table conflicts with existing values"), wxT("Error"),
                                         wxOK | wxICON_ERROR, this);
                        return;
                    }

                    RemoveProblemsColumn (values_table);

                    // merge input table with existing table
                    int num_new = CombineTables (values_table, *input_table);

                    gridAdapter = new CSeqTableGrid(values_table);
                    m_Grid->SetTable(gridAdapter, true);
                    m_Grid->ForceRefresh();
                    x_UpdateProblems();
                }
            }
        }
    }

}



/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_EXPORT_QUALS
 */

void SrcEditDialog::OnExportQualsClick( wxCommandEvent& event )
{
    CSeqTableGrid *gridAdapter = dynamic_cast<CSeqTableGrid*>(m_Grid->GetTable());
    if (!gridAdapter) {
        return;
    }
    CRef<objects::CSeq_table> grid_table = gridAdapter->GetValuesTable();
    if (!grid_table) {
        return;
    }

    CRef<objects::CSeq_table> values_table(new objects::CSeq_table());
    values_table->Assign(*grid_table);
    RemoveProblemsColumn(values_table);
    // fix headers
    NON_CONST_ITERATE(objects::CSeq_table::TColumns, it, values_table->SetColumns()) {
        if ((*it)->IsSetHeader() && (*it)->GetHeader().IsSetTitle()) {
            string title = (*it)->GetHeader().GetTitle();
            size_t pos = NStr::Find(title, "\n");
            if (pos != string::npos) {
                title = title.substr(0, pos);
            }
            (*it)->SetHeader().SetTitle(title);
        }
    }

    SaveTableFile (this, m_SaveFileDir, m_SaveFileName, values_table);
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_QUALCHOICE
 */

void SrcEditDialog::OnQualchoiceSelected( wxCommandEvent& event )
{
    wxString qual_name = m_QualList->GetStringSelection();
    if (NStr::IsBlank(ToStdString(qual_name))) {
        m_AddQualBtn->Enable(false);
    } else {
        m_AddQualBtn->Enable(true);
    }
}

void SrcEditDialog::GetQualChoices(wxArrayString& srcModNameStrings)
{
    size_t i;
    for (i = 0; i < objects::CSubSource::eSubtype_other; i++) {
        try {
            string qual_name = objects::CSubSource::GetSubtypeName (i);
            if (!NStr::IsBlank(qual_name) && x_FindColumn(qual_name) == -1 && !objects::CSubSource::IsDiscouraged(i)) {
                srcModNameStrings.push_back(ToWxString(qual_name));
            }
        } catch (...) {
        }
    }
    for (i = 0; i < objects::COrgMod::eSubtype_other; i++) {
        try {
            string qual_name = objects::COrgMod::GetSubtypeName (i);
            if (!NStr::IsBlank(qual_name) && x_FindColumn(qual_name) == -1&& !objects::COrgMod::IsDiscouraged(i)) {
                srcModNameStrings.push_back(ToWxString(qual_name));
            }
        } catch (...) {
        }
    }

    srcModNameStrings.Sort();

    if (x_FindColumn(s_OrgModNoteName) == -1) {
        srcModNameStrings.push_back(s_OrgModNoteName);
    }
    if (x_FindColumn(s_SubSourceNoteName) == -1) {
        srcModNameStrings.push_back(s_SubSourceNoteName);
    } 
}

END_NCBI_SCOPE
