/*  $Id: fetch_url.cpp 28026 2013-05-06 20:54:53Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Josh Cherry
 *
 * File Description:  fetch data from the net using http
 *
 */


#include <ncbi_pch.hpp>
#include <gui/utils/fetch_url.hpp>

#include <connect/ncbi_conn_stream.hpp>
#include <connect/ncbi_core_cxx.hpp>
#include <connect/ncbi_util.h>

BEGIN_NCBI_SCOPE

void CFetchURL::Fetch(const string& url,
                      string& result,
                      const STimeout* timeout)
{
    // pretend to be a 'real' web browser; some servers
    // care about this
    string header =
        "User-Agent: Mozilla/4.0 (compatible; MSIE 5.01; "
        "Windows NT 5.0; (R1 1.1))";

    CConn_HttpStream http(url, NULL, header, 0 /* parse_header */,
                          NULL /* user_data */, 0 /* adjust */, 
                          0 /* cleanup */, fHCC_AutoReconnect, timeout);

    result.erase();
    char buf[1024];
    while (!http.fail()) {
        http.read(buf, 1024);
        result.append(buf, http.gcount());
    }
}

END_NCBI_SCOPE
