/*  $Id: wx_aln_properties_dlg.cpp 25673 2012-04-20 15:02:57Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:
 *
 * File Description:
 *
 */
#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <gui/widgets/aln_multiple/wx_aln_properties_dlg.hpp>

#include <gui/widgets/wx/dialog_utils.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/color_picker_validator.hpp>

#include <wx/panel.h>
#include <wx/stattext.h>
#include <wx/statline.h>
#include <wx/statbox.h>
#include <wx/combobox.h>
#include <wx/sizer.h>
#include <wx/clrpicker.h>
#include <wx/checklst.h>
#include <wx/valgen.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

BEGIN_NCBI_SCOPE


IMPLEMENT_DYNAMIC_CLASS( CAlnPropertiesDlg, CDialog )

BEGIN_EVENT_TABLE( CAlnPropertiesDlg, CDialog )

////@begin CAlnPropertiesDlg event table entries
    EVT_BUTTON( wxID_OK, CAlnPropertiesDlg::OnOkClick )

////@end CAlnPropertiesDlg event table entries

END_EVENT_TABLE()

CAlnPropertiesDlg::CAlnPropertiesDlg()
{
    Init();
}

CAlnPropertiesDlg::CAlnPropertiesDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

bool CAlnPropertiesDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CAlnPropertiesDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    CDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CAlnPropertiesDlg creation
    return true;
}

CAlnPropertiesDlg::~CAlnPropertiesDlg()
{
////@begin CAlnPropertiesDlg destruction
////@end CAlnPropertiesDlg destruction
}

void CAlnPropertiesDlg::Init()
{
    m_Style = NULL;

////@begin CAlnPropertiesDlg member initialisation
    m_VisColumnsList = NULL;
    m_TextFaceCombo = NULL;
    m_TextSizeCombo = NULL;
    m_SeqFaceCombo = NULL;
    m_SeqSizeCombo = NULL;
////@end CAlnPropertiesDlg member initialisation
}

void CAlnPropertiesDlg::CreateControls()
{
////@begin CAlnPropertiesDlg content construction
    CAlnPropertiesDlg* itemCDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCDialog1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer3 = new wxFlexGridSizer(0, 1, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer3, 1, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxPanel* itemPanel4 = new wxPanel( itemCDialog1, ID_PANEL, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
    itemFlexGridSizer3->Add(itemPanel4, 1, wxGROW|wxGROW, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxVERTICAL);
    itemPanel4->SetSizer(itemBoxSizer5);

    wxFlexGridSizer* itemFlexGridSizer6 = new wxFlexGridSizer(0, 2, 0, 0);
    itemFlexGridSizer3->Add(itemFlexGridSizer6, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Visible Columns:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer6->Add(itemStaticText7, 0, wxALIGN_RIGHT|wxALIGN_TOP|wxALL, 5);

    wxArrayString m_VisColumnsListStrings;
    m_VisColumnsList = new wxCheckListBox( itemCDialog1, ID_VIS_COLUMNS, wxDefaultPosition, wxSize(220, 130), m_VisColumnsListStrings, wxLB_SINGLE );
    itemFlexGridSizer6->Add(m_VisColumnsList, 3, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Text Font:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer6->Add(itemStaticText9, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer10 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer6->Add(itemBoxSizer10, 1, wxGROW|wxALIGN_CENTER_VERTICAL, 5);

    wxArrayString m_TextFaceComboStrings;
    m_TextFaceCombo = new wxComboBox( itemCDialog1, ID_TEXT_FACE_COMBO, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_TextFaceComboStrings, wxCB_READONLY );
    itemBoxSizer10->Add(m_TextFaceCombo, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_TextSizeComboStrings;
    m_TextSizeCombo = new wxComboBox( itemCDialog1, ID_TEXT_SIZE_COMBO, wxEmptyString, wxDefaultPosition, wxSize(50, -1), m_TextSizeComboStrings, wxCB_READONLY );
    itemBoxSizer10->Add(m_TextSizeCombo, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText13 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Sequence Font:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer6->Add(itemStaticText13, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer6->Add(itemBoxSizer14, 0, wxGROW|wxALIGN_CENTER_VERTICAL, 5);

    wxArrayString m_SeqFaceComboStrings;
    m_SeqFaceCombo = new wxComboBox( itemCDialog1, ID_SEQ_FACE_COMBO, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_SeqFaceComboStrings, wxCB_READONLY );
    itemBoxSizer14->Add(m_SeqFaceCombo, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_SeqSizeComboStrings;
    m_SeqSizeCombo = new wxComboBox( itemCDialog1, ID_FACE_SIZE_COMBO, wxEmptyString, wxDefaultPosition, wxSize(50, -1), m_SeqSizeComboStrings, wxCB_READONLY );
    itemBoxSizer14->Add(m_SeqSizeCombo, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer6->AddGrowableCol(1);

    wxStaticLine* itemStaticLine17 = new wxStaticLine( itemCDialog1, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemFlexGridSizer3->Add(itemStaticLine17, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer18 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer3->Add(itemBoxSizer18, 0, wxGROW|wxALIGN_CENTER_VERTICAL, 5);

    wxFlexGridSizer* itemFlexGridSizer19 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer18->Add(itemFlexGridSizer19, 1, wxALIGN_TOP, 5);

    wxStaticText* itemStaticText20 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Text Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer19->Add(itemStaticText20, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl21 = new wxColourPickerCtrl( itemCDialog1, ID_TEXT_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer19->Add(itemColourPickerCtrl21, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText22 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Back Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer19->Add(itemStaticText22, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl23 = new wxColourPickerCtrl( itemCDialog1, ID_BACK_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer19->Add(itemColourPickerCtrl23, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText24 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Sequence Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer19->Add(itemStaticText24, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl25 = new wxColourPickerCtrl( itemCDialog1, ID_SEQ_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer19->Add(itemColourPickerCtrl25, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText26 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Frame Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer19->Add(itemStaticText26, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl27 = new wxColourPickerCtrl( itemCDialog1, ID_FRAME_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer19->Add(itemColourPickerCtrl27, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer19->AddGrowableCol(0);

    wxFlexGridSizer* itemFlexGridSizer28 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer18->Add(itemFlexGridSizer28, 1, wxALIGN_TOP, 5);

    wxStaticText* itemStaticText29 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Segments Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer28->Add(itemStaticText29, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl30 = new wxColourPickerCtrl( itemCDialog1, ID_SEG_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer28->Add(itemColourPickerCtrl30, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText31 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Selected Text Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer28->Add(itemStaticText31, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl32 = new wxColourPickerCtrl( itemCDialog1, ID_SEL_TEXT_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer28->Add(itemColourPickerCtrl32, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText33 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Selected Back Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer28->Add(itemStaticText33, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl34 = new wxColourPickerCtrl( itemCDialog1, ID_SEL_BACK_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer28->Add(itemColourPickerCtrl34, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText35 = new wxStaticText( itemCDialog1, wxID_STATIC, _("Focused Back Color:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer28->Add(itemStaticText35, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxColourPickerCtrl* itemColourPickerCtrl36 = new wxColourPickerCtrl( itemCDialog1, ID_FOCUSED_BACK_COLOR, wxColour(), wxDefaultPosition, wxDefaultSize, wxCLRP_DEFAULT_STYLE );
    itemFlexGridSizer28->Add(itemColourPickerCtrl36, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer28->AddGrowableCol(0);

    wxStaticLine* itemStaticLine37 = new wxStaticLine( itemCDialog1, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemFlexGridSizer3->Add(itemStaticLine37, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW|wxALL, 5);

    itemFlexGridSizer3->AddGrowableRow(0);
    itemFlexGridSizer3->AddGrowableCol(0);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer38 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer38, 0, wxALIGN_RIGHT|wxALL, 5);
    wxButton* itemButton39 = new wxButton( itemCDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer38->AddButton(itemButton39);

    wxButton* itemButton40 = new wxButton( itemCDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer38->AddButton(itemButton40);

    itemStdDialogButtonSizer38->Realize();

    // Set validators
    m_VisColumnsList->SetValidator( wxGenericValidator(& m_VisColumns) );
    m_TextFaceCombo->SetValidator( wxGenericValidator(& m_TextFace) );
    m_TextSizeCombo->SetValidator( wxGenericValidator(& m_TextSize) );
    m_SeqFaceCombo->SetValidator( wxGenericValidator(& m_SeqFace) );
    m_SeqSizeCombo->SetValidator( wxGenericValidator(& m_SeqSize) );
    itemColourPickerCtrl21->SetValidator( CColorPickerValidator(& m_TextColor) );
    itemColourPickerCtrl23->SetValidator( CColorPickerValidator(& m_BackColor) );
    itemColourPickerCtrl25->SetValidator( CColorPickerValidator(& m_SeqColor) );
    itemColourPickerCtrl27->SetValidator( CColorPickerValidator(& m_FrameColor) );
    itemColourPickerCtrl30->SetValidator( CColorPickerValidator(& m_SegmentColor) );
    itemColourPickerCtrl32->SetValidator( CColorPickerValidator(& m_SelTextColor) );
    itemColourPickerCtrl34->SetValidator( CColorPickerValidator(& m_SelBackColor) );
    itemColourPickerCtrl36->SetValidator( CColorPickerValidator(& m_FocusedColor) );
////@end CAlnPropertiesDlg content construction
}

bool CAlnPropertiesDlg::ShowToolTips()
{
    return true;
}
wxBitmap CAlnPropertiesDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CAlnPropertiesDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CAlnPropertiesDlg bitmap retrieval
}
wxIcon CAlnPropertiesDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CAlnPropertiesDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CAlnPropertiesDlg icon retrieval
}


void CAlnPropertiesDlg::SetParams(CWidgetDisplayStyle& style)
{
    m_Style = &style;

    // check visible columns
    m_AllColumns.clear();
    m_VisColumns.Clear();

    size_t n = m_Style->m_Columns.size();
    for( size_t i = 0;  i < n;  i++ )  {
        CWidgetDisplayStyle::SColumn& col = m_Style->m_Columns[i];
        m_AllColumns.push_back(col.m_Name);
        if(col.m_Visible)   {
            m_VisColumns.Add((int)i);
        }
    }

    // initialize Columns Check List
    Init_wxControl(*m_VisColumnsList, m_AllColumns);

    vector<string> values;

    // Populate Font Face combos
    CGlBitmapFont::GetAllFaces(values);
    sort(values.begin(), values.end());

    ITERATE(vector<string>, it, values) {
        wxString wx_s = ToWxString(it->c_str());
        m_TextFaceCombo->Append(wx_s);
        m_SeqFaceCombo->Append(wx_s);
    }
    m_TextFace = GlFontFaceTo_wxString(m_Style->m_TextFont);
    m_TextSize = GlFontSizeTo_wxString(m_Style->m_TextFont);

    //populate Font Size combos
    CGlBitmapFont::GetAllSizes(values);

    ITERATE(vector<string>, it, values) {
        wxString wx_s = ToWxString(it->c_str());
        m_TextSizeCombo->Append(wx_s);
        m_SeqSizeCombo->Append(wx_s);
    }
    m_SeqFace = GlFontFaceTo_wxString(m_Style->m_SeqFont);
    m_SeqSize = GlFontSizeTo_wxString(m_Style->m_SeqFont);

    m_TextColor = m_Style->GetColor(CWidgetDisplayStyle::eText);
    m_BackColor = m_Style->GetColor(CWidgetDisplayStyle::eBack);
    m_SelTextColor = m_Style->GetColor(CWidgetDisplayStyle::eSelectedText);
    m_SelBackColor = m_Style->GetColor(CWidgetDisplayStyle::eSelectedBack);
    m_FocusedColor = m_Style->GetColor(CWidgetDisplayStyle::eFocusedBack);
    m_FrameColor = m_Style->GetColor(CWidgetDisplayStyle::eFrame);
    m_SegmentColor = m_Style->GetColor(CWidgetDisplayStyle::eAlignSegs);
    m_SeqColor = m_Style->GetColor(CWidgetDisplayStyle::eSequence);
}


void CAlnPropertiesDlg::OnOkClick(wxCommandEvent& event)
{
    if(TransferDataFromWindow())    {
        CWidgetDisplayStyle::TColumns& columns = m_Style->m_Columns;

        // reset Visible by default
        NON_CONST_ITERATE(CWidgetDisplayStyle::TColumns, it, columns)   {
            it->m_Visible = false;
        }

        for(  size_t i = 0;  i < m_VisColumns.GetCount(); i++ )  {
            int index = m_VisColumns[i];
            m_Style->m_Columns[index].m_Visible = true;
        }

        wxStringsToGlFont(m_Style->m_TextFont, m_TextFace, m_TextSize);
        wxStringsToGlFont(m_Style->m_SeqFont, m_SeqFace, m_SeqSize);

        m_Style->SetColor(CWidgetDisplayStyle::eText, m_TextColor);
        m_Style->SetColor(CWidgetDisplayStyle::eBack, m_BackColor);
        m_Style->SetColor(CWidgetDisplayStyle::eSelectedText, m_SelTextColor);
        m_Style->SetColor(CWidgetDisplayStyle::eSelectedBack, m_SelBackColor);
        m_Style->SetColor(CWidgetDisplayStyle::eFrame, m_FrameColor);
        m_Style->SetColor(CWidgetDisplayStyle::eFocusedBack, m_FocusedColor);
        m_Style->SetColor(CWidgetDisplayStyle::eAlignSegs, m_SegmentColor);
        m_Style->SetColor(CWidgetDisplayStyle::eSequence, m_SeqColor);

        EndModal(wxID_OK);
    }
}


END_NCBI_SCOPE

