/*  $Id $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *    Test application for plugins
 */


#include <ncbi_pch.hpp>

#include <gui/config/theme_set.hpp>

#include <gui/widgets/config/theme_dlg.hpp>
#include <gui/widgets/config/theme_mediator.hpp>
#include <gui/widgets/config/theme_config_panel2.hpp>

#include <gui/widgets/wx/wx_app.hpp>

BEGIN_NCBI_SCOPE

class CDemoThemeDlgApp: public CwxNCBIApp
{
protected:
    // we are overriding this function to create our main frame window
    virtual bool    x_CreateGUI();
};

DECLARE_APP(ncbi::CDemoThemeDlgApp)
IMPLEMENT_APP_NO_MAIN(ncbi::CDemoThemeDlgApp)
IMPLEMENT_WX_THEME_SUPPORT

class CDemoThemeDlgApplication : public CNCBIwxApplication
{
public:
    CDemoThemeDlgApplication(const string& app_name) :
        CNCBIwxApplication(app_name) {}
    virtual void Init(void);
};

void CDemoThemeDlgApplication::Init(void)
{
    // Create command-line argument descriptions class
    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);

    arg_desc->AddOptionalKey("local", "LocalFile",
                     "Local registry",
                     CArgDescriptions::eInputFile);

    arg_desc->AddKey("global", "GlobalFile",
                     "Global registry",
                     CArgDescriptions::eInputFile);

    arg_desc->AddDefaultKey("out", "Outputfile",
                     "Where to write changes to the registry.",
                     CArgDescriptions::eOutputFile, "-");

    // Specify USAGE context
    arg_desc->SetUsageContext(GetArguments().GetProgramBasename(),
                              "Test for the Configuration dialog.");

    // Setup arg.descriptions for this application
    SetupArgDescriptions(arg_desc.release());

    x_Init_wxApplication();
}

bool CDemoThemeDlgApp::x_CreateGUI()
{
    CGuiRegistry reg;
    CArgs args = CNcbiApplication::Instance()->GetArgs();

    CNcbiIstream& global = args["global"].AsInputFile();
    reg.SetGlobal(global);

    if (args["local"]) {
        CNcbiIstream& local = args["local"].AsInputFile();
        reg.SetLocal(local);
    }

    CThemeSet configSettings("SeqGraphicView", &reg);

    bool write_when_done = false;

    // create panel
/*
    {
        CThemeConfigPanel  preference_panel;

        // create configuration mediator from data set and panel
        CThemeMediator preference_mediator(configSettings, preference_panel);

        // create and display dialog given the mediator.
        //CConfigDlg preference_dialog(preference_mediator);
        CThemeDlg  preference_dialog(preference_mediator);
        int dlg_ret = preference_dialog.ShowModal();

        if (eCancel != dlg_ret) {
            cerr << "ok chosen" << endl;
            write_when_done = true;
        } else {
            cerr << "cancel chosen, no processing done." << endl;
        }
    }
*/
    {
        CThemeConfigPanel2  preference_panel;
        preference_panel.SetLayoutDirection(CThemeConfigPanel2::eLayoutDirection_Vertical);

        // create configuration mediator from data set and panel
        CThemeMediator preference_mediator(configSettings, preference_panel);

        // create and display dialog given the mediator.
        //CConfigDlg preference_dialog(preference_mediator);
        CThemeDlg  preference_dialog(preference_mediator, NULL);
        int dlg_ret = preference_dialog.ShowModal();

        if (eCancel != dlg_ret) {
            cerr << "ok chosen" << endl;
            write_when_done = true;
        } else {
            cerr << "cancel chosen, no processing done." << endl;
        }
    }

/*
    {
        CThemeConfigPanel2  preference_panel;
        preference_panel.SetLayoutDirection(CThemeConfigPanel2::eLayoutDirection_Horizontal);
        // create configuration mediator from data set and panel
        CThemeMediator preference_mediator(configSettings, preference_panel);

        // create and display dialog given the mediator.
        //CConfigDlg preference_dialog(preference_mediator);
        CThemeDlg  preference_dialog(preference_mediator);
        EDialogReturnValue dlg_ret = preference_dialog.ShowModal();
        if (eCancel != dlg_ret) {
            cerr << "ok chosen" << endl;
            write_when_done = true;
        } else {
            cerr << "cancel chosen, no processing done." << endl;
        }
    }
*/
    if (write_when_done  &&  args["out"]) {
        CNcbiOstream& localout = args["out"].AsOutputFile();
        reg.Write(localout);

    }

    return false;
}


END_NCBI_SCOPE

/////////////////////////////////////////////////////////////////////////////
//  main entry point

int main(int argc, const char* argv[])
{
    // create standard NCBI wxWidgets application
    ncbi::CDemoThemeDlgApplication ncbi_app("ThemeDlg demo App");

    // Execute main application function
    return ncbi_app.AppMain(argc, argv, 0, ncbi::eDS_Default, 0);
}
