#ifndef GUI_VIEWGRAPHIC_CONFIG_MEDIAT__HPP
#define GUI_VIEWGRAPHIC_CONFIG_MEDIAT__HPP

/*  $Id: test_config_mediat.hpp 14565 2007-05-18 12:32:01Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *  Configuration Panel Mediator for Graphics View.
 *
 *  Mediates between the panel of widgets CTestConfigPanel
 *  and the data class CTestSettings,
 *  in the context of a CConfigDlg.
 */

#include <gui/widgets/config/config_mediator.hpp>     // base class
#include "test_config_panel.hpp"    // widgets
#include "test_settings.hpp"      // data


BEGIN_NCBI_SCOPE

class CTestConfigMediator : public CConfigMediator
{
public:
    CTestConfigMediator(CTestSettings & data, CTestConfigPanel& widget_group);
    ~CTestConfigMediator();

    /// What should we title the dialog window?
    virtual string              GetWindowTitle(void) const;
    /// Load data into that panel
    void                Load(const string& state);
    /// Load 'factory default' data into that panel.
    void                LoadDefault(void);
    /// Save the panel's data.
    void                Save(const string& state);
    /// Has the data represented by the panel's widgets been modified?
    bool                Modified(void) const;

private:
    CTestSettings&       GetDataSet(void);
    const CTestSettings& GetDataSet(void) const;
    IConfigPanel&        GetPanel(void);

    void        x_Load(CTestSettings & data, CTestConfigPanel& widget_group);
};

END_NCBI_SCOPE

#endif  /* GUI_VIEWGRAPHIC_CONFIG_MEDIAT__HPP */
