#ifndef GUI_WIDGETS_SEQ_GRAPHIC___CONFIG__CPP
#define GUI_WIDGETS_SEQ_GRAPHIC___CONFIG__CPP

/*  $Id: test_settings.cpp 15168 2007-10-15 19:11:34Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *   Preferences for the Graphic Sequence view widget
 *   Independent from FLTK.
 *
 */

#include <ncbi_pch.hpp>
#include "test_settings.hpp"

BEGIN_NCBI_SCOPE

static AutoPtr<IFactoryDefaultSettings> InitFactoryDefaults(void)
{
    CFactoryDefaultSettings::TFDInput fds;

    fds["SelectionColor"]  = "0 0 255";  // blue
    fds["ShowFeatures"]   = "true";
    fds["Name"]   = "blank";

    return AutoPtr<IFactoryDefaultSettings>(new CFactoryDefaultSettings(fds));
}


CTestSettings::CTestSettings(CRef<objects::CPluginConfigCache> config_cache)
    : CSettingsSet(config_cache,
                    "test config",
                    InitFactoryDefaults())
{
    LoadCurrentSettings(eLoad_Current);
}

/*
    Each of the key strings used below in LoadCurrentSettings
    must be in InitFactoryDefaults() above.
    If not the Get() will throw an assertion.
    The conversion can throw CStringException.
*/
bool CTestSettings::LoadCurrentSettings(ELoadValueSource src)
{
    m_ShowFeatures = NStr::StringToBool( Get("ShowFeatures", src) );
    m_SelectionColor = CRgbaColor( Get("SelectionColor", src) );
    m_Name = Get("Name", src);

    return true;
}


bool CTestSettings::SaveCurrentSettings(void)
{
    Set("ShowFeatures", NStr::BoolToString(m_ShowFeatures));
    Set("SelectionColor", m_SelectionColor.ToString(false));
    Set("Name", m_Name);

    return true;
}


END_NCBI_SCOPE

#endif  /* GUI_WIDGETS_SEQ_GRAPHIC___CONFIG__CPP */
