/*  $Id: edit_styles_dlg.cpp 25627 2012-04-13 17:44:23Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <gui/widgets/config/iconfig_mediator.hpp>
#include <gui/widgets/config/edit_styles_dlg.hpp>

#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/stattext.h>
#include <wx/listbox.h>
#include <wx/msgdlg.h>
#include <wx/textdlg.h>

BEGIN_NCBI_SCOPE


/*!
 * CEditStylesDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CEditStylesDlg, wxDialog )


/*!
 * CEditStylesDlg event table definition
 */

BEGIN_EVENT_TABLE( CEditStylesDlg, wxDialog )

////@begin CEditStylesDlg event table entries
    EVT_LISTBOX( ID_LISTBOX1, CEditStylesDlg::OnListbox1Selected )

    EVT_BUTTON( ID_BUTTON1, CEditStylesDlg::OnAddClick )

    EVT_BUTTON( ID_BUTTON4, CEditStylesDlg::OnCopyClick )

    EVT_BUTTON( ID_BUTTON5, CEditStylesDlg::OnRenameClick )

    EVT_BUTTON( ID_BUTTON6, CEditStylesDlg::OnDeleteClick )

////@end CEditStylesDlg event table entries

END_EVENT_TABLE()


/*!
 * CEditStylesDlg constructors
 */

CEditStylesDlg::CEditStylesDlg()
{
    Init();
}

CEditStylesDlg::CEditStylesDlg( IConfigMediator& mediator, wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    m_ConfigMediator.Reset(&mediator);

    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CEditStylesDlg creator
 */

bool CEditStylesDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CEditStylesDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CEditStylesDlg creation
    return true;
}


/*!
 * CEditStylesDlg destructor
 */

CEditStylesDlg::~CEditStylesDlg()
{
////@begin CEditStylesDlg destruction
////@end CEditStylesDlg destruction
}


/*!
 * Member initialisation
 */

void CEditStylesDlg::Init()
{
////@begin CEditStylesDlg member initialisation
////@end CEditStylesDlg member initialisation
}


/*!
 * Control creation for CEditStylesDlg
 */

void CEditStylesDlg::CreateControls()
{
////@begin CEditStylesDlg content construction
    CEditStylesDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText( itemDialog1, wxID_STATIC, _("Saved Styles"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 1, wxGROW|wxALL, 5);

    wxArrayString itemListBox5Strings;
    wxListBox* itemListBox5 = new wxListBox( itemDialog1, ID_LISTBOX1, wxDefaultPosition, wxSize(-1, itemDialog1->ConvertDialogToPixels(wxSize(-1, 120)).y), itemListBox5Strings, wxLB_SINGLE );
    itemBoxSizer4->Add(itemListBox5, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer4->Add(itemBoxSizer6, 0, wxGROW|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemDialog1, ID_BUTTON1, _("Add"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton7, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemDialog1, ID_BUTTON4, _("Copy"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton8, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton( itemDialog1, ID_BUTTON5, _("Rename"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton9, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton10 = new wxButton( itemDialog1, ID_BUTTON6, _("Delete"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton10, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    itemBoxSizer6->Add(7, 8, 1, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton12 = new wxButton( itemDialog1, wxID_OK, _("Done"), wxDefaultPosition, wxDefaultSize, 0 );
    itemButton12->SetDefault();
    itemBoxSizer6->Add(itemButton12, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

////@end CEditStylesDlg content construction

    SetTitle(m_ConfigMediator->GetWindowTitle());

    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    list<string> const& states = m_ConfigMediator->GetStates();
    ITERATE(list<string>, it, states)
        itemListStyles->Append(it->c_str());
    if (itemListStyles->GetCount() > 0)
        itemListStyles->SetSelection(0);

    x_UpdateSetBtns();
}


/*!
 * Should we show tooltips?
 */

bool CEditStylesDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CEditStylesDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CEditStylesDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CEditStylesDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CEditStylesDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CEditStylesDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CEditStylesDlg icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
 */

void CEditStylesDlg::OnAddClick( wxCommandEvent& event )
{
    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    string  new_state;
    if ( m_ConfigMediator->AddState(new_state) ) {
        itemListStyles->Append(new_state.c_str());
        itemListStyles->SetStringSelection(new_state.c_str());
        x_UpdateSetBtns();
    }
    else
        wxMessageBox(_("Failed to add new state."), _("Error"), wxOK | wxICON_ERROR, this);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON4
 */

void CEditStylesDlg::OnCopyClick( wxCommandEvent& event )
{
    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    wxString selection = itemListStyles->GetStringSelection();
    if (selection.empty())
        return;

    string new_state;
    if ( m_ConfigMediator->DuplicateState(selection.c_str(), new_state) ) {
        itemListStyles->Append(new_state.c_str());
        itemListStyles->SetStringSelection(new_state.c_str());
        x_UpdateSetBtns();
    }
    else
        wxMessageBox(_("Can not copy '") + selection + _("' style."), _("Error"), wxOK | wxICON_ERROR, this);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON5
 */

void CEditStylesDlg::OnRenameClick( wxCommandEvent& event )
{
    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    int index = itemListStyles->GetSelection();
    if (index == wxNOT_FOUND)
        return;

    wxString selection = itemListStyles->GetString(index);

    // can we do this?
    if ( ! m_ConfigMediator->CanRenameState(selection.c_str())) {
        wxMessageBox(_("Can not rename '") + selection + _("' style."), _("Error"), wxOK | wxICON_ERROR, this);
        return;
    }

    wxString msg = _("Rename '") + selection + _("' as:");

    wxTextEntryDialog dlg(this, msg, _("Please enter text"), selection, wxOK|wxCANCEL);
    if (dlg.ShowModal() != wxID_OK)
        return;

    wxString req_state = dlg.GetValue();
    if (req_state.empty()  ||  req_state  ==  selection) {
        // requested name blank or no change to the name.
        return;
    }

    string  new_state;
    if ( m_ConfigMediator->RenameState(selection.c_str(), req_state.c_str(), new_state) ) {
        itemListStyles->SetString(index, new_state.c_str());

        // did we rename the mediator's currently loaded state?
        if (m_ConfigMediator->GetCurrentState() == selection.c_str())
            m_ConfigMediator->Load(new_state);

        x_UpdateSetBtns();
    }
    else
        wxMessageBox(_("Can not rename '") + selection + _("' style."), _("Error"), wxOK | wxICON_ERROR, this);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON6
 */

void CEditStylesDlg::OnDeleteClick( wxCommandEvent& event )
{
    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    int index = itemListStyles->GetSelection();
    if (index == wxNOT_FOUND)
        return;

    wxString selection = itemListStyles->GetString(index);
    if (selection.empty())
        return;

    if ( m_ConfigMediator->DeleteState(selection.c_str()) ) {
        itemListStyles->Delete(index);
        // did we delete the currently loaded state?
        if (m_ConfigMediator->GetCurrentState() == selection.c_str()) {
            itemListStyles->SetSelection(0);
            selection = itemListStyles->GetStringSelection();
            m_ConfigMediator->Load(selection.c_str());
        }
        x_UpdateSetBtns();
    }
    else
        wxMessageBox(_("Can not delete '") + selection + _("' style."), _("Error"), wxOK | wxICON_ERROR, this);
}

void CEditStylesDlg::x_UpdateSetBtns()
{
    bool delRenameOn = true;

    wxListBox* itemListStyles = (wxListBox*)FindWindow(ID_LISTBOX1);
    wxString selection = itemListStyles->GetStringSelection();
    if (selection.empty()) {
        FindWindow(ID_BUTTON4)->Enable(false);
        FindWindow(ID_BUTTON5)->Enable(false);
        FindWindow(ID_BUTTON6)->Enable(false);
    }
    else {
        if (!m_ConfigMediator->CanRenameState(selection.c_str()))
            delRenameOn = false;
        FindWindow(ID_BUTTON4)->Enable(true);
        FindWindow(ID_BUTTON5)->Enable(delRenameOn);
        FindWindow(ID_BUTTON6)->Enable(delRenameOn);
    }
}

/*!
 * wxEVT_COMMAND_LISTBOX_SELECTED event handler for ID_LISTBOX1
 */

void CEditStylesDlg::OnListbox1Selected( wxCommandEvent& event )
{
    x_UpdateSetBtns();
}

END_NCBI_SCOPE
