/*  $Id: feat_check_browser.cpp 14565 2007-05-18 12:32:01Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 * File Description: feat_check_browser.cpp
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/config/feat_check_browser.hpp>
#include <objects/seqfeat/SeqFeatData.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CFeatCheckBrowser::CFeatCheckBrowser() :
    CCheckBrowser(0, 0, 250, 430),
    m_Feats(CSeqFeatData::GetFeatList())
{
    user_data(this);
    callback((Fl_Callback*) cb_BrowserClicked);
    when(FL_WHEN_CHANGED | FL_WHEN_NOT_CHANGED);
    // populate the browser
    ITERATE(CFeatList, ft_it, *m_Feats) {
        const CFeatListItem& item = *ft_it;
        string desc = item.GetDescription();
        int feat_type = item.GetType();
        // indent subordinate types.
        if (item.GetSubtype() != CSeqFeatData::eSubtype_any  &&
            (feat_type == CSeqFeatData::e_Prot ||
             feat_type == CSeqFeatData::e_Rna  ||
             feat_type == CSeqFeatData::e_Imp)) {
            desc = "    " + desc;
        }
        Add(desc.c_str(), (void *) &item);
    }
}

CFeatCheckBrowser::~CFeatCheckBrowser()
{
}


void CFeatCheckBrowser::cb_BrowserClicked(CFeatCheckBrowser *w, void *d)
{
    static_cast<CFeatCheckBrowser*>(d)->x_BrowserClicked();
}

void CFeatCheckBrowser::x_BrowserClicked()
{
    int checked = GetLastChecked();
    if (checked) {
        bool val = GetCheck(checked);
        CFeatListItem *feat = static_cast<CFeatListItem*>(data(checked));
        _ASSERT(feat != NULL);
        if (feat->GetSubtype() == CSeqFeatData::eSubtype_any) {
            /* checked some master item */
            if (feat->GetType() == CSeqFeatData::e_not_set) {
                // checked THE master item.
                // make all items match this one.
                for (int r = 1; r <= size(); ++r) {
                    if (r == checked)
                        continue;
                    SetCheck(r, val);
                }
            } else {
                // make all items with this feature type match this one.
                for (int r = 1; r <= size(); ++r) {
                    if (r == checked)
                        continue;
                    CFeatListItem *aFeat = static_cast<CFeatListItem*>(data(r));
                    if (aFeat->GetType() == feat->GetType())
                        SetCheck(r, val);
                }
            }
            redraw();
        }
    }
}



END_NCBI_SCOPE
