/*  $Id: feat_show_panel.cpp 15170 2007-10-15 20:20:26Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 * File Description: feat_show_panel.cpp
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/config/feat_show_panel.hpp>
#include <objects/seqfeat/SeqFeatData.hpp>

BEGIN_NCBI_SCOPE

USING_SCOPE(objects);

CFeatShowPanel::CFeatShowPanel() :
    m_Browser(0),
    m_Feats(CSeqFeatData::GetFeatList())
{
}

CFeatShowPanel::CFeatShowPanel(const  vector<CFeatShowExtraItem> & extras) :
    m_Browser(0),
    m_Feats(CSeqFeatData::GetFeatList()),
    m_Extras(extras)
{
}


CFeatShowPanel::~CFeatShowPanel()
{
}


int CFeatShowPanel::GetNumExtras()
{
    return m_Extras.size();
}


void CFeatShowPanel::cb_BrowserClicked(CCheckBrowser *w, void *d)
{
    static_cast<CFeatShowPanel*>(d)->x_BrowserClicked();
}

void CFeatShowPanel::x_BrowserClicked()
{
    int checked = m_Browser->GetLastChecked();
    if (checked) {
        bool val = m_Browser->GetCheck(checked);
        if (checked >= GetNumExtras()) {
            CFeatListItem *feat = static_cast<CFeatListItem*>(m_Browser->data(checked));
            _ASSERT(feat != NULL);
            if (feat->GetSubtype() == CSeqFeatData::eSubtype_any) {
                /* checked some master item */
                if (feat->GetType() == CSeqFeatData::e_not_set) {
                    // checked THE master item.
                    // make all items match this one.
                    for (int r = GetNumExtras() + 1; r <= m_Browser->size(); ++r) {
                        if (r == checked)
                            continue;
                        m_Browser->SetCheck(r, val);
                    }
                } else {
                    // make all items with this feature type match this one.
                    for (int r = GetNumExtras() + 1; r <= m_Browser->size(); ++r) {
                        if (r == checked)
                            continue;
                        CFeatListItem *aFeat = static_cast<CFeatListItem*>(m_Browser->data(r));
                        if (aFeat->GetType() == feat->GetType())
                            m_Browser->SetCheck(r, val);
                    }
                }
                m_Browser->redraw();
            }
        }
    }
}


wxWindow* CFeatShowPanel::CreateConfigPanel(wxWindow* parent)
{
    return 0;
    // TODO
/*
    if (! m_Browser) {
        m_Browser = new CCheckBrowser(0, 0, 250, 430);
        m_Browser->user_data(this);
        m_Browser->callback((Fl_Callback*) cb_BrowserClicked);
        m_Browser->when(FL_WHEN_CHANGED | FL_WHEN_NOT_CHANGED);
        // populate the browser
        ITERATE(vector<CFeatShowExtraItem>, ext_it, m_Extras) {
            m_Browser->Add(ext_it->GetDesc().c_str(), (void *) & ext_it->GetExtIndex());
        }
        ITERATE(CFeatList, ft_it, *m_Feats) {
            const CFeatListItem& item = *ft_it;
            string desc = item.GetDescription();
            int feat_type = item.GetType();
            // indent subordinate types.
            if (item.GetSubtype() != CSeqFeatData::eSubtype_any  &&
                (feat_type == CSeqFeatData::e_Prot ||
                 feat_type == CSeqFeatData::e_Rna  ||
                 feat_type == CSeqFeatData::e_Imp)) {
                desc = "    " + desc;
            }
            m_Browser->Add(desc.c_str(), (void *) &item);
        }
    }
    return m_Browser;
 */
}




END_NCBI_SCOPE
