/*  $Id $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 *
 */

/// @file theme_config_panel_base.cpp
///

#include <ncbi_pch.hpp>
#include <gui/widgets/config/theme_config_panel_base.hpp>

#include <gui/config/settings_set.hpp>  // for exception decl.

#include <algorithm>

BEGIN_NCBI_SCOPE

IThemeConfigPanelBase::~IThemeConfigPanelBase()
{
}


CThemeConfigPanelBase::CThemeConfigPanelBase() :
    m_ConfigPanel(0)
{
}


CThemeConfigPanelBase::~CThemeConfigPanelBase()
{
}

void CThemeConfigPanelBase::Clear()
{
    m_ItemData.clear();
}

void CThemeConfigPanelBase::AddItem(
    const string& item,
    const string& sel_style,
    const TStyleList& styles,
    const string& desc
)
{
    m_ItemData.push_back(SItemData(item, sel_style, styles, desc));
}

void CThemeConfigPanelBase::SetStyles(const string& type, const TStyleList& styles)
{
    x_SetItemData(type, eFindByType).styles = styles;
}

string CThemeConfigPanelBase::GetCurrentStyle(const string& item) const
{
    return x_GetItemData(item, eFindByType).current_style;
}


void CThemeConfigPanelBase::SetCurrentStyle(const string& item, const string& style)
{
    x_SetItemData(item, eFindByType).current_style = style;
}


string CThemeConfigPanelBase::GetCurrentStyleByDesc(const string& desc) const
{
    return x_GetItemData(desc, eFindByDesc).current_style;
}


void CThemeConfigPanelBase::SetCurrentStyleByDesc(const string& desc, const string& style)
{
    x_SetItemData(desc, eFindByDesc).current_style = style;
}


CThemeConfigPanelBase::TStyleList CThemeConfigPanelBase::GetStylesByDesc(const string& desc) const
{
    return x_GetItemData(desc, eFindByDesc).styles;
}


const CThemeConfigPanelBase::SItemData& CThemeConfigPanelBase::x_GetItemData(const string& key, EKeyType e ) const
{
    vector <SItemData>::const_iterator  data_it;

    if (e == eFindByType) {
        data_it = find_if(m_ItemData.begin(), m_ItemData.end(), PFindType(key));
    } else if (e == eFindByDesc) {
        data_it = find_if(m_ItemData.begin(), m_ItemData.end(), PFindDesc(key));
    }

    if (data_it == m_ItemData.end()) {
        NCBI_THROW(CGuiConfigException, eConfigPanel, "key: " + key + " not found in panel's data.");
    }
    return *data_it;
}


CThemeConfigPanelBase::SItemData& CThemeConfigPanelBase::x_SetItemData(const string& key, EKeyType e )
{
    vector <SItemData>::iterator  data_it;

    if (e == eFindByType) {
        data_it = find_if(m_ItemData.begin(), m_ItemData.end(), PFindType(key));
    } else if (e == eFindByDesc) {
        data_it = find_if(m_ItemData.begin(), m_ItemData.end(), PFindDesc(key));
    }

    if (data_it == m_ItemData.end()) {
        NCBI_THROW(CGuiConfigException, eConfigPanel, "key: " + key + " not found in panel's data.");
    }
    return *data_it;

}


void CThemeConfigPanelBase::AddDetailAction(const string& type, IClickAction& act)
{
    m_DetailClickActions[type] = & act;
}


CRef<IClickAction> CThemeConfigPanelBase::GetClickAction(const string& type) const
{
    CRef<IClickAction> result;
    TClickActionMap::const_iterator it = m_DetailClickActions.find(type);
    if (it != m_DetailClickActions.end()) {
        result = it->second;
    }
    return result;
}


bool CThemeConfigPanelBase::IsClickActions() const
{
    return ! m_DetailClickActions.empty();
}



END_NCBI_SCOPE
