/*  $Id: theme_dlg.cpp 25627 2012-04-13 17:44:23Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <gui/widgets/config/theme_dlg.hpp>
#include <gui/widgets/config/config_panel.hpp>
#include <gui/widgets/config/edit_styles_dlg.hpp>

#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/stattext.h>
#include <wx/choice.h>

BEGIN_NCBI_SCOPE

/*!
 * CThemeDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CThemeDlg, wxDialog )


/*!
 * CThemeDlg event table definition
 */

BEGIN_EVENT_TABLE( CThemeDlg, wxDialog )

////@begin CThemeDlg event table entries
    EVT_CHOICE( ID_CHOICE1, CThemeDlg::OnThemeListSelected )

    EVT_BUTTON( ID_BUTTON2, CThemeDlg::OnShowStateClick )

    EVT_BUTTON( ID_BUTTON3, CThemeDlg::OnDefaultsClick )

    EVT_BUTTON( wxID_OK, CThemeDlg::OnOkClick )

    EVT_BUTTON( wxID_CANCEL, CThemeDlg::OnCancelClick )

////@end CThemeDlg event table entries

END_EVENT_TABLE()


/*!
 * CThemeDlg constructors
 */

CThemeDlg::CThemeDlg()
{
    Init();
}

CThemeDlg::CThemeDlg( IConfigMediator& mediator, wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    m_ConfigMediator.Reset(&mediator);

    Init();
    Create(parent, id, caption, pos, size, style);

    m_ConfigMediator->MakeTempData();

    SetTitle(m_ConfigMediator->GetWindowTitle());

    wxWindow* config_panel = m_ConfigMediator->GetPanel().CreateConfigPanel(this);
    wxWindow* itemWindow = FindWindow(ID_WINDOW);
    wxSizer* sizer = GetSizer();
    sizer->Replace(itemWindow, config_panel);
    itemWindow->Destroy();
    sizer->SetSizeHints(this);
    Fit();
}


/*!
 * CThemeDlg creator
 */

bool CThemeDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CThemeDlg creation
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CThemeDlg creation
    return true;
}


/*!
 * CThemeDlg destructor
 */

CThemeDlg::~CThemeDlg()
{
////@begin CThemeDlg destruction
////@end CThemeDlg destruction
}


/*!
 * Member initialisation
 */

void CThemeDlg::Init()
{
////@begin CThemeDlg member initialisation
////@end CThemeDlg member initialisation
}


/*!
 * Control creation for CThemeDlg
 */

void CThemeDlg::CreateControls()
{
////@begin CThemeDlg content construction
    CThemeDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_RIGHT|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemDialog1, wxID_STATIC, _("Theme:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemChoice5Strings;
    wxChoice* itemChoice5 = new wxChoice( itemDialog1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, itemChoice5Strings, 0 );
    itemBoxSizer3->Add(itemChoice5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton6 = new wxButton( itemDialog1, ID_BUTTON2, _("..."), wxDefaultPosition, wxSize(itemDialog1->ConvertDialogToPixels(wxSize(16, -1)).x, -1), 0 );
    itemBoxSizer3->Add(itemButton6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxWindow* itemWindow7 = new wxWindow( itemDialog1, ID_WINDOW, wxDefaultPosition, itemDialog1->ConvertDialogToPixels(wxSize(200, 200)), wxSIMPLE_BORDER );
    itemBoxSizer2->Add(itemWindow7, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer8 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer8, 0, wxGROW|wxALL, 5);

    wxButton* itemButton9 = new wxButton( itemDialog1, ID_BUTTON3, _("Default Values"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer8->Add(itemButton9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer8->Add(7, 8, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton11 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemButton11->SetDefault();
    itemBoxSizer8->Add(itemButton11, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton12 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer8->Add(itemButton12, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CThemeDlg content construction

    x_SetStates(m_ConfigMediator->GetStates());
    string first_state = m_ConfigMediator->GetCurrentState();
    x_SetLoadedState(first_state);
}


/*!
 * Should we show tooltips?
 */

bool CThemeDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CThemeDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CThemeDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CThemeDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CThemeDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CThemeDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CThemeDlg icon retrieval
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON3
 */

void CThemeDlg::x_SetStates(list<string> const& states)
{
    wxChoice* itemThemeList = (wxChoice*)FindWindow(ID_CHOICE1);
    itemThemeList->Clear();
    ITERATE(list<string>, state_it, states)
        itemThemeList->Append(state_it->c_str());
}

void CThemeDlg::x_SetLoadedState(const string& theme)
{
    wxChoice* itemThemeList = (wxChoice*)FindWindow(ID_CHOICE1);
    itemThemeList->SetStringSelection(theme.c_str());
}

string CThemeDlg::x_GetLoadedState()
{
    wxChoice* itemThemeList = (wxChoice*)FindWindow(ID_CHOICE1);
    return itemThemeList->GetStringSelection().c_str();
}

void CThemeDlg::OnDefaultsClick( wxCommandEvent& event )
{
    m_ConfigMediator->LoadDefault();
}

/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE1
 */

void CThemeDlg::OnThemeListSelected( wxCommandEvent& event )
{
    string this_state = x_GetLoadedState();

    // TODO: Make auto-save optional
    m_ConfigMediator->Save(kEmptyStr);

    m_ConfigMediator->Load(this_state);
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON2
 */

void CThemeDlg::OnShowStateClick( wxCommandEvent& event )
{
    CEditStylesDlg  dlg(*m_ConfigMediator, this);
    dlg.ShowModal();

    // populate our browser again
    x_SetStates(m_ConfigMediator->GetStates());

    // find out what the current state/style is and load that.
    string first_state = m_ConfigMediator->GetCurrentState();
    x_SetLoadedState(first_state);
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CThemeDlg::OnOkClick( wxCommandEvent& event )
{
    m_ConfigMediator->Save(kEmptyStr);
    m_ConfigMediator->MergeTempData();
    EndModal(wxID_OK);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
 */

void CThemeDlg::OnCancelClick( wxCommandEvent& event )
{
    m_ConfigMediator->CancelTempData();
    EndModal(wxID_CANCEL);
}

END_NCBI_SCOPE
