/*  $Id: theme_preview_mediator.cpp 15156 2007-10-11 17:35:18Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/config/theme_preview_mediator.hpp> // this class
#include <gui/config/theme_set.hpp>     // our data
#include <gui/widgets/config/theme_config_panel_base.hpp>    // our gui.

BEGIN_NCBI_SCOPE


CThemePreviewMediator::CThemePreviewMediator(CThemeSet & data, IThemeConfigPanelBase& panel)
    : CConfigMediator(data, panel)
{
}

CThemePreviewMediator::~CThemePreviewMediator()
{
}


/// What should we title the dialog window?
string CThemePreviewMediator::GetWindowTitle(void) const
{
    return "Themes";
}


/// Load data into that panel
void CThemePreviewMediator::Load(const string& state)
{}


/// Load 'factory default' data into that panel.
void CThemePreviewMediator::LoadDefault(void)
{}


/// Save the panel's data.  The only operating part of this class.
void CThemePreviewMediator::Save(const string& )
{
    IThemeConfigPanelBase& my_gui = x_GetMyPanel();
    CThemeSet& my_data = x_GetMyDataSet();

    vector<string>  items = my_data.GetTypes();
    ITERATE(vector<string>, item_it, items) {
        my_data.SetSetsCurrentStyleName(*item_it, my_gui.GetCurrentStyle(*item_it));
    }
}


// the following accessors are just to hide the ugly down casts.
CThemeSet& CThemePreviewMediator::x_GetMyDataSet(void)
{
    return dynamic_cast<CThemeSet&>(*m_DataSet);
}


const CThemeSet& CThemePreviewMediator::x_GetMyDataSet(void) const
{
    return dynamic_cast<const CThemeSet&>(*m_DataSet);
}


IThemeConfigPanelBase& CThemePreviewMediator::x_GetMyPanel(void)
{
    return dynamic_cast<IThemeConfigPanelBase&>(*m_ConfigPanel);
}

END_NCBI_SCOPE
