/*  $Id: authornames_panel.cpp 27497 2013-02-26 14:57:50Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

#include <gui/objutils/undo_manager_interface.hpp>
#include <gui/widgets/edit/edit_object.hpp>

#include <objects/biblio/Auth_list.hpp>
#include <objects/biblio/Author.hpp>
#include <objects/general/Person_id.hpp>
#include <objects/general/Name_std.hpp>

#include <gui/widgets/edit/serial_member_primitive_validators.hpp>

////@begin includes
////@end includes

#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/scrolwin.h>
#include <wx/hyperlink.h>
#include <wx/statline.h>
#include <wx/sizer.h>
#include <wx/stattext.h>

#include <gui/widgets/edit/authornames_panel.hpp>
#include "singleauthor_panel.hpp"
#include "consortium_panel.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*
 * CAuthorNamesPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CAuthorNamesPanel, wxPanel )


/*
 * CAuthorNamesPanel event table definition
 */

BEGIN_EVENT_TABLE( CAuthorNamesPanel, wxPanel )

////@begin CAuthorNamesPanel event table entries
    EVT_HYPERLINK( ID_ADDAUTH, CAuthorNamesPanel::OnAddauthHyperlinkClicked )

    EVT_HYPERLINK( ID_ADDCONSORTIUM, CAuthorNamesPanel::OnAddconsortiumHyperlinkClicked )

////@end CAuthorNamesPanel event table entries

END_EVENT_TABLE()


/*
 * CAuthorNamesPanel constructors
 */

CAuthorNamesPanel::CAuthorNamesPanel()
{
    Init();
}

CAuthorNamesPanel::CAuthorNamesPanel( wxWindow* parent, CSerialObject& object,
         wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) :
     m_Object(0)
{
    Init();
    m_Object = dynamic_cast<CAuth_list*>(&object);
    const CAuth_list& auth_list = dynamic_cast<const CAuth_list&>(*m_Object);
    m_EditedAuthList.Reset((CSerialObject*)CAuth_list::GetTypeInfo()->Create());
    m_EditedAuthList->Assign(auth_list);

    Create(parent, id, pos, size, style);
}


/*
 * CAuthorNamesPanel creator
 */

bool CAuthorNamesPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CAuthorNamesPanel creation
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CAuthorNamesPanel creation
    return true;
}


/*
 * CAuthorNamesPanel destructor
 */

CAuthorNamesPanel::~CAuthorNamesPanel()
{
////@begin CAuthorNamesPanel destruction
////@end CAuthorNamesPanel destruction
}


/*
 * Member initialisation
 */

void CAuthorNamesPanel::Init()
{
////@begin CAuthorNamesPanel member initialisation
    m_ScrolledWindow = NULL;
    m_Sizer = NULL;
////@end CAuthorNamesPanel member initialisation
}


/*
 * Control creation for CAuthorNamesPanel
 */

void CAuthorNamesPanel::CreateControls()
{    
////@begin CAuthorNamesPanel content construction
    CAuthorNamesPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxHyperlinkCtrl* itemHyperlinkCtrl4 = new wxHyperlinkCtrl( itemPanel1, ID_ADDAUTH, _("Add Author"), wxEmptyString, wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
    itemBoxSizer3->Add(itemHyperlinkCtrl4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxHyperlinkCtrl* itemHyperlinkCtrl5 = new wxHyperlinkCtrl( itemPanel1, ID_ADDCONSORTIUM, _("Add Consortium"), wxEmptyString, wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
    itemBoxSizer3->Add(itemHyperlinkCtrl5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticLine* itemStaticLine6 = new wxStaticLine( itemPanel1, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemBoxSizer2->Add(itemStaticLine6, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer7, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel1, wxID_STATIC, _("First Name"), wxDefaultPosition, wxSize(100, -1), wxALIGN_CENTRE );
    itemBoxSizer7->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemPanel1, wxID_STATIC, _("Middle Initial"), wxDefaultPosition, wxSize(100, -1), wxALIGN_CENTRE );
    itemBoxSizer7->Add(itemStaticText9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( itemPanel1, wxID_STATIC, _("Last (Family) Name"), wxDefaultPosition, wxSize(100, -1), wxALIGN_CENTRE );
    itemBoxSizer7->Add(itemStaticText10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText11 = new wxStaticText( itemPanel1, wxID_STATIC, _("Suffix"), wxDefaultPosition, wxSize(50, -1), wxALIGN_CENTRE );
    itemBoxSizer7->Add(itemStaticText11, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText12 = new wxStaticText( itemPanel1, wxID_STATIC, wxEmptyString, wxDefaultPosition, wxSize(40, -1), 0 );
    itemBoxSizer7->Add(itemStaticText12, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ScrolledWindow = new wxScrolledWindow( itemPanel1, ID_PANEL, wxDefaultPosition, wxSize(400, -1), wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(m_ScrolledWindow, 1, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_Sizer = new wxBoxSizer(wxVERTICAL);
    m_ScrolledWindow->SetSizer(m_Sizer);

    ////@end CAuthorNamesPanel content construction
}


bool CAuthorNamesPanel::TransferDataToWindow()
{
    Freeze();
    m_Sizer->Clear(true);

    m_RowHeight = 0;
    int total_height = 0;
    int total_width = 0;
    int num_rows = 0;
    int max_rows_displayed = 6;

    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    if (edited_auth_list.IsSetNames() && edited_auth_list.GetNames().IsStd() && !edited_auth_list.GetNames().GetStd().empty()) {
        for (list< CRef< CAuthor > >::iterator auth = edited_auth_list.SetNames().SetStd().begin();
             auth != edited_auth_list.SetNames().SetStd().end();
             auth++) {    
            if ((*auth)->GetName().IsName()) {
                wxWindow* row = new CSingleAuthorPanel(m_ScrolledWindow, **auth);
                m_Sizer->Add(row, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5); 
                int row_height, row_width;
                row->GetClientSize(&row_width, &row_height);
                if (row_width > total_width) {
                    total_width = row_width;
                }
                if (num_rows < max_rows_displayed) {
                    total_height += row_height;
                }
                num_rows++;
                if (m_RowHeight == 0 || m_RowHeight > row_height) {
                    m_RowHeight = row_height;
                }
            } else if ((*auth)->GetName().IsConsortium()) {
                wxWindow* row = new CConsortiumPanel (m_ScrolledWindow, **auth);
                m_Sizer->Add(row, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
                int row_height, row_width;
                row->GetClientSize(&row_width, &row_height);
                if (row_width > total_width) {
                    total_width = row_width;
                }
                if (num_rows < max_rows_displayed) {
                    total_height += row_height;
                }
                num_rows++;
                if (m_RowHeight == 0 || m_RowHeight > row_height) {
                    m_RowHeight = row_height;
                }
            }
        }
    }
    // now add some empty rows for user to fill in
    while (num_rows < max_rows_displayed) {
        CRef<CAuthor> auth(new CAuthor());
        edited_auth_list.SetNames().SetStd().push_back(auth);
        wxWindow* row = new CSingleAuthorPanel(m_ScrolledWindow, *auth);
        m_Sizer->Add(row, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5); 
        int row_height, row_width;
        row->GetClientSize(&row_width, &row_height);
        if (row_width > total_width) {
            total_width = row_width;
        }
        if (num_rows < max_rows_displayed) {
            total_height += row_height;
        }
        num_rows++;
        if (m_RowHeight == 0 || m_RowHeight > row_height) {
            m_RowHeight = row_height;
        }
    }
        
    m_ScrolledWindow->SetVirtualSize(total_width + 10, total_height);
    m_ScrolledWindow->SetScrollRate(0, m_RowHeight);

    m_ScrolledWindow->FitInside();
    Layout();
    Thaw();

    if (!wxPanel::TransferDataToWindow())
        return false;

    return true;
}


static bool s_IsAuthorEmpty (const CAuthor& auth)
{
    bool is_empty = true;

    if (!auth.IsSetName()) {
        is_empty = true;
    } else if (auth.GetName().IsName()) {
        if (auth.GetName().GetName().IsSetFirst()
            || auth.GetName().GetName().IsSetInitials()
            || auth.GetName().GetName().IsSetLast())
        {
            is_empty = false;
        }
    } else if (auth.GetName().IsConsortium()) {
        if (!NStr::IsBlank(auth.GetName().GetConsortium())) 
        {
            is_empty = false;
        }
    }
    return is_empty;
}


bool CAuthorNamesPanel::x_GetAuthors(bool keep_blank)
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    edited_auth_list.ResetNames();

    wxSizerItemList::iterator node = m_Sizer->GetChildren().begin();
    for (; node != m_Sizer->GetChildren().end(); ++node) {
        wxWindow *w = (*node)->GetWindow();
        if (!w) {
            continue;
        }
        CRef<CAuthor> edited_auth;
        CSingleAuthorPanel* singleauth_panel = dynamic_cast<CSingleAuthorPanel*> (w);
        if (singleauth_panel) {
            edited_auth = singleauth_panel->GetAuthor();
        } else {
            CConsortiumPanel *consort_panel = dynamic_cast<CConsortiumPanel*> (w);
            if (consort_panel) {
                edited_auth = consort_panel->GetAuthor();
            }
        }
        // only take the entries that aren't empty
        if (edited_auth && (keep_blank || !s_IsAuthorEmpty(*edited_auth))) {
            CRef<CAuthor> new_auth(new CAuthor());
            new_auth->Assign(*edited_auth);
            edited_auth_list.SetNames().SetStd().push_back(new_auth);
        }

    }
    return true;
}


bool CAuthorNamesPanel::TransferDataFromWindow()
{
    return x_GetAuthors();
}


void CAuthorNamesPanel::PopulateAuthors (CAuth_list& auth_list)
{
    TransferDataFromWindow();
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    if (edited_auth_list.IsSetNames()) {
        auth_list.SetNames(edited_auth_list.SetNames());      
    } else {
        auth_list.ResetNames();
    }
}


void CAuthorNamesPanel::SetAuthors(const objects::CAuth_list& auth_list)
{
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    edited_auth_list.Assign(auth_list);
    TransferDataToWindow();
}


static int s_FindRow(wxWindow* wnd, wxSizerItemList& itemList)
{
    if (NULL == wnd)
        return -1;

    int row_num = 0;
    for(wxSizerItemList::iterator it = itemList.begin(); it != itemList.end(); ++it, ++row_num) {
        wxWindow* child = (**it).GetWindow();
        if (child && child == wnd)
            return row_num;
    }

    return -1;
}


void CAuthorNamesPanel::DeleteAuthor (wxWindow* link)
{
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);

    // find the row that holds the author to be deleted
    wxSizerItemList& itemList = m_Sizer->GetChildren();

    int row_num = s_FindRow(link, itemList);
    if (row_num > -1) {
        x_GetAuthors(true);
        int row = 0;
        NON_CONST_ITERATE (list< CRef< CAuthor > >, it, edited_auth_list.SetNames().SetStd()) {
            if (row == row_num) {
                it = edited_auth_list.SetNames().SetStd().erase(it);
                break;
            }
            row++;
        }
        TransferDataToWindow();
    }


}


static bool s_IsLastRow(wxWindow* wnd, wxSizerItemList& itemList)
{
    bool is_last = false;
    if (NULL == wnd)
        return false;

    for(wxSizerItemList::iterator it = itemList.begin(); it != itemList.end(); ++it) {
        wxWindow* child = (**it).GetWindow();
        if (child && child == wnd) {
            is_last = true;
        } else {
            if (is_last) {
                is_last = false;
                return is_last;
            }
        }
    }

    return is_last;
}


void CAuthorNamesPanel::AddLastAuthor (wxWindow* link)
{
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);

    // find the row that holds the author being changed
    wxSizerItemList& itemList = m_Sizer->GetChildren();

    if (s_IsLastRow(link, itemList)) {
        CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
        CRef<CAuthor> auth(new CAuthor());
        edited_auth_list.SetNames().SetStd().push_back(auth);
        
        wxWindow* row = new CSingleAuthorPanel(m_ScrolledWindow, *auth);
        s_AddRowToWindow (row);
    }
}

/*
 * Should we show tooltips?
 */

bool CAuthorNamesPanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CAuthorNamesPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CAuthorNamesPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CAuthorNamesPanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CAuthorNamesPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CAuthorNamesPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CAuthorNamesPanel icon retrieval
}


/*
 * wxEVT_COMMAND_HYPERLINK event handler for ID_ADDAUTH
 */

void CAuthorNamesPanel::OnAddauthHyperlinkClicked( wxHyperlinkEvent& event )
{
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    CRef<CAuthor> auth(new CAuthor());
    edited_auth_list.SetNames().SetStd().push_back(auth);
    
    wxWindow* row = new CSingleAuthorPanel(m_ScrolledWindow, *auth);
    s_AddRowToWindow (row);
}


/*
 * wxEVT_COMMAND_HYPERLINK event handler for ID_ADDCONSORTIUM
 */

void CAuthorNamesPanel::OnAddconsortiumHyperlinkClicked( wxHyperlinkEvent& event )
{
    CAuth_list& edited_auth_list = dynamic_cast<CAuth_list&>(*m_EditedAuthList);
    CRef<CAuthor> auth(new CAuthor());
    edited_auth_list.SetNames().SetStd().push_back(auth);
    
    wxWindow* row = new CConsortiumPanel(m_ScrolledWindow, *auth);
    s_AddRowToWindow (row);
}


// utility function for adding new control to window
// adjusts scroll rate if necessary
void CAuthorNamesPanel::s_AddRowToWindow(wxWindow* row)
{
    m_Sizer->Add(row, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
    int row_height;
    row->GetClientSize(0, &row_height);
    if (m_RowHeight == 0 || row_height < m_RowHeight) {
        m_RowHeight = row_height;
        m_ScrolledWindow->SetScrollRate(0, row_height);
    }
    m_ScrolledWindow->FitInside();

    int h, hv;
    m_ScrolledWindow->GetClientSize(0, &h);
    m_ScrolledWindow->GetVirtualSize(0, &hv);

    if (hv > h) {
        int xUnit, yUnit;
        m_ScrolledWindow->GetScrollPixelsPerUnit(&xUnit, &yUnit);
        m_ScrolledWindow->Scroll(-1, (hv - h + yUnit)/yUnit);
    }
    m_ScrolledWindow->Refresh();
}


END_NCBI_SCOPE

