/*  $Id: biosourcepanel.cpp 27459 2013-02-20 16:35:16Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vasuki Palanigobu
 */

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include <ncbi_pch.hpp>
#include <objects/seqfeat/Org_ref.hpp>
#include <objects/seqfeat/OrgName.hpp>
#include <objects/taxon1/taxon1.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/message_box.hpp>

//#include <gui/widgets/wx/sys_path.hpp>
//#include <gui/widgets/wx/auto_complete_combo.hpp>
#include <gui/widgets/wx/auto_complete_text_ctrl.hpp>
#include <gui/widgets/wx/taxon_text_completer.hpp>
#include "biosourcepanel.hpp"

#ifdef _DEBUG
#include <serial/objostr.hpp>
#include <serial/serial.hpp>
#endif


////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

/*
 * CBioSourcePanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBioSourcePanel, wxPanel )


/*
 * CBioSourcePanel event table definition
 */

BEGIN_EVENT_TABLE( CBioSourcePanel, wxPanel )

////@begin CBioSourcePanel event table entries
    EVT_BUTTON( ID_BUTTON, CBioSourcePanel::OnTaxonomyLookup )

////@end CBioSourcePanel event table entries

END_EVENT_TABLE()


/*
 * CBioSourcePanel constructors
 */
static CBioSource dummy;

CBioSourcePanel::CBioSourcePanel():
    m_Source(dummy)
{
    Init();
}

CBioSourcePanel::CBioSourcePanel( wxWindow* parent, objects::CBioSource& source, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ):
    m_Source(source)
{
	Init();
    Create(parent, id, pos, size, style);
}


/*
 * OrganismNamePanel creator
 */

bool CBioSourcePanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBioSourcePanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CBioSourcePanel creation
    return true;
}


/*
 * CBioSourcePanel destructor
 */

CBioSourcePanel::~CBioSourcePanel()
{
////@begin CBioSourcePanel destruction
////@end CBioSourcePanel destruction
}


/*
 * Member initialisation
 */

void CBioSourcePanel::Init()
{
////@begin CBioSourcePanel member initialisation
    m_ScientificNameCtrl = NULL;
    m_CommonNameCtrl = NULL;
    m_LineageCtrl = NULL;
    m_DivisionCtrl = NULL;
////@end CBioSourcePanel member initialisation
}


/*
 * Control creation for OrganismNamePanel
 */

void CBioSourcePanel::CreateControls()
{    
////@begin CBioSourcePanel content construction
    CBioSourcePanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer3 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer3, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("Scientific Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText4, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ScientificNameCtrl = new CAutoCompleteTextCtrl( itemPanel1, ID_TEXTCTRL, wxEmptyString, wxDefaultPosition, wxSize(200, -1), 0 );
    itemFlexGridSizer3->Add(m_ScientificNameCtrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("Common Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText6, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CommonNameCtrl = new wxTextCtrl( itemPanel1, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_CommonNameCtrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel1, wxID_STATIC, _("Taxonomic Lineage"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText8, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_LineageCtrl = new wxTextCtrl( itemPanel1, ID_TEXTCTRL2, wxEmptyString, wxDefaultPosition, wxSize(200, -1), wxTE_MULTILINE );
    itemFlexGridSizer3->Add(m_LineageCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( itemPanel1, wxID_STATIC, _("Division"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText10, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DivisionCtrl = new wxTextCtrl( itemPanel1, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_DivisionCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton12 = new wxButton( itemPanel1, ID_BUTTON, _("Taxonomy Lookup"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemButton12, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

////@end CBioSourcePanel content construction
	//m_ScientificNameCtrl->SetIgnoreCase(true);
	m_ScientificNameCtrl->AutoComplete(new CTaxonTextCompleter());
	
}

bool CBioSourcePanel::TransferDataToWindow(){
    if (!wxPanel::TransferDataToWindow())
		return false;
	x_GetOrganismInfo();
	return true;
}

bool CBioSourcePanel::TransferDataFromWindow() {
    if (!wxPanel::TransferDataFromWindow())
		return false;
	x_SetOrganismInfo();
	return true;
}


void CBioSourcePanel::x_GetOrganismInfo() {
	if (m_Source.IsSetOrg()) {
		if (m_Source.GetOrg().IsSetTaxname()) {
			m_ScientificNameCtrl->SetValue(ToWxString(m_Source.GetOrg().GetTaxname()));
		}
        else {
            m_ScientificNameCtrl->Clear();
        }
		if (m_Source.GetOrg().IsSetCommon()) {
			m_CommonNameCtrl->SetValue(ToWxString(m_Source.GetOrg().GetCommon()));
		}
        else {
            m_CommonNameCtrl->Clear();
        }
		if (m_Source.GetOrg().IsSetOrgname()) {
			if (m_Source.GetOrg().GetOrgname().IsSetLineage()) {
				m_LineageCtrl->SetValue(ToWxString(m_Source.GetOrg().GetOrgname().GetLineage()));
			}
            else {
                m_LineageCtrl->Clear();
            }
			if (m_Source.GetOrg().GetOrgname().IsSetDiv()) {
				m_DivisionCtrl->SetValue(ToWxString(m_Source.GetOrg().GetOrgname().GetDiv()));
			}
            else {
                m_DivisionCtrl->Clear();
            }
		}
	}
}

void CBioSourcePanel::x_SetOrganismInfo() {
	m_Source.SetOrg().SetTaxname(ToStdString(m_ScientificNameCtrl->GetValue()));
	m_Source.SetOrg().SetCommon(ToStdString(m_CommonNameCtrl->GetValue()));
	m_Source.SetOrg().SetOrgname().SetLineage(ToStdString(m_LineageCtrl->GetValue()));
	m_Source.SetOrg().SetOrgname().SetDiv(ToStdString(m_DivisionCtrl->GetValue()));
}


void CBioSourcePanel::PopulateBioSource(CBioSource& source) {
	if (m_Source.IsSetOrg()) {
		if (m_Source.GetOrg().IsSetTaxname()) {
			source.SetOrg().SetTaxname(m_Source.GetOrg().GetTaxname());
		}
		if (m_Source.GetOrg().IsSetCommon()) {
			source.SetOrg().SetCommon(m_Source.GetOrg().GetCommon());
		}
		if (m_Source.GetOrg().IsSetOrgname()) {
			if (m_Source.GetOrg().GetOrgname().IsSetLineage()) {
				source.SetOrg().SetOrgname().SetLineage(m_Source.GetOrg().GetOrgname().GetLineage());
			}
			if (m_Source.GetOrg().GetOrgname().IsSetDiv()) {
				source.SetOrg().SetOrgname().SetDiv(m_Source.GetOrg().GetOrgname().GetDiv());
			}
		}
	}

#ifdef _DEBUG
    CNcbiOstrstream ostr;
    {{
        auto_ptr<CObjectOStream>
            out(CObjectOStream::Open(eSerial_AsnText, ostr));
        *out << m_Source;
    }}
    _TRACE(string(CNcbiOstrstreamToString(ostr)));
#endif

}

/*
 * Should we show tooltips?
 */

bool CBioSourcePanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CBioSourcePanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBioSourcePanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBioSourcePanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CBioSourcePanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBioSourcePanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBioSourcePanel icon retrieval
}


/*
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
 */

void CBioSourcePanel::OnTaxonomyLookup( wxCommandEvent& event )
{
    CTaxon1 lookup;
    lookup.Init();

    x_SetOrganismInfo();
    string name = m_Source.GetOrg().GetTaxname();
    if (name.empty()) {
        name = m_Source.GetOrg().GetCommon();
    }
    if (name.empty()) {
        return;
    }

    // Note:
    // The all-in-one CTaxon1::LookupMerge() function will not take common names 
    // into account. Hence the two-step approach.
    //
    CRef<CTaxon2_data> pLookupData = lookup.GetById(lookup.GetTaxIdByName(name));
    if (!pLookupData) {
        string message = string("Organism ") + m_Source.GetOrg().GetTaxname() +
            + " could not be found.";
        NcbiErrorBox(
            message,
            "Lookup Error");
        return;
    }
    m_Source.SetOrg().Assign(pLookupData->SetOrg());
    //m_Source.ResetSubtype(); 
    GetParent()->TransferDataToWindow();
    event.Skip();
}

END_NCBI_SCOPE
