/*  $Id: comment_panel.cpp 25347 2012-03-01 18:22:54Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include "comment_panel.hpp"

#include <gui/widgets/edit/serial_member_primitive_validators.hpp>

#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

BEGIN_NCBI_SCOPE

////@begin XPM images
////@end XPM images


/*!
 * CCommentPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CCommentPanel, wxPanel )


/*!
 * CCommentPanel event table definition
 */

BEGIN_EVENT_TABLE( CCommentPanel, wxPanel )

////@begin CCommentPanel event table entries
////@end CCommentPanel event table entries

END_EVENT_TABLE()


/*!
 * CCommentPanel constructors
 */

CCommentPanel::CCommentPanel() : m_Object(0)
{
    Init();
}

CCommentPanel::CCommentPanel( wxWindow* parent, CSerialObject& object, objects::CScope&,
                              wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
    : m_Object(&object)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * Commentpanel creator
 */

bool CCommentPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCommentPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CCommentPanel creation
    return true;
}


/*!
 * CCommentPanel destructor
 */

CCommentPanel::~CCommentPanel()
{
////@begin CCommentPanel destruction
////@end CCommentPanel destruction
}


/*!
 * Member initialisation
 */

void CCommentPanel::Init()
{
////@begin CCommentPanel member initialisation
////@end CCommentPanel member initialisation
}


/*!
 * Control creation for Commentpanel
 */

void CCommentPanel::CreateControls()
{
////@begin CCommentPanel content construction
    // Generated by DialogBlocks, 11/01/2012 09:29:05 (unregistered)

    CCommentPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 1, wxGROW|wxALL, 0);

    wxTextCtrl* itemTextCtrl4 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL7, wxEmptyString, wxDefaultPosition, itemPanel1->ConvertDialogToPixels(wxSize(200, 100)), wxTE_MULTILINE );
    if (CCommentPanel::ShowToolTips())
        itemTextCtrl4->SetToolTip(_("Enter general comments about the feature here."));
    itemBoxSizer3->Add(itemTextCtrl4, 1, wxGROW|wxALL, 0);

    // Set validators
    itemTextCtrl4->SetValidator( CSerialTextValidator(*m_Object, "comment") );
////@end CCommentPanel content construction
}


/*!
 * Should we show tooltips?
 */

bool CCommentPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCommentPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCommentPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCommentPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCommentPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCommentPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCommentPanel icon retrieval
}

END_NCBI_SCOPE
