/*  $Id: contactpanel.cpp 27653 2013-03-20 17:05:20Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <gui/widgets/edit/contactpanel.hpp>
#include <gui/widgets/edit/serial_member_primitive_validators.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <objects/submit/Contact_info.hpp>
#include <objects/biblio/Author.hpp>
#include <objects/general/Person_id.hpp>
#include <objects/general/Name_std.hpp>

#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/choice.h>


BEGIN_NCBI_SCOPE

////@begin XPM images
////@end XPM images


/*
 * CContactPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CContactPanel, wxPanel )


/*
 * CContactPanel event table definition
 */

BEGIN_EVENT_TABLE( CContactPanel, wxPanel )

////@begin CContactPanel event table entries
////@end CContactPanel event table entries

END_EVENT_TABLE()


/*
 * CContactPanel constructors
 */

CContactPanel::CContactPanel()
{
    Init();
}

CContactPanel::CContactPanel( wxWindow* parent, CSerialObject& object, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
    : m_Object(&object)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * CContactPanel creator
 */

bool CContactPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CContactPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CContactPanel creation
    return true;
}


/*
 * CContactPanel destructor
 */

CContactPanel::~CContactPanel()
{
////@begin CContactPanel destruction
////@end CContactPanel destruction
}


/*
 * Member initialisation
 */

void CContactPanel::Init()
{
////@begin CContactPanel member initialisation
    m_First = NULL;
    m_MI = NULL;
    m_Last = NULL;
    m_Suffix = NULL;
    m_Phone = NULL;
    m_Fax = NULL;
////@end CContactPanel member initialisation
}


/*
 * Control creation for CContactPanel
 */

void CContactPanel::CreateControls()
{    
////@begin CContactPanel content construction
    CContactPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer3 = new wxFlexGridSizer(2, 4, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("First Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText4, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("M.I."), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText5, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("Last Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText6, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Suffix"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText7, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_First = new wxTextCtrl( itemPanel1, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_First, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_MI = new wxTextCtrl( itemPanel1, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxSize(50, -1), 0 );
    itemFlexGridSizer3->Add(m_MI, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_Last = new wxTextCtrl( itemPanel1, ID_TEXTCTRL6, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_Last, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxArrayString m_SuffixStrings;
    m_SuffixStrings.Add(wxEmptyString);
    m_SuffixStrings.Add(_("Jr."));
    m_SuffixStrings.Add(_("Sr."));
    m_SuffixStrings.Add(_("II"));
    m_SuffixStrings.Add(_("III"));
    m_SuffixStrings.Add(_("IV"));
    m_SuffixStrings.Add(_("V"));
    m_SuffixStrings.Add(_("VI"));
    m_Suffix = new wxChoice( itemPanel1, ID_CHOICE, wxDefaultPosition, wxDefaultSize, m_SuffixStrings, 0 );
    itemFlexGridSizer3->Add(m_Suffix, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxStaticText* itemStaticText12 = new wxStaticText( itemPanel1, wxID_STATIC, _("Please include country code for non-U.S. phone numbers."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText12, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer13 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer13, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText14 = new wxStaticText( itemPanel1, wxID_STATIC, _("Phone"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText14, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer15 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer13->Add(itemBoxSizer15, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_Phone = new wxTextCtrl( itemPanel1, ID_TEXTCTRL7, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer15->Add(m_Phone, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText17 = new wxStaticText( itemPanel1, wxID_STATIC, _("Fax"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer15->Add(itemStaticText17, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Fax = new wxTextCtrl( itemPanel1, ID_TEXTCTRL8, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer15->Add(m_Fax, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText19 = new wxStaticText( itemPanel1, wxID_STATIC, _("Email"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText19, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl20 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL9, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemTextCtrl20, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Set validators
    m_Phone->SetValidator( CSerialAsciiTextValidator(*m_Object, "phone") );
    m_Fax->SetValidator( CSerialAsciiTextValidator(*m_Object, "fax") );
    itemTextCtrl20->SetValidator( CSerialAsciiTextValidator(*m_Object, "email") );
////@end CContactPanel content construction
}


static string s_GetFirstNameInitials(string first_name)
{
    char  buf[2];

    string inits = "";
    buf[1] = 0;

    string::iterator p = first_name.begin();
    while (p != first_name.end()) {
        // skip leading punct
        while (p != first_name.end() && (*p <= ' ' || *p == '-')) {
            p++;
        }
        if (p != first_name.end() && isalpha (*p)) {
            buf[0] = *p;
            inits.append(buf);
            p++;
        }
        // skip rest of name
        while (p != first_name.end() && *p > ' ' && *p != '-') {
            p++;
        }
        if (p != first_name.end() && *p == '-') {
            buf[0] = *p;
            inits.append(buf);
            p++;
        }
    }
    return inits;
}



bool CContactPanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow()) {
        return false;
    }

    objects::CContact_info * contact = dynamic_cast<objects::CContact_info*>(m_Object);
    if (contact->IsSetContact() && contact->GetContact().IsSetName()
        && contact->GetContact().GetName().IsName()) {
          const objects::CName_std& person = contact->GetContact().GetName().GetName();
        if (person.IsSetFirst()) {
            m_First->SetValue(person.GetFirst());
        }
        if (person.IsSetLast()) {
            m_Last->SetValue(person.GetLast());
        }
        if (person.IsSetSuffix()) {
            m_Suffix->SetStringSelection (person.GetSuffix());
        }
        if (person.IsSetInitials()) {
            string first_init = s_GetFirstNameInitials(ToAsciiStdString (m_First->GetValue()));
            string init = person.GetInitials();
            // chop out initials from first name
            if (NStr::StartsWith(init, first_init, NStr::eNocase)) {
                init = init.substr(first_init.length());
            }
            m_MI->SetValue (ToWxString (init));
        }
    } else {
        m_First->SetValue(wxEmptyString);
        m_Last->SetValue(wxEmptyString);
        m_MI->SetValue(wxEmptyString);
        m_Suffix->SetStringSelection(wxEmptyString);
    }

    return true;
}


bool CContactPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow()) {
        return false;
    }

    objects::CContact_info * contact = dynamic_cast<objects::CContact_info*>(m_Object);
    objects::CName_std& person = contact->SetContact().SetName().SetName();
    person.SetFirst(ToAsciiStdString (m_First->GetValue()));
    person.SetLast(ToAsciiStdString (m_Last->GetValue()));
    person.SetSuffix(ToAsciiStdString (m_Suffix->GetStringSelection()));
    string init = s_GetFirstNameInitials(ToAsciiStdString(m_First->GetValue()));
    string mi = ToAsciiStdString (m_MI->GetValue());
    if (!NStr::IsBlank(mi)) {
        if (NStr::IsBlank(init)) {
            init += ".";
        }
        init += mi;
    }
    person.SetInitials(init);
    if (NStr::IsBlank(person.GetFirst())) {
        person.ResetFirst();
    }
    if (NStr::IsBlank(person.GetLast())) {
        person.ResetLast();
    }
    if (NStr::IsBlank(person.GetSuffix())) {
        person.ResetSuffix();
    }
    if (NStr::IsBlank(person.GetInitials())) {
        person.ResetInitials();
    }
    return true;
}

/*
 * Should we show tooltips?
 */

bool CContactPanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CContactPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CContactPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CContactPanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CContactPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CContactPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CContactPanel icon retrieval
}
END_NCBI_SCOPE
