#ifndef GUI_WIDGETS_EDIT___EDIT_OBJECT_FEATURE_PROPAGATE__HPP
#define GUI_WIDGETS_EDIT___EDIT_OBJECT_FEATURE_PROPAGATE__HPP

/*  $Id: edit_object_feature_propagate.hpp 27944 2013-04-29 15:24:08Z ludwigf $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */

#include <corelib/ncbistd.hpp>

#include <objmgr/scope.hpp>
#include <objmgr/seq_entry_handle.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objmgr/util/sequence.hpp>
#include <objects/seqfeat/Seq_feat.hpp>
#include <gui/widgets/edit/edit_object.hpp>

#include <map>
#include <wx/sizer.h>
#include <wx/combobox.h>
#include <wx/checkbox.h>
#include <wx/radiobox.h>

class wxTreebook;
class wxPanel;
class wxSizer;

BEGIN_NCBI_SCOPE

class CEditObjectFeaturePropagate : public CObject, public IEditObject
{
public:
    CEditObjectFeaturePropagate(
        objects::CBioseq_Handle bsh,
        std::vector<const objects::CSeq_feat*>& propagatedFeats)
        : m_PropagatedFeats(propagatedFeats), m_Bsh(bsh), m_Window() 
    { xInitSequenceMap();}

    virtual wxWindow* CreateWindow(wxWindow* parent);
    virtual IEditCommand* GetEditCommand();

private:
    CConstRef<CObject> m_Object;
    CRef<CSerialObject> m_EditedFeature;
    CIRef<IEditCommand> m_EditAction;
    std::vector<const objects::CSeq_feat*>& m_PropagatedFeats;

private:
    void xInitSequenceMap();
    IEditCommand* xGetEditCommandAllSequences();
    IEditCommand* xGetEditCommandSingleSequence(objects::CBioseq_Handle);
    wxSizer* xTargetSizer();
    wxSizer* xFeatureSelectSizer();
    wxSizer* xGapHandlingSizer();
    wxSizer* xCdsHandlingSizer();

    bool xStopCdsAtStopCodon() const;

    objects::CBioseq_Handle m_Bsh;

    objects::CSeq_entry_Handle m_SEH;
    bool m_CreateMode;

    //CRef<CSerialObject> m_EditedDescriptor;
    //CRef<CSerialObject> m_EditedBioseq;

    wxArrayString m_SeqStrings;
    map<string, objects::CBioseq_Handle> m_SeqMap;

    wxPanel* m_Window;
    wxComboBox* m_pTargetCtrl;
    wxRadioBox* m_pFeatureSelectCtrl;
    wxRadioBox* m_pGapHandlingCtrl;
    wxCheckBox* m_pStopCdsAtStopCodon;
    wxCheckBox* m_pExtendCdsPastPartialThreePrime;
    wxCheckBox* m_pCleanupCdsPartials;
    wxCheckBox* m_pFuseAdjacentIntervals;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___EDIT_OBJECT_FEATURE_PROPAGATE__HPP
