#ifndef GUI_WIDGETS_EDIT___EDIT_OBJECT_SEQ_DESC__HPP
#define GUI_WIDGETS_EDIT___EDIT_OBJECT_SEQ_DESC__HPP

/*  $Id: edit_object_seq_desc.hpp 25868 2012-05-25 15:51:36Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */

#include <corelib/ncbistd.hpp>

#include <objmgr/scope.hpp>
#include <objmgr/seq_entry_handle.hpp>

#include <gui/widgets/edit/edit_object.hpp>

class wxTreebook;
class wxPanel;

BEGIN_NCBI_SCOPE

class CEditObjectSeq_desc : public CObject, public IEditObject
{
public:
    CEditObjectSeq_desc(const CObject& object,
                        const objects::CSeq_entry_Handle& seh,
                        objects::CScope& scope, bool createMode)
        : m_Object(&object), m_Scope(&scope), m_SEH(seh), m_CreateMode(createMode), m_Window() {}

    virtual wxWindow* CreateWindow(wxWindow* parent);
    virtual IEditCommand* GetEditCommand();

private:
    enum {
        ID_NOTEBOOK = 10015
    };

    CConstRef<CObject> m_Object;
    CConstRef<CObject> m_Bioseq;
    mutable CRef<objects::CScope> m_Scope;

    objects::CSeq_entry_Handle m_SEH;
    bool m_CreateMode;

    CRef<CSerialObject> m_EditedDescriptor;
    CRef<CSerialObject> m_EditedBioseq;
    CIRef<IEditCommand> m_EditAction;

    wxPanel* m_Window;
    wxTreebook* m_SourceTreebook;
};

class CEditObjectPubdesc : public CObject, public IEditObject
{
public:
    CEditObjectPubdesc(const CObject& object, objects::CScope& scope)
        : m_Object(&object), m_Scope(&scope), m_Window() {}

    virtual wxWindow* CreateWindow(wxWindow* parent);
    virtual IEditCommand* GetEditCommand();

private:
    enum {
        ID_NOTEBOOK = 10015
    };

    CConstRef<CObject> m_Object;
    mutable CRef<objects::CScope> m_Scope;

    CRef<CSerialObject> m_EditedPubdesc;
    CIRef<IEditCommand> m_EditAction;

    wxPanel* m_Window;
};

class CEditObjectMolInfo : public CObject, public IEditObject
{
public:
    CEditObjectMolInfo(const CObject& object, objects::CScope& scope)
        : m_Object(&object), m_Scope(&scope), m_Window() {}

    virtual wxWindow* CreateWindow(wxWindow* parent);
    virtual IEditCommand* GetEditCommand();

private:
    enum {
        ID_NOTEBOOK = 10015
    };

    CConstRef<CObject> m_Object;
    CConstRef<CObject> m_Bioseq;
    mutable CRef<objects::CScope> m_Scope;

    CRef<CSerialObject> m_EditedMolInfo;
    CRef<CSerialObject> m_EditedBioseq;
    CIRef<IEditCommand> m_EditAction;

    wxPanel* m_Window;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___EDIT_OBJECT_SEQ_DESC__HPP
