/*  $Id: edit_object_submit_block.cpp 25922 2012-06-06 12:52:05Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Colleen Bollin, based on a file by Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include "edit_object_submit_block.hpp"

#include <gui/objutils/undo_manager_interface.hpp>
#include <gui/objutils/descriptor_change.hpp>
#include <gui/objutils/cmd_create_desc.hpp>

#include <objects/submit/Submit_block.hpp>
#include <objects/submit/Contact_info.hpp>
#include <objects/biblio/Cit_sub.hpp>
#include <objects/biblio/Auth_list.hpp>
#include <objects/biblio/Affil.hpp>
#include <objects/misc/sequence_macros.hpp>

#include <objmgr/util/sequence.hpp>
#include <objmgr/seqdesc_ci.hpp>
#include <objmgr/bioseq_ci.hpp>

#include <wx/panel.h>
#include <wx/treebook.h>


#ifdef _DEBUG
#include <serial/objostr.hpp>
#include <serial/serial.hpp>
#endif

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


wxWindow* CEditObjectSubmit_block::CreateWindow(wxWindow* parent)
{
    if (m_Window) {
        NCBI_THROW(CException, eUnknown, 
                   "CEditObjectSubmit_block::CreateWindow - Window already created!");
    }

    const CSubmit_block& submit_block = dynamic_cast<const CSubmit_block&>(*m_Object);

    m_EditedSubmitBlock.Reset((CSerialObject*)CSubmit_block::GetTypeInfo()->Create());
    m_EditedSubmitBlock->Assign(submit_block);

    CSubmit_block& edited_submit_block = dynamic_cast<CSubmit_block&>(*m_EditedSubmitBlock);

		m_Window = new wxPanel(parent);
        				
		wxBoxSizer* main_sizer = new wxBoxSizer(wxHORIZONTAL);
    m_Window->SetSizer(main_sizer);

    m_Treebook = new wxTreebook(m_Window, wxID_ANY, wxDefaultPosition, wxSize(-1, 320), wxBK_DEFAULT );
    main_sizer->Add(m_Treebook, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_SubmitBlockPanel = new CSubmitBlockPanel(m_Treebook, edited_submit_block);
    m_Treebook->AddPage(m_SubmitBlockPanel, wxT("Submission"));

    m_ContactPanel = new CContactPanel (m_Treebook, edited_submit_block.SetContact());
		m_Treebook->AddPage(m_ContactPanel, wxT("Contact"));

    CAuth_list& auth_list = edited_submit_block.SetCit().SetAuthors();
    m_AuthorsPanel = new CAuthorNamesPanel(m_Treebook, auth_list);
    m_Treebook->AddPage (m_AuthorsPanel, wxT("Authors"));
    CRef<objects::CAffil> affil(&(auth_list.SetAffil()));

    m_AffilPanel = new CAuthorAffiliationPanel(m_Treebook, affil);
    m_Treebook->AddPage (m_AffilPanel, wxT("Affiliation"));

    return m_Window;
}


IEditCommand* CEditObjectSubmit_block::GetEditCommand()
{
    if (!m_Window)
        return 0;

    if (!m_EditAction) {
        CSubmit_block& edited_submit_block = dynamic_cast<CSubmit_block&>(*m_EditedSubmitBlock);
        m_SubmitBlockPanel->TransferDataFromWindow();
        m_ContactPanel->TransferDataFromWindow();
        m_AuthorsPanel->TransferDataFromWindow();
        m_AuthorsPanel->PopulateAuthors(edited_submit_block.SetCit().SetAuthors());
        m_AffilPanel->TransferDataFromWindow();

        try {
            if (m_CreateMode) {
#if 0
                    m_EditAction.Reset(new CCmdCreateDesc(m_SEH,
                        dynamic_cast<const CSeqdesc&>(m_EditedDescriptor.GetObject())));
#endif
            } else {
                CChangeSubmitBlockCommand* cmd = new CChangeSubmitBlockCommand();
                CObject* actual = (CObject*) m_Object.GetPointer();
                cmd->Add(actual, CConstRef<CObject>(m_EditedSubmitBlock));
                m_EditAction.Reset(cmd);
            }
        } catch (CException& e) {
            LOG_POST(Error << "CEditObjectSeq_desc::GetEditAction(): " << e.GetMsg());
        }
    }

    return m_EditAction.GetPointer();
}


END_NCBI_SCOPE
