/*  $Id: featuregenepanel.cpp 25956 2012-06-14 13:12:27Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>
#include <objects/seqfeat/Seq_feat.hpp>
#include <objects/seqfeat/SeqFeatXref.hpp>
#include <objects/seqfeat/Gene_ref.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objects/seqloc/Seq_id.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/util/seq_loc_util.hpp>
#include <objmgr/util/sequence.hpp>
#include <gui/widgets/wx/wx_utils.hpp>


////@begin includes
////@end includes

#include "featuregenepanel.hpp"

#include <wx/sizer.h>
#include <wx/stattext.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*!
 * CFeatureGenePanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CFeatureGenePanel, wxPanel )


/*!
 * CFeatureGenePanel event table definition
 */

BEGIN_EVENT_TABLE( CFeatureGenePanel, wxPanel )

////@begin CFeatureGenePanel event table entries
    EVT_CHOICE( ID_CHOICE16, CFeatureGenePanel::OnSelectGene )

    EVT_CHECKBOX( ID_CHECKBOX13, CFeatureGenePanel::OnClickSuppress )

////@end CFeatureGenePanel event table entries

END_EVENT_TABLE()


static CBioseq_Handle BioseqHandleFromLocation (CScope* m_Scope, const CSeq_loc& loc)

{
    CBioseq_Handle bsh;
    for ( CSeq_loc_CI citer (loc); citer; ++citer) {
        const CSeq_id& id = citer.GetSeq_id();
        CSeq_id_Handle sih = CSeq_id_Handle::GetHandle(id);
        bsh = m_Scope->GetBioseqHandle (sih, CScope::eGetBioseq_All);
        if (bsh) {
            return bsh;
        }
    }
    return bsh;
}


/*!
 * CFeatureGenePanel constructors
 */

CFeatureGenePanel::CFeatureGenePanel()
{
    Init();
}

CFeatureGenePanel::CFeatureGenePanel( wxWindow* parent, CSerialObject& object, CScope& scope, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
    : m_Object(&object), m_Scope(&scope)
{
    Init();
    m_GeneRef = new CGene_ref();

    CSeq_feat * feat = dynamic_cast<CSeq_feat*>(m_Object);
    const CGene_ref* gref = feat->GetGeneXref();
    if (gref) {
        m_GeneRef->Assign (*gref);
    } else {
        // find gene currently associated with feature
    }
    
    Create(parent, id, pos, size, style);
}


/*!
 * CFeatureGenePanel creator
 */

bool CFeatureGenePanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CFeatureGenePanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CFeatureGenePanel creation
    return true;
}


/*!
 * CFeatureGenePanel destructor
 */

CFeatureGenePanel::~CFeatureGenePanel()
{
////@begin CFeatureGenePanel destruction
////@end CFeatureGenePanel destruction
}


/*!
 * Member initialisation
 */

void CFeatureGenePanel::Init()
{
////@begin CFeatureGenePanel member initialisation
    m_GeneChoice = NULL;
    m_MapType = NULL;
    m_Suppress = NULL;
////@end CFeatureGenePanel member initialisation
}


/*!
 * Control creation for CFeatureGenePanel
 */

void CFeatureGenePanel::CreateControls()
{    
////@begin CFeatureGenePanel content construction
    CFeatureGenePanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("Gene:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_GeneChoiceStrings;
    m_GeneChoice = new wxChoice( itemPanel1, ID_CHOICE16, wxDefaultPosition, wxDefaultSize, m_GeneChoiceStrings, 0 );
    itemBoxSizer3->Add(m_GeneChoice, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_MapTypeStrings;
    m_MapTypeStrings.Add(_("&Overlap"));
    m_MapTypeStrings.Add(_("&Cross-reference"));
    m_MapType = new wxRadioBox( itemPanel1, ID_RADIOBOX4, _("Map by"), wxDefaultPosition, wxDefaultSize, m_MapTypeStrings, 1, wxRA_SPECIFY_ROWS );
    m_MapType->SetSelection(0);
    itemBoxSizer2->Add(m_MapType, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_Suppress = new wxCheckBox( itemPanel1, ID_CHECKBOX13, _("Suppress Gene"), wxDefaultPosition, wxDefaultSize, 0 );
    m_Suppress->SetValue(false);
    itemBoxSizer2->Add(m_Suppress, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

////@end CFeatureGenePanel content construction

    m_GeneChoice->Append(wxEmptyString);
    m_GeneChoice->Append(_("New"));

    CSeq_feat * feat = dynamic_cast<CSeq_feat*>(m_Object);
    string match_label = "";
    const CGene_ref* gs = feat->GetGeneXref();
    if (gs) {        
        gs->GetLabel (&match_label);
        if (NStr::IsBlank (match_label)) {
            m_Suppress->SetValue(true);
            m_GeneChoice->Enable (false);
            m_MapType->Enable (false);
        } else {
            m_MapType->SetSelection(1);
        }
    } 
    m_OverlappingGene = sequence::GetOverlappingGene(feat->GetLocation(), *m_Scope);
    if (NStr::IsBlank (match_label)) {
        // pick out gene by overlap, in case user turns off suppression
        if ( m_OverlappingGene ) {
            m_OverlappingGene->GetData().GetGene().GetLabel(&match_label);
        }
    }

    bool found = false;
    int n = 2;
    m_GeneList.clear();
    CBioseq_Handle bsh = BioseqHandleFromLocation(m_Scope, feat->GetLocation());
    if (bsh) {
        CFeat_CI gene_it(bsh, SAnnotSelector (CSeqFeatData::e_Gene));
        while (gene_it) {
            string label = "";
            const CGene_ref& gr = gene_it->GetSeq_feat()->GetData().GetGene();
            CConstRef<CGene_ref> ref(&gr);
            m_GeneList.push_back (ref);
            gr.GetLabel (&label);
            if (!NStr::IsBlank (label)) {
                m_GeneChoice->Append(ToWxString(label));
                if (NStr::Equal (label, match_label)) {
                    m_GeneChoice->SetSelection (n);
                    found = true;
                }
                ++n;
            }
            ++gene_it;
        }        
    }
    if (!found) {
        // label not found
        if (NStr::IsBlank (match_label)) {
            m_GeneChoice->SetSelection (0);
        } else {
            // create new
            m_GeneChoice->SetSelection(1);
        }
    }

    m_GeneRefPanel = new CGenePanel( itemPanel1, *m_GeneRef, *m_Scope);
    itemBoxSizer2->Add(m_GeneRefPanel, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    if (m_GeneChoice->GetSelection() == 1) {
        m_GeneRefPanel->Enable (true);
    } else {
        m_GeneRefPanel->Enable (false);
    }
}


static void ClearGeneXref (CSeq_feat * feat)
{
    if (!feat->IsSetXref()) {
        return;
    }

    CSeq_feat::TXref::iterator it = feat->SetXref().begin();
    while (it != feat->SetXref().end()) {
        if ((*it)->IsSetData () && (*it)->GetData ().IsGene ()) {
            it = feat->SetXref().erase(it);
        } else {
            ++it;
        }
    }
    if (feat->SetXref().size() == 0) {
        feat->ResetXref();
    }
}


bool CFeatureGenePanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    CSeq_feat * feat = dynamic_cast<CSeq_feat*>(m_Object);
    if (m_Suppress->GetValue()) {
        CGene_ref& s = feat->SetGeneXref();
        s.Reset();
    } else if (m_MapType->GetSelection() == 1) {        
        int which_gene = m_GeneChoice->GetSelection();
        if (which_gene == 0) {
            // empty, same as suppress
            CGene_ref& s = feat->SetGeneXref();
            s.Reset();
        } else if (which_gene == 1) {
            // new gene            
            string label = "";
            m_GeneRef->GetLabel(&label);
            if (NStr::IsBlank (label)) {
                ClearGeneXref (feat);
            } else {
                // create new gene xref
                CGene_ref& s = feat->SetGeneXref();
                s.Assign(*m_GeneRef);
            }
        } else {
            // create gene xref to existing gene
            CGene_ref& s = feat->SetGeneXref();
            s.Assign(*m_GeneList[which_gene - 2]);            
        }
    } else {
        // clear gene xref if there is one, build gene
        ClearGeneXref (feat);
    }

    return true;
}


CRef<CSeq_feat> CFeatureGenePanel::GetNewGene()
{
    TransferDataFromWindow();
    CRef<CSeq_feat> gene_feat;
    if (!m_Suppress->GetValue() 
        && m_MapType->GetSelection() == 0
        && m_GeneChoice->GetSelection() == 1) 
    {        
        gene_feat.Reset(new CSeq_feat());
        gene_feat->SetData().SetGene().Assign (*m_GeneRef);
        // pick location to match
        CSeq_feat * feat = dynamic_cast<CSeq_feat*>(m_Object);
        gene_feat->SetLocation().SetInt().SetId().Assign(*(feat->GetLocation().GetId()));
        gene_feat->SetLocation().SetInt().SetFrom(feat->GetLocation().GetStart(eExtreme_Positional));
        gene_feat->SetLocation().SetInt().SetTo(feat->GetLocation().GetStop(eExtreme_Positional));
        gene_feat->SetLocation().SetStrand(feat->GetLocation().GetStrand());
        bool partial_start = feat->GetLocation().IsPartialStart(eExtreme_Positional);
        bool partial_stop = feat->GetLocation().IsPartialStop(eExtreme_Positional);
        gene_feat->SetLocation().SetPartialStart(partial_start, eExtreme_Positional);
        gene_feat->SetLocation().SetPartialStop(partial_stop, eExtreme_Positional);
        if (partial_start || partial_stop) 
        {
            gene_feat->SetPartial (true);
        }
    }
    return gene_feat;
}


CConstRef<CSeq_feat> CFeatureGenePanel::GetGeneToEdit()
{
    TransferDataFromWindow();
    if (!m_Suppress->GetValue() && m_MapType->GetSelection() == 0 && m_GeneChoice->GetSelection() > 1) {
        return m_OverlappingGene;
    } else {
        CConstRef<CSeq_feat> gene_feat;
        gene_feat.Reset();
        return gene_feat;
    }        
}


/*!
 * Should we show tooltips?
 */

bool CFeatureGenePanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CFeatureGenePanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CFeatureGenePanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CFeatureGenePanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CFeatureGenePanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CFeatureGenePanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CFeatureGenePanel icon retrieval
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX13
 */

void CFeatureGenePanel::OnClickSuppress( wxCommandEvent& event )
{
    if (m_Suppress->GetValue()) {
        m_GeneChoice->Enable (false);
        m_MapType->Enable(false);
        m_GeneRefPanel->Enable(false);
    } else {
        m_GeneChoice->Enable (true);
        m_MapType->Enable(true);
        if (m_GeneChoice->GetSelection() == 1) {
            m_GeneRefPanel->Enable (true);
        } else{
            m_GeneRefPanel->Enable (false);
        }        
    }
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE16
 */

void CFeatureGenePanel::OnSelectGene( wxCommandEvent& event )
{
    if (m_GeneChoice->GetSelection() == 1) {
        m_GeneRefPanel->Enable (true);
    } else{
        m_GeneRefPanel->Enable (false);
    }
}

END_NCBI_SCOPE
