/*  $Id: identifiers_panel.cpp 27370 2013-02-04 18:09:51Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */


#include <ncbi_pch.hpp>
#include <objects/seqfeat/Feat_id.hpp>
#include <objects/seqfeat/SeqFeatXref.hpp>
#include <objects/general/Object_id.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

////@begin includes
////@end includes

#include <wx/panel.h>
#include <wx/sizer.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>
#include <wx/valtext.h>
#include <wx/toplevel.h>

#include "identifiers_panel.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*!
 * CIdentifiersPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CIdentifiersPanel, wxPanel )


/*!
 * CIdentifiersPanel event table definition
 */

BEGIN_EVENT_TABLE( CIdentifiersPanel, wxPanel )

////@begin CIdentifiersPanel event table entries
////@end CIdentifiersPanel event table entries

END_EVENT_TABLE()


/*!
 * CIdentifiersPanel constructors
 */

CIdentifiersPanel::CIdentifiersPanel()
{
    Init();
}

CIdentifiersPanel::CIdentifiersPanel( wxWindow* parent, CSeq_feat& feat, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    m_EditedFeat = new CSeq_feat();
    m_EditedFeat->Assign(feat);
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CIdentifiersPanel creator
 */

bool CIdentifiersPanel::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CIdentifiersPanel creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CIdentifiersPanel creation
    return true;
}


/*!
 * CIdentifiersPanel destructor
 */

CIdentifiersPanel::~CIdentifiersPanel()
{
////@begin CIdentifiersPanel destruction
////@end CIdentifiersPanel destruction
}


/*!
 * Member initialisation
 */

void CIdentifiersPanel::Init()
{
////@begin CIdentifiersPanel member initialisation
    m_FeatureIdCtrl = NULL;
    m_IdXrefCtrl = NULL;
////@end CIdentifiersPanel member initialisation
}


/*!
 * Control creation for CIdentifiersPanel
 */

void CIdentifiersPanel::CreateControls()
{    
////@begin CIdentifiersPanel content construction
    CIdentifiersPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer3 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("Feature ID for this feature"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText4, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_FeatureIdCtrl = new wxTextCtrl( itemPanel1, ID_TEXTCTRL25, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_FeatureIdCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("ID Xref to associated feature"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(itemStaticText6, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_IdXrefCtrl = new wxTextCtrl( itemPanel1, ID_TEXTCTRL26, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer3->Add(m_IdXrefCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CIdentifiersPanel content construction
}


static string s_GetIdXref (CSeq_feat& feat)
{
    string xref = "";
    if (feat.IsSetXref()) {
        ITERATE (CSeq_feat::TXref, xit, feat.GetXref()) { 
            if ((*xit)->IsSetId() && (*xit)->GetId().IsLocal()) {
                if ((*xit)->GetId().GetLocal().IsStr()) {
                    xref = (*xit)->GetId().GetLocal().GetStr();
                    break;
                } else if ((*xit)->GetId().GetLocal().IsId()) {
                    xref = NStr::NumericToString((*xit)->GetId().GetLocal().GetId());
                    break;
                }
            }
        }
    }
    return xref;
}


bool CIdentifiersPanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow()) {
        return false;
    }

    // feature ID
    if (m_EditedFeat->IsSetId() && m_EditedFeat->GetId().IsLocal()) {
        if (m_EditedFeat->GetId().GetLocal().IsStr()) {
            m_FeatureIdCtrl->SetValue(ToWxString(m_EditedFeat->GetId().GetLocal().GetStr()));
        } else if (m_EditedFeat->GetId().GetLocal().IsId()) {
            m_FeatureIdCtrl->SetValue(ToWxString(NStr::NumericToString(m_EditedFeat->GetId().GetLocal().GetId())));
        } else {
            m_FeatureIdCtrl->SetValue(wxEmptyString);
        }          
    } else {
        m_FeatureIdCtrl->SetValue(wxEmptyString);
    }

    // xrefs
    string xref = s_GetIdXref(*m_EditedFeat);
    m_IdXrefCtrl->SetValue (ToWxString (xref));

    return true;
}


static void s_SetFeatId (CFeat_id& feat_id, string val)
{
    int nval = NStr::StringToInt(val);  
    if (nval < 0) {
        feat_id.SetLocal().SetStr(val);
    } else {
        feat_id.SetLocal().SetId(nval);
    }
}

static void s_SetIdXref (CSeq_feat& feat, string val)
{
    bool found = false;
    if (feat.IsSetXref()) {
        NON_CONST_ITERATE (CSeq_feat::TXref, xit, feat.SetXref()) {   
            if ((*xit)->IsSetId() && (*xit)->GetId().IsLocal()) {
                if (NStr::IsBlank(val)) {
                    xit = feat.SetXref().erase(xit);
                } else {
                    s_SetFeatId ((*xit)->SetId(), val);
                }
                found = true;
                break;
            }
        }
    }
    if (!found && !NStr::IsBlank(val)) {
        CRef<CSeqFeatXref> x(new CSeqFeatXref());
        s_SetFeatId (x->SetId(), val);
        feat.SetXref().push_back(x);
    }
}


bool CIdentifiersPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow()) {
      return false;
    }

    // feature ID
    string val = ToStdString (m_FeatureIdCtrl->GetValue());
    if (NStr::IsBlank(val)) {
        if (m_EditedFeat->IsSetId() && m_EditedFeat->GetId().IsLocal()) {
            m_EditedFeat->ResetId();
        }
    } else {
        s_SetFeatId (m_EditedFeat->SetId(), val);
    }

    // xrefs
    val = ToStdString (m_IdXrefCtrl->GetValue());
    s_SetIdXref (*m_EditedFeat, val);

    return true;
}


void CIdentifiersPanel::PopulateIdentifiers(objects::CSeq_feat& seq_feat)
{
    // feature ID
    if (m_EditedFeat->IsSetId()) {
        seq_feat.SetId().Assign(m_EditedFeat->GetId());
    } else {
        seq_feat.ResetId();
    }

    // xrefs
    string xref = s_GetIdXref(*m_EditedFeat);
    s_SetIdXref (seq_feat, xref);

}


/*!
 * Should we show tooltips?
 */

bool CIdentifiersPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CIdentifiersPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CIdentifiersPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CIdentifiersPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CIdentifiersPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CIdentifiersPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CIdentifiersPanel icon retrieval
}

END_NCBI_SCOPE
