#ifndef GUI_WIDGETS_EDIT___JOURNAL_PANEL__HPP
#define GUI_WIDGETS_EDIT___JOURNAL_PANEL__HPP

/*  $Id: journal_panel.hpp 26686 2012-10-24 18:22:06Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 */

#include <corelib/ncbistd.hpp>

#include <objmgr/scope.hpp>

#include <wx/panel.h>
#include <wx/choice.h>

#include <objects/biblio/Title.hpp>
#include <gui/widgets/edit/flexibledate_panel.hpp>

/*!
 * Includes
 */

////@begin includes
////@end includes


/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

class wxTextCtrl;

BEGIN_NCBI_SCOPE

class CSerialObject;

/*!
 * Control identifiers
 */


////@begin control identifiers
#define ID_JOURNAL 10034
#define ID_JOURNAL_TITLE 10035
#define ID_JOURNAL_VOLUME 10036
#define ID_JOURNAL_ISSUE 10037
#define ID_JOURNAL_PAGES 10038
#define ID_JOURNAL_PMID 10044
#define ID_JOURNAL_MUID 10043
#define ID_JOURNAL_ERRATUM 10046
#define ID_JOURNAL_EXPLANATION 10047
#define ID_JOURNAL_PUBSTATUS 10048
#define ID_JOURNAL_LOOKUP_RELAXED 10050
#define ID_JOURNAL_LOOKUP_ISOJTA 10051
#define ID_PMID_LOOKUP 10049
#define SYMBOL_CJOURNALPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CJOURNALPANEL_TITLE _("Journal")
#define SYMBOL_CJOURNALPANEL_IDNAME ID_JOURNAL
#define SYMBOL_CJOURNALPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CJOURNALPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CJournalPanel class declaration
 */

class CJournalPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CJournalPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CJournalPanel();
    CJournalPanel( wxWindow* parent, CSerialObject& object, long *pmid, long *muid, wxWindowID id = SYMBOL_CJOURNALPANEL_IDNAME, const wxPoint& pos = SYMBOL_CJOURNALPANEL_POSITION, const wxSize& size = SYMBOL_CJOURNALPANEL_SIZE, long style = SYMBOL_CJOURNALPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CJOURNALPANEL_IDNAME, const wxPoint& pos = SYMBOL_CJOURNALPANEL_POSITION, const wxSize& size = SYMBOL_CJOURNALPANEL_SIZE, long style = SYMBOL_CJOURNALPANEL_STYLE );

    /// Destructor
    ~CJournalPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    bool TransferDataToWindow();
    bool TransferDataFromWindow();
    CRef<objects::CCit_jour> GetCit_jour() const;

////@begin CJournalPanel event handler declarations

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_JOURNAL_ERRATUM
    void OnJournalErratumSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_JOURNAL_LOOKUP_RELAXED
    void OnJournalLookupRelaxedClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_JOURNAL_LOOKUP_ISOJTA
    void OnJournalLookupIsojtaClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PMID_LOOKUP
    void OnPmidLookupClick( wxCommandEvent& event );

////@end CJournalPanel event handler declarations

////@begin CJournalPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CJournalPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CJournalPanel member variables
    wxTextCtrl* m_JournalTitle;
    wxTextCtrl* m_Volume;
    wxTextCtrl* m_Issue;
    wxTextCtrl* m_Pages;
    wxTextCtrl* m_PmidCtrl;
    wxTextCtrl* m_MuidCtrl;
    wxChoice* m_ErratumCtrl;
    wxTextCtrl* m_ErratumExplanation;
    wxChoice* m_PubstatusCtrl;
    wxBoxSizer* m_PubDateSizer;
    CFlexibleDatePanel* m_PubDatePanel;
////@end CJournalPanel member variables
private:
    CSerialObject* m_Object;
    CRef<CSerialObject> m_EditedJournal;
    long * m_pPmid;
    long * m_pMuid;
    long m_Pmid;
    long m_Muid;
   
    void PushToWindow(const objects::CCit_jour& journal);
};


END_NCBI_SCOPE

#endif  // GUI_WIDGETS_EDIT___JOURNAL_PANEL__HPP
