/*  $Id: loc_gcode_panel.cpp 27919 2013-04-25 12:34:08Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vasuki Palanigobu
 */

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include <ncbi_pch.hpp>
#include <objects/seqfeat/Org_ref.hpp>
#include <objects/seqfeat/OrgName.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include "loc_gcode_panel.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*
 * CLocAndGCodePanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CLocAndGCodePanel, wxPanel )


/*
 * CLocAndGCodePanel event table definition
 */

BEGIN_EVENT_TABLE( CLocAndGCodePanel, wxPanel )

////@begin CLocAndGCodePanel event table entries
////@end CLocAndGCodePanel event table entries

END_EVENT_TABLE()


/*
 * CLocAndGCodePanel constructors
 */
static CBioSource dummy;

 CLocAndGCodePanel::CLocAndGCodePanel():m_Source(dummy)
{
    Init();
}

CLocAndGCodePanel::CLocAndGCodePanel( wxWindow* parent, objects::CBioSource& source, wxWindowID id, const wxPoint& pos, const wxSize& size, long style):
    m_Source(source)
{
	//m_Source = new CBioSource();
    //m_Source->Assign(source);
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * GeneticCodesPanel creator
 */

bool CLocAndGCodePanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CLocAndGCodePanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CLocAndGCodePanel creation
    return true;
}


/*
 * CLocAndGCodePanel destructor
 */

CLocAndGCodePanel::~CLocAndGCodePanel()
{
////@begin CLocAndGCodePanel destruction
////@end CLocAndGCodePanel destruction
}


/*
 * Member initialisation
 */

void CLocAndGCodePanel::Init()
{
////@begin CLocAndGCodePanel member initialisation
    m_LocCtrl = NULL;
    m_OriginCtrl = NULL;
    m_NuclearCodeCtrl = NULL;
    m_MitoCodeCtrl = NULL;
    m_PlastidCodeCtrl = NULL;
    m_BioFocusCtrl = NULL;
////@end CLocAndGCodePanel member initialisation
}


/*
 * Control creation for GeneticCodesPanel
 */

void CLocAndGCodePanel::CreateControls()
{    
////@begin CLocAndGCodePanel content construction
    CLocAndGCodePanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Location and Origin"));
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxHORIZONTAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 0, wxALIGN_LEFT|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer4 = new wxFlexGridSizer(0, 2, 0, 0);
    itemStaticBoxSizer3->Add(itemFlexGridSizer4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("Location of sequence"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText5, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_LocCtrlStrings;
    m_LocCtrlStrings.Add(_("mitochondrion"));
    m_LocCtrlStrings.Add(_("chloroplast"));
    m_LocCtrlStrings.Add(_("apicoplast"));
    m_LocCtrlStrings.Add(_("chloroplast"));
    m_LocCtrlStrings.Add(_("chromatophore"));
    m_LocCtrlStrings.Add(_("chromoplast"));
    m_LocCtrlStrings.Add(_("cyanelle"));
    m_LocCtrlStrings.Add(_("endogenous-virus"));
    m_LocCtrlStrings.Add(_("extrachromosomal"));
    m_LocCtrlStrings.Add(_("genomic"));
    m_LocCtrlStrings.Add(_("hydrogenosome"));
    m_LocCtrlStrings.Add(_("kinetoplast"));
    m_LocCtrlStrings.Add(_("leucoplast"));
    m_LocCtrlStrings.Add(_("macronuclear"));
    m_LocCtrlStrings.Add(_("mitochondrion"));
    m_LocCtrlStrings.Add(_("nucleomorph"));
    m_LocCtrlStrings.Add(_("plasmid"));
    m_LocCtrlStrings.Add(_("plastid"));
    m_LocCtrlStrings.Add(_("proplastid"));
    m_LocCtrlStrings.Add(_("proviral"));
    m_LocCtrl = new wxComboBox( itemPanel1, ID_COMBOBOX5, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_LocCtrlStrings, wxCB_DROPDOWN );
    itemFlexGridSizer4->Add(m_LocCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Origin of Sequence"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer4->Add(itemStaticText7, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_OriginCtrlStrings;
    m_OriginCtrlStrings.Add(_("Natural"));
    m_OriginCtrlStrings.Add(_("Natural mutant"));
    m_OriginCtrlStrings.Add(_("Mutant"));
    m_OriginCtrlStrings.Add(_("Artificial"));
    m_OriginCtrlStrings.Add(_("Synthetic"));
    m_OriginCtrlStrings.Add(_("Other"));
    m_OriginCtrl = new wxComboBox( itemPanel1, ID_COMBOBOX6, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_OriginCtrlStrings, wxCB_DROPDOWN );
    itemFlexGridSizer4->Add(m_OriginCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer9Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Genetic codes"));
    wxStaticBoxSizer* itemStaticBoxSizer9 = new wxStaticBoxSizer(itemStaticBoxSizer9Static, wxHORIZONTAL);
    itemBoxSizer2->Add(itemStaticBoxSizer9, 0, wxALIGN_LEFT|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer10 = new wxFlexGridSizer(0, 2, 0, 0);
    itemStaticBoxSizer9->Add(itemFlexGridSizer10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText11 = new wxStaticText( itemPanel1, wxID_STATIC, _("Nuclear"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer10->Add(itemStaticText11, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_NuclearCodeCtrlStrings;
    m_NuclearCodeCtrlStrings.Add(_("Standard"));
    m_NuclearCodeCtrlStrings.Add(_("Ciliate Nuclear, etc"));
    m_NuclearCodeCtrlStrings.Add(_("Euplotid Nuclear"));
    m_NuclearCodeCtrlStrings.Add(_("Alternative Yeast"));
    m_NuclearCodeCtrlStrings.Add(_("Blepharisma Macronuclear"));
    m_NuclearCodeCtrl = new wxComboBox( itemPanel1, ID_COMBOBOX, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_NuclearCodeCtrlStrings, wxCB_DROPDOWN );
    itemFlexGridSizer10->Add(m_NuclearCodeCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText13 = new wxStaticText( itemPanel1, wxID_STATIC, _("Mitochondrial"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer10->Add(itemStaticText13, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_MitoCodeCtrlStrings;
    m_MitoCodeCtrlStrings.Add(_("Vertebrate"));
    m_MitoCodeCtrlStrings.Add(_("Yeast"));
    m_MitoCodeCtrlStrings.Add(_("Mold etc."));
    m_MitoCodeCtrlStrings.Add(_("Invertebrate"));
    m_MitoCodeCtrlStrings.Add(_("Echinoderm etc."));
    m_MitoCodeCtrlStrings.Add(_("Ascidian"));
    m_MitoCodeCtrlStrings.Add(_("Alternative Flatworm"));
    m_MitoCodeCtrlStrings.Add(_("Chlorophycean"));
    m_MitoCodeCtrlStrings.Add(_("Trematode"));
    m_MitoCodeCtrlStrings.Add(_("Scenedesmus obliquus"));
    m_MitoCodeCtrlStrings.Add(_("Thraustochytrium"));
    m_MitoCodeCtrl = new wxComboBox( itemPanel1, ID_COMBOBOX1, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_MitoCodeCtrlStrings, wxCB_DROPDOWN );
    itemFlexGridSizer10->Add(m_MitoCodeCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText15 = new wxStaticText( itemPanel1, wxID_STATIC, _("Plastid"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer10->Add(itemStaticText15, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_PlastidCodeCtrlStrings;
    m_PlastidCodeCtrlStrings.Add(_("Standard"));
    m_PlastidCodeCtrlStrings.Add(_("Ciliate Nuclear, etc"));
    m_PlastidCodeCtrlStrings.Add(_("Euplotid Nuclear"));
    m_PlastidCodeCtrlStrings.Add(_("Alternative Yeast"));
    m_PlastidCodeCtrlStrings.Add(_("Blepharisma Macronuclear"));
    m_PlastidCodeCtrlStrings.Add(_("Vertebrate"));
    m_PlastidCodeCtrlStrings.Add(_("Yeast"));
    m_PlastidCodeCtrlStrings.Add(_("Mold etc."));
    m_PlastidCodeCtrlStrings.Add(_("Invertebrate"));
    m_PlastidCodeCtrlStrings.Add(_("Echinoderm etc."));
    m_PlastidCodeCtrlStrings.Add(_("Ascidian"));
    m_PlastidCodeCtrlStrings.Add(_("Alternative Flatworm"));
    m_PlastidCodeCtrlStrings.Add(_("Chlorophycean"));
    m_PlastidCodeCtrlStrings.Add(_("Trematode"));
    m_PlastidCodeCtrlStrings.Add(_("Scenedesmus obliquus"));
    m_PlastidCodeCtrlStrings.Add(_("Thraustochytrium"));
    m_PlastidCodeCtrl = new wxComboBox( itemPanel1, ID_COMBOBOX2, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_PlastidCodeCtrlStrings, wxCB_DROPDOWN );
    itemFlexGridSizer10->Add(m_PlastidCodeCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_BioFocusCtrl = new wxCheckBox( itemPanel1, ID_CHECKBOX1, _("Biological focus (if multiple source features)"), wxDefaultPosition, wxDefaultSize, 0 );
    m_BioFocusCtrl->SetValue(false);
    itemBoxSizer2->Add(m_BioFocusCtrl, 0, wxALIGN_LEFT|wxALL, 5);

////@end CLocAndGCodePanel content construction
}

bool CLocAndGCodePanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;
    m_LocCtrl->SetValue(ToWxString(x_GetLocation()));
    m_OriginCtrl->SetValue(ToWxString(x_GetOrigin()));
    m_NuclearCodeCtrl->SetValue(ToWxString(x_GetGeneticCode()));
    m_MitoCodeCtrl->SetValue(ToWxString(x_GetMitochondrialCode()));
	m_PlastidCodeCtrl->SetValue(ToWxString(x_GetPlastidCode()));
	if (m_Source.IsSetIs_focus()) {
		m_BioFocusCtrl->SetValue(true);
	}
	return true;
}

bool CLocAndGCodePanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;
    x_SetLocation();
    x_SetOrigin();
    x_SetGeneticCode();
    x_SetMitochondrialCode();
	  x_SetPlastidCode();
	  if (m_BioFocusCtrl->GetValue()) {
		    m_Source.SetIs_focus();
	  } else {
        m_Source.ResetIs_focus();
    }
	  return true;
}

void CLocAndGCodePanel::x_SetPlastidCode() 
{
    string pgcode = ToStdString(m_PlastidCodeCtrl->GetValue());
	if (NStr::Equal(pgcode, "Vertebrate")) {
		m_Source.SetOrg().SetOrgname().SetPgcode(2);
	} else if (NStr::Equal(pgcode, "Yeast")) {
		m_Source.SetOrg().SetOrgname().SetPgcode(3);
	} else if (NStr::Equal(pgcode, "Mold etc.")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(4);
	} else if (NStr::Equal(pgcode, "Invertebrate")) {
       m_Source.SetOrg().SetOrgname().SetPgcode(5);
    } else if (NStr::Equal(pgcode, "Echinoderm etc." )) {
       m_Source.SetOrg().SetOrgname().SetPgcode(9);
    } else if (NStr::Equal(pgcode, "Ascidian" )) {
		m_Source.SetOrg().SetOrgname().SetPgcode(13);
	} else if (NStr::Equal(pgcode, "Alternative Flatworm")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(14);
    } else if (NStr::Equal(pgcode, "Chlorophycean")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(16);
    } else if (NStr::Equal(pgcode, "Trematode")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(21);
    } else if (NStr::Equal(pgcode, "Scenedesmus obliquus")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(22);
    } else if (NStr::Equal(pgcode, "Thraustochytrium")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(23);
    }else if (NStr::Equal(pgcode, "Standard"))  {
		m_Source.SetOrg().SetOrgname().SetPgcode(1);
	} else if (NStr::Equal(pgcode, "Ciliate Nuclear, etc")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(6);
	} else if (NStr::Equal(pgcode, "Euplotid Nuclear")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(10);
	} else if (NStr::Equal(pgcode, "Alternative Yeast")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(12);
	} else if (NStr::Equal(pgcode, "Blepharisma Macronuclear")) {
        m_Source.SetOrg().SetOrgname().SetPgcode(15);
    }
}

void CLocAndGCodePanel::x_SetMitochondrialCode() 
{
    string mgcode = ToStdString(m_MitoCodeCtrl->GetValue());
	if (NStr::Equal(mgcode, "Vertebrate")) {
		m_Source.SetOrg().SetOrgname().SetMgcode(2);
	} else if (NStr::Equal(mgcode, "Yeast")) {
		m_Source.SetOrg().SetOrgname().SetMgcode(3);
	} else if (NStr::Equal(mgcode, "Mold etc.")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(4);
	} else if (NStr::Equal(mgcode, "Invertebrate")) {
       m_Source.SetOrg().SetOrgname().SetMgcode(5);
    } else if (NStr::Equal(mgcode, "Echinoderm etc." )) {
       m_Source.SetOrg().SetOrgname().SetMgcode(9);
    } else if (NStr::Equal(mgcode, "Ascidian" )) {
		m_Source.SetOrg().SetOrgname().SetMgcode(13);
	} else if (NStr::Equal(mgcode, "Alternative Flatworm")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(14);
    } else if (NStr::Equal(mgcode, "Chlorophycean")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(16);
    } else if (NStr::Equal(mgcode, "Trematode")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(21);
    } else if (NStr::Equal(mgcode, "Scenedesmus obliquus")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(22);
    } else if (NStr::Equal(mgcode, "Thraustochytrium")) {
        m_Source.SetOrg().SetOrgname().SetMgcode(23);
    }
}

void CLocAndGCodePanel::x_SetGeneticCode()
{
    string gcode = ToStdString(m_NuclearCodeCtrl->GetValue());
	if (NStr::Equal(gcode, "Standard"))  {
		m_Source.SetOrg().SetOrgname().SetGcode(1);
	} else if (NStr::Equal(gcode, "Ciliate Nuclear, etc")) {
        m_Source.SetOrg().SetOrgname().SetGcode(6);
	} else if (NStr::Equal(gcode, "Euplotid Nuclear")) {
        m_Source.SetOrg().SetOrgname().SetGcode(10);
	} else if (NStr::Equal(gcode, "Alternative Yeast")) {
        m_Source.SetOrg().SetOrgname().SetGcode(12);
	} else if (NStr::Equal(gcode, "Blepharisma Macronuclear")) {
        m_Source.SetOrg().SetOrgname().SetGcode(15);
    }
}


void CLocAndGCodePanel::x_SetOrigin()
{
    string origin = ToStdString(m_OriginCtrl->GetValue());
	if (NStr::Equal(origin, "Natural")) {
		m_Source.SetOrigin(CBioSource::eOrigin_natural);
	} else if (NStr::Equal(origin, "Natural mutant")) {
		m_Source.SetOrigin(CBioSource::eOrigin_natmut);
	} else if (NStr::Equal(origin, "Mutant")) {
		m_Source.SetOrigin(CBioSource::eOrigin_mut);
	} else if (NStr::Equal(origin, "Artificial")) {
		m_Source.SetOrigin(CBioSource::eOrigin_artificial);
	} else if (NStr::Equal(origin, "Synthetic")) {
		m_Source.SetOrigin(CBioSource::eOrigin_synthetic);
 	} else if (NStr::Equal(origin, "Other")) {
		m_Source.SetOrigin(CBioSource::eOrigin_other);
 	} else {
		m_Source.SetOrigin(CBioSource::eOrigin_unknown);
    }
}

void CLocAndGCodePanel::x_SetLocation()
{
    string location = ToStdString(m_LocCtrl->GetValue());
	if (NStr::Equal(location, "mitochondrion")) {
		m_Source.SetGenome(CBioSource::eGenome_mitochondrion);
	} else if (NStr::Equal(location, "chloroplast")) {
		m_Source.SetGenome(CBioSource::eGenome_chloroplast);
	} else if (NStr::Equal(location, "apicoplast")) {
		m_Source.SetGenome(CBioSource::eGenome_apicoplast);
	} else if (NStr::Equal(location, "chromatophore")) {
		m_Source.SetGenome(CBioSource::eGenome_chromatophore);
	} else if (NStr::Equal(location, "chromoplast")) {
		m_Source.SetGenome(CBioSource::eGenome_chromoplast);
	} else if (NStr::Equal(location, "cyanelle")) {
		m_Source.SetGenome(CBioSource::eGenome_cyanelle);
	} else if (NStr::Equal(location, "endogenous-virus")) {
		m_Source.SetGenome(CBioSource::eGenome_endogenous_virus);
	} else if (NStr::Equal(location, "extrachromosomal")) {
		m_Source.SetGenome(CBioSource::eGenome_extrachrom);
	} else if (NStr::Equal(location, "genomic")) {
		m_Source.SetGenome(CBioSource::eGenome_genomic);
	} else if (NStr::Equal(location, "hydrogenosome")) {
		m_Source.SetGenome(CBioSource::eGenome_hydrogenosome);
	} else if (NStr::Equal(location, "kinetoplast")) {
		m_Source.SetGenome(CBioSource::eGenome_kinetoplast);
	} else if (NStr::Equal(location, "leucoplast")) {
		m_Source.SetGenome(CBioSource::eGenome_leucoplast);
	} else if (NStr::Equal(location, "macronuclear")) {
		m_Source.SetGenome(CBioSource::eGenome_macronuclear);
	} else if (NStr::Equal(location, "nucleomorph")) {
		m_Source.SetGenome(CBioSource::eGenome_nucleomorph);
	} else if (NStr::Equal(location, "plasmid")) {
		m_Source.SetGenome(CBioSource::eGenome_plasmid);
	} else if (NStr::Equal(location, "plastid")) {
		m_Source.SetGenome(CBioSource::eGenome_plastid);
	} else if (NStr::Equal(location, "proplastid")) {
		m_Source.SetGenome(CBioSource::eGenome_proplastid);
	} else if (NStr::Equal(location, "proviral")) {
		m_Source.SetGenome(CBioSource::eGenome_proviral);
	} else {
		m_Source.SetGenome(CBioSource::eGenome_unknown);
    }
}

string CLocAndGCodePanel::x_GetPlastidCode()
{
	if (m_Source.IsSetOrg() && m_Source.GetOrg().IsSetOrgname() 
		&& m_Source.GetOrg().GetOrgname().IsSetPgcode())
    {
        switch(m_Source.GetOrg().GetOrgname().GetPgcode())
        {
            case 2:
                return "Vertebrate";
            case 3:
                return "Yeast";
            case 4:
                return "Mold etc.";
            case 5:
                return "Invertebrate";
            case 9:
                return "Echinoderm etc.";
            case 13:
                return "Ascidian";
            case 14:
                return "Alternative Flatworm";
            case 16:
                return "Chlorophycean";
            case 21:
                return "Trematode";
            case 22:
                return "Scenedesmus obliquus";
            case 23:
                return "Thraustochytrium";
			case 1:
                return "Standard";
            case 6:
                return "Ciliate Nuclear, etc";
            case 10:
                return "Euplotid Nuclear";
            case 12:
                return "Alternative Yeast";
            case 15:
                return "Blepharisma Macronuclear";
        }
    }
	return kEmptyStr;
}

string CLocAndGCodePanel::x_GetMitochondrialCode() 
{
    if (m_Source.IsSetOrg() && m_Source.GetOrg().IsSetOrgname() 
		&& m_Source.GetOrg().GetOrgname().IsSetMgcode())
    {
        switch(m_Source.GetOrg().GetOrgname().GetMgcode())
        {
            case 2:
                return "Vertebrate";
            case 3:
                return "Yeast";
            case 4:
                return "Mold etc.";
            case 5:
                return "Invertebrate";
            case 9:
                return "Echinoderm etc.";
            case 13:
                return "Ascidian";
            case 14:
                return "Alternative Flatworm";
            case 16:
                return "Chlorophycean";
            case 21:
                return "Trematode";
            case 22:
                return "Scenedesmus obliquus";
            case 23:
                return "Thraustochytrium";
        }
    }
	return kEmptyStr;
}

string CLocAndGCodePanel::x_GetGeneticCode()
{
    if (m_Source.IsSetOrg() && m_Source.GetOrg().IsSetOrgname() 
		&& m_Source.GetOrg().GetOrgname().IsSetGcode())
    {         
        switch(m_Source.GetOrg().GetOrgname().GetGcode())
        {
            case 1:
                return "Standard";
            case 6:
                return "Ciliate Nuclear, etc";
            case 10:
                return "Euplotid Nuclear";
            case 12:
                return "Alternative Yeast";
            case 15:
                return "Blepharisma Macronuclear";
        }
    }
	return kEmptyStr;
}

string CLocAndGCodePanel::x_GetOrigin()
{
	if ( ! m_Source.IsSetOrigin() ) {
		return "";
	}

    switch(m_Source.GetOrigin())
    {
        case CBioSource::eOrigin_natural:
            return "Natural";
        case CBioSource::eOrigin_natmut:
            return "Natural mutant";
        case CBioSource::eOrigin_mut:
            return "Mutant";
        case CBioSource::eOrigin_artificial:
            return "Artificial";
        case CBioSource::eOrigin_synthetic:
            return "Synthetic";
        case CBioSource::eOrigin_other:
            return "Other";
        default:
            return "";
    }
}

string CLocAndGCodePanel::x_GetLocation()
{
	if ( ! m_Source.IsSetGenome() ) {
		return "";
	}

    switch(m_Source.GetGenome())
    {
        case CBioSource::eGenome_mitochondrion:
            return "mitochondrion";
        case CBioSource::eGenome_chloroplast:
            return "chloroplast";
        case CBioSource::eGenome_apicoplast:
            return "apicoplast";
        case CBioSource::eGenome_chromatophore:
            return "chromatophore";
        case CBioSource::eGenome_chromoplast:
            return "chromoplast";        
        case CBioSource::eGenome_cyanelle:
            return "cyanelle";  
        case CBioSource::eGenome_endogenous_virus:
            return "endogenous-virus";        
        case CBioSource::eGenome_extrachrom:
            return "extrachromosomal";        
        case CBioSource::eGenome_genomic:
            return "genomic";        
        case CBioSource::eGenome_hydrogenosome:
            return "hydrogenosome";        
        case CBioSource::eGenome_kinetoplast:
            return "kinetoplast";        
        case CBioSource::eGenome_leucoplast:
            return "leucoplast";        
        case CBioSource::eGenome_macronuclear:
            return "macronuclear";        
        case CBioSource::eGenome_nucleomorph:
            return "nucleomorph";        
        case CBioSource::eGenome_plasmid:
            return "plasmid";        
        case CBioSource::eGenome_plastid:
            return "plastid";        
        case CBioSource::eGenome_proplastid:
            return "proplastid";        
        case CBioSource::eGenome_proviral:
            return "proviral";
        default:
            return ""; //??or unknown
    }
}

void CLocAndGCodePanel::PopulateLocAndGC(CBioSource& source)
{
	if (m_Source.IsSetIs_focus()) {
		source.SetIs_focus();
	}
	source.SetGenome(m_Source.GetGenome());
	source.SetOrigin(m_Source.GetOrigin());
	if (m_Source.IsSetOrg() && m_Source.GetOrg().IsSetOrgname()) {
		if (m_Source.GetOrg().GetOrgname().IsSetGcode()) {
			source.SetOrg().SetOrgname().SetGcode(m_Source.GetOrg().GetOrgname().GetGcode());
		}
		if (m_Source.GetOrg().GetOrgname().IsSetMgcode()) {
			source.SetOrg().SetOrgname().SetMgcode(m_Source.GetOrg().GetOrgname().GetMgcode());
		}
		if (m_Source.GetOrg().GetOrgname().IsSetPgcode()) {
			source.SetOrg().SetOrgname().SetPgcode(m_Source.GetOrg().GetOrgname().GetPgcode());
		}
	}
}
/*
 * Should we show tooltips?
 */

bool CLocAndGCodePanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CLocAndGCodePanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CLocAndGCodePanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CLocAndGCodePanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CLocAndGCodePanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CLocAndGCodePanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CLocAndGCodePanel icon retrieval
}

END_NCBI_SCOPE
