/*  $Id: locationl_list_ctrl.cpp 27841 2013-04-11 19:32:41Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>
#include <set>

////@begin includes
////@end includes

#include <objects/seqloc/Na_strand.hpp>
#include <objects/general/Object_id.hpp>
#include <objmgr/bioseq_ci.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <objmgr/scope.hpp>

#include <util/xregexp/regexp.hpp>

#include <gui/widgets/edit/locationl_list_ctrl.hpp>
#include <gui/widgets/wx/spin_ctrl.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/fileartprov.hpp>
#include <gui/objutils/label.hpp>

#include <wx/hyperlink.h>
#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/scrolwin.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/combobox.h>
#include <wx/radiobox.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*!
 * CLocationListCtrl type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CLocationListCtrl, wxPanel )


/*!
 * CLocationListCtrl event table definition
 */

BEGIN_EVENT_TABLE( CLocationListCtrl, wxPanel )

////@begin CLocationListCtrl event table entries
    EVT_BUTTON( ID_BUTTON1, CLocationListCtrl::OnSortClick )

////@end CLocationListCtrl event table entries

    EVT_HYPERLINK(wxID_ANY, CLocationListCtrl::OnDelete)
    EVT_COMMAND(wxID_ANY, wxEVT_SPIN_CTRL_EVENT, CLocationListCtrl::OnSpinCtrl)

END_EVENT_TABLE()

const int kCtrlColNum = 5;

/*!
 * CLocationListCtrl constructors
 */

CLocationListCtrl::CLocationListCtrl()
{
    Init();
}

CLocationListCtrl::CLocationListCtrl( wxWindow* parent, objects::CScope& scope, bool allow_nuc, bool allow_prot,
                                      wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
                                      : m_AllowNuc(allow_nuc), m_AllowProt(allow_prot)
{
    m_Scope.Reset(&scope);
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CLocationListCtrl creator
 */

bool CLocationListCtrl::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CLocationListCtrl creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CLocationListCtrl creation
    return true;
}


/*!
 * CLocationListCtrl destructor
 */

CLocationListCtrl::~CLocationListCtrl()
{
////@begin CLocationListCtrl destruction
////@end CLocationListCtrl destruction
}


/*!
 * Member initialisation
 */

void CLocationListCtrl::Init()
{
////@begin CLocationListCtrl member initialisation
    m_LocType = 0;
    m_ScrolledWindow = NULL;
    m_Partial5 = NULL;
    m_Partial3 = NULL;
////@end CLocationListCtrl member initialisation
    m_Sizer = NULL;
}


/*!
 * Control creation for CLocationListCtrl
 */

void CLocationListCtrl::CreateControls()
{    
////@begin CLocationListCtrl content construction
    CLocationListCtrl* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxPanel* itemPanel3 = new wxPanel( itemPanel1, ID_PANEL1, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(itemPanel3, 1, wxALIGN_LEFT|wxALL, 0);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    itemPanel3->SetSizer(itemBoxSizer4);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(itemBoxSizer5, 0, wxALIGN_LEFT|wxALL, 0);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel3, wxID_STATIC, _("From"), wxDefaultPosition, wxSize(itemPanel3->ConvertDialogToPixels(wxSize(35, -1)).x, -1), wxALIGN_CENTRE );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel3, wxID_STATIC, _("To"), wxDefaultPosition, wxSize(itemPanel3->ConvertDialogToPixels(wxSize(35, -1)).x, -1), wxALIGN_CENTRE );
    itemBoxSizer5->Add(itemStaticText7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel3, wxID_STATIC, _("Strand"), wxDefaultPosition, wxSize(itemPanel3->ConvertDialogToPixels(wxSize(45, -1)).x, -1), wxALIGN_CENTRE );
    itemBoxSizer5->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);

    wxStaticText* itemStaticText9 = new wxStaticText( itemPanel3, wxID_STATIC, _("SeqID"), wxDefaultPosition, wxSize(itemPanel3->ConvertDialogToPixels(wxSize(65, -1)).x, -1), wxALIGN_CENTRE );
    itemBoxSizer5->Add(itemStaticText9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);

    itemBoxSizer5->Add(82, 8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticLine* itemStaticLine11 = new wxStaticLine( itemPanel3, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemBoxSizer4->Add(itemStaticLine11, 0, wxGROW|wxALL, 1);

    m_ScrolledWindow = new wxScrolledWindow( itemPanel3, ID_SCROLLEDWINDOW1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    itemBoxSizer4->Add(m_ScrolledWindow, 1, wxGROW|wxTOP|wxBOTTOM, 0);
    m_ScrolledWindow->SetScrollbars(1, 1, 0, 0);

    wxBoxSizer* itemBoxSizer13 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer13, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);

    wxStaticBox* itemStaticBoxSizer14Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Partialness"));
    wxStaticBoxSizer* itemStaticBoxSizer14 = new wxStaticBoxSizer(itemStaticBoxSizer14Static, wxHORIZONTAL);
    itemBoxSizer13->Add(itemStaticBoxSizer14, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Partial5 = new wxCheckBox( itemPanel1, ID_CHECKBOX1, _("5'"), wxDefaultPosition, wxDefaultSize, 0 );
    m_Partial5->SetValue(false);
    itemStaticBoxSizer14->Add(m_Partial5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Partial3 = new wxCheckBox( itemPanel1, ID_CHECKBOX2, _("3'"), wxDefaultPosition, wxDefaultSize, 0 );
    m_Partial3->SetValue(false);
    itemStaticBoxSizer14->Add(m_Partial3, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemRadioBox17Strings;
    itemRadioBox17Strings.Add(_("&join()"));
    itemRadioBox17Strings.Add(_("&order()"));
    wxRadioBox* itemRadioBox17 = new wxRadioBox( itemPanel1, ID_RADIOBOX6, _("Save As"), wxDefaultPosition, wxDefaultSize, itemRadioBox17Strings, 1, wxRA_SPECIFY_ROWS );
    itemRadioBox17->SetSelection(0);
    if (CLocationListCtrl::ShowToolTips())
        itemRadioBox17->SetToolTip(_("If each interval is separate, and should not be joined with the others to describe the feature, choose order (for example, when annotating multiple primer binding sites)."));
    itemBoxSizer13->Add(itemRadioBox17, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxButton* itemButton18 = new wxButton( itemPanel1, ID_BUTTON1, _("Sort"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer13->Add(itemButton18, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Set validators
    itemRadioBox17->SetValidator( wxGenericValidator(& m_LocType) );
////@end CLocationListCtrl content construction

    m_Sizer = new wxFlexGridSizer(0, kCtrlColNum, 0, 0);
    m_ScrolledWindow->SetSizer(m_Sizer);
    m_ScrolledWindow->FitInside();

    //m_AddIntHyperlink->SetVisitedColour(m_AddIntHyperlink->GetNormalColour());
    m_ScrolledWindow->SetScrollRate(0, 5);
}

void CLocationListCtrl::AddRow(int from, int to, int strand, const wxString& seqID)
{
    int comboIndex = 0;
    switch(strand) {
    default :
        break;
    case eNa_strand_plus :
        comboIndex = 1;
        break;
    case eNa_strand_minus :
        comboIndex = 2;
        break;
    case eNa_strand_other :
        comboIndex = 3;
        break;
    }

    SIntData intData;
    intData.m_From   = from;
    intData.m_To     = to;
    intData.m_Strand = comboIndex;
    intData.m_SeqId  = seqID;
    intData.m_Empty  = false;
    m_Data.push_back(intData);
}

void CLocationListCtrl::SetPartials(bool partial5, bool partial3)
{
    m_Partial5->SetValue(partial5);    
    m_Partial3->SetValue(partial3);
}

void CLocationListCtrl::AddEmptyRow()
{
    SIntData intData;
    intData.m_Empty = true;
    m_Data.push_back(intData);
}

void CLocationListCtrl::ClearRows()
{
    m_Data.clear();
    m_Sizer->Clear(true);
}

CRef<CSeq_loc> CLocationListCtrl::GetSeq_loc()
{
    CRef<CSeq_loc> loc(new CSeq_loc());
    if (m_Data.empty()) {
        loc->CSeq_loc_Base::SetNull();
        return loc;
    }

    bool partial5 = m_Partial5->GetValue();
    bool partial3 = m_Partial3->GetValue();

    if (m_Data.size() == 1) {
        if (m_Data[0].m_Empty) {
            loc->CSeq_loc_Base::SetNull();
        } else {
            CRef<CSeq_interval> seq_int = x_GetInterval(0);
            loc->CSeq_loc_Base::SetInt(*seq_int);
            loc->SetPartialStart(partial5, eExtreme_Biological);
            loc->SetPartialStop(partial3, eExtreme_Biological);
        }
        return loc;
    }

    if (m_LocType == 0) {
        CRef<CPacked_seqint> packed(new CPacked_seqint());
        for (size_t i = 0; i < m_Data.size(); ++i) {
            CRef<CSeq_interval> seq_int = x_GetInterval(i);
            packed->Set().push_back(seq_int);
        }
        loc->CSeq_loc_Base::SetPacked_int(*packed);
    }
    else {
        CRef<CSeq_loc_mix> mix(new CSeq_loc_mix());

        for (size_t i = 0; i < m_Data.size(); ++i) {
            CRef<CSeq_interval> seq_int = x_GetInterval(i);
            CRef<CSeq_loc> int_loc(new CSeq_loc());
            int_loc->CSeq_loc_Base::SetInt(*seq_int);
            mix->Set().push_back(int_loc);
            if (i != m_Data.size() - 1) {
                CRef<CSeq_loc> null_loc(new CSeq_loc());
                null_loc->CSeq_loc_Base::SetNull();
                mix->Set().push_back(null_loc);
            }
        }
        loc->CSeq_loc_Base::SetMix(*mix);
    }
    loc->SetPartialStart(partial5, eExtreme_Biological);
    loc->SetPartialStop(partial3, eExtreme_Biological);
    return loc;
}


static bool s_IsAllDigits(const string& str)
{
    static CRegexp all_digits_regex("^[0-9]+$");
    return all_digits_regex.IsMatch(str);
}


CRef<CSeq_interval> CLocationListCtrl::x_GetInterval(size_t index)
{
    if (m_Data[index].m_Empty) {
        return CRef<CSeq_interval>();
    }
    CRef<CSeq_interval> seq_int(new CSeq_interval());
    seq_int->SetFrom(m_Data[index].m_From - 1);
    seq_int->SetTo(m_Data[index].m_To - 1);

    ENa_strand strand = eNa_strand_unknown;
    switch(m_Data[index].m_Strand) {
        case 1 :
            strand = eNa_strand_plus;
            break;
        case 2 :
            strand = eNa_strand_minus;
            break;
        case 3 :
            strand = eNa_strand_other;
            break;
    }
    seq_int->SetStrand(strand);

    CRef<CSeq_id> seq_id(new CSeq_id());

    string id_str = ToStdString(m_Data[index].m_SeqId);
    try {
        seq_id->Set(id_str);
    } catch (CSeqIdException&) {
        // make a local ID
        seq_id->SetLocal().SetStr(id_str);
    }   
    if (seq_id->IsGi()) {
        // does a sequence with this GI exist in the scope?
        if (!m_Scope->GetBioseqHandle(*seq_id)) {
            seq_id->SetLocal().SetId(seq_id->GetGi());
        }
    }

    seq_int->SetId(*seq_id);

    return seq_int;
}

static void s_DeleteRow(wxSizerItemList::iterator row, wxSizerItemList& itemList)
{
    wxSizerItemList::iterator node = row;
    for (int i = 0; node != itemList.end() && i < kCtrlColNum; ++i) {
        (**node).DeleteWindows();
        node = itemList.erase(node);
    }
}

static wxSizerItemList::iterator s_FindRow(wxWindow* wnd, wxSizerItemList& itemList)
{
    if (NULL == wnd)
        return itemList.end();

    wxSizerItemList::iterator row, it = itemList.begin();

    for(int index = 0; it != itemList.end(); ++it, --index) {
        if (index == 0) {
            row = it;
            index = kCtrlColNum;
        }

        wxSizer* sizer = (**it).GetSizer();
        if (sizer && sizer->GetItem(wnd))
            return row;
        else if ((**it).GetWindow() == wnd)
            return row;
    }

    return it;
}

static wxSizerItemList::iterator s_NextRow(wxSizerItemList::iterator row, wxSizerItemList& itemList)
{
    wxSizerItemList::iterator it = row;
    for (int i = 0; i < kCtrlColNum; ++i, ++it) {
        if (it == itemList.end())
            return itemList.end();
    }
    return it;
}

static wxSizerItemList::iterator s_PrevRow(wxSizerItemList::iterator row, wxSizerItemList& itemList)
{
    wxSizerItemList::iterator it = row;
    for (int i = 0; i < kCtrlColNum; ++i, --it) {
        if (it == itemList.begin())
            return itemList.end();
    }
    return it;
}

static void s_SwapRows(wxSizerItemList::iterator row1,
                       wxSizerItemList::iterator row2,
                       wxSizerItemList& itemList)
{
    for (int i = 0; i < 4; ++i) {
        if (row1 == itemList.end() || row2 == itemList.end())
            break;
        swap(*row1++, *row2++);
    }
}

static void s_GetRowData(wxSizerItemList::iterator row, wxSizerItemList& itemList,
                         int& from, int& to, int& strand, wxString& seqID)
{
    wxSizerItemList::iterator node = row;

    if (node == itemList.end())return;

    wxTextCtrl* text = (wxTextCtrl*)(**node).GetWindow();
    from = wxAtoi(text->GetValue());
    if (++node == itemList.end()) return;

    text = (wxTextCtrl*)(**node).GetWindow();
    to = wxAtoi(text->GetValue());
    if (++node == itemList.end()) return;

    wxComboBox* combo = (wxComboBox*)(**node).GetWindow();
    strand = combo->GetSelection();
    if (++node == itemList.end()) return;

    combo = (wxComboBox*)(**node).GetWindow();
    seqID = combo->GetValue();
}

void CLocationListCtrl::OnDelete (wxHyperlinkEvent& event)
{
    wxSizerItemList& itemList = m_Sizer->GetChildren();
    wxSizerItemList::iterator row = s_FindRow((wxWindow*)event.GetEventObject(), itemList);
    s_DeleteRow(row, itemList);
    if (m_Sizer->GetChildren().size() == 0) {
        x_AddEmptyRow();
    }
    m_ScrolledWindow->FitInside();
}

void CLocationListCtrl::OnSpinCtrl ( wxCommandEvent& evt )
{
    wxWindow* spnCtrl = (wxWindow*)evt.GetEventObject();

    wxSizerItemList& itemList = m_Sizer->GetChildren();
    wxSizerItemList::iterator row = s_FindRow(spnCtrl, itemList);
    if (row == itemList.end())
        return;

    wxSizerItemList::iterator row2;

    switch (evt.GetId()) {
        case CSpinControl::kBtnUp:
            row2 = s_PrevRow(row, itemList);
            s_SwapRows(row, row2, itemList);
            m_Sizer->Layout();
            m_ScrolledWindow->Refresh();
            break;
        case CSpinControl::kBtnDn:
            row2 = s_NextRow(row, itemList);
            s_SwapRows(row, row2, itemList);
            m_Sizer->Layout();
            m_ScrolledWindow->Refresh();
            break;
        case 2 + CSpinControl::kBtnUp:
            {{
                size_t rowPos = itemList.IndexOf(*row);

                int from = 1, to = 1, strand = 0;
                wxString seqID;
                s_GetRowData(row, itemList, from, to, strand, seqID);
                if (from > 1)
                    --from;
                x_AddRow(from, from, strand, seqID, rowPos);
                m_Sizer->Layout();
                m_ScrolledWindow->FitInside();
                m_ScrolledWindow->Refresh();
            }}
            break;
        case 2 + CSpinControl::kBtnDn:
            {{
                int from = 1, to = 1, strand = 0;
                wxString seqID;
                s_GetRowData(row, itemList, from, to, strand, seqID);

                row = s_NextRow(row, itemList);
                size_t rowPos = (row == itemList.end()) ? (size_t)-1 : itemList.IndexOf(*row);
                ++to;
                x_AddRow(to, to, strand, seqID, rowPos);
                m_Sizer->Layout();
                m_ScrolledWindow->FitInside();
                m_ScrolledWindow->Refresh();
            }}
            break;
    }
}

/*
void CLocationListCtrl::OnAddIntClicked( wxHyperlinkEvent& WXUNUSED(event) )
{
    x_AddRow(0, 0, 0, wxT(""));
    m_ScrolledWindow->FitInside();

    int h, hv;
    m_ScrolledWindow->GetClientSize(0, &h);
    m_ScrolledWindow->GetVirtualSize(0, &hv);

    int xUnit, yUnit;
    m_ScrolledWindow->GetScrollPixelsPerUnit(&xUnit, &yUnit);

    if (hv > h) m_ScrolledWindow->Scroll(-1, (hv - h + yUnit)/yUnit);

    wxSizerItemList::iterator node = m_Sizer->GetChildren().end();
    for (int i = 0; i < kCtrlColNum; ++i)
        --node;

    wxTextCtrl* text = (wxTextCtrl*)(**node).GetWindow();
    if (text != 0) {
        text->SetFocus();
        text->SetSelection(-1, -1);
    }
}
 */

void CLocationListCtrl::x_AddRow(int from, int to, int strand, const wxString& seqID, size_t rowPos)
{
    wxTextCtrl *textFrom;
    textFrom = new wxTextCtrl(m_ScrolledWindow, wxID_ANY, wxT(""), wxDefaultPosition,
                          wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(35, -1)).x, -1),
                          wxTE_RIGHT);
    *textFrom << from;
    if (rowPos == (size_t)-1)
        m_Sizer->Add(textFrom, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else 
        m_Sizer->Insert(rowPos++, textFrom, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxTextCtrl *text;
    text = new wxTextCtrl(m_ScrolledWindow, wxID_ANY, wxT(""), wxDefaultPosition,
                          wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(35, -1)).x, -1),
                          wxTE_RIGHT);
    *text << to;
    if (rowPos == (size_t)-1)
        m_Sizer->Add(text, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, text, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxArrayString itemComboBoxStrings;
    itemComboBoxStrings.Add(wxT("N/A"));
    itemComboBoxStrings.Add(wxT("Plus"));
    itemComboBoxStrings.Add(wxT("Minus"));
    itemComboBoxStrings.Add(wxT("Other"));

    wxComboBox* combo = new wxComboBox(m_ScrolledWindow, wxID_ANY, wxT(""),
                                       wxDefaultPosition,
                                       wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(45, -1)).x, -1),
                                       itemComboBoxStrings, wxCB_READONLY);
    combo->SetSelection(strand);

    if (rowPos == (size_t)-1)
        m_Sizer->Add(combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    combo = new wxComboBox(m_ScrolledWindow, wxID_ANY, seqID,
                           wxDefaultPosition,
                           wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(65, -1)).x, -1),
                           m_SeqIds);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxBoxSizer* controlsSizer = new wxBoxSizer(wxHORIZONTAL);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(controlsSizer);
    else
        m_Sizer->Insert(rowPos++, controlsSizer);

    wxHyperlinkCtrl* itemHyperLink = new wxHyperlinkCtrl(m_ScrolledWindow, wxID_ANY, wxT("Delete"), wxT(""));
    itemHyperLink->SetVisitedColour(itemHyperLink->GetNormalColour());
    controlsSizer->Add(itemHyperLink, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    CSpinControl* spinCtrl = new CSpinControl(m_ScrolledWindow, wxID_ANY, wxDefaultPosition);
    controlsSizer->Add(spinCtrl, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    spinCtrl = new CSpinControl(m_ScrolledWindow, wxID_ANY, wxDefaultPosition);
    spinCtrl->UseImageSet(1);
    controlsSizer->Add(spinCtrl, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    textFrom->SetFocus();
    textFrom->SetSelection(-1, -1);
}

void CLocationListCtrl::x_AddEmptyRow(size_t rowPos)
{
    wxTextCtrl *textFrom;
    textFrom = new wxTextCtrl(m_ScrolledWindow, wxID_ANY, wxT(""), wxDefaultPosition,
                          wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(35, -1)).x, -1),
                          wxTE_RIGHT);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(textFrom, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else 
        m_Sizer->Insert(rowPos++, textFrom, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxTextCtrl *text;
    text = new wxTextCtrl(m_ScrolledWindow, wxID_ANY, wxT(""), wxDefaultPosition,
                          wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(35, -1)).x, -1),
                          wxTE_RIGHT);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(text, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, text, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxArrayString itemComboBoxStrings;
    itemComboBoxStrings.Add(wxT("N/A"));
    itemComboBoxStrings.Add(wxT("Plus"));
    itemComboBoxStrings.Add(wxT("Minus"));
    itemComboBoxStrings.Add(wxT("Other"));

    wxComboBox* combo = new wxComboBox(m_ScrolledWindow, wxID_ANY, wxT(""),
                                       wxDefaultPosition,
                                       wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(45, -1)).x, -1),
                                       itemComboBoxStrings, wxCB_READONLY);

    if (rowPos == (size_t)-1)
        m_Sizer->Add(combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    combo = new wxComboBox(m_ScrolledWindow, wxID_ANY, wxT(""),
                           wxDefaultPosition,
                           wxSize(m_ScrolledWindow->ConvertDialogToPixels(wxSize(65, -1)).x, -1),
                           m_SeqIds);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    else
        m_Sizer->Insert(rowPos++, combo, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxBoxSizer* controlsSizer = new wxBoxSizer(wxHORIZONTAL);
    if (rowPos == (size_t)-1)
        m_Sizer->Add(controlsSizer);
    else
        m_Sizer->Insert(rowPos++, controlsSizer);

    wxHyperlinkCtrl* itemHyperLink = new wxHyperlinkCtrl(m_ScrolledWindow, wxID_ANY, wxT("Delete"), wxT(""));
    itemHyperLink->SetVisitedColour(itemHyperLink->GetNormalColour());
    controlsSizer->Add(itemHyperLink, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    CSpinControl* spinCtrl = new CSpinControl(m_ScrolledWindow, wxID_ANY, wxDefaultPosition);
    controlsSizer->Add(spinCtrl, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    spinCtrl = new CSpinControl(m_ScrolledWindow, wxID_ANY, wxDefaultPosition);
    spinCtrl->UseImageSet(1);
    controlsSizer->Add(spinCtrl, 0, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 0);

    textFrom->SetFocus();
    textFrom->SetSelection(-1, -1);
}

void CLocationListCtrl::OnSortClick( wxCommandEvent& WXUNUSED(event) )
{
    vector<SIntData> intVec;

    SIntData intData;
    wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();

    for (;  node != children.end(); ) {
        wxTextCtrl* text = (wxTextCtrl*)(**node).GetWindow();
        if (++node == children.end()) break;
        wxString from = text->GetValue();

        text = (wxTextCtrl*)(**node).GetWindow();
        if (++node == children.end()) break;
        wxString to = text->GetValue();

        wxComboBox* strand_combo = (wxComboBox*)(**node).GetWindow();
        if (++node == children.end()) break;

        wxComboBox* id_combo = (wxComboBox*)(**node).GetWindow();
        if (++node == children.end()) break;

        ++node;  // Controls sizer

        if (from.IsEmpty() && to.IsEmpty()) {
            intData.m_Empty = true;
        } else {
            intData.m_From = wxAtoi(from);
            intData.m_To = wxAtoi(to);
            intData.m_Empty = false;
        }

        intData.m_Strand = strand_combo->GetSelection();
        intData.m_SeqId = id_combo->GetValue();

        intVec.push_back(intData);
    }

    if (intVec.empty())
        return;

    sort(intVec.begin(), intVec.end());

    for (node = children.begin();  node != children.end();  ++node)
        (**node).DeleteWindows();

    children.clear();

    vector<SIntData>::const_iterator it;
    int num_added = 0;
    for (it = intVec.begin(); it != intVec.end(); ++it) {
        if (!it->m_Empty) {
            x_AddRow(it->m_From, it->m_To, it->m_Strand, it->m_SeqId);
            num_added++;
        }
    }
    if (num_added == 0) {
        x_AddEmptyRow();
    }

    m_ScrolledWindow->FitInside();
}

bool CLocationListCtrl::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;

    wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();
    for (;  node != children.end();  ++node)
        (**node).DeleteWindows();
    children.clear();

    vector<SIntData>::const_iterator it;
    set<wxString> idsSet;
    for (it = m_Data.begin(); it != m_Data.end(); ++it)
        idsSet.insert(it->m_SeqId);

    m_SeqIds.Empty();
    if (m_Scope && (m_AllowNuc || m_AllowProt)) {
        CScope::TTSE_Handles tses;
        m_Scope->GetAllTSEs(tses, CScope::eAllTSEs);
        if (m_AllowNuc && m_AllowProt) {
            ITERATE (CScope::TTSE_Handles, handle, tses) {
                for (CBioseq_CI bioseq_it(*handle);  bioseq_it;  ++bioseq_it) {
                    string str;
                    CLabel::GetLabel(*(bioseq_it->GetSeqId()), &str, CLabel::eDefault, m_Scope);
                    m_SeqIds.Add(ToWxString (str));
                }
            }
        } else if (m_AllowNuc) {
            ITERATE (CScope::TTSE_Handles, handle, tses) {
                for (CBioseq_CI bioseq_it(*handle, CSeq_inst::eMol_na);  bioseq_it;  ++bioseq_it) {
                    string str;
                    CLabel::GetLabel(*(bioseq_it->GetSeqId()), &str, CLabel::eDefault, m_Scope);
                    m_SeqIds.Add(ToWxString (str));
                }
            }
        } else {
            ITERATE (CScope::TTSE_Handles, handle, tses) {
                for (CBioseq_CI bioseq_it(*handle, CSeq_inst::eMol_aa);  bioseq_it;  ++bioseq_it) {
                    string str;
                    CLabel::GetLabel(*(bioseq_it->GetSeqId()), &str, CLabel::eDefault, m_Scope);
                    m_SeqIds.Add(ToWxString (str));
                }
            }
        }
    } else {
        set<wxString>::const_iterator it2;
        for (it2 = idsSet.begin(); it2 != idsSet.end(); ++it2)
            m_SeqIds.Add(*it2);
    }

    for (it = m_Data.begin(); it != m_Data.end(); ++it) {
        if (it->m_Empty) {
            x_AddEmptyRow ();
        } else {
            x_AddRow(it->m_From, it->m_To, it->m_Strand, it->m_SeqId);
        }
    }

    m_ScrolledWindow->FitInside();

    return true;
}

bool CLocationListCtrl::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    m_Data.clear();

    SIntData intData;
    wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();

    for (;  node != children.end(); ) {
        wxTextCtrl* text = (wxTextCtrl*)(**node).GetWindow();
        if (++node == children.end()) break;
        wxString from = text->GetValue();

        text = (wxTextCtrl*)(**node).GetWindow();
        if (++node == children.end()) break;
        wxString to = text->GetValue();

        wxComboBox* strand_combo = (wxComboBox*)(**node).GetWindow();
        if (++node == children.end()) break;

        wxComboBox* id_combo = (wxComboBox*)(**node).GetWindow();
        if (++node == children.end()) break;

        ++node;  // "Delete" Hyperlink

        if (from.IsEmpty() && to.IsEmpty()) {
            intData.m_Empty = true;
        } else {
            if (from.IsEmpty()) {
                from = to;
            } else if (to.IsEmpty()) {
                to = from;
            }

            intData.m_From = wxAtoi(from);
            intData.m_To = wxAtoi(to);
            intData.m_Strand = strand_combo->GetSelection();
            intData.m_SeqId = id_combo->GetValue();
            intData.m_Empty = false;
        }
        m_Data.push_back(intData);
    }

    return true;
}

/*!
 * Should we show tooltips?
 */

bool CLocationListCtrl::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CLocationListCtrl::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CLocationListCtrl bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CLocationListCtrl bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CLocationListCtrl::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CLocationListCtrl icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CLocationListCtrl icon retrieval
}

END_NCBI_SCOPE


