/*  $Id: singleauthor_panel.cpp 27653 2013-03-20 17:05:20Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 */


#include <ncbi_pch.hpp>
#include <gui/objutils/undo_manager_interface.hpp>
#include <gui/widgets/edit/edit_object.hpp>
#include <objects/biblio/Author.hpp>
#include <objects/general/Person_id.hpp>
#include <objects/general/Name_std.hpp>

#include <gui/widgets/edit/serial_member_primitive_validators.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

////@begin includes
////@end includes
////@begin includes
////@end includes

#include "singleauthor_panel.hpp"
#include <gui/widgets/edit/authornames_panel.hpp>

#include <wx/sizer.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>
#include "wx/valtext.h"


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*
 * CSingleAuthorPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSingleAuthorPanel, wxPanel )


/*
 * CSingleAuthorPanel event table definition
 */

BEGIN_EVENT_TABLE( CSingleAuthorPanel, wxPanel )

////@begin CSingleAuthorPanel event table entries
    EVT_TEXT( ID_AUTH_LAST, CSingleAuthorPanel::OnAuthLastTextUpdated )

    EVT_HYPERLINK( ID_HYPERLINKCTRL, CSingleAuthorPanel::OnHyperlinkctrlHyperlinkClicked )

////@end CSingleAuthorPanel event table entries

END_EVENT_TABLE()


/*
 * CSingleAuthorPanel constructors
 */

CSingleAuthorPanel::CSingleAuthorPanel()
{
    Init();
}

CSingleAuthorPanel::CSingleAuthorPanel( wxWindow* parent, CAuthor& author,
         wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) :
     m_Author(0)
{
    Init();
    m_Author = new CAuthor();
    m_Author->Assign(author);
    m_Author->SetName().SetName();
    Create(parent, id, pos, size, style);
}


/*
 * CSingleAuthorPanel creator
 */

bool CSingleAuthorPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSingleAuthorPanel creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSingleAuthorPanel creation
    return true;
}


/*
 * CSingleAuthorPanel destructor
 */

CSingleAuthorPanel::~CSingleAuthorPanel()
{
////@begin CSingleAuthorPanel destruction
////@end CSingleAuthorPanel destruction
}


/*
 * Member initialisation
 */

void CSingleAuthorPanel::Init()
{
////@begin CSingleAuthorPanel member initialisation
    m_FirstNameCtrl = NULL;
    m_MiddleInitial = NULL;
    m_LastNameCtrl = NULL;
    m_Suffix = NULL;
////@end CSingleAuthorPanel member initialisation
}


static string s_GetFirstNameInitials(string first_name)
{
    char  buf[2];

    string inits = "";
    buf[1] = 0;

    string::iterator p = first_name.begin();
    while (p != first_name.end()) {
        // skip leading punct
        while (p != first_name.end() && (*p <= ' ' || *p == '-')) {
            p++;
        }
        if (p != first_name.end() && isalpha (*p)) {
            buf[0] = *p;
            inits.append(buf);
            p++;
        }
        // skip rest of name
        while (p != first_name.end() && *p > ' ' && *p != '-') {
            p++;
        }
        if (p != first_name.end() && *p == '-') {
            buf[0] = *p;
            inits.append(buf);
            p++;
        }
    }
    return inits;
}


/*
 * Control creation for CSingleAuthorPanel
 */

void CSingleAuthorPanel::CreateControls()
{    
////@begin CSingleAuthorPanel content construction
    CSingleAuthorPanel* itemPanel1 = this;

    wxFlexGridSizer* itemFlexGridSizer2 = new wxFlexGridSizer(2, 5, 0, 0);
    itemPanel1->SetSizer(itemFlexGridSizer2);

    m_FirstNameCtrl = new wxTextCtrl( itemPanel1, ID_AUTH_FIRST, wxEmptyString, wxDefaultPosition, wxSize(100, -1), 0 );
    itemFlexGridSizer2->Add(m_FirstNameCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_MiddleInitial = new wxTextCtrl( itemPanel1, ID_AUTH_MI, wxEmptyString, wxDefaultPosition, wxSize(100, -1), 0 );
    itemFlexGridSizer2->Add(m_MiddleInitial, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_LastNameCtrl = new wxTextCtrl( itemPanel1, ID_AUTH_LAST, wxEmptyString, wxDefaultPosition, wxSize(100, -1), 0 );
    itemFlexGridSizer2->Add(m_LastNameCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxArrayString m_SuffixStrings;
    m_SuffixStrings.Add(wxEmptyString);
    m_SuffixStrings.Add(_("Jr."));
    m_SuffixStrings.Add(_("Sr."));
    m_SuffixStrings.Add(_("II"));
    m_SuffixStrings.Add(_("III"));
    m_SuffixStrings.Add(_("IV"));
    m_SuffixStrings.Add(_("V"));
    m_SuffixStrings.Add(_("VI"));
    m_Suffix = new wxChoice( itemPanel1, ID_AUTH_SUFFIX, wxDefaultPosition, wxSize(50, -1), m_SuffixStrings, 0 );
    itemFlexGridSizer2->Add(m_Suffix, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxHyperlinkCtrl* itemHyperlinkCtrl7 = new wxHyperlinkCtrl( itemPanel1, ID_HYPERLINKCTRL, _("Delete"), wxEmptyString, wxDefaultPosition, wxSize(50, -1), wxHL_DEFAULT_STYLE );
    itemFlexGridSizer2->Add(itemHyperlinkCtrl7, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

////@end CSingleAuthorPanel content construction
    // first name
    if (m_Author->GetName().GetName().IsSetFirst()) {
        m_FirstName = ToWxString (m_Author->GetName().GetName().GetFirst());
    } else {
        m_FirstName = wxEmptyString;
    }
    m_FirstNameCtrl->SetValidator( wxTextValidator(wxFILTER_NONE, & m_FirstName ));
    // last name
    if (m_Author->GetName().GetName().IsSetLast()) {
        m_LastName = ToWxString (m_Author->GetName().GetName().GetLast());
    } else {
        m_LastName = wxEmptyString;
    }
    m_LastNameCtrl->SetValidator( wxTextValidator(wxFILTER_NONE, & m_LastName ));

    // suffix
    m_Suffix->SetSelection (1);
    string suffix = "";
    if (m_Author->SetName().SetName().IsSetSuffix()) {
        suffix = m_Author->GetName().GetName().GetSuffix();
    }
    
    unsigned int i;
    for (i = 0; i < m_SuffixStrings.size(); i++) {
        if (NStr::EqualNocase(ToAsciiStdString(m_SuffixStrings[i]), suffix)) {
            m_Suffix->SetSelection (i);
        }
    }
}


bool CSingleAuthorPanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;

    
    // need to do parsing for middle initial
    if (m_Author->SetName().SetName().IsSetInitials()) {
        string first_init = s_GetFirstNameInitials (m_Author->SetName().SetName().GetFirst());
        string middle_init = m_Author->GetName().GetName().GetInitials();
        if (NStr::StartsWith(middle_init, first_init, NStr::eNocase)) {
            middle_init = middle_init.substr(first_init.length());
        }
        if (NStr::EndsWith(middle_init, ".")) {
            middle_init = middle_init.substr(0, middle_init.length() - 1);
        }
        
        m_MiddleInitial->SetValue(ToWxString(middle_init));
    }
    

    return true;
}


bool CSingleAuthorPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    string firstname = ToAsciiStdString (m_FirstName);
    if (NStr::IsBlank(firstname)) {
        m_Author->SetName().SetName().ResetFirst();
    } else {
        m_Author->SetName().SetName().SetFirst(firstname);
    }
    string lastname = ToAsciiStdString (m_LastName);
    if (NStr::IsBlank(lastname)) {
        m_Author->SetName().SetName().ResetLast();
    } else {
        m_Author->SetName().SetName().SetLast(lastname);
    }

    // need to do parsing for middle initial
    string mi = ToAsciiStdString(m_MiddleInitial->GetValue());
    if (NStr::IsBlank(firstname)) {
        if (!NStr::IsBlank(mi)) {            
            m_Author->SetName().SetName().SetFirst(mi);
        }
        m_Author->SetName().SetName().ResetInitials();
    } else {
        string inits = s_GetFirstNameInitials(m_Author->SetName().SetName().GetFirst());
        if (!NStr::IsBlank(mi)) {
            inits.append(mi);
        }
        // don't add another period if one is already there
        if (!NStr::EndsWith(mi, ".")) {
            inits.append( ".");
        }
        m_Author->SetName().SetName().SetInitials(inits);
    }

    // suffix
    string suffix = ToAsciiStdString(m_Suffix->GetStringSelection());
    if (NStr::IsBlank(suffix)) {
        m_Author->SetName().SetName().ResetSuffix();
    } else {
        m_Author->SetName().SetName().SetSuffix (suffix.c_str());
    }

    return true;
}


CRef<CAuthor> CSingleAuthorPanel::GetAuthor() const
{
    if (m_Author)
        return m_Author;

    return CRef<CAuthor>();
}



/*
 * Should we show tooltips?
 */

bool CSingleAuthorPanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CSingleAuthorPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSingleAuthorPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSingleAuthorPanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CSingleAuthorPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSingleAuthorPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSingleAuthorPanel icon retrieval
}


/*
 * wxEVT_COMMAND_HYPERLINK event handler for ID_HYPERLINKCTRL
 */

void CSingleAuthorPanel::OnHyperlinkctrlHyperlinkClicked( wxHyperlinkEvent& event )
{
    wxWindow* parent = this->GetParent();

    CAuthorNamesPanel* authlistpanel = dynamic_cast<CAuthorNamesPanel*>(parent);

    while (parent && !authlistpanel) {
        parent = parent->GetParent();
        authlistpanel = dynamic_cast<CAuthorNamesPanel*>(parent);
    }

    if (!authlistpanel) {
        return;
    }
    authlistpanel->DeleteAuthor((wxWindow*)this);
}


/*
 * wxEVT_COMMAND_TEXT_UPDATED event handler for ID_AUTH_LAST
 */

void CSingleAuthorPanel::OnAuthLastTextUpdated( wxCommandEvent& event )
{
    wxTextCtrl* item = (wxTextCtrl*)event.GetEventObject();
    if (NStr::IsBlank(ToAsciiStdString(item->GetLabel()))) {
        return;
    }
    wxWindow* parent = this->GetParent();

    CAuthorNamesPanel* authlistpanel = dynamic_cast<CAuthorNamesPanel*>(parent);

    while (parent && !authlistpanel) {
        parent = parent->GetParent();
        authlistpanel = dynamic_cast<CAuthorNamesPanel*>(parent);
    }

    if (!authlistpanel) {
        return;
    }
    authlistpanel->AddLastAuthor((wxWindow*)this);
}


END_NCBI_SCOPE

