/*  $Id: srcmod_list_panel.cpp 28102 2013-05-08 21:48:25Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vasuki Palanigobu
 */

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include <ncbi_pch.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include "srcmod_list_panel.hpp"
#include <objects/seqfeat/Org_ref.hpp>
#include <objects/seqfeat/OrgName.hpp>
#include <objects/seqfeat/OrgMod.hpp>
#include <objects/seqfeat/SubSource.hpp>

#include <wx/checkbox.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*
 * CSrcModListPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSrcModListPanel, wxPanel )


/*
 * CSrcModListPanel event table definition
 */

BEGIN_EVENT_TABLE( CSrcModListPanel, wxPanel )

////@begin CSrcModListPanel event table entries
////@end CSrcModListPanel event table entries
    EVT_HYPERLINK(wxID_ANY, CSrcModListPanel::OnDelete)
    EVT_CHECKBOX(ID_PCR_CHECKBOX, CSrcModListPanel::OnPCRCheckBox)
    EVT_BUTTON(ID_ADD_PCR_PRIMER, CSrcModListPanel::OnAddPCRPrimer)

END_EVENT_TABLE()


/*
 * CSrcModListPanel constructors
 */

CSrcModListPanel::CSrcModListPanel():
    m_Source(NULL)
{
    Init();
}

CSrcModListPanel::CSrcModListPanel( wxWindow* parent, objects::CBioSource& source, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ):
    m_Source(new CBioSource())
{
    m_Source->Assign(source);
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * CSrcModListPanel creator
 */

bool CSrcModListPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSrcModListPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSrcModListPanel creation
    return true;
}


/*
 * CSrcModListPanel destructor
 */

CSrcModListPanel::~CSrcModListPanel()
{
////@begin CSrcModListPanel destruction
////@end CSrcModListPanel destruction
}


/*
 * Member initialisation
 */

void CSrcModListPanel::Init()
{
////@begin CSrcModListPanel member initialisation
    m_ScrolledWindow = NULL;
    m_Sizer = NULL;
    m_SrcInfo = NULL;
////@end CSrcModListPanel member initialisation
    m_PCRPrimersBoxSizer1 = NULL;
    m_PCRPrimersBoxSizer2 = NULL;
    m_PCRPrimersBoxSizer3 = NULL;
    m_PCRCheckbox = NULL;
    m_PCRPrimersPanel = NULL;
}


/*
 * Control creation for CSrcModListPanel
 */

void CSrcModListPanel::CreateControls()
{    
////@begin CSrcModListPanel content construction
    CSrcModListPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_ScrolledWindow = new wxScrolledWindow( itemPanel1, ID_SCROLLEDWINDOW1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    itemBoxSizer3->Add(m_ScrolledWindow, 1, wxGROW|wxTOP|wxBOTTOM|wxFIXED_MINSIZE, 0);
    m_ScrolledWindow->SetScrollbars(1, 1, 0, 0);
    m_Sizer = new wxFlexGridSizer(0, 2, 0, 0);
    m_ScrolledWindow->SetSizer(m_Sizer);

    m_ScrolledWindow->FitInside();

    itemBoxSizer2->Add(5, 5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer7, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel1, wxID_STATIC, _("Additional Organism/Source Information"), wxDefaultPosition, wxSize(300, -1), wxALIGN_CENTRE );
    itemBoxSizer7->Add(itemStaticText8, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_SrcInfo = new wxTextCtrl( itemPanel1, ID_TEXTCTRL7, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    itemBoxSizer7->Add(m_SrcInfo, 0, wxGROW|wxALL, 5);

////@end CSrcModListPanel content construction
    m_TotalHeight = 0;
    m_TotalWidth = 0;
    m_NumRows = 0;
    m_ScrollRate = 0;

    x_FillList();
    
// PCR-Primers
    wxBoxSizer* itemBoxSizer9 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer9, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    
    m_PCRCheckbox = new wxCheckBox(itemPanel1, ID_PCR_CHECKBOX, wxT("PCR Primers"), wxDefaultPosition, wxDefaultSize);
    if (m_Source->IsSetPcr_primers())
        m_PCRCheckbox->SetValue(true);
    else
        m_PCRCheckbox->SetValue(false);
    itemBoxSizer9->Add(m_PCRCheckbox, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
   
    m_PCRPrimersBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer9->Add(m_PCRPrimersBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
 
    m_PCRPrimersBoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer9->Add(m_PCRPrimersBoxSizer1, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);

    m_PCRPrimersBoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer9->Add(m_PCRPrimersBoxSizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);

   
    
    wxStaticText* itemStaticText12 = new wxStaticText( this, wxID_STATIC, _("Set"), wxDefaultPosition, wxSize(25,-1), wxALIGN_CENTRE );
    m_PCRPrimersBoxSizer1->Add(itemStaticText12, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
    
    wxStaticText* itemStaticText13 = new wxStaticText( this, wxID_STATIC, _("Direction"), wxDefaultPosition, wxSize(80,-1), wxALIGN_CENTRE );
    m_PCRPrimersBoxSizer1->Add(itemStaticText13, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
    
    wxStaticText* itemStaticText14 = new wxStaticText( this, wxID_STATIC, _("Name"), wxDefaultPosition, wxSize(80,-1), wxALIGN_CENTRE );
    m_PCRPrimersBoxSizer1->Add(itemStaticText14, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
    
    wxStaticText* itemStaticText15 = new wxStaticText( this, wxID_STATIC, _("Sequence"), wxDefaultPosition, wxSize(80,-1), wxALIGN_CENTRE );
    m_PCRPrimersBoxSizer1->Add(itemStaticText15, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
    
    wxStaticText* itemStaticText16 = new wxStaticText( this, wxID_STATIC, _("      "), wxDefaultPosition, wxSize(80,-1), wxALIGN_CENTRE );
    m_PCRPrimersBoxSizer1->Add(itemStaticText16, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);
    
    m_PCRPrimersPanel = new CPCRPrimersPanel( this, m_Source.GetPointer() );
    m_PCRPrimersBoxSizer2->Add(m_PCRPrimersPanel,  0, wxGROW|wxTOP|wxBOTTOM|wxFIXED_MINSIZE, 0); 

    wxButton* itemButton1 = new wxButton( this, ID_ADD_PCR_PRIMER, _("Add Another Primer"), wxDefaultPosition, wxDefaultSize, 0 );
    m_PCRPrimersBoxSizer3->Add(itemButton1, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    if (m_PCRCheckbox->IsChecked())
        x_AddPCRPrimersPanel();
    else
        x_ClearPCRPrimersPanel();        
}

void CSrcModListPanel::x_AddPCRPrimersPanel()
{
    m_PCRPrimersBoxSizer1->ShowItems(true);
    m_PCRPrimersBoxSizer2->ShowItems(true);
    m_PCRPrimersBoxSizer3->ShowItems(true);
    Layout();
    Refresh();
}

void CSrcModListPanel::x_ClearPCRPrimersPanel()
{
    m_PCRPrimersBoxSizer1->ShowItems(false);
    m_PCRPrimersBoxSizer2->ShowItems(false);
    m_PCRPrimersBoxSizer3->ShowItems(false);
    Layout();
    Refresh();
}

void CSrcModListPanel::OnPCRCheckBox(wxCommandEvent& event)
{
    if (m_PCRCheckbox->IsChecked())
        x_AddPCRPrimersPanel();
    else
        x_ClearPCRPrimersPanel();  
}

void CSrcModListPanel::OnAddPCRPrimer(wxCommandEvent& event)
{
    m_PCRPrimersPanel->x_AddEmptyRow();
    m_PCRPrimersPanel->x_AdjustScrollWindow();
}

bool IsNonText (string label)
{
    if (NStr::EqualNocase (label, "Environmental-Sample")
      || NStr::EqualNocase (label, "Germline")
      || NStr::EqualNocase (label, "Metagenomic")
      || NStr::EqualNocase (label, "Rearranged")
      || NStr::EqualNocase (label, "Transgenic")
      || NStr::EqualNocase (label, "Metagenomic")) {
        return true;
    } else {
        return false;
    }
}


void CSrcModListPanel::x_AddTextSrcMods ()
{
    m_TotalHeight = 0;
	  size_t cnt = m_Data.size();
    int num_text = 0;

    wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();
    for (;  node != children.end();  ++node)
        (**node).DeleteWindows();
    m_Sizer->Clear(true);

	  for (size_t i = 0; i < cnt; i++) {
		    
        if (NStr::EqualNocase(m_Data[i].name, "note")) {
            //save note for later
        } else if (IsNonText(ToStdString(m_Data[i].name))) {
            if (NStr::IsBlank(ToStdString(m_Data[i].value))) {
                x_AddRow(m_Data[i].name, wxT("TRUE"));
            } else {
                x_AddRow(m_Data[i].name, m_Data[i].value);
            }
            num_text++;
        } else {
			      x_AddRow(m_Data[i].name, m_Data[i].value);
            num_text++;
		    }
    }
	  x_AddEmptyRow ();

	  //Adjusting size of the scroll window
    Freeze();
    int m_MinHeight = 3 * (m_TotalHeight / (num_text + 1));
	  int win_height, win_width;
    m_ScrolledWindow->SetVirtualSize(m_TotalWidth, m_TotalHeight);
	  m_ScrolledWindow->GetSize(&win_width, &win_height);
    m_ScrolledWindow->SetScrollRate(0, m_ScrollRate);
	  m_ScrolledWindow->GetSize(&win_width, &win_height);
	  m_ScrolledWindow->FitInside();
	  m_ScrolledWindow->GetSize(&win_width, &win_height);
	  Layout();
	  m_ScrolledWindow->SetSize(m_TotalWidth + 20, m_MinHeight);
	  m_ScrolledWindow->SetMinSize(wxSize(m_TotalWidth + 20, m_MinHeight));
    Layout();
	  Thaw();
}


bool CSrcModListPanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;

    m_SrcInfo->Clear();

    x_FillList();
    size_t cnt = m_Data.size();
      
    for (size_t i = 0; i < cnt; i++) {
        if (m_Data[i].name == "Note") {
            m_SrcInfo->SetValue(m_Data[i].value);
        }
    }

    x_AddTextSrcMods();


    return true;
}


void CSrcModListPanel::x_AddEmptyRow()
{   
	  CSrcModPanel::SModData data;

	  CSrcModPanel* panel = new CSrcModPanel(m_ScrolledWindow, data);
    m_Sizer->Add(panel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxHyperlinkCtrl* itemHyperlinkCtrl = new wxHyperlinkCtrl( m_ScrolledWindow, wxID_ANY, 
              wxT("Delete"), wxT(""), wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
	  m_Sizer->Add(itemHyperlinkCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    int row_width;
    int row_height;
    panel->GetClientSize(&row_width, &row_height);
    int link_width;
    int link_height;
    itemHyperlinkCtrl->GetClientSize (&link_width, &link_height);
    if (link_height > row_height) {
      row_height = link_height;
    }
    row_width += link_width;

    if (row_width > m_TotalWidth) {
        m_TotalWidth = row_width;
    }
    m_TotalHeight += row_height;
    m_ScrollRate = row_height;
    m_NumRows++;
}


void CSrcModListPanel::x_AddRow(wxString name, wxString value)
{
	  CSrcModPanel::SModData data;
	  data.name = name;
	  data.value = value;

	  CSrcModPanel* panel = new CSrcModPanel(m_ScrolledWindow, data);
	  panel->SetModifierData();
    m_Sizer->Add(panel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    wxHyperlinkCtrl* itemHyperlinkCtrl = new wxHyperlinkCtrl( m_ScrolledWindow, wxID_ANY, 
              wxT("Delete"), wxT(""), wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
	  m_Sizer->Add(itemHyperlinkCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
    int row_width;
    int row_height;
    panel->GetClientSize(&row_width, &row_height);
    int link_width;
    int link_height;
    itemHyperlinkCtrl->GetClientSize (&link_width, &link_height);
    if (link_height > row_height) {
      row_height = link_height;
    }
    row_width += link_width;

    if (row_width > m_TotalWidth) {
        m_TotalWidth = row_width;
    }

    m_TotalHeight += row_height;
    m_ScrollRate = row_height;
    m_NumRows++;
}


void CSrcModListPanel::x_CollectTextMods ()
{
	  wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();
    int pos = 0;
    for (; node != children.end(); ++node) {
        wxWindow *w = (*node)->GetWindow();
        if (!w) {
            continue;
        }
        CSrcModPanel* singlemod_panel = dynamic_cast<CSrcModPanel*> (w);
        if (singlemod_panel) {
	          CSrcModPanel::SModData single_modifier_data = singlemod_panel->GetModifierData();
            if (!NStr::IsBlank(ToStdString(single_modifier_data.name)) && !NStr::IsBlank(ToStdString(single_modifier_data.value))) {
                if (IsNonText(ToStdString(single_modifier_data.name)) && NStr::EqualNocase(ToStdString(single_modifier_data.value), "TRUE")) {
                    x_SetBioSourceModifier(ToStdString(single_modifier_data.name), " ");
                } else {
                    x_SetBioSourceModifier(ToStdString(single_modifier_data.name), ToStdString(single_modifier_data.value));
                }
            }
            pos++;
        }
    }
}


bool CSrcModListPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

	  //Reset modifiers
	  if (m_Source->IsSetSubtype()) {
		    m_Source->ResetSubtype();
	  }
	  if (m_Source->IsSetOrg()  &&  m_Source->GetOrg().IsSetOrgname()  &&  
		    m_Source->GetOrg().GetOrgname().IsSetMod()) {
		    m_Source->SetOrg().SetOrgname().ResetMod();
	  }


	  if (!NStr::IsBlank(ToStdString(m_SrcInfo->GetValue()))) {
		    x_SetBioSourceModifier("Note", ToStdString(m_SrcInfo->GetValue()));
	  }

    x_CollectTextMods ();

	  return true;
}


void CSrcModListPanel::x_SetBioSourceModifier(const string& name, const string& value)
{
    try {
        int subtype = CSubSource::GetSubtypeValue(name);
        x_SetSrcSubMod(subtype, value);
    } catch (std::exception&) {
        try {
            int subtype = COrgMod::GetSubtypeValue(name);
        		x_SetSrcOrgMod(subtype, value);
        } catch (std::exception&) {
            // do something?
        }
    }
}


void CSrcModListPanel::x_SetSrcOrgMod(int subtype, const string& value)
{
	if (m_Source->GetOrg().IsSetOrgname()  &&  
		m_Source->GetOrg().GetOrgname().IsSetMod()) {
			COrgName::TMod& mod = m_Source->SetOrg().SetOrgname().SetMod();
			//On edit
			NON_CONST_ITERATE(COrgName::TMod, mod_it, mod) { 
				if ((*mod_it)->CanGetSubtype()) { 
					if ((*mod_it)->GetSubtype() == subtype) { 
						(*mod_it)->SetSubname(value); 
						return;
					}    
				}    
			}
	}

	//insert new modifier
	CRef<COrgMod> org_mod(new COrgMod()); 
	org_mod->SetSubtype(subtype); 
	org_mod->SetSubname(value); 
	m_Source->SetOrg().SetOrgname().SetMod().push_back(org_mod);
}


void CSrcModListPanel::x_SetSrcSubMod(int subtype, const string& value)
{
	if (m_Source->IsSetSubtype()) {
       CBioSource::TSubtype& types = m_Source->SetSubtype();
        //On edit
        NON_CONST_ITERATE(CBioSource::TSubtype, type_it, types) {
            if( (*type_it)->CanGetSubtype()) {
                if ((*type_it)->GetSubtype() == subtype) {
                    (*type_it)->SetName(value);
                    return;
                }
            }
        }
	}

	//insert new modifier
	CRef<CSubSource> sub_src (new CSubSource());
	sub_src->SetSubtype(subtype);
	sub_src->SetName(value);
	m_Source->SetSubtype().push_back(sub_src);
}


void CSrcModListPanel::x_FillList()
{
    m_Data.clear();
	if (m_Source->IsSetSubtype()) {
		ITERATE(CBioSource::TSubtype, src_it, m_Source->GetSubtype()) {
			CSrcModPanel::SModData data;
      data.name = ToWxString(CSubSource::GetSubtypeName((*src_it)->GetSubtype()));
			data.value = ToWxString((*src_it)->GetName());
			m_Data.push_back(data);
		}
	}

	if (m_Source->IsSetOrg()  &&  m_Source->GetOrg().IsSetOrgname()  &&  
		m_Source->GetOrg().GetOrgname().IsSetMod()) 
	{
		ITERATE(COrgName::TMod, src_it, m_Source->GetOrg().GetOrgname().GetMod())
		{
			CSrcModPanel::SModData data;
      data.name = ToWxString(COrgMod::GetSubtypeName((*src_it)->GetSubtype()));
			data.value = ToWxString((*src_it)->GetSubname());
			m_Data.push_back(data);
		}
	}
}


void CSrcModListPanel::x_UpdateList()
{
    TVecModData::iterator dit = m_Data.begin();
    wxSizerItemList& children = m_Sizer->GetChildren();
    wxSizerItemList::iterator node = children.begin();
    int j = 0;
    int k = -1;
    while (dit != m_Data.end() && node != children.end()) 
    {
        //ignore note
        if (!NStr::EqualNocase(ToStdString (dit->name), "Note"))
        {
            while (k < j && node != children.end()) 
            {
                wxWindow *w = (*node)->GetWindow();
                if (!w) 
                {
                    continue;
                }
                CSrcModPanel* singlemod_panel = dynamic_cast<CSrcModPanel*> (w);
                if (singlemod_panel) 
                {
                    k++;
                    if (k == j) 
                    {
                        CSrcModPanel::SModData single_modifier_data = singlemod_panel->GetModifierData();
                        dit->value = single_modifier_data.value;
                    }
                }
                node++;
            }
            j++;
        }
        dit++;
    }
    while (node != children.end()) {
        // add new items
        wxWindow *w = (*node)->GetWindow();
        if (!w) 
        {
            continue;
        }
        CSrcModPanel* singlemod_panel = dynamic_cast<CSrcModPanel*> (w);
        if (singlemod_panel) 
        {
            CSrcModPanel::SModData single_modifier_data = singlemod_panel->GetModifierData();
            if (!NStr::IsBlank(ToStdString(single_modifier_data.name)) || !NStr::IsBlank(ToStdString(single_modifier_data.value)))
            {
                m_Data.push_back(single_modifier_data);
            }
        }
        node++;
    }
}


void CSrcModListPanel::PopulateModifiers(CBioSource& biosrc)
{
    biosrc.ResetSubtype();
    biosrc.SetOrg().SetOrgname().ResetMod();
    if (m_Source->IsSetSubtype()) {
        ITERATE(CBioSource::TSubtype, src_it, m_Source->GetSubtype()) {
            CRef<CSubSource> sub_src (new CSubSource());
            sub_src->SetSubtype((*src_it)->GetSubtype());
            sub_src->SetName((*src_it)->GetName());
			      biosrc.SetSubtype().push_back(sub_src);
        }
    }
    
    if (m_Source->IsSetOrg()  &&  m_Source->GetOrg().IsSetOrgname()  &&  
        m_Source->GetOrg().GetOrgname().IsSetMod()) 
    {	
        ITERATE(COrgName::TMod, src_it, m_Source->GetOrg().GetOrgname().GetMod())
        {
            CRef<COrgMod> org_mod(new COrgMod()); 
            org_mod->SetSubtype((*src_it)->GetSubtype()); 
            org_mod->SetSubname((*src_it)->GetSubname()); 
            biosrc.SetOrg().SetOrgname().SetMod().push_back(org_mod);
        }
    }
    if (m_PCRPrimersPanel)
        m_PCRPrimersPanel->UpdateBioSourcePrimers(biosrc);
}


static bool s_IsLastRow(wxWindow* wnd, wxSizerItemList& itemList)
{
    bool is_last = false;
    if (NULL == wnd)
        return false;

    for(wxSizerItemList::iterator it = itemList.begin(); it != itemList.end(); ++it) {
        wxWindow* child = (**it).GetWindow();
        if (child && child == wnd) {
            is_last = true;
        } else {
            if (is_last) {
                CSrcModPanel* qual = dynamic_cast<CSrcModPanel*> (child);
		            if (qual) {
                    is_last = false;
                    return is_last;
                }
            }
        }
    }

    return is_last;
}


void CSrcModListPanel::AddLastSrcMod (wxWindow* link)
{
    // find the row that holds the author to be deleted
    wxSizerItemList& itemList = m_Sizer->GetChildren();

    if (s_IsLastRow(link, itemList)) {
        x_AddEmptyRow ();
        m_ScrolledWindow->SetVirtualSize(m_TotalWidth + 10, m_TotalHeight);
        m_ScrolledWindow->SetScrollRate(0, m_ScrollRate);
        m_ScrolledWindow->FitInside();
    }
}


int CSrcModListPanel::x_FindRow(wxWindow* wnd, wxSizerItemList& itemList)
{
    if (NULL == wnd)
        return -1;

	// start with -1, first qual found will bump it to 0
    int row_num = -1;
    for(wxSizerItemList::iterator it = itemList.begin(); it != itemList.end(); ++it) {
        wxWindow* child = (**it).GetWindow();
        if (child && child == wnd)
            return row_num;
        
        CSrcModPanel* qual = dynamic_cast<CSrcModPanel*> (child);
		    if (qual) {
			      row_num++;
		    }
    }

    return -1;
}


void CSrcModListPanel::OnDelete (wxHyperlinkEvent& event)
{
	  // find the control, remove it and the item before it, reset the scrolling
	  wxWindow* wnd = (wxWindow*)event.GetEventObject();
	  if (wnd == NULL) 
    {
		    return;
	  }
    wxSizerItemList& itemList = m_Sizer->GetChildren();
	  int row = x_FindRow (wnd, itemList);
	  if (row > -1) 
    {       
        x_UpdateList();
        int j = 0;
        TVecModData::iterator dit = m_Data.begin();
        while (dit != m_Data.end()) 
        {
		        //ignore note
            if (!NStr::EqualNocase(ToStdString (dit->name), "Note"))
		        {
                if (j == row) {
                    m_Data.erase(dit);
                    break;
                } else {
                    j++;
                }
		        }
            dit++;
        }
        x_AddTextSrcMods ();
	  }
}


/*
 * Should we show tooltips?
 */

bool CSrcModListPanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CSrcModListPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSrcModListPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSrcModListPanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CSrcModListPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSrcModListPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSrcModListPanel icon retrieval
}

END_NCBI_SCOPE
