#ifndef GUI_WIDGETS_EDIT___SRCMOD_LIST_PANEL__HPP
#define GUI_WIDGETS_EDIT___SRCMOD_LIST_PANEL__HPP

/*  $Id: srcmod_list_panel.hpp 28102 2013-05-08 21:48:25Z filippov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vasuki Palanigobu
 */

#include <corelib/ncbistd.hpp>
#include <objects/seqfeat/BioSource.hpp>
#include <wx/panel.h>
#include <wx/hyperlink.h>

#include "pcr_primers_panel.hpp"
#include "srcmod_panel.hpp"

/*!
 * Includes
 */

////@begin includes
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxFlexGridSizer;
////@end forward declarations

class wxCheckListBox;

BEGIN_NCBI_SCOPE

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CSRCMODLISTPANEL 10022
#define ID_SCROLLEDWINDOW1 10015
#define ID_TEXTCTRL7 10031
#define ID_PCR_CHECKBOX 10032
#define SYMBOL_CSRCMODLISTPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CSRCMODLISTPANEL_TITLE _("Source Modifiers")
#define SYMBOL_CSRCMODLISTPANEL_IDNAME ID_CSRCMODLISTPANEL
#define SYMBOL_CSRCMODLISTPANEL_SIZE wxSize(500, 600)
#define SYMBOL_CSRCMODLISTPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CSrcModListPanel class declaration
 */

class CSrcModListPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CSrcModListPanel )
    DECLARE_EVENT_TABLE()

public:

	typedef vector<CSrcModPanel::SModData>	TVecModData;

    /// Constructors
    CSrcModListPanel();
    CSrcModListPanel( wxWindow* parent, objects::CBioSource& source, wxWindowID id = SYMBOL_CSRCMODLISTPANEL_IDNAME, const wxPoint& pos = SYMBOL_CSRCMODLISTPANEL_POSITION, const wxSize& size = SYMBOL_CSRCMODLISTPANEL_SIZE, long style = SYMBOL_CSRCMODLISTPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CSRCMODLISTPANEL_IDNAME, const wxPoint& pos = SYMBOL_CSRCMODLISTPANEL_POSITION, const wxSize& size = SYMBOL_CSRCMODLISTPANEL_SIZE, long style = SYMBOL_CSRCMODLISTPANEL_STYLE );

    /// Destructor
    ~CSrcModListPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CSrcModListPanel event handler declarations

////@end CSrcModListPanel event handler declarations
    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
    void PopulateModifiers(objects::CBioSource& biosrc);

    void AddLastSrcMod (wxWindow* link);

////@begin CSrcModListPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CSrcModListPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CSrcModListPanel member variables
    wxScrolledWindow* m_ScrolledWindow;
    wxFlexGridSizer* m_Sizer;
    wxTextCtrl* m_SrcInfo;
////@end CSrcModListPanel member variables

    void OnDelete (wxHyperlinkEvent& event);
    void OnPCRCheckBox(wxCommandEvent& event);
    void OnAddPCRPrimer(wxCommandEvent& event);

private:
	  CRef<objects::CBioSource>	    m_Source;
	TVecModData					m_Data;
    int							m_NumRows;
    int							m_TotalHeight;
    int							m_TotalWidth;
    int							m_ScrollRate;
    //int							m_MaxRowsDisplayed;
    CPCRPrimersPanel* m_PCRPrimersPanel;
    wxBoxSizer* m_PCRPrimersBoxSizer1;
    wxBoxSizer* m_PCRPrimersBoxSizer2;
    wxBoxSizer* m_PCRPrimersBoxSizer3;
    wxCheckBox* m_PCRCheckbox;

    void	x_FillList();
    void  x_UpdateList();
    void	x_SetSrcSubMod(int subtype, const string& value);
    void	x_SetSrcOrgMod(int subtype, const string& value);
    void	x_SetBioSourceModifier(const string& name, const string& value);
    void	x_AddEmptyRow();
    void	x_AddRow(wxString name, wxString value);
    int   x_FindRow(wxWindow* wnd, wxSizerItemList& itemList);
    void  x_CollectTextMods ();
    void  x_AddTextSrcMods ();
    void x_AddPCRPrimersPanel();
    void x_ClearPCRPrimersPanel();
};

bool IsNonText (string label);

END_NCBI_SCOPE

#endif
    // GUI_WIDGETS_EDIT___SRCMOD_LIST_PANEL__HPP

