/*  $Id: srcmod_panel.cpp 27198 2013-01-09 18:28:17Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vasuki Palanigobu
 */


// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes
#include <ncbi_pch.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include "srcmod_panel.hpp"
#include "srcmod_list_panel.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images


/*
 * CSrcModPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSrcModPanel, wxPanel )


/*
 * CSrcModPanel event table definition
 */

BEGIN_EVENT_TABLE( CSrcModPanel, wxPanel )

////@begin CSrcModPanel event table entries
    EVT_COMBOBOX( ID_MODIFIER_NAME, CSrcModPanel::OnModifierNameSelected )

    EVT_TEXT( ID_TEXTCTRL4, CSrcModPanel::OnTextctrl4TextUpdated )

////@end CSrcModPanel event table entries
    EVT_CHECKBOX( ID_TEXTCTRL4, CSrcModPanel::OnNonTextValueUpdated )
END_EVENT_TABLE()


/*
 * CSrcModPanel constructors
 */

CSrcModPanel::CSrcModPanel()
{
    Init();
}

CSrcModPanel::CSrcModPanel( wxWindow* parent, SModData mod_data, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    m_ModData = mod_data;
    m_SrcModCheckbox = NULL;
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * SrcModPanel creator
 */

bool CSrcModPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSrcModPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSrcModPanel creation
    return true;
}


/*
 * CSrcModPanel destructor
 */

CSrcModPanel::~CSrcModPanel()
{
////@begin CSrcModPanel destruction
////@end CSrcModPanel destruction
}


/*
 * Member initialisation
 */

void CSrcModPanel::Init()
{
////@begin CSrcModPanel member initialisation
    m_Sizer = NULL;
    m_SrcModName = NULL;
    m_SrcModValue = NULL;
////@end CSrcModPanel member initialisation
}


/*
 * Control creation for SrcModPanel
 */

void CSrcModPanel::CreateControls()
{    
////@begin CSrcModPanel content construction
    CSrcModPanel* itemPanel1 = this;

    m_Sizer = new wxBoxSizer(wxHORIZONTAL);
    itemPanel1->SetSizer(m_Sizer);

    wxArrayString m_SrcModNameStrings;
    m_SrcModNameStrings.Add(_("Acronym"));
    m_SrcModNameStrings.Add(_("Anamorph"));
    m_SrcModNameStrings.Add(_("Authority"));
    m_SrcModNameStrings.Add(_("Bio-material"));
    m_SrcModNameStrings.Add(_("Biotype"));
    m_SrcModNameStrings.Add(_("Biovar"));
    m_SrcModNameStrings.Add(_("Breed"));
    m_SrcModNameStrings.Add(_("Cell-line"));
    m_SrcModNameStrings.Add(_("Cell-type"));
    m_SrcModNameStrings.Add(_("Chemovar"));
    m_SrcModNameStrings.Add(_("Chromosome"));
    m_SrcModNameStrings.Add(_("Clone"));
    m_SrcModNameStrings.Add(_("Clone-lib"));
    m_SrcModNameStrings.Add(_("Collected-by"));
    m_SrcModNameStrings.Add(_("Collection-date"));
    m_SrcModNameStrings.Add(_("Common"));
    m_SrcModNameStrings.Add(_("Country"));
    m_SrcModNameStrings.Add(_("Cultivar"));
    m_SrcModNameStrings.Add(_("Culture-collection"));
    m_SrcModNameStrings.Add(_("Dev-stage"));
    m_SrcModNameStrings.Add(_("Dosage"));
    m_SrcModNameStrings.Add(_("Ecotype"));
    m_SrcModNameStrings.Add(_("Endogenous-virus-name"));
    m_SrcModNameStrings.Add(_("Environmental-sample"));
    m_SrcModNameStrings.Add(_("Forma"));
    m_SrcModNameStrings.Add(_("Forma-specialis"));
    m_SrcModNameStrings.Add(_("Frequency"));
    m_SrcModNameStrings.Add(_("Genotype"));
    m_SrcModNameStrings.Add(_("Germline"));
    m_SrcModNameStrings.Add(_("Group"));
    m_SrcModNameStrings.Add(_("Haplogroup"));
    m_SrcModNameStrings.Add(_("Haplotype"));
    m_SrcModNameStrings.Add(_("Identified-by"));
    m_SrcModNameStrings.Add(_("Insertion-seq-name"));
    m_SrcModNameStrings.Add(_("Isolate"));
    m_SrcModNameStrings.Add(_("Isolation-source"));
    m_SrcModNameStrings.Add(_("Lab-host"));
    m_SrcModNameStrings.Add(_("Lat-Lon"));
    m_SrcModNameStrings.Add(_("Linkage-group"));
    m_SrcModNameStrings.Add(_("Map"));
    m_SrcModNameStrings.Add(_("Mating-type"));
    m_SrcModNameStrings.Add(_("Metagenome-source"));
    m_SrcModNameStrings.Add(_("Metagenomic"));
    m_SrcModNameStrings.Add(_("Nat-host"));
    m_SrcModNameStrings.Add(_("Pathovar"));
    m_SrcModNameStrings.Add(_("Phenotype"));
    m_SrcModNameStrings.Add(_("Plasmid-name"));
    m_SrcModNameStrings.Add(_("Plastid-name"));
    m_SrcModNameStrings.Add(_("Pop-variant"));
    m_SrcModNameStrings.Add(_("Rearranged"));
    m_SrcModNameStrings.Add(_("Segment"));
    m_SrcModNameStrings.Add(_("Serogroup"));
    m_SrcModNameStrings.Add(_("Serotype"));
    m_SrcModNameStrings.Add(_("Serovar"));
    m_SrcModNameStrings.Add(_("Sex"));
    m_SrcModNameStrings.Add(_("Specimen-voucher"));
    m_SrcModNameStrings.Add(_("Strain"));
    m_SrcModNameStrings.Add(_("Sub-species"));
    m_SrcModNameStrings.Add(_("Subclone"));
    m_SrcModNameStrings.Add(_("Subgroup"));
    m_SrcModNameStrings.Add(_("Substrain"));
    m_SrcModNameStrings.Add(_("Subtype"));
    m_SrcModNameStrings.Add(_("Synonymn"));
    m_SrcModNameStrings.Add(_("Teleomorph"));
    m_SrcModNameStrings.Add(_("Tissue-lib"));
    m_SrcModNameStrings.Add(_("Tissue-type"));
    m_SrcModNameStrings.Add(_("Transgenic"));
    m_SrcModNameStrings.Add(_("Transposon-name"));
    m_SrcModNameStrings.Add(_("Type"));
    m_SrcModNameStrings.Add(_("Variety"));
    m_SrcModNameStrings.Add(_("Whole-replicon"));
    m_SrcModName = new wxComboBox( itemPanel1, ID_MODIFIER_NAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_SrcModNameStrings, wxCB_READONLY );
    m_Sizer->Add(m_SrcModName, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_SrcModValue = new wxTextCtrl( itemPanel1, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_Sizer->Add(m_SrcModValue, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);

////@end CSrcModPanel content construction
}


void CSrcModPanel::x_SwitchToText()
{
    if (m_SrcModValue == NULL) {
        if (m_Sizer->GetItemCount() > 1) {
            size_t pos = 1;
            m_Sizer->GetItem(pos)->DeleteWindows();
            m_Sizer->Remove(pos);
            m_SrcModCheckbox = NULL;
        }
        m_SrcModValue = new wxTextCtrl( this, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
        m_Sizer->Add(m_SrcModValue, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
        Layout();
    }
}


void CSrcModPanel::x_SwitchToCheckbox()
{
    if (m_SrcModCheckbox == NULL) {
        if (m_Sizer->GetItemCount() > 1) {
            size_t pos = 1;
            m_Sizer->GetItem(pos)->DeleteWindows();
            m_Sizer->Remove(pos);
            m_SrcModValue = NULL;
        }
        m_SrcModCheckbox = new wxCheckBox ( this, ID_TEXTCTRL4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
        m_Sizer->Add(m_SrcModCheckbox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
        Layout();
    }
}


void CSrcModPanel::SetModifierData()
{
    m_SrcModName->SetValue(m_ModData.name);
    if (IsNonText (ToStdString(m_ModData.name))) {
        if (NStr::EqualNocase(ToStdString(m_ModData.value), "TRUE") || NStr::IsBlank(ToStdString(m_ModData.value))) {
            x_SwitchToCheckbox();
            if (NStr::EqualNocase(m_ModData.value, "TRUE")) {
                m_SrcModCheckbox->SetValue(true);
            } else {
                m_SrcModCheckbox->SetValue(false);
            }
        } else {
            x_SwitchToText();
            m_SrcModValue->SetValue(m_ModData.value);
        }
    } else {
        x_SwitchToText();
	      m_SrcModValue->SetValue(m_ModData.value);
    }
}


const CSrcModPanel::SModData& CSrcModPanel::GetModifierData()
{
    wxString mod = m_SrcModName->GetValue();
    if (m_SrcModValue != NULL) {
	      wxString val = m_SrcModValue->GetValue();
        m_ModData.name = mod;
        m_ModData.value = val;
    } else if (m_SrcModCheckbox != NULL) {
        m_ModData.name = mod;
        if (m_SrcModCheckbox->GetValue()) {
            m_ModData.value = wxT("TRUE");
        } else {
            m_ModData.value = wxEmptyString;
        }
    }
    return m_ModData;
}


/*
 * Should we show tooltips?
 */

bool CSrcModPanel::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CSrcModPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSrcModPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSrcModPanel bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CSrcModPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSrcModPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSrcModPanel icon retrieval
}




/*
 * wxEVT_COMMAND_TEXT_UPDATED event handler for ID_TEXTCTRL4
 */

void CSrcModPanel::OnTextctrl4TextUpdated( wxCommandEvent& event )
{
    wxTextCtrl* item = (wxTextCtrl*)event.GetEventObject();
    if (NStr::IsBlank(ToStdString(item->GetValue()))) {
        return;
    }
    wxWindow* parent = this->GetParent();

    CSrcModListPanel* listpanel = dynamic_cast<CSrcModListPanel*>(parent);

    while (parent && !listpanel) {
        parent = parent->GetParent();
        listpanel = dynamic_cast<CSrcModListPanel*>(parent);
    }

    if (!listpanel) {
        return;
    }
    listpanel->AddLastSrcMod((wxWindow*)this);
}


void CSrcModPanel::OnNonTextValueUpdated( wxCommandEvent& event )
{
    wxCheckBox* item = (wxCheckBox*)event.GetEventObject();
    if (!item->GetValue()) {
        return;
    }
    wxWindow* parent = this->GetParent();

    CSrcModListPanel* listpanel = dynamic_cast<CSrcModListPanel*>(parent);

    while (parent && !listpanel) {
        parent = parent->GetParent();
        listpanel = dynamic_cast<CSrcModListPanel*>(parent);
    }

    if (!listpanel) {
        return;
    }
    listpanel->AddLastSrcMod((wxWindow*)this);
}


/*
 * wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_MODIFIER_NAME
 */

void CSrcModPanel::OnModifierNameSelected( wxCommandEvent& event )
{
    GetModifierData();
    SetModifierData();
}


END_NCBI_SCOPE

