/* $Id: vectortrimpanel.cpp 27545 2013-03-04 11:17:11Z choi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Yoon Choi
 */


#include <ncbi_pch.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

////@begin includes
////@end includes

#include <wx/panel.h>
#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/textctrl.h>
#include <wx/msgdlg.h>
#include <wx/log.h> 
#include <wx/stattext.h>

#include <gui/widgets/edit/vectortrimpanel.hpp>
#include <gui/widgets/edit/vectorscreen.hpp>
#include <gui/objutils/cmd_change_bioseq_inst.hpp>
#include <gui/objutils/cmd_change_seq_feat.hpp>
#include <gui/objutils/cmd_change_align.hpp>
#include <gui/objutils/cmd_change_graph.hpp>
#include <gui/objutils/cmd_del_seq_feat.hpp>
#include <gui/objutils/cmd_del_seq_align.hpp>
#include <gui/objutils/cmd_del_bioseq.hpp>
#include <gui/objutils/cmd_del_bioseq_set.hpp>
#include <objects/seq/seqport_util.hpp>
#include <objects/seqfeat/Seq_feat.hpp>
#include <objects/seqfeat/Cdregion.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objects/seqres/Byte_graph.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/align_ci.hpp>
#include <objmgr/graph_ci.hpp>
#include <objmgr/util/seq_loc_util.hpp>
#include <objmgr/util/sequence.hpp>

#include <sstream>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

// Ids do not need to be globally unique as long as they are
// unique within the local context (e.g., unique per dialog)
enum
{
    MYID_BUTTON_RUN_VECSCREEN    = wxID_HIGHEST + 1,
    MYID_BUTTON_SELECT_ALL,
    MYID_BUTTON_SELECT_STRONG_MODERATE,
    MYID_BUTTON_UNSELECT_ALL,
    MYID_CHECKBOX_VIEWGRAPHIC,
    MYID_BUTTON_MAKE_REPORT,
    MYID_BUTTON_TRIM_SELECTED,

    MYID_LISTCTRL_MATCHES,
    MYID_LISTCTRL_LOCATIONS,
    MYID_WEBVIEW_GRAPHIC
};

/*!
 * CVectorTrimPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CVectorTrimPanel, wxPanel )


/*!
 * CVectorTrimPanel event table definition
 */

BEGIN_EVENT_TABLE( CVectorTrimPanel, wxPanel )

////@begin CVectorTrimPanel event table entries

////@end CVectorTrimPanel event table entries
    EVT_BUTTON(MYID_BUTTON_RUN_VECSCREEN, CVectorTrimPanel::OnRunVecscreen)
    EVT_BUTTON(MYID_BUTTON_SELECT_ALL, CVectorTrimPanel::OnSelectAll)
    EVT_BUTTON(MYID_BUTTON_SELECT_STRONG_MODERATE, CVectorTrimPanel::OnSelectStrongModerate)
    EVT_BUTTON(MYID_BUTTON_UNSELECT_ALL, CVectorTrimPanel::OnUnselectAll)
    EVT_CHECKBOX(MYID_CHECKBOX_VIEWGRAPHIC, CVectorTrimPanel::OnViewAlignmentGraphic)
    EVT_BUTTON(MYID_BUTTON_MAKE_REPORT, CVectorTrimPanel::OnMakeReport)
    EVT_BUTTON(MYID_BUTTON_TRIM_SELECTED, CVectorTrimPanel::OnTrimSelected)

END_EVENT_TABLE()


/*!
 * CVectorTrimPanel constructors
 */

CVectorTrimPanel::CVectorTrimPanel()
{
    Init();
}

CVectorTrimPanel::CVectorTrimPanel( wxWindow* parent, 
				    objects::CSeq_entry_Handle seh, 
				    wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
  : m_TopSeqEntry(seh)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CVectorTrimPanel creator
 */

bool CVectorTrimPanel::Create( wxWindow* parent, 
			       wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVectorTrimPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CVectorTrimPanel creation
    return true;
}


/*!
 * CVectorTrimPanel destructor
 */

CVectorTrimPanel::~CVectorTrimPanel()
{
////@begin CVectorTrimPanel destruction
////@end CVectorTrimPanel destruction
}


/*!
 * Member initialisation
 */

void CVectorTrimPanel::Init()
{
////@begin CVectorTrimPanel member initialisation
////@end CVectorTrimPanel member initialisation
}


/*!
 * Control creation for CVectorTrimPanel
 */

void CVectorTrimPanel::CreateControls()
{    
////@begin CVectorTrimPanel content construction
    CVectorTrimPanel* itemPanel1 = this;

    // Top level sizer
    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

////@end CVectorTrimPanel content construction

    // Row 1 - display results
    m_sizerVectorResults = new wxBoxSizer(wxHORIZONTAL);
    m_listMatches = new CMatchesListCtrl( this, MYID_LISTCTRL_MATCHES, wxDefaultPosition, wxDefaultSize, 
                                          wxLC_REPORT | wxLC_SINGLE_SEL );

    // Workaround for Windows only!
    // Highlight the entire row not just the first column in the row.
#ifdef __WXMSW__	
    DWORD dwStyle = ::SendMessage((HWND)m_listMatches->GetHandle(), LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0);
    dwStyle |= LVS_EX_FULLROWSELECT;
    ::SendMessage((HWND)m_listMatches->GetHandle(), LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dwStyle);
#endif


    // This sizer contains 2 elements in the same location.
    // A toggle will show/hide the appropriate element.
    m_sizerVectorResults->Add(m_listMatches, 1, wxEXPAND|wxALL, 5);

    m_viewAlignGraphic = new wxHtmlWindow( this, MYID_WEBVIEW_GRAPHIC );
    m_sizerVectorResults->Add(m_viewAlignGraphic, 1, wxEXPAND|wxALL, 5);
    m_listMatches->SetAlignGraphicView( m_viewAlignGraphic );

    m_listLocations = new CLocationsListCtrl( this, MYID_LISTCTRL_LOCATIONS, wxDefaultPosition, wxDefaultSize, 
					      wxLC_REPORT | wxLC_SINGLE_SEL );
    m_sizerVectorResults->Add(m_listLocations, 1, wxEXPAND|wxALL, 5);
    m_listMatches->SetRangeView( m_listLocations );

    // By default show m_listLocations, hide m_viewAlignGraphic
    m_sizerVectorResults->Hide(m_viewAlignGraphic);
    m_sizerVectorResults->Layout();


    itemBoxSizer2->Add(m_sizerVectorResults, 1, wxEXPAND|wxALL, 5);

    // Row 2 - run blast vecscreen against UniVec db
    wxBoxSizer* sizerSearchVector = new wxBoxSizer(wxHORIZONTAL);
    m_buttonRunVecscreen = new wxButton( this, MYID_BUTTON_RUN_VECSCREEN, _("Search Univec") );
    sizerSearchVector->Add(m_buttonRunVecscreen, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_gaugeProgressBar = new wxGauge( this, wxID_ANY, 100, wxDefaultPosition, wxDefaultSize, wxGA_SMOOTH );
    sizerSearchVector->Add(m_gaugeProgressBar, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
    itemBoxSizer2->Add(sizerSearchVector, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    // Row 3 - select results
    wxBoxSizer* sizerSelectResults = new wxBoxSizer(wxHORIZONTAL);
    m_buttonSelectAll = new wxButton( this, MYID_BUTTON_SELECT_ALL, _("Select All") );
    sizerSelectResults->Add(m_buttonSelectAll, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_buttonSelectStrongModerate = new wxButton( this, MYID_BUTTON_SELECT_STRONG_MODERATE, _("Select Only Strong and Moderate") );
    sizerSelectResults->Add(m_buttonSelectStrongModerate, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_buttonUnselectAll = new wxButton( this, MYID_BUTTON_UNSELECT_ALL, _("Unselect All") );
    sizerSelectResults->Add(m_buttonUnselectAll, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_checkboxViewGraphic = new wxCheckBox( this, MYID_CHECKBOX_VIEWGRAPHIC, _("View Alignment Graphic") );
    sizerSelectResults->Add(m_checkboxViewGraphic, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    itemBoxSizer2->Add(sizerSelectResults, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    // Row 4 - Trim selected sequences
    wxBoxSizer* sizerTrimSelected = new wxBoxSizer(wxHORIZONTAL);
    m_buttonMakeReport = new wxButton( this, MYID_BUTTON_MAKE_REPORT, _("Make Report") );
    sizerTrimSelected->Add(m_buttonMakeReport, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_buttonTrimSelected = new wxButton( this, MYID_BUTTON_TRIM_SELECTED, _("Trim Selected Sequences") );
    sizerTrimSelected->Add(m_buttonTrimSelected, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    itemBoxSizer2->Add(sizerTrimSelected, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
}


bool CVectorTrimPanel::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow())
        return false;


    return true;
}


bool CVectorTrimPanel::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;


    return true;
}


/*!
 * Should we show tooltips?
 */

bool CVectorTrimPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVectorTrimPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVectorTrimPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVectorTrimPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVectorTrimPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVectorTrimPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVectorTrimPanel icon retrieval
}


void CVectorTrimPanel::OnRunVecscreen(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_RUN_VECSCREEN:
        {
	    m_vectorscreen.Run(m_TopSeqEntry, *m_gaugeProgressBar);
	    DisplaySummaryBySeqid(m_vectorscreen.GetVecscreenSummaryBySeqid());
	}
    }
}


void CVectorTrimPanel::OnSelectAll(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_SELECT_ALL:
        {
	    m_listMatches->SelectAll();
	}
    }
}


void CVectorTrimPanel::OnSelectStrongModerate(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_SELECT_STRONG_MODERATE:
        {
	    m_listMatches->SelectStrongModerate();
	}
    }
}


void CVectorTrimPanel::OnUnselectAll(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_UNSELECT_ALL:
        {
	    m_listMatches->UnselectAll();
	}
    }
}


void CVectorTrimPanel::OnViewAlignmentGraphic( wxCommandEvent& event )
{
    switch (event.GetId())
    {
        case MYID_CHECKBOX_VIEWGRAPHIC:
        {
	    if ( m_checkboxViewGraphic->IsChecked() )
	    {
                m_sizerVectorResults->Show(m_viewAlignGraphic);
                m_sizerVectorResults->Hide(m_listLocations);
	    }
  	    else
	    {
                m_sizerVectorResults->Hide(m_viewAlignGraphic);
                m_sizerVectorResults->Show(m_listLocations);
	    }
	    m_sizerVectorResults->Layout();
	}
    }
}


void CVectorTrimPanel::OnMakeReport(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_MAKE_REPORT:
        {
	}
    }
}


void CVectorTrimPanel::OnTrimSelected(wxCommandEvent &event)
{
    switch (event.GetId())
    {
        case MYID_BUTTON_TRIM_SELECTED:
        {
	    x_TrimSelected();
	}
    }
}


class CRangeCmp : public binary_function<SRangeMatch, SRangeMatch, bool>
{
public:
    enum ESortOrder
    {
        eAscending,
        eDescending
    };

    explicit CRangeCmp(ESortOrder sortorder = eAscending)
      : m_sortorder(sortorder)
    {
    };

    bool operator()(const SRangeMatch& a1, const SRangeMatch& a2) 
    {
        if (m_sortorder == eAscending)
	{
	    // Tiebreaker
	    if (a1.m_range.GetTo() == a2.m_range.GetTo()) {
	        return a1.m_range.GetFrom() < a2.m_range.GetFrom();
	    }
	    return a1.m_range.GetTo() < a2.m_range.GetTo();
	}
        else
	{
	    // Tiebreaker
	    if (a1.m_range.GetTo() == a2.m_range.GetTo()) {
	        return a1.m_range.GetFrom() > a2.m_range.GetFrom();
	    }
	    return a1.m_range.GetTo() > a2.m_range.GetTo();
	}
    };

private:
    ESortOrder m_sortorder;
};


void CVectorTrimPanel::x_SetSeqData(const CBioseq_Handle& bsh,
				    CRef<objects::CSeq_inst> inst,
				    const TCuts& sorted_cuts)
{
    // Copy residues to buffer
    CSeqVector vec(bsh, CBioseq_Handle::eCoding_Iupac);
    string seq_string;
    vec.GetSeqData(0, vec.size(), seq_string);

    // Delete residues per cut
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos start = cut.m_range.GetFrom();
	TSeqPos length = cut.m_range.GetTo() - start + 1;
	seq_string.erase(start, length);
    }

    // Update sequence length and sequence data
    inst->SetLength(seq_string.size());
    inst->SetSeq_data().SetIupacna(*new CIUPACna(seq_string));
    CSeqportUtil::Pack(&inst->SetSeq_data());
}


void CVectorTrimPanel::x_SeqIntervalDelete(CRef<CSeq_interval> interval, 
					   TSeqPos cut_from, TSeqPos cut_to,
					   const CSeq_id* seqid,
					   bool& bCompleteCut,
					   bool& bTrimmed)
{
    // Does feature's seqid match selected sequence?
    if ( !(interval->CanGetId() && seqid->Match(interval->GetId())) )
    {
        return;
    }

    // These are required fields
    if ( !(interval->CanGetFrom() && interval->CanGetTo()) )
    {
        return;
    }

    // Feature location
    TSeqPos feat_from = interval->GetFrom();
    TSeqPos feat_to = interval->GetTo();

    // Size of the cut
    TSeqPos cut_size = cut_to - cut_from + 1;

    // Case 1: feature is located completely before the cut
    if (feat_to < cut_from)
    {
        // Nothing needs to be done - cut does not affect feature
        return;
    }

    // Case 2: feature is completely within the cut
    if (feat_from >= cut_from && feat_to <= cut_to)
    {
        // Feature should be deleted
        bCompleteCut = true;
        return;
    }

    // Case 3: feature is completely past the cut
    if (feat_from > cut_to)
    {
        // Shift the feature by the cut_size
        feat_from -= cut_size;
        feat_to -= cut_size;
	interval->SetFrom(feat_from);
	interval->SetTo(feat_to);
	bTrimmed = true;
        return;
    }

    /***************************************************************************
     * Cases below are partial overlapping cases
    ***************************************************************************/
    // Case 4: Cut is completely inside the feature 
    //         OR
    //         Cut is to the "left" side of the feature (i.e., feat_from is 
    //         inside the cut)
    //         OR
    //         Cut is to the "right" side of the feature (i.e., feat_to is 
    //         inside the cut)
    if (feat_to > cut_to) {
        // Left side cut or cut is completely inside feature
        feat_to -= cut_size;
    }
    else {
        // Right side cut
        feat_to = cut_from - 1;
    }

    // Take care of the feat_from from the left side cut case
    if (feat_from >= cut_from) {
        feat_from = cut_to + 1;
	feat_from -= cut_size;
    }

    interval->SetFrom(feat_from);
    interval->SetTo(feat_to);
    bTrimmed = true;
}


void CVectorTrimPanel::x_SeqLocDelete(CRef<CSeq_loc> loc, 
				      TSeqPos from, TSeqPos to,
				      const CSeq_id* seqid,
				      bool& bCompleteCut,
				      bool& bTrimmed)
{
    // Given a seqloc and a range, cut the seqloc

    switch(loc->Which())
    {
        // Single interval
        case CSeq_loc::e_Int:
	{
	    CRef<CSeq_interval> interval(new CSeq_interval);
	    interval->Assign(loc->GetInt());
  	    x_SeqIntervalDelete(interval, from, to, seqid, 
				bCompleteCut, bTrimmed);
	    loc->SetInt(*interval);
	}
	break;

	// Multiple intervals
        case CSeq_loc::e_Packed_int:
	{
	    CRef<CSeq_loc::TPacked_int> intervals(new CSeq_loc::TPacked_int);
	    intervals->Assign(loc->GetPacked_int());
	    if (intervals->CanGet()) {
	        // Process each interval in the list
	        CPacked_seqint::Tdata::iterator it;
	        for (it = intervals->Set().begin(); 
                     it != intervals->Set().end(); ) 
                {
		    // Initial value: assume that all intervals 
		    // will be deleted resulting in bCompleteCut = true.
		    // Later on if any interval is not deleted, then set
		    // bCompleteCut = false
		    if (it == intervals->Set().begin()) {
		        bCompleteCut = true;
		    }

  		    bool bDeleted = false;
		    x_SeqIntervalDelete(*it, from, to, seqid, 
					bDeleted, bTrimmed);

		    // Should interval be deleted from list?
		    if (bDeleted) {
	  	        it = intervals->Set().erase(it);
		    }
		    else {
		        ++it;
			bCompleteCut = false;
		    }
	        }

		// Update the original list
	        loc->SetPacked_int(*intervals);
	    }
	}
	break;

	// Multiple seqlocs
        case CSeq_loc::e_Mix:
	{
	    CRef<CSeq_loc_mix> mix(new CSeq_loc_mix);
	    mix->Assign(loc->GetMix());
	    if (mix->CanGet()) {
	        // Process each seqloc in the list
	        CSeq_loc_mix::Tdata::iterator it;
	        for (it = mix->Set().begin(); 
                     it != mix->Set().end(); ) 
                {
		    // Initial value: assume that all seqlocs
		    // will be deleted resulting in bCompleteCut = true.
		    // Later on if any seqloc is not deleted, then set
		    // bCompleteCut = false
		    if (it == mix->Set().begin()) {
		        bCompleteCut = true;
		    }

  		    bool bDeleted = false;
		    x_SeqLocDelete(*it, from, to, seqid, bDeleted, bTrimmed);

		    // Should seqloc be deleted from list?
		    if (bDeleted) {
	  	        it = mix->Set().erase(it);
		    }
		    else {
		        ++it;
			bCompleteCut = false;
		    }
		}

	        // Update the original list
	        loc->SetMix(*mix);
	    }
	}
	break;

        // Other choices not supported yet 
        default:
	{	    
	}
	break;
    }
}


void CVectorTrimPanel::x_TrimFeatureLocations(CRef<CSeq_feat> feat,
					      const TCuts& sorted_cuts,
					      const CSeq_id* seqid,
					      bool& bFeatureDeleted,
					      bool& bFeatureTrimmed,
					      bool& bProdDeleted,
					      bool& bProdTrimmed)
{
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	// Update Seqloc "feature made from" 
	if (feat->CanGetLocation())
	{
            CRef<CSeq_feat::TLocation> new_location(new CSeq_feat::TLocation);
	    new_location->Assign(feat->GetLocation());
	    x_SeqLocDelete(new_location, from, to, seqid, 
			   bFeatureDeleted, bFeatureTrimmed);
	    feat->SetLocation(*new_location);

	    // No need to cut anymore nor update.  Feature will be completely
	    // deleted.  
	    if (bFeatureDeleted) {
		return;
	    }
	}

	// Update Seqloc "product of process"
	if (feat->CanGetProduct())
	{
            CRef<CSeq_feat::TProduct> new_product(new CSeq_feat::TProduct);
	    new_product->Assign(feat->GetProduct());
	    x_SeqLocDelete(new_product, from, to, seqid, bProdDeleted, bProdTrimmed);
	    feat->SetProduct(*new_product);
	}
    }
}


void CVectorTrimPanel::x_AdjustCdregionFrame(CScope& scope, 
					     CRef<CSeq_feat> feat,
					     const TCuts& sorted_cuts,
					     const CSeq_id* seqid,
					     bool bFeatureTrimmed,
					     bool bProdTrimmed)
{
    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

    // Get partialness and strand of location before cutting
    bool bIsPartialStart = false;
    ENa_strand eStrand = eNa_strand_unknown;
    if (feat->CanGetLocation())
    {
        bIsPartialStart = feat->GetLocation().IsPartialStart(eExtreme_Biological);
        eStrand = feat->GetLocation().GetStrand(); 
    }

    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	// Adjust Seq-feat.data.cdregion frame
	if (feat->CanGetData() && 
	    feat->GetData().GetSubtype() == CSeqFeatData::eSubtype_cdregion &&
	    feat->GetData().IsCdregion() &&
            (bFeatureTrimmed || bProdTrimmed))
	{
	    // Make a copy
	    CRef<CCdregion> new_cdregion(new CCdregion);
	    new_cdregion->Assign(feat->GetData().GetCdregion());

	    // Edit the copy
	    if ( (eStrand == eNa_strand_minus &&
		  to == bsh.GetInst().GetLength() - 1 &&
		  bIsPartialStart)
                 ||
		 (eStrand != eNa_strand_minus && 
                  from == 0 && 
                  bIsPartialStart) )
	    {
	        TSeqPos old_frame = new_cdregion->GetFrame();
                if (old_frame == 0)
                {
		    old_frame = 1;
                }

		TSignedSeqPos new_frame = old_frame - ((to - from + 1) % 3);
                if (new_frame < 1)
                {
                    new_frame += 3;
                }
                new_cdregion->SetFrame((CCdregion::EFrame)new_frame);
	    }

	    // Update the original
	    feat->SetData().SetCdregion(*new_cdregion);
	}
    }
}


void CVectorTrimPanel::x_AdjustInternalCutLocations(TCuts& cuts, 
						    TSeqPos seq_length)
{
    // By default, if there is an internal cut, trim all the
    // way to the 3' end

    for (int ii = 0; ii < cuts.size(); ++ii)
    {
        SRangeMatch& cut = cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	// Is it an internal cut?
	if (from != 0 && to != seq_length-1) {
	    // By default extend the cut to 3' end
  	    cut.m_range.SetTo(seq_length-1);
	}
    }
}


void CVectorTrimPanel::x_MergeCuts(TCuts& sorted_cuts)
{
    // Assume cuts are sorted in Ascending order.

    // Merge abutting and overlapping cuts
    TCuts::iterator it;
    for (it = sorted_cuts.begin(); it != sorted_cuts.end(); )
    {
        SRangeMatch& cut = *it;
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	// Does next cut exist?
	if ( it+1 != sorted_cuts.end() ) {
	    SRangeMatch& next_cut = *(it+1);
	    TSeqPos next_from = next_cut.m_range.GetFrom();
	    TSeqPos next_to = next_cut.m_range.GetTo();

	    if ( next_from <= (to + 1) ) {
	        // Current and next cuts abut or overlap
	        // So adjust current cut and delete next cut
	        cut.m_range.SetTo(next_to);
		sorted_cuts.erase(it+1);

		// Post condition after erase:
		// Since "it" is before the erase, "it" stays valid
		// and still refers to current cut
	    }
	    else {
	        ++it;
	    }
	}
	else {
	    // I'm done
	    break;
	}
    }
}


void CVectorTrimPanel::x_RetranslateCDS(CScope& scope, 
					CRef<CCmdComposite> command, 
					CRef<CSeq_feat> cds)
{
    // Assumption:  cds has been verified to be Cdregion with Product

    // Use Cdregion.Product to get handle to protein bioseq 
    CBioseq_Handle prot_bsh = scope.GetBioseqHandle(cds->GetProduct());

    // Should be a protein!
    if (!prot_bsh.IsProtein())
    {
        return;
    }

    // Make a copy of existing CSeq_inst
    CRef<objects::CSeq_inst> new_inst(new objects::CSeq_inst());
    new_inst->Assign(prot_bsh.GetInst());

    // Make edits to the CSeq_inst copy
    CRef<CBioseq> new_protein_bioseq;
    if (new_inst->IsSetSeq_data())
    {
        // Generate new protein sequence data and length
        new_protein_bioseq = CSeqTranslator::TranslateToProtein(*cds, scope);
	if (new_protein_bioseq->GetInst().GetSeq_data().IsIupacaa()) 
        {
            new_inst->SetSeq_data().SetIupacaa().Set( 
                new_protein_bioseq->GetInst().GetSeq_data().GetIupacaa().Get());
	    new_inst->SetLength( new_protein_bioseq->GetInst().GetLength() );
	}
	else if (new_protein_bioseq->GetInst().GetSeq_data().IsNcbieaa()) 
        {
            new_inst->SetSeq_data().SetNcbieaa().Set( 
                new_protein_bioseq->GetInst().GetSeq_data().GetNcbieaa().Get());
	    new_inst->SetLength( new_protein_bioseq->GetInst().GetLength() );
	}
    }

    if ( !new_protein_bioseq ) {
        return;
    }

    // Update protein sequence data and length
    CRef<CCmdChangeBioseqInst> chgInst (new CCmdChangeBioseqInst(prot_bsh, 
								 *new_inst));
    command->AddCommand(*chgInst);

    // If protein feature exists, update it
    SAnnotSelector sel(CSeqFeatData::e_Prot);
    CFeat_CI prot_feat_ci(prot_bsh, sel);
    for ( ; prot_feat_ci; ++prot_feat_ci ) {
        CRef<CSeq_feat> new_feat(new CSeq_feat());
        new_feat->Assign(prot_feat_ci->GetOriginalFeature());

	if ( new_feat->CanGetLocation() &&
             new_feat->GetLocation().IsInt() &&
             new_feat->GetLocation().GetInt().CanGetTo() )
	{
            new_feat->SetLocation().SetInt().SetTo(
                new_protein_bioseq->GetLength() - 1);

            CIRef<IEditCommand> chgFeat(new CCmdChangeSeq_feat(*prot_feat_ci,
							       *new_feat));
	    command->AddCommand(*chgFeat);
	}
    }
}


bool CVectorTrimPanel::x_FindSegment(const CDense_seg& denseg,
				     CDense_seg::TDim row,
				     TSeqPos pos, 
				     CDense_seg::TNumseg& seg,
                                     TSeqPos& seg_start) const
{
    for (seg = 0; seg < denseg.GetNumseg(); ++seg) {
        TSignedSeqPos start = denseg.GetStarts()[seg * denseg.GetDim() + row];
        TSignedSeqPos len   = denseg.GetLens()[seg];
        if (start != -1) {
            if (pos >= start  &&  pos < start + len) {
	        seg_start = start;
                return true;
            }
        }
    }
    return false;
}


void CVectorTrimPanel::x_CutDensegSegment(CRef<CSeq_align> align, 
					  CDense_seg::TDim row,
					  TSeqPos pos)
{
    // Find the segment where pos occurs for the sequence (identified by 
    // row).
    // If pos is not the start of the segment, cut the segment in two, with 
    // one of the segments using pos as the new start.


    // Find the segment where pos lies
    const CDense_seg& denseg = align->GetSegs().GetDenseg();
    CDense_seg::TNumseg foundseg; 
    TSeqPos seg_start;
    if ( !x_FindSegment(denseg, row, pos, foundseg, seg_start) ) {
        return;
    }

    // Found our segment seg
    // If pos falls on segment boundary, do nothing
    if (pos == seg_start) {
        return;
    }


    // Cut the segment :
    // 1) Allocate a new denseg with numseg size = original size + 1
    // 2) Copy elements before the cut
    // 3) Split segment at pos
    // 4) Copy elements after the cut
    // 5) Replace old denseg with new denseg

    // Allocate a new denseg with numseg size = original size + 1
    CRef<CDense_seg> new_denseg(new CDense_seg);    
    new_denseg->SetDim( denseg.GetDim() );
    new_denseg->SetNumseg( denseg.GetNumseg() + 1 );
    ITERATE( CDense_seg::TIds, idI, denseg.GetIds() ) {
        CSeq_id *si = new CSeq_id;
        si->Assign(**idI);
        new_denseg->SetIds().push_back( CRef<CSeq_id>(si) );
    }

    // Copy elements (starts, lens, strands) before the cut (up to and including
    // foundseg-1 in original denseg)
    for (CDense_seg::TNumseg curseg = 0; curseg < foundseg; ++curseg) {
        // Copy starts
        for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); ++curdim) {
            TSeqPos index = curseg * denseg.GetDim() + curdim;
            new_denseg->SetStarts().push_back( denseg.GetStarts()[index] );
        }

	// Copy lens
        new_denseg->SetLens().push_back( denseg.GetLens()[curseg] );

	// Copy strands
        if ( denseg.IsSetStrands() ) {
            for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); 
                 ++curdim) 
            {
                TSeqPos index = curseg * denseg.GetDim() + curdim;
                new_denseg->SetStrands().push_back(denseg.GetStrands()[index]);
	    }
        }
    }

    // Split segment at pos
    // First find the lengths of the split segments, first_len and second_len
    TSeqPos first_len, second_len;
    TSeqPos index = foundseg * denseg.GetDim() + row;
    if ( !denseg.IsSetStrands() || denseg.GetStrands()[index] != eNa_strand_minus )
    {
        first_len  = pos - seg_start;
	second_len = denseg.GetLens()[foundseg] - first_len;
    } 
    else {
        second_len = pos - seg_start;
        first_len  = denseg.GetLens()[foundseg] - second_len;
    }   

    // Set starts, strands, and lens for the split segments (foundseg and foundseg+1)
    // Populate foundseg in new denseg
    for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); ++curdim) {
        TSeqPos index = foundseg * denseg.GetDim() + curdim;
        if (denseg.GetStarts()[index] == -1) {
	    new_denseg->SetStarts().push_back(-1);
	}
	else if (!denseg.IsSetStrands() || denseg.GetStrands()[index] != eNa_strand_minus) {
	    new_denseg->SetStarts().push_back(denseg.GetStarts()[index]);
	}
	else {
	    new_denseg->SetStarts().push_back(denseg.GetStarts()[index] + second_len);
	}

	if (denseg.IsSetStrands()) {
	    new_denseg->SetStrands().push_back(denseg.GetStrands()[index]);
	}
    }    
    new_denseg->SetLens().push_back(first_len);
    // Populate foundseg+1 in new denseg
    for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); ++curdim) {
        TSeqPos index = foundseg * denseg.GetDim() + curdim;
        if (denseg.GetStarts()[index] == -1) {
	    new_denseg->SetStarts().push_back(-1);
	}
	else if (!denseg.IsSetStrands() || denseg.GetStrands()[index] != eNa_strand_minus) {
	    new_denseg->SetStarts().push_back(denseg.GetStarts()[index] + first_len);
	}
	else {
	    new_denseg->SetStarts().push_back(denseg.GetStarts()[index]);
	}

	if (denseg.IsSetStrands()) {
	    new_denseg->SetStrands().push_back(denseg.GetStrands()[index]);
	}
    }    
    new_denseg->SetLens().push_back(second_len);

    // Copy elements (starts, lens, strands) after the cut (starting from foundseg+1 in 
    // original denseg)
    for (CDense_seg::TNumseg curseg = foundseg+1; curseg < denseg.GetNumseg(); ++curseg) {
        // Copy starts
        for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); ++curdim) {
            TSeqPos index = curseg * denseg.GetDim() + curdim;
            new_denseg->SetStarts().push_back( denseg.GetStarts()[index] );
        }

	// Copy lens
        new_denseg->SetLens().push_back( denseg.GetLens()[curseg] );

	// Copy strands
        if ( denseg.IsSetStrands() ) {
            for (CDense_seg::TDim curdim = 0; curdim < denseg.GetDim(); 
                 ++curdim) 
            {
                TSeqPos index = curseg * denseg.GetDim() + curdim;
                new_denseg->SetStrands().push_back(denseg.GetStrands()[index]);
	    }
        }
    }

    // Update 
    align->SetSegs().SetDenseg(*new_denseg);
}


void CVectorTrimPanel::x_AdjustDensegAlignment(CRef<CSeq_align> align, 
					       CDense_seg::TDim row, 
					       const TCuts& sorted_cuts)
{
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos cut_from = cut.m_range.GetFrom();
        TSeqPos cut_to = cut.m_range.GetTo();

	TSeqPos cut_len = cut_to - cut_from + 1;
        if (cut_to < cut_from) {
            cut_len = cut_from - cut_to + 1;
	    cut_from = cut_to;
	} 

	// Note: row is 0-based

	// May need to cut the segment at both start and stop positions
	// if they do not fall on segment boundaries
	x_CutDensegSegment(align, row, cut_from);
	x_CutDensegSegment(align, row, cut_from + cut_len);

	// Update segment start values for the trimmed sequence row
	const CDense_seg& denseg = align->GetSegs().GetDenseg();
        for (CDense_seg::TNumseg curseg = 0; curseg < denseg.GetNumseg(); ++curseg) {
	    TSeqPos index = curseg * denseg.GetDim() + row;
	    TSeqPos seg_start = denseg.GetStarts()[index];
	    if (seg_start < 0) {
	        // This indicates a gap, no change needed
	    }
	    else if (seg_start < cut_from) {
	        // This is before the cut, no change needed
	    }
	    else if (seg_start >= cut_from && 
                     seg_start + denseg.GetLens()[curseg] <= cut_from + cut_len) {
	        // This is in the gap, indicate it with a -1
	        align->SetSegs().SetDenseg().SetStarts()[index] = -1;
	    }
	    else {
	        // This is after the cut - subtract the cut_len
	        align->SetSegs().SetDenseg().SetStarts()[index] -= cut_len;
	    }
	}
    }
}


void CVectorTrimPanel::x_TrimSelected(CRef<CCmdComposite> command,
				      const CSeq_id* seqid,
				      const SVecscreenResult& vecres)
{
    CScope& scope = m_TopSeqEntry.GetScope();
    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

    // Should be a nuc!
    if (!bsh.IsNucleotide())
    {
        return;
    }


    TCuts sorted_cuts(vecres.m_arrRangeMatch);
    x_GetSortedCuts(sorted_cuts, seqid);


    /***************************************************************************
     * Trim nuc sequence
    ***************************************************************************/
    // Make a copy of existing CSeq_inst
    CRef<objects::CSeq_inst> new_inst(new objects::CSeq_inst());
    new_inst->Assign(bsh.GetInst());

    // Make edits to the CSeq_inst copy
    if (new_inst->IsSetSeq_data())
    {
        x_SetSeqData(bsh, new_inst, sorted_cuts);
    }

    // Swap in the edited CSeq_inst copy against the original 
    CRef<CCmdChangeBioseqInst> chgInst (new CCmdChangeBioseqInst(bsh, *new_inst));
    command->AddCommand(*chgInst);


    /***************************************************************************
     * Trim all annotation that reference the nuc
    ***************************************************************************/
    // Trim features
    SAnnotSelector feat_sel(CSeq_annot::C_Data::e_Ftable);
    CRef<CSeq_id> copy_id(new CSeq_id());
    copy_id->Assign(*seqid);
    CRef<CSeq_loc> search_loc( new CSeq_loc );
    search_loc->SetWhole(*copy_id);
    CFeat_CI feat_ci(scope, *search_loc, feat_sel);
    for (; feat_ci; ++feat_ci)
    {
        // Do seqids match?
        if ( !seqid->Match( *(feat_ci->GetLocationId().GetSeqId()) ) ) {
	    continue;
        }

  	// Make a copy of the feature
        CRef<CSeq_feat> new_feat(new CSeq_feat());
        new_feat->Assign(feat_ci->GetOriginalFeature());

	// Detect complete deletions of feature
	bool bFeatureDeleted = false;

	// Detect case where feature was not deleted but merely trimmed
	bool bFeatureTrimmed = false;

	bool bProdDeleted = false;
	bool bProdTrimmed = false;

	// Make edits to the copy
	x_TrimFeatureLocations(new_feat, sorted_cuts, seqid, 
			       bFeatureDeleted, bFeatureTrimmed,
			       bProdDeleted, bProdTrimmed);

	if (bFeatureDeleted) {
	    // Delete original feature
	    CIRef<IEditCommand> delFeat(new CCmdDelSeq_feat(*feat_ci));
	    command->AddCommand(*delFeat);

	    // If this feat is a Cdregion, then delete the protein sequence
	    if ( feat_ci->IsSetData() && 
		 feat_ci->GetData().Which() == CSeqFeatData::e_Cdregion &&
		 feat_ci->IsSetProduct() )
	    {
	        // Use Cdregion feat.product seqloc to get protein bioseq handle
	        CBioseq_Handle prot_h = 
                    scope.GetBioseqHandle(feat_ci->GetProduct());

                // Should be a protein!
                if ( prot_h.IsProtein() ) {
		    // NOTE: 
		    // CCmdDelBioseqInst implementation already takes care of
		    // renormalizing the nuc-prot set if necessary.
	            CRef<CCmdDelBioseqInst> 
                        delProt(new CCmdDelBioseqInst(prot_h));
		    command->AddCommand(*delProt);
		}
	    }
	}
	else {
	    if (bFeatureTrimmed) {
		// If this feat is a Cdregion, then RETRANSLATE the protein
		// sequence AND adjust any protein feature
		if ( new_feat->IsSetData() && 
		     new_feat->GetData().Which() == CSeqFeatData::e_Cdregion &&
		     new_feat->IsSetProduct() )
		{
                    x_AdjustCdregionFrame(scope,
                                          new_feat,
					  sorted_cuts,
					  seqid,
					  bFeatureTrimmed,
					  bProdTrimmed);

		    // In order to retranslate correctly, we need to create a 
		    // new scope with the trimmed sequence data.

		    // Keep track of original seqinst
		    CRef<objects::CSeq_inst> orig_inst(new objects::CSeq_inst());
		    orig_inst->Assign(bsh.GetInst());

		    // Update the seqinst to the trimmed version, set the scope
		    // and retranslate
		    CBioseq_EditHandle bseh = bsh.GetEditHandle();
		    bseh.SetInst(*new_inst);
		    CScope& new_scope = bseh.GetScope();
		    x_RetranslateCDS(new_scope, command, new_feat);

		    // Restore the original seqinst
		    bseh.SetInst(*orig_inst);
		}

		// Swap edited copy with original feature
		CIRef<IEditCommand> chgFeat(new CCmdChangeSeq_feat(*feat_ci, 
								   *new_feat));
		command->AddCommand(*chgFeat);
	    }
	}
    }
}


void CVectorTrimPanel::x_GetSortedCuts(TCuts& sorted_cuts,
				       const CSeq_id* seqid)
{
    CScope& scope = m_TopSeqEntry.GetScope();
    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

    /***************************************************************************
     * Merge abutting and overlapping cuts
     * Adjust internal cuts to 3' end
    ***************************************************************************/
    CRangeCmp asc(CRangeCmp::eAscending);
    sort(sorted_cuts.begin(), sorted_cuts.end(), asc);

    // Merge abutting and overlapping cuts
    x_MergeCuts(sorted_cuts);

    // Adjust internal cuts to 3' end
    x_AdjustInternalCutLocations(sorted_cuts, bsh.GetBioseqLength());


    /***************************************************************************
     * Sort the cuts in descending order
    ***************************************************************************/
    // Sort the ranges from greatest to least so that sequence
    // data and metadata will be deleted from greatest loc to smallest loc.
    // That way we don't have to adjust coordinate values after 
    // each delete.
    CRangeCmp descend(CRangeCmp::eDescending);
    sort(sorted_cuts.begin(), sorted_cuts.end(), descend);
}


void CVectorTrimPanel::x_TrimAlignments(CRef<CCmdComposite> command,
					const TVecscreenSummaryBySeqid& trimOnly)
{
    // Adjust alignments
    SAnnotSelector align_sel(CSeq_annot::C_Data::e_Align);
    CAlign_CI align_ci(m_TopSeqEntry, align_sel);
    for (; align_ci; ++align_ci)
    {
        // So far, handle DENSEG type only
        const CSeq_align& align = *align_ci;
	if ( align.CanGetSegs() && 
             align.GetSegs().Which() == CSeq_align::C_Segs::e_Denseg )
	{
  	    // Make sure mandatory fields are present in the denseg
	    const CDense_seg& denseg = align.GetSegs().GetDenseg();
	    if (! (denseg.CanGetDim() && denseg.CanGetNumseg() && 
	           denseg.CanGetIds() && denseg.CanGetStarts() &&
		   denseg.CanGetLens()) )
	    {
	        continue;
	    }

	    // Make a copy of the alignment
	    CRef<CSeq_align> new_align(new CSeq_align());
	    new_align->Assign(align_ci.GetOriginalSeq_align());

   	    // Make edits to the copy
	    TVecscreenSummaryBySeqid::const_iterator cit;
            for (cit = trimOnly.begin(); cit != trimOnly.end(); ++cit)
            {
	        const CSeq_id* seqid = cit->first;
	        const SVecscreenResult& result = cit->second;

		// On which "row" does the seqid lie?
		const CDense_seg::TIds& ids = denseg.GetIds();
		CDense_seg::TDim row = -1;
		for (int ii = 0; ii < ids.size(); ++ii) {
		    if ( ids[ii]->Match(*seqid) ) {
			row = ii;
			break;
		    }
		}
		if ( row < 0 || !denseg.CanGetDim() || row >= denseg.GetDim() ) {
		    continue;
		}

		TCuts sorted_cuts(result.m_arrRangeMatch);
		x_GetSortedCuts(sorted_cuts, seqid);
		x_AdjustDensegAlignment(new_align, row, sorted_cuts);
	    }

	    // Swap edited copy with the original alignment
	    CIRef<IEditCommand> chgAlign(
	        new CCmdChangeAlign( align_ci.GetSeq_align_Handle(), 
                                     *new_align ));
	    command->AddCommand(*chgAlign);
	}
	else {
  	    // For other alignment types, delete them.  This is what
  	    // C Toolkit does (see AdjustAlignmentsInAnnot)
	    CIRef<IEditCommand> delAlign(
	        new CCmdDelSeq_align( align_ci.GetSeq_align_Handle() ));
	    command->AddCommand(*delAlign);
	}
    }
}


void CVectorTrimPanel::x_UpdateSeqGraphLoc(CRef<CSeq_graph> new_graph, 
					   const TCuts& sorted_cuts,
					   const CSeq_id* seqid)
{
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	if (new_graph->CanGetLoc())
	{
            CRef<CSeq_graph::TLoc> new_loc(new CSeq_graph::TLoc);
	    new_loc->Assign(new_graph->GetLoc());
	    bool bDeleted = false;
	    bool bTrimmed = false;
	    x_SeqLocDelete(new_loc, from, to, seqid, bDeleted, bTrimmed);
	    new_graph->SetLoc(*new_loc);
	}
    }
}


void CVectorTrimPanel::x_GetTrimCoordinates(const TCuts& sorted_cuts, 
					    const CSeq_id* seqid, 
					    TSeqPos& trim_start, 
					    TSeqPos& trim_stop)
{
    CScope& scope = m_TopSeqEntry.GetScope();
    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

    // Set defaults
    trim_start = 0;
    trim_stop = bsh.GetInst().GetLength() - 1;

    // Assumptions :
    // All cuts have been sorted.  Internal cuts were converted to terminal.
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	// Left-side terminal cut.  Update trim_start if necessary.
	if ( from == 0 ) {
	    if ( trim_start <= to ) {
	        trim_start = to + 1;
	    }
	}

	// Right-side terminal cut.  Update trim_stop if necessary.
	if ( to == bsh.GetInst().GetLength() - 1 ) {
	    if ( trim_stop >= from ) {
	        trim_stop = from - 1;
	    }
	}
    }
}


void CVectorTrimPanel::x_TrimSeqGraphData(CRef<CSeq_graph> new_graph, 
					  const CMappedGraph& orig_graph,
					  const TCuts& sorted_cuts,
					  const CSeq_id* seqid)
{
    // Get range that original seqgraph data covers
    TSeqPos graph_start = new_graph->GetLoc().GetStart(eExtreme_Positional);
    TSeqPos graph_stop = new_graph->GetLoc().GetStop(eExtreme_Positional);

    // Get range of trimmed sequence
    TSeqPos trim_start;
    TSeqPos trim_stop;
    x_GetTrimCoordinates(sorted_cuts, seqid, trim_start, trim_stop);

    // Determine range over which to copy seqgraph data from old to new
    TSeqPos copy_start = graph_start;
    if (trim_start > graph_start) {
        copy_start = trim_start;
    }
    TSeqPos copy_stop = graph_stop;
    if (trim_stop < graph_stop) {
        copy_stop = trim_stop;
    }

    // Copy over seqgraph data values.  Handle BYTE type only (see 
    // C Toolkit's GetGraphsProc function in api/sqnutil2.c)
    CSeq_graph::TGraph& dst_data = new_graph->SetGraph();
    dst_data.Reset();
    const CSeq_graph::TGraph& src_data = orig_graph.GetGraph();
    switch ( src_data.Which() ) {
    case CSeq_graph::TGraph::e_Byte:
        // Keep original min, max, axis
        dst_data.SetByte().SetMin(src_data.GetByte().GetMin());
        dst_data.SetByte().SetMax(src_data.GetByte().GetMax());
        dst_data.SetByte().SetAxis(src_data.GetByte().GetAxis());

        // Copy start/stop values are relative to bioseq coordinate system.
        // Change them so that they are relative to graph location.
	copy_start -= graph_start;
	copy_stop -= graph_start;

	// Update data values
        dst_data.SetByte().SetValues();
        dst_data.SetByte().SetValues().insert(
            dst_data.SetByte().SetValues().end(), 
	    src_data.GetByte().GetValues().begin() + copy_start,
	    src_data.GetByte().GetValues().begin() + copy_stop + 1);

	// Update numvals
	new_graph->SetNumval(copy_stop - copy_start + 1);

	// Update seqloc
	x_UpdateSeqGraphLoc(new_graph, sorted_cuts, seqid);
        break;
    }
}


void CVectorTrimPanel::x_TrimSeqGraphs(CRef<CCmdComposite> command,
				       const TVecscreenSummaryBySeqid& trimOnly)
{
    SAnnotSelector graph_sel(CSeq_annot::C_Data::e_Graph);
    CGraph_CI graph_ci(m_TopSeqEntry, graph_sel);
    for (; graph_ci; ++graph_ci)
    {
        // Only certain types of graphs are supported.
        // See C Toolkit function GetGraphsProc in api/sqnutil2.c
        const CMappedGraph& graph = *graph_ci;
	if ( graph.IsSetTitle() && 
             (NStr::CompareNocase( graph.GetTitle(), "Phrap Quality" ) == 0 ||
              NStr::CompareNocase( graph.GetTitle(), "Phred Quality" ) == 0 ||
              NStr::CompareNocase( graph.GetTitle(), "Gap4" ) == 0) )
	{
	    // Make a copy of the graph
	    CRef<CSeq_graph> new_graph(new CSeq_graph());
	    new_graph->Assign(graph.GetOriginalGraph());

   	    // Make edits to the copy
	    TVecscreenSummaryBySeqid::const_iterator cit;
            for (cit = trimOnly.begin(); cit != trimOnly.end(); ++cit)
            {
	        const CSeq_id* seqid = cit->first;
	        const SVecscreenResult& result = cit->second;

		// Find matching seqid referred to by the graph
		if ( graph.GetLoc().GetId()->Match(*seqid) ) {
	  	    TCuts sorted_cuts(result.m_arrRangeMatch);
		    x_GetSortedCuts(sorted_cuts, seqid);

		    x_TrimSeqGraphData(new_graph, graph, sorted_cuts, seqid);

	            // Swap edited copy with the original graph
		    CIRef<IEditCommand> chgGraph(
		        new CCmdChangeGraph( graph.GetSeq_graph_Handle(),
					     *new_graph ));
		    command->AddCommand(*chgGraph);
		    break;
		}
	    }
	}
    }
}


bool CVectorTrimPanel::x_IsCompleteCut(const TCuts& sorted_cuts, 
				       TSeqPos seq_length)
{
    // Assume cuts are sorted in Ascending order.


    // Special case:  no cuts
    if (sorted_cuts.empty()) {
        return false;
    }


    // Special case:  only one cut
    if (sorted_cuts.size() == 1) {
        const SRangeMatch& cut = sorted_cuts[0];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();
	if (from == 0 && to == seq_length - 1) {
	    return true;
	}
	else {
	    return false;
	}
    }


    // 2 or more cuts exist
    TSeqPos min_from = -1;
    TSeqPos max_to = -1;
    for (int ii = 0; ii < sorted_cuts.size(); ++ii)
    {
        const SRangeMatch& cut = sorted_cuts[ii];
        TSeqPos from = cut.m_range.GetFrom();
        TSeqPos to = cut.m_range.GetTo();

	if (ii == 0) {
	    min_from = from;
	    max_to = to;
	}
	else {
	    // If the current cut is discontiguous with previous cut(s)
	    // then we do NOT have a complete cut
	    if (from > max_to + 1) {
	        // Gap exists, so this is a discontiguous cut
	        return false;
	    }

	    min_from = min(min_from, from);
	    max_to = max(max_to, to);
	}
    }

    if (min_from == 0 && max_to == seq_length - 1) {
        return true;
    }

    return false;
}


void CVectorTrimPanel::x_FindCompleteDeletions(const TVecscreenSummaryBySeqid& seqidSummary,
					       TVecscreenSummaryBySeqid& completeDeletions,
					       TVecscreenSummaryBySeqid& trimOnly)
{
    // Put complete deletions into completeDeletions container
    // Put partial cuts into trimOnly container

    CScope& scope = m_TopSeqEntry.GetScope();

    TVecscreenSummaryBySeqid::const_iterator cit;
    for (cit = seqidSummary.begin(); cit != seqidSummary.end(); ++cit)
    {
        const CSeq_id* seqid = cit->first;
	const SVecscreenResult& vecres = cit->second;

	// Will the selected sequence be completely deleted by the trim?
	if (vecres.m_Selected)
	{
	    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

            // Should be a nuc!
	    if ( !bsh.IsNucleotide() ) {
	        continue;
	    }

	    // Sort ranges in ascending order
	    TCuts sorted_cuts(vecres.m_arrRangeMatch);
	    CRangeCmp asc(CRangeCmp::eAscending);
	    sort(sorted_cuts.begin(), sorted_cuts.end(), asc);

            // Merge abutting and overlapping cuts
	    x_MergeCuts(sorted_cuts);

            // Adjust internal cuts to 3' end
	    TSeqPos nuc_length = bsh.GetBioseqLength();
	    x_AdjustInternalCutLocations(sorted_cuts, nuc_length);

	    // Is it a complete cut of the entire sequence?
	    if (x_IsCompleteCut(sorted_cuts, nuc_length)) {
	        completeDeletions.insert(TVecscreenSummaryBySeqid::value_type(seqid, 
									      vecres));
	    }
	    else {
	        trimOnly.insert(TVecscreenSummaryBySeqid::value_type(seqid, 
								     vecres));
	    }
	}
    }
}


void CVectorTrimPanel::x_DeleteSelected(CRef<CCmdComposite> command,
					const CSeq_id* seqid)
{
    CScope& scope = m_TopSeqEntry.GetScope();
    CBioseq_Handle bsh = scope.GetBioseqHandle(*seqid);

    // Should be a nuc!
    if (!bsh.IsNucleotide())
    {
        return;
    }

    // Is nuc bioseq part of a nucprot set?  If so, delete the nucprot
    // set.  Otherwise delete just the nuc.
    CBioseq_set_Handle bssh = bsh.GetParentBioseq_set();
    if (bssh && bssh.CanGetClass() && 
        bssh.GetClass() == CBioseq_set::eClass_nuc_prot) 
    {
        // Delete the nucprot set
        CRef<CCmdDelBioseqSet> delSet(new CCmdDelBioseqSet(bssh));
	command->AddCommand(*delSet);
    }
    else 
    {
        // Delete just the nuc
        CRef<CCmdDelBioseqInst> delSeq(new CCmdDelBioseqInst(bsh));
	command->AddCommand(*delSeq);
    }
}


void CVectorTrimPanel::x_TrimSelected()
{
    // Create a new composite command for later execution
    m_Command.Reset( new CCmdComposite( "Trim Vector" ) );


    const TVecscreenSummaryBySeqid& seqidSummary = 
        m_listMatches->GetVecscreenSummaryBySeqid();


    // Detect if some sequences are 100% contamination sequences.
    // Warn user that these sequences will be completely removed.
    // Give user chance to CANCEL.
    TVecscreenSummaryBySeqid completeDeletions;
    TVecscreenSummaryBySeqid trimOnly;
    x_FindCompleteDeletions(seqidSummary, completeDeletions, trimOnly);
    if ( !completeDeletions.empty() ) {
        CListReportDlg* listReport = new CListReportDlg(this);

        wxString header;
	header << "The following " << completeDeletions.size() 
	       << " sequences are 100% matches to vector and \n"
	       << "will be deleted from your submission:";
	listReport->SetHeader(header);

	wxString list;
	TVecscreenSummaryBySeqid::const_iterator cit;
	for (cit = completeDeletions.begin(); cit != completeDeletions.end(); ++cit)
	{
	    const CSeq_id* seqid = cit->first;
	    const SVecscreenResult& vecres = cit->second;

	    list << seqid->AsFastaString() << "\n";
	}
	listReport->SetList(list);

	wxString footer;
	footer << "Do you want to continue?";
	listReport->SetFooter(footer);

	if ( listReport->ShowModal() == wxID_CANCEL ) {
	    // User canceled
	    return;
	}
    }


    // Delete selected sequences
    TVecscreenSummaryBySeqid::const_iterator cit;
    for (cit = completeDeletions.begin(); cit != completeDeletions.end(); ++cit)
    {
        const CSeq_id* seqid = cit->first;

	x_DeleteSelected(m_Command, seqid);
    }


    // Trim selected sequences
    for (cit = trimOnly.begin(); cit != trimOnly.end(); ++cit)
    {
        const CSeq_id* seqid = cit->first;
	const SVecscreenResult& result = cit->second;

	x_TrimSelected(m_Command, seqid, result);
    }

    // Trim alignments
    x_TrimAlignments(m_Command, trimOnly);

    // Trim seq-graphs
    x_TrimSeqGraphs(m_Command, trimOnly);


    // Execute all the stored commands
    m_Command->Execute();
}


void CVectorTrimPanel::
DisplaySummary(const vector<CVecscreenRun::SVecscreenSummary>& 
               vecscreen_summary)
{
    m_listMatches->UpdateData(vecscreen_summary);
}


void CVectorTrimPanel::
DisplaySummaryBySeqid(const TVecscreenSummaryBySeqid& 
		      vecscreen_summary)
{
    m_listMatches->UpdateData(vecscreen_summary);
}


CRef<CCmdComposite> CVectorTrimPanel::GetCommand()
{
    return m_Command;
}


IMPLEMENT_DYNAMIC_CLASS( CMatchesListCtrl, wxListCtrl )


BEGIN_EVENT_TABLE(CMatchesListCtrl, wxListCtrl)
    EVT_LIST_COL_CLICK(MYID_LISTCTRL_MATCHES, CMatchesListCtrl::OnColClick)
    EVT_LIST_ITEM_SELECTED(MYID_LISTCTRL_MATCHES, CMatchesListCtrl::OnSelected)
    EVT_LIST_ITEM_ACTIVATED(MYID_LISTCTRL_MATCHES, CMatchesListCtrl::OnDoubleClick)
END_EVENT_TABLE()


CMatchesListCtrl::CMatchesListCtrl()
{
}


CMatchesListCtrl::CMatchesListCtrl(wxWindow *parent,
				   const wxWindowID id,
				   const wxPoint& pos,
				   const wxSize& size,
				   long style)
{
    Create(parent, id, pos, size, style);
}


CMatchesListCtrl::~CMatchesListCtrl()
{
}


bool CMatchesListCtrl::Create(wxWindow *parent, 
			      wxWindowID id,
			      const wxPoint &pos,
			      const wxSize &size, 
			      long style)
{
    wxListCtrl::Create(parent, id, pos, size, style);

    // Set line item graphic properties
    m_listItemAttr.SetTextColour(*wxBLUE);
    m_listItemAttr.SetBackgroundColour(*wxLIGHT_GREY);
    m_listItemAttr.SetFont(wxNullFont);

    // Set up the column properties
    wxListItem col0;
    col0.SetColumn(0);
    col0.SetText( _("Select") );
    col0.SetWidth(50);
    col0.SetAlign( wxLIST_FORMAT_CENTRE );
    InsertColumn( 0, col0 );

    wxListItem col1;
    col1.SetColumn(1);
    col1.SetText( _("Location") );
    col1.SetWidth(100);
    InsertColumn( 1, col1 );

    wxListItem col2;
    col2.SetColumn(2);
    col2.SetText( _("Seqid") );
    col2.SetWidth(150);
    InsertColumn( 2, col2 );

    wxListItem col3;
    col3.SetColumn(3);
    col3.SetText( _("Match") );
    col3.SetWidth(150);
    InsertColumn( 3, col3 );

    return true;
}


void CMatchesListCtrl::SetAlignGraphicView( wxHtmlWindow* view )
{
    m_AlignView = view;
}


void CMatchesListCtrl::SetRangeView( CLocationsListCtrl* view )
{
    m_RangeView = view;
}


bool CMatchesListCtrl::UpdateData(const vector<CVecscreenRun::SVecscreenSummary>& 
				  vecscreen_summary)
{
    m_vecscreen_summary = vecscreen_summary;

    // Clean slate
    DeleteAllItems();

    // Set value in each column of each row
    for (int ii = 0; ii < m_vecscreen_summary.size(); ++ii)
    {
        const CVecscreenRun::SVecscreenSummary& summary = m_vecscreen_summary[ii];

        // Insert a row
        wxListItem row;
	row.SetId(ii);
	InsertItem(row);

	// Select the row by default by showing "X"
	SetItem( ii, 0, _( "X" ) );

	// 5` or 3` End
	SetItem( ii, 1, _( "" ) );

	// Seqid
	SetItem( ii, 2, _( summary.seqid->AsFastaString() ) );

	// Match type
	SetItem( ii, 3, _( summary.match_type ) );
    }

    return true;
}


const wxString& CMatchesListCtrl::x_GetSelected(const SVecscreenResult& vecres)
{
    if (vecres.m_Selected)
    {
        return _( "*" );
    }
    else
    {
        return _( "" );
    }
}


void CMatchesListCtrl::x_MergeHitLoc(SRangeMatch::EHitLocation& final_hitloc,
				     const SRangeMatch::EHitLocation& current_hitloc)
{
    if (final_hitloc == current_hitloc)
    {
        // The two are identical - nothing needs to be done
    }
    else
    if (final_hitloc == SRangeMatch::eHitLocation_Unknown || 
	current_hitloc == SRangeMatch::eHitLocation_Unknown)
    {
        // At least one is Unknown
        final_hitloc = SRangeMatch::eHitLocation_Unknown;
    }
    else
    if (final_hitloc == SRangeMatch::eHitLocation_Mixed || 
	current_hitloc == SRangeMatch::eHitLocation_Mixed)
    {
        // At least one is Mixed
        final_hitloc = SRangeMatch::eHitLocation_Mixed;
    }
    else
    {
        // One must be Terminal, and the other Internal
        final_hitloc = SRangeMatch::eHitLocation_Mixed;
    }
}


const wxString& CMatchesListCtrl::x_GetHitLocation(const SVecscreenResult& vecres)
{
    // Possible values
    // Terminal: 5' End or 3' End or both
    // Internal: NOT Terminal
    // Mixed   : both terminal and internal hits found
    // Unknown : default

    static const TSeqPos kTerminalLenFromEnd = 50;

    SRangeMatch::EHitLocation final_hitloc = SRangeMatch::eHitLocation_Unknown;
    bool bInitialized = false;
    for (int ii = 0; ii < vecres.m_arrRangeMatch.size(); ++ii)
    {
        const SRangeMatch& rangematch = vecres.m_arrRangeMatch[ii];

	// Filter out "suspect" 
	if (NStr::FindNoCase(rangematch.m_match_type, "suspect") != NPOS)
	{
	    continue;
	}

	const TSeqPos loc_start = rangematch.m_range.GetFrom();
	const TSeqPos loc_stop = rangematch.m_range.GetTo();

	// Is current match terminal or internal?
	SRangeMatch::EHitLocation current_hitloc = SRangeMatch::eHitLocation_Unknown;
	if (loc_start != loc_stop &&
            (loc_start < kTerminalLenFromEnd || 
             vecres.m_SeqLen - loc_stop < kTerminalLenFromEnd))
	{
  	    current_hitloc = SRangeMatch::eHitLocation_Terminal;
	}
	else
	{
  	    current_hitloc = SRangeMatch::eHitLocation_Internal;
	}

	// Calculate final hit location type
	if ( !bInitialized )
	{
	    final_hitloc = current_hitloc;
	    bInitialized = true;
	}
	else
	{
	    x_MergeHitLoc(final_hitloc, current_hitloc);
	}
    }


    // Display result
    switch (final_hitloc)
    {
        case SRangeMatch::eHitLocation_Terminal:
	    return _( "Terminal" );

        case SRangeMatch::eHitLocation_Internal:
  	    return _( "Internal" );

        case SRangeMatch::eHitLocation_Mixed:
  	    return _( "Mixed" );

        case SRangeMatch::eHitLocation_Unknown:
  	    return _( "Unknown" );
    }
}


const wxString& CMatchesListCtrl::x_GetSeqid(const SVecscreenResult& vecres)
{
    return _( vecres.m_FastaSeqid );
}


const wxString& CMatchesListCtrl::x_GetMatchType(const SVecscreenResult& vecres)
{
    // Concatenate the match type(s)
    stringstream ssMatchType;
    for (int ii = 0; ii < vecres.m_arrRangeMatch.size(); ++ii)
    {
	if (ii > 0)
	{
	    ssMatchType << "; ";
	}
	ssMatchType <<  vecres.m_arrRangeMatch[ii].m_match_type;
    }
    return _( ssMatchType.str() );
}


bool CMatchesListCtrl::UpdateData(const TVecscreenSummaryBySeqid& 
				  vecscreen_summary)
{
    m_seqidSummary = vecscreen_summary;
    x_UpdateData();
    return true;
}


void CMatchesListCtrl::x_UpdateData()
{
    // Clean slate
    DeleteAllItems();

    // Set value in each column of each row
    TVecscreenSummaryBySeqid::const_iterator cit;
    int row = 0;
    for (cit = m_seqidSummary.begin(); cit != m_seqidSummary.end(); ++cit)
    {
        const CSeq_id* seqid = cit->first;
	const SVecscreenResult& vecres = cit->second;

	// Insert a row
	wxListItem item;
	item.SetId(row);
	InsertItem(item);

	// Column to indicate selected or not
	SetItem( row, 0, x_GetSelected(vecres) );

	// 5` or 3` End
	SetItem( row, 1, x_GetHitLocation(vecres) );

	// Seqid
	SetItem( row, 2, x_GetSeqid(vecres) );

	// Match type
        SetItem( row, 3, x_GetMatchType(vecres) );

	++row;
    }
}


void CMatchesListCtrl::SelectAll()
{
    TVecscreenSummaryBySeqid::iterator it;
    int row = 0;
    for (it = m_seqidSummary.begin(); it != m_seqidSummary.end(); ++it)
    {
        const CSeq_id* seqid = it->first;
	SVecscreenResult& vecres = it->second;

	vecres.m_Selected = true;
    }

    x_UpdateData();
}


void CMatchesListCtrl::SelectStrongModerate()
{
    TVecscreenSummaryBySeqid::iterator it;
    int row = 0;
    for (it = m_seqidSummary.begin(); it != m_seqidSummary.end(); ++it)
    {
        const CSeq_id* seqid = it->first;
	SVecscreenResult& vecres = it->second;

	vecres.m_Selected = false;
	for (int ii = 0; ii < vecres.m_arrRangeMatch.size(); ++ii)
	{
	    const SRangeMatch& rangematch = vecres.m_arrRangeMatch[ii];
	    if ( NStr::FindNoCase(rangematch.m_match_type, "strong") != NPOS || 
                 NStr::FindNoCase(rangematch.m_match_type, "moderate") != NPOS )
	    {
	        vecres.m_Selected = true;
	        break;
	    }
	}
    }

    x_UpdateData();
}


void CMatchesListCtrl::UnselectAll()
{
    TVecscreenSummaryBySeqid::iterator it;
    int row = 0;
    for (it = m_seqidSummary.begin(); it != m_seqidSummary.end(); ++it)
    {
        const CSeq_id* seqid = it->first;
	SVecscreenResult& vecres = it->second;

	vecres.m_Selected = false;
    }

    x_UpdateData();
}


void CMatchesListCtrl::OnColClick(wxListEvent& event)
{
    // Use this callback to sort

    int col = event.GetColumn();

}


void CMatchesListCtrl::OnSelected(wxListEvent& event)
{
    if ( GetWindowStyle() & wxLC_REPORT )
    {
        int selected_row = event.GetIndex();

        TVecscreenSummaryBySeqid::const_iterator cit;
        int count = 0;
        for (cit = m_seqidSummary.begin(); cit != m_seqidSummary.end(); ++cit)
        {
	    if (count == selected_row)
	    {
	        if (m_AlignView)
		{
		    m_AlignView->SetPage(cit->second.m_AlignmentGraphic);
		}
	        if (m_RangeView)
		{
		    m_RangeView->UpdateData(cit->second);
		}
	        break;
	    }

	    ++count;
	}
    }
}


void CMatchesListCtrl::OnDoubleClick(wxListEvent& event)
{
    if ( GetWindowStyle() & wxLC_REPORT )
    {
        int selected_row = event.GetIndex();

        TVecscreenSummaryBySeqid::iterator it;
        int count = 0;
        for (it = m_seqidSummary.begin(); it != m_seqidSummary.end(); ++it)
        {
	    if (count == selected_row)
	    {
	        SVecscreenResult& vecres = it->second;

		// Toggle the value
		vecres.m_Selected ? vecres.m_Selected = false :
                                    vecres.m_Selected = true;

	        break;
	    }

	    ++count;
	}
    }

    x_UpdateData();
}


IMPLEMENT_DYNAMIC_CLASS( CLocationsListCtrl, wxListCtrl )


BEGIN_EVENT_TABLE(CLocationsListCtrl, wxListCtrl)
    EVT_LIST_ITEM_SELECTED(MYID_LISTCTRL_LOCATIONS, CLocationsListCtrl::OnSelected)
END_EVENT_TABLE()


CLocationsListCtrl::CLocationsListCtrl()
{
}


CLocationsListCtrl::CLocationsListCtrl(wxWindow *parent,
				       const wxWindowID id,
				       const wxPoint& pos,
				       const wxSize& size,
				       long style)
{
    Create(parent, id, pos, size, style);
}


CLocationsListCtrl::~CLocationsListCtrl()
{
}


bool CLocationsListCtrl::Create(wxWindow *parent, 
				wxWindowID id,
				const wxPoint &pos,
				const wxSize &size, 
				long style)
{
    wxListCtrl::Create(parent, id, pos, size, style);

    // Set line item graphic properties
    m_listItemAttr.SetTextColour(*wxBLUE);
    m_listItemAttr.SetBackgroundColour(*wxLIGHT_GREY);
    m_listItemAttr.SetFont(wxNullFont);

    // Set up the column properties
    wxListItem col0;
    col0.SetColumn(0);
    col0.SetText( _("Seqid") );
    col0.SetWidth(200);
    InsertColumn( 0, col0 );

    wxListItem col1;
    col1.SetColumn(1);
    col1.SetText( _("Range") );
    col1.SetWidth(250);
    InsertColumn( 1, col1 );

    return true;
}


bool CLocationsListCtrl::UpdateData(const SVecscreenResult& vecres)
{
    // Clean slate
    DeleteAllItems();

    // Set value in each column of each row
    for (int ii = 0; ii < vecres.m_arrRangeMatch.size(); ++ii)
    {
        const SRangeMatch& rangematch = vecres.m_arrRangeMatch[ii];

        // Insert a row
        wxListItem row;
	row.SetId(ii);
	InsertItem(row);

	// Seqid
	SetItem( ii, 0, _( vecres.m_FastaSeqid ) );

	// Range
	stringstream ssRange;
	ssRange << rangematch.m_range.GetFrom() + 1 
                << "-" << rangematch.m_range.GetTo() + 1;
	SetItem( ii, 1, _( ssRange.str() ) );
    }

    return true;
}


void CLocationsListCtrl::OnSelected(wxListEvent& event)
{
    if ( GetWindowStyle() & wxLC_REPORT )
    {
        wxListItem info;
        info.m_itemId = event.GetIndex();

    }
}


IMPLEMENT_DYNAMIC_CLASS( CListReportDlg, wxDialog )


BEGIN_EVENT_TABLE( CListReportDlg, wxDialog )
END_EVENT_TABLE()


CListReportDlg::CListReportDlg()
{
    Init();
}

CListReportDlg::CListReportDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


bool CListReportDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
    return true;
}


CListReportDlg::~CListReportDlg()
{
}


void CListReportDlg::Init()
{
}


void CListReportDlg::CreateControls()
{
    wxBoxSizer* dialogSizer = new wxBoxSizer(wxVERTICAL);
    this->SetSizer(dialogSizer);


    // Display header portion of dialog
    m_textHeader = new wxStaticText( this, 
				     wxID_ANY,
				     _( "" ),
				     wxDefaultPosition,
				     wxDefaultSize,
				     wxALIGN_LEFT );
    dialogSizer->Add(m_textHeader, 0, wxEXPAND|wxALL, 5);


    // Display message list in a scrollable text field
    m_textList = new wxTextCtrl( this,
				 wxID_ANY, 
				 wxEmptyString, 
				 wxDefaultPosition, 
				 wxSize(600, 450),
				 wxTE_MULTILINE|wxTE_READONLY );
    dialogSizer->Add(m_textList, 1, wxEXPAND|wxALL, 5);


    // Display footer portion of dialog
    m_textFooter = new wxStaticText( this, 
				     wxID_ANY,
				     _( "" ),
				     wxDefaultPosition,
				     wxDefaultSize,
				     wxALIGN_LEFT );
    dialogSizer->Add(m_textFooter, 0, wxEXPAND|wxALL, 5);


    // Display Ok/Cancel buttons
    wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
    wxButton* okButton = new wxButton( this, wxID_OK, _( "Ok" ), wxDefaultPosition, wxDefaultSize, 0 );
    wxButton* cancelButton = new wxButton( this, wxID_CANCEL, _( "Cancel" ), wxDefaultPosition, wxDefaultSize, 0 );
    buttonSizer->Add(okButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    buttonSizer->Add(cancelButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    dialogSizer->Add(buttonSizer, 0, wxEXPAND|wxALL, 5);
}


void CListReportDlg::SetHeader(const wxString& header)
{
    m_textHeader->SetLabel(header);
}


void CListReportDlg::SetList(const wxString& list)
{
    m_textList->SetValue(list);
}


void CListReportDlg::SetFooter(const wxString& footer)
{
    m_textFooter->SetLabel(footer);
}


END_NCBI_SCOPE

