/*  $Id: bookmarks_handler.cpp 17877 2008-09-24 13:03:09Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vlad Lebedev
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/gl/bookmarks_handler.hpp>
#include <gui/widgets/wx/gui_event.hpp>

#include <gui/opengl/glhelpers.hpp>

BEGIN_NCBI_SCOPE


////////////////////////////////////////////////////////////////////////////////
/// class CBookmarksHandler

BEGIN_EVENT_TABLE(CBookmarksHandler, wxEvtHandler)
    EVT_LEFT_DOWN(CBookmarksHandler::OnLeftDown)
    EVT_LEFT_DCLICK(CBookmarksHandler::OnLeftDoubleClick)
    EVT_MOTION(CBookmarksHandler::OnMotion)
END_EVENT_TABLE()

const int kMarkerSize = 6;


CBookmarksHandler::CBookmarksHandler(EOrientation orient)
: m_Orientation(orient),
  m_Host(NULL)
{
    //m_Tooltip.EnableActiveMode(static_cast<ITooltipClient*>(this));
    //m_Tooltip.SetMode(CTooltip::eHideOnMove);
}


CBookmarksHandler::~CBookmarksHandler()
{
}


void CBookmarksHandler::SetPane(CGlPane* pane)
{
    m_Pane = pane;
}


wxEvtHandler* CBookmarksHandler::GetEvtHandler()
{
    return this;
}


void CBookmarksHandler::SetHost(IBookmarksHandlerHost* pHost)
{
    m_Host = pHost;
}


IGenericHandlerHost* CBookmarksHandler::GetGenericHost()
{
    return dynamic_cast<IGenericHandlerHost*>(m_Host);
}


void CBookmarksHandler::SetOrientation(EOrientation orient)
{
    m_Orientation = orient;
}


///////////////////////////////////////////////////////////////////////////////
/// ITooltip Implementation
/// TC_NeedTooltip() and TC_GetTooltip() is evrything needed to show toolitps
/// in "active" mode
/*bool CBookmarksHandler::TC_NeedTooltip(int x, int y)
{
    const TModelRect& rc = m_Pane->GetVisibleRect();

    bool horz = m_Orientation == eHorz;
    TBookmarks book_marks = m_Host->BHH_GetBookmarks(
            TSeqPos(horz ? rc.Left() : rc.Bottom()),
            TSeqPos(horz ? rc.Right(): rc.Top())
    );

    TModelUnit x_pos = m_Host->BHH_GetModelByWindow(x, eHorz);
    TModelUnit y_pos = m_Host->BHH_GetModelByWindow(y, eVert);
    TModelPoint mouse_point(x_pos, y_pos);

    ITERATE (TBookmarks, iter, book_marks) {
        CRef<CBookmark> b_mark = *iter;
        TModelRect rc = x_GetMarkRect(b_mark);

        if (rc.PtInRect(mouse_point) ) {
            m_TooltipText = b_mark->GetLabel();
            return true;
        }
    }
    return false;
}

string CBookmarksHandler::TC_GetTooltip(int& x, int& y, int& w, int& h)
{
    return m_TooltipText;
}

const Fl_Widget* CBookmarksHandler::TC_GetWidget() const
{
    return m_Host->BHH_GetWidget();
}
*/
// ITooltip end

////////////////////////////////////////////////////////////////////////////////
// event handlers
void CBookmarksHandler::OnLeftDown(wxMouseEvent& event)
{
    event.Skip();
}


void CBookmarksHandler::OnLeftDoubleClick(wxMouseEvent& event)
{
    CGUIEvent::EGUIState state = CGUIEvent::wxGetSelectState(event);
    if (state == CGUIEvent::eSelectState)   {
        const TModelRect& rc = m_Pane->GetVisibleRect();

        bool horz = m_Orientation == eHorz;
        TBookmarks book_marks = m_Host->BHH_GetBookmarks(
                TSeqPos(horz ? rc.Left() : rc.Bottom()),
                TSeqPos(horz ? rc.Right(): rc.Top())
        );

        wxPoint pos = event.GetPosition();
        TModelUnit x_pos = m_Host->BHH_GetModelByWindow(pos.x, eHorz);
        TModelUnit y_pos = m_Host->BHH_GetModelByWindow(pos.y, eVert);
        TModelPoint click_point(x_pos, y_pos);

        ITERATE (TBookmarks, iter, book_marks) {
            CRef<CBookmark> b_mark = *iter;
            TModelRect rc = x_GetMarkRect(b_mark);

            if (rc.PtInRect(click_point) ) {
                m_Host->BHH_ZoomToRange(b_mark->GetRange());
                GetGenericHost()->GHH_Redraw();
                return;
            }
        }
    }
    event.Skip();
}


TModelRect CBookmarksHandler::x_GetMarkRect(CRef<CBookmark> mark) const
{
    TModelRect rc     = m_Pane->GetVisibleRect();
    TModelUnit half_x = m_Pane->UnProjectWidth(kMarkerSize);
    TModelUnit half_y = m_Pane->UnProjectHeight(kMarkerSize);
    TSeqRange r       = mark->GetRange();
    TModelUnit pos    = r.GetFrom() + (r.GetTo() - r.GetFrom()) / 2;

    TModelRect res;
    if (m_Orientation == eHorz) {
        res = TModelRect(pos - half_x, rc.Bottom(),
                         pos + half_x, rc.Bottom() + half_y * 2);
    } else {
        res = TModelRect(rc.Left(), pos - half_y,
                         rc.Left() + half_x * 2, pos + half_y);
    }
    return res;
}


void CBookmarksHandler::OnMotion(wxMouseEvent& event)
{
    if(event.Dragging())  {
        // always handle drags
    } else {
        // TODO - must change cursor when mouse is over a bookmark
        event.Skip();
    }
}


////////////////////////////////////////////////////////////////////////////////
/// helper functions
void CBookmarksHandler::x_DrawMarker(int x_c, int y_c, int half) const
{
    glBegin(GL_POLYGON);
        if (m_Orientation == eHorz) { // top & bottom
            glVertex2d(x_c, y_c + half);
            glVertex2d(x_c + half, y_c);
            glVertex2d(x_c + half, y_c - half);
            glVertex2d(x_c - half, y_c - half);
            glVertex2d(x_c - half, y_c);
        } else { // left & right
            glVertex2d(x_c + half, y_c);
            glVertex2d(x_c, y_c - half);
            glVertex2d(x_c - half, y_c - half);
            glVertex2d(x_c - half, y_c + half);
            glVertex2d(x_c, y_c + half);
        }
    glEnd();
}


void CBookmarksHandler::Render(CGlPane& pane)
{
    CGlAttrGuard AttrGuard(GL_ENABLE_BIT | GL_POLYGON_BIT | GL_HINT_BIT);

    const TModelRect& rc = pane.GetVisibleRect();
    const TVPRect&   rcV = pane.GetViewport();

    pane.OpenPixels();

    bool horz = (m_Orientation == eHorz);
    TBookmarks book_marks = m_Host->BHH_GetBookmarks(
            TSeqPos(horz ? rc.Left() : rc.Bottom()),
            TSeqPos(horz ? rc.Right(): rc.Top())
    );

    ITERATE (TBookmarks, iter, book_marks) {
        CRef<CBookmark> b_mark = *iter;
        TSeqRange range = b_mark->GetRange();

        TSeqPos pos = range.GetFrom() + (range.GetTo() - range.GetFrom()) / 2;

        TVPPoint p;
        if (horz) {
            TVPUnit cur_pos = pane.ProjectX(pos);
            p = TVPPoint(cur_pos, rcV.Bottom() + kMarkerSize);
        } else {
            TVPUnit cur_pos = pane.ProjectY(pos);
            p = TVPPoint(rcV.Left() + kMarkerSize, cur_pos);
        }

        glColor4f(0.0f, 0.0f, 1.0f, 0.3f);
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        x_DrawMarker(p.X(), p.Y(), kMarkerSize);

        glColor4f(0.0f, 0.0f, 1.0f, 0.9f);  // bright border
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
        x_DrawMarker(p.X(), p.Y(), kMarkerSize);
    }
    pane.Close();
}


END_NCBI_SCOPE
