/*  $Id: edit_macro_dlg.cpp 24312 2011-09-09 20:29:12Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <gui/widgets/macro_edit/edit_macro_dlg.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/utils/macro_parse.hpp>

#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/textctrl.h>
#include <wx/msgdlg.h>

BEGIN_NCBI_SCOPE

/*!
 * CEditMacroDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CEditMacroDlg, CDialog )


/*!
 * CEditMacroDlg event table definition
 */

BEGIN_EVENT_TABLE( CEditMacroDlg, CDialog )

////@begin CEditMacroDlg event table entries
    EVT_BUTTON( ID_BUTTON, CEditMacroDlg::OnParseClick )

////@end CEditMacroDlg event table entries

END_EVENT_TABLE()


/*!
 * CEditMacroDlg constructors
 */

CEditMacroDlg::CEditMacroDlg()
{
    Init();
}

CEditMacroDlg::CEditMacroDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CEditMacroDlg creator
 */

bool CEditMacroDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CEditMacroDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    CDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CEditMacroDlg creation
    return true;
}


/*!
 * CEditMacroDlg destructor
 */

CEditMacroDlg::~CEditMacroDlg()
{
////@begin CEditMacroDlg destruction
////@end CEditMacroDlg destruction
}


/*!
 * Member initialisation
 */

void CEditMacroDlg::Init()
{
////@begin CEditMacroDlg member initialisation
    m_TextCtrl = NULL;
////@end CEditMacroDlg member initialisation

    SetRegistryPath( "Workbench.Dialogs.MacroEditor" );
}


/*!
 * Control creation for CEditMacroDlg
 */

void CEditMacroDlg::CreateControls()
{    
////@begin CEditMacroDlg content construction
    CEditMacroDlg* itemCDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCDialog1->SetSizer(itemBoxSizer2);

    m_TextCtrl = new wxTextCtrl( itemCDialog1, ID_TEXTCTRL, wxEmptyString, wxDefaultPosition, wxSize(350, 250), wxTE_MULTILINE );
    itemBoxSizer2->Add(m_TextCtrl, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxGROW|wxALL, 5);

    itemBoxSizer4->Add(5, 5, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton6 = new wxButton( itemCDialog1, ID_BUTTON, _("Parse"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemButton6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemCDialog1, wxID_CANCEL, _("Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemButton7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CEditMacroDlg content construction
}


/*!
 * Should we show tooltips?
 */

bool CEditMacroDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CEditMacroDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CEditMacroDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CEditMacroDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CEditMacroDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CEditMacroDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CEditMacroDlg icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
 */

void CEditMacroDlg::OnParseClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON in CEditMacroDlg.
    // Before editing this code, remove the block markers.
    wxString wstr = m_TextCtrl->GetValue();
    string s = ToStdString(wstr);

    CQueryMacro qm;
    try {
        qm.Parse(s.c_str());
    } 
    catch (std::exception& ex) 
    {
        wxMessageBox(ToWxString(ex.what()));
    }

    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON in CEditMacroDlg. 
}

END_NCBI_SCOPE
