/*  $Id: rel_obj_list_ctrl.cpp 25482 2012-03-27 15:00:20Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Yury Voronov
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <wx/sizer.h>
#include <wx/choice.h>
#include <wx/statbox.h>
#include <wx/button.h>
#include <wx/msgdlg.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

#include <objmgr/object_manager.hpp>

#include <gui/objutils/obj_convert.hpp>

#include <gui/widgets/wx/table_utils.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/widgets/object_list/object_list_widget.hpp>
#include <gui/widgets/object_list/rel_obj_list_ctrl.hpp>

#include <gui/widgets/object_list/object_list_widget_sel.hpp>

BEGIN_NCBI_SCOPE

/*!
 * CRelatedObjectListCtrl type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CRelatedObjectListCtrl, wxPanel )


/*!
 * CRelatedObjectListCtrl event table definition
 */

BEGIN_EVENT_TABLE( CRelatedObjectListCtrl, wxPanel )

////@begin CRelatedObjectListCtrl event table entries
    EVT_TEXT( ID_TEXTCTRL1, CRelatedObjectListCtrl::OnAccessionUpdated )

    EVT_BUTTON( ID_BUTTON1, CRelatedObjectListCtrl::OnAddAccessionClick )

    EVT_CHOICE( ID_CHOICE1, CRelatedObjectListCtrl::OnChoice1Selected )

////@end CRelatedObjectListCtrl event table entries

END_EVENT_TABLE()


/*!
 * CRelatedObjectListCtrl constructors
 */

CRelatedObjectListCtrl::CRelatedObjectListCtrl() 
    : m_AccValidator( NULL )
{
    Init();
}

CRelatedObjectListCtrl::CRelatedObjectListCtrl( 
    wxWindow* parent, 
    wxWindowID id, 
    const wxPoint& pos, 
    const wxSize& size, 
    long style 
) 
    : m_AccValidator( NULL )
{
    Init();
    Create( parent, id, pos, size, style );
}


/*!
 * CRelatedObjectListCtrl creator
 */

bool CRelatedObjectListCtrl::Create( 
    wxWindow* parent, 
    wxWindowID id, 
    const wxPoint& pos, 
    const wxSize& size, 
    long style 
){
    bool f_single_sel = (style & wxLC_SINGLE_SEL) != 0;
    style &= ~wxLC_SINGLE_SEL;

////@begin CRelatedObjectListCtrl creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();

    style = m_List1->GetWindowStyle();
    if( f_single_sel ){
        style |= wxLC_SINGLE_SEL;
    } else {
        style &= ~wxLC_SINGLE_SEL;
    }
    m_List1->SetWindowStyle( style );

    Centre();
////@end CRelatedObjectListCtrl creation
    return true;
}


/*!
 * CRelatedObjectListCtrl destructor
 */

CRelatedObjectListCtrl::~CRelatedObjectListCtrl()
{
////@begin CRelatedObjectListCtrl destruction
////@end CRelatedObjectListCtrl destruction
}


/*!
 * Member initialisation
 */

void CRelatedObjectListCtrl::Init()
{
////@begin CRelatedObjectListCtrl member initialisation
    m_DoSelectAll = false;
    m_Accession = NULL;
    m_AddBtn = NULL;
    m_Choice1 = NULL;
    m_List1 = NULL;
////@end CRelatedObjectListCtrl member initialisation

    CRef<objects::CObjectManager> obj_mgr = objects::CObjectManager::GetInstance();
    m_Scope.Reset(new objects::CScope(*obj_mgr));
    m_Scope->AddDefaults();
}


/*!
 * Control creation for CRelatedObjectListCtrl
 */

void CRelatedObjectListCtrl::CreateControls()
{
////@begin CRelatedObjectListCtrl content construction
    CRelatedObjectListCtrl* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemPanel1, ID_TITLE_BOX, _("Static"));
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemStaticBoxSizer3->Add(itemBoxSizer4, 0, wxGROW|wxALL, 0);

    m_Accession = new wxTextCtrl( itemPanel1, ID_TEXTCTRL1, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_Accession, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_AddBtn = new wxButton( itemPanel1, ID_BUTTON1, _("Add Accession"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_AddBtn, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer4->Add(7, 1, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_Choice1Strings;
    m_Choice1 = new wxChoice( itemPanel1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, m_Choice1Strings, 0 );
    itemBoxSizer4->Add(m_Choice1, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);

    m_List1 = new CObjectListWidget( itemPanel1, ID_LISTCTRL1, wxDefaultPosition, wxDefaultSize, wxLC_REPORT );
    itemStaticBoxSizer3->Add(m_List1, 1, wxGROW|wxALL, 2);

////@end CRelatedObjectListCtrl content construction

    x_UpdateUI();

    SetTitle(wxT("Select Object"));
}

void CRelatedObjectListCtrl::SetAccessionValidator(IObjectListAccValidator* validator)
{
    m_AccValidator = validator;
    x_UpdateUI();
}

void CRelatedObjectListCtrl::x_UpdateUI()
{
    wxSizer* hSizer = m_Choice1->GetContainingSizer();
    wxSizer* mSizer = GetSizer();

    if( !m_AccValidator && m_RelationMap.size() <= 1 ){

        mSizer->Show( hSizer, false, true );

    } else {
        mSizer->Show( hSizer, true, true );
        if( m_AccValidator ){
            m_Accession->Show( true );
            m_AddBtn->Show( true );
            x_UpdateAddBtn();
        
        } else {
            m_Accession->Show(false);
            m_AddBtn->Show(false);
        }

        m_Choice1->Show( m_RelationMap.size() > 1 );
    }

    mSizer->Layout();
}

void CRelatedObjectListCtrl::x_UpdateAddBtn()
{
    wxString acc = m_Accession->GetValue();
    acc.Trim(false);
    acc.Trim(true);

    m_AddBtn->Enable( !acc.IsEmpty() );
}

void CRelatedObjectListCtrl::SetTitle( const wxString& value )
{
    wxStaticBox* staticBox = (wxStaticBox*)FindWindow( ID_TITLE_BOX );
    if( staticBox ){
        staticBox->SetLabel( value );
    }
}


void CRelatedObjectListCtrl::OnChoice1Selected( wxCommandEvent& event )
{
    x_FillObjects( event.GetString() );
}

void CRelatedObjectListCtrl::GetObjects( TConstScopedObjects& objects ) const
{
    objects = m_Objects;
}

void CRelatedObjectListCtrl::SetObjects( TConstScopedObjects& objects, const string& type )
{
    m_Objects = objects;

    x_FillRelationMap( type );
}

void CRelatedObjectListCtrl::x_FillRelationMap( const string& type )
{
    m_RelationMap.clear();

    set<const CObject*> objects_used;
    ITERATE( TConstScopedObjects, it, m_Objects ){
        const CObject* p_obj = it->object;
        objects::CScope* p_scope = const_cast<objects::CScope*>(it->scope.GetPointer());
        if( !p_obj || !p_scope || objects_used.find( p_obj ) != objects_used.end() ){
            continue;
        }
        const CObject& object = *p_obj;
        objects::CScope& scope = *p_scope;

        CObjectConverter::TRelationVector relations;
        CObjectConverter::FindRelations( scope, object, type, relations );

        ITERATE( CObjectConverter::TRelationVector, itr, relations ){
            if( relations.size() > 1 && itr == relations.begin() ){
                continue;
            }

            const CRelation& rel = **itr;
            string relName = rel.GetProperty( type );

            CRelation::TObjects related;
            rel.GetRelated( scope, object, related, CRelation::fConvert_NoExpensive );

            /*
            TConstScopedObjects converts;
            ITERATE( CRelation::TObjects, rel_iter, related ){
                const CObject& rel_obj = rel_iter->GetObject();

                if( objects_used.insert( rel_obj ).second ){
            */



            if( !related.empty() ){
                TConstScopedObjects& rel_vec = m_RelationMap[relName];

                ITERATE( CRelation::TObjects, rel_itr, related ){
                    rel_vec.push_back( 
                        SConstScopedObject( rel_itr->GetObject(), scope ) 
                    );
                }
            }
        }
    }
}

static const wxChar* kAllConversions = wxT("All Conversions");


bool CRelatedObjectListCtrl::TransferDataToWindow()
{
    if( m_RelationMap.size() > 1 ){
        m_Choice1->Append( kAllConversions );

        ITERATE( TRelationMap, it, m_RelationMap ){
            m_Choice1->Append( ToWxString( it->first ) );
        }
        m_Choice1->Select(0);
        x_FillObjects( kAllConversions );
    
    } else if( m_RelationMap.size() > 0 ){
        x_FillObjects( ToWxString( m_RelationMap.begin()->first ) );
    }

    x_UpdateUI();

    return wxPanel::TransferDataToWindow();
}

/*!
* Transfer data from the window
*/

bool CRelatedObjectListCtrl::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow())
        return false;

    TConstScopedObjects selection;
    m_List1->GetSelection(m_Selection);
    if (m_Selection.empty()) {
        wxMessageBox(wxT("Please, select location."), wxT("Error"),
                     wxOK | wxICON_ERROR, this);
        m_List1->SetFocus();
        return false;
    }

    return true;
}

void CRelatedObjectListCtrl::x_FillObjects( const wxString& conv )
{
    SaveSettings();

    m_List1->Clear();

    if( conv == kAllConversions ){
        TConstScopedObjects objs;
        NON_CONST_ITERATE( TRelationMap, it, m_RelationMap ){
            objs.insert( objs.end(), it->second.begin(), it->second.end() );
        }
        m_List1->SetObjects( objs );

    } else {
        m_List1->SetObjects( m_RelationMap[ToStdString( conv )] );
    }

    LoadSettings();

    if( m_List1->GetItemCount() > 0 ){
        if( m_DoSelectAll ){
            m_List1->SelectAll();

        } else {
            m_List1->SetItemState(
                0, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED,
                wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED
            );
        }
    }
}

static const char *kListTag = "List";

void CRelatedObjectListCtrl::SaveSettings() const
{
    if( !m_RegPath.empty() ){
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadWriteView table_view;

        reg_path = CGuiRegistryUtil::MakeKey( m_RegPath, kListTag );
        table_view = gui_reg.GetReadWriteView( reg_path );
        ncbi::SaveTableSettings( *m_List1, table_view );
    }
}

void CRelatedObjectListCtrl::LoadSettings()
{
    if( !m_RegPath.empty() ){
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        string reg_path;
        CGuiRegistry::TReadView table_view;

        reg_path = CGuiRegistryUtil::MakeKey( m_RegPath, kListTag );
        table_view = gui_reg.GetReadView( reg_path );
        ncbi::LoadTableSettings( *m_List1, table_view );
    }
}

/*!
 * Should we show tooltips?
 */

bool CRelatedObjectListCtrl::ShowToolTips()
{
    return true;
}

void CRelatedObjectListCtrl::OnAccessionUpdated( wxCommandEvent& WXUNUSED(event) )
{
    x_UpdateAddBtn();
}

const string kAddedStr("Added");

void CRelatedObjectListCtrl::OnAddAccessionClick( wxCommandEvent& WXUNUSED(event) )
{
    static wxString caption(wxT("Cannot add accession"));

    if( !m_AccValidator ){
        return;
    }

    wxString acc = m_Accession->GetValue();
    acc.Trim(false);
    acc.Trim(true);

    CRef<objects::CSeq_id> id(new objects::CSeq_id());

    try {
        id->Set(ToStdString(acc));
    }
    catch (objects::CSeqIdException&) {
        wxString msg(wxT("Accession \""));
        msg += acc + wxT("\" not recognized as a valid accession.");
        wxMessageBox(msg, caption, wxOK | wxICON_ERROR);
        return;
    }

    wxString errMsg;
    CConstRef<CObject> obj = m_AccValidator->Validate(*id, *m_Scope, errMsg);
    if (!obj) {
        if (errMsg.empty()) {
            errMsg = wxT("Accession \"");
            errMsg += acc + wxT("\" doesn't not fit this list.");
        }
        wxMessageBox(errMsg, caption, wxOK | wxICON_ERROR);
        return;
    }

    wxString addedStr = ToWxString(kAddedStr);

    if( m_RelationMap.find( kAddedStr ) == m_RelationMap.end() ){
        if( m_RelationMap.size() == 1 ){
            m_Choice1->Append(kAllConversions);
            ITERATE( TRelationMap, it, m_RelationMap ){
                m_Choice1->Append(ToWxString(it->first));
            }
        }
        m_Choice1->Append(addedStr);
    }

    m_RelationMap[kAddedStr].push_back( SConstScopedObject( *obj, *m_Scope ) );

    if( m_RelationMap.size() == 2 ){
        x_UpdateUI();
        Refresh();
    }

    m_Choice1->SetStringSelection(addedStr);
    x_FillObjects(addedStr);

    int count = m_List1->GetItemCount();
    if (count > 1) {
        m_List1->SetItemState(0, 0, wxLIST_STATE_SELECTED);
        m_List1->SetItemState(count - 1, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    }
}

END_NCBI_SCOPE
