/*  $Id: text_item_default.cpp 23924 2011-06-22 15:16:46Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/objutils/interface_registry.hpp>
#include <gui/widgets/object_list/object_text_item.hpp>

#include <gui/widgets/text_widget/draw_styled_text.hpp>

#include <gui/objutils/label.hpp>

BEGIN_NCBI_SCOPE

class CTextItemDefault : public CObjectTextItem
{
public:
    static CTextItemDefault* CreateObject(SConstScopedObject& object, ICreateParams* params);
    virtual void x_RenderText(CStyledTextOStream& ostream, CTextPanelContext* context) const;

private:
    CTextItemDefault() {}
};

void initCTextItemDefault()
{
    CInterfaceRegistry::RegisterFactory(
            typeid(ITextItem).name(),
            new CObjectInterfaceFactory<CTextItemDefault>());
}

CTextItemDefault* CTextItemDefault::CreateObject(SConstScopedObject& object, ICreateParams* params)
{
    CObjectTextParams* ot_params = dynamic_cast<CObjectTextParams*>(params);
    CObjectTextParams::Context context = ot_params ? ot_params->GetContext() : CObjectTextParams::kBrief;

    CTextItemDefault* itemObject = new CTextItemDefault();
    itemObject->x_SetObject(object.object);
    itemObject->x_SetScope(object.scope);
    itemObject->x_SetContext(context);

    return itemObject;
}

void CTextItemDefault::x_RenderText(CStyledTextOStream& ostream, CTextPanelContext* /*context*/) const
{
    x_Indent(ostream);

    string text;

    if (m_Context == CObjectTextParams::kFull) {
        CLabel::GetLabel(*m_Object, &text, CLabel::eDescription, m_Scope);
    }
    else {
        CLabel::GetLabel(*m_Object, &text, CLabel::eDescriptionBrief, m_Scope);
    }

    ostream << text;
}

END_NCBI_SCOPE
