/*  $Id: phylo_tree_widget.cpp 27937 2013-04-26 13:38:43Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladimir Tereshkov
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_widget.hpp>

#include <corelib/ncbitime.hpp>
#include <gui/widgets/wx/sys_path.hpp>

#include <gui/opengl/glfont.hpp>
#include <gui/types.hpp>
#include <gui/utils/view_event.hpp>
#include <gui/graph/igraph_utils.hpp>
#include <gui/framework/status_bar_service.hpp>
#include <gui/framework/event_log_service_impl.hpp>
#include <gui/framework/service.hpp>
//#include <gui/print/print_options.hpp>
#include <gui/widgets/wx/print_handler_wx.hpp>
#include <gui/widgets/phylo_tree/phylo_image_grabber.hpp>
#include <gui/widgets/phylo_tree/phylo_pdf_grabber.hpp>
#include <gui/widgets/wx/save_images_setup_dlg.hpp>
#include "save_tree_pdf_dlg.hpp"
#include <gui/core/commands.hpp>

#include <gui/widgets/phylo_tree/phylo_tree_rect_cladogram.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_slanted_cladogram.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_force.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_radial.hpp>
#include <gui/widgets/phylo_tree/wx_phylo_settings_dlg.hpp>
#include <gui/widgets/phylo_tree/wx_phylo_edit_dlg.hpp>
#include <gui/widgets/phylo_tree/wx_label_edit_dlg.hpp>
#include <gui/opengl/glresmgr.hpp>
#include <gui/opengl/glrender.hpp>
#include <gui/widgets/wx/message_box.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/ui_command.hpp>
#include <gui/widgets/wx/sticky_tooltip_wnd.hpp>
#include <gui/widgets/wx/fileartprov.hpp>
#include <gui/objutils/biotree_attr_reader.hpp>

#include <gui/widgets/wx/message_box.hpp>
#include <gui/widgets/wx/unfocused_scrollbar.hpp>
#include <gui/widgets/wx/fileartprov.hpp>

#include <util/image/image.hpp>
#include <util/image/image_io.hpp>

#include <wx/settings.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/textctrl.h>
#include <wx/tglbtn.h>
#include <wx/printdlg.h>
#include <wx/cmndata.h>

#include <list>

#include <wx/menu.h>

BEGIN_NCBI_SCOPE

////////////////////////////////////////////////////////////////////////////////
/// class CPhyloTreeWidget
void CPhyloTreeWidget::RegisterCommands(CUICommandRegistry& cmd_reg,
                                         wxFileArtProvider& /* provider */)
{
    static bool initialized = false;
    if (initialized)
        return;
    initialized = true;

    cmd_reg.RegisterCommand(eCmdEditLabel, "Edit Label", "Edit Label", "", "");
    cmd_reg.RegisterCommand(eCmdEditNode, "Properties", "Properties", "", "");

    cmd_reg.RegisterCommand(eCmdNodeCut, "Cut", "Cut", "", "");
    cmd_reg.RegisterCommand(eCmdNodePaste, "Paste", "Paste", "", "");
    cmd_reg.RegisterCommand(eCmdMoveUp, "Move Up", "Move Up", "", "");
    cmd_reg.RegisterCommand(eCmdMoveDown, "Move Down", "Move Down", "", "");

    cmd_reg.RegisterCommand(eCmdNodeNewChild, "New Child", "New Child", "", "");
    cmd_reg.RegisterCommand(eCmdNodeNewParent, "New Parent", "New Parent", "", "");

    cmd_reg.RegisterCommand(eCmdRemoveSubtree, "Subtree", "Subtree", "", "");
    cmd_reg.RegisterCommand(eCmdRemoveNode, "Node", "Node", "", "");

    cmd_reg.RegisterCommand(eCmdExportSelected, "Save Selected to Newick file...", "Save Selected to Newick file...", "", "");
    cmd_reg.RegisterCommand(eCmdFilter, "Display the subtree", "Display the subtree", "", "");
    cmd_reg.RegisterCommand(eCmdNewViewFromSelection, "Create a new copy of the subtree", "Create a new copy of the subtree", "", "");
    cmd_reg.RegisterCommand(eCmdSort, "Sort Tree", "Sort Tree", "", "");

    cmd_reg.RegisterCommand(eCmdSetGraphType1,
        "Rectangle Cladogram", "Rectangle Cladogram", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdSetGraphType2,
        "Slanted Cladogram", "Slanted Cladogram", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdSetGraphType3,
        "Radial Tree", "Radial Tree", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdSetGraphType4,
        "Force Layout", "Force Layout", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdUseDistances,
        "Use Distances", "Use Distances", "", "", "", "", wxITEM_CHECK);

    cmd_reg.RegisterCommand(eCmdZoomXY,
        "Proportional", "Proportional", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdZoomY,
        "Vertical", "Vertical", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdZoomX,
        "Horizontal", "Horizontal", "", "", "", "", wxITEM_RADIO);

    cmd_reg.RegisterCommand(eCmdRerootTree, "Make this Root", "Make this Root", "", "");
    cmd_reg.RegisterCommand(eCmdZoomToSubtree,
        "Re-root and show only child nodes", "Re-root and show only child nodes", "menu::zoom_all", "");


    cmd_reg.RegisterCommand(eCmdLabelsForLeavesOnly,
        "For Leaves Only", "For Leaves Only", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdLabelsVisible,
        "Show All", "Show All", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdLabelsHidden,
        "Hide All", "Hide All", "", "", "", "", wxITEM_RADIO);

    cmd_reg.RegisterCommand(eCmdAlNone, "None", "None", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdAlPhylip, "Phylip-Style", "Phylip-Style", "", "", "", "", wxITEM_RADIO);
    cmd_reg.RegisterCommand(eCmdAlPaml, "PAML-Style", "PAML-Style", "", "", "", "", wxITEM_RADIO);

    cmd_reg.RegisterCommand(eCmdShowAll, "Expand All", "Expand All", "", "");
    cmd_reg.RegisterCommand(eCmdChildsCollapse, "Collapse", "Collapse", "", "");
    cmd_reg.RegisterCommand(eCmdChildsExpand, "Expand", "Expand", "", "");
    cmd_reg.RegisterCommand(eCmdHighlightCollapsed, "Highlight Collapsed\tCTRL-L", "Highlight Collapsed", "", "", "", "", wxITEM_CHECK);

    cmd_reg.RegisterCommand(eSelectAll, "Select All", "Select All", "", "");
    cmd_reg.RegisterCommand(eClearSelection, "Clear Selection", "Clear Selection", "", "");

    cmd_reg.RegisterCommand(eCmdClean, "Remove single child nodes", "Remove single child nodes", "", "");
    cmd_reg.RegisterCommand(eCmdFilterDistances, "Distance Filtering...", "Distance Filtering...", "", "");

    cmd_reg.RegisterCommand(eCmdUseSplines, "Use Splines", "Use Splines", "", "", "", "", wxITEM_CHECK);

    cmd_reg.RegisterCommand(eCmdMySettings, "Settings...", "Settings...", "", "");

    cmd_reg.RegisterCommand(eCmdLoadAttributes, "Load Attributes...", "Attributes...", "", "");
}

CPhyloTreeWidget::CPhyloTreeWidget(wxWindow* parent, wxWindowID id,
                                   const wxPoint& pos, const wxSize& size, long style,
                                   const wxString& name)
: CGlWidgetBase(parent, id, pos, size, style, name)
, m_pDataSource(NULL)
, m_pMatrixPane(NULL)
, m_ServiceLocator(NULL)
, m_ParentPanel(NULL)
, m_QueryPanel(NULL)
{
    // setup Port
    m_Port.SetAdjustmentPolicy(CGlPane::fAdjustAll, CGlPane::fAdjustAll);
    m_Port.SetMinScaleX(1 / 1000.0);
    m_Port.SetMinScaleY(1 / 1000.0);    
    m_Port.SetOriginType(CGlPane::eOriginLeft, CGlPane::eOriginBottom);
    m_Port.EnableZoom(true, true);
}

CPhyloTreeWidget::~CPhyloTreeWidget()
{
    delete m_QueryPanel;
    m_QueryPanel = NULL;
}

static
WX_DEFINE_MENU(sPopupMenu)
    WX_MENU_ITEM(eCmdEditLabel)
    WX_MENU_ITEM(eCmdEditNode)
    WX_SUBMENU("Edit")
        WX_SUBMENU("Move")
            WX_MENU_ITEM(eCmdNodeCut)
            WX_MENU_ITEM(eCmdNodePaste)
            WX_MENU_SEPARATOR()
            WX_MENU_ITEM(eCmdMoveUp)
            WX_MENU_ITEM(eCmdMoveDown)
        WX_END_SUBMENU()
        WX_SUBMENU("Add")
            WX_MENU_ITEM(eCmdNodeNewChild)
            WX_MENU_ITEM(eCmdNodeNewParent)
        WX_END_SUBMENU()
        WX_SUBMENU("Remove")
            WX_MENU_ITEM(eCmdRemoveSubtree)
            WX_MENU_ITEM(eCmdRemoveNode)
        WX_END_SUBMENU()
    WX_END_SUBMENU()

    //WX_MENU_ITEM(eCmdExportSelected)
    // WX_MENU_ITEM(eCmdFilter)
      WX_MENU_ITEM(eCmdNewViewFromSelection)
    WX_MENU_ITEM(eCmdSort)

    WX_MENU_SEPARATOR()
    WX_SUBMENU("Layout")
        WX_MENU_ITEM(eCmdSetGraphType1)
        WX_MENU_ITEM(eCmdSetGraphType2)
        WX_MENU_ITEM(eCmdSetGraphType3)
        WX_MENU_ITEM(eCmdSetGraphType4)
        WX_MENU_SEPARATOR()
        WX_MENU_ITEM(eCmdUseDistances)
    WX_END_SUBMENU()

    WX_SUBMENU("Zoom Behavior")
        WX_MENU_ITEM(eCmdZoomXY)
        WX_MENU_ITEM(eCmdZoomY)
        WX_MENU_ITEM(eCmdZoomX)
    WX_END_SUBMENU()

    WX_SUBMENU("Re-root")
        WX_MENU_ITEM(eCmdRerootTree)
        WX_MENU_ITEM(eCmdZoomToSubtree)
    WX_END_SUBMENU()

    WX_SUBMENU("Label Visibility")
        WX_MENU_ITEM(eCmdLabelsForLeavesOnly)
        WX_MENU_ITEM(eCmdLabelsVisible)
        WX_MENU_ITEM(eCmdLabelsHidden)
        WX_MENU_SEPARATOR()
        WX_SUBMENU("Auto-Labeling")
            WX_MENU_ITEM(eCmdAlNone)
            WX_MENU_ITEM(eCmdAlPhylip)
            WX_MENU_ITEM(eCmdAlPaml)
        WX_END_SUBMENU()
    WX_END_SUBMENU()

    WX_MENU_SEPARATOR()

    WX_MENU_ITEM(eCmdShowAll)
    WX_MENU_ITEM(eCmdChildsCollapse)
    WX_MENU_ITEM(eCmdChildsExpand)
    WX_MENU_ITEM(eCmdHighlightCollapsed)

    WX_MENU_SEPARATOR()

    WX_MENU_ITEM(eSelectAll)
    WX_MENU_ITEM(eClearSelection)

    WX_MENU_ITEM(eCmdClean)

    WX_MENU_ITEM(eCmdUseSplines)
    WX_MENU_SEPARATOR()
    WX_MENU_ITEM(eCmdMySettings)
    WX_MENU_SEPARATOR()
    WX_MENU_ITEM(eCmdLoadAttributes)

WX_END_MENU()

BEGIN_EVENT_TABLE(CPhyloTreeWidget, CGlWidgetBase)
    EVT_MENU(wxID_PRINT, CPhyloTreeWidget::OnPrint)
    EVT_UPDATE_UI(wxID_PRINT, CPhyloTreeWidget::OnEnablePrintCmdUpdate)
    EVT_MENU(eCmdSaveImages, CPhyloTreeWidget::OnSaveImages)
    EVT_UPDATE_UI(eCmdSaveImages, CPhyloTreeWidget::OnEnableSaveImagesCmdUpdate)
    EVT_MENU(eCmdSavePdf, CPhyloTreeWidget::OnSavePdf)
    EVT_UPDATE_UI(eCmdSavePdf, CPhyloTreeWidget::OnEnableSavePdfCmdUpdate)

    EVT_CONTEXT_MENU(CPhyloTreeWidget::OnContextMenu)

    EVT_MENU(eCmdSetGraphType1, CPhyloTreeWidget::OnSetGraphType1)
    EVT_MENU(eCmdSetGraphType2, CPhyloTreeWidget::OnSetGraphType2)
    EVT_MENU(eCmdSetGraphType3, CPhyloTreeWidget::OnSetGraphType3)
    EVT_MENU(eCmdSetGraphType4, CPhyloTreeWidget::OnSetGraphType4)
    EVT_MENU(eCmdUseDistances,  CPhyloTreeWidget::OnUseDistances)
    EVT_MENU(eCmdUseSplines,    CPhyloTreeWidget::OnUseSplines)
    EVT_MENU(eCmdMySettings,    CPhyloTreeWidget::OnOpenPropertiesDlg)
    EVT_MENU(eCmdRerootTree,    CPhyloTreeWidget::OnRerootTree)
    EVT_MENU(eCmdLoadAttributes,CPhyloTreeWidget::OnLoadAttributes)

    EVT_MENU(eCmdEditNode,      CPhyloTreeWidget::OnEditNode)
    EVT_UPDATE_UI(eCmdEditNode, CPhyloTreeWidget::OnUpdateEditNode)

    EVT_MENU(eCmdFilterDistances,   CPhyloTreeWidget::OnFilterDistances)

    EVT_MENU(eSelectAll,        CPhyloTreeWidget::OnSelectAll)
    EVT_MENU(eClearSelection,   CPhyloTreeWidget::OnClearSelection)
    EVT_MENU(eInverseSelection, CPhyloTreeWidget::OnInverseSelection)

    EVT_MENU(eCmdChildsCollapse,  CPhyloTreeWidget::OnChildsCollapse)
    EVT_MENU(eCmdChildsExpand,    CPhyloTreeWidget::OnChildsExpand)
    EVT_MENU(eCmdHighlightCollapsed, CPhyloTreeWidget::OnHightlightCollapsed)

    EVT_MENU(eCmdZoomToSubtree,    CPhyloTreeWidget::OnZoomToSubtree)
    EVT_UPDATE_UI(eCmdZoomToSubtree,    CPhyloTreeWidget::OnUpdateZoomToSubtree)

    EVT_MENU(eCmdShowAll,    CPhyloTreeWidget::OnShowAll)
    EVT_UPDATE_UI(eCmdShowAll,   CPhyloTreeWidget::OnUpdateShowAll)

    EVT_MENU(eCmdSimplify,   CPhyloTreeWidget::OnSimplify)
    EVT_UPDATE_UI(eCmdSimplify,   CPhyloTreeWidget::OnUpdateSimplify)

    EVT_UPDATE_UI(eCmdChildsCollapse,  CPhyloTreeWidget::OnUpdateChildsCollapse)
    EVT_UPDATE_UI(eCmdChildsExpand,    CPhyloTreeWidget::OnUpdateChildsExpand)

    EVT_UPDATE_UI(eCmdFilter,        CPhyloTreeWidget::OnUpdateFilter)

    EVT_UPDATE_UI(eCmdRerootTree,    CPhyloTreeWidget::OnUpdateRerootTree)
    EVT_UPDATE_UI(eCmdUseDistances,  CPhyloTreeWidget::OnUpdateUseDistances)
    EVT_UPDATE_UI(eCmdUseSplines,    CPhyloTreeWidget::OnUpdateUseSplines)
    EVT_UPDATE_UI(eCmdSetGraphType1, CPhyloTreeWidget::OnUpdateSetGraphType1)
    EVT_UPDATE_UI(eCmdSetGraphType2, CPhyloTreeWidget::OnUpdateSetGraphType2)
    EVT_UPDATE_UI(eCmdSetGraphType3, CPhyloTreeWidget::OnUpdateSetGraphType3)
    EVT_UPDATE_UI(eCmdSetGraphType4, CPhyloTreeWidget::OnUpdateSetGraphType4)

    EVT_MENU(eCmdZoomXY,       CPhyloTreeWidget::OnZoomXY)
    EVT_UPDATE_UI(eCmdZoomXY,  CPhyloTreeWidget::OnUpdateZoomXY)
    EVT_MENU(eCmdZoomX,        CPhyloTreeWidget::OnZoomX)
    EVT_UPDATE_UI(eCmdZoomX,   CPhyloTreeWidget::OnUpdateZoomX)
    EVT_MENU(eCmdZoomY,        CPhyloTreeWidget::OnZoomY)
    EVT_UPDATE_UI(eCmdZoomY,   CPhyloTreeWidget::OnUpdateZoomY)
    EVT_MENU(eCmdZoomSel,      CPhyloTreeWidget::OnZoomToSelection)
    EVT_MENU(eCmdBack,         CPhyloTreeWidget::OnGoBack)
    EVT_MENU(eCmdForward,      CPhyloTreeWidget::OnGoForward)
    EVT_UPDATE_UI(eCmdBack,    CPhyloTreeWidget::OnUpdateGoBack)
    EVT_UPDATE_UI(eCmdForward, CPhyloTreeWidget::OnUpdateGoForward)
    EVT_UPDATE_UI(eCmdZoomSel, CPhyloTreeWidget::OnUpdateZoomSel)
    EVT_UPDATE_UI(eCmdZoomSeq, CPhyloTreeWidget::OnUpdateZoomSequence)

    EVT_MENU(eCmdSort,      CPhyloTreeWidget::OnSort)
    EVT_MENU(eCmdClean,     CPhyloTreeWidget::OnClean)
    EVT_MENU(eCmdFilter,    CPhyloTreeWidget::OnFilter)

    EVT_MENU_RANGE(eCmdLabelsForLeavesOnly, eCmdLabelsHidden,      CPhyloTreeWidget::OnLabels)
    EVT_UPDATE_UI_RANGE(eCmdLabelsForLeavesOnly, eCmdLabelsHidden, CPhyloTreeWidget::OnUpdateLabels)

    EVT_MENU_RANGE(eCmdAlNone, eCmdAlPaml,      CPhyloTreeWidget::OnAl)
    EVT_UPDATE_UI_RANGE(eCmdAlNone, eCmdAlPaml, CPhyloTreeWidget::OnUpdateAl)

    EVT_MENU_RANGE(eCmdNodeCopy, eCmdRemoveNode,      CPhyloTreeWidget::OnEdit)
    EVT_UPDATE_UI_RANGE(eCmdNodeCopy, eCmdRemoveNode, CPhyloTreeWidget::OnUpdateEdit)

    EVT_MENU(eCmdEditLabel,      CPhyloTreeWidget::OnEditLabel)
    EVT_UPDATE_UI(eCmdEditLabel, CPhyloTreeWidget::OnUpdateEditLabel)

	// subtree export
	EVT_MENU(eCmdExportSelected,          CPhyloTreeWidget::OnExportSelected)
	EVT_MENU(eCmdExportSelectedNew,       CPhyloTreeWidget::OnExportSelectedNew)
    EVT_MENU(eCmdNewViewFromSelection,    CPhyloTreeWidget::OnNewViewFromSelection)

    EVT_UPDATE_UI(eCmdExportSelected,        CPhyloTreeWidget::OnUpdateExportSelected)
	EVT_UPDATE_UI(eCmdExportSelectedNew,     CPhyloTreeWidget::OnUpdateExportSelected)
    EVT_UPDATE_UI(eCmdNewViewFromSelection,  CPhyloTreeWidget::OnUpdateExportSelected)

    EVT_BUTTON(eCmdSearchTip, CPhyloTreeWidget::OnSearchTip)
    EVT_BUTTON(eCmdZoomTip, CPhyloTreeWidget::OnZoomTip)
    EVT_BUTTON(eCmdInfoTip, CPhyloTreeWidget::OnInfoTip)
    EVT_BUTTON(eCmdTipActive, CPhyloTreeWidget::OnTipActivated)
    EVT_BUTTON(eCmdTipInactive, CPhyloTreeWidget::OnTipDeactivated)

END_EVENT_TABLE()

void CPhyloTreeWidget::x_CreateControls()
{
    // Make sure we have loaded needed bitmaps:
    static bool first_tree_window = false;
    if( !first_tree_window ){
        wxFileArtProvider* provider = GetDefaultFileArtProvider();
        first_tree_window = true;

        provider->RegisterFileAlias(wxT("menu::dm_start"), wxT("playhs.png"));
        provider->RegisterFileAlias(wxT("menu::dm_stop"),  wxT("stophs.png"));
        provider->RegisterFileAlias(wxT("menu::back"), wxT("back.png"));
        provider->RegisterFileAlias(wxT("menu::forward"), wxT("forward.png"));
        provider->RegisterFileAlias(wxT("menu::help"), wxT("help.png"));
    }


    _ASSERT( ! x_GetPane());

    m_ParentPanel = new wxPanel(this);

    x_CreatePane();

    CGlWidgetPane* child_pane = x_GetPane();
    _ASSERT(child_pane);

    wxBoxSizer* box_sizer = new wxBoxSizer(wxVERTICAL);
    m_ParentPanel->SetSizer(box_sizer);
    
    wxFlexGridSizer* sizer = new wxFlexGridSizer(2, 2, 0, 0);
    sizer->AddGrowableCol(0);
    sizer->AddGrowableRow(0);

    m_QueryPanel = new CQueryParsePanel(this, m_pDataSource.GetPointerOrNull());
    m_QueryPanel->Create(m_ParentPanel);
    m_QueryPanel->SetServiceLocator(m_ServiceLocator);

    box_sizer->Add(m_QueryPanel, 0, wxEXPAND);
    sizer->Add(child_pane, 1, wxEXPAND);

    wxScrollBar*
    vsb = new CUnfocusedScrollbar(m_ParentPanel, ID_VSCROPLLBAR, wxDefaultPosition,
                                  wxDefaultSize, wxSB_VERTICAL);
    sizer->Add(vsb, 0, wxEXPAND);

    wxScrollBar*
    hsb = new CUnfocusedScrollbar(m_ParentPanel, ID_HSCROPLLBAR, wxDefaultPosition,
                                  wxDefaultSize, wxSB_HORIZONTAL);
    sizer->Add(hsb, 0, wxEXPAND);

    vsb->SetMinSize(wxSize(wxDefaultCoord, 4));
    hsb->SetMinSize(wxSize(4, wxDefaultCoord));

    sizer->AddSpacer(1);
    box_sizer->Add(sizer, 1, wxEXPAND);

    wxBoxSizer* frame_sizer = new wxBoxSizer(wxHORIZONTAL);
    frame_sizer->Add(m_ParentPanel, 1, wxALL|wxGROW, 0);
    SetSizerAndFit(frame_sizer);
}

void CPhyloTreeWidget::OnContextMenu(wxContextMenuEvent& event)
{
    wxPoint ms_pos = ScreenToClient(event.GetPosition());

    m_PopupPoint.m_X = m_pMatrixPane->SHH_GetModelByWindow(ms_pos.x, eHorz);
    m_PopupPoint.m_Y = m_pMatrixPane->SHH_GetModelByWindow(ms_pos.y, eVert);

    CUICommandRegistry& cmd_reg = CUICommandRegistry::GetInstance();
    auto_ptr<wxMenu> menu(cmd_reg.CreateMenu(sPopupMenu));
    PopupMenu(menu.get());
}


void  CPhyloTreeWidget::SetPopupMenuItems(CMenuItem * itm)
{
    m_pPopupItems = itm;
}

void CPhyloTreeWidget::x_CreatePane()
{
    m_pMatrixPane = new CPhyloTreePane(this);

    // defauld model space for renderer
    double w = 2000.;
    double h = 2000.;
    
    m_pMatrixPane->AddRenderer(new CPhyloRectCladogram(w, h));
    m_pMatrixPane->AddRenderer(new CPhyloSlantedCladogram(w, h));
    m_pMatrixPane->AddRenderer(new CPhyloRadial(w, h));
    m_pMatrixPane->AddRenderer(new CPhyloForce(w, h));
}


CGlWidgetPane*    CPhyloTreeWidget::x_GetPane()
{
    return static_cast<CGlWidgetPane*>(m_pMatrixPane);
}


void CPhyloTreeWidget::x_SetPortLimits()
{
    if (m_pMatrixPane) {
        double DimX = m_pMatrixPane->GetCurrRenderer()->GetDimX();
        double DimY = m_pMatrixPane->GetCurrRenderer()->GetDimY();
        m_Port.SetModelLimitsRect(TModelRect(0, 0, DimX, DimY));
    }
}

void   CPhyloTreeWidget::SetPortLimits(const TModelRect & rect,  bool bZoomAll)
{
    //m_pDataSource->Refresh();
    const TModelRect mr = m_Port.GetModelLimitsRect();
    m_Port.SetModelLimitsRect(rect);

    double scale_factor = std::max(100.0, (double)(m_pDataSource->GetNumNodes()/2));

    m_Port.SetMinScaleX(m_Port.GetZoomAllScaleX() / scale_factor);
    m_Port.SetMinScaleY(m_Port.GetZoomAllScaleY() / scale_factor);

    if (bZoomAll) {
        m_Port.ZoomAll();
    }
    else {
        TModelUnit sx = (rect.Width()  / mr.Width());
        TModelUnit sy = (rect.Height() / mr.Height());

        m_Port.SetScale(m_Port.GetScaleX() * sx, m_Port.GetScaleY() * sy);
        x_UpdateOnZoom();
    }
}

void CPhyloTreeWidget::SetDataSource(CPhyloTreeDataSource* p_ds)
{   
    if (m_QueryPanel != NULL) {
        m_QueryPanel->SetDataSource(p_ds);
    }

    double scale_factor = std::max(100.0, (double)(p_ds->GetNumNodes()/2));
    m_Port.SetMinScaleX(m_Port.GetZoomAllScaleX() / scale_factor);
    m_Port.SetMinScaleY(m_Port.GetZoomAllScaleY() / scale_factor);   

    CRef<CGlRender> rmgrgl = CGlResMgr::Instance().GetCurrentRenderer();
    //rmgrgl->SetModel(p_ds->GetModel());

    if (!m_pDataSource){   
        m_pDataSource = p_ds;        
        x_Update();
    }
    else {    
        m_pDataSource = p_ds;
        x_SoftUpdate();
    }
}

void CPhyloTreeWidget::SetDataSourceNoUpdate(CPhyloTreeDataSource* p_ds)
{
    m_pDataSource = p_ds;

    m_pMatrixPane->UpdateDataSource();

    double scale_factor = std::max(100.0, (double)(p_ds->GetNumNodes()/2));
    m_Port.SetMinScaleX(m_Port.GetZoomAllScaleX() / scale_factor);
    m_Port.SetMinScaleY(m_Port.GetZoomAllScaleY() / scale_factor); 

    if (m_QueryPanel != NULL) {
        m_QueryPanel->SetDataSource(p_ds);
    }
}

void CPhyloTreeWidget::RemoveCurrentDataSource()
{
    m_pDataSource = NULL;

    if (m_pMatrixPane) {
        m_pMatrixPane->RemoveCurrentDataSource();
    }

    if (m_QueryPanel != NULL) {
        m_QueryPanel->SetDataSource(NULL);
    }
}


CPhyloTreeDataSource*  CPhyloTreeWidget::GetDS(void)
{
    return m_pDataSource;
}


CGlPane& CPhyloTreeWidget::GetPort(void)
{
    return m_Port;
}


const CGlPane& CPhyloTreeWidget::GetPort(void) const
{
    return m_Port;
}

void    CPhyloTreeWidget::OnPrint(wxCommandEvent& /* evt */)
{
    m_pMatrixPane->SetContext();

    size_t w = m_pMatrixPane->GetPane().GetViewport().Width();
    size_t h = m_pMatrixPane->GetPane().GetViewport().Height();
    
    // create the CImage
    CRef<CImage> img(new CImage(w, h, 3));

    //_TRACE("Size (w,h,buf): (" << w << ", " << h << ", " << image_size << ")");

    // Read using single-byte alignment
    GLint alignment;
    glGetIntegerv(GL_PACK_ALIGNMENT, &alignment);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glReadPixels(0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, img->SetData());
    glPixelStorei(GL_PACK_ALIGNMENT, alignment);
    img->Flip();
    
    //wxImage wximg(img->GetWidth(),
    //              img->GetHeight(), 
    //             (unsigned char*)img->GetData(), true);
    //wximg.SaveFile(wxT("c:\\temp\\image_save1.bmp"),wxBITMAP_TYPE_BMP);

    CPrintHanderWx  print_handler(img,
                                  1 /*1 page */,
                                  wxT("Print Test"),
                                  30 /* units per centimeter*/);
   
    print_handler.SetImage(img);


    //QuickPrint*  myprint = new QuickPrint( 5 /* 5 pages */,
    //                                       wxT("wxPrint test"),
    //                                       30 /* 30 units per centimeter */ );
    if (!print_handler.performPageSetup(true))
    {
        // user cancelled
        return;
    }
    
    wxPrintDialogData data(print_handler.getPrintData());
    wxPrinter printer(&data);
    const bool success = printer.Print(NULL, &print_handler, true /* show dialog */);
  
    
    if (!success)
    {
        std::cerr << "Failed!!\n";
        return;
    }
}

void CPhyloTreeWidget::OnEnablePrintCmdUpdate(wxUpdateUIEvent& event)
{
    // disable until print works correctly.
    event.Enable(false);
}

void CPhyloTreeWidget::OnSaveImages(wxCommandEvent& /* evt */)
{
    m_pMatrixPane->SetContext(); 

    size_t w = m_pMatrixPane->GetPane().GetViewport().Width();
    size_t h = m_pMatrixPane->GetPane().GetViewport().Height();
    
    // create the CImage
    CRef<CImage> img(new CImage(w, h, 3));

    // Read using single-byte alignment
    glReadBuffer(GL_FRONT);
    GLint alignment;
    glGetIntegerv(GL_PACK_ALIGNMENT, &alignment);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glReadPixels(0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, img->SetData());
    glPixelStorei(GL_PACK_ALIGNMENT, alignment);
    glReadBuffer(GL_BACK);
    //img->Flip();

    m_pMatrixPane->GetPane().SetModelLimitsRect(m_Port.GetModelLimitsRect());
    CPhyloImageGrabber grb(*m_pMatrixPane);
    CSaveImagesSetupDlg dlg(img, &grb, this);
    dlg.ShowModal();
}

void CPhyloTreeWidget::OnEnableSaveImagesCmdUpdate(wxUpdateUIEvent& event)
{
    event.Enable(true);
}

void    CPhyloTreeWidget::OnSavePdf(wxCommandEvent & evt)
{
    CSaveTreePdfDlg  tree_pdf_dlg(m_pMatrixPane, this);
    tree_pdf_dlg.ShowModal();
}

void    CPhyloTreeWidget::OnEnableSavePdfCmdUpdate(wxUpdateUIEvent& evt)
{
    evt.Enable(true);
}

void    CPhyloTreeWidget::OnSetEqualScale(wxCommandEvent& /* evt */)
{
    TModelUnit scale = min(m_Port.GetScaleX(), m_Port.GetScaleY());
    m_Port.SetScale(scale, scale);
    x_UpdateOnZoom();
}

// zoom behavior
void CPhyloTreeWidget::OnZoomXY(wxCommandEvent& /* evt */) 
{
    m_pScheme->SetZoomBehavior(CPhyloTreeScheme::eZoomXY);
    CGlPane& port = GetPort();

    /// With proportional zoom, visible region must be in 
    /// same proportion of limits rect, so if it isn't, update
    /// the scale to be equal be reducing the larger dimension.
    TModelRect v = port.GetVisibleRect();
    TModelRect l = port.GetModelLimitsRect();
    TModelRect new_rect = v;

    double limits_ratio = l.Width()/l.Height();
    double visible_ratio = v.Width()/v.Height();
    
    if (visible_ratio == limits_ratio) {
        return; 
    }
    else if (visible_ratio < limits_ratio) {
        double target_width = l.Width()*(v.Height()/l.Height());
        double center = (v.Left()+v.Right())/2.0;
        new_rect.SetLeft(center - target_width/2.0);
        new_rect.SetRight(center + target_width/2.0);       
    }
    else if (visible_ratio > limits_ratio) {
        double target_height = l.Height()*(v.Width()/l.Width());
        double center = (v.Top()+v.Bottom())/2.0;
        new_rect.SetBottom(center - target_height/2.0);
        new_rect.SetTop(center + target_height/2.0);
    }
    
    port.ZoomRect(new_rect);
    m_pMatrixPane->GetPane() = port;
    x_UpdateOnZoom();

    x_SoftUpdate();
    Refresh();
}

void CPhyloTreeWidget::OnZoomX(wxCommandEvent& /* evt */)  
{
    m_pScheme->SetZoomBehavior(CPhyloTreeScheme::eZoomX);
    x_UpdateOnZoom();
    x_SoftUpdate();
    Refresh();
}

void CPhyloTreeWidget::OnZoomY(wxCommandEvent& /* evt */)  
{
    m_pScheme->SetZoomBehavior(CPhyloTreeScheme::eZoomY);
    x_SoftUpdate();
    Refresh();
}

void CPhyloTreeWidget::OnZoomToSelection(wxCommandEvent& /* evt */)
{
    std::vector<CPhyloTreeNode*> nodes = m_pDataSource->GetSelectedNodes();

    if (nodes.size() > 0) {
        CVect2<float> min_pos = numeric_limits<float>::max();
        CVect2<float> max_pos = numeric_limits<float>::min();
        TModelRect sel_rect;       

        for (size_t i=0; i<nodes.size(); ++i) {
            min_pos.X() = std::min(min_pos.X(), nodes[i]->GetValue().X());
            min_pos.Y() = std::min(min_pos.Y(), nodes[i]->GetValue().Y());

            max_pos.X() = std::max(max_pos.X(), nodes[i]->GetValue().X());
            max_pos.Y() = std::max(max_pos.Y(), nodes[i]->GetValue().Y());
        }
       
        sel_rect.Init(min_pos.X(), min_pos.Y(), max_pos.X(), max_pos.Y());
        ZoomRect(sel_rect);
        x_UpdateOnZoom();
    }
}

void CPhyloTreeWidget::OnGoBack(wxCommandEvent& /*evt*/)
{
    m_pMatrixPane->GoBack();
    x_UpdateOnZoom();
}

void CPhyloTreeWidget::OnGoForward(wxCommandEvent& /*evt*/)
{
    m_pMatrixPane->GoForward();
    x_UpdateOnZoom();
}

void CPhyloTreeWidget::OnUpdateGoBack(wxUpdateUIEvent& event)
{
    event.Enable(m_pMatrixPane->CanGoBack());
}

void CPhyloTreeWidget::OnUpdateGoForward(wxUpdateUIEvent& event)
{
    event.Enable(m_pMatrixPane->CanGoForward());
}

void CPhyloTreeWidget::OnUpdateZoomXY(wxUpdateUIEvent& evt)
{
    evt.Check(m_pScheme->GetZoomBehavior()==CPhyloTreeScheme::eZoomXY);
}

void CPhyloTreeWidget::OnUpdateZoomX(wxUpdateUIEvent& evt)
{
    evt.Check(m_pScheme->GetZoomBehavior()==CPhyloTreeScheme::eZoomX);
}

void CPhyloTreeWidget::OnUpdateZoomY(wxUpdateUIEvent& evt)
{
    evt.Check(m_pScheme->GetZoomBehavior()==CPhyloTreeScheme::eZoomY);
}

void CPhyloTreeWidget::OnUpdateZoomSel(wxUpdateUIEvent& evt)
{
    // only allow selection zoom if one or more nodes are selected
    if ( m_pDataSource->GetSelectedNodes().size() > 0)
        evt.Enable(true);
    else
        evt.Enable(false);

}

void CPhyloTreeWidget::OnUpdateZoomSequence(wxUpdateUIEvent& evt)
{
    // sequence zoom does not make sense for a tree, so disable it
    evt.Enable(false);
}

void CPhyloTreeWidget::OnSetGraphType1(wxCommandEvent& /* evt */)
{
    SetCurrRenderer(0);
}


void CPhyloTreeWidget::OnSetGraphType2(wxCommandEvent& /* evt */)
{
    SetCurrRenderer(1);
}


void CPhyloTreeWidget::OnSetGraphType3(wxCommandEvent& /* evt */)
{
    SetCurrRenderer(2);
}


void CPhyloTreeWidget::OnSetGraphType4(wxCommandEvent& /* evt */)
{
    SetCurrRenderer(3);
}


void CPhyloTreeWidget::OnUpdateSetGraphType1(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRendererIdx()==0);
}


void CPhyloTreeWidget::OnUpdateSetGraphType2(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRendererIdx()==1);
}


void CPhyloTreeWidget::OnUpdateSetGraphType3(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRendererIdx()==2);
}


void CPhyloTreeWidget::OnUpdateSetGraphType4(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRendererIdx()==3);
}


void CPhyloTreeWidget::OnUseDistances(wxCommandEvent& /* evt */)
{
   vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
   if (!vRend.empty()) {      
       SetUseDistances(!(*vRend.begin())->GetDistRendering());
       x_SoftUpdate();
   }
}


void CPhyloTreeWidget::SetUseDistances(bool bDist)
{
   vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
   ITERATE(vector <IPhyloTreeRenderer*>, it, vRend){
     (*it)->SetDistRendering(bDist && (*it)->SupportsDistanceRendering());
   }
   m_pDataSource->MarkAllDirty();
}


void CPhyloTreeWidget::OnUpdateUseDistances(wxUpdateUIEvent& evt)
{
    if (m_pMatrixPane->GetCurrRenderer() != NULL) {
        evt.Enable(m_pMatrixPane->GetCurrRenderer()->SupportsDistanceRendering());
        evt.Check(m_pMatrixPane->GetCurrRenderer()->GetDistRendering());
        return;
    }

    vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
    if (!vRend.empty()){
        evt.Check((*vRend.begin())->GetDistRendering());
    }
}

void CPhyloTreeWidget::OnUseSplines(wxCommandEvent& /* evt */)
{
   vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
   ITERATE(vector <IPhyloTreeRenderer*>, it, vRend){
     (*it)->SetSplinesRendering(!(*it)->GetSplinesRendering());
   }
   x_SoftUpdate();
}


void CPhyloTreeWidget::OnUpdateUseSplines(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRenderer()->GetSplinesRendering());
}

void CPhyloTreeWidget::UpdateViewingArea()
{
    x_UpdateScrollbars();

    /// notify our parent that we've changed
    CEvent evt(CEvent::eEvent_Message, CViewEvent::eWidgetRangeChanged);
    Send(&evt, ePool_Parent);
}

// zoom preserving update
void CPhyloTreeWidget::x_SoftUpdate()
{
     m_pMatrixPane->SoftUpdate();
}

void CPhyloTreeWidget::x_SaveStates()
{
    if (m_pMatrixPane) {
        m_pMatrixPane->SaveCurrentView();
    }
}

void CPhyloTreeWidget::x_Update()
{
    x_SetPortLimits();    
    x_UpdatePane();
    x_UpdateScrollbars();
    x_RedrawControls();
}


void CPhyloTreeWidget::x_UpdatePane()
{
    if (m_pMatrixPane) {
        m_pMatrixPane->Update();
    }
}


void CPhyloTreeWidget::OnSetScaleXY(TModelUnit scale_x, const TModelPoint& point)
{
    double ratio = m_Port.GetScaleY()/m_Port.GetScaleX();

    CPhyloTreeScheme::TZoomBehavior zb =
        m_pScheme->GetZoomBehavior();

    TModelUnit sx = scale_x;
    TModelUnit sy = scale_x;

    // handle cases of only one axes zoom
    if      (zb == CPhyloTreeScheme::eZoomX)    sy = m_Port.GetScaleY();
    else if (zb == CPhyloTreeScheme::eZoomY)    sx = m_Port.GetScaleX();
    else if (zb == CPhyloTreeScheme::eZoomXY)   sy *= ratio;

    m_Port.SetScaleRefPoint(sx, sy, point);

    x_UpdateOnZoom();
}

void CPhyloTreeWidget::OnUpdateSelChanged()
{    
    m_pMatrixPane->BufferedRender();
    m_pMatrixPane->Refresh();
}


void CPhyloTreeWidget::SetCurrRenderer(Int4 idx)
{
   m_pMatrixPane->SetFocus();
   m_pMatrixPane->SetCurrRendererIdx(idx);
   m_pDataSource->MarkAllDirty();        

   x_Update();
}


vector<string>  CPhyloTreeWidget::GetRenderersNames(void)
{
    vector<string> retValue;
    int size = m_pMatrixPane->GetRenderers().size();
    for (Int4 i=0; i<size; i++) {
        retValue.push_back(m_pMatrixPane->GetRenderers()[i]->GetDescription());
    }
    return retValue;
}


void  CPhyloTreeWidget::SendSelChangedEvent()
{
    // notify parents
    CEvent evt(CEvent::eEvent_Message, CViewEvent::eWidgetSelectionChanged);
    Send(&evt, ePool_Parent);

    m_pMatrixPane->BufferedRender();
}

void CPhyloTreeWidget::OnLoadAttributes(wxCommandEvent& /* evt */)
{
    SWFileDlgData data(wxT("Open attributes file"), wxALL_FILES_PATTERN, wxFD_OPEN);
    string error = "";
    if ( NcbiFileBrowser(data) == wxID_OK) {
        wxString filename = data.GetFilename();
        try {
            this->SetCursor(*wxHOURGLASS_CURSOR);        
            CBioTreeAttrReader::TAttrTable table;
            {{
                CBioTreeAttrReader reader;
                CNcbiIfstream is(filename.fn_str());
                reader.Read(is, table);
            }}

            m_pDataSource->ApplyAttributes(table);
            x_Update(); 
            SendEditEvent();
            //m_pDataSource->MarkAllDirty();
            //x_SoftUpdate(); 
        } catch(CException& e)  {
            error = e.GetMsg();
        } catch(std::exception& e)  {
            error = e.what();
        } 
        this->SetCursor(*wxSTANDARD_CURSOR);
    }
    if( ! error.empty())    {
        wxString s = ToWxString(error);
        wxMessageBox(s, wxT("Error loading file."), wxOK | wxICON_ERROR);
    }
}

void CPhyloTreeWidget::OnOpenPropertiesDlg(wxCommandEvent& /* evt */)
{
    // temporary scheme
    CRef <CPhyloTreeScheme> tmpScheme;
    tmpScheme.Reset(&m_pMatrixPane->GetCurrRenderer()->GetScheme());

    // show the dialog
    CwxPhyloSettingsDlg dlg(this);
    dlg.SetParams(m_pDataSource.GetPointer(), tmpScheme.GetPointer());

    if(dlg.ShowModal() == wxID_OK) {    
        SetScheme(tmpScheme.GetObject());
        m_pDataSource->Relabel(tmpScheme->SetLabelFormat());
        x_SoftUpdate();
    }
}

void CPhyloTreeWidget::OnRerootTree(wxCommandEvent& /* evt */)
{
//   m_pMatrixPane->take_focus();
   CPhyloTreeNode * hover = m_pDataSource->GetCurrentNode();
   if (hover){
       // reroot it
       m_pDataSource->RerootTree(hover);

       // invalidate selections
       m_pDataSource->SetSelection(m_pDataSource->GetTree(),
                                false, true, false);
   }
   x_SoftUpdate();
   SendEditEvent();
}

void CPhyloTreeWidget::OnUpdateExportSelected(wxUpdateUIEvent& evt)
{
	evt.Enable(m_pDataSource->GetSelectedIds().size()>0);   
}

void CPhyloTreeWidget::OnUpdateRerootTree(wxUpdateUIEvent& evt)
{
    bool bSimplification = m_pMatrixPane->GetCurrRenderer()->GetSimplification();
    evt.Enable(
        !bSimplification &&
        m_pDataSource->GetCurrentNode()!=NULL &&
        m_pDataSource->GetCurrentNode()->Expanded());
}


void CPhyloTreeWidget::OnUpdateFilter(wxUpdateUIEvent& evt)
{
    bool bSimplification = m_pMatrixPane->GetCurrRenderer()->GetSimplification();
    evt.Enable(
        !bSimplification &&
        m_pDataSource->GetSelectedIds().size()>0);
}


void CPhyloTreeWidget::OnUpdateChildsCollapse(wxUpdateUIEvent& evt)
{
    CPhyloTreeNode * current = m_pDataSource->GetCurrentNode();
    bool bSimplification = m_pMatrixPane->GetCurrRenderer()->GetSimplification();
    evt.Enable(
        !bSimplification && current && current->CanExpandCollapse(IPhyGraphicsNode::eHideChilds));
}

void CPhyloTreeWidget::OnUpdateChildsExpand(wxUpdateUIEvent& evt)
{
    CPhyloTreeNode * current = m_pDataSource->GetCurrentNode();
    bool bSimplification = m_pMatrixPane->GetCurrRenderer()->GetSimplification();
    evt.Enable(
        !bSimplification && current && current->CanExpandCollapse(IPhyGraphicsNode::eShowChilds));
}


void CPhyloTreeWidget::SetScheme(CPhyloTreeScheme & sl)
{
    m_pScheme.Reset(&sl);
    m_pScheme->SaveCurrentSettings();
    ITERATE(CPhyloTreePane::TRenderers, renderer, m_pMatrixPane->GetRenderers()){
        (*renderer)->SetScheme(sl);
    }

#ifdef BUFFERED_RENDER
    if (m_pDataSource.NotNull())
        m_pDataSource->GetModel().SetScheme(sl);
#endif
}

void CPhyloTreeWidget::SetServiceLocator(IServiceLocator* s)
{
    x_GetPane()->SetServiceLocator(s);
    m_ServiceLocator = s;
}

void CPhyloTreeWidget::SetRegistryPath(const string& reg_path)
{
    m_RegPath = reg_path;
}

void CPhyloTreeWidget::LoadSettings()
{
    CPhyloTreeScheme * style = new CPhyloTreeScheme();

    if (!style->LoadCurrentSettings())
        return;
    
    SetScheme(*style);

    if (m_QueryPanel != NULL) {
        m_QueryPanel->SetRegistryPath(m_RegPath);
        m_QueryPanel->LoadSettings();
    }
}

void CPhyloTreeWidget::SaveSettings() const
{
    if (m_QueryPanel != NULL)
        m_QueryPanel->SaveSettings();
}

void CPhyloTreeWidget::OnSelectAll(wxCommandEvent& /* evt */)
{
    m_pDataSource->SetSelection(m_pDataSource->GetTree(),
                                true, true, false);
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnClearSelection(wxCommandEvent& /* evt */)
{
    m_pDataSource->SetSelection(m_pDataSource->GetTree(),
                                false, true, false);
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnInverseSelection(wxCommandEvent& /* evt */)
{
}

void CPhyloTreeWidget::OnChildsExpand(wxCommandEvent& /* evt */)
{
//   m_pMatrixPane->take_focus();
   CPhyloTreeNode * hover = m_pDataSource->GetCurrentNode();
   if (hover){
       m_pDataSource->ExpandCollapse(hover, IPhyGraphicsNode::eShowChilds);
   }
   x_SoftUpdate();
   SendEditEvent();
}

void CPhyloTreeWidget::OnHightlightCollapsed(wxCommandEvent& /* evt */)
{
   vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
   ITERATE(vector <IPhyloTreeRenderer*>, it, vRend){
     (*it)->SetHighlightColllapsed(!(*it)->GetHighlightCollapsed());
   }

   m_pDataSource->MarkAllDirty();
   x_SoftUpdate();
   SendEditEvent();
}

void CPhyloTreeWidget::OnChildsCollapse(wxCommandEvent& /* evt */)
{
//   m_pMatrixPane->take_focus();
   CPhyloTreeNode * hover = m_pDataSource->GetCurrentNode();
   if (hover){
       m_pDataSource->ExpandCollapse(hover, IPhyGraphicsNode::eHideChilds);
   }
   x_SoftUpdate();
   SendEditEvent();

}

void CPhyloTreeWidget::OnZoomToSubtree(wxCommandEvent& /* evt */)
{
//    m_pMatrixPane->take_focus();
    CPhyloTreeNode * hover  = m_pDataSource->GetCurrentNode();
    if (hover){
        CPhyloTreeNode * parent = static_cast<CPhyloTreeNode*>(hover->GetParent());

        // reroot it
        m_pDataSource->RerootTree(hover);

        // invalidate selections
        m_pDataSource->SetSelection(m_pDataSource->GetTree(),
                                false, true, false);

        // if possible, hide other nodes
        if (parent) {
            m_pDataSource->ExpandCollapse(parent, IPhyGraphicsNode::eHideChilds);
        }
    }
    x_SoftUpdate();
    SendEditEvent();
}

void CPhyloTreeWidget::OnUpdateZoomToSubtree(wxUpdateUIEvent& evt)
{
     CPhyloTreeNode * current = m_pDataSource->GetCurrentNode();
     bool bSimplification = m_pMatrixPane->GetCurrRenderer()->GetSimplification();

     evt.Enable(
        !bSimplification && current &&
        current->Expanded() && !current->IsLeafEx());
}


void CPhyloTreeWidget::OnSearchTip(wxCommandEvent & evt)
{
    // Need the tool tip window for the event - we stored its pointer in
    // the button's (event objects) client data when the button was created
    wxEvtHandler* obj = dynamic_cast<wxEvtHandler*>(evt.GetEventObject());
    if (obj != NULL) {
        CTooltipFrame* f = static_cast<CTooltipFrame*>(obj->GetClientData());
        std::string tip_id = f->GetTipInfo().GetTipID();

        int id = NStr::StringToInt(tip_id);
        CPhyloTreeNode* tip_node  = m_pDataSource->GetNode(id);
        if (tip_node){        
            TModelPoint  pt((*tip_node)->X(), (*tip_node)->Y());
            
            wxPoint win_pos = m_pMatrixPane->GetScreenPosition();

            // move to center of screen (if current zoom level allows)
            TModelUnit center_x = m_pMatrixPane->GetScreenRect().x +
                                  m_pMatrixPane->GetScreenRect().GetWidth()/2 - 
                                  win_pos.x;
            TModelUnit center_y = m_pMatrixPane->GetScreenRect().y - win_pos.y;
            center_y = m_pMatrixPane->GetRect().GetHeight()/2-center_y;

            // Get rectangle for tool tip within the window using standard
            // cartesian (and opengl) coords. (Lower left is (0,0)).
            wxRect tip_rect = f->GetScreenRect();
            tip_rect.x = tip_rect.x - win_pos.x;
            tip_rect.y = tip_rect.y - win_pos.y;
            tip_rect.y = m_pMatrixPane->GetRect().GetHeight() -
                         (tip_rect.y + tip_rect.GetHeight());

            // Could potentially add code to make sure glyph doesn't wind up
            // under this, or some other, tooltip here.  But that's not easy
            // since we don't have pointers to other tips (here) and we don't
            // know how much the actual scroll will be since it is constrained
            // when we are zoomed out.
           
            TModelPoint center_model_coord = 
                m_pMatrixPane->GetPane().UnProject((int)center_x, (int)center_y);

            double dx = center_model_coord.X()-pt.X();
            double dy = center_model_coord.Y()-pt.Y();         
                        
            Scroll(-dx, -dy);

            std::string tip_id = f->GetTipInfo().GetTipID();
            int id = NStr::StringToInt(tip_id);          
            m_pMatrixPane->GetCurrRenderer()->PointToNode(id, tip_rect, 0.5);
            m_pMatrixPane->Refresh();
        }
    }
}

void CPhyloTreeWidget::OnZoomTip(wxCommandEvent & evt)
{
    // Need the tool tip window for the event - we stored its pointer in
    // the button's (event objects) client data when the button was created
    wxEvtHandler* obj = dynamic_cast<wxEvtHandler*>(evt.GetEventObject());
    if (obj != NULL) {
        CTooltipFrame* f = static_cast<CTooltipFrame*>(obj->GetClientData());
        std::string tip_id = f->GetTipInfo().GetTipID();

        int id = NStr::StringToInt(tip_id);
        CPhyloTreeNode* tip_node  = m_pDataSource->GetNode(id);
        if (tip_node){
            TModelPoint  pt((*tip_node)->X(), (*tip_node)->Y());             

            // Scroll to the glyph prior to zooming:
            OnSearchTip(evt);
            
            float def_node_size = (float)
                m_pMatrixPane->GetCurrRenderer()->GetDefaultNodeSize(tip_node);
            
            // Compute distance between nodes here since not all renderers
            // do it the same (and they return int, but we can't round)
            float dimy = (float)m_pMatrixPane->GetCurrRenderer()->GetDimY();
            float size = (float)m_pDataSource->GetSize();
            float scale_y = (float)m_pMatrixPane->GetPane().GetScaleY();
            float dist_between_nodes =  dimy/(size * scale_y);

            // should not happen, but just in case:
            if (dist_between_nodes <= 0.0f) {              
                m_pMatrixPane->Refresh();
                return;
            }                           
           
            // Normally labels are displayed if the distance between nodes 
            // is > 2*node size.  We set the zoom factor here to be twice that
            // zoom level. We do not zoom in more if view is already zoomed.
            float zfact = 4.0f*def_node_size/dist_between_nodes;

            if (zfact > 1.0f) {

                // Zoom based on current zoom behavior.  If user has selected
                // horizonal zoom, zooming in generally won't provide much
                // if any benefit.  With veritcal or proportional zoom, this
                // should guarantee label visibility.
                CPhyloTreeScheme::TZoomBehavior zb = m_pScheme->GetZoomBehavior();
                CGlPane::EZoomOptions options = CGlPane::fZoomXY;
                if (zb == CPhyloTreeScheme::eZoomX) 
                    options = CGlPane::fZoomX;
                else if (zb == CPhyloTreeScheme::eZoomY) 
                    options = CGlPane::fZoomY;

                ZoomPoint(pt, zfact, options);
            }
            m_pMatrixPane->Refresh();
        }
    }
}

void CPhyloTreeWidget::OnInfoTip(wxCommandEvent & evt)
{
    // Need the tool tip window for the event - we stored its pointer in
    // the button's (event objects) client data when the button was created
    wxEvtHandler* obj = dynamic_cast<wxEvtHandler*>(evt.GetEventObject());
    if (obj != NULL) {
        CTooltipFrame* f = static_cast<CTooltipFrame*>(obj->GetClientData());
        std::string tip_id = f->GetTipInfo().GetTipID();

        int id = NStr::StringToInt(tip_id);
        CPhyloTreeNode* tip_node  = m_pDataSource->GetNode(id);
        if (tip_node){
            CwxPhyloEditDlg dlg(this);
            dlg.SetParams(tip_node);

            if(dlg.ShowModal() == wxID_OK) {
                m_pDataSource->Relabel("");
                x_SoftUpdate();
                SendEditEvent();
            }
        }
    }
}

void CPhyloTreeWidget::OnTipActivated(wxCommandEvent & evt)
{
    // Need the tool tip window for the event - we stored its pointer in
    // the button's (event objects) client data when the button was created
    wxEvtHandler* obj = dynamic_cast<wxEvtHandler*>(evt.GetEventObject());
    if (obj != NULL) {
        CTooltipFrame* f = static_cast<CTooltipFrame*>(obj->GetClientData());
        std::string tip_id = f->GetTipInfo().GetTipID();
        int id = NStr::StringToInt(tip_id);
        m_pMatrixPane->GetCurrRenderer()->SetActiveTooltipNode(id);
        OnUpdateSelChanged();
    }
}

void CPhyloTreeWidget::OnTipDeactivated(wxCommandEvent& /* evt */)
{
    m_pMatrixPane->GetCurrRenderer()->SetActiveTooltipNode(-1);
    OnUpdateSelChanged();
}

void CPhyloTreeWidget::OnShowAll(wxCommandEvent& /* evt */)
{
    m_pDataSource->ShowAll();
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnUpdateShowAll(wxUpdateUIEvent& evt)
{
    evt.Enable(!m_pMatrixPane->GetCurrRenderer()->GetSimplification());
}

void CPhyloTreeWidget::OnSimplify(wxCommandEvent& /* evt */)
{
//    m_pMatrixPane->take_focus();

    // expand all before making simplification
    if (!m_pMatrixPane->GetCurrRenderer()->GetSimplification()){
        m_pDataSource->ShowAll();
    }

    vector <IPhyloTreeRenderer*> vRend = m_pMatrixPane->GetRenderers();
    ITERATE(vector <IPhyloTreeRenderer*>, it, vRend){
      (*it)->SetSimplification(!(*it)->GetSimplification());
    }
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnUpdateSimplify(wxUpdateUIEvent& evt)
{
    evt.Check(m_pMatrixPane->GetCurrRenderer()->GetSimplification());
}

void CPhyloTreeWidget::OnEditNode(wxCommandEvent& /* evt */)
{   
    CPhyloTreeNode * hover  = m_pDataSource->GetCurrentNode();
    if (hover){
        CwxPhyloEditDlg dlg(this);
        dlg.SetParams(hover);

        if(dlg.ShowModal() == wxID_OK) {
            m_pScheme->SetTooltipFormat() = m_pDataSource->GenerateTooltipFormat();
            m_pDataSource->Relabel("");
            x_SoftUpdate();
            SendEditEvent();
        }
    }
}

void CPhyloTreeWidget::OnUpdateEditNode(wxUpdateUIEvent& evt)
{
    evt.Enable(m_pDataSource->GetCurrentNode()!=NULL);
}


void CPhyloTreeWidget::OnFilterDistances(wxCommandEvent& /* evt */)
{
    /*
    auto_ptr <CPhyloTreeFilterDlg> filterDialog(new CPhyloTreeFilterDlg(m_pDataSource->GetMinDistance(),
                                                                        m_pDataSource->GetNormDistance()));
    if (filterDialog->ShowModal() == eOK){
        m_pDataSource->FilterDistances(filterDialog->GetDistance());
        m_pDataSource->Refresh();
        x_SoftUpdate();
        SendEditEvent();
    }
    */
}

void    CPhyloTreeWidget::x_ZoomIn(int /* options */)
{   
    CPhyloTreeScheme::TZoomBehavior zb =
        m_pScheme->GetZoomBehavior();

    if      (zb == CPhyloTreeScheme::eZoomX) CGlWidgetBase::x_ZoomIn(CGlPane::fZoomX);
    else if (zb == CPhyloTreeScheme::eZoomY) CGlWidgetBase::x_ZoomIn(CGlPane::fZoomY);
    else                                     CGlWidgetBase::x_ZoomIn(CGlPane::fZoomXY);
}


void CPhyloTreeWidget::x_ZoomOut(int /* options */)
{   
    CPhyloTreeScheme::TZoomBehavior zb =
        m_pScheme->GetZoomBehavior();

    if      (zb == CPhyloTreeScheme::eZoomX) CGlWidgetBase::x_ZoomOut(CGlPane::fZoomX);
    else if (zb == CPhyloTreeScheme::eZoomY) CGlWidgetBase::x_ZoomOut(CGlPane::fZoomY);
    else                                     CGlWidgetBase::x_ZoomOut(CGlPane::fZoomXY);
}

void CPhyloTreeWidget::OnSort(wxCommandEvent& /* evt */)
{
    m_pDataSource->Sort();
    x_SoftUpdate();
    SendEditEvent();
}

void CPhyloTreeWidget::OnClean(wxCommandEvent& /* evt */)
{
    m_pDataSource->Clean();
    x_SoftUpdate();
    SendEditEvent();
}

void CPhyloTreeWidget::OnFilter(wxCommandEvent& /* evt */)
{
    m_pDataSource->Filter();
    x_SoftUpdate();
    SendEditEvent();
}

void CPhyloTreeWidget::OnLabels(wxCommandEvent & evt)
{
    switch (evt.GetId()){
    case eCmdLabelsForLeavesOnly:
        m_pScheme->SetLabelVisibility(CPhyloTreeScheme::eLabelsForLeavesOnly);
        break;
    case eCmdLabelsVisible:
        m_pScheme->SetLabelVisibility(CPhyloTreeScheme::eLabelsVisible);
        break;
    case eCmdLabelsHidden:
        m_pScheme->SetLabelVisibility(CPhyloTreeScheme::eLabelsHidden);
        break;
    default:
        break;
    }
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnUpdateLabels(wxUpdateUIEvent& evt)
{
    switch (evt.GetId()){
    case eCmdLabelsForLeavesOnly:
        evt.Check(m_pScheme->GetLabelVisibility()==CPhyloTreeScheme::eLabelsForLeavesOnly);
        break;
    case eCmdLabelsVisible:
        evt.Check(m_pScheme->GetLabelVisibility()==CPhyloTreeScheme::eLabelsVisible);
        break;
    case eCmdLabelsHidden:
        evt.Check(m_pScheme->GetLabelVisibility()==CPhyloTreeScheme::eLabelsHidden);
        break;
    default:
        break;
    }
}

void CPhyloTreeWidget::OnAl(wxCommandEvent & evt)
{
    switch (evt.GetId()){
    case eCmdAlNone:
        m_pScheme->SetAutoLabels(CPhyloTreeScheme::eAlNone);
        break;
    case eCmdAlPhylip:
        m_pScheme->SetAutoLabels(CPhyloTreeScheme::eAlPhylip);
        break;
    case eCmdAlPaml:
        m_pScheme->SetAutoLabels(CPhyloTreeScheme::eAlPaml);
        break;
    default:
        break;
    }
    m_pDataSource->MarkAllDirty();
    x_SoftUpdate();
}

void CPhyloTreeWidget::OnUpdateAl(wxUpdateUIEvent& evt)
{
    switch (evt.GetId()){
    case eCmdAlNone:
        evt.Check(m_pScheme->GetAutoLabels()==CPhyloTreeScheme::eAlNone);
        break;
    case eCmdAlPhylip:
        evt.Check(m_pScheme->GetAutoLabels()==CPhyloTreeScheme::eAlPhylip);
        break;
    case eCmdAlPaml:
        evt.Check(m_pScheme->GetAutoLabels()==CPhyloTreeScheme::eAlPaml);
        break;
    default:
        break;
    }
}


void CPhyloTreeWidget::OnEdit(wxCommandEvent & evt)
{
    switch (evt.GetId()){
    case eCmdNodeCopy:      m_pDataSource->Copy();          break;
    case eCmdNodeCut:       m_pDataSource->Cut();           break;
    case eCmdNodePaste:     m_pDataSource->Paste();         break;
    case eCmdNodeNewChild:  m_pDataSource->NewNode();       break;
    case eCmdNodeNewParent: m_pDataSource->NewNode(false);  break;
    case eCmdRemoveNode:    m_pDataSource->Remove(false);   break;
    case eCmdRemoveSubtree: m_pDataSource->Remove();        break;
    case eCmdMoveUp:        m_pDataSource->MoveNode(true);  break;
    case eCmdMoveDown:      m_pDataSource->MoveNode(false); break;
    default: break;
    }

    m_pDataSource->MarkAllDirty();
    x_SoftUpdate();

    SendEditEvent();
}

void CPhyloTreeWidget::OnUpdateEdit(wxUpdateUIEvent& evt)
{
    switch (evt.GetId()){
    case eCmdNodeCopy:
        evt.Enable(m_pDataSource->GetCurrentNode()!=NULL);
        break;
    case eCmdNodeCut:
        evt.Enable((m_pDataSource->GetCurrentNode()!=NULL) &&
                         (m_pDataSource->GetCurrentNode()->GetParent()!=NULL));
        break;
    case eCmdNodePaste:
        evt.Enable((m_pDataSource->GetCurrentNode()!=NULL) &&
                         (m_pDataSource->GetClipboard()!=NULL));
        break;
    case eCmdNodeNewChild:
        evt.Enable(m_pDataSource->GetCurrentNode()!=NULL);
        break;
    case eCmdNodeNewParent:
    case eCmdMoveUp:
    case eCmdMoveDown:
        evt.Enable((m_pDataSource->GetCurrentNode()!=NULL) &&
                         (m_pDataSource->GetCurrentNode()->GetParent()!=NULL));
        break;
    case eCmdRemoveNode:
    case eCmdRemoveSubtree:
        evt.Enable(m_pDataSource->GetCurrentNode()!=NULL);
        break;
    default:
        break;
    }
}

void CPhyloTreeWidget::SendEditEvent()
{
    CEvent evt(CEvent::eEvent_Message, eCmdSomethingEdited);
    Send(&evt, ePool_Parent);
}


void CPhyloTreeWidget::OnUpdateEditLabel(wxUpdateUIEvent& evt)
{
    evt.Enable(m_pDataSource->GetCurrentNode()!=NULL);
}

void CPhyloTreeWidget::OnEditLabel(wxCommandEvent& /* evt */)
{
    EditLabel();
}

void CPhyloTreeWidget::EditLabel()
{
    CPhyloTreeNode * hover  = m_pDataSource->GetCurrentNode();
    if (hover){ 
        CwxLabelEditDlg dlg(this);
        dlg.SetParams(hover);

        if(dlg.ShowModal() == wxID_OK) {
            m_pDataSource->Relabel("");
            x_SoftUpdate();
            SendEditEvent();
        }       
    }
}

void CPhyloTreeWidget::OnExportSelectedNew(wxCommandEvent& /* evt */)
{
	CEvent eee(CEvent::eEvent_Message, eCmdExportSelectedNew);
    Send(&eee, ePool_Parent);
}

void CPhyloTreeWidget::OnExportSelected(wxCommandEvent& /* evt */)
{
    CEvent eee(CEvent::eEvent_Message, eCmdExportSelected);
    Send(&eee, ePool_Parent);
}

void CPhyloTreeWidget::OnNewViewFromSelection(wxCommandEvent& /* evt */)
{
	CEvent eee(CEvent::eEvent_Message, eCmdNewViewFromSelection);
    Send(&eee, ePool_Parent);
}

void CPhyloTreeWidget::QueryStart()
{
    this->SetCursor(*wxHOURGLASS_CURSOR);        
    m_pMatrixPane->Disable(); 
    m_QueryPanel->SetCursor(*wxSTANDARD_CURSOR);
}
   
void CPhyloTreeWidget::QueryEnd()
{
    this->SetCursor(*wxSTANDARD_CURSOR);
    m_pMatrixPane->Enable();
    IterateSelection(0);
}

void CPhyloTreeWidget::IterateSelection(int dir)
{
    CPhyloTreeNode* node;
    
    // Go tot the next/previous node in the selection set, and return it
    node = m_pDataSource->IterateOverSelNodes(dir, m_QueryPanel->IsSelectAll());

    // If we have a node, check to see if it is currently on screen.  If
    // it isn't, scroll the window so that it is visible. Do not scroll it
    // to the screen center, but rather half way toward the center. That
    // gives a better impression to the user of which way the screen is
    // moving.
    if (node){        
        TModelPoint  pt((*node)->X(), (*node)->Y());

        TVPPoint ppt = m_pMatrixPane->GetPane().Project(pt.X(), pt.Y());
        if (!m_pMatrixPane->GetPane().GetViewport().PtInRect(ppt)) {
            wxPoint win_pos = m_pMatrixPane->GetScreenPosition();

            //Get center of screen in model coords
            TModelUnit center_x = m_pMatrixPane->GetScreenRect().x +
                                  m_pMatrixPane->GetScreenRect().GetWidth()/2 - 
                                  win_pos.x;
            TModelUnit center_y = m_pMatrixPane->GetScreenRect().y - win_pos.y;
            center_y = m_pMatrixPane->GetRect().GetHeight()/2-center_y;

            TModelPoint center_model_coord = 
                m_pMatrixPane->GetPane().UnProject((int)center_x, (int)center_y);

            double dx = 0.0;
            double dy = 0.0;

            // Get viewport
            TVPRect vp = m_pMatrixPane->GetPane().GetViewport();

            // The point is outside the viewport.  Determine to which side(s)
            // and scroll it so that the point is 25% within the viewport
            // relative to the side(s) that it is currently on.
            if (ppt.X() > vp.Width()) {
                double off = ppt.X() - vp.Width();               
                dx = (center_model_coord.X() - pt.X());
                dx *= (off + vp.Width()*0.25)/(off + vp.Width()*0.5);
            }
            else if (ppt.X() < 0.0) {
                double off = -ppt.X();               
                dx = (center_model_coord.X() - pt.X());
                dx *= (off + vp.Width()*0.25)/(off + vp.Width()*0.5);
            }

            if (ppt.Y() > vp.Height()) {
                double off = ppt.Y() - vp.Height();
                dy = (center_model_coord.Y()-pt.Y());
                dy *= (off + vp.Height()*0.25)/(off + vp.Height()*0.5);
            }
            else if (ppt.Y() < 0.0) {
                double off = -ppt.Y();               
                dy = (center_model_coord.Y() - pt.Y());
                dy *= (off + vp.Height()*0.25)/(off + vp.Height()*0.5);
            }    

            Scroll(-dx, -dy);
        }
    }

    SendSelChangedEvent();
    OnUpdateSelChanged();    
}

void CPhyloTreeWidget::SetSelectAll(bool b)
{
    if (m_pDataSource != NULL) {
        m_pDataSource->IterateOverSelNodes(0, b);
        SendSelChangedEvent();
        OnUpdateSelChanged();        
    }
}

void CPhyloTreeWidget::SetHideUnselected(bool b)
{
    m_pScheme->SetSelectionVisibility(b ?
                                      CPhyloTreeScheme::eHighlightSelection :
                                      CPhyloTreeScheme::eNormalSelection); 
    m_pMatrixPane->BufferedRender();
    OnUpdateSelChanged();
}


END_NCBI_SCOPE
