/*  $Id: flat_file_context.cpp 27922 2013-04-25 16:51:42Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/seq/flat_file_context.hpp>
#include <gui/widgets/seq/flat_file_view_params.hpp>

#include <gui/widgets/seq/flat_file_header_item.hpp>
#include <gui/widgets/text_widget/composite_text_item.hpp>
#include <gui/widgets/text_widget/expand_traverser.hpp>

#include <objmgr/seqdesc_ci.hpp>
#include <objmgr/feat_ci.hpp>

#include <objtools/format/items/locus_item.hpp>
#include <objtools/format/items/defline_item.hpp>
#include <objtools/format/items/source_item.hpp>
#include <objtools/format/items/comment_item.hpp>
#include <objtools/format/items/feature_item.hpp>
#include <objtools/format/items/genome_project_item.hpp>

#include "features_group.hpp"
#include "sequence_group.hpp"

#include <gui/widgets/wx/ui_command.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/menu.h>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

class CFlatFileViewState : public CTextPanelState
{
public:
    CFlatFileViewState() : m_SavedPosX(0), m_SavedPosY(0) {}

    virtual void SaveState(CTextItemPanel& textPanel);
    virtual void RestoreState(CTextItemPanel& textPanel, const vector<ITextItem*>& items);
    virtual void RestorePos(CTextItemPanel& textPanel);

private:
    void x_SaveFeatures(CCompositeTextItem* composite, CFlatFileViewContext* context);
    void x_RestoreFeatures(CCompositeTextItem* composite, CFlatFileViewContext* context);

    int m_SavedPosX;
    int m_SavedPosY;

    map<string, list<bool> > m_HeaderState;

    set<CSeq_feat_Handle> m_ExpandedFeats;
    list<bool> m_FeatGroups;

    list<bool> m_SeqBlocks;
};

void CFlatFileViewState::SaveState(CTextItemPanel& textPanel)
{
    textPanel.GetViewStart (&m_SavedPosX, &m_SavedPosY);

    CFlatFileViewContext* ffContext = (CFlatFileViewContext*)textPanel.GetContext();

    CCompositeTextItem* composite = (CCompositeTextItem*)textPanel.GetItem();
    if (!composite)
        return;
/*
    for (CCompositeIterator sect_it(*composite, typeid(CCompositeTextItem)); sect_it; ++sect_it) {
        CCompositeTextItem& section = dynamic_cast<CCompositeTextItem&>(*sect_it);
        for (size_t i = 0; i < section.GetItemCount(); ++i) {
            CExpandItem* expandItem = dynamic_cast<CExpandItem*>(section.GetItem(i));
            if (expandItem) {
                CFlatFileHeader* headerItem = 
                    dynamic_cast<CFlatFileHeader*>(expandItem->GetCollapsedItem());
                if (headerItem) {
                    m_HeaderState[headerItem->GetTextValue()].push_back(expandItem->IsExpanded());
                }
            }

            CFeaturesGroup* featGroup = dynamic_cast<CFeaturesGroup*>(expandItem);
            if (featGroup) {
                x_SaveFeatures((CCompositeTextItem*)featGroup->GetExpandedItem(), ffContext);
            }

            CSequenceGroup* seqGroup = dynamic_cast<CSequenceGroup*>(expandItem);
            if (seqGroup) {
                CCompositeTextItem* group = (CCompositeTextItem*)seqGroup->GetExpandedItem();
                for (size_t j = 0; j < group->GetItemCount(); ++j) {
                    IExpandable* expandable = dynamic_cast<IExpandable*>(group->GetItem(j));
                    if (expandable)
                        m_SeqBlocks.push_back(expandable->IsExpanded());
                }
            }
        }
    }
 */
}

void CFlatFileViewState::RestoreState(CTextItemPanel& textPanel, const vector<ITextItem*>& items)
{
    CFlatFileViewContext* ffContext = (CFlatFileViewContext*)textPanel.GetContext();

    for (size_t i = 0; i < items.size(); ++i) {
        CExpandItem* expandItem = dynamic_cast<CExpandItem*>(items[i]);
        if (expandItem) {
            CFlatFileHeader* headerItem = 
                dynamic_cast<CFlatFileHeader*>(expandItem->GetCollapsedItem());
            if (headerItem) {
                map<string, list<bool> >::iterator it = m_HeaderState.find(headerItem->GetTextValue());
                if (it != m_HeaderState.end()) {
                    list<bool>& list = it->second;
                    if (!list.empty()) {
                        if (list.front()) {
                            if (!expandItem->IsExpanded())
                                expandItem->Toggle(ffContext);
                        }
                        else {
                            if (expandItem->IsExpanded())
                                expandItem->Toggle(ffContext);
                        }
                        list.pop_front();
                    }
                }
            }
        }

        CFeaturesGroup* featGroup = dynamic_cast<CFeaturesGroup*>(expandItem);
        if (featGroup) {
            x_RestoreFeatures((CCompositeTextItem*)featGroup->GetExpandedItem(), ffContext);
        }

        CSequenceGroup* seqGroup = dynamic_cast<CSequenceGroup*>(expandItem);
        if (seqGroup) {
            CCompositeTextItem* composite = (CCompositeTextItem*)seqGroup->GetExpandedItem();
            for (size_t j = 0; j < composite->GetItemCount(); ++j) {
                IExpandable* expandable = dynamic_cast<IExpandable*>(composite->GetItem(j));
                if (expandable == 0)
                    continue;

                if (!m_SeqBlocks.empty()) {
                    if (m_SeqBlocks.front()) {
                        if (!expandable->IsExpanded()) 
                            expandable->Toggle(ffContext);
                    }
                    else {
                        if (expandable->IsExpanded())
                            expandable->Toggle(ffContext);
                    }
                    m_SeqBlocks.pop_front();
                }
            }
        }
    }
}

void CFlatFileViewState::RestorePos(CTextItemPanel& textPanel)
{
    int x, y;
    textPanel.GetVirtualSize(&x, &y);

    int xUnit, yUnit;
    textPanel.GetScrollPixelsPerUnit(&xUnit, &yUnit);
    if (y/yUnit > m_SavedPosY)
        textPanel.Scroll (m_SavedPosX, m_SavedPosY);
}

void CFlatFileViewState::x_SaveFeatures(CCompositeTextItem* composite, CFlatFileViewContext* context)
{
    for (size_t i = 0; i < composite->GetItemCount(); ++i) {
        ITextItem* featItem = composite->GetItem(i);

        IExpandable* expandable = dynamic_cast<IExpandable*>(featItem);
        if (expandable == 0 || !expandable->IsExpanded())
            continue;

        CConstRef<CObject> object = featItem->GetAssosiatedObject();
        const CSeq_feat* feat = dynamic_cast<const CSeq_feat*>(object.GetPointer());
        if (!feat)
            continue;

        CSeq_feat_Handle handle = context->GetScope().GetSeq_featHandle(*feat, CScope::eMissing_Null);
        if (!handle)
            continue;

        m_ExpandedFeats.insert(handle);
    }
}

void CFlatFileViewState::x_RestoreFeatures(CCompositeTextItem* composite, CFlatFileViewContext* context)
{
    for (size_t i = 0; i < composite->GetItemCount(); ++i) {
        ITextItem* featItem = composite->GetItem(i);

        IExpandable* expandable = dynamic_cast<IExpandable*>(featItem);
        if (expandable == 0)
            continue;

        CConstRef<CObject> object = featItem->GetAssosiatedObject();
        const CSeq_feat* feat = dynamic_cast<const CSeq_feat*>(object.GetPointer());
        if (!feat)
            continue;

        CSeq_feat_Handle handle = context->GetScope().GetSeq_featHandle(*feat, CScope::eMissing_Null);
        if (!handle)
            continue;

        if (m_ExpandedFeats.find(handle) != m_ExpandedFeats.end()) {
            if (!expandable->IsExpanded())
                expandable->Toggle(context);
        }
        else {
            if (expandable->IsExpanded())
                expandable->Toggle(context);
        }
    }
}

enum
{
    kExpandAllFeatures = 200,
    kCollapseAllFeatures,
    kExpandSeqBlocks,
    kCollapseSeqBlocks,
    kCollapseSequences,
    kExpandAll
};

BEGIN_EVENT_TABLE( CFlatFileViewContext, CTextPanelContext )
    EVT_MENU(kExpandAllFeatures, CFlatFileViewContext::OnExpandAllFeatures)
    EVT_MENU(kCollapseAllFeatures, CFlatFileViewContext::OnCollapseAllFeatures)
    EVT_MENU(kExpandSeqBlocks, CFlatFileViewContext::OnExpandSeqBlocks)
    EVT_MENU(kCollapseSeqBlocks, CFlatFileViewContext::OnCollapseSeqBlocks)

    EVT_MENU(kCollapseSequences, CFlatFileViewContext::OnCollapseSequences)
    EVT_MENU(kExpandAll, CFlatFileViewContext::OnExpandAll)
END_EVENT_TABLE()

CFlatFileViewContext::CFlatFileViewContext(
    objects::IFormatter& formatter,
    objects::CScope& scope,
    IBioseqEditor* bioseqEditor)
    : CTextPanelContext(3),
      m_Formatter(&formatter),
      m_Scope(&scope),
      m_BioseqEditor(bioseqEditor)
{
    if (m_BioseqEditor)
        m_BioseqEditor->SetCallBack(this);
}

void CFlatFileViewContext::SetPanel(CTextItemPanel* panel)
{
    if (m_Panel && m_BioseqEditor) {
        m_Panel->PopEventHandler();
    }

    CTextPanelContext::SetPanel(panel);

    if (m_Panel && m_BioseqEditor) {
        m_Panel->PushEventHandler(m_BioseqEditor);
    }
}

void CFlatFileViewContext::RegisterCommands(CUICommandRegistry& cmd_reg, wxFileArtProvider&)
{
    static bool registered = false;
    if (!registered) {
        registered = true;

        cmd_reg.RegisterCommand(kExpandAllFeatures,
            "Expand All Features",
            "Expand All Features",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command expands(opens) All Features of a FlatFile view");
        cmd_reg.RegisterCommand(kCollapseAllFeatures,
            "Collapse All Features",
            "Collapse All Features",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command collapses(closes) All Features of a FlatFile view");
        cmd_reg.RegisterCommand(kExpandSeqBlocks,
            "Expand Sequence",
            "Expand Sequence",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command expands(opens) All Sequence Blocks of a FlatFile view");
        cmd_reg.RegisterCommand(kCollapseSeqBlocks,
            "Collapse Sequence",
            "Collapse Sequence",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command collapses(closes) Sequence Blocks of a FlatFile view");
        cmd_reg.RegisterCommand(kCollapseSequences,
            "Collapse Sequences",
            "Collapse Sequences",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command collapses(closes) Sequence Sections of a FlatFile view");
        cmd_reg.RegisterCommand(kExpandAll,
            "Expand All",
            "Expand All",
            NcbiEmptyString,
            NcbiEmptyString,
            "Command expands(opens) all of a FlatFile view");
    }
}

static
WX_DEFINE_MENU(kFlatFileContextMenu)
    WX_MENU_SEPARATOR_L("Edit Selection")
    WX_MENU_SEPARATOR_L("FlatFile Blocks")
    WX_MENU_ITEM(kCollapseSequences)
    WX_MENU_ITEM(kExpandAll)
WX_END_MENU()

wxMenu* CFlatFileViewContext::CreateMenu() const
{
    CUICommandRegistry& cmd_reg = CUICommandRegistry::GetInstance();
    wxMenu* menu = cmd_reg.CreateMenu(kFlatFileContextMenu);
    if (m_BioseqEditor) {
        auto_ptr<wxMenu> editorMenu(m_BioseqEditor->CreateContextMenu());
        if (editorMenu.get()) {
            Merge(*menu, *editorMenu);
        }
    }
    return menu;
}

void CFlatFileViewContext::EditItem(ITextItem* WXUNUSED(item), bool controlDown, bool shiftDown)
{
    if (!m_BioseqEditor || controlDown || shiftDown)
        return;

    m_BioseqEditor->EditSelection();
}

CTextPanelState* CFlatFileViewContext::GetViewState() const
{
    if (m_Panel == 0)
        return 0;

    CFlatFileViewState* state = new CFlatFileViewState();
    state->SaveState(*m_Panel);
    return state;
}

void CFlatFileViewContext::OnExpandAllFeatures(wxCommandEvent& WXUNUSED(event))
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;
/*
    bool update = false;
    for (CCompositeIterator sect_it(*composite, typeid(CCompositeTextItem)); sect_it; ++sect_it) {
        for (CCompositeIterator it(dynamic_cast<CCompositeTextItem&>(*sect_it), typeid(CFeaturesGroup));
                it; ++it) {
            dynamic_cast<CFeaturesGroup&>(*it).ExpandAll(this);
            update = true;
        }
    }

    if (update) {
        m_Panel->Layout();
        m_Panel->Refresh();
    }
 */
}

void CFlatFileViewContext::OnCollapseAllFeatures(wxCommandEvent& WXUNUSED(event))
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;
/*
    bool update = false;
    for (CCompositeIterator sect_it(*composite, typeid(CCompositeTextItem)); sect_it; ++sect_it) {
        for (CCompositeIterator it(dynamic_cast<CCompositeTextItem&>(*sect_it), typeid(CFeaturesGroup));
                it; ++it) {
            dynamic_cast<CFeaturesGroup&>(*it).CollapseAll(this);
            update = true;
        }
    }

    if (update) {
        m_Panel->Layout();
        m_Panel->Refresh();
    }
 */
}

void CFlatFileViewContext::OnExpandSeqBlocks(wxCommandEvent& WXUNUSED(event))
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;
/*
    bool update = false;
    for (CCompositeIterator sect_it(*composite, typeid(CCompositeTextItem)); sect_it; ++sect_it) {
        for (CCompositeIterator it(dynamic_cast<CCompositeTextItem&>(*sect_it), typeid(CSequenceGroup));
                it; ++it) {
            dynamic_cast<CSequenceGroup&>(*it).ExpandAll(this);
            update = true;
        }
    }

    if (update) {
        m_Panel->Layout();
        m_Panel->Refresh();
    }
 */
}

void CFlatFileViewContext::OnCollapseSeqBlocks(wxCommandEvent& WXUNUSED(event))
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;
/*
    bool update = false;
    for (CCompositeIterator sect_it(*composite, typeid(CCompositeTextItem)); sect_it; ++sect_it) {
        for (CCompositeIterator it(dynamic_cast<CCompositeTextItem&>(*sect_it), typeid(CSequenceGroup));
                it; ++it) {
            dynamic_cast<CSequenceGroup&>(*it).CollapseAll(this);
            update = true;
        }
    }

    if (update) {
        m_Panel->Layout();
        m_Panel->Refresh();
    }
 */
}

void CFlatFileViewContext::OnCollapseSequences(wxCommandEvent& event)
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;

    CExpandTraverser expander(*this, false);
    composite->Traverse(expander, false);

    m_Panel->Layout();
    m_Panel->Refresh();
}

void CFlatFileViewContext::OnExpandAll(wxCommandEvent& event)
{
    CCompositeTextItem* composite = (CCompositeTextItem*)m_Panel->GetItem();
    if (!composite)
        return;

    CExpandTraverser expander(*this, true);
    composite->Traverse(expander, true);

    m_Panel->Layout();
    m_Panel->Refresh();
}

static
WX_DEFINE_MENU(kFlatFileBioseqMenu)
    WX_SUBMENU("BioSource")
        WX_MENU_SEPARATOR_L("Edit Bioseq")
    WX_END_SUBMENU()
WX_END_MENU()

void CFlatFileViewContext::OnEditBioseq(objects::CBioseq_Handle& bh, wxPoint pos)
{
    if (!m_BioseqEditor)
        return;

    pos = m_Panel->CalcScrolledPosition(pos);

    CUICommandRegistry& cmd_reg = CUICommandRegistry::GetInstance();
    auto_ptr<wxMenu> menu(cmd_reg.CreateMenu(kFlatFileBioseqMenu));

    {{
        auto_ptr<wxMenu> editorMenu(m_BioseqEditor->CreateBioseqMenu());
        if (editorMenu.get()) {
            Merge(*menu, *editorMenu);
        }
    }}

    CleanupSeparators(*menu);

    m_CurBioseq = bh;
    m_Panel->PopupMenu(menu.get(), pos);
    m_CurBioseq.Reset();
}

void CFlatFileViewContext::GetCurrentSelection(TEditObjectList& objects)
{
    ITERATE (set<ITextItem*>, it, m_SelectedItems) {
        CFlatFileTextItem* ffItem = 0;
        CExpandItem* expandItem = dynamic_cast<CExpandItem*>(*it);

        if (expandItem)
            ffItem = dynamic_cast<CFlatFileTextItem*>(expandItem->GetExpandedItem());
        else
            ffItem = dynamic_cast<CFlatFileTextItem*>(*it);

        if (!ffItem)
            continue;

        const objects::IFlatItem* item = ffItem->m_Item.GetPointer();

        {{
            const CSourceItem* sourceItem = dynamic_cast<const CSourceItem*>(item);
            if (sourceItem) {
                CBioseqContext* ctx = sourceItem->GetContext();
                if (!ctx)
                    continue;

                CBioseq_Handle& bh = ctx->GetHandle();
                CSeqdesc_CI dsrc_it(bh, CSeqdesc::e_Source);
                if ( dsrc_it ) {
                    objects.push_back(CEditObject(*dsrc_it, bh.GetSeq_entry_Handle()));
                    continue;
                }
                CFeat_CI fsrc_it(bh, CSeqFeatData::e_Biosrc);
                if (fsrc_it) {
                    CSeq_feat_Handle fh = *fsrc_it;
                    objects.push_back(CEditObject(*fh.GetOriginalSeq_feat(), bh.GetSeq_entry_Handle()));
                    continue;
                }
                continue;
            }
        }}

        {{
            const CCommentItem* commentItem = dynamic_cast<const CCommentItem*>(item);
            if (commentItem) {
                const CSeqdesc* desc = dynamic_cast<const CSeqdesc*>(commentItem->GetObject());
                if (desc) {
                    CBioseqContext* ctx = commentItem->GetContext();
                    if (!ctx)
                        continue;

                    CBioseq_Handle& bh = ctx->GetHandle();

                    objects.push_back(CEditObject(*desc, bh.GetSeq_entry_Handle()));
                    continue;
                }
            }
        }}
        {{
            const CSourceFeatureItem* sourceFeatureItem 
                = dynamic_cast<const CSourceFeatureItem*>(item);
            if (sourceFeatureItem) {
                const CSeq_feat* seq_feat = dynamic_cast<const CSeq_feat*>(item->GetObject());
                if (!seq_feat)
                    continue;

                CBioseqContext* ctx = sourceFeatureItem->GetContext();
                if (!ctx)
                    continue;

                CBioseq_Handle& bh = ctx->GetHandle();

                const CSeq_feat::TData& data = seq_feat->GetData();
                if (data.IsBiosrc()) {
                    const CBioSource& bioSource = data.GetBiosrc();
                    CSeqdesc_CI dsrc_it(ctx->GetHandle(), CSeqdesc::e_Source);
                    if ( dsrc_it ) {
                        const CSeqdesc& seqDesc = *dsrc_it;
                        if (seqDesc.IsSource()) {
                            const CBioSource& bioSource2 = seqDesc.GetSource();
                            if (&bioSource2 == &bioSource) {
                                objects.push_back(CEditObject(seqDesc, bh.GetSeq_entry_Handle()));
                                continue;
                            }
                        }
                    }
                }

                CFeat_CI fsrc_it(ctx->GetHandle(), CSeqFeatData::e_Biosrc);
                if (fsrc_it) {
                    CSeq_feat_Handle fh = *fsrc_it;
                    objects.push_back(CEditObject(*fh.GetOriginalSeq_feat(), bh.GetSeq_entry_Handle()));
                    continue;
                }
                continue;
            }
        }}
        {{
            const CLocusItem* locusItem 
                = dynamic_cast<const CLocusItem*>(item);
            if (locusItem) {
                CBioseqContext* ctx = locusItem->GetContext();
                if (!ctx)
                    continue;

                CBioseq_Handle& bh = ctx->GetHandle();

                const CMolInfo* molInfo = dynamic_cast<const CMolInfo*>(item->GetObject());
                if (!molInfo)
                    continue;
                CSeqdesc_CI dsrc_it(ctx->GetHandle(), CSeqdesc::e_Molinfo);
                for (; dsrc_it; ++dsrc_it) {
                    const CSeqdesc& seqDesc = *dsrc_it;
                    if (&seqDesc.GetMolinfo() == molInfo) {
                        objects.push_back(CEditObject(seqDesc, bh.GetSeq_entry_Handle()));
                        continue;
                    }
                }
            }
        }}
        {{
            const CDeflineItem* defLineItem = dynamic_cast<const CDeflineItem*>(item);
            if (defLineItem) {
                CBioseqContext* ctx = defLineItem->GetContext();
                if (!ctx)
                    continue;

                CBioseq_Handle& bh = ctx->GetHandle();

                CRef<CTitleEdit> titleEdit(new CTitleEdit());
                titleEdit->SetTitle(defLineItem->GetDefline());
                objects.push_back(CEditObject(*titleEdit, bh.GetSeq_entry_Handle()));
                continue;
            }
        }}
        {{
            const CReferenceItem* refItem 
                = dynamic_cast<const CReferenceItem*>(item);
            if (refItem) {
                CBioseqContext* ctx = refItem->GetContext();
                if (!ctx)
                    continue;

                CBioseq_Handle& bh = ctx->GetHandle();

                const CSeq_feat* seq_feat = dynamic_cast<const CSeq_feat*>(item->GetObject());
                if (seq_feat) {
                    CSeq_feat_Handle fh = ctx->GetScope().GetSeq_featHandle(*seq_feat, CScope::eMissing_Null);
                    if (fh)
                        objects.push_back(CEditObject(*fh.GetOriginalSeq_feat(), bh.GetSeq_entry_Handle()));
                    continue;
                } else {
                    const CPubdesc* pubdesc = dynamic_cast<const CPubdesc*>(item->GetObject());
                    if (pubdesc) {
                        CSeqdesc_CI dsrc_it(ctx->GetHandle(), CSeqdesc::e_Pub);
                        for (; dsrc_it; ++dsrc_it) {
                            const CSeqdesc& seqDesc = *dsrc_it;
                            if (&seqDesc.GetPub() == pubdesc) {
                                objects.push_back(CEditObject(seqDesc, bh.GetSeq_entry_Handle()));
                                continue;
                            }
                        }
                    }
                }
                continue;
            }
        }}
        {{
            const CFeatureItem* featureItem 
                = dynamic_cast<const CFeatureItem*>(item);
            if (featureItem) {
                const CSeq_feat* seq_feat = dynamic_cast<const CSeq_feat*>(item->GetObject());
                if (!seq_feat)
                    continue;

                CBioseqContext* ctx = featureItem->GetContext();
                if (!ctx)
                    continue;

                CSeq_feat_Handle fh = ctx->GetScope().GetSeq_featHandle(*seq_feat, CScope::eMissing_Null);
                if (fh) {
                    CBioseq_Handle& bh = ctx->GetHandle();
                    objects.push_back(CEditObject(*fh.GetOriginalSeq_feat(), bh.GetSeq_entry_Handle()));
                }
                continue;
            }
        }}
        {{
            const CGenomeProjectItem* genomeProjectItem 
                = dynamic_cast<const CGenomeProjectItem*>(item);
            if (genomeProjectItem ) {
                CBioseqContext* ctx = genomeProjectItem ->GetContext();
                if (!ctx)
                    continue;

                // extract all the useful user objects
                for (CSeqdesc_CI desc(ctx->GetHandle(), CSeqdesc::e_User);  desc;  ++desc) {
                    const CUser_object& uo = desc->GetUser();

                    if ( !uo.GetType().IsStr() ) {
                        continue;
                    }
                    if( NStr::EqualNocase( uo.GetType().GetStr(), "DBLink" ) ) {
                        CBioseq_Handle& bh = ctx->GetHandle();
                        const CSeqdesc& seqDesc = *desc;
                        objects.push_back(CEditObject(seqDesc, bh.GetSeq_entry_Handle()));
                        break;
                    }
                }
                continue;
            }
        }}
    }
}

CBioseq_Handle CFlatFileViewContext::GetCurrentBioseq()
{
    return m_CurBioseq;
}

END_NCBI_SCOPE
