/*  $Id: text_retrieve_job.cpp 26643 2012-10-18 17:54:05Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*
* File Description:
*
*/

#include <ncbi_pch.hpp>

#include <gui/widgets/seq/text_retrieve_job.hpp>
#include <gui/widgets/text_widget/text_item_panel.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

IAppJob::EJobState CTextRetrieveJob::Run()
{
    string jobName = x_GetJobName();
    string err_msg, logMsg = jobName + " - exception in Run() ";

    IAppJob::EJobState jobState = eCompleted;

    try {
        jobState = x_Run();
    }
    catch (CException& e) {
        err_msg = e.GetMsg();
        LOG_POST(Error << logMsg << GetDescr() << ". " << err_msg);
        //e.ReportAll();
    } catch (std::exception& e) {
        err_msg = GetDescr() + ". " + e.what();
        LOG_POST(Error << logMsg << GetDescr() << ". " << err_msg);
    }
    catch (const CCancelException&) {
        return eCanceled;
    }

    if (IsCanceled())
        return eCanceled;

    if (!err_msg.empty()) {
        string s = err_msg;
        m_Error.Reset(new CAppJobError(s));
        return eFailed;
    }

    return jobState;
}


void CTextRetrieveJob::x_CheckCancelled() const
{
    if (IsCanceled()) {
        throw CCancelException();
    }
}

CConstIRef<IAppJobProgress> CTextRetrieveJob::GetProgress()
{
    CTextViewRetrieveResult* result = new CTextViewRetrieveResult();
    CTextViewRetrieveProgress* progress = new CTextViewRetrieveProgress(result);
    return CConstIRef<IAppJobProgress>(progress);
}

CRef<CObject> CTextRetrieveJob::GetResult()
{
    return CRef<CObject>(m_Result.GetPointer());
}

CConstIRef<IAppJobError> CTextRetrieveJob::GetError()
{
    return CConstIRef<IAppJobError>(m_Error.GetPointer());
}

END_NCBI_SCOPE
