/*  $Id: pane_content_conf.cpp 17711 2008-09-04 20:33:08Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert Smith
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/seq_graphic/pane_content_conf.hpp>
///#include <gui/widgets/config/feat_show_panel.hpp>
#include <objects/seqfeat/SeqFeatData.hpp>


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);



CPaneContentConf::CPaneContentConf(CSettingsSet& settings)
    : CFeatConfigShow(settings)
{
}


// generic accessors
bool CPaneContentConf::GetShowAnnot(EShowAnnotType e)
{
    bool val = false;
    switch (e) {
    case eShowAlignments:
        val = GetShowAlignments();
        break;
    case eShowPWAlignLabels:
        val = GetShowPWAlignLabels();
        break;
    case eShowSwitchPoints:
        val = GetShowSwitchPoints();
        break;
    case eShowCDSProductFeats:
        val = GetShowCDSProductFeats();
        break;
    case eShowAlignmentFeats:
        val = GetShowAlignmentFeats();
        break;
    case eShowSegmentMap:
        val = GetShowSegmentMap();
        break;
    case eShowGraphs:
        val = GetShowGraphs();
        break;
    case eShowProtProd:
        val = GetShowProtProd();
        break;
    }
    return val;
}


void CPaneContentConf::SetShowAnnot(bool b, EShowAnnotType e)
{
    switch (e) {
    case eShowAlignments:
        SetShowAlignments(b);
        break;
    case eShowPWAlignLabels:
        SetShowPWAlignLabels(b);
        break;
    case eShowSwitchPoints:
        SetShowSwitchPoints(b);
        break;
    case eShowCDSProductFeats:
        SetShowCDSProductFeats(b);
        break;
    case eShowAlignmentFeats:
        SetShowAlignmentFeats(b);
        break;
    case eShowSegmentMap:
        SetShowSegmentMap(b);
        break;
    case eShowGraphs:
        SetShowGraphs(b);
        break;
    case eShowProtProd:
        SetShowProtProd(b);
        break;
    }
}


bool CPaneContentConf::GetHistogram(TFeatSubtype subtype) const
{
    return m_Settings.GetBool(m_Settings.GetFeatReadKey(subtype, kEmptyStr, "histogram") );
}


void CPaneContentConf::SetHistogram(TFeatSubtype subtype, bool hist)
{
    m_Settings.Set(CSettingsSet::GetFeatWriteKey(subtype, kEmptyStr) +
                   CSettingsSet::kDefaultDelim + "histogram",
                   hist);
}


//
// class CPaneContentMediator
//

/*CPaneContentMediator::CPaneContentMediator(CSettingsSet& data, CFeatShowPanel& panel) :
    CConfigMediator(data, panel)
{}

CPaneContentMediator::~CPaneContentMediator()
{}


/// What should we title the dialog window?
string  CPaneContentMediator::GetWindowTitle(void) const
{
    return "Features to Show";
}

/// Load data into that panel
void CPaneContentMediator::Load(const string& state)
{
    if ( ! state.empty()) { // if state is empty use the current style in the data.
        m_DataSet->SetCurrentStyleName(state);
    }

    CFeatShowPanel& my_gui = x_GetMyPanel();
    x_Load(*m_DataSet, my_gui);
}

/// Load 'factory default' data into that panel.
void CPaneContentMediator:: LoadDefault(void)
{
    CFeatShowPanel& my_gui = x_GetMyPanel();

    m_DataSet->SetDefaultCurrentStyle();
    x_Load(*m_DataSet, my_gui);
    m_DataSet->RestoreCurrentStyle();
}


void CPaneContentMediator::x_Load(CSettingsSet& settings, CFeatShowPanel& widget_holder)
{
    CCheckBrowser* widgets = dynamic_cast<CCheckBrowser *>(widget_holder.GetConfigPanel());
    _ASSERT(widgets);

    CPaneContentConf data(settings);

    int browser_row = 1;
    for (; browser_row <= widget_holder.GetNumExtras() && browser_row <= widgets->size(); ++browser_row) {
        CPaneContentConf::EShowAnnotType show_what =  *(CPaneContentConf::EShowAnnotType *) widgets->data(browser_row);
        widgets->SetCheck(browser_row, data.GetShowAnnot(show_what) );
    }
    for (; browser_row <= widgets->size(); ++browser_row) {
        CFeatListItem* feat_info = (CFeatListItem*) widgets->data(browser_row);
        widgets->SetCheck(browser_row,
                data.GetShow(feat_info->GetSubtype()));
    }
}


/// Save the panel's settings into the configuration data.
void CPaneContentMediator::Save(const string&)
{
    CPaneContentConf data_set(*m_DataSet);
    CFeatShowPanel& widget_holder = x_GetMyPanel();
    CCheckBrowser* widgets = dynamic_cast<CCheckBrowser *>(widget_holder.GetConfigPanel());
    _ASSERT(widgets);

    int browser_row = 1;
    for (; browser_row <= widget_holder.GetNumExtras() && browser_row <= widgets->size(); ++browser_row) {
        CPaneContentConf::EShowAnnotType show_what =  *(CPaneContentConf::EShowAnnotType *) widgets->data(browser_row);
        data_set.SetShowAnnot(widgets->GetCheck(browser_row), show_what);
    }
    for (; browser_row <= widgets->size(); ++browser_row) {
        CFeatListItem* feat_info = (CFeatListItem*) widgets->data(browser_row);
        data_set.SetShow(feat_info->GetSubtype(),
                         widgets->GetCheck(browser_row) );
    }
}

/// Has the data represented by the panel's widgets been modified?
bool CPaneContentMediator::Modified(void) const
{
    return false;
}

// the following accessors are just to hide the ugly down casts.

CFeatShowPanel& CPaneContentMediator::x_GetMyPanel(void)
{
    return dynamic_cast<CFeatShowPanel&>(*m_ConfigPanel);
}
*/
END_NCBI_SCOPE
