/*  $Id: segment_smear_glyph.cpp 27503 2013-02-26 18:16:55Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Liangshou Wu
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/seq_graphic/segment_smear_glyph.hpp>
#include <gui/widgets/seq_graphic/segment_config.hpp>
#include <gui/widgets/seq_graphic/rendering_ctx.hpp>
#include <gui/opengl/glrender.hpp>


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CSegmentSmearGlyph::CSegmentSmearGlyph(auto_ptr<TSegMap> map)
    : m_SegMap(map)
{}


bool CSegmentSmearGlyph::NeedTooltip(const TModelPoint& /*p*/, string& /*tt*/, string& /*t_title*/) const
{
    return false;
}

void CSegmentSmearGlyph::GetTooltip(const TModelPoint& /*p*/, string& /*tt*/, string& /*t_title*/) const
{
}


bool CSegmentSmearGlyph::IsClickable() const
{
    return true;
}


void CSegmentSmearGlyph::x_Draw() const
{
    if ( !m_SegMap.get() ) return;

    CGlRender& gl = GetGl();

    TModelUnit top = GetTop();
    TModelUnit bottom = top + GetHeight();
    TSegMap::runlen_iterator seg_it = m_SegMap->RunLenBegin();
    for (; seg_it; seg_it.Advance()) {
        TValueType value = seg_it.GetValue();
        _ASSERT(value < m_Colors.size());
        TSeqPos f = seg_it.GetSeqPosition();
        TSeqPos t = seg_it.GetSeqRunEndPosition();
        if (value == (TValueType)CSegmentConfig::eGap) {
            gl.ColorC(m_Colors[value]);
            m_Context->DrawQuad(f, top, t, bottom);
        } else {
            m_Context->Draw3DQuad(f, top, t, bottom, m_Colors[value], false);
        }
    }
}


void CSegmentSmearGlyph::x_UpdateBoundingBox()
{
    CRef<CSGSequenceDS> seq_ds = m_Context->GetSeqDS();
    m_Width  = seq_ds->GetSequenceLength();
    m_Pos.m_X = 0.0;
}


END_NCBI_SCOPE
