/*  $Id: seqgraphic_preview.cpp 17877 2008-09-24 13:03:09Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Robert G. Smith
 *
 */


#include <ncbi_pch.hpp>

#include <gui/widgets/seq_graphic/seqgraphic_render.hpp>
#include <gui/widgets/seq_graphic/seqgraphic_ds.hpp>
#include <gui/widgets/seq_graphic/seqgraphic_conf.hpp>


#include <gui/widgets/seq_graphic/seqgraphic_preview.hpp>

BEGIN_NCBI_SCOPE

CSeqGraphicPreview::CSeqGraphicPreview(int x, int y, int wid, int ht)
    : CGlCanvas(x, y, wid, ht, 0)
{
    m_Renderer.Reset(new CSeqGraphicRenderer());
    resizable(this);
}

CSeqGraphicPreview::CSeqGraphicPreview(wxWindow* parent, wxWindowID id,
                                   const wxPoint& pos, const wxSize& size, long style)
: CGlWidgetBase(parent, id, pos, size, style),
  //m_DataSource(NULL),
  m_SeqGraphicPane(NULL)
{
    // setup Port
    m_Port.SetAdjustmentPolicy(CGlPane::fAdjustAll, CGlPane::fAdjustAll);
    m_Port.SetMinScaleX(1 / 30.0);
    m_Port.SetOriginType(CGlPane::eOriginLeft, CGlPane::eOriginTop);
    m_Port.EnableZoom(true, true);
}





CSeqGraphicPreview::~CSeqGraphicPreview()
{
}


// called by CGlCanvas::draw()
void CSeqGraphicPreview::draw(void)
{
    if (!valid()) {
        TVPRect rc(0, 0, w() - 0, h() - 0);
        m_Renderer->Resize(rc);
    }

    glClearColor(1, 1, 1, 0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    if ( !m_DS ) {
        return;
    }
    m_Renderer->ClearDisplayList();
    m_Renderer->Render();
}


void CSeqGraphicPreview::SetDataSource(CSeqGraphicDataSource* ds)
{
    TVPRect rc(0, 0, w() - 0, h() - 0);
    m_Renderer->SetDataSource(rc, ds);
    m_DS.Reset(ds);
    m_DS->SetBackgroundLoading(false);

    invalidate();
    redraw();
}


const CSeqGraphicDataSource* CSeqGraphicPreview::GetDataSource(void) const
{
    return m_DS.GetPointer();
}


void CSeqGraphicPreview::SetConfig(CRef<CSeqGraphicConfig> config)
{
    m_ConfigSettings = config;
    m_Renderer->SetConfig(m_ConfigSettings);
}


const CSeqGraphicConfig* CSeqGraphicPreview::GetConfig()
{
    return m_ConfigSettings.GetPointer();
}


void CSeqGraphicPreview::UpdateConfig()
{
    m_ConfigSettings->Uncache();
    m_Renderer->UpdateConfig();
    redraw();
}


END_NCBI_SCOPE
