/*  $Id: wx_save_pdf_dlg.cpp 28186 2013-05-21 21:02:13Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Liangshou Wu
 *
 * File Description:
 *
 */
#include <ncbi_pch.hpp>////@begin includes
////@end includes

#include "wx_save_pdf_dlg.hpp"
#include <wx/filename.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/panel.h>
#include <gui/widgets/wx/regexp_validator.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/image_grabber.hpp>
#include <gui/widgets/wx/glframebuffer.hpp>
#include <gui/widgets/wx/message_box.hpp>
#include <gui/widgets/wx/ind_progress_bar.hpp>
#include <gui/widgets/seq_graphic/sequence_pdf_grabber.hpp>
#include <gui/objutils/registry.hpp>
#include <gui/objutils/utils.hpp>
#include <corelib/ncbifile.hpp>
#include "seqgraphic_pane.hpp"

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

static TVPUnit kMaxPdfSize = 14400;

IMPLEMENT_DYNAMIC_CLASS( CwxSavePdfDlg, wxDialog )

BEGIN_EVENT_TABLE( CwxSavePdfDlg, wxDialog )

////@begin CwxSavePdfDlg event table entries
    EVT_BUTTON( ID_FILEPATH, CwxSavePdfDlg::OnFilepathClick )

    EVT_BUTTON( ID_SAVE, CwxSavePdfDlg::OnSaveClick )

    EVT_BUTTON( ID_OPEN, CwxSavePdfDlg::OnOpenClick )

    EVT_BUTTON( ID_CANCEL, CwxSavePdfDlg::OnCancelClick )

////@end CwxSavePdfDlg event table entries
    EVT_TIMER(-1, CwxSavePdfDlg::OnTimer)

END_EVENT_TABLE()

CwxSavePdfDlg::CwxSavePdfDlg()
    : m_Timer(this)
    , m_Pane(NULL)
    , m_ProgressBar(NULL)
{
    Init();
}

CwxSavePdfDlg::CwxSavePdfDlg(CSeqGraphicPane* pane,
                             wxWindow *parent)
    : m_Timer(this)
    , m_Pane(pane)
{
    Init();
    Create(parent);

}

bool CwxSavePdfDlg::Create( wxWindow* parent, wxWindowID id,
                           const wxString& caption,
                           const wxPoint& pos,
                           const wxSize& size,
                           long style )
{
////@begin CwxSavePdfDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    Centre();
////@end CwxSavePdfDlg creation
    return true;
}

CwxSavePdfDlg::~CwxSavePdfDlg()
{
////@begin CwxSavePdfDlg destruction
////@end CwxSavePdfDlg destruction
    SaveSettings();
}

void CwxSavePdfDlg::Init()
{
////@begin CwxSavePdfDlg member initialisation
    m_SeqRange = _("0:0");
    m_Panel = NULL;
    m_FileNameBox = NULL;
    m_SeqRangeBox = NULL;
    m_InfoSizer = NULL;
    m_Info = NULL;
    m_SaveButton = NULL;
    m_OpenButton = NULL;
////@end CwxSavePdfDlg member initialisation
    m_ReadyToSave = false;
    m_OpenPdf = false;
    LoadSettings();
}

void CwxSavePdfDlg::CreateControls()
{    
////@begin CwxSavePdfDlg content construction
    CwxSavePdfDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    m_Panel = new wxPanel( itemDialog1, ID_PANEL3, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    m_Panel->SetName(_T("aaaaa"));
    itemBoxSizer2->Add(m_Panel, 0, wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    m_Panel->SetSizer(itemBoxSizer4);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(itemBoxSizer5, 0, wxGROW|wxLEFT|wxRIGHT|wxTOP, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( m_Panel, wxID_STATIC, _("Save As:"), wxDefaultPosition, wxSize(100, -1), wxALIGN_RIGHT );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_FileNameBox = new wxTextCtrl( m_Panel, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxSize(200, -1), 0 );
    itemBoxSizer5->Add(m_FileNameBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( m_Panel, ID_FILEPATH, _("..."), wxDefaultPosition, wxDefaultSize, 0 );
    itemButton8->SetName(_T("FilePath"));
    itemBoxSizer5->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer9 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(itemBoxSizer9, 0, wxGROW|wxLEFT|wxRIGHT|wxBOTTOM, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( m_Panel, wxID_STATIC, _("Sequence Range:"), wxDefaultPosition, wxSize(100, -1), wxALIGN_RIGHT );
    itemBoxSizer9->Add(itemStaticText10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_SeqRangeBox = new wxTextCtrl( m_Panel, ID_TEXTCTRL6, wxEmptyString, wxDefaultPosition, wxSize(200, -1), 0 );
    m_SeqRangeBox->SetHelpText(_("Range examples: 1000-2000;  1000..2000; 10,000:2,000,000;  100k:1m;  1000[space]2000; 1000[tab]2000"));
    if (CwxSavePdfDlg::ShowToolTips())
        m_SeqRangeBox->SetToolTip(_("Range examples: 1000-2000;  1000..2000; 10,000:2,000,000;  100k:1m;  1000[space]2000; 1000[tab]2000"));
    itemBoxSizer9->Add(m_SeqRangeBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer9->Add(5, 5, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_InfoSizer = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(m_InfoSizer, 0, wxGROW|wxALL, 5);

    m_Info = new wxStaticText( itemDialog1, wxID_STATIC, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_InfoSizer->Add(m_Info, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    itemBoxSizer2->Add(5, 1, 1, wxALIGN_CENTER_HORIZONTAL|wxALL, 1);

    wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer16, 0, wxALIGN_RIGHT|wxLEFT|wxRIGHT|wxBOTTOM, 5);

    m_SaveButton = new wxButton( itemDialog1, ID_SAVE, _("Save"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer16->Add(m_SaveButton, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

    m_OpenButton = new wxButton( itemDialog1, ID_OPEN, _("Save && Open"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer16->Add(m_OpenButton, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

    wxButton* itemButton19 = new wxButton( itemDialog1, ID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer16->Add(itemButton19, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

    // Set validators
    m_SeqRangeBox->SetValidator( CRegexpValidator("^[ \t]*[1-9][0-9,]*[ \t]*[kKmM]?((([ \t]*([-:]|\\.\\.)[ \t]*)|([ \t]+))[1-9][0-9,]*[ \t]*[kKmM]?)?[ \t]*$","1000\n1000-2000\n1000..2000\n10,000:2,000,000\n100k:1m\n1000 2000\n1000    2000" , & m_SeqRange) );
////@end CwxSavePdfDlg content construction
}

void CwxSavePdfDlg::SetSeqRange(TSeqPos from, TSeqPos to)
{
    m_OrgRange.Set(from, to);
    string range_str = NStr::NumericToString(from + 1, NStr::fWithCommas);
    range_str += "-";
    range_str += NStr::NumericToString(to, NStr::fWithCommas);
    m_SeqRange = ToWxString(range_str);
    //m_SeqRangeBox->SetValue(m_SeqRange);
    m_Panel->TransferDataToWindow();

    CConstRef<objects::CSeq_id> id =
        m_Pane->GetDataSource()->GetBioseqHandle().GetSeqId();
    id->GetLabel(&m_FileName, objects::CSeq_id::eContent);
    wxString path = ToWxString(m_Path);
    path += wxFileName::GetPathSeparator();
    path += ToStdString(m_FileName + ".pdf");
    m_FileNameBox->SetValue(path);
}


void CwxSavePdfDlg::SetViewport(const TVPRect& vp)
{
    m_VP = vp;
}


bool CwxSavePdfDlg::ShowToolTips()
{
    return true;
}

wxBitmap CwxSavePdfDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxSavePdfDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxSavePdfDlg bitmap retrieval
}
wxIcon CwxSavePdfDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxSavePdfDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxSavePdfDlg icon retrieval
}


void CwxSavePdfDlg::OnOpenClick( wxCommandEvent& event )
{
    m_OpenPdf = true;
    OnSaveClick(event);
}


void CwxSavePdfDlg::OnSaveClick( wxCommandEvent& event )
{
    if ( !m_Pane ) return;

    if ( !Validate() ) {
        NcbiMessageBox("The input range is invlaid!");
        return;
    }

    m_Panel->TransferDataFromWindow();
    TSeqPos from = 0, to = 0;

    // validate sequence range
    if ( !x_GetSeqRange(from, to)  ||  from >= to) {
        NcbiMessageBox("The input range is invlaid!");
        return;
    }

    // validate file path
    wxFileName file(m_FileNameBox->GetValue());
    wxString filename = file.GetFullName();
    size_t pos = filename.find_last_of(_T(".pdf"));
    if (pos < filename.length()) {
        filename = filename.SubString(0, pos - 4);
    }
    wxString path = file.GetPath();
    if (filename.IsEmpty()  ||  path.IsEmpty()  ||
        !wxFileName::DirExists(path) ) {
        NcbiMessageBox("The given directory doesn't exist!");
        return;
    }

    TVPUnit new_width = 
        (TVPUnit)((double)m_VP.Width() / m_OrgRange.GetLength() * (to - from + 1));

    if (new_width > kMaxPdfSize) {
        string msg = "The Image is too wide. The projected width is ";
        msg += NStr::NumericToString(new_width);
        msg += " which is over the limit (";
        msg += NStr::NumericToString(kMaxPdfSize, NStr::fWithCommas);
        msg += "pixels) permitted!";
        NcbiMessageBox(msg);
        return;
    }

    m_Path = ToStdString(path);
    m_FileName = ToStdString(filename);

    m_Pane->SetViewportWidth(from, to, new_width);    
    m_Pane->AdjustViewPortHeightToImage();
    m_Pane->UpdateData(from, to);

    // add a progress bar to indicate that data are being updated
    m_Info->SetLabel(_("Preparing data...  Please Wait!"));
    m_ProgressBar = new CIndProgressBar(this, wxID_ANY, wxDefaultPosition, 240);
    m_InfoSizer->Add(m_ProgressBar, 0, wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxBOTTOM, 5);
    m_InfoSizer->Layout();
    Layout();
    m_Panel->Enable(false);
    m_OpenButton->Enable(false);
    m_SaveButton->Enable(false);

    // start the timer to check if the data loading is finished
    m_Timer.Start(300);
}


void CwxSavePdfDlg::OnTimer(wxTimerEvent&)
{
    if (m_ReadyToSave) {
        m_Timer.Stop();
        m_ReadyToSave = false;
        m_Pane->AdjustViewPortHeightToImage();
        CSequencePdfGrabber grb_pdf(*m_Pane);
        grb_pdf.SetOutputInfo(m_Path, m_FileName, "pdf");
        IImageGrabber::eCaptureResult res =
            grb_pdf.GrabImages(0/*dummy value, not used currently*/);

        // Set the cursor to its value prior to the save
        SetCursor(*wxSTANDARD_CURSOR);
        if (res == IImageGrabber::eOpenGLError) {
            wxMessageBox(wxT("Graphics system does not support in-memory image save feature"), 
                wxT("Error Saving"), wxOK);
        } else if (res == IImageGrabber::eFileError) {
            wxMessageBox(wxT("File error occured during save. Make sure file and directory names are valid."), 
                wxT("Error Saving"), wxOK);
        } else if (m_OpenPdf) {
            wxString full_path = m_FileNameBox->GetValue();
            wxLaunchDefaultApplication(full_path);
            m_OpenPdf = false;
        }
        EndModal(wxID_CANCEL);
    } else if (m_Pane->AllJobsFinished()) {
        m_Timer.Stop();
        m_Info->SetLabel(_("Saving image to PDF..."));
        m_ProgressBar->Show(false);
        m_Timer.Start(100);
        m_ReadyToSave = true;
        SetCursor(*wxHOURGLASS_CURSOR);
    }
}

static const string kSavePdfBaseKey("GBPlugins.SavePdfDialog");
static const string kPdfPathKey = "PdfPath";

void CwxSavePdfDlg::LoadSettings()
{
    CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
    CGuiRegistry::TReadView view = gui_reg.GetReadView(kSavePdfBaseKey);
    string path = view.GetString(kPdfPathKey, "");
    if ( !path.empty()  &&  CDirEntry(path).Exists() ) {
        m_Path = path;
    } else {
        // create a default path under user folder
        m_Path = ToStdString(wxGetHomeDir());
    }
}


void CwxSavePdfDlg::SaveSettings() const
{
    if ( !m_Path.empty() ) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        CGuiRegistry::TReadWriteView view = gui_reg.GetReadWriteView(kSavePdfBaseKey);
        view.Set(kPdfPathKey, m_Path);
    }
}


void CwxSavePdfDlg::OnFilepathClick( wxCommandEvent& event )
{
    wxFileDialog dlg(this, wxT("Set File Name"),
        ToWxString(m_Path), ToWxString(m_FileName + ".pdf"),
        wxT("*.pdf"), wxFD_SAVE);

    if (dlg.ShowModal() != wxID_OK)
        return;

    wxString filename = dlg.GetPath();
    m_FileNameBox->SetValue(filename);
}


void CwxSavePdfDlg::OnCancelClick( wxCommandEvent& event )
{
    // Cancel all jobs
    m_Timer.Stop();
    EndModal(wxID_CANCEL);
}


bool CwxSavePdfDlg::x_GetSeqRange(TSeqPos& from, TSeqPos& to) const
{
    string pos_str = ToStdString(m_SeqRange);
    long f, t;
    bool valid = CSeqUtils::StringToRange(pos_str, f, t);
    if (valid  &&  f > 0  &&  t > 0) {
        from = (TSeqPos)(f - 1);
        to = (TSeqPos)t;
        return true;
    }
    return false;
}

END_NCBI_SCOPE

