/*  $Id: wx_save_pdf_dlg.hpp 28186 2013-05-21 21:02:13Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Liangshou Wu
 *
 * File Description:
 *
 */
#ifndef _WX_SAVE_PDF_DLG_H_
#define _WX_SAVE_PDF_DLG_H_


/*!
 * Includes
 */

////@begin includes
////@end includes

#include <corelib/ncbiobj.hpp>
#include <wx/dialog.h>
#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/timer.h>
#include <gui/opengl/glpane.hpp>


////@begin forward declarations
class wxBoxSizer;
////@end forward declarations
class wxPanel;

/*!
 * Control identifiers
 */

BEGIN_NCBI_SCOPE

/*!
 * Forward declarations
 */

class CSeqGraphicPane;
class CIndProgressBar;


////@begin control identifiers
#define ID_CWXSAVEPDFDLG 10048
#define ID_PANEL3 10052
#define ID_TEXTCTRL5 10049
#define ID_FILEPATH 10050
#define ID_TEXTCTRL6 10051
#define ID_SAVE 10054
#define ID_OPEN 10053
#define ID_CANCEL 10055
#define SYMBOL_CWXSAVEPDFDLG_STYLE wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CWXSAVEPDFDLG_TITLE _("Save Image As PDF")
#define SYMBOL_CWXSAVEPDFDLG_IDNAME ID_CWXSAVEPDFDLG
#define SYMBOL_CWXSAVEPDFDLG_SIZE wxSize(440, 190)
#define SYMBOL_CWXSAVEPDFDLG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CwxSavePdfDlg class declaration
 */

class CwxSavePdfDlg: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CwxSavePdfDlg )
    DECLARE_EVENT_TABLE()

public:
    CwxSavePdfDlg();

    CwxSavePdfDlg(CSeqGraphicPane* pane, wxWindow *parent = NULL);

    bool Create( wxWindow* parent,
        wxWindowID id = SYMBOL_CWXSAVEPDFDLG_IDNAME,
        const wxString& caption = SYMBOL_CWXSAVEPDFDLG_TITLE,
        const wxPoint& pos = SYMBOL_CWXSAVEPDFDLG_POSITION,
        const wxSize& size = SYMBOL_CWXSAVEPDFDLG_SIZE,
        long style = SYMBOL_CWXSAVEPDFDLG_STYLE );

    ~CwxSavePdfDlg();

    void Init();

    void CreateControls();

    void SetSeqRange(TSeqPos from, TSeqPos to);
    void SetViewport(const TVPRect& vp);

    void OnTimer(wxTimerEvent&);

    void LoadSettings();
    void SaveSettings() const;

////@begin CwxSavePdfDlg event handler declarations

    void OnFilepathClick( wxCommandEvent& event );

    void OnSaveClick( wxCommandEvent& event );

    void OnOpenClick( wxCommandEvent& event );

    void OnCancelClick( wxCommandEvent& event );

////@end CwxSavePdfDlg event handler declarations

////@begin CwxSavePdfDlg member function declarations

    wxString GetSeqRange() const { return m_SeqRange ; }
    void SetSeqRange(wxString value) { m_SeqRange = value ; }

    wxBitmap GetBitmapResource( const wxString& name );

    wxIcon GetIconResource( const wxString& name );
////@end CwxSavePdfDlg member function declarations

    static bool ShowToolTips();

////@begin CwxSavePdfDlg member variables
    wxPanel* m_Panel;
    wxTextCtrl* m_FileNameBox;
    wxTextCtrl* m_SeqRangeBox;
    wxBoxSizer* m_InfoSizer;
    wxStaticText* m_Info;
    wxButton* m_SaveButton;
    wxButton* m_OpenButton;
private:
    wxString m_SeqRange;
////@end CwxSavePdfDlg member variables

private:

    bool x_GetSeqRange(TSeqPos& from, TSeqPos& to) const;

    wxTimer m_Timer;
    CSeqGraphicPane* m_Pane;
    CIndProgressBar* m_ProgressBar;
    string           m_Path;
    string           m_FileName;
    TSeqRange        m_OrgRange;
    TVPRect          m_VP;
    bool             m_ReadyToSave;
    /// view PDF file after finished.
    bool             m_OpenPdf;
};

END_NCBI_SCOPE

#endif
    // _WX_SAVE_PDF_DLG_H_
