/*  $Id: wx_settings_dlg.cpp 25628 2012-04-13 17:49:41Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include <corelib/ncbitime.hpp>
#include <corelib/ncbimisc.hpp>

#include <gui/widgets/wx/dialog_utils.hpp>
#include <gui/widgets/wx/color_picker_validator.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

#include <gui/config/settings_set.hpp>
#include <gui/widgets/seq_graphic/seqgraphic_widget.hpp>
//#include <gui/widgets/seq_graphic/seqgraphic_ds.hpp>

//#include <objects/seqfeat/Seq_feat.hpp>
//#include <objects/seqfeat/SeqFeatData.hpp>
//#include <gui/widgets/seq/feature_check_dialog.hpp>


#include <wx/icon.h>
#include <wx/bitmap.h>
#include <wx/statbox.h>
#include <wx/panel.h>

#include <list>
#include <algorithm>
#include <gui/utils/string_utils.hpp>

#include <objmgr/object_manager.hpp>
#include <objects/seqset/Seq_entry.hpp>
#include <objmgr/scope.hpp>
#include <serial/serial.hpp>
#include <serial/objistr.hpp>
#include <wx/statbox.h>
#include <wx/sizer.h>

#include "content_check_dialog.hpp"
#include "wx_settings_dlg.hpp"

////@begin XPM images
////@end XPM images

#include "preview_data.hpp"
const string kPreviewId("lcl|fake_nuc");

BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CwxSettingsDlg, wxDialog )

BEGIN_EVENT_TABLE( CwxSettingsDlg, wxDialog )

////@begin CwxSettingsDlg event table entries
    EVT_LISTBOX( ID_LISTBOX, CwxSettingsDlg::OnListboxSelected )

    EVT_HYPERLINK( ID_HYPERLINKCTRL1, CwxSettingsDlg::OnAddNewHyperlinkClicked )

    EVT_HYPERLINK( ID_HYPERLINKCTRL2, CwxSettingsDlg::OnRemoveHyperlinkClicked )

    EVT_HYPERLINK( ID_HYPERLINKCTRL3, CwxSettingsDlg::OnDuplicateHyperlinkClicked )

    EVT_CHOICE( ID_CHOICE, CwxSettingsDlg::OnChoiceSelected )

    EVT_CHOICE( ID_CHOICE6, CwxSettingsDlg::OnChoiceSelected )

    EVT_CHOICE( ID_CHOICE7, CwxSettingsDlg::OnChoiceSelected )

    EVT_CHOICE( ID_CHOICE8, CwxSettingsDlg::OnChoiceSelected )

    EVT_CHOICE( ID_CHOICE9, CwxSettingsDlg::OnChoiceSelected )

    EVT_BUTTON( ID_BUTTON5, CwxSettingsDlg::OnButtonFeatureSelect )

    EVT_CHECKBOX( ID_CHECKBOX3, CwxSettingsDlg::OnSideLabelClick )

    EVT_BUTTON( ID_BUTTON, CwxSettingsDlg::OnButtonClick )

    EVT_BUTTON( wxID_OK, CwxSettingsDlg::OnOkClick )

    EVT_BUTTON( wxID_CANCEL, CwxSettingsDlg::OnCancelClick )

////@end CwxSettingsDlg event table entries

END_EVENT_TABLE()

CwxSettingsDlg::CwxSettingsDlg()
{
    Init();
}

CwxSettingsDlg::CwxSettingsDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

bool CwxSettingsDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CwxSettingsDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CwxSettingsDlg creation
    return true;
}

CwxSettingsDlg::~CwxSettingsDlg()
{
////@begin CwxSettingsDlg destruction
////@end CwxSettingsDlg destruction
}

void CwxSettingsDlg::Init()
{
////@begin CwxSettingsDlg member initialisation
    m_List = NULL;
    m_Decorations = NULL;
    m_Colors = NULL;
    m_Spacing = NULL;
    m_Layout = NULL;
    m_Content = NULL;
    m_SidedLabel = NULL;
    m_SeqWidget = NULL;
////@end CwxSettingsDlg member initialisation
}

void CwxSettingsDlg::CreateControls()
{
////@begin CwxSettingsDlg content construction
    CwxSettingsDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxGROW|wxALL, 0);

    wxStaticBox* itemStaticBoxSizer4Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Themes"));
    wxStaticBoxSizer* itemStaticBoxSizer4 = new wxStaticBoxSizer(itemStaticBoxSizer4Static, wxHORIZONTAL);
    itemBoxSizer3->Add(itemStaticBoxSizer4, 0, wxGROW|wxALL, 5);

    wxArrayString m_ListStrings;
    m_List = new wxListBox( itemDialog1, ID_LISTBOX, wxDefaultPosition, wxSize(-1, 100), m_ListStrings, wxLB_SINGLE );
    itemStaticBoxSizer4->Add(m_List, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxVERTICAL);
    itemStaticBoxSizer4->Add(itemBoxSizer6, 0, wxGROW|wxALL, 0);

    wxHyperlinkCtrl* itemHyperlinkCtrl7 = new wxHyperlinkCtrl( itemDialog1, ID_HYPERLINKCTRL1, _("Add New"), wxEmptyString, wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
    itemBoxSizer6->Add(itemHyperlinkCtrl7, 0, wxALIGN_LEFT|wxALL, 5);

    wxHyperlinkCtrl* itemHyperlinkCtrl8 = new wxHyperlinkCtrl( itemDialog1, ID_HYPERLINKCTRL2, _("Remove"), wxEmptyString, wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
    itemBoxSizer6->Add(itemHyperlinkCtrl8, 0, wxALIGN_LEFT|wxALL, 5);

    wxHyperlinkCtrl* itemHyperlinkCtrl9 = new wxHyperlinkCtrl( itemDialog1, ID_HYPERLINKCTRL3, _("Duplicate"), wxEmptyString, wxDefaultPosition, wxDefaultSize, wxHL_DEFAULT_STYLE );
    itemHyperlinkCtrl9->Show(false);
    itemBoxSizer6->Add(itemHyperlinkCtrl9, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer10Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Theme Options"));
    wxStaticBoxSizer* itemStaticBoxSizer10 = new wxStaticBoxSizer(itemStaticBoxSizer10Static, wxVERTICAL);
    itemBoxSizer3->Add(itemStaticBoxSizer10, 1, wxGROW|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer11 = new wxFlexGridSizer(2, 4, 0, 0);
    itemStaticBoxSizer10->Add(itemFlexGridSizer11, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText12 = new wxStaticText( itemDialog1, wxID_STATIC, _("Decorations"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer11->Add(itemStaticText12, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_DecorationsStrings;
    m_Decorations = new wxChoice( itemDialog1, ID_CHOICE, wxDefaultPosition, wxDefaultSize, m_DecorationsStrings, 0 );
    itemFlexGridSizer11->Add(m_Decorations, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText14 = new wxStaticText( itemDialog1, wxID_STATIC, _("Colors"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer11->Add(itemStaticText14, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_ColorsStrings;
    m_Colors = new wxChoice( itemDialog1, ID_CHOICE6, wxDefaultPosition, wxDefaultSize, m_ColorsStrings, 0 );
    itemFlexGridSizer11->Add(m_Colors, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText16 = new wxStaticText( itemDialog1, wxID_STATIC, _("Spacing"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer11->Add(itemStaticText16, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_SpacingStrings;
    m_Spacing = new wxChoice( itemDialog1, ID_CHOICE7, wxDefaultPosition, wxDefaultSize, m_SpacingStrings, 0 );
    itemFlexGridSizer11->Add(m_Spacing, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText18 = new wxStaticText( itemDialog1, wxID_STATIC, _("Layout"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer11->Add(itemStaticText18, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_LayoutStrings;
    m_Layout = new wxChoice( itemDialog1, ID_CHOICE8, wxDefaultPosition, wxDefaultSize, m_LayoutStrings, 0 );
    itemFlexGridSizer11->Add(m_Layout, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText20 = new wxStaticText( itemDialog1, wxID_STATIC, _("Content"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer11->Add(itemStaticText20, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_ContentStrings;
    m_Content = new wxChoice( itemDialog1, ID_CHOICE9, wxDefaultPosition, wxDefaultSize, m_ContentStrings, 0 );
    itemFlexGridSizer11->Add(m_Content, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton22 = new wxButton( itemDialog1, ID_BUTTON5, _("..."), wxDefaultPosition, wxSize(25, -1), 0 );
    itemFlexGridSizer11->Add(itemButton22, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer23 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer11->Add(itemBoxSizer23, 1, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5);

    m_SidedLabel = new wxCheckBox( itemDialog1, ID_CHECKBOX3, _("Side label"), wxDefaultPosition, wxDefaultSize, 0 );
    m_SidedLabel->SetValue(false);
    itemBoxSizer23->Add(m_SidedLabel, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton25 = new wxButton( itemDialog1, ID_BUTTON, _("Set defaults"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer23->Add(itemButton25, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer11->AddGrowableRow(0);
    itemFlexGridSizer11->AddGrowableRow(1);
    itemFlexGridSizer11->AddGrowableRow(2);
    itemFlexGridSizer11->AddGrowableCol(1);
    itemFlexGridSizer11->AddGrowableCol(3);

    wxStaticBox* itemStaticBoxSizer26Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Preview"));
    wxStaticBoxSizer* itemStaticBoxSizer26 = new wxStaticBoxSizer(itemStaticBoxSizer26Static, wxHORIZONTAL);
    itemBoxSizer2->Add(itemStaticBoxSizer26, 1, wxGROW|wxALL, 5);

    m_SeqWidget = new CSeqGraphicWidget( itemDialog1, ID_PANEL1, wxDefaultPosition, wxSize(400, 300), wxSTATIC_BORDER|wxTAB_TRAVERSAL );
    m_SeqWidget->SetBackgroundColour(wxColour(255, 255, 255));
    itemStaticBoxSizer26->Add(m_SeqWidget, 1, wxGROW|wxALL, 5);

    wxStaticLine* itemStaticLine28 = new wxStaticLine( itemDialog1, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemBoxSizer2->Add(itemStaticLine28, 0, wxGROW|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer29 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer29, 0, wxALIGN_RIGHT|wxALL, 5);
    wxButton* itemButton30 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer29->AddButton(itemButton30);

    wxButton* itemButton31 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer29->AddButton(itemButton31);

    itemStdDialogButtonSizer29->Realize();

////@end CwxSettingsDlg content construction

	  m_SeqWidget->Create();
	
}

bool CwxSettingsDlg::ShowToolTips()
{
    return true;
}
wxBitmap CwxSettingsDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxSettingsDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxSettingsDlg bitmap retrieval
}
wxIcon CwxSettingsDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxSettingsDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxSettingsDlg icon retrieval
}

void CwxSettingsDlg::SetConfig(CRef<CSeqGraphicConfig> cfg)
{		
	m_Cfg.Reset(cfg);	
	m_Cfg->MakeTempData();
	
	x_UpdateThemes();
	x_UpdateThemeOptions();
	
	
	
	m_SeqWidget->SetConfig(m_Cfg);
	m_SeqWidget->SetDataSource(&x_GetFakeDataSource());
	m_SeqWidget->Disable();
}



void CwxSettingsDlg::OnListboxSelected( wxCommandEvent& event )
{
	wxArrayInt sel;
	m_List->GetSelections(sel);
	if (sel.size() == 1) {		
		std::list<std::string> lst = m_Cfg->GetStyleNames();
		std::list<std::string>::iterator it = lst.begin();
		while(sel[0]--) it++;

		m_Cfg->Uncache();
		m_Cfg->SetCurrentStyleName(*it);
				
		x_UpdateThemeOptions();		
		x_UpdateConfig();
	}
}



void CwxSettingsDlg::x_UpdateConfig()
{
	m_SeqWidget->SetConfig(m_Cfg);		

	m_Cfg->SetSetsCurrentStyleName("GraphicalViewContent",
                                   ToStdString(m_Content->GetStringSelection()));
    m_Cfg->SetSetsCurrentStyleName("GraphicalViewColors",
                                   ToStdString(m_Colors->GetStringSelection()));
    m_Cfg->SetSetsCurrentStyleName("FeatureDecorate",
                                   ToStdString(m_Decorations->GetStringSelection()));
    m_Cfg->SetSetsCurrentStyleName("Layout",
                                   ToStdString(m_Layout->GetStringSelection()));
    m_Cfg->SetSetsCurrentStyleName("Sizes",
                                   ToStdString(m_Spacing->GetStringSelection()));
		
	m_SeqWidget->Refresh();	
}

void CwxSettingsDlg::x_UpdateThemeOptions()
{
	// filling decorations combo	
	m_Decorations->Freeze();
	m_Colors->Freeze();
	m_Content->Freeze();
	m_Layout->Freeze();
	m_Spacing->Freeze();

	m_Decorations->Clear();
	m_Colors->Clear();
	m_Content->Clear();
	m_Layout->Clear();
	m_Spacing->Clear();	

	std::vector<std::string> types = m_Cfg->GetTypes();
	for (std::vector<std::string>::iterator type = types.begin(); type!=types.end(); type++){
		std::string				curVal  = m_Cfg->GetSetsCurrentStyleName(*type);
		std::list<std::string>  valLst  = m_Cfg->GetSetsStyles(*type);		

		for (std::list<std::string>::iterator it=valLst.begin();  it!=valLst.end(); it++) {		
			if (*type == "FeatureDecorate") {						
				m_Decorations->Append(ToWxString(*it));
				if (*it == curVal) {
					m_Decorations->Select(m_Decorations->GetCount()-1);					
				}
			}			
			else if (*type == "GraphicalViewColors") {						
				m_Colors->Append(ToWxString(*it));
				if (*it == curVal) {
					m_Colors->Select(m_Colors->GetCount()-1);
				}
			}			
			else if (*type == "GraphicalViewContent") {						
				m_Content->Append(ToWxString(*it));
				if (*it == curVal) {
					m_Content->Select(m_Content->GetCount()-1);
				}
			}			
			else if (*type == "Layout") {						
				m_Layout->Append(ToWxString(*it));
				if (*it == curVal) {
					m_Layout->Select(m_Layout->GetCount()-1);
				}
			}			
			else if (*type == "Sizes") {						
				m_Spacing->Append(ToWxString(*it));
				if (*it == curVal) {
					m_Spacing->Select(m_Spacing->GetCount()-1);
				}
			}			
		}	
	}	

	m_Decorations->Thaw();
	m_Colors->Thaw();
	m_Content->Thaw();
	m_Layout->Thaw();
	m_Spacing->Thaw();

}

void CwxSettingsDlg::x_UpdateThemes()
{
	m_List->Clear();
	// list of styles
	wxArrayString arrStr;
	string currStyle = m_Cfg->GetCurrentStyleName();
	// no idea why NON_CONST_ITERATE is not working here throwing runtime
	// exception on MSVC8
	std::list<std::string> lst = m_Cfg->GetStyleNames();
	int selItem = wxNOT_FOUND;
	for (std::list<std::string>::iterator it=lst.begin();
	   it!=lst.end(); it++) {		
		arrStr.Add(ToWxString(*it));
		if (*it == currStyle) {
			selItem = arrStr.size() - 1;
		}
	}	
	m_List->Set(arrStr);
	m_List->SetSelection(selItem);
}


CSeqGraphicDataSource& CwxSettingsDlg::x_GetFakeDataSource()
{
    USING_SCOPE(objects);

    if (! m_PreviewDS) {
        // read in data from a static memory into a seq entry.
        string preview_data_str = CStringUtil::BuildString(s_PreviewData);
        CRef<CSeq_entry> preview_TSE(new CSeq_entry);

        CNcbiIstrstream is(preview_data_str.c_str(), strlen(preview_data_str.c_str()));
        auto_ptr<CObjectIStream>
            pd_in(CObjectIStream::Open(eSerial_AsnText, is));
        *pd_in >> *preview_TSE;
        _TRACE("CSettings::x_GetFakeDataSource(): "
               "text preview data source read from string: \""
               << preview_data_str.substr(0,20) << " ... \"" );

        CRef<CScope> preview_scope(new CScope(*CObjectManager::GetInstance()) );
        preview_scope->AddTopLevelSeqEntry(*preview_TSE);

        CSeq_id preview_id(kPreviewId);

        // create data source from the above.
        m_PreviewDS.Reset(new CSeqGraphicDataSource(*preview_scope, preview_id));
		m_PreviewDS->SetBackgroundLoading(false);
    }

    return *m_PreviewDS;
}

void CwxSettingsDlg::OnChoiceSelected( wxCommandEvent& event )
{
	m_Cfg->Uncache();
	x_UpdateConfig();
}

void CwxSettingsDlg::OnButtonClick( wxCommandEvent& event )
{	
	m_Cfg->Uncache();		
	m_Cfg->SetDefaultCurrentStyle();

	x_UpdateThemeOptions();	
	
	m_Cfg->RestoreCurrentStyle();	
	x_UpdateThemes();
	x_UpdateConfig();	
}


void CwxSettingsDlg::OnAddNewHyperlinkClicked( wxHyperlinkEvent& event )
{
	m_Cfg->AddStyle();
	x_UpdateThemes();
}


void CwxSettingsDlg::OnRemoveHyperlinkClicked( wxHyperlinkEvent& event )
{
	m_Cfg->DeleteStyle(m_Cfg->GetCurrentStyleName());
	x_UpdateThemes();
}

void CwxSettingsDlg::OnDuplicateHyperlinkClicked( wxHyperlinkEvent& event )
{
	m_Cfg->DuplicateStyle(m_Cfg->GetCurrentStyleName());	
	x_UpdateThemes();
}

void CwxSettingsDlg::OnButtonFeatureSelect( wxCommandEvent& event )
{	
    CContentCheckDialog dlg(NULL);
    dlg.SetConfig(m_Cfg);
    if(dlg.ShowModal() == wxID_OK) {
	    m_Cfg->Uncache();
	    x_UpdateConfig();
    }
}

void CwxSettingsDlg::OnOkClick( wxCommandEvent& event )
{		
	m_Cfg->MergeTempData();
	event.Skip();
}


void CwxSettingsDlg::OnCancelClick( wxCommandEvent& event )
{
	m_Cfg->CancelTempData();			
    event.Skip();
}


END_NCBI_SCOPE



void CwxSettingsDlg::OnSideLabelClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX3 in CwxSettingsDlg.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX3 in CwxSettingsDlg. 
}

