/*  $Id: wx_settings_dlg.hpp 22112 2010-09-22 21:14:41Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vlad Lebedev
 *
 * File Description:
 *
 */
#ifndef _WX_SETTINGS_DLG_H_
#define _WX_SETTINGS_DLG_H_


/*!
 * Includes
 */


#include <gui/widgets/seq_graphic/seqgraphic_conf.hpp>
#include <wx/dialog.h>
#include <wx/sizer.h>
#include <wx/clrpicker.h>
#include <wx/radiobox.h>
#include <wx/stattext.h>
#include <wx/listbox.h>
#include <wx/choice.h>

////@begin includes
#include "wx/hyperlink.h"
#include "wx/statline.h"
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class CSeqGraphicWidget;
////@end forward declarations

/*!
 * Control identifiers
 */


BEGIN_NCBI_SCOPE

////@begin control identifiers
#define ID_CWXSETTINGSDLG 10004
#define ID_LISTBOX 10010
#define ID_HYPERLINKCTRL1 10005
#define ID_HYPERLINKCTRL2 10006
#define ID_HYPERLINKCTRL3 10012
#define ID_CHOICE 10000
#define ID_CHOICE6 10001
#define ID_CHOICE7 10002
#define ID_CHOICE8 10003
#define ID_CHOICE9 10007
#define ID_BUTTON5 10009
#define ID_CHECKBOX3 10030
#define ID_BUTTON 10008
#define ID_PANEL1 10015
#define SYMBOL_CWXSETTINGSDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CWXSETTINGSDLG_TITLE _("Graphical View Preferences")
#define SYMBOL_CWXSETTINGSDLG_IDNAME ID_CWXSETTINGSDLG
#define SYMBOL_CWXSETTINGSDLG_SIZE wxSize(400, 600)
#define SYMBOL_CWXSETTINGSDLG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CwxSettingsDlg class declaration
 */

class CwxSettingsDlg: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CwxSettingsDlg )
    DECLARE_EVENT_TABLE()

public:
    CwxSettingsDlg();
    CwxSettingsDlg( wxWindow* parent, wxWindowID id = SYMBOL_CWXSETTINGSDLG_IDNAME, const wxString& caption = SYMBOL_CWXSETTINGSDLG_TITLE, const wxPoint& pos = SYMBOL_CWXSETTINGSDLG_POSITION, const wxSize& size = SYMBOL_CWXSETTINGSDLG_SIZE, long style = SYMBOL_CWXSETTINGSDLG_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CWXSETTINGSDLG_IDNAME, const wxString& caption = SYMBOL_CWXSETTINGSDLG_TITLE, const wxPoint& pos = SYMBOL_CWXSETTINGSDLG_POSITION, const wxSize& size = SYMBOL_CWXSETTINGSDLG_SIZE, long style = SYMBOL_CWXSETTINGSDLG_STYLE );

    ~CwxSettingsDlg();

    void Init();

    void CreateControls();

////@begin CwxSettingsDlg event handler declarations

    void OnListboxSelected( wxCommandEvent& event );

    void OnAddNewHyperlinkClicked( wxHyperlinkEvent& event );

    void OnRemoveHyperlinkClicked( wxHyperlinkEvent& event );

    void OnDuplicateHyperlinkClicked( wxHyperlinkEvent& event );

    void OnChoiceSelected( wxCommandEvent& event );

    void OnButtonFeatureSelect( wxCommandEvent& event );

    void OnSideLabelClick( wxCommandEvent& event );

    void OnButtonClick( wxCommandEvent& event );

    void OnOkClick( wxCommandEvent& event );

    void OnCancelClick( wxCommandEvent& event );

////@end CwxSettingsDlg event handler declarations

////@begin CwxSettingsDlg member function declarations

    wxBitmap GetBitmapResource( const wxString& name );

    wxIcon GetIconResource( const wxString& name );
////@end CwxSettingsDlg member function declarations

    static bool ShowToolTips();
	void SetConfig(CRef<CSeqGraphicConfig> cfg);		
	CRef<CSeqGraphicConfig> m_Cfg;
	
	CRef<CSeqGraphicDataSource> m_PreviewDS;

	void x_UpdateThemeOptions();
	void x_UpdateThemes();
	void x_UpdateConfig();

	CSeqGraphicDataSource& x_GetFakeDataSource();

////@begin CwxSettingsDlg member variables
    wxListBox* m_List;
    wxChoice* m_Decorations;
    wxChoice* m_Colors;
    wxChoice* m_Spacing;
    wxChoice* m_Layout;
    wxChoice* m_Content;
    wxCheckBox* m_SidedLabel;
    CSeqGraphicWidget* m_SeqWidget;
////@end CwxSettingsDlg member variables
};

END_NCBI_SCOPE

#endif
    // _WX_SETTINGS_DLG_H_
