/*  $Id: seq_text_panel.cpp 25496 2012-03-27 20:41:38Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 */

#include <ncbi_pch.hpp>

#include <wx/sizer.h>
#include <wx/choice.h>
#include <wx/stattext.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/textctrl.h>
#include <wx/button.h>

#include <gui/widgets/seq_text/seq_text_panel.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

////@begin includes
////@end includes

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

////@begin XPM images
////@end XPM images

/*!
 * CSeqTextPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSeqTextPanel, wxPanel )


/*!
 * CSeqTextPanel event table definition
 */

BEGIN_EVENT_TABLE( CSeqTextPanel, wxPanel )

////@begin CSeqTextPanel event table entries
    EVT_BUTTON( ID_BUTTON1, CSeqTextPanel::OnFindBwdClick )

    EVT_BUTTON( ID_BUTTON2, CSeqTextPanel::OnFindFwdClick )

    EVT_CHOICE( ID_CHOICE1, CSeqTextPanel::OnFeatureTypeSelected )

////@end CSeqTextPanel event table entries

END_EVENT_TABLE()


/*!
 * CSeqTextPanel constructors
 */

CSeqTextPanel::CSeqTextPanel()
{
    Init();
}

CSeqTextPanel::CSeqTextPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CSeqTextPanel creator
 */

bool CSeqTextPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSeqTextPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSeqTextPanel creation
    return true;
}


/*!
 * CSeqTextPanel destructor
 */

CSeqTextPanel::~CSeqTextPanel()
{
////@begin CSeqTextPanel destruction
////@end CSeqTextPanel destruction
}


/*!
 * Member initialisation
 */

void CSeqTextPanel::Init()
{
////@begin CSeqTextPanel member initialisation
    m_MousePos = NULL;
    m_SeqTextWidget = NULL;
////@end CSeqTextPanel member initialisation
}


/*!
 * Control creation for CSeqTextPanel
 */

void CSeqTextPanel::CreateControls()
{
////@begin CSeqTextPanel content construction
    CSeqTextPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, wxT("Find"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl5 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL1, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemTextCtrl5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton6 = new wxButton( itemPanel1, ID_BUTTON1, wxT("Bwd"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemButton6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton( itemPanel1, ID_BUTTON2, wxT("Fwd"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemButton7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel1, wxID_STATIC, wxT("Current Position:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_MousePos = new wxStaticText( itemPanel1, wxID_STATIC, wxT(""), wxDefaultPosition, wxSize(itemPanel1->ConvertDialogToPixels(wxSize(40, -1)).x, -1), wxNO_BORDER );
    itemBoxSizer3->Add(m_MousePos, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( itemPanel1, wxID_STATIC, wxT("Show"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString itemChoice11Strings;
    itemChoice11Strings.Add(wxT("None"));
    itemChoice11Strings.Add(wxT("Gene"));
    itemChoice11Strings.Add(wxT("CDS"));
    itemChoice11Strings.Add(wxT("mRNA"));
    itemChoice11Strings.Add(wxT("misc_RNA"));
    itemChoice11Strings.Add(wxT("STS"));
    wxChoice* itemChoice11 = new wxChoice( itemPanel1, ID_CHOICE1, wxDefaultPosition, wxDefaultSize, itemChoice11Strings, 0 );
    itemChoice11->SetStringSelection(wxT("None"));
    itemBoxSizer3->Add(itemChoice11, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_SeqTextWidget = new CSeqTextWidget( itemPanel1, ID_WINDOW, wxDefaultPosition, itemPanel1->ConvertDialogToPixels(wxSize(300, 200)), wxSUNKEN_BORDER );
    itemBoxSizer2->Add(m_SeqTextWidget, 1, wxGROW|wxALL, 0);

////@end CSeqTextPanel content construction
    m_SeqTextWidget->Create();
    m_SeqTextWidget->SetHost(this);

    int subtype = m_SeqTextWidget->GetCaseFeatureSubtype();
    wxChoice* choice = (wxChoice*)FindWindow(ID_CHOICE1);

    switch (subtype) {
        case objects::CSeqFeatData::eSubtype_gene:
            choice->SetSelection (1);
            break;
        case objects::CSeqFeatData::eSubtype_cdregion:
            choice->SetSelection (2);
            break;
        case objects::CSeqFeatData::eSubtype_mRNA:
            choice->SetSelection (3);
            break;
        case objects::CSeqFeatData::eSubtype_misc_RNA:
            choice->SetSelection (4);
            break;
        case objects::CSeqFeatData::eSubtype_STS:
            choice->SetSelection (5);
            break;
    }
}


/*!
 * Should we show tooltips?
 */

bool CSeqTextPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSeqTextPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSeqTextPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSeqTextPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSeqTextPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSeqTextPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSeqTextPanel icon retrieval
}

bool CSeqTextPanel::Init(SConstScopedObject& object)
{
    wxBusyCursor wait;

    const CSeq_id* id = dynamic_cast<const CSeq_id*>(object.object.GetPointer());
    if (id) {
        CBioseq_Handle handle = object.scope->GetBioseqHandle(*id);
        m_DataSource.Reset(new CSeqTextDataSource(handle, *object.scope));
        m_SeqTextWidget->SetDataSource (*m_DataSource);
        return true;
    }

    const CSeq_entry* seq_entry = dynamic_cast<const CSeq_entry*>(object.object.GetPointer());
    if (seq_entry) {
        m_DataSource.Reset(new CSeqTextDataSource(*const_cast<CSeq_entry*>(seq_entry), *object.scope));
        m_SeqTextWidget->SetDataSource (*m_DataSource);
        return true;
    }

    const CBioseq_Handle* handle = dynamic_cast<const CBioseq_Handle*>(object.object.GetPointer());
    if (handle) {
        m_DataSource.Reset(new CSeqTextDataSource(*handle, *object.scope));
        m_SeqTextWidget->SetDataSource (*m_DataSource);
        return true;
    }

    const CSeq_loc* loc = dynamic_cast<const CSeq_loc*>(object.object.GetPointer());
    if (loc) {
        m_DataSource.Reset(new CSeqTextDataSource(*const_cast<CSeq_loc*>(loc), *object.scope));
        m_SeqTextWidget->SetDataSource (*m_DataSource);
        return true;
    }

    return false;
}

void CSeqTextPanel::STWH_ReportMouseOverPos(TSeqPos pos)
{
    m_MousePos->SetLabel(ToWxString(NStr::IntToString(pos)));
}


void CSeqTextPanel::OnFeatureTypeSelected( wxCommandEvent& event )
{
    objects::CSeqFeatData::ESubtype subtype = objects::CSeqFeatData::eSubtype_bad;
    switch (event.GetInt()) {
        case 1:
            subtype = objects::CSeqFeatData::eSubtype_gene;
            break;
        case 2:
            subtype = objects::CSeqFeatData::eSubtype_cdregion;
            break;
        case 3:
            subtype = objects::CSeqFeatData::eSubtype_mRNA;
            break;
        case 4:
            subtype = objects::CSeqFeatData::eSubtype_misc_RNA;
            break;
        case 5:
            subtype = objects::CSeqFeatData::eSubtype_STS;
            break;
    }

    m_SeqTextWidget->ChooseCaseFeature (subtype);
}


void CSeqTextPanel::OnFindBwdClick( wxCommandEvent& event )
{
    wxTextCtrl* find = (wxTextCtrl*)FindWindow(ID_TEXTCTRL1);
    string value = ToStdString(find->GetValue());
    value = NStr::TruncateSpaces(value);
    if (!value.empty())
        m_SeqTextWidget->FindPrev(value);
}

void CSeqTextPanel::OnFindFwdClick( wxCommandEvent& event )
{
    wxTextCtrl* find = (wxTextCtrl*)FindWindow(ID_TEXTCTRL1);
    string value = ToStdString(find->GetValue());
    value = NStr::TruncateSpaces(value);
    if (!value.empty())
        m_SeqTextWidget->FindNext(value);
}

END_NCBI_SCOPE
