/*  $Id: seqtextviewoptions.cpp 25496 2012-03-27 20:41:38Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Colleen Bollin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include "seqtextviewoptions.h"

#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/radiobut.h>
#include <wx/choice.h>
#include <wx/button.h>
#include <wx/icon.h>


BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CwxSeqTextViewOptions, wxDialog )


BEGIN_EVENT_TABLE( CwxSeqTextViewOptions, wxDialog )

////@begin CwxSeqTextViewOptions event table entries
    EVT_BUTTON( ID_SEQTEXTDLG_OK, CwxSeqTextViewOptions::OnOkClick )

    EVT_BUTTON( ID_SEQTEXTDLG_CANCEL, CwxSeqTextViewOptions::OnCancelClick )

////@end CwxSeqTextViewOptions event table entries

END_EVENT_TABLE()


CwxSeqTextViewOptions::CwxSeqTextViewOptions()
{
    m_Config.Reset (new CSeqTextConfig());

    Init();
    x_LoadSettings ();
}

CwxSeqTextViewOptions::CwxSeqTextViewOptions( wxWindow* parent, CRef<CSeqTextConfig> cfg, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    if (cfg) {
        m_Config.Reset (cfg);
    } else {
        m_Config.Reset (new CSeqTextConfig());
    }

    Init();
    Create(parent, id, caption, pos, size, style);
    x_LoadSettings ();
}


/*!
 * CwxSeqTextViewOptions creator
 */

bool CwxSeqTextViewOptions::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CwxSeqTextViewOptions creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CwxSeqTextViewOptions creation
    return true;
}


/*!
 * CwxSeqTextViewOptions destructor
 */

CwxSeqTextViewOptions::~CwxSeqTextViewOptions()
{
////@begin CwxSeqTextViewOptions destruction
////@end CwxSeqTextViewOptions destruction
}


/*!
 * Member initialisation
 */

void CwxSeqTextViewOptions::Init()
{
////@begin CwxSeqTextViewOptions member initialisation
    m_UpperCase = NULL;
    m_LowerCase = NULL;
    m_FontSize = NULL;
    m_FeatColor = NULL;
    m_CodonDisplay = NULL;
    m_DispCoord = NULL;

////@end CwxSeqTextViewOptions member initialisation
}


/*!
 * Control creation for CwxSeqTextViewOptions
 */

void CwxSeqTextViewOptions::CreateControls()
{
////@begin CwxSeqTextViewOptions content construction
    CwxSeqTextViewOptions* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemDialog1, wxID_STATIC, wxT("Show feature in"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_UpperCase = new wxRadioButton( itemDialog1, ID_SEQTEXTDLG_UPPERCASE, wxT("Upper case"), wxDefaultPosition, wxDefaultSize, 0 );
    m_UpperCase->SetValue(true);
    itemBoxSizer4->Add(m_UpperCase, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_LowerCase = new wxRadioButton( itemDialog1, ID_SEQTEXTDLG_LOWERCASE, wxT("Lower case"), wxDefaultPosition, wxDefaultSize, 0 );
    m_LowerCase->SetValue(false);
    itemBoxSizer4->Add(m_LowerCase, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer8 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer8, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemDialog1, wxID_STATIC, wxT("Font Size"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer8->Add(itemStaticText9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_FontSizeStrings;
    m_FontSizeStrings.Add(wxT("6"));
    m_FontSizeStrings.Add(wxT("8"));
    m_FontSizeStrings.Add(wxT("10"));
    m_FontSizeStrings.Add(wxT("12"));
    m_FontSizeStrings.Add(wxT("14"));
    m_FontSizeStrings.Add(wxT("18"));
    m_FontSizeStrings.Add(wxT("20"));
    m_FontSizeStrings.Add(wxT("24"));
    m_FontSize = new wxChoice( itemDialog1, ID_SEQTEXTDLG_FONTSIZE, wxDefaultPosition, wxDefaultSize, m_FontSizeStrings, 0 );
    m_FontSize->SetStringSelection(wxT("12"));
    itemBoxSizer8->Add(m_FontSize, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer11 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer11, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer12 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer11->Add(itemBoxSizer12, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText13 = new wxStaticText( itemDialog1, wxID_STATIC, wxT("Feature Coloration"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer12->Add(itemStaticText13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_FeatColorStrings;
    m_FeatColorStrings.Add(wxT("None"));
    m_FeatColorStrings.Add(wxT("All"));
    m_FeatColorStrings.Add(wxT("Selected"));
    m_FeatColorStrings.Add(wxT("MouseOver"));
    m_FeatColor = new wxChoice( itemDialog1, ID_SEQTEXTDLG_FEATCOLOR, wxDefaultPosition, wxDefaultSize, m_FeatColorStrings, 0 );
    m_FeatColor->SetStringSelection(wxT("All"));
    itemBoxSizer12->Add(m_FeatColor, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer15 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer15, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer15->Add(itemBoxSizer16, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText17 = new wxStaticText( itemDialog1, wxID_STATIC, wxT("Codon Display"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer16->Add(itemStaticText17, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_CodonDisplayStrings;
    m_CodonDisplayStrings.Add(wxT("None"));
    m_CodonDisplayStrings.Add(wxT("All"));
    m_CodonDisplayStrings.Add(wxT("Selected"));
    m_CodonDisplayStrings.Add(wxT("MouseOver"));
    m_CodonDisplay = new wxChoice( itemDialog1, ID_SEQTEXTDLG_CODONDISPLAY, wxDefaultPosition, wxDefaultSize, m_CodonDisplayStrings, 0 );
    m_CodonDisplay->SetStringSelection(wxT("None"));
    itemBoxSizer16->Add(m_CodonDisplay, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer19 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer19, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText20 = new wxStaticText( itemDialog1, wxID_STATIC, wxT("Display Coordinates"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer19->Add(itemStaticText20, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_DispCoordStrings;
    m_DispCoordStrings.Add(wxT("Absolute"));
    m_DispCoordStrings.Add(wxT("Relative"));
    m_DispCoord = new wxChoice( itemDialog1, ID_SEQTEXTDLG_DISPLAYCOORD, wxDefaultPosition, wxDefaultSize, m_DispCoordStrings, 0 );
    m_DispCoord->SetStringSelection(wxT("Absolute"));
    if (CwxSeqTextViewOptions::ShowToolTips())
        m_DispCoord->SetToolTip(wxT("Display coordinates relative to start of sequence or to start of selection"));
    itemBoxSizer19->Add(m_DispCoord, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer22 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer22, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton23 = new wxButton( itemDialog1, ID_SEQTEXTDLG_OK, wxT("Ok"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer22->Add(itemButton23, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton24 = new wxButton( itemDialog1, ID_SEQTEXTDLG_CANCEL, wxT("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer22->Add(itemButton24, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CwxSeqTextViewOptions content construction
}


/*!
 * Should we show tooltips?
 */

bool CwxSeqTextViewOptions::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CwxSeqTextViewOptions::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxSeqTextViewOptions bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxSeqTextViewOptions bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CwxSeqTextViewOptions::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxSeqTextViewOptions icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxSeqTextViewOptions icon retrieval
}


void CwxSeqTextViewOptions::x_SaveSettings()
{
    // case for selected feature type
    if (m_UpperCase->GetValue()) {
        m_Config->SetShowFeatAsLower (false);
    } else {
        m_Config->SetShowFeatAsLower (true);
    }

    //display coordinates
    if (m_DispCoord->GetSelection() == 0) {
        m_Config->SetShowAbsolutePosition (true);
    } else {
        m_Config->SetShowAbsolutePosition (false);
    }

    // font size
    string str(ToStdString(m_FontSize->GetStringSelection()));
    int font_size = NStr::StringToInt (str);
    m_Config->SetFontSize(font_size);

    // feat coloring choice
    m_Config->SetFeatureColorationChoice ((CSeqTextPaneConfig::EFeatureDisplayType) m_FeatColor->GetSelection());

    // codon display choice
    m_Config->SetCodonDrawChoice ((CSeqTextPaneConfig::EFeatureDisplayType) m_CodonDisplay->GetSelection());
}


void CwxSeqTextViewOptions::x_LoadSettings()
{
    // case for selected feature type
    if (m_Config->GetShowFeatAsLower()) {
        m_UpperCase->SetValue(false);
        m_LowerCase->SetValue(true);
    } else {
        m_UpperCase->SetValue(true);
        m_LowerCase->SetValue(false);
    }

    //display coordinates
    if (m_Config->GetShowAbsolutePosition()) {
        m_DispCoord->SetSelection(0);
    } else {
        m_DispCoord->SetSelection(1);
    }

    // font size
    int font_size = m_Config->GetFontSize ();
    string str = NStr::IntToString (font_size);
    m_FontSize->SetStringSelection(ToWxString(str));

    // feat coloring choice
    m_FeatColor->SetSelection (m_Config->GetFeatureColorationChoice ());

    // codon display choice
    m_CodonDisplay->SetSelection (m_Config->GetCodonDrawChoice ());

}


void CwxSeqTextViewOptions::OnOkClick( wxCommandEvent& event )
{
    x_SaveSettings();
    EndModal(ID_SEQTEXTDLG_OK);
}


void CwxSeqTextViewOptions::OnCancelClick( wxCommandEvent& event )
{
    EndModal(ID_SEQTEXTDLG_CANCEL);
}


END_NCBI_SCOPE
