/*  $Id: text_item.cpp 27273 2013-01-18 18:33:40Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/text_widget/text_item.hpp>
#include <gui/widgets/text_widget/selection_helper.hpp>
#include <gui/widgets/text_widget/text_item_panel.hpp>
#include <gui/widgets/text_widget/draw_text_stream.hpp>
#include <gui/widgets/text_widget/calc_size_stream.hpp>

#include <wx/dcclient.h>

BEGIN_NCBI_SCOPE

void CTextItem::CalcSize(wxDC& dc, CTextPanelContext* context)
{
    CCalcSizeOStream ostream(dc, context->GetTextSizeCache(),
                             dc.GetCharHeight(), context->GetIndentWidth());
    x_RenderText(ostream, context);
    m_Size = ostream.GetSize();
    m_Size.IncBy(context->GetLeftMargin(), 0);
    m_LineCount = ostream.GetLineCount();
}

void CTextItem::Draw(wxDC& dc, const wxRect& updateRect, CTextPanelContext* context)
{
    if (IsSelectable()) {
        CSelectionHelper& selHelper = CSelectionHelper::GetInstance();
        selHelper.SetItem(this);
        selHelper.Draw(dc, updateRect, context);
    }

    CTextBlock block = context->GetPanel()->GetTextSelection().Normalized();
    block.ShiftRow(-GetStartLine());

    CTextPosition start = block.GetStart();
    CTextPosition end = block.GetEnd();

    TextPosToCollapsed(start, context);
    TextPosToCollapsed(end, context);
    block = CTextBlock(start, end);

    CDrawTextOStream ostream(dc, context->GetLeftMargin(), 0,
                             dc.GetCharHeight(), context->GetIndentWidth(),
                             block.GetStart().GetRow(), block.GetEnd().GetRow(),
                             block.GetStart().GetCol(), block.GetEnd().GetCol());
    x_RenderText(ostream, context);
}

void CTextItem::x_Indent(CStyledTextOStream& ostream) const
{
     ostream << Indent(GetIndent());
}

void CTextItem::MouseEvent(wxMouseEvent& event, CTextPanelContext& context)
{
    wxRect rect(wxPoint(0, 0), GetSize());
    if (rect.Contains(event.GetPosition())) {
        context.MouseOverItem(this);
    }

    if (IsSelectable()) {
        CSelectionHelper& selHelper = CSelectionHelper::GetInstance();
        selHelper.SetItem(this);
        if (selHelper.ProcessEvent(event))
            return;
    }
    event.Skip();
}

ITextItem* CTextItem::GetItemByLine(int lineNum, bool /*expanded*/)
{
    if (lineNum >= 0 && lineNum < GetLineCount())
        return this;
    return 0;
}

END_NCBI_SCOPE
