/*  $Id: compressed_file.cpp 27055 2012-12-12 22:00:46Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/wx/compressed_file.hpp>

#include <util/compress/zlib.hpp>
#include <util/compress/bzip2.hpp>

BEGIN_NCBI_SCOPE

CNcbiIstream& CCompressedFile::GetIstream()
{
    if (m_Istream)
        return *m_Istream;

    if (m_Fmt == CFormatGuess::eUnknown) {
        m_Fmt = CFormatGuess(m_FileStream).GuessFormat();
        m_FileStream.seekg(0, ios_base::beg);
    }

    m_Istream = &m_FileStream;

    switch (m_Fmt) {
    case CFormatGuess::eGZip:
        m_Compressed.reset(new CCompressionIStream(m_FileStream,
                  new CZipStreamDecompressor(CZipCompression::fGZip), CCompressionStream::fOwnProcessor));
        m_Istream = m_Compressed.get();
        break;
    case CFormatGuess::eBZip2:
        m_Compressed.reset(new CCompressionIStream(m_FileStream,
                  new CBZip2StreamDecompressor(), CCompressionStream::fOwnProcessor));
        m_Istream = m_Compressed.get();
        break;
    default:
        break;
    }

    return *m_Istream;
}

void CCompressedFile::Reset()
{
    m_Istream = 0;
    m_Compressed.reset();
    m_FileStream.seekg(0, ios_base::beg);
}

END_NCBI_SCOPE
