/////////////////////////////////////////////////////////////////////////////
// Name:        demo_tablectrl_frame.cpp
// Purpose:
// Author:      Yury Voronov
// Modified by:
// Created:     30/08/2007 16:15:41
// RCS-ID:
// Copyright:
// Licence:
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include <ncbi_pch.hpp>

#include <wx/filedlg.h>
#include <wx/textdlg.h>

////@begin includes
////@end includes

#include "demo_tablectrl_frame.hpp"
#include "demo_tablemodel.hpp"

#include <serial/serial.hpp>
#include <serial/objistr.hpp>
#include <serial/iterator.hpp>

#include <objects/seq/Seq_annot.hpp>
#include <objects/seqloc/Seq_id.hpp>
#include <objtools/data_loaders/genbank/gbloader.hpp>
#include <objmgr/object_manager.hpp>
#include <objmgr/scope.hpp>

#include <gui/widgets/wx/table_listctrl.hpp>
#include <gui/objutils/utils.hpp>

USING_NCBI_SCOPE;
USING_SCOPE(objects);

/*!
 * CwxDemoTableCtrlFrame type definition
 */

IMPLEMENT_CLASS( CwxDemoTableCtrlFrame, wxFrame )


/*!
 * CwxDemoTableCtrlFrame event table definition
 */

BEGIN_EVENT_TABLE( CwxDemoTableCtrlFrame, wxFrame )

////@begin CwxDemoTableCtrlFrame event table entries
    EVT_MENU( wxID_OPEN, CwxDemoTableCtrlFrame::OnOpenClick )

    EVT_MENU( wxID_LOAD, CwxDemoTableCtrlFrame::OnLoadClick )

////@end CwxDemoTableCtrlFrame event table entries

END_EVENT_TABLE()


/*!
 * CwxDemoTableCtrlFrame constructors
 */

CwxDemoTableCtrlFrame::CwxDemoTableCtrlFrame()
{
    Init();
}

CwxDemoTableCtrlFrame::CwxDemoTableCtrlFrame( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * CwxDemoTableCtrlFrame creator
 */

bool CwxDemoTableCtrlFrame::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CwxDemoTableCtrlFrame creation
    wxFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    Centre();
////@end CwxDemoTableCtrlFrame creation

    CRef<CObjectManager> obj_mgr = CObjectManager::GetInstance();
    CGBDataLoader::RegisterInObjectManager( *obj_mgr );

    return true;
}


/*!
 * CwxDemoTableCtrlFrame destructor
 */

CwxDemoTableCtrlFrame::~CwxDemoTableCtrlFrame()
{
////@begin CwxDemoTableCtrlFrame destruction
////@end CwxDemoTableCtrlFrame destruction
}


/*!
 * Member initialisation
 */

void CwxDemoTableCtrlFrame::Init()
{
////@begin CwxDemoTableCtrlFrame member initialisation
    m_ObjectManager = CObjectManager::GetInstance();
    m_TableCtrl = NULL;
////@end CwxDemoTableCtrlFrame member initialisation

    CGBDataLoader::RegisterInObjectManager(*m_ObjectManager);
}


/*!
 * Control creation for CwxDemoTableCtrlFrame
 */

void CwxDemoTableCtrlFrame::CreateControls()
{
////@begin CwxDemoTableCtrlFrame content construction
    // Generated by DialogBlocks, Sat 16 Feb 2008 11:37:08 AM EST (unregistered)

    CwxDemoTableCtrlFrame* itemFrame1 = this;

    wxMenuBar* menuBar = new wxMenuBar;
    wxMenu* itemMenu3 = new wxMenu;
    itemMenu3->Append(wxID_OPEN, _("&Open...\tCtrl+O"), _(""), wxITEM_NORMAL);
    itemMenu3->Append(wxID_LOAD, _("&Load...\tCtrl+L"), _(""), wxITEM_NORMAL);
    itemMenu3->AppendSeparator();
    itemMenu3->Append(wxID_EXIT, _("E&xit\tAlt+F4"), _(""), wxITEM_NORMAL);
    menuBar->Append(itemMenu3, _("&File"));
    wxMenu* itemMenu8 = new wxMenu;
    itemMenu8->Append(wxID_ABOUT, _("About..."), _(""), wxITEM_NORMAL);
    menuBar->Append(itemMenu8, _("Help"));
    itemFrame1->SetMenuBar(menuBar);

    m_TableCtrl = new CwxTableListCtrl( itemFrame1, ID_TABLECTRL, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_VIRTUAL );

////@end CwxDemoTableCtrlFrame content construction

    m_TableCtrl->SetModel( new ncbi::CwxDemoTableModel(), true );
}


/*!
 * Should we show tooltips?
 */

bool CwxDemoTableCtrlFrame::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CwxDemoTableCtrlFrame::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxDemoTableCtrlFrame bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxDemoTableCtrlFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CwxDemoTableCtrlFrame::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxDemoTableCtrlFrame icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxDemoTableCtrlFrame icon retrieval
}

bool CwxDemoTableCtrlFrame::LoadAccession( const wxString& anAccession ){

    CRef<CSeq_id> id;
    id.Reset( new CSeq_id() );

    string accession = ToStdString(anAccession);

    try {
        id->Set( accession );

    } catch( CSeqIdException& ){
        string msg("Accession '");
        msg += accession;
        msg += "' not recognized as a valid accession";

        wxMessageDialog dlg(NULL, _("Unhandled Accession"),
                            ToWxString(msg), wxOK | wxICON_ERROR);
        dlg.ShowModal();

        return false;
    }

    CRef<CScope> scope;
    scope.Reset( new CScope(*m_ObjectManager) );
    scope->AddDefaults();

    // retrieve our sequence
    CBioseq_Handle handle = scope->GetBioseqHandle( *id );
    if( !handle ){
        string msg("Can't find sequence for accession '");
        msg += accession;
        msg += "'";
        wxMessageDialog dlg(NULL, _("Sequence Not Found"),
                            ToWxString(msg), wxOK | wxICON_ERROR);
        dlg.ShowModal();

        return false;
    }


    CRef<CSeq_loc> loc;
    loc.Reset( new CSeq_loc( CSeq_loc::e_Whole ) );
    loc->SetWhole( *id );

    m_DataSource.Init( *loc, *scope );

    m_TableCtrl->SetModel( &m_DataSource );

    return true;
}

bool CwxDemoTableCtrlFrame::LoadFile( const wxString& aFileName )
{
    CRef<CSeq_entry> seq( new CSeq_entry() );

    try {
        string filename = ToStdString(aFileName);
        auto_ptr<CObjectIStream> os( CObjectIStream::Open( eSerial_AsnText, filename ) );

        *os >> *seq;

    } catch( CException& /*e*/) {

        string msg("This type of file is not supported");
        wxMessageDialog dlg(NULL, _("Error loading file"),
                            ToWxString(msg), wxOK | wxICON_ERROR);
        dlg.ShowModal();

        return false;
    }

    CRef<CSeq_id> id;
    id.Reset( new CSeq_id() );

    if( seq->IsSeq() ){
        CBioseq& bioseq = seq->SetSeq();
        if( bioseq.CanGetId() ){
            list< CRef< CSeq_id > >& id_list = bioseq.SetId();
            if( !id_list.empty() ){
                id.Reset( id_list.front() );
            }
        }
    }

    CRef<CScope> scope;
    scope.Reset( new CScope(*m_ObjectManager) );
    scope->AddTopLevelSeqEntry(*seq);
    scope->AddDefaults();

    CRef<CSeq_loc> loc;
    loc.Reset( new CSeq_loc( CSeq_loc::e_Whole ) );
    loc->SetWhole( *id );

    m_DataSource.Init( *loc, *scope );

    m_TableCtrl->SetModel( &m_DataSource );

    return true;
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_OPEN
 */

void CwxDemoTableCtrlFrame::OnOpenClick( wxCommandEvent& event )
{
    wxFileDialog dlg(
        this,
        _("Choose a data file"), wxEmptyString, wxEmptyString,
        _("ASN files (*.asn)|*.asn"), wxOPEN | wxFILE_MUST_EXIST
    );
    if( dlg.ShowModal() != wxID_OK ){

        if( m_TableCtrl ){
            int numCols = m_TableCtrl->GetNumColumnsVisible();
            for( int i = 0; i < numCols; i++ ){
                LOG_POST( "col map: " << i << "->" << m_TableCtrl->GetColumnOrder(i) );
                LOG_POST( "col map vis: " << i << "<-" << m_TableCtrl->GetColumnIndexFromOrder(i) );
            }
        }
        return;
    }

    LoadFile( dlg.GetPath() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_LOAD
 */

void CwxDemoTableCtrlFrame::OnLoadClick( wxCommandEvent& event )
{
    // Show Choose/Accession dialog
    wxTextEntryDialog dlg(
        this,
        _("Please enter an accession from GenBank"), _("Enter accession"),
        wxEmptyString,
        wxOK | wxCANCEL
    );

    if( dlg.ShowModal() != wxID_OK ){
        if( m_TableCtrl ){
            m_TableCtrl->SetColumnOrder( 2, 5 );
            m_TableCtrl->Refresh();
        }
        return;
    }

    LoadAccession( dlg.GetValue() );
}
