#ifndef WX_DEMO___DEMO_TABLE_MODEL__HPP
#define WX_DEMO___DEMO_TABLE_MODEL__HPP
/*  $Id: demo_tablemodel.hpp 25550 2012-04-06 15:50:17Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Yury Voronov
 *
 * File Description:
 *
 */

#include <gui/widgets/wx/table_model.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

BEGIN_NCBI_SCOPE

#define DEMO_MODEL_NUM_ROWS 21
#define DEMO_MODEL_NUM_COLS 7

enum {
    eNameCol,
    eValidCol,
    eTypeCol,
    eLengthCol,
    eRatioCol,

    eMaxCols
};

/*
class CDemoTableModel : public CAbstractTableModel {

    struct DataRow {
        string name;
        bool valid;
        char type;
        int length;
        double ratio;
        //CRgbaColor color;
    };

    private:

    DataRow data[DEMO_MODEL_NUM_ROWS];


    public:

    CDemoTableModel();

    virtual int GetNumRows() const {
        return DEMO_MODEL_NUM_ROWS;
    }
    virtual int GetNumColumns() const {
        return DEMO_MODEL_NUM_COLS;
    }
    virtual const CAnyType GetValueAt( int row, int col ) const {
        switch( col ){
        case eNameCol:
            return data[row].name;
        case eValidCol:
            return data[row].valid;
        case eTypeCol:
            return data[row].type;
        case eLengthCol:
            return data[row].length;
        case eRatioCol:
            return data[row].ratio;
        //case 4:
        //    return data[row].color;
        default:
            return (char*)"NO DATA";
        }
    }

    virtual const type_info& GetColumnType( int aColIx ) const {

        switch( aColIx ){
        case eNameCol:
            return typeid( string );
        case eValidCol:
            return typeid(bool);
        case eTypeCol:
            return typeid(char);
        case eLengthCol:
            return typeid(int);
        case eRatioCol:
            return typeid(double);
        //case 4:
        //    return typeid(CRgbaColor);
        default:
            return typeid(char*);
        }
    }

    virtual const CAnyType GetColumnName( int aColIx ) const
    {
        switch( aColIx ){
        case eNameCol:
            return (char*)"String";
        case eValidCol:
            return (char*)"Bool";
        case eTypeCol:
            return (char*)"Char";
        case eLengthCol:
            return (char*)"Int";
        case eRatioCol:
            return (char*)"Double";
        //case 4:
        //    return (char*)"Color";
        default:
            return (char*)"UNKNOWN";
        }
    }

    virtual void SetValueAt( int row, int col, const CAnyType& aValue )
    {
        switch( col ){
        case eNameCol:
            data[row].name = aValue.GetValue<string>();
            break;
        case eValidCol:
            data[row].valid = aValue.GetValue<bool>();
            break;
        case eTypeCol:
            data[row].type = aValue.GetValue<char>();
            break;
        case eLengthCol:
            data[row].length = aValue.GetValue<int>();
            break;
        case eRatioCol:
            data[row].ratio = aValue.GetValue<double>();
            break;
        //case 4:
        //    data[row].color = aValue.GetValue<CRgbaColor>();
        //    break;
        }
    }

    ///
    virtual bool IsEditableAt( int row, int col )
    {
        switch( col ){
        case eNameCol:
        case eValidCol:
        case eTypeCol:
        case eLengthCol:
        //case 4:
            return true;
        default:
            return false;
        }
    }

};

*/

class CwxDemoTableModel : public CwxAbstractTableModel {

    struct DataRow {
        string name;
        bool valid;
        char type;
        int length;
        double ratio;
        //CRgbaColor color;
    };

    private:

    DataRow data[DEMO_MODEL_NUM_ROWS];


    public:

    CwxDemoTableModel();

    virtual int GetNumRows() const {
        return DEMO_MODEL_NUM_ROWS;
    }
    virtual int GetNumColumns() const {
        return DEMO_MODEL_NUM_COLS;
    }
    virtual wxVariant GetValueAt( int row, int col ) const {
        switch( col ){
        case eNameCol:
            return ToWxString(data[row].name);
        case eValidCol:
            return data[row].valid;
        case eTypeCol:
            return data[row].type;
        case eLengthCol:
            return data[row].length;
        case eRatioCol:
            return data[row].ratio;
        //case 4:
        //    return data[row].color;
        default:
            return _("NO DATA");
        }
    }

    virtual wxString GetColumnType( int aColIx ) const {

        switch( aColIx ){
        case eNameCol:
            return _("string");
        case eValidCol:
            return _("bool");
        case eTypeCol:
            return _("char");
        case eLengthCol:
            return _("int");
        case eRatioCol:
            return _("double");
        //case 4:
        //    return typeid(CRgbaColor);
        default:
            return _("string");
        }
    }

    virtual wxString GetColumnName( int aColIx ) const
    {
        switch( aColIx ){
        case eNameCol:
            return _("String");
        case eValidCol:
            return _("Bool");
        case eTypeCol:
            return _("Char");
        case eLengthCol:
            return _("Int");
        case eRatioCol:
            return _("Double");
        //case 4:
        //    return "Color";
        default:
            return _("UNKNOWN");
        }
    }

    virtual void SetValueAt( int row, int col, const wxVariant& aValue )
    {
        switch( col ){
        case eNameCol:
            data[row].name = ToStdString(aValue.GetString());
            break;
        case eValidCol:
            data[row].valid = aValue.GetBool();
            break;
        case eTypeCol:
            data[row].type = aValue.GetChar();
            break;
        case eLengthCol:
            data[row].length = aValue.GetLong();
            break;
        case eRatioCol:
            data[row].ratio = aValue.GetDouble();
            break;
        //case 4:
        //    data[row].color = aValue.GetValue<CRgbaColor>();
        //    break;
        }
    }

    ///
    virtual bool IsEditableAt( int row, int col )
    {
        switch( col ){
        case eNameCol:
        case eValidCol:
        case eTypeCol:
        case eLengthCol:
        //case 4:
            return true;
        default:
            return false;
        }
    }

};

END_NCBI_SCOPE

#endif // WX_DEMO___DEMO_TABLE_MODEL__HPP
