/*  $Id: dialog.cpp 26091 2012-07-19 18:21:09Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/wx/dialog.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

BEGIN_NCBI_SCOPE

CDialog::CDialog()
{
}


void CDialog::SetRegistryPath(const string& path)
{
    m_RegPath = path; // store for later use
}


static const char* kWinRectTag = "WindowRect";

void CDialog::SaveSettings() const
{
    if( ! m_RegPath.empty())   {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();

        {{
            string regPath = "WindowPos." + GetVideoId() + "." + m_RegPath;
            CGuiRegistry::TReadWriteView view = gui_reg.GetReadWriteView(regPath, CGuiRegistry::ePriority_Local - 1);
            wxRect rc = GetRect();
            SaveWindowRectToRegistry(rc, view, kWinRectTag);
        }}

        CGuiRegistry::TReadWriteView view = gui_reg.GetReadWriteView(m_RegPath);
        x_SaveSettings(view);
    }
}


void CDialog::LoadSettings()
{
    if( ! m_RegPath.empty())   {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();

        {{
            string regPath = "WindowPos." + GetVideoId() + "." + m_RegPath;
            CGuiRegistry::TReadView view = gui_reg.GetReadView(regPath);
            wxRect rc = GetRect();
            LoadWindowRectFromRegistry(this, rc, view, kWinRectTag);
            SetSize(rc);
        }}

        CGuiRegistry::TReadView view = gui_reg.GetReadView(m_RegPath);
        x_LoadSettings(view);
    }
}


void CDialog::InitDialog()
{
    LoadSettings();

    wxDialog::InitDialog();
}


void CDialog::EndModal(int retCode)
{
    SaveSettings();

    wxDialog::EndModal(retCode);
}


void CDialog::x_SaveSettings(CGuiRegistry::TReadWriteView& view) const
{
}


void CDialog::x_LoadSettings(CGuiRegistry::TReadView& view)
{
}


END_NCBI_SCOPE
