/*  $Id: wx_save_pdf_base_dlg.cpp 27965 2013-05-01 18:40:03Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Liangshou Wu, Bob Falk
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <gui/widgets/wx/wx_save_pdf_base_dlg.hpp>

#include <gui/widgets/wx/regexp_validator.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/message_box.hpp>
#include <gui/widgets/wx/ind_progress_bar.hpp>
#include <gui/objutils/registry.hpp>
#include <corelib/ncbifile.hpp>

#include <wx/filename.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/panel.h>

////@begin XPM images
////@end XPM images


BEGIN_NCBI_SCOPE
/*!
 * CwxSavePdfBaseDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CwxSavePdfBaseDlg, wxDialog )


/*!
 * CwxSavePdfBaseDlg event table definition
 */

BEGIN_EVENT_TABLE( CwxSavePdfBaseDlg, wxDialog )

////@begin CwxSavePdfBaseDlg event table entries
    EVT_BUTTON( ID_FILEPATH, CwxSavePdfBaseDlg::OnFilepathClick )

    EVT_BUTTON( ID_SAVE, CwxSavePdfBaseDlg::OnSaveClick )

    EVT_BUTTON( ID_OPEN, CwxSavePdfBaseDlg::OnOpenClick )

    EVT_BUTTON( ID_CANCEL, CwxSavePdfBaseDlg::OnCancelClick )

////@end CwxSavePdfBaseDlg event table entries

END_EVENT_TABLE()


/*!
 * CwxSavePdfBaseDlg constructors
 */

CwxSavePdfBaseDlg::CwxSavePdfBaseDlg()
{
    Init();
}

CwxSavePdfBaseDlg::CwxSavePdfBaseDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CwxSavePdfBaseDlg creator
 */

bool CwxSavePdfBaseDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CwxSavePdfBaseDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CwxSavePdfBaseDlg creation
    return true;
}


/*!
 * CwxSavePdfBaseDlg destructor
 */

CwxSavePdfBaseDlg::~CwxSavePdfBaseDlg()
{
////@begin CwxSavePdfBaseDlg destruction
////@end CwxSavePdfBaseDlg destruction
}


/*!
 * Member initialisation
 */

void CwxSavePdfBaseDlg::Init()
{
////@begin CwxSavePdfBaseDlg member initialisation
    m_Panel = NULL;
    m_FileNameBox = NULL;
    m_SaveOptionsSizer = NULL;
    m_InfoSizer = NULL;
    m_Info = NULL;
    m_SaveButton = NULL;
    m_OpenButton = NULL;
////@end CwxSavePdfBaseDlg member initialisation
    m_ReadyToSave = false;
    m_OpenPdf = false;
}


/*!
 * Control creation for CwxSavePdfBaseDlg
 */

void CwxSavePdfBaseDlg::CreateControls()
{    
////@begin CwxSavePdfBaseDlg content construction
    CwxSavePdfBaseDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    m_Panel = new wxPanel( itemDialog1, ID_PANEL3, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(m_Panel, 0, wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    m_Panel->SetSizer(itemBoxSizer4);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(itemBoxSizer5, 0, wxGROW|wxLEFT|wxRIGHT|wxTOP, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( m_Panel, wxID_STATIC, _("Save As:"), wxDefaultPosition, wxSize(100, -1), wxALIGN_RIGHT );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_FileNameBox = new wxTextCtrl( m_Panel, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxSize(200, -1), 0 );
    itemBoxSizer5->Add(m_FileNameBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( m_Panel, ID_FILEPATH, _("..."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_SaveOptionsSizer = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(m_SaveOptionsSizer, 0, wxGROW|wxLEFT|wxRIGHT|wxBOTTOM, 5);

    m_SaveOptionsSizer->Add(5, 5, 1, wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5);

    m_InfoSizer = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(m_InfoSizer, 0, wxGROW|wxLEFT|wxRIGHT|wxBOTTOM, 5);

    m_Info = new wxStaticText( itemDialog1, wxID_STATIC, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_InfoSizer->Add(m_Info, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    itemBoxSizer2->Add(5, 1, 1, wxALIGN_CENTER_HORIZONTAL|wxALL, 1);

    wxBoxSizer* itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer14, 0, wxALIGN_RIGHT|wxLEFT|wxRIGHT|wxBOTTOM, 5);

    m_SaveButton = new wxButton( itemDialog1, ID_SAVE, _("Save"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer14->Add(m_SaveButton, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

    m_OpenButton = new wxButton( itemDialog1, ID_OPEN, _("Save && Open"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer14->Add(m_OpenButton, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

    wxButton* itemButton17 = new wxButton( itemDialog1, ID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer14->Add(itemButton17, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5);

////@end CwxSavePdfBaseDlg content construction
}


/*!
 * Should we show tooltips?
 */

bool CwxSavePdfBaseDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CwxSavePdfBaseDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxSavePdfBaseDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxSavePdfBaseDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CwxSavePdfBaseDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxSavePdfBaseDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxSavePdfBaseDlg icon retrieval
}

static const string kPdfPathKey = "PdfPath";

void CwxSavePdfBaseDlg::LoadSettings(const string& pdf_base_key)
{
    CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
    CGuiRegistry::TReadView view = gui_reg.GetReadView(pdf_base_key);
    string path = view.GetString(kPdfPathKey, "");
    if ( !path.empty()  &&  CDirEntry(path).Exists() ) {
        m_Path = path;
    } else {
        // create a default path under user folder
        m_Path = ToStdString(wxGetHomeDir());
    }
}


void CwxSavePdfBaseDlg::SaveSettings(const string& pdf_base_key) const
{
    if ( !m_Path.empty() ) {
        CGuiRegistry& gui_reg = CGuiRegistry::GetInstance();
        CGuiRegistry::TReadWriteView view = gui_reg.GetReadWriteView(pdf_base_key);
        view.Set(kPdfPathKey, m_Path);
    }
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FILEPATH
 */

void CwxSavePdfBaseDlg::OnFilepathClick( wxCommandEvent& event )
{
    wxFileDialog dlg(this, wxT("Set File Name"),
        ToWxString(m_Path), ToWxString(m_FileName + ".pdf"),
        wxT("*.pdf"), wxFD_SAVE);

    if (dlg.ShowModal() != wxID_OK)
        return;

    wxString filename = dlg.GetPath();
    m_FileNameBox->SetValue(filename);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SAVE
 */

void CwxSavePdfBaseDlg::OnSaveClick( wxCommandEvent& event )
{
    // validate file path
    wxFileName file(m_FileNameBox->GetValue());
    wxString filename = file.GetFullName();
    size_t pos = filename.find_last_of(wxT(".pdf"));
    if (pos < filename.length()) {
        filename = filename.SubString(0, pos - 4);
    }
    wxString path = file.GetPath();
    if (filename.IsEmpty()  ||  path.IsEmpty()  ||
        !wxFileName::DirExists(path) ) {
        NcbiMessageBox("The given directory doesn't exist!");
        return;
    }

    m_Path = ToStdString(path);
    m_FileName = ToStdString(filename);

    x_SavePdf();

    /*
    m_Info->SetLabel(_("Preparing data...  Please Wait!"));
    m_ProgressBar = new CIndProgressBar(this, wxID_ANY, wxDefaultPosition, 240);
    m_InfoSizer->Add(m_ProgressBar, 0, wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT|wxBOTTOM, 5);
    m_InfoSizer->Layout();
    Layout();
    m_Panel->Enable(false);
    m_OpenButton->Enable(false);
    m_SaveButton->Enable(false);

    // start the timer to check if the data loading is finished
    m_Timer.Start(300);
    */
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_OPEN
 */

void CwxSavePdfBaseDlg::OnOpenClick( wxCommandEvent& event )
{
    m_OpenPdf = true;
    OnSaveClick(event);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_CANCEL
 */

void CwxSavePdfBaseDlg::OnCancelClick( wxCommandEvent& event )
{
    // Cancel all jobs
    //m_Timer.Stop();
    EndModal(wxID_CANCEL);
}

END_NCBI_SCOPE

