#!/bin/bash
# ===========================================================================
#
#                            PUBLIC DOMAIN NOTICE
#               National Center for Biotechnology Information
#
#  This software/database is a "United States Government Work" under the
#  terms of the United States Copyright Act.  It was written as part of
#  the author's official duties as a United States Government employee and
#  thus cannot be copyrighted.  This software/database is freely available
#  to the public for use. The National Library of Medicine and the U.S.
#  Government have not placed any restriction on its use or reproduction.
#
#  Although all reasonable efforts have been taken to ensure the accuracy
#  and reliability of the software and data, the NLM and the U.S.
#  Government do not and cannot warrant the performance or results that
#  may be obtained by using this software or data. The NLM and the U.S.
#  Government disclaim all warranties, express or implied, including
#  warranties of performance, merchantability or fitness for any particular
#  purpose.
#
#  Please cite the author in any work or product based on this material.
#
# ===========================================================================

# Checkout script for the autobuild process. Additionally, checks source files for certain conditions.
# $1 - working directory
# $2 - email of the build watcher

export CVSROOT=:pserver:${USER}@cvsvault:/src/NCBI/vault.ncbi

WDIR="$1"
if [ "${WDIR}" == "" ]; then
    WDIR="."
fi
EMAIL="$2"
if [ "${EMAIL}" == "" ]; then
    EMAIL="anatoly.boshkin@nih.gov"
fi

function email_error
{
    echo $@ | mail -s "Missing #include <sysalloc.h> found" $EMAIL
}

LOG=${WDIR}/autobuild/results/checkout.log

cd ${WDIR}

# imitate an update, search for lines indicating changed files
cvs -qn up -dA 2>/dev/null | grep "^U" >/dev/null 
if [ $? == 0 ] ; then
    echo "$(date) changes found..." >>${LOG}
    # now, update for real
    cvs -q up -dA 1>>${LOG} 2>>${LOG}
    # report the time of the latest checkout, this will kick off the build
    date >${WDIR}/autobuild/last_checkout

    ############################################# check sources
    #
    # missing include <sysalloc.h> in library code
    ALLOC_SUBDIRS="${WDIR}/libs"
    FILES="$(find ${ALLOC_SUBDIRS} -name *.c )"
    if [ "${FILES}" != "" ]
    then
        FILES=$(echo "$FILES" | grep -v libs/ext | grep -v libs/kapp | grep -v libs/klib | grep -v /unix/ | grep -v /sun/ | grep -v /linux/ | grep -v /mac/ | grep -v /bsd/)
    fi
    if [ "${FILES}" != "" ]
    then
        RESULT="           $(grep -L sysalloc.h `grep calloc  ${FILES} -rIwl`)"
        RESULT="${RESUILT} $(grep -L sysalloc.h `grep malloc  ${FILES} -rIwl`)"
        RESULT="${RESUILT} $(grep -L sysalloc.h `grep realloc ${FILES} -rIwl`)"
        RESULT="${RESUILT} $(grep -L sysalloc.h `grep free    ${FILES} -rIwl`)"
        RESULT=$(echo "${RESULT}" | uniq -u | sort | tr -d ' ')

        if [ "${RESULT}" != "" ]
        then
            echo "${RESULT}" | mail -s "Checkout validation failed: Missing #include <sysalloc.h> found" $EMAIL
        fi    
    fi
    
    #
    # use of standard string handling routines
    FILES="$(find ${WDIR} \( -name *.c -o -name *.cpp \) )"
    if [ "${FILES}" != "" ]
    then
        RESULT="           $(grep strdup ${FILES} -rIwl)"
        RESULT="${RESUILT} $(grep strlen ${FILES} -rIwl)"
        RESULT="${RESUILT} $(grep strcpy ${FILES} -rIwl)"
        RESULT="${RESUILT} $(grep strncpy ${FILES} -rIwl)"
        RESULT=$(echo "${RESULT}" | uniq -u | sort | tr -d ' ')
        #if [ "${RESULT}" != "" ]
        #then
        #    echo "${RESULT}" #| mail -s "Checkout validation failed: use of strdup/strlen/strcpy/strncpy found" $EMAIL
        #fi     
    fi
    
    #
    #  CR
    #TBD
    
    #
    #  executable bits on non-executable files
    #TBD
    
fi

