/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

/**
* Unit tests for VResolver interfaces
*/

#include <ktst/unit_test.hpp>
#include <stdexcept>

#include <sysalloc.h>

#include <kns/kns_mgr.h>
#include <kns/curl-file.h>
#include <kfs/file.h>


TEST_SUITE(KnsTestSuite);

using namespace std;

class KnsManagerFixture
{
public:
    KnsManagerFixture()
    : mgr(0)
    {
        if (KNSManagerMake(&mgr) != 0)
            throw logic_error("KnsManagerFixture: KNSManagerMake failed");
    }
    ~KnsManagerFixture()
    {
        if (mgr && KNSManagerRelease(mgr) != 0)
            throw logic_error("~KnsManagerFixture: KNSManagerRelease failed");
    }
    
    KNSManager* mgr;
};

TEST_CASE(KnsManagerMakeRelease)
{   
    KNSManager* mgr;
    REQUIRE_RC(KNSManagerMake(&mgr)); 
    REQUIRE_NOT_NULL(mgr);
    REQUIRE_RC(KNSManagerRelease(mgr)); 
}

FIXTURE_TEST_CASE(KnsManagerAvail, KnsManagerFixture)
{   
    REQUIRE_RC(KNSManagerAvail(mgr));
}

FIXTURE_TEST_CASE(KnsManagerVersion, KnsManagerFixture)
{   
    const char* version = 0;
    REQUIRE_RC(KNSManagerCurlVersion(mgr, &version));    
    REQUIRE_NOT_NULL(version);
}

TEST_CASE(KCurlFileOpen)
{   
    const KFile* file = 0;
    REQUIRE_RC(KCurlFileMake(&file, "http://www.yahoo.com", false));    
    REQUIRE_NOT_NULL(file);
    uint64_t size = 0;
    REQUIRE_RC(KFileSize(file, &size));    
    REQUIRE_GT(size, (uint64_t)0);
    REQUIRE_RC(KFileRelease(file));
}

//////////////////////////////////////////// Main
extern "C"
{

#include <kapp/args.h>
#include <kfg/config.h>

ver_t CC KAppVersion ( void )
{
    return 0x1000000;
}
rc_t CC UsageSummary (const char * progname)
{
    return 0;
}

rc_t CC Usage ( const Args * args )
{
    return 0;
}
const char UsageDefaultName[] = "test-kns";

rc_t CC KMain ( int argc, char *argv [] )
{
    KConfigDisableUserSettings();
    rc_t rc=KnsTestSuite(argc, argv);
    return rc;
}

}
