#ifndef GUI_OBJUTILS___TABLE_DATA_SNP__HPP
#define GUI_OBJUTILS___TABLE_DATA_SNP__HPP

/*  $Id: table_data_snp.hpp 27500 2013-02-26 15:35:11Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Dmitry Rudnev
 *
 * File Description:
 *
 */
#include <gui/utils/event_handler.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/objutils/table_data_base.hpp>
#include <objects/seqalign/Seq_align.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objmgr/scope.hpp>
#include <objtools/snputil/snp_bitfield.hpp>
#include <objects/variation/Variation.hpp>

BEGIN_NCBI_SCOPE

class NCBI_GUIOBJUTILS_EXPORT CTableDataSNP 
    : public CObject,
      public CTableDataBase
{
public:
    // number of entries and order must correspond with s_ColNames
    enum EColContents {
        eRsId = 0,
        eAlleles,
        eSequence,
        ePosition,
        eFXNClass,
        eWeight,
        // ePopFreq, -- dropped as asked in SNP-4726
        eVariationClass,
        eResourceLink,
        // eMapping, -- dropped as asked in SNP-4726
        eFreqValidation,
        // eInHaplotypeSet, -- dropped as asked in SNP-4726
        eQualityCheck,
        ePhenotype,
        e1000Genomes,
        eMaxColNum
    };

    // data load status
    enum EDataLoadStatus {
        eNotReady,
        eFailed,
        eReady
    };

    // maximal amount of SNP rows we should realistically see in a SNP table
    static const size_t c_MaxRows;

    // objects[0].object is supposed to contain a Seq-loc over which the SNP features will be loaded, anything else will be ignored
	CTableDataSNP(TConstScopedObjects& objects);
    ~CTableDataSNP();

    /// @name ITableData interface implementation
    /// @{
    virtual ColumnType GetColumnType(size_t col) const;
    virtual bool       IsHtmlColumn(size_t col) const;
    virtual string     GetColumnLabel(size_t col) const;

    virtual size_t GetRowsCount() const;
    virtual size_t GetColsCount() const;

    virtual string GetStringValue(size_t row, size_t col) const;
    virtual string GetHtmlValue(size_t /*row*/, size_t /*col*/) const;
    virtual long   GetIntValue(size_t row, size_t col) const;
    virtual double GetRealValue(size_t row, size_t col) const;
    
	// the object will be represented as a CVariation
	virtual SConstScopedObject GetObjectValue(size_t row, size_t col) const;
    virtual SConstScopedObject GetRowObject(size_t row) const;

    CRef<objects::CScope> GetScope() { return m_Scope; }
    /// @}

    // internally, the list of SNPs is stored as CVariations
    // gene_id is stored as one of the other-ids
    
    typedef vector<CRef<objects::CVariation> > TSnpDataList;

    void OnJobNotification(CEvent* evt);

    DECLARE_EVENT_MAP();

private:
    void x_LoadData();
    CConstRef<objects::CVariation> x_GetRowData(size_t row) const;
    CRef<objects::CScope> m_Scope;
    CConstRef<objects::CSeq_loc>    m_OrigLoc;
    CRef<CObjectFor<TSnpDataList> > m_listSnps;

    int m_hJob;     // Job ID
    EDataLoadStatus m_DataLoadStatus;
};


END_NCBI_SCOPE

#endif  // GUI_OBJUTILS___TABLE_DATA_SNP_HPP
