#ifndef PKG_ALIGNMENT___BLAST_SEARCH_OPTIONS_PANEL__HPP
#define PKG_ALIGNMENT___BLAST_SEARCH_OPTIONS_PANEL__HPP

/*  $Id: blast_search_options_panel.hpp 22209 2010-09-30 20:41:30Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

#include <gui/objutils/objects.hpp>

#include <algo/blast/api/blast_types.hpp>

#include "wx/panel.h"

////@begin includes
#include "wx/listctrl.h"
#include "wx/valgen.h"
////@end includes

////@begin forward declarations
class CObjectListWidget;
////@end forward declarations

class wxComboBox;
class wxRadioButton;
class wxStaticText;
class wxButton;
class wxStaticBox;


////@begin control identifiers
#define SYMBOL_CBLASTSEARCHOPTIONSPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CBLASTSEARCHOPTIONSPANEL_TITLE _("BLAST Search Options Panel")
#define SYMBOL_CBLASTSEARCHOPTIONSPANEL_IDNAME ID_CBLASTSEARCHOPTIONSPANEL
#define SYMBOL_CBLASTSEARCHOPTIONSPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CBLASTSEARCHOPTIONSPANEL_POSITION wxDefaultPosition
////@end control identifiers

#include <wx/panel.h>


BEGIN_NCBI_SCOPE

/** @addtogroup GUI_PKG_ALIGNMENT
 *
 * @{
 */

class CObjectListWidget;
class CBLASTParams;
class CNetBLASTUIDataSource;


class CBLASTSearchOptionsPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS( CBLASTSearchOptionsPanel )
    DECLARE_EVENT_TABLE()

public:
    CBLASTSearchOptionsPanel();
    CBLASTSearchOptionsPanel( wxWindow* parent, wxWindowID id = SYMBOL_CBLASTSEARCHOPTIONSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CBLASTSEARCHOPTIONSPANEL_POSITION, const wxSize& size = wxSize(), long style = SYMBOL_CBLASTSEARCHOPTIONSPANEL_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CBLASTSEARCHOPTIONSPANEL_IDNAME, const wxPoint& pos = SYMBOL_CBLASTSEARCHOPTIONSPANEL_POSITION, const wxSize& size = wxSize(), long style = SYMBOL_CBLASTSEARCHOPTIONSPANEL_STYLE );

    ~CBLASTSearchOptionsPanel();

    void Init();

    void CreateControls();

    bool TransferDataToWindow();

    bool TransferDataFromWindow();

////@begin CBLASTSearchOptionsPanel event handler declarations

    void OnNucRadioSelected( wxCommandEvent& event );

    void OnProtRadioSelected( wxCommandEvent& event );

    void OnProgramComboSelected( wxCommandEvent& event );

    void OnBrowseDatabases( wxCommandEvent& event );

////@end CBLASTSearchOptionsPanel event handler declarations

////@begin CBLASTSearchOptionsPanel member function declarations

    wxString GetDatabase() const { return m_Database ; }
    void SetDatabase(wxString value) { m_Database = value ; }

    wxString GetQuery() const { return m_Query ; }
    void SetQuery(wxString value) { m_Query = value ; }

    wxBitmap GetBitmapResource( const wxString& name );

    wxIcon GetIconResource( const wxString& name );
////@end CBLASTSearchOptionsPanel member function declarations

    static bool ShowToolTips();

////@begin CBLASTSearchOptionsPanel member variables
    wxRadioButton* m_NucRadio;
    wxRadioButton* m_ProtRadio;
    CObjectListWidget* m_QueryListCtrl;
    wxComboBox* m_ProgramCombo;
    wxStaticText* m_DatabaseStatic;
    wxComboBox* m_DatabaseCombo;
    wxButton* m_BrowseDBBtn;
    wxStaticText* m_EntrezStatic;
    wxComboBox* m_QueryCombo;
    wxPanel* m_SubjPanel;
    wxStaticBox* m_SubjSizer;
    CObjectListWidget* m_SubjectListCtrl;
protected:
    wxString m_Program;
    wxArrayString m_ProgramLabels;
    wxString m_Database;
    wxString m_Query;
    enum {
        ID_CBLASTSEARCHOPTIONSPANEL = 10000,
        ID_NUC_RADIO = 10001,
        ID_PROT_RADIO = 10002,
        ID_BUTTON2 = 10010,
        ID_LIST = 10003,
        ID_PROGRAM_COMBO = 10004,
        ID_DB_COMBO = 10005,
        ID_DB_BTN = 10006,
        ID_ENTREZ_COMBO = 10007,
        ID_SUBJ_PANEL = 10026,
        ID_SUBJ_ADD = 10011,
        ID_SUBJ_LIST = 10012
    };
////@end CBLASTSearchOptionsPanel member variables

public:
    /// @name IRegSettings interface implementation
    /// @{
    virtual void    SetRegistryPath(const string& path);
    virtual void    LoadSettings();
    virtual void    SaveSettings() const;
    /// @}

    void    SetRemote(bool remote) { m_Remote = remote; }

    void    Setup(CNetBLASTUIDataSource& ds, const string& tool_name, bool remote);
    void    SetParams(CBLASTParams* params,
                      TConstScopedObjects* nuc_objects,
                      TConstScopedObjects* prot_objects);
    void    SelectInputType();

protected:
    void x_OnInputTypeChanged();
    void x_OnSubjInputTypeChanged();

protected:
    string  m_RegPath;

    CNetBLASTUIDataSource*  m_DataSource;
    string  m_ToolName;
    bool    m_Remote; // Remote vs, Local BLAST

    TConstScopedObjects*  m_NucObjects;
    TConstScopedObjects*  m_ProtObjects;

    CBLASTParams* m_Params;

    bool m_QueryListDirty;
    bool m_SubjectListDirty;

};


/* @} */

END_NCBI_SCOPE

#endif
    // _BLAST_SEARCH_OPTIONS_PANEL_H_
