#ifndef PKG_SEQUENCE___FASTA_OBJECT_LOADER__HPP
#define PKG_SEQUENCE___FASTA_OBJECT_LOADER__HPP

/*  $Id: fasta_object_loader.hpp 27061 2012-12-13 15:49:53Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Roman Katargin
*
* File Description:
*
*/

#include <corelib/ncbistd.hpp>

#include <gui/utils/execute_unit.hpp>
#include <gui/utils/object_loader.hpp>

#include <wx/string.h>

#include <gui/packages/pkg_sequence/fasta_load_params.hpp>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

///////////////////////////////////////////////////////////////////////////////
/// CFastaObjectLoader
class CFastaObjectLoader : public CObject, public IObjectLoader, public IExecuteUnit
{
public:
    CFastaObjectLoader(const CFastaLoadParams& params, const vector<wxString>& filenames);

    /// @name IObjectLoader implementation
    /// @{
    virtual TObjects& GetObjects();
    virtual string GetDescription() const;
    /// @}

    /// @name IExecuteUnit implementation
    /// @{
    virtual bool PreExecute();
    virtual bool Execute(ICanceled& canceled);
    virtual bool PostExecute();
    /// @}

private:
    CFastaLoadParams  m_Params;
    vector<wxString>  m_FileNames;
    TObjects          m_Objects;
};

/* @} */

END_NCBI_SCOPE


#endif // PKG_SEQUENCE___FASTA_OBJECT_LOADER__HPP
