#ifndef GUI_PACKAGES_PKG_SNP_TABLE__MODEL_SNP_TABLE__HPP
#define GUI_PACKAGES_PKG_SNP_TABLE__MODEL_SNP_TABLE__HPP

/*  $Id: model_snp_table.hpp 21518 2010-06-10 15:34:57Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Melvin Quintos
 *
 * File Description:
 *    SNP Table DS - Data source for SNP Table
 */

#include <gui/types.hpp>

#include <gui/framework/observable.hpp>
#include <gui/objutils/objects.hpp>
#include <gui/utils/event_handler.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/widgets/wx/table_model.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/scope.hpp>

#include <gui/widgets/snp/filter/snp_filter.hpp>
#include <gui/widgets/snp/snp_data.hpp> // TODO MRQ move file location

BEGIN_NCBI_SCOPE

class CJobUpdate;

///////////////////////////////////////////////////////////////////////////////
/// CSnpTableModel
/// Data source for Feature Table widget. Implements TableModel (for CTableControl)
/// and is responsible for loading and updating data asynchronously.
///
class CSnpTableModel
    : public CObjectEx
    , public CwxAbstractTableModel
    , public CObservable
{
/****** PUBLIC INTERFACE SECTION ******/
public:

    CSnpTableModel();

    int                 GetNumRowsUnfiltered() const;
    const SSnpData &    GetSnpAt(long pos) const;

    // Launches Jobs
    void LoadData(const objects::CSeq_loc& loc,
                  objects::CScope* scope=0, bool showGwasSnps = false);
    void ApplyFilter(const SSnpFilter &filter);

    // Event Handlers
    void OnJobNotification( CEvent* evt );

    /// @name IwxTableModel implementation
    /// @{
    virtual int GetNumRows() const;
    virtual int GetNumColumns() const;

    virtual wxVariant   GetValueAt( int row, int col ) const;
    virtual wxString    GetColumnName( int col ) const;
    virtual wxString    GetColumnType( int col ) const;
    /// @}


    DECLARE_EVENT_MAP();

private:
    void x_StartLoadJob();
    void x_StartFilterJob();

    // void x_DisseminateJobUpdate( 
    //     int job_id, IAppJob::EJobState state, const string& msg
    // );

    void x_Update(CJobUpdate& update);


private:
    bool                    m_bAppliedFilter;
    bool                    m_bShowOnlySpecial;
    int                     m_nUnfilteredSnps;
    int                     m_hJob;     // Job ID
    string                  m_strStatus;

    CRef<objects::CScope>           m_Scope;
    CConstRef<objects::CSeq_loc>    m_Loc;
    CConstRef<SSnpFilter>           m_Filter;

    CRef<TSnpListObject>            m_listUnfilteredSnps;
    CRef<TSnpPtrListObject >        m_listSnps;

    size_t                          m_numRows;

    // the filter defining what features need to be loaded
    objects::SAnnotSelector m_Selector;
    /// app job notification and control
    CAppJobDispatcher::TJobID m_ActiveJob;

};

END_NCBI_SCOPE

#endif  /// GUI_PACKAGES_PKG_SNP_TABLE__MODEL_SNP_TABLE__HPP
