#ifndef GUI_PACKAGES_SNP__VIEW_SNP_TABLE__HPP
#define GUI_PACKAGES_SNP__VIEW_SNP_TABLE__HPP

/*  $Id: view_snp_table.hpp 27556 2013-03-05 16:04:52Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */
#include <gui/core/project_view_impl.hpp>
#include <gui/core/open_view_manager_impl.hpp>
#include <gui/objutils/reg_settings.hpp>

#include <objmgr/scope.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <objects/seqloc/Seq_id.hpp>
#include <gui/objutils/object_index.hpp>
#include <gui/objutils/reg_settings.hpp>
#include <objmgr/annot_selector.hpp>

//#include <gui/services/seqloc_search_context.hpp>

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

class CSnpTableWidget;
class CSnpTableModel;

///////////////////////////////////////////////////////////////////////////////
/// CSnpTableView
class CSnpTableView
    : public CProjectView
    , public IRegSettings
{
    friend class CSnpTableViewFactory;

public:
    CSnpTableView();

    /// @name IWMClient interface implementation
    /// @{
    virtual wxWindow* GetWindow();
    /// @}

    /// @name IView interface implementation
    /// @{
    virtual const CViewTypeDescriptor& GetTypeDescriptor() const;

    virtual void CreateViewWindow(wxWindow* parent);
    virtual void DestroyViewWindow();
    /// @}

    /// @name IProjectView implementation
    /// @{
    virtual bool InitView(TConstScopedObjects& objects, const objects::CUser_object* params);
    virtual void OnProjectChanged();
    /// @}

    /// @name ISelectionClient Implementation
    /// @{
    virtual void GetSelection(CSelectionEvent& evt) const;
    virtual void GetSelection(TConstScopedObjects& objs) const;
    /// @}

    /// @name CProjectViewBase overridables
    /// @{
    virtual void GetCompatibleToolBars(vector<string>& names);
    virtual void GetVisibleRanges(CVisibleRange& vrange) const;
    /// @}

    /// @name IRegSettings interface implementation
    /// @{
    virtual void SetRegistryPath(const string& reg_path);
    virtual void LoadSettings();
    virtual void SaveSettings() const;
    /// @}

    /// @name IVisibleRangeClient interface implementation
    /// @{
    virtual void OnVisibleRangeChanged (const CVisibleRange& vrange,
                                        IVisibleRangeClient* source);
    ///  @}

protected:    
    void x_InitMenuResources();
    void x_DoDialogShowSpecial();
    void x_Update();
    //void x_UpdateStatusMessage();
    void x_RebuildObjectIndex();
    void x_SetVisibleRange(const objects::CSeq_loc& loc);

    /// @name CProjectView overridables
    /// @{
    virtual void x_OnSetSelection( CSelectionEvent& evt );
    /// @}

    /// @name CProjectViewBase overridables
    /// @{
    virtual void x_CreateMenuBarMenu();
    virtual const CObject* x_GetOrigObject() const;
    /// @}

private:
    bool                    m_bShowOnlySpecial;

    /// original data object passed to the view
    CConstRef<objects::CSeq_loc>    m_OrigLoc;
    objects::SAnnotSelector         m_Sel;

    CRef<objects::CScope>   m_Scope;
    CRef<objects::CSeq_id>  m_Seq_id;

    CObjectIndex            m_ObjectIndex;

    CSnpTableWidget *       m_tableWidget;
    CRef<CSnpTableModel>    m_DataSource;

    string m_RegPath; // path to the setting in CGuiRegistry

    auto_ptr<CViewTypeDescriptor>   m_TypeDescr;
};


/* @} */

END_NCBI_SCOPE


#endif  // GUI_PACKAGES_SNP__VIEW_SNP_TABLE__HPP
