#ifndef GUI_PACKAGES_PKG_SNP__WIDGET_SNP_TABLE__HPP
#define GUI_PACKAGES_PKG_SNP__WIDGET_SNP_TABLE__HPP

/*  $Id: widget_snp_table.hpp 23938 2011-06-22 18:50:32Z wuliangs $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Melvin Quintos
 *
 * File Description:
 *    CSnpTableWidget - a widget representing SNPs in a table
 */

#include <gui/types.hpp>

#include <gui/packages/pkg_snp/table/model_snp_table.hpp>

#include <gui/framework/observable.hpp>
#include <gui/objutils/objects.hpp>
#include <gui/objutils/registry.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/utils/command.hpp>
#include <gui/widgets/wx/table_listctrl.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objmgr/feat_ci.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <objmgr/scope.hpp>

#include <vector>

class wxFileArtProvider;

BEGIN_NCBI_SCOPE

class CUICommandRegistry;
class CSnpTableModel;
class CTableControl;
class CScrollPanel;
class CBox;


/*****************************************************************************
   CSnpTableWidget
   Created and Initialized by CSnpTableView
   Used to Layout GUI elements of SNP Table View and manage jobs for fetching
     data
   Owns objects for GUI elements
******************************************************************************/
class CSnpTableWidget
    : public CwxTableListCtrl
    , public IObserver
{
public:

    CSnpTableWidget() {}

    CSnpTableWidget(
        wxWindow* parent,
        wxWindowID id = wxID_ANY,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxLC_REPORT | wxLC_VIRTUAL
    );

    virtual ~CSnpTableWidget();

    static void RegisterCommands(CUICommandRegistry& cmd_reg, wxFileArtProvider& provider);

    /// Get the selected objects
    void GetSelection( TConstObjects& objs ) const;
    void ShowAnalysisValues(bool show);
    void SetDataSource(CSnpTableModel* new_ds);   

    /// @name Command and Event handlers
    /// @{
    void OnContextMenu( wxContextMenuEvent& event );
    void OnEventLinkout( wxCommandEvent& event );
    void OnExportCSV( wxCommandEvent& event );
    void OnSelectFilter( wxCommandEvent& event);
    /// @}


    /// @name IObserver interface implementation
    /// @{
    virtual void    OnUpdate(IObserverUpdate& /*update*/){};
    /// @}

private:

    /// @name wxListCtrl reimplementation
    /// @{
    virtual wxListItemAttr* OnGetItemAttr( long aRow ) const;
    /// @}

    void x_PrepareMenu(wxMenu *);
    //void x_Filter();
    //void x_Update();
    //void x_UpdateStatusBar();
    //void x_ShowProgressBar(bool show);

private:
    bool                    m_bAppliedFilter;
    bool                    m_bShowStudies;
    int                     m_nUnfilteredSnps;
    string                  m_strStatus;

    CRef<CSnpTableModel>     m_SnpTableModel;

    //CConstRef<SSnpFilter>           m_Filter;

    /// @name ITimerListener implementation
    /// @{
    //void OnTimeout(int timer_id);
    /// @}

protected:

    DECLARE_EVENT_TABLE()

private:
    /// forbidden
    CSnpTableWidget(const CSnpTableWidget&);
    CSnpTableWidget& operator=(const CSnpTableWidget&);
};


END_NCBI_SCOPE

#endif  /// GUI_PACKAGES_PKG_SNP__WIDGET_SNP_TABLE__HPP
