/*  $Id: snp_filter_list_dlg.hpp 25657 2012-04-18 14:18:39Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 */
#ifndef GUI_WIDGETS_SNP___SNP_FILTER_LIST_DLG_HPP
#define GUI_WIDGETS_SNP___SNP_FILTER_LIST_DLG_HPP

#include <corelib/ncbistd.hpp>

/*!
 * Includes
 */

#include <wx/listbox.h>
#include <wx/button.h>

////@begin includes
////@end includes

#include <gui/widgets/snp/filter/snp_filter.hpp>

#include <wx/dialog.h>

/*!
 * Forward declarations
 */

////@begin forward declarations
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CSNPFILTERLISTDIALOG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CSNPFILTERLISTDIALOG_TITLE _("SNP Filters")
#define SYMBOL_CSNPFILTERLISTDIALOG_IDNAME ID_MYDIALOG
#define SYMBOL_CSNPFILTERLISTDIALOG_SIZE wxSize(400, 300)
#define SYMBOL_CSNPFILTERLISTDIALOG_POSITION wxDefaultPosition
////@end control identifiers


BEGIN_NCBI_SCOPE

/*!
 * CSnpFilterListDialog class declaration
 */

class CSnpFilterListDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CSnpFilterListDialog )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CSnpFilterListDialog();
    CSnpFilterListDialog( wxWindow* parent, wxWindowID id = SYMBOL_CSNPFILTERLISTDIALOG_IDNAME, const wxString& caption = SYMBOL_CSNPFILTERLISTDIALOG_TITLE, const wxPoint& pos = SYMBOL_CSNPFILTERLISTDIALOG_POSITION, const wxSize& size = SYMBOL_CSNPFILTERLISTDIALOG_SIZE, long style = SYMBOL_CSNPFILTERLISTDIALOG_STYLE );

    /// Creation
    bool CreateX( bool bShowEditButton=false, wxWindow* parent=0, wxWindowID id = SYMBOL_CSNPFILTERLISTDIALOG_IDNAME, const wxString& caption = SYMBOL_CSNPFILTERLISTDIALOG_TITLE, const wxPoint& pos = SYMBOL_CSNPFILTERLISTDIALOG_POSITION, const wxSize& size = SYMBOL_CSNPFILTERLISTDIALOG_SIZE, long style = SYMBOL_CSNPFILTERLISTDIALOG_STYLE );

    /// Destructor
    ~CSnpFilterListDialog();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CSnpFilterListDialog event handler declarations

    /// wxEVT_COMMAND_LISTBOX_SELECTED event handler for ID_LISTBOX1
    void OnListboxSelection( wxCommandEvent& event );

    /// wxEVT_LEFT_DOWN event handler for ID_LISTBOX1
    void OnMouseEvent( wxMouseEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
    void OnButtonEditClick( wxCommandEvent& event );

////@end CSnpFilterListDialog event handler declarations

////@begin CSnpFilterListDialog member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CSnpFilterListDialog member function declarations

    /// Retrieves SNP Filter
    string GetFilter(SSnpFilter *filter=0);

    /// Should we show tooltips?
    static bool ShowToolTips();

private:
    void x_UpdateListBox();
    void x_IsFilterSelected(bool is_selected);
    void x_OnBrowserChange();

////@begin CSnpFilterListDialog member variables
    wxListBox* m_ListBox;
    wxButton* m_BtnEdit;
    wxButton* m_BtnOK;
    wxButton* m_BtnCancel;
private:
    int m_nSelectedRow;
    map<string, SSnpFilter> m_mapFilters;
    bool m_bShowEditButton;
    /// Control identifiers
    enum {
        ID_MYDIALOG = 10003,
        ID_LISTBOX1 = 10008,
        ID_BUTTON_EDIT = 10011
    };
////@end CSnpFilterListDialog member variables


};

END_NCBI_SCOPE

#endif    // _GUI_WIDGETS_SNP___SNP_FILTER_LIST_DLG_HPP
