/*  $Id: snp_filter_ui.hpp 25657 2012-04-18 14:18:39Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */
#ifndef GUI_WIDGETS_SNP___SNP_FILTER_UI_HPP
#define GUI_WIDGETS_SNP___SNP_FILTER_UI_HPP

/*!
 * Includes
 */
#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <gui/widgets/snp/filter/group_checkboxes.hpp>
#include <gui/widgets/snp/filter/group_radios.hpp>
#include <gui/widgets/snp/filter/snp_filter.hpp>


////@begin includes
#include "wx/notebook.h"
////@end includes

#include <wx/dialog.h>
#include <wx/listbox.h>
#include <wx/button.h>
#include <wx/statbox.h>

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxNotebook;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CSNPFILTERUI_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CSNPFILTERUI_TITLE _("SNP Filters")
#define SYMBOL_CSNPFILTERUI_IDNAME ID_MYDIALOG
#define SYMBOL_CSNPFILTERUI_SIZE wxSize(400, 300)
#define SYMBOL_CSNPFILTERUI_POSITION wxDefaultPosition
////@end control identifiers

BEGIN_NCBI_SCOPE

/*!
 * CSnpFilterUI class declaration
 */

class /*NCBI_GUIWIDGETS_SNP_EXPORT*/ CSnpFilterUI: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CSnpFilterUI )
    DECLARE_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////
// PUBLIC typedefs / enums / structs
///////////////////////////////////////////////////////////////////////////////
public:

enum EFilterState {
    eModified,
    eDeleted
};

typedef map<string, CRef<SSnpFilter> >      TMapFilters;
typedef map<string, EFilterState >          TMapFilterStates;

static const int kChoiceAny = -1;  // Use when property field's value doesn't matter

///////////////////////////////////////////////////////////////////////////////
// PUBLIC Methods
///////////////////////////////////////////////////////////////////////////////
public:
    //static CSnpFilterUI & GetInstance();

    /// Constructors
    CSnpFilterUI();
    CSnpFilterUI( wxWindow* parent, wxWindowID id = SYMBOL_CSNPFILTERUI_IDNAME, const wxString& caption = SYMBOL_CSNPFILTERUI_TITLE, const wxPoint& pos = SYMBOL_CSNPFILTERUI_POSITION, const wxSize& size = SYMBOL_CSNPFILTERUI_SIZE, long style = SYMBOL_CSNPFILTERUI_STYLE );

    /// Creation
    bool CreateX( wxWindow* parent, wxWindowID id = SYMBOL_CSNPFILTERUI_IDNAME, const wxString& caption = SYMBOL_CSNPFILTERUI_TITLE, const wxPoint& pos = SYMBOL_CSNPFILTERUI_POSITION, const wxSize& size = SYMBOL_CSNPFILTERUI_SIZE, long style = SYMBOL_CSNPFILTERUI_STYLE );

    /// Destructor
    virtual ~CSnpFilterUI();

////@begin CSnpFilterUI event handler declarations

    /// wxEVT_COMMAND_LISTBOX_SELECTED event handler for ID_LISTBOX
    void OnListboxSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ADD
    void OnButtonAddClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_DELETE
    void OnButtonDeleteClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_RENAME
    void OnButtonRenameClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_COPY
    void OnButtonCopyClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_SAVE
    void OnButtonSaveClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_IMPORT
    void OnButtonImportClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EXPORT
    void OnButtonExportClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
    void OnOkClick( wxCommandEvent& event );

////@end CSnpFilterUI event handler declarations
    void OnPropertyChanged( wxCommandEvent& event );

////@begin CSnpFilterUI member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CSnpFilterUI member function declarations

//    const TMapConstFilters & GetFilters() const;
    void SetSelection(const string &name); // select the named filter if it exists
    int  GetSelectedRow() const;

    /// Should we show tooltips?
//    static bool ShowToolTips();

///////////////////////////////////////////////////////////////////////////////
// PRIVATE Methods
///////////////////////////////////////////////////////////////////////////////
private:
    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    void x_Clear();
    void x_SetupTabs();
    void x_OnBrowserChange();
    void x_OnBtnSave();
    void x_AddSorted(const string &text);
    void x_CheckChanges();
    void x_EnableSave(bool);
    void x_EnableCopyDelete(bool enable);
    void x_LoadFromRegistry();
    void x_LoadFilter();
    void x_ResetFilterTab();
    void x_AddFilter(const string &name, SSnpFilter *filter);
    void x_RemoveFilter(const string& name);

public:

////@begin CSnpFilterUI member variables
    wxListBox* m_Browser;
    wxButton* m_btnDelete;
    wxButton* m_btnRename;
    wxButton* m_btnCopy;
    wxStaticBox* m_FilterPane;
    wxNotebook* m_Tabs;
    wxButton* m_btnSave;
    /// Control identifiers
    enum {
        ID_MYDIALOG = 10003,
        ID_LISTBOX = 10000,
        ID_BUTTON_ADD = 10004,
        ID_BUTTON_DELETE = 10005,
        ID_BUTTON_RENAME = 10006,
        ID_BUTTON_COPY = 10007,
        ID_NOTEBOOK = 10001,
        ID_BUTTON_SAVE = 10014,
        ID_BUTTON_IMPORT = 10009,
        ID_BUTTON_EXPORT = 10010
    };
////@end CSnpFilterUI member variables

    enum {
        ID_CHECKBOX_BASE = 11000,
        ID_RADIOBOX_BASE = 12000,
    };

///////////////////////////////////////////////////////////////////////////////
// PRIVATE Member variables
///////////////////////////////////////////////////////////////////////////////
private:
    TMapFilters                     m_mapFilters;      
    TMapFilterStates                m_mapChangeInfo;
    
    bool m_bNeedsSave;
    int  m_nSelectedRow;

    CGroupCheckboxes    *m_GrpChecksLinks;
    CGroupCheckboxes    *m_GrpChecksGeneFxn;
    CGroupCheckboxes    *m_GrpChecksMapping;
    CGroupRadioButtons  *m_GrpRadiosWeight;
    CGroupCheckboxes    *m_GrpChecksAlleleFreq;
    CGroupRadioButtons  *m_GrpRadiosAlleleFreq;
    CGroupCheckboxes    *m_GrpChecksGaP;
    CGroupRadioButtons  *m_GrpRadiosVarClass;
    CGroupCheckboxes    *m_GrpChecksQuality;

};

END_NCBI_SCOPE

#endif // GUI_WIDGETS_SNP___SNP_FILTER_UI_HPP
