#ifndef GUI_WIDGETS_SNP___TRACK_CREATOR_HPP
#define GUI_WIDGETS_SNP___TRACK_CREATOR_HPP

/*  $Id: track_creator_ui.hpp 18397 2008-11-28 19:45:17Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <corelib/ncbistd.hpp>

//#include <gui/utils/dialog.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/widgets/snp/filter/snp_filter.hpp>

#include <objects/seqloc/Seq_loc.hpp>
#include <objects/seq/Seq_annot.hpp>
#include <objmgr/scope.hpp>

#include <wx/timer.h>
#include <wx/dialog.h>
#include <wx/textctrl.h>
#include <wx/combobox.h>
#include <wx/gauge.h>
#include <wx/stattext.h>
#include <wx/button.h>

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CTRACKCREATORUI_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CTRACKCREATORUI_TITLE _("Create Filtered SNP Track")
#define SYMBOL_CTRACKCREATORUI_IDNAME ID_CTRACKCREATORUI
#define SYMBOL_CTRACKCREATORUI_SIZE wxSize(400, 300)
#define SYMBOL_CTRACKCREATORUI_POSITION wxDefaultPosition
////@end control identifiers

BEGIN_NCBI_SCOPE

/*!
 * CTrackCreatorUI class declaration
 */

class CTrackCreatorUI: public wxDialog
{
    DECLARE_DYNAMIC_CLASS( CTrackCreatorUI )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CTrackCreatorUI();
    CTrackCreatorUI( wxWindow* parent, wxWindowID id = SYMBOL_CTRACKCREATORUI_IDNAME, const wxString& caption = SYMBOL_CTRACKCREATORUI_TITLE, const wxPoint& pos = SYMBOL_CTRACKCREATORUI_POSITION, const wxSize& size = SYMBOL_CTRACKCREATORUI_SIZE, long style = SYMBOL_CTRACKCREATORUI_STYLE );

    /// Creation
    bool Create( objects::CScope *scope, objects::CSeq_loc *loc, wxWindow* parent=NULL, wxWindowID id = SYMBOL_CTRACKCREATORUI_IDNAME, const wxString& caption = SYMBOL_CTRACKCREATORUI_TITLE, const wxPoint& pos = SYMBOL_CTRACKCREATORUI_POSITION, const wxSize& size = SYMBOL_CTRACKCREATORUI_SIZE, long style = SYMBOL_CTRACKCREATORUI_STYLE );

    /// Destructor
    ~CTrackCreatorUI();

    /// Initialises member variables
    void Init();
    void InitControls();  // called by Create

    /// Creates the controls and sizers
    void CreateControls();

////@begin CTrackCreatorUI event handler declarations

    /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_COMBOBOX
    void OnComboboxSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
    void OnButtonEditClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
    void OnOkClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
    void OnCancelClick( wxCommandEvent& event );

////@end CTrackCreatorUI event handler declarations

    void OnTimer(wxTimerEvent& event);

    CRef<objects::CSeq_annot> GetResult();


////@begin CTrackCreatorUI member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CTrackCreatorUI member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

private:
    void x_EnabledEditOK(bool enable);
    void x_UpdateChoices();
    void x_OnChoice();
    void x_UpdateProgress();
    bool x_IsJobComplete();
    void x_Finish();

private:

    bool m_bCallFinish;
    SSnpFilter  m_Filter;
    CRef<objects::CSeq_loc> m_Loc;
    CRef<objects::CScope> m_Scope;
    CRef<objects::CSeq_annot> m_Annot;
    CAppJobDispatcher::TJobID m_ActiveJob;

    wxTimer m_Timer;


////@begin CTrackCreatorUI member variables
    wxTextCtrl* m_txtID;
    wxTextCtrl* m_txtRange;
    wxComboBox* m_choiceFilter;
    wxTextCtrl* m_txtName;
    wxGauge* m_Progress;
    wxStaticText* m_txtStatus;
    wxButton* m_btnEdit;
    wxButton* m_btnOK;
    /// Control identifiers
    enum {
        ID_CTRACKCREATORUI = 10002,
        ID_TEXTCTRL1 = 10015,
        ID_TEXTCTRL2 = 10017,
        ID_COMBOBOX = 10018,
        ID_TEXTCTRL_NAME = 10020,
        ID_GAUGE = 10016,
        ID_BUTTON_EDIT = 10019
    };
////@end CTrackCreatorUI member variables
    enum {
        ID_TIMER1 = 11000
    };
};

END_NCBI_SCOPE

#endif // GUI_WIDGETS_SNP___TRACK_CREATOR_HPP

