/*$Id: blast_search_params_panel.cpp 27823 2013-04-08 20:09:33Z voronov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_alignment/blast_search_params_panel.hpp>
#include <gui/packages/pkg_alignment/blast_search_params.hpp>

#include <gui/widgets/object_list/object_list_widget.hpp>

#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/wx/number_validator.hpp>
#include <gui/widgets/wx/float_text_validator.hpp>
#include <gui/widgets/wx/async_call.hpp>
#include <gui/widgets/wx/tax_id_helper.hpp>

#include <gui/core/winmask_files.hpp>
#include <gui/core/commands.hpp>

#include <objtools/blast/services/blast_services.hpp>

#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/image.h>
#include <wx/statbox.h>
#include <wx/stattext.h>
#include <wx/checkbox.h>
#include <wx/combobox.h>
#include <wx/choice.h>
#include <wx/msgdlg.h>



////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE


IMPLEMENT_DYNAMIC_CLASS( CBLASTSearchParamsPanel, wxPanel )

BEGIN_EVENT_TABLE( CBLASTSearchParamsPanel, wxPanel )
////@begin CBLASTSearchParamsPanel event table entries
    EVT_COMBOBOX( ID_GEN_CODE, CBLASTSearchParamsPanel::OnGenCodeSelected )

    EVT_CHOICE( ID_REPEAT_TYPE, CBLASTSearchParamsPanel::OnRepeatTypeSelected )

    EVT_CHOICE( ID_WMTI_CHOICE, CBLASTSearchParamsPanel::OnWindowmaskerTaxIdSelected )

    EVT_BUTTON( ID_BUTTON, CBLASTSearchParamsPanel::OnWMDownload )

////@end CBLASTSearchParamsPanel event table entries
END_EVENT_TABLE()


CBLASTSearchParamsPanel::CBLASTSearchParamsPanel()
{
    Init();
}


CBLASTSearchParamsPanel::CBLASTSearchParamsPanel( wxWindow* parent, 
                                                 wxWindowID id, 
                                                 const wxPoint& pos, 
                                                 const wxSize& size, 
                                                 long style)
{
    Init();
    Create(parent, id, pos, size, style);
}


bool CBLASTSearchParamsPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    wxPanel::Create( parent, id, pos, size, style );

    Hide();

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();

    return true;
}


CBLASTSearchParamsPanel::~CBLASTSearchParamsPanel()
{
////@begin CBLASTSearchParamsPanel destruction
////@end CBLASTSearchParamsPanel destruction
}

void CBLASTSearchParamsPanel::Init()
{
////@begin CBLASTSearchParamsPanel member initialisation
    m_WMTaxId = 0;
    m_GeneralSizer = NULL;
    m_ThreshLabel = NULL;
    m_ThreshInput = NULL;
    m_GenCodeLabel = NULL;
    m_GenCodeCombo = NULL;
    m_RepeatType = NULL;
    m_LCRegionsCheck = NULL;
    m_RepeatsCheck = NULL;
    m_WMStaticBox = NULL;
    m_WMTaxIds = NULL;
    m_WMDownload = NULL;
    m_CompartCheck = NULL;
    m_chbStandalone = NULL;
////@end CBLASTSearchParamsPanel member initialisation

    m_Params = NULL;
}


void CBLASTSearchParamsPanel::CreateControls()
{
////@begin CBLASTSearchParamsPanel content construction
    CBLASTSearchParamsPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText( itemPanel1, wxID_STATIC, _("Selected options:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxGROW|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer5Static = new wxStaticBox(itemPanel1, wxID_ANY, _("General Parameters"));
    wxStaticBoxSizer* itemStaticBoxSizer5 = new wxStaticBoxSizer(itemStaticBoxSizer5Static, wxVERTICAL);
    itemBoxSizer4->Add(itemStaticBoxSizer5, 1, wxGROW|wxALL, 5);

    m_GeneralSizer = new wxFlexGridSizer(0, 2, 0, 0);
    itemStaticBoxSizer5->Add(m_GeneralSizer, 1, wxGROW, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Word size:"), wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(itemStaticText7, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl8 = new wxTextCtrl( itemPanel1, ID_WORDSIZE, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(itemTextCtrl8, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText( itemPanel1, wxID_STATIC, _("e-value:"), wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(itemStaticText9, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl10 = new wxTextCtrl( itemPanel1, ID_E_VALUE, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(itemTextCtrl10, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW|wxALL, 5);

    m_ThreshLabel = new wxStaticText( itemPanel1, wxID_STATIC, _("Threshold:"), wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(m_ThreshLabel, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ThreshInput = new wxTextCtrl( itemPanel1, ID_THRESHOLD, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(m_ThreshInput, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW|wxALL, 5);

    m_GenCodeLabel = new wxStaticText( itemPanel1, ID_GEN_CODE, _("Use genetic code:"), wxDefaultPosition, wxDefaultSize, 0 );
    m_GeneralSizer->Add(m_GenCodeLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_GenCodeComboStrings;
    m_GenCodeCombo = new wxComboBox( itemPanel1, ID_GEN_CODE, wxEmptyString, wxDefaultPosition, wxSize(itemPanel1->ConvertDialogToPixels(wxSize(100, -1)).x, -1), m_GenCodeComboStrings, wxCB_READONLY );
    m_GeneralSizer->Add(m_GenCodeCombo, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_GeneralSizer->AddGrowableCol(1);

    wxBoxSizer* itemBoxSizer15 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer4->Add(itemBoxSizer15, 1, wxALIGN_TOP, 5);

    wxStaticBox* itemStaticBoxSizer16Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Filtering Parameters"));
    wxStaticBoxSizer* itemStaticBoxSizer16 = new wxStaticBoxSizer(itemStaticBoxSizer16Static, wxVERTICAL);
    itemBoxSizer15->Add(itemStaticBoxSizer16, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer17 = new wxBoxSizer(wxVERTICAL);
    itemStaticBoxSizer16->Add(itemBoxSizer17, 1, wxGROW, 5);

    wxCheckBox* itemCheckBox18 = new wxCheckBox( itemPanel1, ID_LOW_COMPLEXITY, _("Filter low complexity regions"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox18->SetValue(false);
    itemBoxSizer17->Add(itemCheckBox18, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText19 = new wxStaticText( itemPanel1, ID_STATIC_REPEAT_TYPE, _("Species specific repeats for:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer17->Add(itemStaticText19, 0, wxALIGN_LEFT|wxALL, 5);

    wxArrayString m_RepeatTypeStrings;
    m_RepeatType = new wxChoice( itemPanel1, ID_REPEAT_TYPE, wxDefaultPosition, wxDefaultSize, m_RepeatTypeStrings, 0 );
    itemBoxSizer17->Add(m_RepeatType, 0, wxALIGN_LEFT|wxALL, 5);

    m_LCRegionsCheck = new wxCheckBox( itemPanel1, ID_MASK_LOWERCASE, _("Mask lowercase regions"), wxDefaultPosition, wxDefaultSize, 0 );
    m_LCRegionsCheck->SetValue(false);
    itemBoxSizer17->Add(m_LCRegionsCheck, 0, wxALIGN_LEFT|wxALL, 5);

    m_RepeatsCheck = new wxCheckBox( itemPanel1, ID_MASK_REPEATS, _("Mask repeats"), wxDefaultPosition, wxDefaultSize, 0 );
    m_RepeatsCheck->SetValue(false);
    itemBoxSizer17->Add(m_RepeatsCheck, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer23 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer23, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText24 = new wxStaticText( itemPanel1, wxID_STATIC, _("BLAST algorithm requires at most one filtering database. It may be either"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticText24->Wrap(500);
    itemBoxSizer23->Add(itemStaticText24, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT, 5);

    wxStaticText* itemStaticText25 = new wxStaticText( itemPanel1, wxID_STATIC, _("Species Specific Repeats database above or Windowmasker Filtering below."), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticText25->Wrap(500);
    itemBoxSizer23->Add(itemStaticText25, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT, 5);

    m_WMStaticBox = new wxStaticBox(itemPanel1, wxID_ANY, _("Windowmasker filtering"));
    wxStaticBoxSizer* itemStaticBoxSizer26 = new wxStaticBoxSizer(m_WMStaticBox, wxHORIZONTAL);
    itemBoxSizer2->Add(itemStaticBoxSizer26, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText27 = new wxStaticText( itemPanel1, wxID_STATIC, _("Windowmasker tax IDs:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer26->Add(itemStaticText27, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_WMTaxIdsStrings;
    m_WMTaxIds = new wxChoice( itemPanel1, ID_WMTI_CHOICE, wxDefaultPosition, wxDefaultSize, m_WMTaxIdsStrings, 0 );
    itemStaticBoxSizer26->Add(m_WMTaxIds, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemStaticBoxSizer26->Add(7, 8, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_WMDownload = new wxButton( itemPanel1, ID_BUTTON, _("Download..."), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer26->Add(m_WMDownload, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer31Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Advanced Parameters"));
    wxStaticBoxSizer* itemStaticBoxSizer31 = new wxStaticBoxSizer(itemStaticBoxSizer31Static, wxHORIZONTAL);
    itemBoxSizer2->Add(itemStaticBoxSizer31, 0, wxGROW|wxALL, 5);

    wxTextCtrl* itemTextCtrl32 = new wxTextCtrl( itemPanel1, ID_ADV_TEXT, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer31->Add(itemTextCtrl32, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CompartCheck = new wxCheckBox( itemPanel1, ID_COMPARTMENTS, _("Link related hits together"), wxDefaultPosition, wxDefaultSize, 0 );
    m_CompartCheck->SetValue(false);
    if (CBLASTSearchParamsPanel::ShowToolTips())
        m_CompartCheck->SetToolTip(_("BLAST returns separate alignments for each query, and these separate alignments \ncan further be ordered into sets offering consistent non-overlapping query and subject \ncoverage.  The sequence viewer offers the ability to evaluate the original BLAST hits \non-the-fly and link together alignments that meet a strict definition of non-overlapping \nquery and subject coverage.\n"));
    itemBoxSizer2->Add(m_CompartCheck, 0, wxALIGN_LEFT|wxALL, 15);

    wxBoxSizer* itemBoxSizer34 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer34, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText35 = new wxStaticText( itemPanel1, wxID_STATIC, _("Job Title:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer34->Add(itemStaticText35, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl36 = new wxTextCtrl( itemPanel1, ID_TITLE_TEXT, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer34->Add(itemTextCtrl36, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    //itemBoxSizer2->Add(7, 32, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_chbStandalone = new wxCheckBox( itemPanel1, ID_STANDALONE, _("Run tool as standalone"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chbStandalone->SetValue(false);
    itemBoxSizer2->Add(m_chbStandalone, 0, wxALIGN_LEFT|wxALL, 5);

    // Set validators
    itemTextCtrl8->SetValidator( CNumberValidator(& m_WordSize, 0, 100) );
    itemTextCtrl10->SetValidator( CFloatTextValidator(& m_eValue) );
    m_ThreshInput->SetValidator( CNumberValidator(& m_Threshold, 0, 100) );
    m_GenCodeCombo->SetValidator( wxGenericValidator(& m_GeneticCode) );
    itemCheckBox18->SetValidator( wxGenericValidator(& m_FilterLowComplex) );
    m_LCRegionsCheck->SetValidator( wxGenericValidator(& m_MaskLCRegions) );
    m_RepeatsCheck->SetValidator( wxGenericValidator(& m_MaskRepeats) );
    m_WMTaxIds->SetValidator( wxGenericValidator(& m_WMTaxId) );
    itemTextCtrl32->SetValidator( wxGenericValidator(& m_AdvParams) );
    m_CompartCheck->SetValidator( wxGenericValidator(& m_FindCompart) );
    itemTextCtrl36->SetValidator( wxGenericValidator(& m_JobTitle) );
////@end CBLASTSearchParamsPanel content construction

    x_InitGeneticCodesCombo();

    m_RepeatType->Append(wxT("none"));
    CBLASTParams::TRepeatLibs libs = CBLASTParams::GetAllRepeatLibs();
    ITERATE (CBLASTParams::TRepeatLibs, iter, libs) {
        m_RepeatType->Append(ToWxString(*iter));
    }
}


void CBLASTSearchParamsPanel::x_InitGeneticCodesCombo()
{
    vector<string> labels;
    CBLASTParams::GetGeneticCodeLabels(labels);

    wxArrayString arr;
    ToArrayString(labels, arr);

    m_GenCodeCombo->Clear();
    m_GenCodeCombo->Append(arr);
}


bool CBLASTSearchParamsPanel::ShowToolTips()
{
    return true;
}


wxBitmap CBLASTSearchParamsPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBLASTSearchParamsPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBLASTSearchParamsPanel bitmap retrieval
}


wxIcon CBLASTSearchParamsPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBLASTSearchParamsPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBLASTSearchParamsPanel icon retrieval
}


void CBLASTSearchParamsPanel::SetParams( CBLASTParams* params, bool local )
{
    m_Params = params;
    m_Local = local;
}


// hide / show some of the controls depending on the program
void CBLASTSearchParamsPanel::x_HideShowInputs()
{
    blast::EProgram prg = m_Params->GetCurrProgram();

    bool en_gc = CBLASTParams::NeedsGeneticCode(prg);
    bool en_thresh = CBLASTParams::NeedsThreshold(prg);

    m_ThreshLabel->Show(en_thresh);
    m_ThreshInput->Show(en_thresh);

    m_GenCodeLabel->Show(en_gc);
    m_GenCodeCombo->Show(en_gc);

    m_LCRegionsCheck->Show(m_Local);
    m_RepeatsCheck->Show(m_Local);

    m_chbStandalone->Show(m_Local);

    m_RepeatType->Show(!m_Local);
    FindWindow(ID_STATIC_REPEAT_TYPE)->Show(!m_Local);

    GetSizer()->Show(m_WMStaticBox->GetContainingSizer(), m_Params->IsNucInput());

    Layout();
}


class CGetNetTaxIds : public CAsyncCall::Call
{
public:
    CGetNetTaxIds(vector<int>& taxIds) : m_TaxIds(taxIds) {}

    virtual void Execute() {
        CBlast4_get_windowmasked_taxids_reply::Tdata data =
            CBlastServices().GetTaxIdWithWindowMaskerSupport();
        copy(data.begin(), data.end(), back_inserter(m_TaxIds));
    }
private:
    vector<int>& m_TaxIds;
};


bool CBLASTSearchParamsPanel::TransferDataToWindow()
{
    blast::EProgram prg = m_Params->GetCurrProgram();
    CBLASTParams::SProgParams& prg_params =  m_Params->GetCurrParams();

    if(CBLASTParams::NeedsThreshold(prg)) {
        m_Threshold = prg_params.m_Threshold;
    }

    m_WordSize = prg_params.m_WordSize;
    m_eValue = prg_params.m_eValue;
    m_FilterLowComplex = prg_params.m_FilterLowComplex;

    if(CBLASTParams::NeedsGeneticCode(prg)) {
        int code_id = prg_params.m_GeneticCode;
        m_GeneticCode = ToWxString(CBLASTParams::GetGeneticCodeLabel(code_id));
    }

    if(m_Local) {
        m_MaskLCRegions = prg_params.m_MaskLowercase;
        m_MaskRepeats = prg_params.m_MaskRepeats;
    }

    int pos = m_RepeatType->FindString(ToWxString(prg_params.m_RepeatLib));
    if (pos == wxNOT_FOUND) {
        pos = m_RepeatType->FindString(wxT("none"));
    }
    if (pos != wxNOT_FOUND) {
        m_RepeatType->SetSelection(pos);
    }

    m_AdvParams = ToWxString(m_Params->GetAdvancedParams());
    m_JobTitle = ToWxString(m_Params->GetJobTitle());

    x_HideShowInputs();

    if( m_Local ){
        m_chbStandalone->SetValue( m_Params->IsStandaloneRequested() );
    }

    m_CompartCheck->SetValue( m_Params->IsCompartmentsRequested() );


    if (m_Params->IsNucInput()) {
        vector<int> masks;

        if ( m_Local ) {
            CWinMaskerFileStorage::GetInstance().GetWinMaskerTaxIds( masks );
        }
        else {
            try {
                CGetNetTaxIds getTaxIds(masks);
                CAsyncCall::Execute(getTaxIds);
            } catch( const CException& e ) {
                LOG_POST(Error << "Failed to get Net WinMasks: " << e.GetMsg());
            } catch( const exception& e ) {
                LOG_POST(Error << "Failed to get Net WinMasks: " << e.what());
            }
        }

        int selectTaxId = prg_params.m_WM_TaxId;
        CTaxIdHelper::FillTaxIds(*m_WMTaxIds, masks, selectTaxId);
    }

    return wxPanel::TransferDataToWindow();
}


bool CBLASTSearchParamsPanel::TransferDataFromWindow()
{
    if(wxPanel::TransferDataFromWindow())   {
        blast::EProgram prg = m_Params->GetCurrProgram();
        CBLASTParams::SProgParams& prg_params =  m_Params->GetCurrParams();

        if(m_Local) {
            prg_params.m_MaskLowercase = m_MaskLCRegions;
            prg_params.m_MaskRepeats = m_MaskRepeats;
        }

        if(CBLASTParams::NeedsThreshold(prg)) {
            prg_params.m_Threshold = m_Threshold;
        }

        prg_params.m_WordSize = m_WordSize;
        prg_params.m_eValue = m_eValue;
        prg_params.m_FilterLowComplex = m_FilterLowComplex;

        if(CBLASTParams::NeedsGeneticCode(prg)) {
            string s = ToStdString(m_GeneticCode);
            prg_params.m_GeneticCode = CBLASTParams::GetGeneticCodeByLabel(s);
        }

        prg_params.m_RepeatLib =
            m_Local ? "none" : ToStdString(m_RepeatType->GetStringSelection());
        m_Params->GetAdvancedParams() = ToStdString(m_AdvParams);
        m_Params->GetJobTitle() = ToStdString(m_JobTitle);

        if( m_Local ){
            m_Params->StandaloneRequested( m_chbStandalone->GetValue() );
        }

        m_Params->FindCompartments( m_CompartCheck->GetValue() );

        int taxId = 0, index = m_WMTaxIds->GetSelection();
        if (wxNOT_FOUND != index)
            taxId = (int)(size_t)m_WMTaxIds->GetClientData(index);
        prg_params.m_WM_TaxId = taxId;

        if( taxId != 0 ){
            CWinMaskerFileStorage& storage = CWinMaskerFileStorage::GetInstance();
            m_Params->SetWM_Dir( ToStdString(storage.GetPath()) );
        }

        return true;
    }
    return false;
}


void CBLASTSearchParamsPanel::OnGenCodeSelected( wxCommandEvent& event )
{
}

void CBLASTSearchParamsPanel::OnRepeatTypeSelected( wxCommandEvent& event )
{
    if( m_RepeatType->IsEnabled() ){
        m_WMTaxIds->Enable( event.GetSelection() == 0 );
        m_WMDownload->Enable( event.GetSelection() == 0 );
    }
}

void CBLASTSearchParamsPanel::OnWindowmaskerTaxIdSelected( wxCommandEvent& event )
{
    if( m_WMTaxIds->IsEnabled() ){
        m_RepeatType->Enable( event.GetSelection() == 0 );
    }
}


void CBLASTSearchParamsPanel::OnWMDownload( wxCommandEvent& event )
{
    event.SetEventType( wxEVT_COMMAND_MENU_SELECTED );
    event.SetId( eCmdWindowMaskerFiles );

    wxTheApp->GetTopWindow()->GetEventHandler()->ProcessEvent( event );

    wxString result = event.GetString();
    if( !result.empty() ){

        string msg;
        if( result == wxT("Download") ){
            msg = "WindowMasker database download task started.";
            msg += "\nPlease re-run Tools dialog later, when the task finishes.";

        } else if( result == wxT("Reconfigured") ){
            msg = "WindowMasker database path configuration changed.";
            msg += "\nPlease re-start Tools dialog.";
        } else {
            msg = "WindowMasker configuration changed.";
            msg += "\nPlease re-start Tools dialog.";
        }

        wxMessageBox( ToWxString(msg) ,wxT("Genome Workbench"), wxOK | wxICON_EXCLAMATION );

        wxCloseEvent clev( wxEVT_CLOSE_WINDOW, wxID_ANY );
        clev.SetEventObject( this );
        clev.ResumePropagation( wxEVENT_PROPAGATE_MAX );

        ProcessEvent( clev );
    }
}

END_NCBI_SCOPE

