/*  $Id: net_blast_load_option_panel.cpp 27150 2012-12-31 16:07:52Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_alignment/net_blast_load_option_panel.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/sizer.h>
#include <wx/image.h>
#include <wx/icon.h>
#include <wx/stattext.h>
#include <wx/statbox.h>


////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CNetBLASTLoadOptionPanel, wxPanel )

BEGIN_EVENT_TABLE( CNetBLASTLoadOptionPanel, wxPanel )
////@begin CNetBLASTLoadOptionPanel event table entries
    EVT_TEXT( ID_RID_INPUT, CNetBLASTLoadOptionPanel::OnAccInputUpdated )

////@end CNetBLASTLoadOptionPanel event table entries
END_EVENT_TABLE()


CNetBLASTLoadOptionPanel::CNetBLASTLoadOptionPanel()
{
    Init();
}


CNetBLASTLoadOptionPanel::CNetBLASTLoadOptionPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


bool CNetBLASTLoadOptionPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CNetBLASTLoadOptionPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CNetBLASTLoadOptionPanel creation
    return true;
}


CNetBLASTLoadOptionPanel::~CNetBLASTLoadOptionPanel()
{
////@begin CNetBLASTLoadOptionPanel destruction
////@end CNetBLASTLoadOptionPanel destruction
}


void CNetBLASTLoadOptionPanel::Init()
{
////@begin CNetBLASTLoadOptionPanel member initialisation
    m_RIDInput = NULL;
////@end CNetBLASTLoadOptionPanel member initialisation
}


void CNetBLASTLoadOptionPanel::CreateControls()
{
////@begin CNetBLASTLoadOptionPanel content construction
    CNetBLASTLoadOptionPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemPanel1, wxID_ANY, _("BLAST RIDs to load"));
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 1, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("Separate BLAST RIDs with spaces, commas or semicolons"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer3->Add(itemStaticText4, 0, wxGROW|wxALL, 5);

    m_RIDInput = new CRichTextCtrl( itemPanel1, ID_RID_INPUT, wxEmptyString, wxDefaultPosition, wxSize(200, 50), wxWANTS_CHARS|wxTE_RICH2|wxTE_MULTILINE );
    itemStaticBoxSizer3->Add(m_RIDInput, 1, wxGROW|wxALL, 5);

////@end CNetBLASTLoadOptionPanel content construction
}


string CNetBLASTLoadOptionPanel::GetInput() const
{
    return ToStdString(m_RIDInput->GetValue());
}


void CNetBLASTLoadOptionPanel::SetInput(const string& input)
{
    _ASSERT(m_RIDInput);
    m_RIDInput->SetValue(ToWxString(input));
}


bool CNetBLASTLoadOptionPanel::IsInputValid()
{
    return true;
}


void CNetBLASTLoadOptionPanel::OnAccInputUpdated( wxCommandEvent& event )
{
    // Remove formatting that may be added on mac paste events (and
    // on mac wx 2.9.3 paste handler is not called)
    m_RIDInput->RemoveFormatting();

    x_ValidateInput();
}

static string sDelim(" \t\n,;");

void CNetBLASTLoadOptionPanel::x_ValidateInput()
{
    //TODO
}


void CNetBLASTLoadOptionPanel::GetRIDs(vector<string>& rids)
{
    string text = ToStdString(m_RIDInput->GetValue());

    vector<string> tokens;
    NStr::Tokenize(text, sDelim, tokens, NStr::eMergeDelims);

    rids = tokens;
}


bool CNetBLASTLoadOptionPanel::ShowToolTips()
{
    return true;
}


wxBitmap CNetBLASTLoadOptionPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CNetBLASTLoadOptionPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CNetBLASTLoadOptionPanel bitmap retrieval
}

wxIcon CNetBLASTLoadOptionPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CNetBLASTLoadOptionPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CNetBLASTLoadOptionPanel icon retrieval
}


END_NCBI_SCOPE
