/*  $Id: bins_glyph_types.cpp 19838 2009-08-12 12:30:51Z quintosm $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Melvin Quintos
*
* File Description:
*    CBinsGlyph -- Contains method definitions for CBinsGlyph that relate
*       to type specific information (i.e.  x_GetTooltipCited, x_GetTooltipGaP)
*
*/

#include <ncbi_pch.hpp>
#include <gui/packages/pkg_snp/bins/bins_glyph.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

void CBinsGlyph::x_GetTooltipCITED(const CBinsGlyph::SBin& bin, string& tt) const
{
    // Display count
    int counts = bin.count;

    if(counts > 15) {
        tt += ">15 Variants\n";
    }
    else {
        tt += NStr::IntToString(counts);
        tt += " Variant(s)\n";
    }

    // Display at most 15
    for (int i=0; i < counts && i < 15; i++) {
        // display rsid
        unsigned int rsid = bin.snps[i]->snpid;
        tt += "\nrs";
        tt += NStr::UIntToString(rsid);

        // display pmids
        tt += "\nPmid(s): ";
        tt += bin.snps[i]->pmids;
    }

    if (counts > 15) {
        tt += "\n\nZoom in for more";
    }
}

void CBinsGlyph::x_GetTooltipCLIN(const CBinsGlyph::SBin& bin, string& tt) const
{
    // Display count
    int counts = bin.count;

    if(counts > 15) {
        tt += ">15 Variants\n";
    }
    else {
        tt += NStr::IntToString(counts);
        tt += " Variant(s)\n";
    }

    // Display at most 15
    for (int i=0; i < counts && i < 15; i++) {
        // display rsid
        unsigned int rsid = bin.snps[i]->snpid;
        tt += "\nrs";
        tt += NStr::UIntToString(rsid);
    }

    if (counts > 15) {
        tt += "\n\nZoom in for more";
    }    
}

void CBinsGlyph::x_GetTooltipGAP(const CBinsGlyph::SBin& bin, string& tt) const
{
    // display rsid
    int rsid = bin.sigSnp->snpid;
    if(rsid != 0) {
        tt += "Significant: rs ";
        tt += NStr::IntToString(rsid);
    }

    // Calculate value
    double value = bin.sigSnp->pvalue;

    if(value > 15)
        tt += "\n-log10(pval): >15";
    else {
        tt += "\n-log10(pval): ";
        tt += NStr::DoubleToString(value);
    }

    // Calculate the count
    int counts = bin.count;

    if(counts > 15) {
        tt += "\n\nSNP Count: >15";
    }
    else {
        tt += "\n\nSNP Count: ";
        tt += NStr::IntToString(counts);
    }
}

void CBinsGlyph::x_GetTooltipGCAT(const CBinsGlyph::SBin& bin, string& tt) const
{
    // display rsid
    int rsid = bin.sigSnp->snpid;
    if(rsid != 0) {
        tt += "Significant: rs ";
        tt += NStr::IntToString(rsid);
    }

    // Calculate value
    double value = bin.sigSnp->pvalue;

    if(value > 15)
        tt += "\n-log10(pval): >15";
    else {
        tt += "\n-log10(pval): ";
        tt += NStr::DoubleToString(value);
    }

    // Calculate the count
    int counts = bin.count;

    if(counts > 15) {
        tt += "\n\nSNP Count: >15";
    }
    else {
        tt += "\n\nSNP Count: ";
        tt += NStr::IntToString(counts);
    }
}


END_NCBI_SCOPE
