/*  $Id: group_checkboxes.cpp 19286 2009-05-15 12:57:01Z quintosm $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/snp/filter/group_checkboxes.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/window.h>
#include <wx/checkbox.h>
#include <wx/sizer.h>

BEGIN_NCBI_SCOPE

void CGroupCheckboxes::OnClick(wxCommandEvent &event)
{
    wxWindow *o;
    o = reinterpret_cast<wxWindow*>(event.GetEventObject());
    if ( o ) {
        SetCheck(o, event.IsChecked());
    }
    event.Skip(); // continue to call other listeners
}

CGroupCheckboxes::CGroupCheckboxes(
    wxWindow * parent,
    int x,
    int y,
    int width,
    int height)
    : m_nX(x), m_nY(y), m_nWidth(width), m_nHeight(height), m_pParent(parent) 
{
    m_nCurrentY = y;
    m_bHasUIChanges = false;
    m_pParent->PushEventHandler(this);
}

void CGroupCheckboxes::Add(int id, const char * text)
{
    wxCheckBox* checkbox = new wxCheckBox(  m_pParent,
                                            id,
                                            ToWxString(text),
                                            wxPoint(m_nX, m_nCurrentY), // pos
                                            wxDefaultSize );

    // Register Callback
    Connect(id, wxEVT_COMMAND_CHECKBOX_CLICKED,
        wxCommandEventHandler(CGroupCheckboxes::OnClick));

    // Add control to parent's sizer (if exists)
    wxSizer *s = m_pParent->GetSizer();
    if (s) {
        s->Add(checkbox, 0, wxTOP|wxBOTTOM|wxLEFT, 5 );
    }

    // Create mappings
    m_mapButtons[checkbox] = id;
    m_mapIDs[id]           = checkbox;

    // increment height for future controls
    m_nCurrentY += m_nHeight;
}

void  CGroupCheckboxes::GetCheckList(list<int> *list_checked)
{
    list_checked->clear();

    typedef set<int> TSetInt;
    ITERATE(TSetInt, iter, m_setChecked) {
        list_checked->push_back(*iter);
    }
}

void CGroupCheckboxes::Reset()
{
    typedef map<wxWindow *, int> TMapButtons;
    ITERATE (TMapButtons, iter, m_mapButtons) {
        wxCheckBox* b;
        b = (wxCheckBox*)iter->first;
        b->SetValue(false);
    }

    m_bHasUIChanges = false;
    m_setChecked.clear();
}

bool CGroupCheckboxes::HasChanges() const
{
    return m_bHasUIChanges ;
}

void CGroupCheckboxes::SetCheck(int id, bool v)
{
    wxCheckBox * button;

    map<int, wxWindow*>::iterator it;
    it = m_mapIDs.find(id);

    if (it != m_mapIDs.end()) {
        button = (wxCheckBox*) it->second;
        button->SetValue(v);
        SetCheck(button, v);     // this also sets m_bHasUIChanges to true
        m_bHasUIChanges = false; // reset to false since this is not a UI change
    }
}

void CGroupCheckboxes::SetCheck(wxWindow *o, bool check)
{
    map<wxWindow*, int>::iterator it;
    it = m_mapButtons.find(o);

    if (it != m_mapButtons.end()) {
        int checkbox_id = it->second;

        if (check) {
            m_setChecked.insert(checkbox_id);
        }
        else {
            m_setChecked.erase(checkbox_id);
        }

        m_bHasUIChanges = true;
    }

}

END_NCBI_SCOPE

