/*  $Id: group_radios.cpp 19286 2009-05-15 12:57:01Z quintosm $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/snp/filter/group_radios.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/window.h>
#include <wx/radiobut.h>
#include <wx/sizer.h>

BEGIN_NCBI_SCOPE

void CGroupRadioButtons::OnClick(wxCommandEvent &event)
{
    wxWindow *o;
    o = reinterpret_cast<wxWindow*>(event.GetEventObject());
    if ( o ) {
        SetChoice(o);
    }
    event.Skip(); // continue to call other listeners
}

CGroupRadioButtons::CGroupRadioButtons(
    wxWindow * parent,
    int x,
    int y,
    int width,
    int height)
    : m_nChoice(-1), m_nX(x), m_nY(y), m_nWidth(width), m_nHeight(height), m_pParent(parent) 
{
    m_nCurrentY = y;
    m_bHasUIChanges = false;
    m_bFirstButton  = true;  // Set flag indicating the first button needs to be created
    parent->PushEventHandler(this); // put event handler to Window parent

    Connect(wxID_ANY, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
        wxCommandEventHandler(CGroupRadioButtons::OnClick));
}

void CGroupRadioButtons::Add(int id, const char * text)
{
    wxRadioButton* button;
    long style = 0;
    if (m_bFirstButton) {
        style = wxRB_GROUP;
        m_bFirstButton = false;
    }

    button = new wxRadioButton(  m_pParent,
                                 wxID_ANY,
                                 ToWxString(text),
                                 wxPoint(m_nX, m_nCurrentY), // pos
                                 wxDefaultSize,
                                 style);

    // Add control to parent's sizer (if exists)
    wxSizer *s = m_pParent->GetSizer();
    if (s) {
        s->Add(button, 0, wxTOP|wxBOTTOM|wxLEFT, 5 );
    }

    // Create mappings
    m_mapButtons[button] = id;
    m_mapIDs[id]         = button;

    // increment height for future controls
    m_nCurrentY += m_nHeight;
}

int  CGroupRadioButtons::GetChoice()
{
    return m_nChoice;
}

void CGroupRadioButtons::Reset()
{
    wxRadioButton* b;

    b = (wxRadioButton*)m_mapIDs.begin()->second;

    if (b) {
        b->SetValue(true);
        m_nChoice = m_mapButtons[b];
    }

    m_bHasUIChanges = false;

}

bool CGroupRadioButtons::HasChanges() const
{
    return m_bHasUIChanges;
}

void CGroupRadioButtons::SetChoice(int id)
{
    map<int, wxWindow *>::iterator it;
    it = m_mapIDs.find(id);

    if (it != m_mapIDs.end() ) {
        wxRadioButton * o = (wxRadioButton*) it->second;
        o->SetValue(true);
        SetChoice(o);
        m_bHasUIChanges = false;
    }
}

void CGroupRadioButtons::SetChoice(wxWindow *o)
{
    map<wxWindow *, int>::iterator it;
    it = m_mapButtons.find(o);
    if (it != m_mapButtons.end()) {
        m_nChoice = it->second;
        m_bHasUIChanges = true;
    }
}

END_NCBI_SCOPE

