/*  $Id: snp_filter_list_dlg.cpp 25491 2012-03-27 17:52:21Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include <gui/widgets/snp/filter/snp_filter_list_dlg.hpp>
#include <gui/objutils/registry.hpp>
#include <gui/widgets/snp/filter/snp_filter_ui.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <wx/sizer.h>
#include <wx/icon.h>
#include <wx/bitmap.h>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

/*!
 * CSnpFilterListDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSnpFilterListDialog, wxDialog )


/*!
 * CSnpFilterListDialog event table definition
 */

BEGIN_EVENT_TABLE( CSnpFilterListDialog, wxDialog )

////@begin CSnpFilterListDialog event table entries
    EVT_LISTBOX( ID_LISTBOX1, CSnpFilterListDialog::OnListboxSelection )

    EVT_BUTTON( ID_BUTTON_EDIT, CSnpFilterListDialog::OnButtonEditClick )

////@end CSnpFilterListDialog event table entries

END_EVENT_TABLE()


/*!
 * CSnpFilterListDialog constructors
 */

CSnpFilterListDialog::CSnpFilterListDialog()
{
    Init();
}

CSnpFilterListDialog::CSnpFilterListDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    CreateX(false, parent, id, caption, pos, size, style);
}


/*!
 * CSnpFilterUI creator
 */

bool CSnpFilterListDialog::CreateX( bool showEditButton, wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSnpFilterListDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSnpFilterListDialog creation
    m_bShowEditButton = showEditButton;

    if (m_bShowEditButton) {
        m_BtnEdit->Show(true);
    }
    x_UpdateListBox();

    return true;
}


/*!
 * CSnpFilterListDialog destructor
 */

CSnpFilterListDialog::~CSnpFilterListDialog()
{
////@begin CSnpFilterListDialog destruction
////@end CSnpFilterListDialog destruction
}


/*!
 * Member initialisation
 */

void CSnpFilterListDialog::Init()
{
////@begin CSnpFilterListDialog member initialisation
    m_nSelectedRow = 0;
    m_ListBox = NULL;
    m_BtnEdit = NULL;
    m_BtnOK = NULL;
    m_BtnCancel = NULL;
////@end CSnpFilterListDialog member initialisation
    m_bShowEditButton=false;
    m_nSelectedRow = wxNOT_FOUND;

}


/*!
 * Control creation for CSnpFilterUI
 */

void CSnpFilterListDialog::CreateControls()
{
////@begin CSnpFilterListDialog content construction
    CSnpFilterListDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxArrayString m_ListBoxStrings;
    m_ListBox = new wxListBox( itemDialog1, ID_LISTBOX1, wxDefaultPosition, wxSize(-1, 245), m_ListBoxStrings, wxLB_SINGLE|wxLB_ALWAYS_SB|wxLB_SORT );
    itemBoxSizer2->Add(m_ListBox, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer4->Add(itemBoxSizer5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_BtnEdit = new wxButton( itemDialog1, ID_BUTTON_EDIT, wxT("Edit"), wxDefaultPosition, wxDefaultSize, 0 );
    m_BtnEdit->Show(false);
    itemBoxSizer5->Add(m_BtnEdit, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxFIXED_MINSIZE, 5);

    itemBoxSizer5->Add(85, 0, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer8 = new wxStdDialogButtonSizer;

    itemBoxSizer4->Add(itemStdDialogButtonSizer8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    m_BtnOK = new wxButton( itemDialog1, wxID_OK, wxT("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    m_BtnOK->SetDefault();
    itemStdDialogButtonSizer8->AddButton(m_BtnOK);

    m_BtnCancel = new wxButton( itemDialog1, wxID_CANCEL, wxT("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer8->AddButton(m_BtnCancel);

    itemStdDialogButtonSizer8->Realize();

    // Connect events and objects
    m_ListBox->Connect(ID_LISTBOX1, wxEVT_LEFT_DOWN, wxMouseEventHandler(CSnpFilterListDialog::OnMouseEvent), NULL, this);
////@end CSnpFilterListDialog content construction

}

void CSnpFilterListDialog::OnMouseEvent(wxMouseEvent &event)
{
    wxPoint p = event.GetPosition();
    int selected = m_ListBox->HitTest(p);
    if (selected == wxNOT_FOUND) {
        m_nSelectedRow = wxNOT_FOUND;
        m_ListBox->SetSelection(wxNOT_FOUND);
        x_OnBrowserChange();
    }
    else {
        event.Skip();
    }
}

/*!
 * Should we show tooltips?
 */

bool CSnpFilterListDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSnpFilterListDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSnpFilterListDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSnpFilterListDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSnpFilterListDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSnpFilterListDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSnpFilterListDialog icon retrieval
}

/*!
 * GetFilter
 */
string CSnpFilterListDialog::GetFilter(SSnpFilter *store_filter)
{
    string strFilter;

    if ( (m_nSelectedRow != wxNOT_FOUND)) {
        string name = ToStdString(m_ListBox->GetString(m_nSelectedRow));
        const SSnpFilter &f = m_mapFilters[name];        
        f.SerializeTo(strFilter);

        if (store_filter) 
            store_filter->SerializeFrom(strFilter);
    }

    return strFilter;
}


/*!
 * x_UpdateBrowser
 */
void CSnpFilterListDialog::x_UpdateListBox()
{
    m_ListBox->Clear();
    m_mapFilters.clear();

    // Read filters from registry
    CGuiRegistry& reg = CGuiRegistry::GetInstance();
    CGuiRegistry::TReadWriteView view = reg.GetReadWriteView("GBENCH.Plugins.View.Snp");

    typedef list<string> TListStrings;
    TListStrings filters;
    view.GetStringList("filters", filters);

    SSnpFilter filter; // a temporary filter object

    ITERATE(TListStrings, iter, filters) {
        filter.SerializeFrom(*iter);
        m_mapFilters[filter.name] = filter;  // use default copying of data
        m_ListBox->Append(ToWxString(filter.name));
    }

    x_IsFilterSelected(false);
}

/*!
 * x_IsFilterSelected
 */
void CSnpFilterListDialog::x_IsFilterSelected(bool is_selected)
{
    if (is_selected) {
        m_BtnOK->SetLabel(wxT("OK"));
    }
    else {
        m_BtnOK->SetLabel(wxT("No Filter"));
    }
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
 */

void CSnpFilterListDialog::OnButtonEditClick( wxCommandEvent& /*event*/ )
{
    string name;
    CSnpFilterUI ui(this);
        
    if (m_nSelectedRow != wxNOT_FOUND) {
        name = ToStdString(m_ListBox->GetString(m_nSelectedRow));
        ui.SetSelection(name);
    }

    ui.Center();
    if (ui.ShowModal() == wxID_OK) {
        x_UpdateListBox();

        int sel = ui.GetSelectedRow();
        m_ListBox->SetSelection(sel);
        x_OnBrowserChange();
    }    
}


void CSnpFilterListDialog::x_OnBrowserChange()
{
    m_nSelectedRow = m_ListBox->GetSelection();

    if (m_nSelectedRow != wxNOT_FOUND)
        x_IsFilterSelected(true);
    else
        x_IsFilterSelected(false);
}

/*!
 * wxEVT_COMMAND_LISTBOX_SELECTED event handler for ID_LISTBOX1
 */

void CSnpFilterListDialog::OnListboxSelection( wxCommandEvent& /*event*/ )
{
    x_OnBrowserChange();
}


END_NCBI_SCOPE
