/*  $Id: snp_filter_ui_load.cpp 18611 2009-01-09 21:45:44Z quintosm $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/snp/filter/snp_filter_ui.hpp>
#include <gui/widgets/snp/filter/group_checkboxes.hpp>
#include <gui/widgets/snp/filter/group_radios.hpp>

#include <gui/widgets/wx/wx_utils.hpp>


BEGIN_NCBI_SCOPE

void CSnpFilterUI::x_LoadFilter()
{
    _ASSERT(m_nSelectedRow!=wxNOT_FOUND);

    x_ResetFilterTab();

    string text;
    text = ToStdString(m_Browser->GetString(m_nSelectedRow));
    SSnpFilter & filter = *m_mapFilters[text]; // retrieve filter

    // change title
    m_FilterPane->SetLabel(ToWxString(text));

    // go through each control and set the correct values to them
    if (filter.checkAlleleFreqChoice) {
        m_GrpRadiosAlleleFreq->SetChoice(filter.choiceAlleleFreq);
    }
    if (filter.checkAlleleFreqList) {
        ITERATE(SSnpFilter::TList, iter, filter.listAlleleFreqs) {
            m_GrpChecksAlleleFreq->SetCheck(*iter, true);
        }
    }
    if (filter.checkGaP_Hapmap) {
        ITERATE(SSnpFilter::TList, iter, filter.listGaP_Hapmaps) {
            m_GrpChecksGaP->SetCheck(*iter, true);
        }
    }
    if (filter.checkGeneFxn) {
        ITERATE(SSnpFilter::TList, iter, filter.listGeneFxns) {
            m_GrpChecksGeneFxn->SetCheck(*iter, true);
        }
    }
    if (filter.checkLinks) {
        ITERATE(SSnpFilter::TList, iter, filter.listLinks) {
            m_GrpChecksLinks->SetCheck(*iter, true);
        }
    }
    if (filter.checkMapping) {
        ITERATE(SSnpFilter::TList, iter, filter.listMappings) {
            m_GrpChecksMapping->SetCheck(*iter, true);
        }
    }
    if (filter.checkQualityCheck) {
        ITERATE(SSnpFilter::TList, iter, filter.listQualityChecks) {
            m_GrpChecksQuality->SetCheck(*iter, true);
        }
    }
    if (filter.checkVarClass) {
        m_GrpRadiosVarClass->SetChoice(filter.choiceVarClass);
    }
    if (filter.checkWeight) {
        m_GrpRadiosWeight->SetChoice(filter.choiceWeight);
    }

    m_bNeedsSave = false;
}

END_NCBI_SCOPE
