/*  $Id: snp_filter_ui_save.cpp 18611 2009-01-09 21:45:44Z quintosm $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/snp/filter/snp_filter_ui.hpp>
#include <gui/widgets/snp/filter/group_checkboxes.hpp>
#include <gui/widgets/snp/filter/group_radios.hpp>

#include <gui/widgets/wx/wx_utils.hpp>

BEGIN_NCBI_SCOPE

void CSnpFilterUI::x_OnBtnSave()
{
    _ASSERT(m_nSelectedRow!=wxNOT_FOUND);

    // retrieve the filter
    string name;
    name = ToStdString(m_Browser->GetString(m_nSelectedRow));
    SSnpFilter & filter = *m_mapFilters[name]; // retreive filter

    // overwrite all properties of the filter
    m_GrpChecksLinks        ->GetCheckList(&filter.listLinks);
    m_GrpChecksAlleleFreq   ->GetCheckList(&filter.listAlleleFreqs);
    m_GrpChecksGaP          ->GetCheckList(&filter.listGaP_Hapmaps);
    m_GrpChecksMapping      ->GetCheckList(&filter.listMappings);
    m_GrpChecksQuality      ->GetCheckList(&filter.listQualityChecks);
    m_GrpChecksGeneFxn      ->GetCheckList(&filter.listGeneFxns);

    filter.choiceAlleleFreq = m_GrpRadiosAlleleFreq ->GetChoice();
    filter.choiceVarClass   = m_GrpRadiosVarClass   ->GetChoice();
    filter.choiceWeight     = m_GrpRadiosWeight     ->GetChoice();

    filter.checkLinks           = (filter.listLinks.empty())        ? false : true;
    filter.checkAlleleFreqList  = (filter.listAlleleFreqs.empty())  ? false : true;
    filter.checkGaP_Hapmap      = (filter.listGaP_Hapmaps.empty())  ? false : true;
    filter.checkMapping         = (filter.listMappings.empty())     ? false : true;
    filter.checkQualityCheck    = (filter.listQualityChecks.empty())? false : true;
    filter.checkGeneFxn         = (filter.listGeneFxns.empty())     ? false : true;

    filter.checkAlleleFreqChoice = (filter.choiceAlleleFreq == kChoiceAny) ? false : true;
    filter.checkVarClass         = (filter.choiceVarClass   == kChoiceAny) ? false : true;
    filter.checkWeight           = (filter.choiceWeight     == kChoiceAny) ? false : true;

    m_mapChangeInfo[name] = eModified;

    m_bNeedsSave = false;
    x_EnableSave(false);
}

END_NCBI_SCOPE
