/*  $Id: snp_filter_ui_tabs.cpp 27035 2012-12-11 19:21:58Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/snp/filter/group_checkboxes.hpp>
#include <gui/widgets/snp/filter/group_radios.hpp>
#include <gui/widgets/snp/filter/snp_filter_ui.hpp>

#include <wx/scrolwin.h>
#include <wx/sizer.h>

BEGIN_NCBI_SCOPE

void CSnpFilterUI::x_SetupTabs()
{
    int grp_x        = 0,   grp_y           = 0;
    int btn_width = 25, btn_height = 25;

    wxScrolledWindow* s;
    wxSizer*          o;
    long style  = wxSUNKEN_BORDER|wxVSCROLL|wxHSCROLL|wxTAB_TRAVERSAL;

    // Create the tab groups

    // The 'Links' Tab
    {{

    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);

    m_GrpChecksLinks = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp = m_GrpChecksLinks;
    grp->Add( CSnpBitfield::eHasLinkOut,
        "Has SubmitterLinkOut From SNP->SubSNP->Batch.link_out");
    grp->Add( CSnpBitfield::eHasSnp3D,
        "Has 3D structure SNP3D");
    grp->Add( CSnpBitfield::eHasSTS,
        "Has STS Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eHasEntrez,
        "Has EntrezGene Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eHasProbeDB,
        "Has ProbeDB Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eHasGEO,
        "Has GEO Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eHasAssembly,
        "Has Assembly Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eHasTrace,
        "Has Trace Query Entrez to get the current links");
    grp->Add( CSnpBitfield::eFromMgcClone,
        "From MGC clone We have ~20K rs. This bit could be set from specific submitter handle/ batch_id");
    grp->Add( CSnpBitfield::eHasOrganism,
        "Has OrganismDBLink (Ex. Jackson Lab for mouse)");
    grp->Add( CSnpBitfield::eHasShortReadArchive,
        "Has Short Read Archive link");
    grp->Add( CSnpBitfield::eHasPubmedArticle,
        "Links exist to PubMed Central article");
    grp->Add( CSnpBitfield::eHasProvisionalTPA,
        "Links exist to Provisional Third Party Annotation (TPA)");
    grp->Add( CSnpBitfield::eIsPrecious,
        "SNP is Precious (Clinical or Pubmed Cited links exist)");
    grp->Add( CSnpBitfield::eIsClinical,
        "SNP is Clinical (LSDB, OMIM, TPA, Diagnostic links exist)");

    m_Tabs->AddPage(s, wxT("Links"));
    }}


    // 'Gene Fxn' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);

    m_GrpChecksGeneFxn = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp = m_GrpChecksGeneFxn;
    grp->Add(CSnpBitfield::eUnknownFxn,
        "Unknown");
    grp->Add(CSnpBitfield::eMultipleFxn,
        "Multiple");
    grp->Add(CSnpBitfield::eIntron,
        "In Intron");
    grp->Add(CSnpBitfield::eDonor,
        "In donor splice-site");
    grp->Add(CSnpBitfield::eAcceptor,
        "In acceptor splice site");
    grp->Add(CSnpBitfield::eUTR,
        "In Exon. location is in a spliced transcript. Is \"untranslated region\" (UTR) if \"In CDS\" is false");
    grp->Add(CSnpBitfield::eSynonymous,
        "In coding region (CDS). A subset of \"Exon\" excluding \"UTR\": SYNONYMOUS if bits 5-7 are false");
    grp->Add(CSnpBitfield::eStopGain,
        "Has STOP-Gain. Changes to STOP codon (TER)");
    grp->Add(CSnpBitfield::eStopLoss,
        "Has STOP-Loss. Changes the encoded STOP codon (TER)");
    grp->Add(CSnpBitfield::eMissense,
        "Is non-synonymous Missense. Changes protein peptide");
    grp->Add(CSnpBitfield::eFrameshift,
        "Is non-synonymous Frameshift. Changes all downstream amino acids");
    grp->Add(CSnpBitfield::eInGene,
        "In gene segment Defined as sequence intervals covered by a gene ID but not having an aligned transcript. FxnCode = 11");
    grp->Add(CSnpBitfield::eInGene5,
        "In 5' gene region FxnCode = 15");
    grp->Add(CSnpBitfield::eInGene3,
        "In 3' gene region FxnCode = 13");
    grp->Add(CSnpBitfield::eInUTR5,
        "In 5' UTR Location is in an untranslated region (UTR). FxnCode = 55");
    grp->Add(CSnpBitfield::eInUTR3,
        "In 3' UTR Location is in an untranslated region (UTR). FxnCode = 53");

    m_Tabs->AddPage(s, wxT("Gene Fxn"));
    }}

    // 'Mapping' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);


    m_GrpRadiosWeight = new CGroupRadioButtons(  s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupRadioButtons *grp = m_GrpRadiosWeight;
    grp->Add(kChoiceAny,"Weight Any");
    grp->Add( 0,        "Weight 0  (Unmapped)");
    grp->Add( 1,        "Weight 1  (Mapped exactly once)");
    grp->Add( 2,        "Weight 2  (Mapped exactly twice)");
    grp->Add( 3,        "Weight 3+ (Many mappings)");

    grp->SetChoice(kChoiceAny);

    m_GrpChecksMapping = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp2 = m_GrpChecksMapping;
    grp2->Add(CSnpBitfield::eIsAssemblySpecific,
        "Is Assembly specific. This bit is 1 if the snp only maps to one assembly");
    grp2->Add(CSnpBitfield::eHasAssemblyConflict,
        "Has Assembly conflict. This is for weight 1 and 2 snp that maps to different chromosomes on different assemblies");
    grp2->Add(CSnpBitfield::eHasOtherSameSNP,
        "Has other snp with exactly the same set of mapping position on NCBI refernce assembly");
    grp2->Add(CSnpBitfield::eHasReference,
        "Has Reference. A coding region variation where one allele in the set is identical to the reference sequence. FxnCode = 8");

    m_Tabs->AddPage(s, wxT("Mapping"));
    }}

    // 'AlleleFreq' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);


    m_GrpRadiosAlleleFreq = new CGroupRadioButtons(  s,
                                    grp_x + 5,
                                    (grp_y + btn_height*2) + 5, // make room for the previous 2 checkboxes
                                    btn_width,
                                    btn_height);

    CGroupRadioButtons *grp = m_GrpRadiosAlleleFreq;
    grp->Add(kChoiceAny,
        "Any Population");
    grp->Add(CSnpBitfield::e5PctMinorAllele1Plus,
        ">5% minor allele frequency in 1+ populations");
    grp->Add(CSnpBitfield::e5PctMinorAlleleAll,
        ">5% minor allele frequency in each and all populations");

    grp->SetChoice(kChoiceAny);


    m_GrpChecksAlleleFreq = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp2 = m_GrpChecksAlleleFreq;
    grp2->Add(CSnpBitfield::eIsDoubleHit_depr,
        "[Deprecated] Is doubleHit. This bit is set if the rs# is in Jim Mullikin's double hit submission which has been only on human snp");
    grp2->Add(CSnpBitfield::eIsMutation,
        "Is mutation (journal citation, explicit fact) low frequency variation that is cited in journal and other reputable sources.");
    grp2->Add(CSnpBitfield::eIsValidated,
        "Is validated (\"by-frequency\").  This bit is set if the snp has 2+ minor allele count based on frequency or genotype  data.");

    m_Tabs->AddPage(s, wxT("Allele Freq"));
    }}

    // 'GaP/Hapmap' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);

    m_GrpChecksGaP = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp = m_GrpChecksGaP;
    grp->Add(CSnpBitfield::eHasGenotype,
        "Genotypes available. The snp has individual genotype (in SubInd table)");
    grp->Add(CSnpBitfield::eInHaplotypeSet,
        "In Haplotype tagging set");
    grp->Add(CSnpBitfield::eInGenotypeKit,
        "Marker is on high density genotyping kit (50K density or greater). The snp may have phenotype associations present in dbGaP");
    grp->Add(CSnpBitfield::eHasOMIM_OMIA,
        "Has OMIM/OMIA");
    grp->Add(CSnpBitfield::eHasMicroattribution,
        "Has Microattribution/Third-Party Annotation (TPA:GWAS,PAGE)");
    grp->Add(CSnpBitfield::eHasLodScore,
        "Has LOD score");
    grp->Add(CSnpBitfield::eHasLowdbGaPPValue,
        "Has significant association in dbGaP study");
    grp->Add(CSnpBitfield::eHasDiseaseInfo,
        "Submitted as a disease-related mutation and/or present in a locus-specific database");
    grp->Add(CSnpBitfield::eHasTranscriptionFactor,
        "Has transcription factor");
    grp->Add(CSnpBitfield::eHasClinicalAssay,
        "Variation is interrogated in a clinical diagnostic assay");
    grp->Add(CSnpBitfield::eHasMeSH,
        "Has MeSH is linked to a disease");
    grp->Add(CSnpBitfield::ePhase3Genotyped,
        "Phase 3 genotyped  filtered, non-redundant");
    grp->Add(CSnpBitfield::eTGPPhase1,
        "1000Genomes Phase 1");
    grp->Add(CSnpBitfield::eTGPPilot,
        "1000Genomes Pilot (1,2,3)");
    grp->Add(CSnpBitfield::eTGPValidated,
        "1000Genomes for subset that passed positive second platform validation");
    grp->Add(CSnpBitfield::eTGPOnlyNotExclusive,
        "RS Cluster has a 1000Genomes submission");
    grp->Add(CSnpBitfield::eTGPNoneNotExclusive,
        "RS Cluster has submission from source other than 1000Genomes");

    m_Tabs->AddPage(s, wxT("GaP/Hapmap/1000Genomes"));
    }}

    // 'VarClass' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);

    m_GrpRadiosVarClass = new CGroupRadioButtons(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupRadioButtons *grp = m_GrpRadiosVarClass;
    grp->Add(kChoiceAny,
        "Any");
    grp->Add(CSnpBitfield::eUnknownVariation,
        "Unknown variation");
    grp->Add(CSnpBitfield::eSingleBase,
        "single-base polymorphism SNP");
    grp->Add(CSnpBitfield::eDips,
        "dips (deletion/insertion)");
    grp->Add(CSnpBitfield::eHeterozygous,
        "Heterozygous");
    grp->Add(CSnpBitfield::eMicrosatellite,
        "Microsatellite");
    grp->Add(CSnpBitfield::eNamedSNP,
        "Named SNP e.g. (Alu)");
    grp->Add(CSnpBitfield::eNoVariation,
        "No Variation");
    grp->Add(CSnpBitfield::eMixed,
        "Mixed class");
    grp->Add(CSnpBitfield::eMultiBase,
        "Multi-base polymorphism");

    grp->SetChoice(kChoiceAny);

    m_Tabs->AddPage(s, wxT("VarClass"));
    }}

    // 'Quality' Tab
    {{
    s = new wxScrolledWindow( m_Tabs, wxID_ANY, wxDefaultPosition, wxDefaultSize, style );
    o = new wxBoxSizer(wxVERTICAL);
    s->SetScrollRate(20, 20);
    s->SetSizer(o);

    m_GrpChecksQuality = new CGroupCheckboxes(s, grp_x + 5, grp_y + 5, btn_width, btn_height);

    CGroupCheckboxes *grp = m_GrpChecksQuality;
    grp->Add(CSnpBitfield::eHasGenotypeConflict,
        "Has Genotype Conflict Same (rs, ind), different genotype. N/N is not included");
    grp->Add(CSnpBitfield::eIsStrainSpecific,
        "Is Strain Specific");
    grp->Add(CSnpBitfield::eHasMemberSsConflict,
        "RS has two or more submissions with conflict alleles");
    grp->Add(CSnpBitfield::eIsWithdrawn,
        "Is Withdrawn by submitter If one member ss is withdrawn by submitter, then this bit is set. If all member ss' are withdrawn, then the rs is deleted to SNPHistory");
    grp->Add(CSnpBitfield::eIsContigAlleleAbsent,
        "Contig allele not present in SNP allele list. The reference sequence allele at the mapped position is not present in the SNP allele list, adjusted for orientation");
    grp->Add(CSnpBitfield::eIsSomatic,
        "Somatic, not Germline - Variation detected in somatic tissue (e.g. cancer tumor).  It is not known to exist in heritable DNA");

    m_Tabs->AddPage(s, wxT("Quality"));
    }}


}


END_NCBI_SCOPE
