/*  $Id: track_creator_ui.cpp 25491 2012-03-27 17:52:21Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/widgets/snp/filter/track_creator_ui.hpp>
#include <gui/widgets/snp/filter/snp_filter_ui.hpp>
#include <gui/widgets/wx/message_box.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <objects/seqres/Seq_graph.hpp>

#include <fstream>
#include <wx/sizer.h>

#include "snp_filter_job.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

/////////////////////////////////////////////////////////////////////////////
// Public Member Functions
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Private Member Functions
/////////////////////////////////////////////////////////////////////////////

void CTrackCreatorUI::x_Finish()
{
    // Get the result
    CRef<CObject> result;
    CAppJobDispatcher& disp = CAppJobDispatcher::GetInstance();
    result = disp.GetJobResult(m_ActiveJob);

    if (result.NotEmpty()) {
        CRef<CSeq_graph> ref_graph;
        ref_graph = (CSeq_graph*)result.GetPointer();

        string name = ToStdString(m_txtName->GetValue());
        ref_graph->SetTitle(name);
        m_Annot.Reset( new CSeq_annot());
        m_Annot->SetNameDesc("filteredTrack");
        m_Annot->SetTitleDesc(name);
        m_Annot->SetData().SetGraph().push_back(ref_graph);
    }

    EndModal(wxOK);
}

CRef<CSeq_annot> CTrackCreatorUI::GetResult()
{
    return m_Annot;
}

void CTrackCreatorUI::x_UpdateChoices()
{
    /*
    This funciton will be redone.
    The master list of filters will be stored in the GUI Registry
    There will be a helper function to get a mapping of the filters.
    */

    /*
    const CSnpFilterUI::TMapFilters &map = CSnpFilterUI::GetInstance().GetFilters();
    m_choiceFilter->Clear();

    ITERATE(CSnpFilterUI::TMapFilters, iter, map) {
        string name = iter->first;
        m_choiceFilter->Append(ToWxString(name));
    }
    */
}


void CTrackCreatorUI::x_OnChoice()
{
    // Will be re-written.  See x_UpdateChoices
    /*
    string choice = ToStdString(m_choiceFilter->GetValue());
    const CSnpFilterUI::TMapFilters &map = CSnpFilterUI::GetInstance().GetFilters();
    CSnpFilterUI::TMapFilters::const_iterator it = map.find(choice);
    if (it != map.end()) {
        m_Filter = *(it->second);
        m_txtName->SetLabel(ToWxString(m_Filter.name)); // update the name
    }
    */
}

void CTrackCreatorUI::x_EnabledEditOK(bool enable)
{
    if (enable) {
        m_btnOK->Enable();
        m_btnEdit->Enable();
        m_Progress->Disable();
    }
    else {
        // user hit the 'OK' button
        m_btnOK->Disable();
        m_btnEdit->Disable();
        m_Progress->SetValue(0);
        m_Progress->Disable();
        m_txtStatus->SetLabel(wxT("Retrieving data ..."));
    }
}

bool CTrackCreatorUI::x_IsJobComplete()
{
    bool isComplete = false;

    CAppJobDispatcher& disp = CAppJobDispatcher::GetInstance();

    if (disp.GetJobState(m_ActiveJob)==IAppJob::eCompleted) {
        isComplete = true;
    }

    return isComplete;
}

void CTrackCreatorUI::x_UpdateProgress()
{
    static bool lastValueWasZero = false; // minimize freq of changes to of m_txtStatus

    CAppJobDispatcher&           disp = CAppJobDispatcher::GetInstance();
    CConstIRef<IAppJobProgress>  prog = disp.GetJobProgress(m_ActiveJob);
    float                           v = prog->GetNormDone();

    int nVal = (int)(v*100);
    m_Progress->SetValue(nVal);

    if (nVal == 0 && lastValueWasZero == false) {
        lastValueWasZero = true;
    }
    else if ( nVal!=0 && lastValueWasZero ) {
        lastValueWasZero = false;
        m_txtStatus->SetLabel(wxT("Filtering ..."));
    }
}


/*!
 * CTrackCreatorUI type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CTrackCreatorUI, wxDialog )


/*!
 * CTrackCreatorUI event table definition
 */

BEGIN_EVENT_TABLE( CTrackCreatorUI, wxDialog )

////@begin CTrackCreatorUI event table entries
    EVT_COMBOBOX( ID_COMBOBOX, CTrackCreatorUI::OnComboboxSelected )

    EVT_BUTTON( ID_BUTTON_EDIT, CTrackCreatorUI::OnButtonEditClick )

    EVT_BUTTON( wxID_OK, CTrackCreatorUI::OnOkClick )

    EVT_BUTTON( wxID_CANCEL, CTrackCreatorUI::OnCancelClick )

////@end CTrackCreatorUI event table entries

    EVT_TIMER( ID_TIMER1, CTrackCreatorUI::OnTimer)

END_EVENT_TABLE()


/*!
 * CTrackCreatorUI constructors
 */

CTrackCreatorUI::CTrackCreatorUI()
//  : m_ActiveJob(-1), m_bCallFinish(false), m_nChoiceFilter(-1)
{
    Init();
}

CTrackCreatorUI::CTrackCreatorUI( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(0,0,parent, id, caption, pos, size, style);
}


/*!
 * CTrackCreatorUI creator
 */

bool CTrackCreatorUI::Create( objects::CScope *scope, objects::CSeq_loc *loc, wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CTrackCreatorUI creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CTrackCreatorUI creation

    m_Scope.Reset(scope);
    m_Loc.Reset(loc);

    x_EnabledEditOK(true);

    m_Progress->SetRange(100);
    m_Progress->SetValue(0);

    // Set the ID
    string name = m_Loc->GetId()->GetSeqIdString(true);
    m_txtID->SetLabel(ToWxString(name));

    // set the Name
    name = name+" [filtered]";
    m_txtName->SetLabel(ToWxString(name));

    // Set the range
    string strRange;
    TSeqRange range = m_Loc->GetTotalRange();
    strRange  = NStr::UIntToString(range.GetFrom(), NStr::fWithCommas) + " to ";
    strRange += NStr::UIntToString(range.GetTo(), NStr::fWithCommas);
    m_txtRange->SetLabel(ToWxString(strRange));

    // update internal filters
    x_UpdateChoices();
    return true;
}


/*!
 * CTrackCreatorUI destructor
 */

CTrackCreatorUI::~CTrackCreatorUI()
{
////@begin CTrackCreatorUI destruction
////@end CTrackCreatorUI destruction
}


/*!
 * Member initialisation
 */

void CTrackCreatorUI::Init()
{
////@begin CTrackCreatorUI member initialisation
    m_txtID = NULL;
    m_txtRange = NULL;
    m_choiceFilter = NULL;
    m_txtName = NULL;
    m_Progress = NULL;
    m_txtStatus = NULL;
    m_btnEdit = NULL;
    m_btnOK = NULL;
////@end CTrackCreatorUI member initialisation
    m_ActiveJob = -1;
    m_bCallFinish = false;
    m_Timer.SetOwner(this, ID_TIMER1);
}


/*!
 * Control creation for CTrackCreatorUI
 */

void CTrackCreatorUI::CreateControls()
{
////@begin CTrackCreatorUI content construction
    CTrackCreatorUI* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemDialog1, wxID_STATIC, _("ID:"), wxDefaultPosition, wxSize(40, -1), 0 );
    itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtID = new wxTextCtrl( itemDialog1, ID_TEXTCTRL1, wxT(""), wxDefaultPosition, wxSize(230, -1), wxTE_READONLY|wxSUNKEN_BORDER );
    itemBoxSizer4->Add(m_txtID, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer7, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemDialog1, wxID_STATIC, _("Range:"), wxDefaultPosition, wxSize(40, -1), 0 );
    itemBoxSizer7->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtRange = new wxTextCtrl( itemDialog1, ID_TEXTCTRL2, wxT(""), wxDefaultPosition, wxSize(230, -1), wxTE_READONLY|wxSUNKEN_BORDER );
    itemBoxSizer7->Add(m_txtRange, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer3->Add(5, 5, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxBoxSizer* itemBoxSizer11 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer11, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxStaticText* itemStaticText12 = new wxStaticText( itemDialog1, wxID_STATIC, _("Filter:"), wxDefaultPosition, wxSize(40, -1), 0 );
    itemBoxSizer11->Add(itemStaticText12, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_choiceFilterStrings;
    m_choiceFilter = new wxComboBox( itemDialog1, ID_COMBOBOX, wxT(""), wxDefaultPosition, wxSize(230, -1), m_choiceFilterStrings, wxCB_READONLY );
    itemBoxSizer11->Add(m_choiceFilter, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer14, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxStaticText* itemStaticText15 = new wxStaticText( itemDialog1, wxID_STATIC, _("Name:"), wxDefaultPosition, wxSize(40, -1), 0 );
    itemBoxSizer14->Add(itemStaticText15, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtName = new wxTextCtrl( itemDialog1, ID_TEXTCTRL_NAME, wxT(""), wxDefaultPosition, wxSize(230, -1), 0 );
    itemBoxSizer14->Add(m_txtName, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer17 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer17, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_Progress = new wxGauge( itemDialog1, ID_GAUGE, 100, wxDefaultPosition, wxSize(150, 20), wxGA_HORIZONTAL );
    m_Progress->SetValue(1);
    itemBoxSizer17->Add(m_Progress, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_txtStatus = new wxStaticText( itemDialog1, wxID_STATIC, wxT(""), wxDefaultPosition, wxSize(150, -1), 0 );
    itemBoxSizer17->Add(m_txtStatus, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    itemBoxSizer17->Add(5, 12, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    wxBoxSizer* itemBoxSizer21 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer17->Add(itemBoxSizer21, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT, 5);

    m_btnEdit = new wxButton( itemDialog1, ID_BUTTON_EDIT, _("Add / Edit"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer21->Add(m_btnEdit, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer21->Add(70, 5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer24 = new wxStdDialogButtonSizer;

    itemBoxSizer17->Add(itemStdDialogButtonSizer24, 0, wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT|wxTOP, 5);
    m_btnOK = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer24->AddButton(m_btnOK);

    wxButton* itemButton26 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer24->AddButton(itemButton26);

    itemStdDialogButtonSizer24->Realize();

////@end CTrackCreatorUI content construction
}


/*!
 * Should we show tooltips?
 */

bool CTrackCreatorUI::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CTrackCreatorUI::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CTrackCreatorUI bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CTrackCreatorUI bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CTrackCreatorUI::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CTrackCreatorUI icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CTrackCreatorUI icon retrieval
}



/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
 */

void CTrackCreatorUI::OnButtonEditClick( wxCommandEvent& /*event*/ )
{
    CSnpFilterUI ui(this);

    if (m_choiceFilter->GetSelection() != wxNOT_FOUND) {
        ui.SetSelection(ToStdString(m_choiceFilter->GetValue()));
    }

    ui.Center();
    ui.ShowModal();

    x_UpdateChoices();

    int row = ui.GetSelectedRow();

    if (row != wxNOT_FOUND) {
        m_choiceFilter->Select(row);
        x_OnChoice();
    }
}

/*!
 * wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_COMBOBOX
 */

void CTrackCreatorUI::OnComboboxSelected( wxCommandEvent& /*event*/ )
{
    x_OnChoice();
}

void CTrackCreatorUI::OnTimer(wxTimerEvent& /*event*/)
{
    // The use of 'call_finish' creates one more Timeout message
    // to be processed after the job completes.
    // allowing the user to see "100%" completion in progress bar
    if (m_bCallFinish) {
        m_Timer.Stop();
        x_Finish();
        m_bCallFinish = false; // reset for next time
    }
    else {
        x_UpdateProgress();
    }

    // We check job status after we update the progress.
    if (x_IsJobComplete()) {
        m_bCallFinish = true;
    }
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CTrackCreatorUI::OnOkClick( wxCommandEvent& /*event*/ )
{
    CRef<CSnpFilterJob> job( new CSnpFilterJob( *m_Scope, *m_Loc, m_Filter) );
    CAppJobDispatcher& disp = CAppJobDispatcher::GetInstance();

    if (m_choiceFilter->GetSelection() == wxNOT_FOUND) {
        NcbiMessageBox("Please select a filter",
                        eDialog_Ok | eDialog_Modal | eDialog_Centered,
                        eIcon_Exclamation,
                        "No Filter Selected");
        return;
    }

    try {
        m_ActiveJob     = disp.StartJob( *job, "ObjManagerEngine");

        x_EnabledEditOK(false);

        m_Timer.Start(250);

    } catch( CAppJobException& e ){
        LOG_POST( Error
            << "CTrackCreatorUI::x_OnOK(): Failed to start job: "
            << e.GetMsg()
        );
        e.ReportAll();
    }
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
 */

void CTrackCreatorUI::OnCancelClick( wxCommandEvent& event )
{

    try {{

    if (m_Timer.IsRunning()) {
        m_Timer.Stop();

        // cancel the job
        CAppJobDispatcher& disp = CAppJobDispatcher::GetInstance();
        disp.CancelJob(m_ActiveJob);

        // reset
        m_txtStatus->SetLabel(wxT(""));
        m_Progress->SetValue(0);
        x_EnabledEditOK(true);
        m_bCallFinish = false;
    }
    else {
        event.Skip();
    }

    }}
    catch (CAppJobException &ex) {
        LOG_POST( Error
            << "CTrackCreatorUI::x_OnCancel(): Failed to cancel job: "
            << ex.GetMsg()
        );

        if (x_IsJobComplete())
            x_Finish();

    }
    catch (std::exception&) {
        _ASSERT(false);
    }

}


END_NCBI_SCOPE
