#ifndef GUI_PLUGINS_DOC_ASSN_STUDY___STUDY_LOADER__HPP
#define GUI_PLUGINS_DOC_ASSN_STUDY___STUDY_LOADER__HPP

/*  $Id: analysis_loader.hpp 22653 2010-12-08 20:08:04Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *    Loads association study files and creates graph and ftable annotations.
 *
 */

#include <string>
#include <list>

#include <objmgr/object_manager.hpp>
#include <gui/packages/pkg_snp/gwas_track_load_params.hpp>

#include "analysis_file.hpp"
#include <gui/utils/muparser/muParser.h>

BEGIN_NCBI_SCOPE

namespace objects {
class CObjectManager;
class CScope;
class CSeq_graph;
class CSeq_feat;
class CSeq_annot;
class CSeq_annot_Handle;
}

class CAnalysisFileLoader
{
////////////////////////////////////////
// PUBLIC TYPEDEF/STRUCTS
////////////////////////////////////////
public:

typedef list<string>                        TListStrings;
typedef list< CRef<objects::CSeq_annot> >   TListAnnots;
typedef set<string>                         TSetStrings;
typedef map<string,string>                  TMapStrings;

////////////////////////////////////////
// PUBLIC METHODS
////////////////////////////////////////
public:

    CAnalysisFileLoader(int comp = 10000);

    void Load(const wxString& filename);
    void Load(const wxString& filename, const CGwasLoadParams &params ); 

    void SetCompression(int compression);

    void GetAnnots(TListAnnots *list);

//////////////////////////////////////////////////////////////////////////////
// PRIVATE TYPEDEF/STRUCTS
//////////////////////////////////////////////////////////////////////////////
private:

struct SValue {
    string          id;
    double          value;
};

typedef map<unsigned int, SValue>   TMapValues;     // values by position
typedef map<string, TMapValues>      TMapMapValues;   // value-map by accession
//////////////////////////////////////////////////////////////////////////////
// PRIVATE METHODS
//////////////////////////////////////////////////////////////////////////////
private:
    string  x_GetLabel() const;
    void    x_ReadData(TMapMapValues *);

    void x_AddAnnotDescr(objects::CSeq_annot*);
    CRef<objects::CSeq_annot> x_CreateSeqGraph(const string& accName, const TMapValues& pvals);
    CRef<objects::CSeq_annot> x_CreateSeqFtable(const string& accName,const TMapValues &pvals);


    void x_CreateSeqAnnots(const TMapMapValues &);

    char    x_Translate(double pval);
    char    x_MaxValueAndCount(const TMapValues &, unsigned long pos, int comp);

//////////////////////////////////////////////////////////////////////////////
// PRIVATE Data
//////////////////////////////////////////////////////////////////////////////
private:
    bool                m_bUseRankFunc;
    int                 m_nCompression;     // Compression of seq_graph

    TListStrings        m_listStudyNames;   // names of loaded study files
    TListAnnots         m_listAnnots;

    const CGwasLoadParams*          m_pParams;

    auto_ptr<CAnalysisFile>         m_AnaFile;

    CRef<objects::CObjectManager>   m_ObjMgr;
    CRef<objects::CScope>           m_Scope;

    mu::Parser                      m_RankFunc;
};



END_NCBI_SCOPE

#endif  // GUI_PLUGINS_DOC_ASSN_STUDY___STUDY_LOADER__HPP

