/*  $Id: dll_register.cpp 27389 2013-02-06 18:39:15Z rudnev@NCBI.NLM.NIH.GOV $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *    Defines "SNP" package for GBENCH.
 */

#include <ncbi_pch.hpp>

#include <gui/framework/gui_package.hpp>
#include <gui/utils/extension_impl.hpp>

#include <gui/widgets/wx/ui_command.hpp>

#include "gwas_track_load_manager.hpp"
#include <gui/packages/pkg_snp/table/factory_snp_table.hpp>

#include <gui/packages/pkg_snp/track/snp_track.hpp>
#include <gui/packages/pkg_snp/track/snp_ds.hpp>
#include <gui/packages/pkg_snp/gap/gap_track.hpp>
#include <gui/packages/pkg_snp/gap/gap_ds.hpp>
#include <gui/packages/pkg_snp/hapmap/hapmap_track.hpp>
#include <gui/packages/pkg_snp/hapmap/hapmap_ds.hpp>
#include <gui/packages/pkg_snp/ld/ld_track.hpp>
#include <gui/packages/pkg_snp/ld/ld_ds.hpp>
#include <gui/packages/pkg_snp/bins/bins_track.hpp>
#include <gui/packages/pkg_snp/bins/bins_ds.hpp>

BEGIN_NCBI_SCOPE

///////////////////////////////////////////////////////////////////////////////
/// Declare the Package Object

class CSnpPackage : public IGuiPackage
{
public:
    virtual string GetName() const
    {
        return "SNP";
    }
    virtual void GetVersion(size_t& verMajor, size_t& verMinor, size_t& verPatch) const;

    virtual bool Init();
    virtual void Shut() {}

protected:
    void    x_RegisterCommands();
};


void CSnpPackage::GetVersion(size_t& verMajor, size_t& verMinor, size_t& verPatch) const
{
    verMajor = 1;
    verMinor = 0;
    verPatch = 0;
}


bool CSnpPackage::Init()
{
    // Register Views
    // disabled as a new SNP Table view is implemented in GB-1758
	// CExtensionDeclaration("view_manager_service::view_factory",
	//                          new CSnpTableLegacyViewFactory());

    // Register File Loaders
    CExtensionDeclaration ("file_format_loader_manager",
                          new CGwasTrackLoadManager());


    CExtensionDeclaration("seqgraphic_data_source_type",//EXT_POINT__SEQGRAPHIC_DATA_SOURCE_TYPE,
                          new CSGSnpDSType());
    CExtensionDeclaration("seqgraphic_data_source_type",
                          new CSGGapDSType());
    CExtensionDeclaration("seqgraphic_data_source_type",//EXT_POINT__SEQGRAPHIC_DATA_SOURCE_TYPE,
                          new CSGHapmapDSType());
    CExtensionDeclaration("seqgraphic_data_source_type",//EXT_POINT__SEQGRAPHIC_DATA_SOURCE_TYPE,
                          new CLDBlockDSType());
    CExtensionDeclaration("seqgraphic_data_source_type",//EXT_POINT__SEQGRAPHIC_DATA_SOURCE_TYPE,
                          new CBinsDSType());

    CExtensionDeclaration(EXT_POINT__SEQGRAPHIC_LAYOUT_TRACK_FACTORY,
                          new CSnpTrackFactory());
    CExtensionDeclaration(EXT_POINT__SEQGRAPHIC_LAYOUT_TRACK_FACTORY,
                          new CGapTrackFactory());
    CExtensionDeclaration(EXT_POINT__SEQGRAPHIC_LAYOUT_TRACK_FACTORY,
                          new CHapmapTrackFactory());
    CExtensionDeclaration(EXT_POINT__SEQGRAPHIC_LAYOUT_TRACK_FACTORY,
                          new CLDBlockTrackFactory());
    CExtensionDeclaration(EXT_POINT__SEQGRAPHIC_LAYOUT_TRACK_FACTORY,
                          new CBinsTrackFactory());



    // Register Data sources
    // Declare GWAS Data source
    // CExtensionDeclaration("ui_data_source_type", new CNetBLASTUIDataSourceType());

    // Register Tools
    // CExtensionDeclaration("ui_algo_tool_manager", new CBLASTSearchToolManager());
    // TODO CExtensionDeclaration("ui_algo_tool_manager", new CSnpFilterTrackTool());

    x_RegisterCommands();
    return true;
}


// register commands defined in the package
void CSnpPackage::x_RegisterCommands()
{
    // CUICommandRegistry& cmd_reg = CUICommandRegistry::GetInstance();

    // TODO Register Commands
    // cmd_reg.RegisterCommand(eCmdRetriveBlastJobs, "Retrieve", "Retrieve",
    //                         "", "Retrieve BLAST Job results from Net BLAST server");

}


///////////////////////////////////////////////////////////////////////////////
/// Declare Package Entry Point
extern "C" {
    NCBI_PACKAGEENTRYPOINT_EXPORT IGuiPackage* NCBIGBenchGetPackage()
    {
        return new CSnpPackage();
    }
}

END_NCBI_SCOPE
