/*  $Id: gwas_load_job.cpp 22653 2010-12-08 20:08:04Z katargir $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data,  the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties,  express or implied,  including
*  warranties of performance,  merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Authors:  Melvin Quintos
*/


#include <ncbi_pch.hpp>

#include <objects/seqfeat/Imp_feat.hpp>
#include <objects/seqloc/Seq_interval.hpp>
#include <objmgr/util/seq_loc_util.hpp>
#include <objmgr/object_manager.hpp>

#include "gwas_load_job.hpp"
#include "analysis_loader.hpp"

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

CGwasLoadJob::CGwasLoadJob(CProjectService* service,
                             const CGwasLoadParams& params,
                             const vector<wxString>& filenames)
: CDataLoadingAppJob(service), m_Params(params), m_FileNames(filenames)
{
    m_Descr = "Loading GWAS Track Files";
}

void CGwasLoadJob::x_CreateProjectItems()
{
    CAnalysisFileLoader loader;
    CAnalysisFileLoader::TListAnnots annots;

    loader.Load(m_FileNames[0], m_Params);
    loader.GetAnnots(&annots);

    NON_CONST_ITERATE( CAnalysisFileLoader::TListAnnots, it, annots) {
        CRef<CSeq_annot> &annot = (*it);

        CRef<CProjectItem> item(new CProjectItem());
        item->SetItem().SetAnnot(*annot);

        string label = annot->GetName();
        item->SetLabel(label);

        m_Items.push_back(item);
    }
}

END_NCBI_SCOPE
