/*  $Id: gwas_track_params_panel.cpp 25502 2012-03-27 21:23:47Z kuznets $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 */


#include <ncbi_pch.hpp>

////@begin includes
////@end includes

#include "gwas_track_params_panel.hpp"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/choice.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

/*!
 * CGwasTrackParamsPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CGwasTrackParamsPanel, wxPanel )


/*!
 * CGwasTrackParamsPanel event table definition
 */

BEGIN_EVENT_TABLE( CGwasTrackParamsPanel, wxPanel )

////@begin CGwasTrackParamsPanel event table entries
////@end CGwasTrackParamsPanel event table entries

END_EVENT_TABLE()


/*!
 * CGwasTrackParamsPanel constructors
 */

CGwasTrackParamsPanel::CGwasTrackParamsPanel()
{
    Init();
}

CGwasTrackParamsPanel::CGwasTrackParamsPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    CreateX(parent, id, pos, size, style);
}


/*!
 * GWASTrackParamsPanel creator
 */

bool CGwasTrackParamsPanel::CreateX( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CGwasTrackParamsPanel creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CGwasTrackParamsPanel creation
    return true;
}


/*!
 * CGwasTrackParamsPanel destructor
 */

CGwasTrackParamsPanel::~CGwasTrackParamsPanel()
{
////@begin CGwasTrackParamsPanel destruction
////@end CGwasTrackParamsPanel destruction
}


/*!
 * Member initialisation
 */

void CGwasTrackParamsPanel::Init()
{
////@begin CGwasTrackParamsPanel member initialisation
////@end CGwasTrackParamsPanel member initialisation
}


/*!
 * Control creation for GWASTrackParamsPanel
 */

void CGwasTrackParamsPanel::CreateControls()
{
////@begin CGwasTrackParamsPanel content construction
    CGwasTrackParamsPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT|wxTOP, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("Title"), wxDefaultPosition, wxSize(-1, 20), 0 );
    itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("Analysis Type"), wxDefaultPosition, wxSize(80, 20), 0 );
    itemBoxSizer4->Add(itemStaticText6, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer3->Add(itemBoxSizer7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl8 = new wxTextCtrl( itemPanel1, ID_TEXTCTRL, wxT(""), wxDefaultPosition, wxSize(300, 20), 0 );
    itemBoxSizer7->Add(itemTextCtrl8, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxArrayString itemChoice9Strings;
    itemChoice9Strings.Add(_("Association"));
    itemChoice9Strings.Add(_("Linkage"));
    wxChoice* itemChoice9 = new wxChoice( itemPanel1, ID_TEXTCTRL1, wxDefaultPosition, wxSize(300, 20), itemChoice9Strings, 0 );
    itemBoxSizer7->Add(itemChoice9, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    // Set validators
    itemTextCtrl8->SetValidator( wxGenericValidator(& GetData().m_sTitle) );
    itemChoice9->SetValidator( wxGenericValidator(& GetData().m_eTrackType) );
////@end CGwasTrackParamsPanel content construction
}


/*!
 * Should we show tooltips?
 */

bool CGwasTrackParamsPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CGwasTrackParamsPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CGwasTrackParamsPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CGwasTrackParamsPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CGwasTrackParamsPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CGwasTrackParamsPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CGwasTrackParamsPanel icon retrieval
}

/*!
 * Transfer data to the window
 */

bool CGwasTrackParamsPanel::TransferDataToWindow()
{
    return wxPanel::TransferDataToWindow();
}

/*!
 * Transfer data from the window
 */

bool CGwasTrackParamsPanel::TransferDataFromWindow()
{
    return wxPanel::TransferDataFromWindow();
}

END_NCBI_SCOPE
