/*  $Id: factory_snp_table.cpp 27035 2012-12-11 19:21:58Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_snp/table/factory_snp_table.hpp>

#include <ncbi_pch.hpp>

#include <gui/packages/pkg_snp/table/view_snp_table.hpp>
#include <gui/packages/pkg_snp/table/widget_snp_table.hpp>
#include <gui/packages/pkg_snp/table/model_snp_table.hpp>

#include <gui/framework/workbench.hpp>
#include <gui/core/project_service.hpp>

#include <gui/objutils/utils.hpp>
#include <gui/objutils/label.hpp>
#include <gui/objutils/user_type.hpp>

#include <corelib/ncbistre.hpp>

#include <serial/typeinfo.hpp>

#include <gui/types.hpp>

#include <gui/widgets/wx/table_utils.hpp>
#include <gui/widgets/wx/fileartprov.hpp>
#include <gui/widgets/wx/wx_utils.hpp>

#include <objects/seqloc/Seq_interval.hpp>
#include <objmgr/util/sequence.hpp>

BEGIN_NCBI_SCOPE

USING_SCOPE(objects);

static CProjectViewTypeDescriptor s_TypeDescr(
    "SNP Table View (Legacy)",       // type name
    "snp_table_view (Legacy)",       // icon alias TODO
    "Show tabular information about features",
    "The SNP Table View provides a tabular break-down of SNPs for a sequence",
    "SNP_TABLE_VIEW_LEG",       // help ID
    "Sequence",             // category
    false,                   // not a singleton
    "Seq-loc",
    eOneObjectAccepted
);
///////////////////////////////////////////////////////////////////////////////
/// CSnpTableLegacyViewFactory
string  CSnpTableLegacyViewFactory::GetExtensionIdentifier() const
{
    static string sid("snp_table_legacy_view_factory");
    return sid;
}

string CSnpTableLegacyViewFactory::GetExtensionLabel() const
{
    static string slabel("SNP Table Legacy View Factory");
    return slabel;
}


void CSnpTableLegacyViewFactory::RegisterCommands(CUICommandRegistry& cmd_reg,
                                              wxFileArtProvider& provider)
{
    CSnpTableWidget::RegisterCommands(cmd_reg, provider);
}



void CSnpTableLegacyViewFactory::RegisterIconAliases(wxFileArtProvider& provider)
{
    string alias = GetViewTypeDescriptor().GetIconAlias();
    provider.RegisterFileAlias(ToWxString(alias), wxT("feature_table_view.png"));
}


const CProjectViewTypeDescriptor& CSnpTableLegacyViewFactory::GetProjectViewTypeDescriptor() const
{
    return s_TypeDescr;
}


IView* CSnpTableLegacyViewFactory::CreateInstance() const
{
    return new CSnpTableView();
}


IView* CSnpTableLegacyViewFactory::CreateInstanceByFingerprint(const TFingerprint& /*fingerprint*/) const
{
    return NULL;
}


int CSnpTableLegacyViewFactory::TestInputObjects(TConstScopedObjects& objects)
{
    bool found_good = false;
    bool found_bad = false;
    for( size_t i = 0; i < objects.size(); i++ ){
        const CObject* obj = objects[i].object;
        const type_info& type = typeid(*obj);

        found_good = (
            typeid(CSeq_loc) == type
            //|| typeid(CSeq_align) == type
            //|| typeid(CBioseq) == type
        );
    }

    if( found_good ){
        return fCanShowSeparated | (found_bad ? fCanShowSome : fCanShowAll);
    }
    return 0; // can show nothing
}

END_NCBI_SCOPE
