#ifndef GUI_WIDGETS_SNP___SNP_TABLE_COL_NAMES__HPP
#define GUI_WIDGETS_SNP___SNP_TABLE_COL_NAMES__HPP

/*  $Id: snp_table_col_names.hpp 27035 2012-12-11 19:21:58Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * File Description:
 *  This file lists the names of the columns used in the SNP Table View.
 *  It is only intended to be included by snp_table_ds.cpp
 *
 */

enum EUserColNames {
    eRsId = 0,
    eStudyName,
    eStudyValue,
    eAlleles,
    eSequence,
    ePosition,
    eSnp3d,
    eGene,
    eWeight,
    eGtyAvailable,
    eOmim,
    eDiseaseInfo,
    eVariationClass,
    eBitString
};

static const char * USER_COL_NAMES[] =
{
"RS ID",
"Track",
"Value",
"Alleles",
"Sequence",
"Location",
"Snp3D",
"FXN Class",
"Weight",
"Pop Freq",
"OMIM/OMIA",
"Disease",
"Variation Class",
"Bitfield"
};

/* Order must match SNP bitfield format version TODO */
static const char * BIT_COL_NAMES[] =
{
// F1, Link
"LinkOut",
//"Snp3D",
"STS",
"Entrez",
"ProbeDB",
"GEO",
"Assembly",
"Trace",
"MgcClone",
"Organism",

// F2, Gene
/*
"GeneInt",
"Exon",
"CDS",
"Nonsense",
"Missense",
"Frameshift",
*/

// F3, Map
"AsmSpecific",
"AsmConflict",
"OtherSNP",

// F4, Freq
"5Pct 1+",
"5Pct ALL",
"Validated",
"Mutation",

// F5, GTY
//"Genotype",  Special case
"Haplotype",
"GenotypeKit",

// F6, Hapmap
//"Phase1 Attempted",
//"Phase1 Genotyped",
//"Phase2 Attempted",
//"Phase2 Genotyped",
//"Phase3 Attempted",
"Phase3 Genotyped",

// F7, Phenotype
"OMIM/OMIA",
"SnpRIF",
"LodScore",
"Low dbGaP p-Value",
"DiseaseInfo",
"Transcription Factor",
"Clinical Assay",
"MeSH",

// F9, Quality Check
"GTY Conflict",
"Strain Specific",
"Mendel Error",
// "Hardy-WeinberDeviation",
// "Member SS Conflict",
"Withdrawn"
};

static const CSnpBitfield::EProperty BIT_TYPE[] =
{
// F1, Link
CSnpBitfield::eHasLinkOut,
//SNP_BIT(CSnpBitfield::eLink, CSnpBitfield::eHasSnp3D        ),
CSnpBitfield::eHasSTS          ,
CSnpBitfield::eHasEntrez       ,
CSnpBitfield::eHasProbeDB      ,
CSnpBitfield::eHasGEO          ,
CSnpBitfield::eHasAssembly     ,
CSnpBitfield::eHasTrace        ,
CSnpBitfield::eFromMgcClone    ,
CSnpBitfield::eHasOrganism     ,

// F2, Gene
/*
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eInGeneInterval  ),
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eInExon          ),
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eInCDS           ),
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eInNonsense      ),
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eInMissense      ),
SNP_BIT(CSnpBitfield::eGene, CSnpBitfield::eIsFrameshift    ),
*/

// F3, Map
CSnpBitfield::eIsAssemblySpecific       ,
CSnpBitfield::eHasAssemblyConflict      ,
CSnpBitfield::eHasOtherSameSNP          ,

// F4, Freq
CSnpBitfield::e5PctMinorAllele1Plus    ,
CSnpBitfield::e5PctMinorAlleleAll      ,
// deprecated CSnpBitfield::eIsDoubleHit             ,
CSnpBitfield::eIsValidated,
CSnpBitfield::eIsMutation              ,

// F5, GTY
//SNP_BIT(CSnpBitfield::eGty, CSnpBitfield::eHasGenotype              ),
CSnpBitfield::eInHaplotypeSet           ,
CSnpBitfield::eInGenotypeKit            ,

// F6, Hapmap/TGP
// deprecated CSnpBitfield::ePhase1Attempted       ,
// deprecated CSnpBitfield::ePhase1Genotyped       ,
// deprecated CSnpBitfield::ePhase2Attempted       ,
// deprecated CSnpBitfield::ePhase2Genotyped       ,
// deprecated CSnpBitfield::ePhase3Attempted       ,
CSnpBitfield::ePhase3Genotyped       ,

// F7, Phenotype
CSnpBitfield::eHasOMIM_OMIA               ,
CSnpBitfield::eHasMicroattribution        ,
CSnpBitfield::eHasLodScore                ,
CSnpBitfield::eHasLowdbGaPPValue                 ,
CSnpBitfield::eHasDiseaseInfo             ,
CSnpBitfield::eHasTranscriptionFactor     ,
CSnpBitfield::eHasClinicalAssay           ,
CSnpBitfield::eHasMeSH                    ,

// F9, Quality Check
CSnpBitfield::eHasGenotypeConflict         ,
CSnpBitfield::eIsStrainSpecific            ,
// deprecated CSnpBitfield::eHasMendelError              ,
// deprecated CSnpBitfield::eHasHardyWeinbergDeviation   ,
CSnpBitfield::eHasMemberSsConflict         ,
CSnpBitfield::eIsWithdrawn

};

static const int NUM_USER_COLUMNS   = sizeof(USER_COL_NAMES)/sizeof(char *);
static const int NUM_BIT_COLUMNS    = sizeof(BIT_COL_NAMES )/sizeof(char *);
static const int NUM_COLUMNS        = NUM_BIT_COLUMNS + NUM_USER_COLUMNS;


#endif // GUI_WIDGETS_SNP___SNP_TABLE_COL_NAMES__HPP

