#ifndef GUI_WIDGETS_SNP___SNP_TABLE_JOB__HPP
#define GUI_WIDGETS_SNP___SNP_TABLE_JOB__HPP

/*  $Id: snp_table_job.hpp 26649 2012-10-18 20:31:07Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 *  File Description:
 *
 */

#include <gui/types.hpp>
#include <gui/utils/app_job.hpp>
#include <gui/utils/app_job_impl.hpp>
#include <objmgr/scope.hpp>
#include <objects/seqloc/Seq_loc.hpp>

#include <gui/widgets/snp/snp_data.hpp>
#include <gui/widgets/snp/filter/snp_filter.hpp>

BEGIN_NCBI_SCOPE
BEGIN_objects_SCOPE

class CSnpTableJob : public CJobCancelable
{
public:
    enum EJobType {
        eJobFilter,
        eJobFetch
    };

    struct SResult : public CObject {
        EJobType eJobType;
        CRef<CObjectFor<TSnpList> >     listSnpsFetched;
        CRef<CObjectFor<TSnpPtrList> >  listSnpsFiltered;
    };

public:
    CSnpTableJob( CScope& scope, const CSeq_loc& loc, bool fetchStudies );
    CSnpTableJob( TSnpListObject * unfiltered_snps, const SSnpFilter & filter );
    virtual ~CSnpTableJob() {}

    /// @name IAppJob implementation
    /// @{
    virtual EJobState                   Run();
    virtual CConstIRef<IAppJobProgress> GetProgress();
    virtual CRef<CObject>               GetResult();
    virtual CConstIRef<IAppJobError>    GetError();
    virtual string                      GetDescr() const;
    /// @}

private:
    void x_FetchOnlySpecial();
    void x_FetchAll();
    void x_ApplyFilter();

private:
    bool    m_bFetchOnlySpecial;

    size_t  m_Total;
    size_t  m_Completed;

    EJobType                m_eType;

    CRef<CScope>            m_Scope;
    CConstRef<CSeq_loc>     m_Loc;

    CRef<CAppJobError>      m_Error;
    CRef<CObject>           m_Result;
    CRef<SSnpFilter>        m_Filter;
};

END_objects_SCOPE
END_NCBI_SCOPE

#endif // GUI_WIDGETS_SNP___SNP_TABLE_JOB__HPP

