/*  $Id: utils.cpp 19371 2009-06-09 20:32:58Z quintosm $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Melvin Quintos
 *
 * Description:  This file implements functions in the snp_utils namespace
 *
 */

#include <ncbi_pch.hpp>
#include <gui/packages/pkg_snp/utils/utils.hpp>
#include <objmgr/seqdesc_ci.hpp>
#include <objmgr/table_field.hpp>

BEGIN_NCBI_SCOPE
BEGIN_SCOPE(NSnpUtils)
USING_SCOPE(objects);

// Performs iterative binary search to find 'pos_start' and 'pos_end'
// pos_end points at one index greater than the last item found in range.
// Algorithm is borrowed from Niklaus Wirth's Pascal implementation
// This may throw an exception if 'CTableFieldHandle::TryGet' fails
void FindRange(const CSeq_annot_Handle& annot,
                        int from, int to,
                        int *pos_start, int *pos_end)
{
    size_t rows = annot.GetSeq_tableNumRows();
    CTableFieldHandle<int> col_pos("pos");

    *pos_start = -1;
    *pos_end   = rows;

    // Find the 'from' value
    int i = 0;
    int j = rows-1;
    int k = 0;
    int Ak = 0;
    do {
        k = (i+j)/2;
        col_pos.TryGet(annot, k, Ak);
        if (Ak < from ) {
            *pos_start = k;
            i = k+1;
        } else {
            j = k-1;
        }
    } while (Ak != from && i <= j);

    // position start to be inclusive (catch boundary condition)
    (*pos_start) = (from == Ak) ? k : (*pos_start)+1;

    // find the 'to' value
    i=0;
    j=rows-1;
    Ak = 0;
    do {
        k = (i+j)/2;
        col_pos.TryGet(annot, k, Ak);
        if (to > Ak) {
            i = k+1;
        }
        else {
            *pos_end = k;
            j = k-1;
        }
    } while (Ak != to && i <= j);

    // if exact match for 'to', then increase end to include in range.
    if (Ak == to)
        (*pos_end)++;
}

END_SCOPE(NSnpUtils)
END_NCBI_SCOPE
